
#include "arexx.h"
#include "arexxsyslib.h"

#ifdef RVI
#include "rvi.h"
#endif
 
#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <proto/dos.h>

#include "flyerlib.h"
#include "flyer.h"
#include "flyerClipFmt.h"

#define MAX_PORT_NAME   16
#define MAX_PORT_LENGTH 24
#define MAX_ERROR_NAME  28
#define MAX_EXTENSION    8

#define NULLCHAR '\0'

struct ARexxContext
   {
   struct MsgPort *arexx_port;
   struct Library *rexx_base;
   long   outstanding;
   char   port_name[MAX_PORT_LENGTH +1];
   char   extension[MAX_EXTENSION + 1];
#ifdef RVI
   char   error_name[MAX_ERROR_NAME + 1];
#endif
   };

#define AREXXCONTEXT
typedef struct ARexxContext *ARexxContext;

#include "SimpleRexx.h"

static char *StrNUpper(char *dest, char *src, unsigned int len);

static char *UiToStr(unsigned num, char *buf);

ARexxContext ARexxInit(char *, char *, BOOL);
LONG get_reply(ARexxContext  context);

struct Library *DosBase;

ULONG Reprocess(char *, ULONG);

ULONG WriteClipStart(ULONG frames, ULONG Length, char *);

struct Library *FlyerBase;

struct Join
   {
   char    Name[100];
   ULONG   StartFrame;
   ULONG   FrameCount;
   UWORD   type;
   } join[256];

ARexxContext rob1;

FILE *bill, *input_file, *jeff_file;

WORD testbuff[256];

/* 2.0 Version string for c:Version to find */
UBYTE vers[] = "\0$VER: Video Knife 0.2";

main(int argc,char **argv)
{
LONG result;
ARexxContext rob1;
struct Library *RexxSysLib = NULL;
struct RexxMsg *bill2;
ULONG	res, err;
ULONG  OUTFILE2, OUTFILE3, OUTFILE4;  
UWORD  i, j;
ULONG Total = 0, WIDGET = 0, back;
BPTR handle;
ULONG frames;
char myfile[100];
UBYTE Audio, Video;
int Diff = 0;

struct FlyerVolume FlyVol, FlyVol2, FlyVol3;

struct ClipInfo	Buffer, Buffer2;
struct ClipAction act;
struct FrameHeader FrameBuffer;

UWORD pathcount;
UWORD *pathpt;
char temp[256];
char outpath[256];
char buff5[256];
char *tokenpt;
char number[100];

pathpt = &pathcount;

if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) 
   {
   printf("Open flyer.library failed\n");
   exit(0);
   }

if ((RexxSysLib = OpenLibrary("rexxsyslib.library",0))==0)
   {
   printf("Open rexxsyslib.library failed\n");
   exit(0);
   }

rob1 = ARexxInit("PROWAVE",NULL,FALSE);
if (!rob1)
   printf("We are confused \n");

Buffer.len = sizeof(struct ClipInfo);

bill2 = CallARexxFunc(rob1,"PROJECT_REXX_PORT","REQ_ERROR",0,1,"Setting Up Flyer"); 
if (!bill2)
   printf("It didn't work\n");
else
   result = get_reply(rob1);

FlyVol.Board = 0;
FlyVol.Flags = 0;
FlyVol.Path = argv[1];
res = GetClipInfo(&FlyVol,&Buffer);

if (res)
   {
   printf("Error %ld - %ls\n",res,Error2String(res));
   bill2 = CallARexxFunc(rob1,"PROJECT_REXX_PORT","REQ_ERROR",0,1,Error2String(res)); 
   if (!bill2)
      printf("It didn't work\n");
   else
      result = get_reply(rob1);
   goto ending;   
   }

OUTFILE2 = Buffer.Fields/4;

for (j=0;j<OUTFILE2 ;j++)
   {
   act.VidStartField = j*4;

   act.Volume = &FlyVol;

   res = GetFrameHeader(&act,&FrameBuffer);

   if (res)
      {
      Printf("Super Widget Error on i = %d \n",i);
      printf("Error %ld - %ls\n",res,Error2String(res));
      bill2 = CallARexxFunc(rob1,"PROJECT_REXX_PORT","REQ_ERROR",0,1,Error2String(res)); 
      if (!bill2)
         printf("It didn't work\n");
      else
         result = get_reply(rob1);
      goto ending;;
      }
   printf("%d   %ld  \n",j,FrameBuffer.Length);
   }

Total = Total + 1;

ending:

ARexxFree(rob1);

if (RexxSysLib)
   CloseLibrary(RexxSysLib);

CloseLibrary(FlyerBase);

return(0);
}

ULONG WriteClipStart(ULONG frames, ULONG Length,char *pathname)
{
BPTR	handle;
struct ClipHeader *ch;
UWORD i;
UBYTE BlkBuff[512];

handle = Open(pathname,MODE_OLDFILE);

if (!handle) 
   {
   printf("Did not write clip successfully\n");
   return(1);
   }

/* Build and write Clip header */
ch = (struct ClipHeader *)&BlkBuff;
memset(ch,0,sizeof(struct ClipHeader));

/* Read the Ignore Stuff */
Read(handle,BlkBuff,512);

i = Close(handle);

ch->Fields = frames * 4;
ch->DataEnd = Length-1;
ch->Tail = 0;
ch->Length = Length;

					// No index table!
handle = Open(pathname,MODE_OLDFILE);
if (!handle) 
   {
   printf("Did not write clip successfully\n");
   return(1);
   }

if (Write(handle,ch,sizeof(struct ClipHeader)) != sizeof(struct ClipHeader))
   printf("Clip header write error\n");

i = Close(handle);

return(0);		// Okay
}

ULONG Reprocess(char *pathname, ULONG frames)
{
struct FlyerVolume FlyVol,*volptr;
struct ClipAction	 Action,*actptr;
ULONG	res;

actptr = &Action;
volptr = &FlyVol;

/* Let Flyer find original clip and prepare to process it */

volptr->Board = 0;
volptr->Path = pathname;		// Full path and filename (on Flyer drive)
volptr->Flags = 0;							// None
actptr->Volume = volptr;					// Link together
actptr->ReturnTime = RT_STOPPED;			// Don't return until done

res = StartClipCutList(actptr,1);		// Ready for destructive reprocessing
if (res) 
   {
   printf("Error %ld on StartClipCutList - %ls\n",res,Error2String(res));
   return(res);
   }

/* Now define one clip to make from original */

volptr->Board = 0;
volptr->Path = pathname;					// Make new clip the same name
volptr->Flags = 0;							// None
actptr->Volume = volptr;					// Link together
actptr->ReturnTime = RT_STOPPED;			// Don't return until done
actptr->AudStartField = 0;					// Reprocess all of clip
actptr->AudFieldCount = frames * 4;
actptr->VidStartField = 0;					// Reprocess all of clip
actptr->VidFieldCount = frames * 4;

actptr->Flags = CAF_REPROCESS;		// Force Flyer to re-process this!

actptr->Flags |= CAF_VIDEO | CAF_AUDIOL | CAF_AUDIOR;

res = AddClipCut(actptr);					// Define the new clip
if (res) 
   {
   printf("Error %ld on AddClipCut - %ls\n",res,Error2String(res));
   res = EndClipCutList(0);				// Let Flyer clean up
   return(res);
   }

res = EndClipCutList(1);				// Do it now!!!
if (res) 
   {
   printf("Error %ld on EndClipCutList - %ls\n",res,Error2String(res));
   }

return(FERR_OKAY);
}

LONG get_reply(ARexxContext  context)
{
   struct RexxMsg   *reply;
   unsigned long    i;
   LONG             temp_code;

   /* poll for reply */
   i=0;
      while ( ((reply = GetARexxMsg(context)) == NULL) && (++i < 1000) )
      {
         Delay(5);
      }
 

   if (i == 1000)
   {
      printf("timeout\n");
      /* did not get a response in a long time, just return */
      return(0);
   }
   else
   {
            if (IsARexxReply( reply) )
            { 
         temp_code = reply->rm_Result1;
         FreeARexxMsg(context, reply);
      }
      /* otherwise, don't care */
   }
   return( temp_code);
}

ARexxContext ARexxInit(char *name, char *extension, BOOL multiple)
{
struct Library *RexxSysBase;
struct MsgPort *port;
ARexxContext  context;
unsigned count;
char  *ptr;

context = AllocMem(sizeof(struct ARexxContext), MEMF_PUBLIC | MEMF_CLEAR);

if (!context)
   return(NULL);

RexxSysBase = OpenLibrary((UBYTE *) "rexxsyslib.library",0L);

if (RexxSysBase)
   {
   context->rexx_base = RexxSysBase;
   if (!extension)
      extension = "rexx";
   StrNUpper(context->extension, extension, MAX_EXTENSION);
   ptr = StrNUpper(context->port_name, name, MAX_PORT_NAME);
#ifdef RVI
   strcpy(context->error_name, context->port_name);
   strcat(context->error_name, ".LASTERROR");
#endif

   Forbid();
   if (multiple)
      {
      *ptr++='.';
      for (count = 1; count < 100; ++count)
         {
         UiToStr(count, ptr);
         port = FindPort((UBYTE *) context->port_name);
         if (!port) break;
         }
      }
   else
      {
      port = FindPort((UBYTE *) context->port_name);
      }

   if (!port)
      context->arexx_port = CreatePort ((UBYTE *) context->port_name,0);
   Permit();
   }

if (!context->rexx_base || !context->arexx_port)
   {
   ARexxFree(context);
   context = NULL;
   }

return(context);
}

void ARexxFree (ARexxContext context)
{
struct RexxMsg *msg;
struct Library *RexxSysBase;

if (!context)
   return;

if (context->rexx_base)
   {
   RexxSysBase = context->rexx_base;
   context->port_name[0] = NULLCHAR;
   while (context->outstanding > 0 && context->arexx_port != NULL)
      {
      WaitPort(context->arexx_port);
      while ((msg = GetARexxMsg(context) ) != NULL)
         {
         if (!FreeARexxMsg(context, msg))
            ReplyARexxMsg(context, msg, 100, NULL, 0);
         }
      }
   if (context->arexx_port)
      {
      Forbid();
      while ((msg = GetARexxMsg(context)) != NULL)
         ReplyARexxMsg(context, msg, 100, NULL, 0);
      DeletePort(context->arexx_port);
      Permit();
      }
   CloseLibrary(RexxSysBase);
   }

FreeMem(context, sizeof(struct ARexxContext));

}

char *ArexxName(ARexxContext context)
{
if (context)
   return(context->port_name);
else
   return(NULL);
}

ULONG ARexxSignal(ARexxContext context)
{
if (context)
   return(1UL << context->arexx_port->mp_SigBit);
else
   return(0);
}

struct Library *ARexxBase(ARexxContext context)
{
if (context)
   return(context->rexx_base);
else
   return(NULL);
}

BOOL IsARexxReply( struct RexxMsg *msg)
{
if (msg)
   return( (BOOL)(msg->rm_Node.mn_Node.ln_Type == NT_REPLYMSG ));
else
   return(FALSE);
}

struct RexxMsg *GetARexxMsg(ARexxContext context)
{
struct RexxMsg *msg;

if (!context)
   return(NULL);

msg = (struct RexxMsg *) GetMsg(context->arexx_port);

if (!msg)
   return(NULL);

if (IsARexxReply(msg))
   --context->outstanding;

return(msg);
}

BOOL FreeARexxMsg(ARexxContext context, struct RexxMsg *msg)
{
struct Library *RexxSysBase;

if (!context || !msg)
   return(FALSE);

if (!IsARexxReply(msg))
   return(FALSE);

RexxSysBase = context->rexx_base;

ClearRexxMsg(msg, 16);

if (msg->rm_Result1 == 0 && msg->rm_Result2 != NULL)
   DeleteArgstring((char *) msg->rm_Result2);

DeleteRexxMsg(msg);

return(TRUE);

}

struct RexxMsg *SendARexxMsg(ARexxContext context, char *port_name,
              char *command, ULONG flags)
{
struct MsgPort *port;
struct RexxMsg *msg;
struct Library *RexxSysBase;

if (!context || !command)
   return(NULL);

if (!port_name)
   port_name = "REXX";

RexxSysBase = context->rexx_base;

msg = CreateRexxMsg(context->arexx_port, context->extension,
              context->port_name);

if (!msg)
   return(NULL);

flags &= ( RXFF_NOIO | RXFF_RESULT | RXFF_STRING | RXFF_TOKEN );

msg->rm_Action = RXCOMM | flags;

msg->rm_Args[0] = (STRPTR) CreateArgstring(command, strlen(command));

msg->rm_Result1 = 0;

msg->rm_Result2 = (LONG) NULL;

if (!msg->rm_Args[0])
   {
   DeleteRexxMsg(msg);
   return(NULL);
   }

Forbid();

if ((port = FindPort((UBYTE *) port_name )) != NULL)
   {
   PutMsg(port, (struct Message *) msg);
   ++context->outstanding;
   }
else
   {
   DeleteArgstring((char *) msg->rm_Args[0]);
   DeleteRexxMsg(msg);
   msg = NULL;
   }

Permit();
return(msg);

}


struct RexxMsg *CallARexxFunc (ARexxContext context, char *port_name,
       char *function, ULONG flags, int num_args, ...)
{
va_list        argptr;
struct MsgPort *port;
struct RexxMsg *msg;
struct Library *RexxSysBase;
BOOL   quit;
int   i;
char *arg;

if (!context || !function || num_args < 0)
   return(NULL);

if (num_args > 15)
   num_args = 15;

if (!port_name)
   port_name = "REXX";

RexxSysBase = context->rexx_base;

msg = CreateRexxMsg(context->arexx_port, context->extension,
            context->port_name);

if (!msg)
   return(NULL);

flags &= (RXFF_NOIO | RXFF_RESULT | RXFF_STRING);

msg->rm_Action = RXFUNC | flags;

msg->rm_Action |= ( num_args & RXARGMASK);

msg->rm_Args[0] = (STRPTR) CreateArgstring(function, strlen(function));

msg->rm_Result1 = 0;
msg->rm_Result2 = (LONG) NULL;

if (!msg->rm_Args[0])
   {
   DeleteRexxMsg(msg);
   return(NULL);
   }

va_start (argptr, num_args);

for (i=1, quit=FALSE; i<= num_args && !quit; ++i)
   {
   arg = (char *) va_arg(argptr, char *);

   if (!arg)
      arg = "";

   msg->rm_Args[i] = (STRPTR) CreateArgstring(arg, strlen(arg));

   if (!msg->rm_Args[i])
      quit = TRUE;

   }

va_end(argptr);

if (quit)
   {
   ClearRexxMsg(msg, 16);
   DeleteRexxMsg(msg);
   return(NULL);
   }

Forbid();

if ((port=FindPort((UBYTE *) port_name)) != NULL)
   {
   PutMsg(port, (struct Message *) msg);
   ++context->outstanding;
   }
else
   {
   ClearRexxMsg(msg, 16);
   DeleteRexxMsg(msg);
   msg = NULL;
   }


Permit();

return(msg);

}


BOOL  ReplyARexxMsg( ARexxContext context,  struct RexxMsg  *msg,
                     LONG return_code,  char *result_string,
                     LONG error_code)
{

   struct Library  *RexxSysBase;

   if ( !context || !msg )
      return( FALSE );

   if ( IsARexxReply (msg) )
      return( FALSE );

   /* setup the library pointer */

   RexxSysBase = context->rexx_base;

   /* Set the results */

   msg->rm_Result1 = return_code;
   msg->rm_Result2 = (LONG) NULL;

   /* if no error and result was requested, set the result string */

   if ( result_string != NULL && return_code == RC_OK
        && (msg->rm_Action & RXFF_RESULT) != 0 )
      {
       msg->rm_Result2 = (LONG) CreateArgstring( result_string,
                                 strlen( result_string) );
      }
   else if  (return_code != RC_OK)
      {
         msg->rm_Result2 = error_code;
      }

   ReplyMsg( (struct Message *)msg );

   return(TRUE);
}



BOOL AddFuncHost (ARexxContext context,  LONG priority)
{

   struct RexxMsg  *msg;
   struct Library  *RexxSysBase;
   struct MsgPort  *port;

   if( !context )
      return(FALSE);

   /* setup the library pointer */

   RexxSysBase = context->rexx_base;

   /* Get a new message packet */

   msg = CreateRexxMsg( context->arexx_port, context->extension,
                        context->port_name);

   if (!msg)
      return(FALSE);


   msg->rm_Action = RXADDFH | RXFF_NONRET;
   msg->rm_Args[0] = (STRPTR) context->port_name;
   msg->rm_Args[1] = (STRPTR) priority;

   /* find the port */

   Forbid();

   if( ( port = FindPort( (UBYTE *) "REXX" ) ) != NULL )
   {
      PutMsg( port, (struct Message *) msg);
   }
   else
   {
      ClearRexxMsg( msg, 16);
      DeleteRexxMsg( msg );
      msg = NULL;
   }

   Permit();

   return( (BOOL)(msg != NULL) );
}


BOOL RemFuncHost( ARexxContext context)
{
   struct RexxMsg  *msg;
   struct Library  *RexxSysBase;
   struct MsgPort  *port;

   if (!context)
      return(FALSE);

   /* setup the library pointer */

   RexxSysBase = context->rexx_base;

   /* get a new message packet */

   msg = CreateRexxMsg (context->arexx_port, context->extension,
                        context->port_name);

   if (!msg)
      return(FALSE);

   /* tell the resident process to remove the port name to the Library
    * List... we don't expect a reply
    */

   msg->rm_Action   = RXREMLIB | RXFF_NONRET;
   msg->rm_Args[0]  = (STRPTR)  context->port_name;

   /* find the port */

   Forbid();

   if( ( port = FindPort( (UBYTE *) "REXX") ) != NULL )
   {
      PutMsg( port, (struct Message *) msg);
   }
   else
   {
      ClearRexxMsg( msg, 16);
      DeleteRexxMsg (msg);
   }

   Permit();

   return( (BOOL)(msg != NULL) );
}




BOOL  SetARexxLastError( ARexxContext context, struct RexxMsg *msg,
                        char *error)
{

   #ifndef RVI
      return(FALSE);
   #else
      if (!context || !msg || !error)
         return(FALSE);

      if (!CheckRexxMsg (msg) )
         return(FALSE);

      return( SetRexxVar( msg, context->error_name,
                           error, strlen(error) ) == 0 )
   #endif
}




static char *StrNUpper( char *dest,  char *src,  unsigned int len)
{

   for( ; *src != NULLCHAR && len-- > 0; ++src)
   {
      if (islower (*src))
         *dest++ = toupper( *src );
      else
         *dest++ = *src;
   }

   *dest = NULLCHAR;

   return(dest);
}



static char *UiToStr( unsigned num, char *buf)
{
   num %= 100;

   buf[0] = (num / 10) + '0';
   buf[1] = (num % 10) + '0';
   buf[2] = NULLCHAR;

   return(buf);
}



