/*********************************************************************\
*
* FlyerProc - Processes D2 data out of source clip into new clip
*
* $Id: FlyerProc.c,v 1.0 1995/08/31 15:48:35 Flick Exp $
*
* $Log: FlyerProc.c,v $
*Revision 1.0  1995/08/31  15:48:35  Flick
*FirstCheckIn
*
*
* Copyright (c) 1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	08/23/95		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>
#include <dos.h>

#include "flyerlib.h"
#include "flyer.h"
#include "FlyerPrivate.h"

#define	DBUG(x)		/* x */

void Cleanup(void);
ULONG ReadField(char *name, ULONG field, BOOL IlaceRvs);
ULONG WriteField(char *name, ULONG field, UBYTE *buff);
UBYTE *CorrectPhase(UBYTE srcfld, BOOL IlaceRvs, UBYTE dstfld);
void __asm InvertField(register __a0 UBYTE *src, register __a1 UBYTE *dest);

struct Library *FlyerBase;

struct FlyerVolume	FlyVol;
struct ClipAction		Action;
struct VidCompInfo	CompInfo;
struct ClipInfo		Cinfo;

UBYTE	*BigBuffer,*InvBuffer;
BOOL	Abort = FALSE, Corrected = FALSE;
BOOL	FIR = FALSE;

int Breaker(void)
{
	printf("Aborting\n");

	Abort = TRUE;

	return(0);			// Continue for now
}


main(int argc,char **argv)
{
	ULONG	res;
	LONG	lofield,hifield,destfield,srcfield,startfield,stopfield,fields,srcmod,destdupe;
	char	*srcname,*dstname,*parse,chr;
	int	i;
	BOOL	reverse,IlaceRvs,strobe=FALSE;
	UBYTE	*buff;

	if ((argc > 6) && (stricmp(argv[argc-1],"FIR")==0))
	{
		FIR = TRUE;
		argc--;
	}

	if ((argc!=6)||(argv[1][0]=='?')) {
		printf("Usage: %ls <src pathname> <low field> <high field> <dest pathname> <process> [FIR]\n",argv[0]);
		printf("Currently supported processes:\n");
		printf("   x#   for speed-up       (i.e. x2)\n");
		printf("   /#   for slow-motion    (i.e. /3)\n");
		printf("   s#,# for strobe effect  (hold#,every#)\n");
		printf("   -    for reverse motion (examples: -x1 or -/2)\n");
		printf("\nVersion 1.0\n");
		printf("Copyright (c) 1995, NewTek, Inc.\nWritten by Marty Flickinger\n");
		return(20);
	}

   onbreak(Breaker);

	srcname = argv[1];
	stcd_l(argv[2],&lofield);
	stcd_l(argv[3],&hifield);
	dstname = argv[4];

	parse = argv[5];
	reverse = FALSE;	srcmod = 1;	 destdupe = 1;		// Defaults
	if (*parse == '-')
	{
		reverse = TRUE;
		parse++;
	}

	chr = *parse++;
	chr = toupper(chr);

	if (chr == '/')
	{
		stcd_i(parse,&i);
		destdupe = i;		// "/#"
	}
	else if (chr == 'X')
	{
		stcd_i(parse,&i);
		srcmod = i;			// "x#"
	}
	else if (chr == 'S')
	{
		parse += stcd_i(parse,&i);
		destdupe = i;			// "s#,#"
		stcd_i(parse+1,&i);
		srcmod = i;

		strobe = TRUE;
//		srcmod += destdupe;		// Modulo is copy+skip
	}
	else
	{
		printf("Invalid argument '%c'\n",chr);
		return(20);
	}

//	// Automatically shut off FIR filtering if x1 reverse (color phase is always right)
// THIS IS NO LONGER TRUE, AS WE HAVE TO SHIFT A LINE TO CORRECT FOR INTERLACE!!!
//	if ((srcmod==1) && (destdupe==1) && (reverse))
//		FIR = FALSE;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0)
	{
		printf("Open flyer.library failed\n");
		Cleanup();
		return(10);
	}

	BigBuffer = (UBYTE *)AllocVec(TOASTERLINES*SAMPLESPERLINE, MEMF_PUBLIC);
	InvBuffer = (UBYTE *)AllocVec(TOASTERLINES*SAMPLESPERLINE, MEMF_PUBLIC);
	if ((!BigBuffer) || (!InvBuffer))
	{
		printf("Couldn't allocate necessary buffer(s)\n");	
		Cleanup();
		return(10);
	}

	FlyVol.Board = 0;
	FlyVol.Flags = 0;
	FlyVol.SCSIdrive = 0;
	FlyVol.Path = srcname;
	Cinfo.len = sizeof(struct ClipInfo);
	res = GetClipInfo(&FlyVol,&Cinfo);
	if (res)
		printf("ClipInfo Error %ld - %ls\n",res,Error2String(res));
	else
	{
		// Clip in/out points to legal values
		if (lofield >= Cinfo.Fields)
			lofield = Cinfo.Fields-1;
		if (hifield >= Cinfo.Fields)
			hifield = Cinfo.Fields-1;

		fields = hifield-lofield+1;

		//****************************//
		// Effective new clip speed = srcmod/destdupe
		// Sign of srcmod determines direction of motion
		printf("Generating %ld fields ",((fields+srcmod-1)/srcmod)*destdupe);
		if (destdupe>1)
			printf("at %ld/%ld speed",srcmod,destdupe);
		else
			printf("at %ldx speed",srcmod);

		if (strobe)
			printf(" (strobed)");
		if (reverse)
			printf(" in reverse");
		printf("\n");
		//****************************//

		if (reverse)
		{
			startfield = hifield;
			stopfield = startfield-fields;
			srcmod = -srcmod;						// Step backwards thru source
		}
		else
		{
			startfield = lofield;
			stopfield = startfield+fields;
		}



		//*** Here we go! ***//

		srcfield = startfield;
		destfield=0;
		while (!Abort)
		{
			if (reverse)
			{
				if (srcfield <= stopfield)
					break;
			}
			else
			{
				if (srcfield >= stopfield)
					break;
			}

			if ((srcmod==-1) && (destdupe==1) && (!(srcfield & 1)))
				IlaceRvs = TRUE;
			else
				IlaceRvs = FALSE;

			res = ReadField(srcname,srcfield,IlaceRvs);			// Read source field
                        printf("This is a Read %d \n",srcfield);
			if (res != FERR_OKAY)	break;

			for (i=0; (i<destdupe) && (!Abort); i++)
			{
				buff = CorrectPhase(srcfield,IlaceRvs,destfield);	// Correct color phase, if required

				printf("Writing field %ld...\n",destfield+1);

				res = WriteField(dstname,destfield,buff);	// Write new output field
				DBUG(printf("WF result %ld\n",res);)
				if (res != FERR_OKAY)	break;
				destfield++;
			}

			srcfield += srcmod;

			if (res != FERR_OKAY)
			{
				printf("Stopped on error %ld - %ls\n",res,Error2String(res));
				break;
			}
			chkabort();
		}
	}

	if (Abort)
		printf("Wrote %ld fields before aborting\n",destfield);

	Cleanup();

	return(0);
}


void Cleanup(void)
{
	if (BigBuffer)		FreeVec(BigBuffer);
	if (InvBuffer)		FreeVec(InvBuffer);
	if (FlyerBase)		CloseLibrary(FlyerBase);
}


ULONG ReadField(char *name, ULONG field, BOOL IlaceRvs)
{
	int	line;
	UBYTE	*buffer,modes;
	ULONG	res = FERR_OKAY;

	Corrected = FALSE;		// Invalidate inverted field data

	DBUG(printf("read ");)

	FlyVol.Board = 0;
	FlyVol.Flags = 0;
	FlyVol.SCSIdrive = 0;
	FlyVol.Path = name;
	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_STOPPED;

	modes = 0;
	if (!FIR)
		modes |= FWF_NOFIR;								// No FIR filtering required

	res = OpenReadField(&Action,field,modes);
	if (res)
		printf("\nRead open error %ld - %ls\n",res,Error2String(res));
	else
	{
		buffer = BigBuffer;

          if ((field&1)== 0)
		{
                printf("Lets Skip Lines \n");
        	Action.ReturnTime = RT_STOPPED;
		SkipLines(&Action,1);
		DBUG(printf("Skip1\n");)
		}

		for (line=0; line<TOASTERLINES; line++)
		{
			Action.ReturnTime = RT_STOPPED;
			res = FlyerReadLine(&Action,buffer);
			if (res)
			{
				printf("\nRead error %ld on line %ld - %ls\n",res,line,Error2String(res));
				CloseField(&Action);
				return(res);
			}
			buffer += SAMPLESPERLINE;

			chkabort();
		}

		res = CloseField(&Action);
		if (res)
			printf("\nRead close error %ld - %ls\n",res,Error2String(res));
	}

	return(res);
}


ULONG WriteField(char *name, ULONG field, UBYTE *buffer)
{
int	toler,line;
UBYTE	*buff,modes;
ULONG	res = FERR_OKAY,res2 = FERR_OKAY;

DBUG(printf("write\n");)

FlyVol.Board = 0;
FlyVol.Flags = 0;
FlyVol.SCSIdrive = 0;
FlyVol.Path = name;
Action.Volume = &FlyVol;
Action.ReturnTime = RT_STOPPED;

for (toler=0; toler<4; toler++)
   {
   DBUG(printf("Tolerance %ld\n",toler);)

   CompInfo.Algo = (toler<2) ? ALGO_D2 : ALGO_SN;
   CompInfo.Tolerance = toler & 1;
   CompInfo.RndFreq = 122;
   CompInfo.FIRcomp = 2;
   CompInfo.DataSize = 0;		//Let Flyer pick

   modes = FWF_APPEND;
		if (field==0)
			modes |= FWF_NEW;
		if (toler > 0)
			modes = FWF_REWRITE | FWF_APPEND;		// NEW not needed here???
		if (!FIR)
			modes |= FWF_NOFIR;								// No FIR filtering required

		res = OpenWriteField(&Action,field,modes,&CompInfo);
		if (res)
		{
			printf("\nWrite open error %ld - %ls\n",res,Error2String(res));
			toler = 99;			// Break out, don't try any other tolerances!
		}
		else
		{
			buff = buffer;

                if (((field/4)&1) == 0)
                  {
                  if ((field%4) == 0)
                     SkipLines(&Action,1);
                   if ((field%4) == 2)
                      SkipLines(&Action,1);
                   }

			for (line=0; line<TOASTERLINES; line++)
			{
				Action.ReturnTime = RT_STOPPED;
				res = FlyerWriteLine(&Action,buff);
				if (res)
				{
					if (res != FERR_FULL)
						printf("\nWrite error %ld on line %ld - %ls\n",res,line,Error2String(res));
					else if (res == FERR_FULL)
					{
						DBUG(printf("FULL on line %ld\n",line);)
					}
					break;
				}
				buff += SAMPLESPERLINE;

				chkabort();
			}

			res2 = CloseField(&Action);
			if (res2 == FERR_FULL)
			{
				DBUG(printf("FULL on close\n");)
			}
			if ((res2) && (res2 != FERR_FULL))
				printf("\nWrite close error %ld - %ls\n",res2,Error2String(res2));

			if (res == FERR_OKAY)		// If WriteLine didn't fail, let "close" error show thru
				res = res2;
		}

		DBUG(printf("Result %ld\n",res);)

		if (res == FERR_OKAY)			// Succeeded
			break;
		else if (res != FERR_FULL)		// Something other than "Full"
			break;
		// If FERR_FULL, try again at next quality down

		if (Abort)
			break;

	}

	DBUG(printf("Result2 %ld\n",res);)

	return(res);
}


UBYTE *CorrectPhase(UBYTE srcfld, BOOL IlaceRvs, UBYTE dstfld)
{
int	diffbits;
UBYTE	*buffer;

if (IlaceRvs)
   srcfld ^= 1;		// Flip to other interlace field ( I<->II or III<->IV )

diffbits = (srcfld ^ dstfld) & 3;		// XOR low 2 bits

//if (srcfld%4 == 0)
//   diffbits = 0;

if (((dstfld/4)&1) == 0)
   {
   if ((dstfld%4) == 0)
      diffbits = 0;
   if ((dstfld%4) == 2)
      diffbits = 0;
   if ((dstfld%4) == 1)
      diffbits = 1;
   if ((dstfld%4) == 3)
      diffbits = 1;
   }





// Fields I & IV have same phase, as do fields II and III
if ((diffbits == 0) || (diffbits == 3))	// No correction needed?
   buffer = BigBuffer;							// If not, just write out original field
else
   {
   // We need inverted phase data...
   printf("This is the wrong phase \n");
   if (Corrected)
      {
			DBUG(printf("(Already corrected)\n");)
		}
		else
		{
                        printf("Let's Correct it \n");
			InvertField(BigBuffer,InvBuffer);  
			Corrected = TRUE;
			DBUG(printf("Corrected\n");)
		}

		buffer = InvBuffer;
	}

	return(buffer);		// Which one to write out
}
