/*********************************************************************\
*
* BootDSP - Download and boot DSP program
*				(This version accepts Intel Hex files)
*
* $Id: BootDSP.c,v 1.0 1995/05/05 15:48:03 Flick Exp $
*
* $Log: BootDSP.c,v $
*Revision 1.0  1995/05/05  15:48:03  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/23/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

int ReadFile(char *,int);
int IntelToBinary(UBYTE *,ULONG);
int ASCIItoBinary(char **ptr);
void FreeFiles(void);
void GetArg(char *arg,int *var,int lo,int hi,char *label);

struct Library *FlyerBase;

#define	OUTLENGTH	8192

UBYTE	*FileBuffer,*OutBuffer;
ULONG	FileLength;

main(int argc,char **argv)
{
	int	unit,binflag;
#if 0
	int	i,j;
#endif
	ULONG	res;
	LONG	size;

	if ((argc<3)||(argc>4)||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <file> [BIN]\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");

	if ((argc==4)&&(stricmp(argv[3],"BIN")==0))
		binflag = 1;
	else
		binflag = 0;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	size = ReadFile(argv[2],binflag);
	if (size) {
#if 0
		for (i=0;i<size;i+=16) {
			printf("%04lx: ",i);
			for (j=0;j<16;j++) {
				if ((i+j)<size)
					printf("%02lx ",OutBuffer[i+j]);
			}
			printf("\n");
		}
#endif
		if (binflag)
			res = DSPboot(unit,size,FileBuffer);
		else
			res = DSPboot(unit,size,OutBuffer);

		if (res)
			printf("Error %ld - %ls\n",res,Error2String(res));
		else
			printf("DSP download okay\n");
	}
	CloseLibrary(FlyerBase);
	FreeFiles();

	return(0);
}


int ReadFile(char *name,int binflag)
{
	char	filename[80];
	FILE	*fp;
	int	read;

	strcpy(filename,name);
	if (binflag) {
		if (stricmp(".bin",&filename[strlen(filename)-4]))
			strcat(filename,".bin");
	}
	else {
		if (stricmp(".BNM",&filename[strlen(filename)-4]))
			strcat(filename,".BNM");
	}

	read = 0;
	fp = fopen(filename,"r");
	if (fp) {
		fseek(fp,0,SEEK_END);
		FileLength = ftell(fp);
		fseek(fp,0,SEEK_SET);

		FileBuffer = AllocMem(FileLength,MEMF_PUBLIC);
		if (FileBuffer) {
			read = fread(FileBuffer,FileLength,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",filename);

	if (read==1) {
		if (binflag)
			return((int)FileLength);
		else
			return(IntelToBinary(FileBuffer,FileLength));
	}
	else
		return(0);
}


int IntelToBinary(UBYTE *buff,ULONG len)
{
	char 		*from,*end;
	int		length,run,addr,temp,data,cs;

	from = (char *)buff;
	end = (char *)buff + len;

	/* Allocate place to build binary data */
	OutBuffer = AllocMem(OUTLENGTH,MEMF_PUBLIC | MEMF_CLEAR);
	if (OutBuffer) {
		length = 0;
		while (from < end) {
			switch (*from++) {
			case 0x0A:
			case 0x0D:
				break;
			case ':':
				cs = 0;

				run = ASCIItoBinary(&from);
				if (run<0)
					return(0);

				addr = ASCIItoBinary(&from);
				if (addr<0)
					return(0);
				temp = ASCIItoBinary(&from);
				if (temp<0)
					return(0);
				addr = (addr<<8) + temp;

				temp = ASCIItoBinary(&from);
				if (temp<0)
					return(0);
				if (temp == 1) {
					temp = ASCIItoBinary(&from);
					break;
				}
				while (run--) {
					data = ASCIItoBinary(&from);
					if (data<0)
						return(0);
					OutBuffer[addr++] = (BYTE)data;
					cs += data;
				}
				if (addr > length)
					length = addr;

				/* Get checksum */
				temp = ASCIItoBinary(&from);
				if (temp<0)
					return(0);
/*				printf("Checksum %lx (mine was %lx)\n",temp,cs); */
				break;
			default:
				printf("Bad input format!\n");
				break;
			}
		}
	}
	else
		printf("Could not allocate memory\n");

	return(length);
}


int ASCIItoBinary(char **ptr)
{
	char 		*p,c1,c2;
	int		v;

	p = *ptr;

	c1 = toupper(*p);
	if (isxdigit(c1)) {
		p++;
		if (c1>'9')
			v = ((UBYTE)c1)-'A'+10;
		else
			v = ((UBYTE)c1)-'0';
		v *= 16;
		
		c2 = toupper(*p);
		if (isxdigit(c2)) {
			p++;
			if (c2>'9')
				v += ((UBYTE)c2)-'A'+10;
			else
				v += ((UBYTE)c2)-'0';

			*ptr = p;		/* Move user's pointer */
			return(v);
		}
		else printf("'%lc' is not a hex digit!\n",c2);
	}
	else printf("'%lc' is not a hex digit!\n",c1);

	return(-1);
}


void FreeFiles(void)
{
	if (FileBuffer)
		FreeMem(FileBuffer,FileLength);
	if (OutBuffer)
		FreeMem(OutBuffer,OUTLENGTH);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
