/*********************************************************************\
*
* CalibFlyer - Flyer hardware calibration
*
* $Id: CalibFlyer.c,v 1.0 1995/05/05 15:48:09 Flick Exp $
*
* $Log: CalibFlyer.c,v $
*Revision 1.0  1995/05/05  15:48:09  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	11/01/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/intuition.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>
#include <intuition/intuition.h>
#include <clib/dos_protos.h>
#include <clib/MathIeeeSingTrans_protos.h>
#include <pragmas/MathIeeeSingTrans_pragmas.h>
#include <libraries/mathieeesp.h>

#include "flyerlib.h"
#include "flyer.h"

#define	DBUGOPT(x)		if (debug & 1) { x }
#define	DBUG(x)			if (debug & 2) { x }
#define	DBUGACAL(x)		if (debug & 4) { x }
#define	DBUGVERB(x)		if (debug & 8) { x }
#define	DBUGRESULT(x)	if (debug & 16) { x }
#define	DBUGMATH(x)		if (debug & 32) { x }

/* Ped/Gain test */
#define	PGLINES			2		/* Number of lines to read when avging */
#define	PGPIXELSIN		40		/* Pixels in from edges to take samples */
#define	MAXCHROMA		2		/* Max chroma allowed for luma-only readings */
#define	LINTHRESHLO		-20
#define	LINTHRESHHI		20
#define	NTSC_BLANK		60
#define	NTSC_BLACK		71
#define	NTSC_WHITE		200

/* Auto-hue test */
#define	TARGVECTOR		123.0		/* Target vector for auto-hue */
#define	SAMPLEFIELDS	1			/* Fields to average per sample */
#define	SAMPLELINES		2			/* Lines per field to average per sample */
#define	RECPIXELSIN		248		/* Start pixel for averaging */
#define	RECQUADS			64			/* Number of quads to sample */
#define	BURSTTHRESHLO	-4
#define	BURSTTHRESHHI	4
#define	LOWOFFSET		20			/* Lowest pixel offset during record */
#define	STARTOFFSET		22			/* Start pixel offset during record */
#define	SKEWSETTLETIME	4			/* Ticks for new skew setting to settle */
#define	JITTER			1.0		/* Allowable jitter (+/-) */
#define	DFLTGOODENUF	0.2

#define	DELAYSAMPLES	10

#define	SLOWDOWN			1

int	NoiseThreshLo,NoiseThreshHi;
int	Lowest,Highest,debug,Optimize;
FLOAT	GoodEnuf;		/* Stop when we get this or better */

int EnsureStable(FLOAT err);
int CheckRange(int edgelo, int coarselo, int finelo,
					int edgehi, int coarsehi, int finehi,
					FLOAT close);
void GetArg(char *arg,int *var,int lo,int hi,char *label);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);
ULONG OpenLiveField(UBYTE field);
ULONG CloseLiveField(void);
ULONG GetNextLine(UBYTE *buffer);
int ReturnHit(void);
int AvgQuad(UBYTE *ptr,int *badvideo);
void YIQ(UBYTE *ptr);
int  AverageQuads(LONG *minI,LONG *minQ,LONG *plsI,LONG *plsQ);
void ShowVal(int val,int x,int y);
void ShowBlinker(int x,int y);
void ShowString(char *string,int x,int y);
int  WaitIntMsg(void);
int  GetIntMsg(void);
void FlushIntMsgs(void);
void PresetSliders(int count);
void SetupSlider(struct Gadget *gad,UWORD span, UWORD pos);
void Doit(int value,int saveit);
void Doit3(int which,int value,int saveit);
ULONG	ScrollBar2Pos(ULONG max,struct Gadget *gad);
void HandleSliders(int count);
void SaveToE2(void);
void PlayClip(int channel,char *name,int length);
void SliderLabels(int count);
FLOAT CalcBurstPhase(int skipframes,int *badvidflag);

#define	POSSPAN	752*2
#define	POSBASE	-752

struct	Window			*Window;
struct	IntuitionBase  *IntuitionBase;
struct	RastPort			*RP;

struct	IntuiMessage	*IntMsg;
ULONG		MIClass;
USHORT	theID;
struct	Gadget	*MIAddress;
int		AdjType,Toggle;
ULONG		oldclkpos,oldcrspos,oldfinepos,oldpospos;
ULONG		newclkpos,newcrspos,newfinepos,newpospos;

struct Library	*MathIeeeSingTransBase;
struct Library *FlyerBase;

struct FlyerVolume	FlyVol;
struct ClipAction		Action;

UBYTE Buffer[SAMPLESPERLINE];

BPTR	handle;

USHORT	BoxVectors[] = {104,14, 0,14, 0,0, 104,0, 104,14};
struct	Border BoxBorder = {-2,-2,1,0,JAM1,5,BoxVectors,NULL};

struct TextAttr MyFont = {(STRPTR)"topaz.font",TOPAZ_EIGHTY,0,0};

char ValueBuffer[60] = " ";
struct	IntuiText ValueText =
	{1,0,JAM2,0,0,&MyFont,(unsigned char *)&ValueBuffer,NULL};


#define	GID_CLKSLIDER		1
#define	GID_COARSESLIDER	2
#define	GID_FINESLIDER		3
#define	GID_POSSLIDER		4
#define	GID_EXIT				5


struct PropInfo PosSliderInfo = {
	AUTOKNOB|FREEHORIZ,0,0,
	0x0100,MAXBODY,
};

struct Image PosSliderImage;
struct Gadget PosSliderGadget = {
	NULL,
	20,20,600,10,
	GADGHCOMP,
	RELVERIFY,
	PROPGADGET,
	(APTR)&PosSliderImage,
	NULL,
	NULL,
	0,
	(APTR)&PosSliderInfo,
	GID_POSSLIDER,
	NULL
};

struct PropInfo ClkSliderInfo = {
	AUTOKNOB|FREEHORIZ,0,0,
	0x0100,MAXBODY,
};

struct Image ClkSliderImage;
struct Gadget ClkSliderGadget = {
	&PosSliderGadget,
	20,42,400,10,
	GADGHCOMP,
	RELVERIFY,
	PROPGADGET,
	(APTR)&ClkSliderImage,
	NULL,
	NULL,
	0,
	(APTR)&ClkSliderInfo,
	GID_CLKSLIDER,
	NULL
};


struct PropInfo CrsSliderInfo = {
	AUTOKNOB|FREEHORIZ,0,0,
	0x0100,MAXBODY,
};

struct Image CrsSliderImage;
struct Gadget CrsSliderGadget = {
	&ClkSliderGadget,
	20,64,400,10,
	GADGHCOMP,
	RELVERIFY,
	PROPGADGET,
	(APTR)&CrsSliderImage,
	NULL,
	NULL,
	0,
	(APTR)&CrsSliderInfo,
	GID_COARSESLIDER,
	NULL
};

struct PropInfo FineSliderInfo = {
	AUTOKNOB|FREEHORIZ,0,0,
	0x0100,MAXBODY,
};

struct Image FineSliderImage;
struct Gadget FineSliderGadget = {
	&CrsSliderGadget,
	20,86,400,10,
	GADGHCOMP,
	RELVERIFY,
	PROPGADGET,
	(APTR)&FineSliderImage,
	NULL,
	NULL,
	0,
	(APTR)&FineSliderInfo,
	GID_FINESLIDER,
	NULL
};

struct IntuiText ExitText = {
   1,0,JAM1,32,2,&MyFont,"Exit",NULL};
struct Gadget ExitGadget = {		/* Exit button */
	NULL,100,90,101,11,GADGHCOMP,RELVERIFY,BOOLGADGET,
   (APTR)&BoxBorder,NULL,&ExitText,0,NULL,
	GID_EXIT,
	NULL};

struct Menu Menu = {
	NULL,
	10,0,220,0,
	MENUENABLED,
	"Written by Marty Flickinger",
	NULL
};

struct NewWindow MiscWindow =
	{
	0,0,300,120,
	0,1,
	GADGETUP | INTUITICKS | CLOSEWINDOW,
	WINDOWCLOSE | WINDOWDEPTH | WINDOWDRAG | ACTIVATE | NOCAREREFRESH,
	NULL,		/* Gadget 1st */
	NULL,
	NULL,
	NULL,
	NULL,
	100,25,
	640,400,
	WBENCHSCREEN
};

struct NewWindow AutoHueWindow =
	{
	0,0,300,115,
	0,1,
	GADGETUP | CLOSEWINDOW,
	WINDOWCLOSE | WINDOWDEPTH | WINDOWDRAG | ACTIVATE | NOCAREREFRESH,
	&ExitGadget,		/* Gadget 1st */
	NULL,
	NULL,
	NULL,
	NULL,
	100,25,
	640,400,
	WBENCHSCREEN
};

struct NewWindow PedGainWindow =
	{
	0,0,640,130,
	0,1,
	GADGETUP | INTUITICKS | CLOSEWINDOW,
	WINDOWCLOSE | WINDOWDEPTH | WINDOWDRAG | ACTIVATE | NOCAREREFRESH,
	&PosSliderGadget,		/* Gadget 1st */
	NULL,
	NULL,
	NULL,
	NULL,
	100,25,
	640,400,
	WBENCHSCREEN
};

struct NewWindow RecPhaseWindow =
	{
	0,0,640,200,
	0,1,
	GADGETUP | MOUSEMOVE | CLOSEWINDOW | INTUITICKS,
	WINDOWCLOSE | WINDOWDRAG | WINDOWDEPTH | ACTIVATE | NOCAREREFRESH,
	&FineSliderGadget,		/* Gadget 1st */
	NULL,
	"Record phase test",
	NULL,
	NULL,
	100,25,
	640,400,
	WBENCHSCREEN
};

struct NewWindow PlayPhaseWindow =
	{
	0,0,640,200,
	0,1,
	GADGETUP | MOUSEMOVE | CLOSEWINDOW | INTUITICKS,
	WINDOWCLOSE | WINDOWDRAG | WINDOWDEPTH | ACTIVATE | NOCAREREFRESH,
	&FineSliderGadget,		/* Gadget 1st */
	NULL,
	"Play phase test",
	NULL,
	NULL,
	100,25,
	640,400,
	WBENCHSCREEN
};

struct TestType {
	char *typename;
	int	typenum;
};

#define	TEST_Genlock	1
#define	TEST_RefDelay	2
#define	TEST_PedGain	3
#define	TEST_AutoRec	4
#define	TEST_PlayA		5
#define	TEST_PlayB		6
#define	TEST_ManualRec	7
#define	TEST_Dump2		8
#define	TEST_Field2		9
#define	TEST_LineBin	10

struct TestType TestTypes[] = {
	"GC",TEST_Genlock,
	"RD",TEST_RefDelay,
	"PG",TEST_PedGain,
	"AR",TEST_AutoRec,
	"PA",TEST_PlayA,
	"PB",TEST_PlayB,

	"MR",TEST_ManualRec,
	"D2",TEST_Dump2,
	"F2",TEST_Field2,
	"LB",TEST_LineBin,
	0,0
};

main(int argc,char **argv)
{
	int	lo,hi,i,unit,test,retry,badline,do1,do2,do3,offnum;
	ULONG	res;
	FLOAT berr,besterr,btotal1,btotal2,result;
	int	offset,edge,coarse,fine;
	int	bestoffset,bestedge,bestcoarse,bestfine;
	int	badvideo,abort;
	FILE	*fp;
	UBYTE	data;
	UWORD	eeword;
	BPTR	filehandle;

	test = 0;
	if (argc >= 5) {
		stcd_i(argv[4],&debug);
		argc--;
	}

	if (argc >= 4) {
		stcd_i(argv[3],&Optimize);
		argc--;
	}

	if (argc >= 3) {
		stcd_i(argv[2],&i);
		GoodEnuf = ((FLOAT)i)/10.0;
		argc--;
	}
	else
		GoodEnuf = DFLTGOODENUF;
/*	printf("GoodEnuf = %.1f\n",GoodEnuf); */

	if (argc>=2) {
		for (i=0;TestTypes[i].typename;i++) {
			if (stricmp(TestTypes[i].typename,argv[1])==0) {
				test = TestTypes[i].typenum;
				break;
			}
		}
	}

	if ((argc!=2)||(argv[1][0]=='?')||(test==0)) {
		printf("Usage: %ls <test> [<stoperr> [<opt>]]\n",argv[0]);
		printf("where 'test' is\n");
		printf("GC  - Calibrate Genlock Controller\n");
		printf("RD  - Calibrate board's Reference Delay\n");
		printf("PG  - Calibrate Pedestal & Gain pots\n");
		printf("AR  - Auto-calibrate Record Phase\n");
		printf("PA  - Calibrate 'A' Play Phase\n");
		printf("PB  - Calibrate 'B' Play Phase\n");
		printf("\nOptimization level is 0 (worst) to 2 (best)\n");
		return(20);
	}

	IntuitionBase = (struct IntuitionBase *)OpenLibrary("intuition.library",0);
	if (!IntuitionBase) {
		printf("Can't open intuition.library\n");
		exit(20);
	}

	if((MathIeeeSingTransBase = OpenLibrary("mathieeesingtrans.library", 0)) == 0) {
		printf("Can't open MathIeeeSingTrans.library\n");
		exit(20);
	}

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	unit = 0;

	FlyVol.Board = unit;
	FlyVol.SCSIdrive = 0;
	FlyVol.Path = 0;
/*	Action.ReturnTime = RT_STOPPED; */

#if 0
	res = FIRcoef(unit,0,FIR100);
	Delay(25);
#endif

	handle = Input();
	
	switch (test) {
	case TEST_Genlock:
		MiscWindow.Title = "Genlock Calibration";
		result = 0.0;
		Window = (struct Window *)OpenWindow(&MiscWindow);
		if (Window) {
			SetMenuStrip(Window, &Menu);
			RP = Window->RPort;

			res = ReadEEreg(0,12,&eeword);
			if (eeword == 0xFFFF)
				ShowString("UNCALIBRATED",10,15);
			else
				ShowString("(already calibrated)",10,15);

			/* Unlock Flyer */
			SBusWrite(unit,0x44,0xD);
			Delay(10);

			if (res = SBusRead(unit,0x44,&data),(data & 0x80)) {
				ShowString("Waiting for Flyer to lose sync...",10,25);
				while (res = SBusRead(unit,0x44,&data),(data & 0x80)) { }
				Delay(50);
			}
			ShowString("Calibrating...                   ",10,25);

			res = SetFlooby(unit,0,4,0);		/* Tell PLL CPU to auto-cal and save */

			res = ReadEEreg(0,12,&eeword);
			if (eeword == 0xFFFF)
				ShowString("Failed",10,45);
			else {
				ShowString("Succeeded",10,45);
				result = 1.0;
			}

			/* Lock Flyer */
			SBusWrite(unit,0x44,0x5);
			Delay(80);

			FlushIntMsgs();
			ClearMenuStrip(Window);
			CloseWindow(Window);
		}

		break;
	case TEST_RefDelay:
		res = RecordMode(unit);

		MiscWindow.Title = "Reference Delay Calibration";
		MiscWindow.FirstGadget = &ExitGadget;

		Window = (struct Window *)OpenWindow(&MiscWindow);
		if (Window) {
			SetMenuStrip(Window, &Menu);
			RP = Window->RPort;

			res = ReadEEreg(0,16,&eeword);
			if (eeword == 0xFFFF)
				ShowString("UNCALIBRATED",10,15);
			else
				ShowString("(already calibrated)",10,15);

			ShowString("Select modulated ramp",10,25);
			Delay(100);
			ShowString("Calibrating...",10,35);

			retry = 0;
			abort = 0;
			for (;;) {
				res = SBusWrite(unit,0xF5,0x27);		/* Select 3.58 reference */
				NoiseThreshLo = LINTHRESHLO;
				NoiseThreshHi = LINTHRESHHI;
				btotal1 = 0.0;
				for (i=0;i<DELAYSAMPLES;i++) {
					btotal1 += CalcBurstPhase(1,&badvideo);
				}
				DBUGACAL(printf("3.58 total = %.1f\n",btotal1);)

				res = SBusWrite(unit,0xF5,0x22);		/* Select Toaster main */

				if (!WaitIntMsg()) {
					abort = 1;
					break;
				}
				if (MIClass == GADGETUP) {
					if (theID == GID_EXIT) {
						abort = 1;
						break;
					}
				}

				NoiseThreshLo = BURSTTHRESHLO;
				NoiseThreshHi = BURSTTHRESHHI;
				btotal2 = 0.0;
				for (i=0;i<DELAYSAMPLES;i++) {
					btotal2 += CalcBurstPhase(1,&badvideo);
				}
				DBUGACAL(printf("ModRamp total = %.1f\n",btotal2);)

				if (btotal1 < btotal2)
					btotal1 += (FLOAT)(DELAYSAMPLES*360);

				berr = btotal1 - btotal2;
				DBUGACAL(printf("Delay is %.1f\n",berr/10.0);)

/*				if ((berr>=0.0)&&(berr<100.0)) */
					break;
#if 0
				if (!WaitIntMsg()) {
					abort = 1;
					break;
				}
				if (MIClass == GADGETUP) {
					if (theID == GID_EXIT) {
						abort = 1;
						break;
					}
				}

				ShowString("Retrying...   ",10,35);
				ShowVal(++retry,98,35);
#endif
			}

			if (!abort) {
				i = (int)berr;
				DBUG(printf("Wrote %ld to EEPROM\n",i);)
				res = WriteEEreg(0,16,i);
			}

			res = ReadEEreg(0,16,&eeword);
			if ((eeword == 0xFFFF)||(abort))
				ShowString("Failed",10,45);
			else {
				ShowString("Succeeded",10,45);
				result = 999.0;
			}

			Delay(50);

			FlushIntMsgs();
			ClearMenuStrip(Window);
			CloseWindow(Window);
		}

		result = berr/10.0;
		break;
	case TEST_PedGain:
		res = RecordMode(unit);
		AdjType = 2;		/* Record */

#if 0
		/* Digitize Toaster main out -- Sync from Toaster Main too */
		res = FlyerInputSel(unit,FI_ToasterMain,FS_ToasterMain);
#endif

		Delay(10);		/* Wait for new input to settle */

		PedGainWindow.Title = "Pedestal/Gain test";

		Window = (struct Window *)OpenWindow(&PedGainWindow);
		if (Window) {
			SetMenuStrip(Window, &Menu);
			RP = Window->RPort;

			PresetSliders(1);

			SliderLabels(1);

			/* Calibrate input pedestal and gain */
			ShowString("Adjust gain and pedestal pots",10,35);
			ShowString("Click on close to exit",10,45);
			ShowString("Gain Error",10,75);
			ShowString("Ped Error",120,75);

			badline = 0;

			while (WaitIntMsg()) {
				HandleSliders(1);

				if (MIClass == GADGETUP) {
					if (theID == GID_EXIT) {
						break;
					}
				}

				DBUG(printf("*** CAPTURE ***\n");)

				res = OpenLiveField(0);
#if 0
				res = GetNextLine(Buffer);
				lo = AvgQuad(Buffer+20,&badvideo);
				hi = AvgQuad(Buffer+SAMPLESPERLINE-20,&badvideo);
#endif
				res = GetNextLine(Buffer);		/* Skip opposite phase line */
				lo = hi = 0;
				for (i=0;i<PGLINES;i++) {
#if 0
					res = GetNextLine(Buffer);		/* Skip opposite phase line */
#endif
					res = GetNextLine(Buffer);
					lo += AvgQuad(Buffer+PGPIXELSIN,&badvideo);
					if (badvideo)
						break;

					hi += AvgQuad(Buffer+SAMPLESPERLINE-PGPIXELSIN,&badvideo);
					if (badvideo)
						break;
				}
				res = CloseLiveField();

				hi /= PGLINES;
				lo /= PGLINES;

				if (badvideo) {
					ShowString("---",50,85);
					ShowString("---",160,85);
					ShowString("No test pattern or horizontal position needs adjusted",10,110);
					badline = 1;

					DBUGVERB(
					for (i=0;i<32;i+=4)
						YIQ(&Buffer[i]);
					for (i=SAMPLESPERLINE-32;i<SAMPLESPERLINE;i+=4)
						YIQ(&Buffer[i]);
					)
				}
				else {
/*					ShowBlinker(250,85); */
					ShowVal((hi-lo)-(NTSC_WHITE-NTSC_BLACK),50,85);
					ShowVal(lo-NTSC_BLACK,160,85);
					if (badline) {
						ShowString("                                                     ",10,110);
						badline = 0;
					}
				}
			}

			FlushIntMsgs();
			ClearMenuStrip(Window);
			CloseWindow(Window);
		}

#if 0
		res = FlyerInputSel(unit,FI_Toaster1,FS_ToasterMain);
#endif

		result = 0.0;
		break;
	case TEST_AutoRec:
		res = RecordMode(unit);

#if 0
		/* This causes Flyer to set baselines for 2 skew channels */
		res = SetFlooby(0,0,0x10,0);
#endif

#if 0
		/* Digitize Toaster main out -- Sync from Toaster Main too */
		res = FlyerInputSel(unit,FI_ToasterMain,FS_ToasterMain);
#endif

		Delay(10);		/* Wait for new input to settle */

		AutoHueWindow.Title = "Record auto-hue";

		Window = (struct Window *)OpenWindow(&AutoHueWindow);
		if (Window) {
			SetMenuStrip(Window, &Menu);
			RP = Window->RPort;

			ShowString("Auto calibrating record phase...",10,30);

			NoiseThreshLo = BURSTTHRESHLO;
			NoiseThreshHi = BURSTTHRESHHI;

			abort = 0;
			besterr = 9999.9;

			for (offnum=0,offset=STARTOFFSET;offnum<4;offnum++,offset++) {
				if (offset > (LOWOFFSET+3)) {
					offset = LOWOFFSET;
				}
				res = FlyerWriteCalib(0,CALIB_HRECOFFSETA,offset,0);

				/* Check 'edge' range to see if we can skip entire offset */
				DBUGOPT(printf("[%ld,xx,xx,xx]",offset);)
				do1 = CheckRange(0,0,0, 3,9,7, (float)100.0);

				for (edge=0;(do1)&&(edge<4);edge++) {
					res = FlyerWriteCalib(0,CALIB_ADC_PHASE_EDGE,edge,0);

					/* Check 'coarse' range to see if we can skip entire edge */
					DBUGOPT(printf("[%ld,%ld,xx,xx]",offset,edge);)
					do2 = CheckRange(-1,0,0, -1,9,7, (float)50.0);

					for (coarse=0;(do2)&&(coarse<10);coarse++) {
						res = FlyerWriteCalib(0,CALIB_ADC_PHASE_COURSE,coarse,0);

						/* Check 'fine' range to see if we can skip entire coarse */
						DBUGOPT(printf("[%ld,%ld,%ld,xx]",offset,edge,coarse);)
						do3 = CheckRange(-1,-1,0, -1,-1,7, (float)20.0);

						for (fine=0;(do3)&&(fine<8);fine++) {
							res = FlyerWriteCalib(0,CALIB_ADC_PHASE_FINE,fine,0);

							if (GetIntMsg()) {			/* Intuition message? */
								if ((MIClass == GADGETUP) && (theID == GID_EXIT)) {
									abort = 1;
									break;
								}
							}

							berr = CalcBurstPhase(1,&badvideo)-TARGVECTOR;
							DBUGACAL(printf("%ld,%ld,%ld,%ld (%.1f)\n",offset,edge,coarse,fine,berr);)
							if (badvideo) {		/* Skip this one if bad */
								DBUGRESULT(printf("(bad)\n");)
								berr = 0.0;
								continue;
							}
							else {
								DBUGRESULT(printf("(%.1f)\n",berr);)

								if (berr < 0.0)
									berr = -berr;

								if (berr < besterr) {
									if (EnsureStable(berr)) {	/* Stable enough to use? */
										besterr = berr;

										bestoffset = offset;
										bestedge = edge;
										bestcoarse = coarse;
										bestfine = fine;

										sprintf(ValueBuffer,"Best so far: %.1f    ",berr);
										PrintIText(RP,&ValueText,10,50);

										if (berr <= GoodEnuf) {
											abort = 1;
											break;
										}
									}
								}
							}
						}
						if ((Optimize==1)&&	/* Gross pixel error optimization? */
							(berr > 100.0))	/* More than 1.5 edge LSB's off? */
							break;				/* Then skip all 'coarse' values */
						if (abort)
							break;
					}
					if ((Optimize==1)&&	/* Gross pixel error optimization? */
						(berr > 100.0))	/* More than 90 off? */
						break;				/* Then skip all 'edge' values */
					if (abort)
						break;
				}
				if (abort)
					break;
			}

			FlushIntMsgs();
			ClearMenuStrip(Window);
			CloseWindow(Window);
		}

		if (besterr < 50) {
			res = FlyerWriteCalib(0,CALIB_HRECOFFSETA,bestoffset,0);
			res = FlyerWriteCalib(0,CALIB_ADC_PHASE_EDGE,bestedge,0);
			res = FlyerWriteCalib(0,CALIB_ADC_PHASE_COURSE,bestcoarse,0);
			res = FlyerWriteCalib(0,CALIB_ADC_PHASE_FINE,bestfine,0);
		}

		DBUGACAL(printf("Best is %ld/%ld/%ld/%ld (err = %.1f)\n",
		bestoffset,bestedge,bestcoarse,bestfine,besterr);)

#if 0
		/* Save to EEPROM */
		res = FlyerWriteCalib(0,CALIB_HRECOFFSETA,offset,1);
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_EDGE,edge,1);
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_COURSE,coarse,1);
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_FINE,fine,1);
#endif

		result = besterr;
/*			break;    -- fall thru into manual screen */

	case TEST_ManualRec:
		res = RecordMode(unit);
		AdjType = 2;

#if 0
		/* Digitize Toaster main out -- Sync from Toaster Main too */
		res = FlyerInputSel(unit,FI_ToasterMain,FS_ToasterMain);
#endif

		Delay(10);		/* Wait for new input to settle */

		Window = (struct Window *)OpenWindow(&RecPhaseWindow);
		if (Window) {
			SetMenuStrip(Window, &Menu);
			RP = Window->RPort;

			PresetSliders(4);

			SliderLabels(4);

			ShowString("Adjust record phase (to near 0 error)",10,125);
			ShowString("Click on close box to save and exit",10,170);

			NoiseThreshLo = BURSTTHRESHLO;
			NoiseThreshHi = BURSTTHRESHHI;

			while (WaitIntMsg()) {
				HandleSliders(4);

				berr = CalcBurstPhase(1,&badvideo)-TARGVECTOR;

/*				ShowBlinker(150,150); */

				if (badvideo)
					sprintf(ValueBuffer,"-BAD-    ");
				else
					sprintf(ValueBuffer,"%.1f    ",berr);
				PrintIText(RP,&ValueText,80,150);
			}

/*			if (!badvideo) */
			SaveToE2();

			ClearMenuStrip(Window);
			CloseWindow(Window);
		}

#if 0
		res = FlyerInputSel(unit,FI_Toaster1,FS_ToasterMain);
#endif

		result = berr;
		break;
	case TEST_PlayA:
	case TEST_PlayB:
		res = PlayMode(unit);
		AdjType = (test==TEST_PlayA)?0:1;

		Window = (struct Window *)OpenWindow(&PlayPhaseWindow);
		if (Window) {
			SetMenuStrip(Window, &Menu);
			RP = Window->RPort;

			PresetSliders(4);

			SliderLabels(4);

/*			ShowString("Display Toaster Input 3",10,115); */
/*			ShowString("Display Toaster Input 4",10,115); */
			ShowString("Adjust phase for proper bars on vectorscope",10,125);
			ShowString("Click on close box to save and exit",10,170);

/*			PlayClip(0,"SMPTEbars",4); */
/*			PlayClip(1,"SMPTEbars",4); */

			while (WaitIntMsg()) {
				HandleSliders(4);
			}
			SaveToE2();
			ClearMenuStrip(Window);
			CloseWindow(Window);
		}

		result = 0.0;
		break;
	case TEST_Dump2:
		printf("Select modulated ramp\n");

		res = OpenLiveField(0);

		res = GetNextLine(Buffer);

#if 0
		for (i=0;i<SAMPLESPERLINE;i+=4) {
			YIQ(&Buffer[i]);
		}

		printf("Hit return to see next line\n");
		while (!ReturnHit()) {}

		res = GetNextLine(Buffer);

		res = CloseLiveField();

		for (i=0;i<SAMPLESPERLINE;i+=4) {
			YIQ(&Buffer[i]);
		}

		printf("Hit return to see next line\n");
		while (!ReturnHit()) {}

		res = OpenLiveField(SLOWDOWN);

		res = GetNextLine(Buffer);
#endif

		res = CloseLiveField();

		for (i=0;i<SAMPLESPERLINE;i+=4) {
			YIQ(&Buffer[i]);
		}

		result = 0.0;
		break;
	case TEST_LineBin:
		res = OpenLiveField(0);
		res = GetNextLine(Buffer);
		res = CloseLiveField();

		filehandle = Open("RAM:junk",MODE_NEWFILE);
		if (filehandle) {
			Write(filehandle,Buffer,SAMPLESPERLINE);
			Close(filehandle);
		}

		result = 0.0;
		break;
	case TEST_Field2:
		res = OpenLiveField(0);
		res = GetNextLine(Buffer);
		res = CloseLiveField();
		res = OpenLiveField(1);
		res = GetNextLine(Buffer);
		res = CloseLiveField();

		for (i=0;i<SAMPLESPERLINE;i+=4) {
			YIQ(&Buffer[i]);
		}

		result = 0.0;
		break;
	}

#if 0
	res = FIRcoef(unit,0,FIR050);
#endif

	/* Save result in environment var */
	fp = fopen("ENV:CalibResult","w");
	if (fp) {
		sprintf(ValueBuffer,"%.1f",result);
		fwrite(ValueBuffer,strlen(ValueBuffer),1,fp);
		fclose(fp);
	}

	CloseLibrary(FlyerBase);
	CloseLibrary(MathIeeeSingTransBase);
	CloseLibrary((struct Library *)IntuitionBase);

	return(0);
}


int EnsureStable(FLOAT err)
{
	int	i,badvideo;
	FLOAT	berr,diff;

	if (err > 3.0)		/* If not really close yet, don't worry about stability */
		return(1);

	for (i=1;i<5;i++) {
		berr = CalcBurstPhase(0,&badvideo)-TARGVECTOR;
		if (badvideo) {
			DBUGRESULT(printf("Very Unstable!\n");)
			return(0);
		}
		if (berr<0)
			berr = -berr;


		diff = berr-err;
		if (diff<0)
			diff = -diff;

		if (diff > JITTER) {
			DBUGRESULT(printf("Unstable! (%.1f,%.1f)\n",berr,err);)
			return(0);
		}
	}

	DBUGRESULT(printf("(Stable)\n");)
	return(1);
}


int CheckRange(int edgelo, int coarselo, int finelo,
					int edgehi, int coarsehi, int finehi,
					FLOAT close)
{
	FLOAT	loerr,hierr;
	ULONG	res;
	int	badvideo;

	if (Optimize<2)		/* Range optimization enabled?  If not, do range */
		return(1);

	if (edgelo>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_EDGE,edgelo,0);
	if (coarselo>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_COURSE,coarselo,0);
	if (finelo>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_FINE,finelo,0);

	loerr = CalcBurstPhase(1,&badvideo)-TARGVECTOR;
	DBUGOPT(printf("Lo side:");)
	if (badvideo) {
		DBUGOPT(printf("bad video\n");)
		return(1);		/* Do range */
	}
	DBUGOPT(printf("%.1f\n",loerr);)

	if (edgehi>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_EDGE,edgehi,0);
	if (coarsehi>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_COURSE,coarsehi,0);
	if (finehi>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_FINE,finehi,0);

	hierr = CalcBurstPhase(1,&badvideo)-TARGVECTOR;
	DBUGOPT(printf("Hi side:");)
	if (badvideo) {
		DBUGOPT(printf("bad video\n");)
		return(1);		/* Do range */
	}
	DBUGOPT(printf("%.1f\n",hierr);)

	if (loerr<0.0) {
		if (hierr>=0.0)
			return(1);			/* Do range if sign changes */
	}
	else {
		if (hierr<0.0)
			return(1);			/* Do range if sign changes */
	}

	if (loerr < 0.0)
		loerr = -loerr;
	if (hierr < 0.0)
		hierr = -hierr;

	if ((loerr < close) || (hierr < close))
		return(1);			/* Do range if either is fairly close */

	return(0);			/* Made it thru all tests!  Skip entire range! */
}


/* Average 4 quads (xxx in from edge) for x lines from each of flds 0 and 1 */
int AverageQuads(LONG *minI,LONG *minQ,LONG *plsI,LONG *plsQ)
{
	register	UBYTE	*ptr;
	int	i,j,k,avglum,lastlum,badflag,deltalum;
	ULONG	res;

	*minI = *minQ = *plsI = *plsQ = 0;

	badflag = 0;

	Lowest = Highest = 0;

	for (i=0;i<SAMPLEFIELDS;i++) {
		Delay(SLOWDOWN);

		res = OpenLiveField(i);

		Delay(SLOWDOWN);

		if (i & 1) {
			res = GetNextLine(Buffer);		/* Skip whole line to be in phase?!?!? */
		}
		else {		/* Do it here too! */
			res = GetNextLine(Buffer);		/* Skip half lines */
			res = GetNextLine(Buffer);		/* Skip whole line to be in phase?!?!? */
		}
		for (j=1;j<=SAMPLELINES;j++) {
			DBUG(printf("Field %ld, line %ld:\n",i,j);)
			res = GetNextLine(Buffer);
			ptr = &Buffer[RECPIXELSIN];	/* Skip into burst area */
			lastlum = 999;
			for (k=1;k<=RECQUADS;k++) {
				if ((j & 1) == 0) {		/* ++-- */
					DBUGVERB(printf("+");)
				}
				else {						/* --++ */
					DBUGVERB(printf("-");)
				}
				DBUGVERB(YIQ(ptr);)

				/* On first line, check for crappy video */
				if (j==1) {
					avglum = (ptr[0]+ptr[1]+ptr[2]+ptr[3]);
					DBUGVERB(printf("(%ld)",avglum);)
					if (lastlum != 999) {
						deltalum = (avglum-lastlum)>>2;		/* Into D2 units */

						if ((deltalum>NoiseThreshHi) || (deltalum<NoiseThreshLo)) {
							DBUGACAL(printf("broke on this: %ld\n",deltalum);)
							badflag = 1;
							break;
						}
					}
					lastlum = avglum;
				}

				if ((j & 1) == 0) {			/* ++-- */
					*plsI += *ptr++;
					*plsQ += *ptr++;
					*minI += *ptr++;
					*minQ += *ptr++;
				}
				else {							/* --++ */
					*minI += *ptr++;
					*minQ += *ptr++;
					*plsI += *ptr++;
					*plsQ += *ptr++;
				}
			}
			if (badflag)
				break;
		}
		res = CloseLiveField();
	}

	return(badflag);
}


ULONG OpenLiveField(UBYTE field)
{
	ULONG	res;

	FlyVol.Board = 0;
	FlyVol.SCSIdrive = 0;
	FlyVol.Path = 0;

	Action.Volume = &FlyVol;

	Action.ReturnTime = RT_STOPPED;
	res = OpenReadField(&Action,field,0);
	if (res)
		printf("\nOLF error %ld - %ls\n",res,Error2String(res));

	return(res);
}


ULONG CloseLiveField(void)
{
	ULONG	res;

	Action.ReturnTime = RT_STOPPED;
	res = CloseField(&Action);
	if (res)
		printf("\nCLF error %ld - %ls\n",res,Error2String(res));

	return(res);
}


ULONG GetNextLine(UBYTE *buffer)
{
	ULONG	res;

	Action.ReturnTime = RT_STOPPED;
	res = FlyerReadLine(&Action,Buffer);
	if (res)
		printf("\nGLL error %ld - %ls\n",res,Error2String(res));

	return(res);
}


int ReturnHit(void)
{
	if (WaitForChar(handle,30)) {
		getchar();
		return(1);
	}
	return(0);
}


int AvgQuad(UBYTE *ptr,int *badvideo)
{
	int	avg,i1,i2,q1,q2;

	i1 = *ptr++;
	q1 = *ptr++;
	i2 = *ptr++;
	q2 = *ptr++;

	avg = (i1 + q1 + i2 + q2)>>2;

	DBUGVERB(printf("%ld (%ld,%ld,%ld,%ld)\n",avg,i1-avg,q1-avg,i2-avg,q2-avg);)

	if (abs(i1 - avg) > MAXCHROMA)
		*badvideo = 1;
	else if (abs(q1 - avg) > MAXCHROMA)
		*badvideo = 1;
	else if (abs(i2 - avg) > MAXCHROMA)
		*badvideo = 1;
	else if (abs(q2 - avg) > MAXCHROMA)
		*badvideo = 1;
	else
		*badvideo = 0;

	return(avg);
}

void YIQ(UBYTE *ptr)
{
	int	p1,p2,p3,p4,Y,I,Q;

	p1 = *ptr++;
	p2 = *ptr++;
	p3 = *ptr++;
	p4 = *ptr++;

	Y = (p1 + p2 + p3 + p4)/4;
	I = ((Y - p1) + (p3 - Y))/2;
	Q = ((Y - p2) + (p4 - Y))/2;

	printf("%ld %ld %ld %ld     Y=%ld, I=%ld, Q=%ld\n",p1,p2,p3,p4,Y,I,Q);
}

void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}

void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}

void ShowVal(int val,int x,int y)
{
	sprintf(ValueBuffer,"%ld    ",val);
	PrintIText(RP,&ValueText,x,y);
}


void ShowBlinker(int x,int y)
{
	sprintf(ValueBuffer,"%ls",(Toggle?"`":"'"));
	Toggle ^= 1;
	PrintIText(RP,&ValueText,x,y);
}


void ShowString(char *string,int x,int y)
{
	strcpy(ValueBuffer,string);
	PrintIText(RP,&ValueText,x,y);
}


int WaitIntMsg(void)
{
	int	close;

	close = 0;

	do {
		IntMsg = (struct IntuiMessage *)GetMsg(Window->UserPort);
		while (IntMsg == 0) {
			if (close)
				return(0);
			else {
				Wait(1<<Window->UserPort->mp_SigBit);
				IntMsg = (struct IntuiMessage *)GetMsg(Window->UserPort);
			}
		}

		MIClass = IntMsg->Class;  /* Get pertinent info */
		MIAddress = (struct Gadget *)IntMsg->IAddress;
		if ((MIAddress) && (MIClass == GADGETUP))
			theID = MIAddress->GadgetID;
		else
			theID = 0;
		ReplyMsg((struct Message *)IntMsg);			/* Reply to message quickly */

		if (MIClass == CLOSEWINDOW)
			close = 1;			/* Get all remaining messages and close window! */
	} while (close);

	return(1);
}


int GetIntMsg(void)
{
	IntMsg = (struct IntuiMessage *)GetMsg(Window->UserPort);
	if (IntMsg) {
		MIClass = IntMsg->Class;  /* Get pertinent info */
		MIAddress = (struct Gadget *)IntMsg->IAddress;
		if ((MIAddress) && (MIClass == GADGETUP))
			theID = MIAddress->GadgetID;
		else
			theID = 0;
		ReplyMsg((struct Message *)IntMsg);			/* Reply to message quickly */
		return(1);
	}
	return(0);
}


void FlushIntMsgs(void)
{
	while (GetIntMsg()) {}
}


ULONG	ScrollBar2Pos(ULONG max,struct Gadget *gad)
{
	ULONG	new;

	new = ((struct PropInfo *)gad->SpecialInfo)->HorizPot+1;
	new = ((max-1) * new)>>15;
	if (new & 1) new++;
	new>>=1;

	if (new >= max)
		new = max-1;

	return(new);
}


void HandleSliders(int count)
{
	newpospos = ScrollBar2Pos(POSSPAN,&PosSliderGadget);
	if (newpospos != oldpospos) {				/* Slider changed since last? */
		Doit(POSBASE+newpospos,0);
		oldpospos = newpospos;
	}

	if (count > 1) {
		newclkpos = ScrollBar2Pos(4,&ClkSliderGadget);
		if (newclkpos != oldclkpos) {				/* Slider changed since last? */
			Doit3(0,newclkpos,0);
			oldclkpos = newclkpos;
		}
		newcrspos = ScrollBar2Pos(10,&CrsSliderGadget);
		if (newcrspos != oldcrspos) {				/* Slider changed since last? */
			Doit3(1,newcrspos,0);
			oldcrspos = newcrspos;
		}
		newfinepos = ScrollBar2Pos(8,&FineSliderGadget);
		if (newfinepos != oldfinepos) {				/* Slider changed since last? */
			Doit3(2,newfinepos,0);
			oldfinepos = newfinepos;
		}
	}
}


void SaveToE2(void)
{
	Doit3(0,oldclkpos,1);
	Doit3(1,oldcrspos,1);
	Doit3(2,oldfinepos,1);
	Doit(POSBASE+oldpospos,1);
	DBUG(printf("Saved!\n");)
}


void Doit(int val,int saveit)
{
	ULONG	res;

	ShowVal(val,200,12);

	res = FlyerWriteCalib(0,AdjType+CALIB_HPLAYOFFSETA,val,saveit);
	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
}


void Doit3(int which,int val,int saveit)
{
	ULONG	res;

	ShowVal(val,200,12+(which+1)*22);

	res = FlyerWriteCalib(0,AdjType*3+which,val,saveit);
	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
}


void PresetSliders(int count)
{
	ULONG	res;
	WORD 	value;

	res = FlyerReadCalib(0,AdjType+CALIB_HPLAYOFFSETA,&value);
	oldpospos = value-POSBASE;
	SetupSlider(&PosSliderGadget,POSSPAN,oldpospos);
	ShowVal(POSBASE+oldpospos,200,12);

	if (count > 1) {
		res = FlyerReadCalib(0,AdjType*3+CALIB_DACA_PHASE_EDGE,&value);
		oldclkpos = value;
		SetupSlider(&ClkSliderGadget,4,oldclkpos);
		ShowVal(oldclkpos,200,34);

		res = FlyerReadCalib(0,AdjType*3+CALIB_DACA_PHASE_COURSE,&value);
		oldcrspos = value;
		SetupSlider(&CrsSliderGadget,10,oldcrspos);
		ShowVal(oldcrspos,200,56);

		res = FlyerReadCalib(0,AdjType*3+CALIB_DACA_PHASE_FINE,&value);
		oldfinepos = value;
		SetupSlider(&FineSliderGadget,8,oldfinepos);
		ShowVal(oldfinepos,200,78);
	}
}

void SetupSlider(struct Gadget *gad,UWORD span, UWORD pos)
{
	ULONG	p,body;

	/* Set appropriate size(s) */
	body = (0x10000/(ULONG)span) & 0xFFFF;
/*	info->HorizBody  = body; */

	p = (ULONG)pos;
	p = (p<<16)/(ULONG)(span-1);

	if (p>0xFFFF)
		p = 0xFFFF;

/*	info->HorizPot = p; */

	NewModifyProp(gad,Window,0,AUTOKNOB|FREEHORIZ,p,0,body,MAXBODY,1);
}


void PlayClip(int channel,char *name,int length)
{
	ULONG res;
	struct FlyerVolume FlyVol;
	struct ClipAction	 Action;

	FlyVol.Path = name;
	FlyVol.Board = 0;
	FlyVol.SCSIdrive = 0;

	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_STOPPED;

	Action.Channel = channel;
	Action.Flags = CAF_VIDEO;
	Action.VidStartField = 0;
	Action.VidFieldCount = length;
	Action.AudStartField = 0;
	Action.AudFieldCount = length;
	Action.GoClock = 0;

	res = FlyerPlay(&Action);
	if (res)
		printf("Bars Error %ld - %ls\n",res,Error2String(res));
}


void SliderLabels(int count)
{
	ShowString("Position (90)",18,12);
	if (count > 1) {
		ShowString("Clk (22.5)",18,34);
		ShowString("Coarse (~4)",18,56);
		ShowString("Fine (~1)",18,78);
	}
}


FLOAT CalcBurstPhase(int skipframes,int *badvidflag)
{
	LONG	mI,mQ,pI,pQ,I,Q;
	FLOAT	tan,base;
	ULONG	res;
	int	i;

	Delay(SKEWSETTLETIME);

	Delay(SLOWDOWN);

	for (i=0;i<skipframes;i++) {
		res = OpenLiveField(0);			/* Throw away 'n' color frames */
		res = CloseLiveField();
	}

	Delay(SLOWDOWN);

	*badvidflag = AverageQuads(&mI,&mQ,&pI,&pQ);
	DBUGACAL(printf("Lo/Hi errors: %ld %ld\n",Lowest,Highest);)
	if (*badvidflag==0) {

		DBUGACAL(printf("-I,-Q,+I,+Q = %ld,%ld,%ld,%ld\n",mI,mQ,pI,pQ);)

		I = (pI - mI)/2;
		Q = (pQ - mQ)/2;

		DBUGACAL(printf("I,Q = %ld,%ld ",
		I/(SAMPLEFIELDS*SAMPLELINES*RECQUADS),
		Q/(SAMPLEFIELDS*SAMPLELINES*RECQUADS));)

		if (Q<0.0) {
			base = 180.0;
		}
		else {
			base = 0.0;
		}
		DBUGMATH(printf("base = %.1f\n",base);)

		tan = ((FLOAT)I)/((FLOAT)Q);
		DBUGMATH(printf("ratio = %.1f\n",tan);)

		tan = IEEESPAtan(tan)*180.0/(PI;		/* Imbalanced ')' in include */
		DBUGMATH(printf("tan = %.1f\n",tan);)

		tan += base+33;
		DBUGMATH(printf("based = %.1f\n",tan);)

		if (tan<0.0)
			tan += 360.0;
		if (tan>=360.0)
			tan -= 360.0;
		DBUGMATH(printf("final = %.1f\n",tan);)
	}
	else
		tan = 999.9;

	return(tan);
}
