/*********************************************************************\
*
* CpuRead - Reads data from CPU map
*
* $Id: CpuRead.c,v 1.0 1995/05/05 15:48:34 Flick Exp $
*
* $Log: CpuRead.c,v $
*Revision 1.0  1995/05/05  15:48:34  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/24/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

BOOL ValidTest(ULONG addr);
void GetArg(char *arg,int *var,int lo,int hi,char *label);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);
UBYTE Byte2Ascii(UBYTE input);

struct Library *FlyerBase;

UBYTE Ascii[40];

main(int argc,char **argv)
{
	int	unit,frag;
	ULONG	res,addr,count;
	UWORD	data;
	UBYTE	*ascptr;
	BOOL	direct;

	if ((argc<3)||(argc>5)) {
		printf("Usage: %ls <board> <address> [<count>] [SRAM]\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	GetLongArg(argv[2],&addr,0,0xFFFFFFFF,"Address");
	if (argc>=4)
		GetLongArg(argv[3],&count,1,0xFFFFFFFF,"Count");
	else
		count=2;

	if (!ValidTest(addr))
		exit(20);

	if (((addr>=0xC00000)&&(addr<0xC40000))
		&& (argc==5) && (!stricmp(argv[4],"SRAM"))) {
		direct = TRUE;
	}
	else
		direct = FALSE;


	if (!direct) {
		if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
			printf("Open flyer.library failed\n");
			return;
		}
	}

	ascptr = &Ascii[0];

	frag = addr % 16;
	if (frag) {
		printf("%06lx: ",addr);
		if (argc>=4) {
			while (frag) {
				printf("     ");
				frag-=2;
				*ascptr++ = ' ';
				*ascptr++ = ' ';
			}
		}
	}
	while ((count>0)&&(ValidTest(addr))) {
		if ((addr % 16)==0) {
			*ascptr = 0;
			printf("     %ls\n%06lx: ",Ascii,addr);
			ascptr = &Ascii[0];
		}
		if (direct) {
			data = *((UWORD *)(((ULONG)addr - 0xC00000) + 0xEC0000));
		}
		else {
			data = 0xFFFF;
			res = CPUread(unit,addr,&data);
			if (res) {
				printf("Error %ld - %ls\n",res,Error2String(res));
				break;
			}
		}
		printf("%04lx ",data);
		addr += 2;

		*ascptr++ = Byte2Ascii(data>>8);
		*ascptr++ = Byte2Ascii(data & 0xFF);

		if (count>=2)
			count -= 2;
		else
			count = 0;
	}
	printf("\n\n");

	if (FlyerBase)
		CloseLibrary(FlyerBase);

	return(0);
}


UBYTE Byte2Ascii(UBYTE input)
{
	UBYTE out;

	out = input;

	if ((out<32) || (out>126))
		out = '.';

	return(out);
}


BOOL ValidTest(ULONG addr)
{
	if ((addr % 2)>0) {
		printf("Address on odd boundary\n");
		return(FALSE);
	}

	if (((addr>=0x000000)&&(addr<0x040000))||		/* ROM/RAM */
		 ((addr>=0xA00000)&&(addr<0xA00200))||		/* Serial Bus */
		 ((addr>=0xA10000)&&(addr<0xA10600))||		/* SCSI chips */
		 ((addr>=0xA20000)&&(addr<0xA20200))||		/* 3-port controller */
		 ((addr>=0xA21000)&&(addr<0xA21400))||		/* FIR chip */
		 ((addr>=0xA22000)&&(addr<0xA22200))||		/* DMA chip */
		 ((addr>=0xA30000)&&(addr<0xA30200))||		/* Serial UART */
		 ((addr>=0xC00000)&&(addr<0xC40000))||		/* SRAM */
		 ((addr>=0xF00000)&&(addr<0xF20000))||		/* F0 ROM */
		 ((addr>=0xF80000)&&(addr<0xFA0000)))		/* F8 ROM (removed) */
		return(TRUE);
	else {
		printf("Address not decoded\n");
		return(FALSE);
	}
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}

void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
