/*********************************************************************\
*
* CpuWrite - Writes data to CPU map
*
* $Id: CpuWrite.c,v 1.0 1995/05/05 15:48:37 Flick Exp $
*
* $Log: CpuWrite.c,v $
*Revision 1.0  1995/05/05  15:48:37  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/24/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

BOOL ValidTest(ULONG addr);
void GetArg(char *arg,int *var,int lo,int hi,char *label);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);

struct Library *FlyerBase;

main(int argc,char **argv)
{
	int	unit;
	ULONG	res,addr,data;

	if (argc!=4) {
		printf("Usage: %ls <board> <address> <data>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	GetLongArg(argv[2],&addr,0,0xFFFFFFFF,"Address");
	GetLongArg(argv[3],&data,0,0xFFFF,"Data");

	if (!ValidTest(addr))
		exit(20);

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	res = CPUwrite(unit,addr,(UWORD)data);
	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
	else
		printf("Okay\n");

	CloseLibrary(FlyerBase);

	return(0);
}


BOOL ValidTest(ULONG addr)
{
	if ((addr % 2)>0) {
		printf("Address on odd boundary\n");
		return(FALSE);
	}

	if (((addr>=0x000000)&&(addr<0x040000))||		/* ROM/RAM */
		 ((addr>=0xA00000)&&(addr<0xA00200))||		/* Serial Bus */
		 ((addr>=0xA10000)&&(addr<0xA10600))||		/* SCSI chips */
		 ((addr>=0xA20000)&&(addr<0xA20200))||		/* 3-port controller */
		 ((addr>=0xA21000)&&(addr<0xA21400))||		/* FIR chip */
		 ((addr>=0xA22000)&&(addr<0xA22200))||		/* DMA chip */
		 ((addr>=0xA30000)&&(addr<0xA30200))||		/* Serial UART */
		 ((addr>=0xC00000)&&(addr<0xC40000))||		/* SRAM */
		 ((addr>=0xF00000)&&(addr<0xF20000))||		/* F0 ROM */
		 ((addr>=0xF80000)&&(addr<0xFA0000)))		/* F8 ROM */
		return(TRUE);
	else {
		printf("Address not decoded\n");
		return(FALSE);
	}
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}

void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
