/*********************************************************************\
*
* CuttingRoom - Test Flyer cutting room functions
*
* $Id: CuttingRoom.c,v 1.0 1995/05/05 15:48:43 Flick Exp $
*
* $Log: CuttingRoom.c,v $
*Revision 1.0  1995/05/05  15:48:43  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	01/24/95		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

ULONG ReadScriptFile(char *name);
void FreeFile(void);
int WaitHere(int);
void GetArg(char *arg,int *var,int lo,int hi,char *label);

#define	THEMAX	100

struct Entry {
	ULONG	drive;
	char	name[80];
	ULONG	vidflag;
	ULONG	audchans;
	ULONG	startfld;
	ULONG	fldlen;
};

struct Entry Sequence[THEMAX];

struct Library *FlyerBase;
struct FlyerVolume FlyVol;
struct ClipAction	 Action;

struct ClipInfo	clipinfo;

UBYTE	*FileBuffer;
ULONG	FileLength;
BPTR	Handle;

main(int argc,char **argv)
{
	int	Destructive;

	int	i,ndx,stepmode;
	ULONG	res;
	LONG	size;
	char *scriptptr,*endptr;
	struct Entry *entptr;
	struct FlyerVolume *volptr;
	struct ClipAction	 *actptr;

	if ((argc<3)||(argc>4)||(argv[1][0]=='?')) {
		printf("Usage: %ls <script> <stepmode> [destructive]\n",argv[0]);
		return(20);
	}

	GetArg(argv[2],&stepmode,0,1,"Step mode");

	if ((argc==4)&&(stricmp(argv[3],"destructive")==0))
		Destructive = 1;
	else
		Destructive = 0;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	Handle = Input();

	size = ReadScriptFile(argv[1]);
	if (size) {
		endptr = FileBuffer+FileLength;
		for (ndx=0,scriptptr=FileBuffer;(ndx<THEMAX)&&(scriptptr<endptr);) {
			if (*scriptptr == '>') {
				entptr = &Sequence[ndx];
				scriptptr++;
				sscanf(scriptptr,"%ld %ls %ld %ld %ld %ld\n",
					&entptr->drive,
					entptr->name,
					&entptr->vidflag,
					&entptr->audchans,
					&entptr->startfld,
					&entptr->fldlen
				);
				ndx++;
			}

	      do {
				if (scriptptr >= endptr)
					break;
			} while (*scriptptr++ != 10);
		}

		printf("Master Clip: %ls on drive %ld\n",
		Sequence[0].name,Sequence[0].drive);
		for (i=1;i<ndx;i++) {
			entptr = &Sequence[i];
			printf("SubClip %ld: %ls to drive %ld\n",i,entptr->name,entptr->drive);
			printf("        v/a = %ld/%ld  flds=%ld/%ld\n",
				entptr->vidflag,entptr->audchans,entptr->startfld,entptr->fldlen);
		}


		/* Open new cutclip list */
		printf("Ready to open CutClip list -- ");
		if (WaitHere(stepmode))
			goto abort;

		FlyVol.Flags = FVF_USENUMS;
		FlyVol.Board = 0;
		FlyVol.Path = Sequence[0].name;
		FlyVol.SCSIdrive = Sequence[0].drive;

		actptr = &Action;
		actptr->Volume = &FlyVol;
		actptr->ReturnTime = RT_STOPPED;

		res = StartClipCutList(&Action,Destructive);
		if (res) {
			printf("Error %ld - %ls\n",res,Error2String(res));
			goto abort;
		}

		for (i=1;i<ndx;i++) {
			entptr = &Sequence[i];

			volptr = &FlyVol;
			volptr->Flags = FVF_USENUMS;
			volptr->Board = 0;
			volptr->Path = entptr->name;
			volptr->SCSIdrive = entptr->drive;

			actptr = &Action;
			actptr->Volume = volptr;
			actptr->ReturnTime = RT_STOPPED;
			actptr->AudStartField = entptr->startfld;
			actptr->AudFieldCount = entptr->fldlen;
			actptr->VidStartField = entptr->startfld;
			actptr->VidFieldCount = entptr->fldlen;

			actptr->Flags = 0;
			if (entptr->vidflag)
				actptr->Flags |= CAF_VIDEO;
			if (entptr->audchans == 1)
				actptr->Flags |= CAF_AUDIOL;
			if (entptr->audchans == 2)
				actptr->Flags |= CAF_AUDIOL | CAF_AUDIOR;

			printf("About to add SubClip %ls (%ld,%ld) -- ",volptr->Path,
					actptr->VidStartField,actptr->VidFieldCount);

			if (WaitHere(stepmode))
				goto abort;
	
			res = AddClipCut(&Action);
			if (res) {
				printf("Error %ld - %ls\n",res,Error2String(res));
				res = EndClipCutList(0);		/* Clean up */
				goto abort;
			}
		}

		printf("Ready to do it! -- ");
		if (WaitHere(stepmode))
			goto abort;

		res = EndClipCutList(1);
		if (res) {
			printf("Error %ld - %ls\n",res,Error2String(res));
			goto abort;
		}
	}

abort:
	CloseLibrary(FlyerBase);
	FreeFile();

	return(0);
}


ULONG ReadScriptFile(char *name)
{
	FILE	*fp;
	int	read;

	read = 0;
	fp = fopen(name,"r");
	if (fp) {
		fseek(fp,0,SEEK_END);
		FileLength = ftell(fp);
		fseek(fp,0,SEEK_SET);

		FileBuffer = AllocMem(FileLength,MEMF_PUBLIC);
		if (FileBuffer) {
			read = fread(FileBuffer,FileLength,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",name);

	if (read==1)
		return(FileLength);
	else
		return(0);
}

void FreeFile(void)
{
	if (FileLength)
		FreeMem(FileBuffer,FileLength);
}


int WaitHere(int stepmode)
{
	if (stepmode == 0) {
		printf("\n");
	}
	else {
		printf("Hit ENTER to proceed...\n");
		for (;;) {
			if (WaitForChar(Handle,30)) {
				if (getchar()=='q') {
					printf("Aborted\n");
					getchar();
					return(1);
				}
				printf("Here we go...\n");
				break;
			}
		}
	}
	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
