/********************************************************************
* $DirCache.c$
* $Id: DirCache.c,v 2.9 1995/01/10 13:16:46 pfrench Exp $
* $Log: DirCache.c,v $
*Revision 2.9  1995/01/10  13:16:46  pfrench
*Hopefully fixed problem with grazer going to non-mounted
*volumes.  Now list pointer in getdircache will always
*point to a list.  Just now it will point to a global,
*empty list if a directory cache couldn't be made
*
*Revision 2.8  1994/12/22  21:04:38  pfrench
*Small fix to cache validation, makes sure it's a directory
*before building the cache.
*
*Revision 2.7  1994/11/18  16:29:15  pfrench
*Found nasty bug that could return stack garbage if the
*path was invalid.
*
*Revision 2.6  1994/09/28  15:41:24  pfrench
*Made cacheing much faster and much more robust for
*supporting multiple Grazer Windows.
*
*Revision 2.5  1994/09/27  15:27:34  pfrench
*Removed freedircache as it was un-save and is now auto
*matically invalidated.
*
*Revision 2.4  1994/09/26  22:07:59  pfrench
*Now does auto-invalidation when directory date is
*modified.
*
*Revision 2.3  1994/09/22  16:41:44  pfrench
*Now expunge returns errors compatible with V39 exec
*MemHandler routines
*
*Revision 2.2  1994/09/22  13:59:36  pfrench
*Added provisions not to cache the Volumes/Devices/Assigns croutons
*
*Revision 2.1  1994/09/20  22:49:51  pfrench
*First working version of dircache
*
*Revision 2.0  1994/09/17  21:41:35  pfrench
*FirstCheckIn
*
* Copyright (c)1992 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*********************************************************************/
#include <exec/types.h>
#include <exec/memory.h>
#include <exec/interrupts.h>
#include <dos/dos.h>
#include <dos/dosextens.h>
#include <stdio.h>
#include <string.h>
#include <crouton_all.h>

#include <filelist.h>
#include <grazer.h>

#include <proto/exec.h>
#include <proto/dos.h>

#ifndef PROTO_PASS
#include <proto.h>
#endif

extern char filepathbuf[300];

/* stub functions in MoreToastSupport.a */
LONG __asm AddTBMemHandler(register __a0 struct Interrupt *);
LONG __asm RemTBMemHandler(register __a0 struct Interrupt *);

static LONG __saveds __asm dc_LowMemHandler(
	register __a0 struct MemHandlerData *memh,
	register __a1 struct DirCacheMaster *dcm,
	register __a6 struct Library *SysBase );

LONG InitGrazerDirCache(void);
LONG CloseGrazerDirCache(void);

static LONG dc_Delete( struct DirCache *dc );
static struct DirCache *dc_Create( STRPTR pathname, struct Grazer *Grazer );

struct DirCacheMaster
{
	struct MinList	dcm_dirs;

	struct MinList	dcm_emptylist;

	/* LONG Align */

	struct FileInfoBlock	dcm_fib;
};

static struct DirCacheMaster global_dcm;

struct DirCache
{
	struct Node			dc_node;
	UBYTE					dc_lockcount;
	UBYTE					dc_status;

	struct DateStamp	dc_ds;

	struct MinList 	dc_files;
	struct MinList 	dc_filtered_files;
};

#define DCSTATUS_NORMAL		0
#define DCSTATUS_FILTERED	1
#define DCSTATUS_FLUSHED	2

static struct Interrupt dircache_interrupt = 
{
	NULL,NULL,				/* ln_Succ, ln_Pred */
	NT_INTERRUPT,50,		/* ln_Type, ln_Pri */
	"Grazer DirCache",	/* ln_Name */

	&global_dcm,						/* is_Data */
	(void(*)())dc_LowMemHandler	/* is_Code */
};

/******************************************************
 *	WARNING:
 *
 *		This expunge function gets called during a
 *		system memory panic. (During a Forbid()).
 *		And it will most likely get called by some
 *		other task.
 *
 *		You _must_ not do anything that causes the
 *		CPU to Wait(), that means NO I/O!!  You should
 *		pretty much only do FreeMem()s
 *
 ******************************************************
 */
static LONG __saveds __asm dc_LowMemHandler(

	/* ONLY VALID IN V39 AND GREATER */
	register __a0 struct MemHandlerData *memh,

	register __a1 struct DirCacheMaster *dcm,
	register __a6 struct Library *SysBase )
{
	LONG					 retval = MEM_DID_NOTHING;
	struct DirCache	*dc,*next_dc;

	for ( dc = (struct DirCache *) dcm->dcm_dirs.mlh_Head;
			next_dc = (struct DirCache *) dc->dc_node.ln_Succ;
			dc = next_dc )
	{
		// If this directory is not locked
		if ( !dc->dc_lockcount )
		{
			retval = MEM_ALL_DONE;
			Remove( (struct Node *) dc );
			dc_Delete(dc);
		}
	}

	return(retval);
}
/******************************************************/

ULONG GetDirCache( STRPTR path, struct List **pList, struct Grazer *Grazer )
{
	struct DirCache	*dc = NULL;
	BPTR					 lock;

	/* Initialize this to point to an empty list */
	*pList = (struct List *)&global_dcm.dcm_emptylist;
	
	// MUST convert to REAL volume/pathname with correct capitalization
	if ( lock = Lock(path,SHARED_LOCK) )
	{
		BOOL			gotvolpath;

		gotvolpath = NameFromLock(lock,filepathbuf,sizeof(filepathbuf));
		Examine(lock,&global_dcm.dcm_fib);
		UnLock(lock);

		// WARNING: "lock" no longer valid

		/* This had better be a directory */
		if ( gotvolpath && (global_dcm.dcm_fib.fib_DirEntryType > 0) )
		{
			// Get Volumes/Devices/Assigns sends in STRPTR ""
			if ( strlen(path) )
				path = filepathbuf;

			Forbid();
			{
				// Because FindName() only looks at the first longword
				// of the list structure, we can trick it into traversing
				// all nodes with the same name inside the while()
				dc = (struct DirCache *)&global_dcm.dcm_dirs;

				while ( dc = (struct DirCache *)FindName((struct List *)dc,path) )
				{
					// Because each Grazer modifies state information for each
					// node in the list (Selected, etc.) we don't want to get
					// a dircache being used by two grazers at the same time.
					if ( !dc->dc_lockcount )
					{
						if (	(dc->dc_ds.ds_Tick == global_dcm.dcm_fib.fib_Date.ds_Tick) &&
								(dc->dc_ds.ds_Minute == global_dcm.dcm_fib.fib_Date.ds_Minute) &&
								(dc->dc_ds.ds_Days == global_dcm.dcm_fib.fib_Date.ds_Days) )
						{
							dc->dc_lockcount++;
							break;
						}
						else
						{
							// Delete this cache since it's no longer valid
							struct DirCache	*pred_dc;

							// This pred_dc trick works even if "dc" is the
							// first node in the list.  Think about it.

							pred_dc = (struct DirCache *)dc->dc_node.ln_Pred;

							Remove( (struct Node *)dc );
							dc_Delete(dc);

							dc = pred_dc;	/* so the next FindName() works */
						}
					}
				}
			}
			Permit();

			if ( dc )
			{
				// Add filtered files back into list
				struct Node *node;

				while ( node = RemHead( (struct List *)&dc->dc_filtered_files) )
				{
					AddTail( (struct List *)&dc->dc_files, node );
				}

				for ( node = (struct Node *)dc->dc_files.mlh_Head;
						node->ln_Succ;
						node = node->ln_Succ )
				{
					((struct EditNode *)node)->Status = EN_NORMAL;
					((struct EditNode *)node)->Redraw = TRUE;
				}
			}
			else
			{
				if ( dc = dc_Create(path,Grazer) )
				{
					dc->dc_lockcount++;
					dc->dc_ds.ds_Days = global_dcm.dcm_fib.fib_Date.ds_Days;
					dc->dc_ds.ds_Minute = global_dcm.dcm_fib.fib_Date.ds_Minute;
					dc->dc_ds.ds_Tick = global_dcm.dcm_fib.fib_Date.ds_Tick;

					Forbid();
					AddTail( (struct List *)&global_dcm.dcm_dirs,(struct Node *)dc );
					Permit();
				}
			}

			if ( dc )
			{
				*pList = (struct List *)&dc->dc_files;
			}
		}
	}

	return((ULONG)dc);
}

ULONG UngetDirCache( ULONG udc )
{
	struct DirCache	*dc = (struct DirCache *)udc;

	Forbid();
	if ( !(--dc->dc_lockcount) )
	{
		// This is a Volume/Devices/Assigns list
		if ( !strlen(dc->dc_node.ln_Name) )
		{
			Remove( (struct Node *) dc );
			dc_Delete(dc);
		}
	}
	Permit();
	return(0);
}

struct List *GetDirCacheFilterList( ULONG udc )
{
	struct DirCache	*dc = (struct DirCache *)udc;
	return( (struct List *)&dc->dc_filtered_files );
}

/******************************************************/
LONG InitGrazerDirCache(void)
{
	LONG			retval = TRUE;
	NewList( (struct List *) &global_dcm.dcm_dirs );
	NewList( (struct List *) &global_dcm.dcm_emptylist );
	AddTBMemHandler( &dircache_interrupt );
	return(retval);
}

LONG CloseGrazerDirCache(void)
{
	LONG					 retval = TRUE;
	struct DirCache	*dc;

	RemTBMemHandler( &dircache_interrupt );

	/* At This point, we will want to free all remaining cached dirs */

	while ( dc = (struct DirCache *) RemTail((struct List *)&global_dcm.dcm_dirs) )
	{
		dc_Delete(dc);
	}
	
	return(retval);
}
/******************************************************/
static struct DirCache *dc_Create( STRPTR pathname, struct Grazer *Grazer )
{
	struct DirCache	*dc;
	LONG					 ok = FALSE;

	if ( dc = (struct DirCache *)AllocMem(sizeof(*dc),MEMF_PUBLIC|MEMF_CLEAR) )
	{
		if ( dc->dc_node.ln_Name = AllocMem(strlen(pathname)+1,MEMF_PUBLIC) )
		{
			strcpy( dc->dc_node.ln_Name, pathname );
			NewList( (struct List *) &dc->dc_files );
			NewList( (struct List *) &dc->dc_filtered_files );

			// Get all the files and their GrazerNodes for this directory.

			ok = BuildFileList(Grazer->Path,Grazer->ValidAttributes,
		 		(struct List *)&dc->dc_files,
		 		AllocGrazerNode,FreeGrazerNode,GrazerFIBNode,GetGrazerClass );
		}

		if ( !ok )
		{
			dc_Delete(dc);
			dc = NULL;
		}
	}

	return(dc);
}

static LONG dc_Delete( struct DirCache *dc )
{
	LONG					 retval = 0;
	struct SmartNode	*sn;

	while ( sn = (struct SmartNode *)RemTail((struct List *)&dc->dc_files) )
	{
		if( ((struct GrazerNode *)sn)->BitMap)
			FreeIconBM(((struct GrazerNode *)sn)->BitMap);
		FreeSmartNode(sn);
	}

	while ( sn = (struct SmartNode *)RemTail((struct List *)&dc->dc_filtered_files) )
	{
		if( ((struct GrazerNode *)sn)->BitMap)
			FreeIconBM(((struct GrazerNode *)sn)->BitMap);
		FreeSmartNode(sn);
	}

	if ( dc->dc_node.ln_Name )
		FreeMem(dc->dc_node.ln_Name,strlen(dc->dc_node.ln_Name)+1);

	FreeMem( dc, sizeof(*dc) );

	return(retval);
}
