/********************************************************************
* $DirTabs.c$
* $Id: DirTabs.c,v 5.12 1995/10/09 16:40:20 Flick Exp $
* $Log: DirTabs.c,v $
*Revision 5.12  1995/10/09  16:40:20  Flick
*Removed unnecessary usage of popup.h
*
*Revision 5.11  1995/08/07  09:49:31  Flick
*Eliminated pesky compiler warning
*
*Revision 5.10  1995/02/03  10:46:32  pfrench
*Now saves directory tabs after every "set"
*
*Revision 5.9  1995/01/20  09:53:04  pfrench
*Fixed some set-dirtabs problems Dave noticed
*
*Revision 5.8  1995/01/17  19:26:25  pfrench
*Now directory tabs don't need to back up after doing a printf
*
*Revision 5.7  1994/12/29  16:17:58  CACHELIN4000
*Fix set tab string requester w/ OK and CANCEL buttons
*
*Revision 5.6  1994/12/23  13:28:56  pfrench
*Set Tab now updates path of old tabs
*
*Revision 5.5  1994/12/22  17:04:07  pfrench
*Allow cycling past end of tabs when last one filled
*
*Revision 5.4  1994/12/22  15:51:37  pfrench
*Now saves directory tabs back out to file
*
*Revision 5.3  1994/12/21  23:51:55  pfrench
*Finally a tweak I like
*
*Revision 5.2  1994/12/21  23:16:42  pfrench
*Removed explicit rendering calls from directory tabs,
*as the text is now being handled by intuition.
*
*Revision 5.1  1994/12/21  21:30:40  pfrench
*Hopefully final tweaking of grid position
*
*Revision 5.0  1994/12/21  20:04:22  pfrench
*FirstCheckIn
*
*/

#include <exec/types.h>
#include <exec/memory.h>
#include <intuition/intuition.h>

#ifndef INTUITION_GADGETCLASS_H
#include <intuition/gadgetclass.h>
#endif

#ifndef CLASSES_H
#include "classes.h"
#endif

#ifndef CLASSBASE_H
#include "classbase.h"
#endif

#ifndef PROOF_LIB_H
#include <proof_lib.h>
#endif

#ifndef PROOF_STREAM_H
#include "stream.h"
#endif

#include "dirtabs.h"

#include <request.h>
#include <gadgets.h>

#include <editwindow.h>
#include <editswit.h>
#include <panel.h>
#include <project.h>
#include <grazer.h>
#include <filelist.h>
#include <grazerrequest.h>

#ifndef PROTO_PASS
#include <proto.h>
#endif

#include <crouton_all.h>

#include	<clib/exec_protos.h>
#include	<pragmas/exec_pragmas.h>

#include	<clib/graphics_protos.h>
#include	<pragmas/graphics_pragmas.h>

#include	<clib/intuition_protos.h>
#include	<pragmas/intuition_pragmas.h>

#define REG(x)	register __##x
//This version causes an annoying incompatibility warning (???)
//#define HOOKFUNC	ULONG(* __asm)(REG(a0) struct Hook *, REG(a2) APTR, REG(a1) APTR)
#define HOOKFUNC	ULONG (*)()

/* *********************************************** */

// Global Directory Tab Data

struct DirTabDir
{
	struct Node	dtd_node;			/* ln_Name is a SmartString */
	char			dtd_label[30];		/* Label for dir tab */
};

static struct List dirtab_list;
LONG global_setting_dirtab = FALSE;

/* *********************************************** */

extern struct Gadget	 Gadget1;
extern struct Library *ProofBase;
extern struct ClassBase	*ClassBase;

extern struct Library *IntuitionBase;
extern struct Library *GfxBase;
extern struct TextFont *EditFont,*YellowFont;
extern struct TextAttr TextAttr;
extern char filepathbuf[300];
extern char destfilepathbuf[300];

extern LONG KPrintF( STRPTR fmt, ... );

/* *********************************************** */

ULONG ReadDirectoryTabs( STRPTR filename );
ULONG SaveDirectoryTabs( STRPTR filename );
ULONG FreeDirectoryTabs( void );
LONG GetListLen( struct List *list );
struct Node *GetListNode( struct List *list, LONG num );

/**********************************************************/

/********************************************************************
 *
 *	Private structures for directory tab object
 *
 ********************************************************************
 *
 */
 
struct DirTabData
{
	/* We'll trick Intuition here by stuffing a gadget at */
	/* the beginning of the structure, so we can just addglist */

	struct Gadget		 dt_cyclegad;

	struct Gadget		 dt_assigngad;

	/* Only seven fit across a grazer */
	struct Gadget		 dt_tabgad[7];

	struct IntuiText	 dt_itxt[7];

	/* --------------------------------------- */

	struct Window		*dt_win;

	struct IBox			 dt_box;

	LONG					 dt_curtab;

	LONG					 dt_firsttab;

};

static ULONG dt_Init( struct Class *cl, APTR ob, struct TagItem *ti );
static ULONG dt_SetAttrs( struct Class *cl, APTR ob, struct TagItem *ti );
static ULONG dt_ReThink( struct Class *cl, APTR ob );
static ULONG dt_GetAttrs( struct Class *cl, APTR ob, struct TagItem *ti );

static struct DirTabDir *dtd_Create( char *dirname, char *label );
static void dtd_Delete( struct DirTabDir *dtd );

static ULONG __saveds __asm dirtab_Dispatch(
	REG(a0) struct Class *cl,
	REG(a2) APTR ob,
	REG(a1) Msg msg );

struct Class *dirtab_AddClass( void )
{
	struct Class *cl;

	/* Initialize global "dirtab_list" */
	NewList( &dirtab_list );

	if ( cl = cl_MakeClass( DIRTABCLASS,
									ROOTCLASS, NULL,
									sizeof(struct DirTabData), 0) )
	{
		cl->cl_Dispatch.h_Entry = (HOOKFUNC)dirtab_Dispatch;
		
		if ( !pb_AddClass(cl) )
		{
			cl_FreeClass(cl);
			cl = NULL;
		}
	}

	return(cl);
}

static struct EditWindow *HandleDirTabCycle(
	struct EditWindow *Edit,
	struct IntuiMessage	*im );

static struct EditWindow *HandleDirTabSet(
	struct EditWindow *Edit,
	struct IntuiMessage	*im );

static struct EditWindow *HandleDirTabSelect(
	struct EditWindow *Edit,
	struct IntuiMessage	*im );

static LONG RenderDirTabLabel(
	struct RastPort *rp,
	struct Gadget *gad,
	char *label );

static ULONG __saveds __asm dirtab_Dispatch(
	REG(a0) struct Class *cl,
	REG(a2) APTR ob,
	REG(a1) Msg msg )
{
	ULONG					 retval = 0;

	switch ( msg->MethodID )
	{
		case OM_NEW:
		{
			// OM_NEW sends true "class" in the "object" ptr
			if ( ob = (APTR) ob_DoSuperMethodA(cl,ob,msg) )
			{
				struct opSet		*ops = (struct opSet *)msg;
				// struct DirTabData	*dt = (struct DirTabData *)INST_DATA(cl,ob);

				if ( dt_Init(cl,ob,ops->ops_Tags) )
					retval = (ULONG) ob;

				if ( !retval )
				{
					ob_CoerceMethod(cl,ob,OM_DISPOSE);
				}
			}
		}
		break;
		
		case OM_DISPOSE:
		{
			// struct DirTabData		*dt = (struct DirTabData *)INST_DATA(cl,ob);

			retval = ob_DoSuperMethodA(cl,ob,msg);
		}
		break;
		
		case OM_SET:
		{
			struct opSet			*ops = (struct opSet *)msg;
			retval = dt_SetAttrs(cl,ob,ops->ops_Tags);
		}
		break;

		case OM_GET:
		{
			struct opSet			*ops = (struct opSet *)msg;
			retval = dt_GetAttrs(cl,ob,ops->ops_Tags);
		}
		break;

		/* **************************************************** */
		/* **************************************************** */

		case DTM_CycleTabs:
		{
			struct DirTabData	*dt = (struct DirTabData *)INST_DATA(cl,ob);
			struct opDTCycle	*dtc = (struct opDTCycle *)msg;
			LONG					 listlen,newfirst;

			listlen = GetListLen(&dirtab_list);

			newfirst = dt->dt_firsttab + dtc->dtc_amount;

			if ( newfirst > listlen )			/* Past end, move to first */
				newfirst = 0;
			else if ( newfirst < 0 )			/* Past beginning, move to end */
				newfirst = (listlen / 7) * 7;

			/* There is some motion */
			if ( newfirst != dt->dt_firsttab )
			{
				dt->dt_firsttab = newfirst;

				ob_DoMethod(ob,DTM_RefreshTabs);
			}

		}
		break;

		case DTM_RefreshTabs:
		{
			struct DirTabData	*dt = (struct DirTabData *)INST_DATA(cl,ob);
			WORD					 i;

			for ( i = 0; i < 7; i++ )
			{
				struct DirTabDir	*dtd;

				/* Set GadgetID's to their corresponding values */
				dt->dt_tabgad[i].GadgetID = dt->dt_firsttab + i;

				if ( dtd = (struct DirTabDir *)GetListNode(&dirtab_list,dt->dt_firsttab+i) )
				{
					extern struct TextExtent LastExtent;
					extern struct EditWindow *EditTop;

					extern VOID __asm SafeColorText(
						register __a1 struct RastPort *RP,
						register __a0 STRPTR String,
						register __d0 WORD Length);

					extern UWORD __asm SafeFitText(
						register __a1 struct RastPort *RP,
						register __a0 STRPTR String,
						register __d0 WORD Length,
						register __d1 WORD WidthAvail,
						register __d2 BOOL RenderIt);

					struct RastPort	*rp = EditTop->Window->RPort;
					char					*label = dtd->dtd_label;
					struct Gadget		*gad;
					LONG					 x,len;

					gad = &dt->dt_tabgad[i];

					len = SafeFitText(rp,label,strlen(label),gad->Width - 8,FALSE);
					label[len] = '\0';

					x = (gad->Width - LastExtent.te_Width) >> 1;

					dt->dt_tabgad[i].GadgetText = &dt->dt_itxt[i];
					dt->dt_itxt[i].IText = dtd->dtd_label;
					dt->dt_itxt[i].LeftEdge = x;
				}
				else
				{
					dt->dt_tabgad[i].GadgetText = NULL;
				}
			}
		}
		break;

		default:
			retval = ob_DoSuperMethodA(cl,ob,msg);
		break;
	}

	return(retval);
}

static ULONG dt_Init( struct Class *cl, APTR ob, struct TagItem *ti )
{
	ULONG						 retval;
	struct DirTabData		*dt = (struct DirTabData *)INST_DATA(cl,ob);

	struct Image			*img;
	struct Gadget			*gad;

	/* No tab currently selected */
	dt->dt_curtab = -1;

	/* Note that we're modifying and copying the gadget structures
	 * as we progress along in this function.
	 */

	/* Initialize tab-cycle image */
	/* -------------------------------------------------------- */
	dt->dt_cyclegad.SpecialInfo = ob;

	dt->dt_cyclegad.Flags |= (GFLG_GADGHIMAGE|GFLG_GADGIMAGE);
	dt->dt_cyclegad.GadgetID = ID_DIRTAB_CYCLE;
	dt->dt_cyclegad.GadgetType = GTYP_BOOLGADGET;
	dt->dt_cyclegad.Activation = ( GACT_IMMEDIATE | GACT_RELVERIFY );
	dt->dt_cyclegad.UserData = (APTR) HandleDirTabCycle;

	/* Need Cycle Image */
	gad = FindGadget(&Gadget1,ID_DIRTAB_CYCLE);
	img = (struct Image *)gad->GadgetRender;
	dt->dt_cyclegad.GadgetRender = gad->GadgetRender;
	dt->dt_cyclegad.SelectRender = gad->SelectRender;
	dt->dt_cyclegad.Width = img->Width;
	dt->dt_cyclegad.Height = img->Height;

	/* Initialize tab-assign image */
	/* -------------------------------------------------------- */
	dt->dt_assigngad = dt->dt_cyclegad;
	dt->dt_assigngad.GadgetID = ID_DIRTAB_SET;
	dt->dt_assigngad.UserData = (APTR) HandleDirTabSet;

	/* Need Assign Image */
	gad = FindGadget(&Gadget1,ID_DIRTAB_SET);
	img = (struct Image *)gad->GadgetRender;
	dt->dt_assigngad.GadgetRender = gad->GadgetRender;
	dt->dt_assigngad.SelectRender = gad->SelectRender;
	dt->dt_assigngad.Width = img->Width;
	dt->dt_assigngad.Height = img->Height;


	/* Initialize first directory tab gadget */
	/* -------------------------------------------------------- */
	dt->dt_tabgad[0] = dt->dt_assigngad;
	dt->dt_tabgad[0].GadgetID = 0;
	dt->dt_tabgad[0].UserData = (APTR) HandleDirTabSelect;

	/* Need Tab Images */
	gad = FindGadget(&Gadget1,ID_DIRTAB_TAB);
	img = (struct Image *)gad->GadgetRender;
	dt->dt_tabgad[0].GadgetRender = gad->GadgetRender;
	dt->dt_tabgad[0].SelectRender = gad->SelectRender;
	dt->dt_tabgad[0].Width = img->Width;
	dt->dt_tabgad[0].Height = img->Height;


	/* Copy first tab gad struct to the other tab gads */
	/* -------------------------------------------------------- */
	dt->dt_tabgad[1] = dt->dt_tabgad[0];
	dt->dt_tabgad[2] = dt->dt_tabgad[0];
	dt->dt_tabgad[3] = dt->dt_tabgad[0];
	dt->dt_tabgad[4] = dt->dt_tabgad[0];
	dt->dt_tabgad[5] = dt->dt_tabgad[0];
	dt->dt_tabgad[6] = dt->dt_tabgad[0];


	/* Initialize first intuittext struct */
	/* -------------------------------------------------------- */
	dt->dt_itxt[0].ITextFont = &TextAttr;
	dt->dt_itxt[0].FrontPen = 3;
	dt->dt_itxt[0].DrawMode = JAM2;
	dt->dt_itxt[0].TopEdge = 4;

	/* Copy first intuittext struct to the others */
	/* -------------------------------------------------------- */
	dt->dt_itxt[1] = dt->dt_itxt[0];
	dt->dt_itxt[2] = dt->dt_itxt[0];
	dt->dt_itxt[3] = dt->dt_itxt[0];
	dt->dt_itxt[4] = dt->dt_itxt[0];
	dt->dt_itxt[5] = dt->dt_itxt[0];
	dt->dt_itxt[6] = dt->dt_itxt[0];

	/* Set Attributes for object, gathering box of object */
	retval = dt_SetAttrs(cl,ob,ti);

	/* Create Gadget chain */
	/* -------------------------------------------------------- */
	dt->dt_cyclegad.NextGadget = &dt->dt_assigngad;
	dt->dt_assigngad.NextGadget = &dt->dt_tabgad[0];
	dt->dt_tabgad[0].NextGadget = &dt->dt_tabgad[1];
	dt->dt_tabgad[1].NextGadget = &dt->dt_tabgad[2];
	dt->dt_tabgad[2].NextGadget = &dt->dt_tabgad[3];
	dt->dt_tabgad[3].NextGadget = &dt->dt_tabgad[4];
	dt->dt_tabgad[4].NextGadget = &dt->dt_tabgad[5];
	dt->dt_tabgad[5].NextGadget = &dt->dt_tabgad[6];
	dt->dt_tabgad[6].NextGadget = NULL;

	return(retval);
}

static ULONG dt_SetAttrs( struct Class *cl, APTR ob, struct TagItem *ti )
{
	struct DirTabData		*dt = (struct DirTabData *)INST_DATA(cl,ob);
	struct TagItem			*s_ti = ti;
	ULONG						 retval = TRUE;
	
	while ( ti = ti_NextTagItem(&s_ti) )
	{
		switch (ti->ti_Tag)
		{
			case GA_Left:		dt->dt_box.Left = ti->ti_Data;	break;
			case GA_Top:		dt->dt_box.Top = ti->ti_Data;		break;
			case GA_Width:		dt->dt_box.Width = ti->ti_Data;	break;
			case GA_Height:	dt->dt_box.Height = ti->ti_Data;	break;

			case DTA_Window:
				dt->dt_win = (struct Window *)ti->ti_Data;
			break;
		}
	}

	dt_ReThink(cl,ob);

	return(retval);
}

static ULONG dt_ReThink( struct Class *cl, APTR ob )
{
	struct DirTabData		*dt = (struct DirTabData *)INST_DATA(cl,ob);
	ULONG		retval = 0;

	/* Align tops of all gadgets */
	dt->dt_tabgad[0].TopEdge = dt->dt_tabgad[1].TopEdge = 
	dt->dt_tabgad[2].TopEdge = dt->dt_tabgad[3].TopEdge = 
	dt->dt_tabgad[4].TopEdge = dt->dt_tabgad[5].TopEdge = 
	dt->dt_tabgad[6].TopEdge = dt->dt_box.Top - 2;
	dt->dt_cyclegad.TopEdge = dt->dt_assigngad.TopEdge = dt->dt_box.Top - 2;

	/* Arrange gadget horizontal positions */
	/* -------------------------------------------------------- */
	dt->dt_tabgad[0].LeftEdge	= dt->dt_box.Left;
	dt->dt_tabgad[1].LeftEdge	= dt->dt_tabgad[0].LeftEdge + dt->dt_tabgad[0].Width;
	dt->dt_tabgad[2].LeftEdge	= dt->dt_tabgad[1].LeftEdge + dt->dt_tabgad[1].Width;
	dt->dt_tabgad[3].LeftEdge	= dt->dt_tabgad[2].LeftEdge + dt->dt_tabgad[2].Width;
	dt->dt_tabgad[4].LeftEdge	= dt->dt_tabgad[3].LeftEdge + dt->dt_tabgad[3].Width;
	dt->dt_tabgad[5].LeftEdge	= dt->dt_tabgad[4].LeftEdge + dt->dt_tabgad[4].Width;
	dt->dt_tabgad[6].LeftEdge	= dt->dt_tabgad[5].LeftEdge + dt->dt_tabgad[5].Width;
	dt->dt_cyclegad.LeftEdge	= dt->dt_tabgad[6].LeftEdge + dt->dt_tabgad[6].Width - 1;
	dt->dt_assigngad.LeftEdge	= dt->dt_cyclegad.LeftEdge + dt->dt_cyclegad.Width;

	return(retval);
}

static ULONG dt_GetAttrs( struct Class *cl, APTR ob, struct TagItem *ti )
{
	struct DirTabData		*dt = (struct DirTabData *)INST_DATA(cl,ob);
	struct TagItem			*s_ti = ti;
	ULONG						 retval = 0;
	
	while ( ti = ti_NextTagItem(&s_ti) )
	{
		ULONG			*stor = (ULONG *)ti->ti_Data;

		/* add one to num attributes gotten */
		retval++;

		switch (ti->ti_Tag)
		{
			case GA_Left:		*stor = dt->dt_box.Left;		break;
			case GA_Top:		*stor = dt->dt_box.Top;			break;
			case GA_Width:		*stor = dt->dt_box.Width;		break;
			case GA_Height:	*stor = dt->dt_box.Height;		break;


			/* default case subtracts num attributes set */
			default:				retval--;							break;
		}
	}

	return(retval);
}

static struct EditWindow *HandleDirTabCycle(
	struct EditWindow *Edit,
	struct IntuiMessage	*im )
{
	if ( im->Class == IDCMP_GADGETUP )
	{
		APTR	tab_ob = ((struct Gadget *)im->IAddress)->SpecialInfo;

		/* Cycle forward/backward through tabs */

		if ( im->Qualifier & (IEQUALIFIER_LSHIFT|IEQUALIFIER_RSHIFT) )
			ob_DoMethod(tab_ob,DTM_CycleTabs,-7);
		else
			ob_DoMethod(tab_ob,DTM_CycleTabs,7);

		ob_DoMethod(tab_ob,DTM_RefreshTabs);

		/* Redraw the new tabs */
		RefreshGList(tab_ob,Edit->Window,NULL,2+7);
	}

	return(Edit);
}

static struct EditWindow *HandleDirTabSet(
	struct EditWindow *Edit,
	struct IntuiMessage	*im )
{
	if ( im->Class == IDCMP_GADGETUP )
	{
		DisplayMessage("Pick a directory tab to assign current path");
		global_setting_dirtab = TRUE;
	}

	return(Edit);
}

static STRPTR AssignLabelText[] = {
	"Please enter the text to",
	"appear on the tab's label",
};

static struct EditWindow *HandleDirTabSelect(
	struct EditWindow *Edit,
	struct IntuiMessage	*im )
{
	if (	(im->Class == IDCMP_GADGETUP) || (im->Class == IDCMP_GADGETDOWN) )
	{
		struct Grazer		*Grazer = (struct Grazer *)Edit->Special;
		struct Gadget		*gad = (struct Gadget *)im->IAddress;
		APTR					 tab_ob = gad->SpecialInfo;
		struct DirTabDir	*dtd;

		/* What tab does this represent */
		dtd = (struct DirTabDir *)GetListNode(&dirtab_list,gad->GadgetID);

		if ( im->Class == IDCMP_GADGETUP )
		{
			if ( global_setting_dirtab )
			{
				BOOL			new_dirtab = FALSE;

				DisplayMessage(NULL);
				global_setting_dirtab = FALSE;

				if ( !dtd )
				{
					/* Otherwise, create a new one */
					dtd = dtd_Create(GetCString(Grazer->Path),NULL);
					new_dirtab = TRUE;
				}

				if ( dtd )
				{
					strcpy(filepathbuf,dtd->dtd_label);

					if ( SimpleRequest(Edit->Window,
							AssignLabelText,2,
							REQ_STRING|REQ_CENTER|REQ_H_CENTER|REQ_OK_CANCEL,
							filepathbuf) )
					{
						strcpy(dtd->dtd_label,filepathbuf);

						/* Already Exists, change the path */
						EraseSmartString((struct SmartString *)dtd->dtd_node.ln_Name);
						AppendSmartString(Grazer->Path,(struct SmartString *)dtd->dtd_node.ln_Name);

						if ( new_dirtab )
						{
							AddTail( &dirtab_list, (struct Node *)dtd );
						}
						
						SaveDirectoryTabs("Edit_Support/DirTabs");
					}
					else if ( new_dirtab )
					{
						dtd_Delete(dtd);
					}

					ob_DoMethod(tab_ob,DTM_RefreshTabs);
					RefreshGList(tab_ob,Edit->Window,NULL,2+7);
				}

				/* Prompt for Direcory Tab Label */
			}
			else
			{
				if ( dtd )
				{
					EraseSmartString(Grazer->Path);
					AppendSmartString((struct SmartString *)dtd->dtd_node.ln_Name,Grazer->Path);
					DoAllNewDir(Edit);
				}
			}
		}
		else	/* GADGETDOWN */
		{
			if ( dtd )
			{
				/* Just render the text in the yellow-snow font */
				/* Note that this only works the first time */
				struct RastPort	*rp = Edit->Window->RPort;

				SetFont(rp,YellowFont);
				RenderDirTabLabel(rp,gad,dtd->dtd_label);
				SetFont(rp,EditFont);
			}
		}
	}

	return(Edit);
}

static LONG RenderDirTabLabel( struct RastPort *rp, struct Gadget *gad, char *label )
{
	extern struct TextExtent LastExtent;

	extern VOID __asm SafeColorText(
		register __a1 struct RastPort *RP,
		register __a0 STRPTR String,
		register __d0 WORD Length);

	extern UWORD __asm SafeFitText(
		register __a1 struct RastPort *RP,
		register __a0 STRPTR String,
		register __d0 WORD Length,
		register __d1 WORD WidthAvail,
		register __d2 BOOL RenderIt);

	LONG				x,y,len;

	len = SafeFitText(rp,label,strlen(label),gad->Width - 8,FALSE);

	x = (gad->Width - LastExtent.te_Width) >> 1;
	y = (gad->Height - 4);

	Move(	rp, gad->LeftEdge + x, gad->TopEdge + y );

	SafeColorText(rp,label,len);
	
	return(0);
}

/* ************************************************************************ */
/* 	Global Directory Tab Maintenance routines										 */
/* ************************************************************************ */

ULONG ReadDirectoryTabs( STRPTR filename )
{
	ULONG			retval = 0;
	APTR			fh;			/* Proof object file */

	if ( fh = ob_NewObject( NULL,STREAMBUFFERCLASS,
						STMBUFA_StmClassName, FILESTREAMCLASS,
						STMBUFA_BufSize,	1024,
						FSTMA_FileName, filename,
						TAG_DONE) )
	{
		char							*buf;

		while ( buf = (char *)ob_Gets(fh,filepathbuf,sizeof(filepathbuf)) )
		{
			char			*p;

			// null from comment to end of line
			if ( p = strchr(buf,';') )
			{
				*p = '\0';
			}

			if ( strlen(buf) > 0 )
			{
				char		*path = NULL;
				char		*label = NULL;

				/*	Example line in the file looks like
				 *
				 *	"Work:AlphaProject/Rev1","Alpha1"
				 *
				 */

				/* First char of path specification */
				if ( p = strchr(buf,'\"') )
				{
					path = ++p;

					/* Last char of path specification */
					if ( p = strchr(p,'\"') )
					{
						*p++ = '\0';

						/* First char of label */
						if ( p = strchr(p,'\"') )
						{
							label = ++p;

							/* last char of label */
							if ( p = strchr(p,'\"') )
							{
								*p = '\0';
							}
						}
						else
						{
							/* No Label specified, scan back to directory name */
						}
					}
				}

				if ( path && label )
				{
					struct DirTabDir *dtd;

					/* NULL last label character */
					label[29] = '\0';
					
					if ( dtd = dtd_Create(path,label) )
					{
						AddTail( &dirtab_list, (struct Node *)dtd );
						retval++;
					}
				}
			}
		}

		ob_Dispose(fh);
	}

	return(retval);
}

ULONG SaveDirectoryTabs( STRPTR filename )
{
	ULONG			retval = 0;
	APTR			fh;			/* Proof object file */

	if ( fh = ob_NewObject( NULL,STREAMBUFFERCLASS,
						STMBUFA_StmClassName, FILESTREAMCLASS,
						STMBUFA_BufSize,	(1024 * 2),
						FSTMA_FileName,	filename,
						FSTMA_OpenMode,	MODE_NEWFILE,
						TAG_DONE) )
	{
		struct DirTabDir	*dtd;

		for ( dtd = (struct DirTabDir *)dirtab_list.lh_Head;
				dtd->dtd_node.ln_Succ;
				dtd = (struct DirTabDir *)dtd->dtd_node.ln_Succ )
		{
			if ( !(retval % 7) )
			{
				ob_Printf(fh,"\n; Bank %ld Directory Tabs\n",(retval / 7));
			}

			ob_Printf( fh,"\"%s\",\"%s\"\n",
				GetCString((struct SmartString *)dtd->dtd_node.ln_Name),
				dtd->dtd_label );

			retval++;
		}

		retval = ob_Dispose(fh);
	}

	return(retval);
}

ULONG FreeDirectoryTabs( void )
{
	struct DirTabDir	*dtd;

	while ( dtd = (struct DirTabDir *) RemTail(&dirtab_list) )
	{
		dtd_Delete(dtd);
	}

	return(0);
}

static struct DirTabDir *dtd_Create( char *dirname, char *label )
{
	struct DirTabDir *dtd;

	if ( dtd = (struct DirTabDir *)AllocMem(sizeof(*dtd),MEMF_CLEAR) )
	{
		if ( dtd->dtd_node.ln_Name = (char *)AllocSmartString(dirname,0) )
		{
			if ( label )
				strcpy(dtd->dtd_label,label);
		}
		else
		{
			dtd_Delete(dtd);
			dtd = NULL;
		}
	}

	return(dtd);
}

static void dtd_Delete( struct DirTabDir *dtd )
{
	if ( dtd->dtd_node.ln_Name )
		FreeSmartString( (struct SmartString *)dtd->dtd_node.ln_Name );

	FreeMem(dtd,sizeof(*dtd));
}

/* ************************************************************************ */
/* 	General purpose MinList routines													 */
/* ************************************************************************ */

LONG GetListLen( struct List *list )
{
	LONG			listlen = 0;
	struct Node *node;

	for ( node = list->lh_Head;
			node = node->ln_Succ; )
	{
		listlen++;
	}

	return(listlen);
}

struct Node *GetListNode( struct List *list, LONG num )
{
	struct Node *node;

	for (	node = list->lh_Head;
			node->ln_Succ;
			node = node->ln_Succ )
	{
		if ( !num )
			break;

		num--;
	}

	if ( !node->ln_Succ )
		node = NULL;

	return(node);
}

