/********************************************************************
* DragHelp.c - implementation independent dragging routines
* $Id: DragHelp.c,v 2.1 1994/09/26 17:07:57 pfrench Exp $
* $Log: DragHelp.c,v $
*Revision 2.1  1994/09/26  17:07:57  pfrench
*WaitForVPos() had problem when optimized, required
*struct Custom * to be volatile.
*
*Revision 2.0  1994/02/17  16:24:35  Kell
*FirstCheckIn
*
*Revision 2.0  94/02/17  15:57:44  Kell
*FirstCheckIn
*
*Revision 2.0  94/02/17  14:44:52  Kell
*FirstCheckIn
*
*
* Copyright (c)1992 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	12-17-92	Steve H		Convert to use SmartStrings
*	12-17-92	Steve H		Last Update
*********************************************************************/
#include <exec/types.h>
#include <exec/memory.h>
#include <intuition/intuition.h>
#include <crouton_all.h>

#include <draghelp.h>

#include <proto/exec.h>
#include <proto/graphics.h>

#ifndef PROTO_PASS
#include <scrollgrid.p>
#include <draghelp.p>
#include <graphichelp.p>
#endif

struct Float {
	WORD FloatLeftEdge;
	WORD FloatTopEdge;		// offset into FloatBM (clipped to RenderRP)
	WORD RPLeftEdge;
	WORD RPTopEdge;		// offset into RenderRP (clipped)
	WORD Width;	// current width/height <= SaveBM width/height (clipped)
	WORD Height; // All WORDs instead of UWORDs to avoid constant conversions
	BOOL  Saved;			// is image currently saved?
	struct BitMap SaveBM;
	struct RastPort *RenderRP; // typically Screen's RastPort
};

#define SCREEN_PEN 1

/****** DragHelp/AllocFloat *********************************************
*
*   NAME   
*	AllocFloat
*
*   SYNOPSIS
*	struct Float *AllocFloat(WORD Width,WORD Height,UWORD Depth,
*		struct RastPort *RastPort)
*
*   FUNCTION
*	
*
*********************************************************************
*/
struct Float *AllocFloat(WORD Width, WORD Height, UWORD Depth,
	struct RastPort *RastPort)
{
	struct Float *Float = NULL;

	if (Width && Height && Depth && RastPort &&
		(Float = SafeAllocMem(sizeof(struct Float),MEMF_CLEAR))) {
		Float->RenderRP = RastPort;
		InitBitMap(&Float->SaveBM,Depth,Width,Height);
		if (!(HelpAllocBitMapPlanes(&Float->SaveBM,MEMF_CHIP))) {
			FreeFloat(Float);
			Float = NULL;
		}
	}
	return(Float);
}

/****** DragHelp/FreeFloat **********************************************
*
*   NAME   
*	FreeFloat
*
*   SYNOPSIS
*	VOID FreeFloat(struct Float *Float)
*
*   FUNCTION
*	
*
*********************************************************************
*/
VOID FreeFloat(struct Float *Float)
{
	if (Float) {
		HelpFreeBitMapPlanes(&Float->SaveBM);
		FreeMem(Float,sizeof(struct Float));
	}
}

/****** DragHelp/RenderFloat ********************************************
*
*   NAME   
*	RenderFloat
*
*   SYNOPSIS
*	BOOL RenderFloat(struct Float *Float,WORD LeftEdge,WORD TopEdge,
*		WORD Width,WORD Height,struct BitMap *Render)
*
*   FUNCTION
*	Clips to RenderRP->BitMap, saves area, renders Render bitmap
*	If Render NULL, destination area cleared	
*
*********************************************************************
*/
BOOL RenderFloat(struct Float *Float,WORD LeftEdge,WORD TopEdge,
	WORD Width,WORD Height,struct BitMap *Render)
{
	BOOL Success = FALSE;
	struct BitMap *BM;

	if (Float && Width && Height) {
		BM = Float->RenderRP->BitMap;

		if (LeftEdge < 0) {
			if ((-LeftEdge) > (Float->SaveBM.BytesPerRow << 3)) return(FALSE);
			Float->FloatLeftEdge = (-LeftEdge);
			Width += LeftEdge;
			LeftEdge = 0;
		} else Float->FloatLeftEdge = 0;

		if (TopEdge < 0) {
			if ((-TopEdge) > Float->SaveBM.Rows) return(FALSE);
			Float->FloatTopEdge = (-TopEdge);
			Height += TopEdge;
			TopEdge = 0;
		} else Float->FloatTopEdge = 0;

		if ((LeftEdge + Width) > (BM->BytesPerRow << 3))
			Width = (BM->BytesPerRow << 3) - LeftEdge;

		if ((TopEdge + Height) > BM->Rows)
			Height = BM->Rows - TopEdge;

		Float->RPLeftEdge = LeftEdge;
		Float->RPTopEdge = TopEdge;
		Float->Width = Width;
		Float->Height = Height;

		// save area under
		BltBitMap(BM,LeftEdge,TopEdge,&Float->SaveBM,
			Float->FloatLeftEdge,Float->FloatTopEdge,Width,Height,
			0xc0,0xff,NULL);

		// render onto screen
		if (Render) 
		BltBitMap(Render,Float->FloatLeftEdge,Float->FloatTopEdge,
			BM,LeftEdge,TopEdge,Width,Height,
			0xc0,0xff,NULL);

		else {
			SetAPen(Float->RenderRP,SCREEN_PEN);
			SetDrMd(Float->RenderRP,JAM2);
			RectFill(Float->RenderRP,LeftEdge,TopEdge,LeftEdge+Width-1,
				TopEdge+Height-1);
		}

		Float->Saved = Success = TRUE;
	}
	return(Success);
}

/****** DragHelp/RestoreFloat *******************************************
*
*   NAME   
*	RestoreFloat
*
*   SYNOPSIS
*	BOOL RestoreFloat(struct Float *Float)
*
*   FUNCTION
*	
*
*********************************************************************
*/
BOOL RestoreFloat(struct Float *Float)
{
	BOOL Success = FALSE;

	if (Float && Float->Saved) {
		BltBitMap(&Float->SaveBM,Float->FloatLeftEdge,Float->FloatTopEdge,
			Float->RenderRP->BitMap,
			Float->RPLeftEdge,Float->RPTopEdge,Float->Width,Float->Height,
			0xc0,0xff,NULL);
		Float->Saved = FALSE;
		Success = TRUE;
	}
	return(Success);
}

/****** DragHelp/WaitForVPos ********************************************
*
*   NAME   
*	WaitForVPos
*
*   SYNOPSIS
*	VOID WaitForVPos(UWORD Position)
*
*   FUNCTION
*	
*
*********************************************************************
*/
VOID WaitForVPos(UWORD Position)
{
	volatile struct Custom *custom = (struct Custom *)0xdff000;

	if (Position > 250) Position = 250;
	while ((custom->vhposr>>8) > Position) { };
	while ((custom->vhposr>>8) < Position) { };
}

/****** DragHelp/ClipToGadget *******************************************
*
*   NAME   
*	ClipToGadget
*
*   SYNOPSIS
*	VOID ClipToGadget(struct Box *Box,struct Gadget *Gadget)
*
*   FUNCTION
*	
*
*********************************************************************
*/
VOID ClipToGadget(struct Box *Box,struct Gadget *Gadget)
{
	WORD X1,Y1,X2,Y2,GX2,GY2;

	if (Box && Gadget) {
		X1 = Box->LeftEdge;
		Y1 = Box->TopEdge;
		X2 = Box->LeftEdge + Box->Width - 1;
		Y2 = Box->TopEdge + Box->Height - 1;
		GX2 = Gadget->LeftEdge + Gadget->Width - 1;
		GY2 = Gadget->TopEdge + Gadget->Height - 1;

		if (X1 < Gadget->LeftEdge) X1 = Gadget->LeftEdge;
		if (Y1 < Gadget->TopEdge) Y1 = Gadget->TopEdge;
		if (X2 > GX2) X2 = GX2;
		if (Y2 > GY2) Y2 = GY2;

		Box->LeftEdge = X1;
		Box->TopEdge = Y1;
		Box->Width = X2 - X1 + 1;
		Box->Height = Y2 - Y1 + 1;
	}
}

// end of draghelp.c
