/*********************************************************************\
*
* DriveFirmware - Downloads firmware to hard drive
*
* $Id: DriveFirmware.c,v 1.0 1995/05/05 15:49:04 Flick Exp $
*
* $Log: DriveFirmware.c,v $
*Revision 1.0  1995/05/05  15:49:04  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	10/27/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);
ULONG ReadBinFile(char *);
void FreeFile(void);


struct Library *FlyerBase;
struct FlyerVolume FlyVol;

UBYTE	*Buffer;
ULONG	Length;

struct DriveParam {
	UBYTE	scsiver,inqlen;
	ULONG	size;
};

main(int argc,char **argv)
{
	int	unit,drive,i,j,len;
	ULONG	res;
	UBYTE	MiscBuff[18],drives,InqBuff[255];
	struct	DriveParam	Stats[8];

	if ((argc!=4)||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <drive> <file>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	FlyVol.Board = unit;
	GetArg(argv[2],&drive,0,23,"Drive");
	FlyVol.SCSIdrive = drive;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	printf("Initializing SCSI channel\n");
	res = SCSIinit(&FlyVol);
	if (res) {
		printf("Error %ld - %ls\n",res,Error2String(res));
	}
	printf("Finding drives on bus\n");
	res = FindDrives(&FlyVol,MiscBuff);
	if (res) {
		printf("Error %ld - %ls\n",res,Error2String(res));
	}

	drives = MiscBuff[0];
	for (i=0;i<8;i++) {
		Stats[i].scsiver = MiscBuff[2+i];
		Stats[i].inqlen  = MiscBuff[10+i]+5;
	}

	len = Stats[drive % 8].inqlen;
	res = Inquiry(&FlyVol,len,InqBuff);
	if (res) {
		printf("Inquiry Error %ld - %ls\n",res,Error2String(res));
	}
	else {
		printf("   ");
		for (j=8;j<=31;j++)
			printf("%c",InqBuff[j]);
		printf("\n   Rev: ");
		for (j=32;j<=35;j++)
			printf("%c",InqBuff[j]);
		if (len>=43) {
			printf("   Date: ");
			for (j=36;j<=43;j++)
				printf("%c",InqBuff[j]);
		}
		if (len>=55) {
			printf("   SerNum: ");
			for (j=44;j<=55;j++)
				printf("%c",InqBuff[j]);
		}
		printf("\n");

#if 0
		size = ReadBinFile(argv[3]);
		if (size) {
/* Build SCSI command to download firmware here */

		res = FlyerSCSIdirect(board,drive,&SCSIinfo,sizeof(struct SCSIinfo));

		}
		else
			printf("Could not locate firmware file\n");
#endif
	}

	FreeFile();
	CloseLibrary(FlyerBase);

	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}


ULONG ReadBinFile(char *name)
{
	char	filename[80];
	FILE	*fp;
	int	read;

	strcpy(filename,name);
	if (stricmp(".BIN",&filename[strlen(filename)-4]))
		strcat(filename,".BIN");

	read = 0;
	fp = fopen(filename,"r");
	if (fp) {
		fseek(fp,0,SEEK_END);
		Length = ftell(fp);
		fseek(fp,0,SEEK_SET);

		Buffer = AllocMem(Length,MEMF_PUBLIC);
		if (Buffer) {
			read = fread(Buffer,Length,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",filename);

	if (read==1)
		return(Length);
	else
		return(0);
}

void FreeFile(void)
{
	if (Length)
		FreeMem(Buffer,Length);
}
