/*********************************************************************\
*
* Flyer FileSystem - Interfaces AmigaDOS to the Flyer card(s)
*							Module 3 - Locks and errors
*
* $Id: FlyerFS3.c,v 1.1 1995/08/04 15:02:51 Flick Exp $
*
* $Log: FlyerFS3.c,v $
*Revision 1.1  1995/08/04  15:02:51  Flick
*Big cleanup -- removed all globals
*
*Revision 1.0  1995/05/05  15:49:41  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/22/94		Marty	created
\*********************************************************************/

#include "exec/types.h"
#include "exec/memory.h"
#include "exec/ports.h"
#include "exec/io.h"
#include "exec/alerts.h"
#include "devices/trackdisk.h"
#include "devices/scsidisk.h"
#include "dos/dosextens.h"
#include "intuition/intuition.h"
#include	"workbench/workbench.h"
#include "proto/exec.h"
#include "proto/dos.h"
#include "proto/intuition.h"

#include "flyerlib.h"
#include "flyer.h"
#include	"FlyerFS.h"

#include "FlyerFS3.ps"
#include "FlyerFS1.p"
#include "FlyerFS2.p"
#include "FlyerFS3.p"

#define	DEBUGGING	0

#if DEBUGGING
#define	DBUG(x)	x
void kprintf(char *, ... );
#else
#define	DBUG(x)	/* nada */
#endif


BPTR RequestLock(struct FSbase *FS,ULONG blk,int acc,ULONG grip)
{
	struct	LongLock *ll;
//	LONG		temp;

/* This whole section can probably be removed (no write exclusivity) */
	for (ll = (struct LongLock *)BADDR(FS->MasterLockList)
	  ; ll
	  ; ll = (struct LongLock *)BADDR(ll->ll_Link))
	{
		if (ll->ll_Key == blk)
		{
			if ((acc == EXCLUSIVE_LOCK)||(ll->ll_Access == EXCLUSIVE_LOCK))
			{
				FS->DOSerror = ERROR_OBJECT_IN_USE;
				return(DOSFALSE);
			}
			break;
		}
	}

	ll = (struct LongLock *)AllocMem(sizeof(struct LongLock),MEMF_PUBLIC|MEMF_CLEAR);
	if (ll == NULL)
	{
		FS->DOSerror = ERROR_NO_FREE_STORE;
		return(DOSFALSE);
	}
	ll->ll_Key = blk;		/* I never use this, it's just for applications to see */
	ll->ll_Access = acc;
	ll->ll_Task = (APTR)FS->myMsgPort;	/* Ptr to my task port */
	ll->ll_Volume = FS->myVolume;			/* BPTR to DLT_VOLUME DosList entry */
	ll->ll_Link = FS->MasterLockList;
	ll->ll_Grip = grip;

	FS->MasterLockList = (BPTR)MKBADDR(ll);	// Put on top of Lock list

	return(FS->MasterLockList);
}


LONG RemoveLock(struct FSbase *FS,BPTR lock)
{
	struct	LongLock		*ll;
	struct	DeviceList	*vol;
	BPTR		*p;
	BOOL		VolLock = FALSE;
	ULONG		grip;

	ll = (struct LongLock *)BADDR(lock);
	if (ll == NULL)
		return(DOSTRUE);		// no lock

	vol = (struct DeviceList *)BADDR(ll->ll_Volume);

	p = (BPTR *)&FS->MasterLockList;
	while (*p)
	{
		if (*p == lock) goto LockFound;
		p = (BPTR *)BADDR(*p);
	}

	if (vol->dl_Type == DLT_VOLUME)
	{
		VolLock = TRUE;						// if one found here, belongs to volume
		p = (BPTR *)&vol->dl_LockList;
		while (*p)
		{
			if (*p == lock) goto LockFound;
			p = (BPTR *)BADDR(*p);
		}
	}
	FS->DOSerror = ERROR_INVALID_LOCK;
	return(DOSFALSE);

LockFound:
	*p = ll->ll_Link;									// unlink me
	grip = ll->ll_Grip;
	FreeMem((UBYTE *)ll,sizeof(struct LongLock));

	FS->FlyVol.Path = 0;
	if (FlyerFreeGrip(&FS->FlyVol,grip))
	{
		FS->DOSerror = ERROR_INVALID_LOCK;
		return(DOSFALSE);
	}

//	/* For removable support only */
//	if ((VolLock)&&(vol->dl_LockList == NULL))
//		DeleteVolume((BPTR)MKBADDR(vol));

	return(DOSTRUE);
}


BOOL GetLocksInfo(struct FSbase *FS,BPTR lck,ULONG *grip,ULONG *block)
{
	struct	LongLock		*lock,*ll;
	struct	DeviceList	*vol;

	lock = (struct LongLock *)BADDR(lck);

	if (lock == NULL)
	{
		if (FS->ID == ID_FLYER_DISK)
		{
			*grip = 0;
			*block = 0;
			return(TRUE);
		}
		else if (FS->ID == ID_NO_DISK_PRESENT)
			FS->DOSerror = ERROR_NO_DISK;
		else
			FS->DOSerror = ERROR_NOT_A_DOS_DISK;
		return(FALSE);
	}
	else
	{
		for (ll = (struct LongLock *)BADDR(FS->MasterLockList)
		  ; ll
		  ; ll = (struct LongLock *)BADDR(ll->ll_Link))
		{
			DBUG(kprintf("try %lx ",ll);)
			if (ll == lock)
			{
				*block = lock->ll_Key;
				*grip = lock->ll_Grip;
				return(TRUE);
			}
		}

		vol = (struct DeviceList *)BADDR(lock->ll_Volume);
		if (vol->dl_Type == DLT_VOLUME)					// FIXED! (was '=')
			FS->DOSerror = ERROR_DEVICE_NOT_MOUNTED;
		else
			FS->DOSerror = ERROR_INVALID_LOCK;
		DBUG(kprintf("!!!");)
		return(FALSE);
	}
}


ULONG Lock2Grip(BPTR lck)
{
	struct	LongLock		*lock;

	lock = (struct LongLock *)BADDR(lck);

	if (lock == NULL)
		return(0);
	else
		return(lock->ll_Grip);
}



/*******************/
/* Errors and such */
/*******************/


const struct IntuiText OkayText = {
	0,1,JAM2,6,3,NULL,
	"Okay",
	NULL
};

const struct IntuiText RetryText = {
	0,1,JAM2,6,3,NULL,
	"Retry",
	NULL
};

const struct IntuiText CancelText = {
	0,1,JAM2,6,3,NULL,
	"Cancel",
	NULL
};



int MakeRequest(struct FSbase *FS,struct IntuiText *text,int rw)
{
	struct IntuitionBase *IntuitionBase;
	int	status = 0;

	DBUG(kprintf("ReqMaker (%lx)\n",FS->myProcess->pr_WindowPtr);)

	if ((ULONG)FS->myProcess->pr_WindowPtr>=0)  /* Don't if req are cancelled */
	{
		IntuitionBase = (struct IntuitionBase *)OpenLibrary("intuition.library",33);
		if (IntuitionBase)
		{
			if (rw)
			{
				status = AutoRequest((struct Window *)FS->myProcess->pr_WindowPtr,text,
				(struct IntuiText *)&RetryText,
				(struct IntuiText *)&CancelText,
				DISKINSERTED,0,320,62);
			}
			else
			{
				status = AutoRequest((struct Window *)FS->myProcess->pr_WindowPtr,text,NULL,
				(struct IntuiText *)&OkayText,
				0,0,320,62);
			}
			CloseLibrary((struct Library *)IntuitionBase);
		}
	}
	return(status);
}


const struct IntuiText	ReadWrite2Text = {
	0,1,JAM2,15,15,NULL,
	"has a read/write error",
	NULL
};

const struct IntuiText ReadWriteText_tmp = {
	0,1,JAM2,15,5,NULL,
	NULL,
	&ReadWrite2Text
};


ULONG ReadWriteError(struct FSbase *FS,BYTE num)
{
	char		buf[64];
	UBYTE		*ptr;
	int		stat;
	struct	DeviceList	*p1;

	DBUG(kprintf("R/W error routine\n");)

	FS->DOSerror = num;
	if (num == TDERR_DiskChanged)
		return((ULONG)FS->myVolume);

	CopyMem("Volume ",buf,7);
	p1 = (struct DeviceList *)BADDR(FS->myVolume);
	ptr = (UBYTE *)BADDR(p1->dl_Name);
	CopyMem(ptr+1,buf+7,*ptr);
	buf[7+(*ptr)] = 0;

/*	strncat(buf,ptr+1,*ptr);  OLDER METHOD */

	// Copy template of Intuitext into my local FS
	CopyMem((APTR)&ReadWriteText_tmp,&FS->ReadWriteText,sizeof(struct IntuiText));
	FS->ReadWriteText.IText = buf;			// Customize now

	stat = MakeRequest(FS,&FS->ReadWriteText,1);
	return((ULONG)stat);
}


const struct IntuiText DupeVol3Text = {
	0,1,JAM2,15,25,NULL,
	"Please change to unique name",
	NULL
};

const struct	IntuiText	DupeVol2Text_tmp = {
	0,1,JAM2,15,15,NULL,
	NULL,
	&DupeVol3Text
};

const struct IntuiText DupeVolText_tmp = {
	0,1,JAM2,15,5,NULL,
	"Flyer volume naming conflict:",
	&DupeVol2Text_tmp			// Must be changed when preparing
};


ULONG DuplicateVolumeError(struct FSbase *FS,struct DeviceList *vol)
{
	UBYTE		*ptr;
	int		stat;

	ptr = (UBYTE *)BADDR(vol->dl_Name);

	DBUG(kprintf("DupeVol:%ls\n",ptr+1);)

	// Copy templates of Intuitext's into my local FS
	CopyMem((APTR)&DupeVolText_tmp,&FS->DupeVolText,sizeof(struct IntuiText));
	CopyMem((APTR)&DupeVol2Text_tmp,&FS->DupeVol2Text,sizeof(struct IntuiText));
	FS->DupeVol2Text.IText = ptr+1;						// Customize now
	FS->DupeVolText.NextText = &FS->DupeVol2Text;	// Link together

	stat = MakeRequest(FS,(struct IntuiText *)&FS->DupeVolText,0);
	return((ULONG)stat);
}


struct Library *OpenFlyer(struct FSbase *FS)
{
	FS->FlyBase = OpenLibrary(FLYERLIBNAME,FLYER_MIN_VER);

	return(FS->FlyBase);
}


void CloseFlyer(struct FSbase *FS)
{
	if (FS->FlyBase)
		CloseLibrary(FS->FlyBase);
	FS->FlyBase = NULL;
}


int GetDiskState(struct FSbase *FS)
{
	ULONG	error;

	FS->FlyVol.Path = NULL;

	error = FlyerDriveCheck(&FS->FlyVol);			// Talk to Flyer...

	DBUG(kprintf("DrvChk error code %ld\n",error);)

	if (error == FERR_OKAY)
		return(1);
	else
	{
		return(0);
	}
}


BOOL CheckName(struct FSbase *FS,char *name)
{
	int	len;
	char	c;
	char	*p = name;

	FS->DOSerror = ERROR_INVALID_COMPONENT_NAME;		// If any problem

	len = *p++;
	if ((len==NULL) || (len>30))
		return(FALSE);

	while (len--)
	{
		c = (*p++) & 0x7F;
		if ((c==':') || (c=='/') || (c<' '))
			return(FALSE);
	}
	FS->DOSerror = 0;
	return(TRUE);
}
