/*********************************************************************\
*
* FlyerHDTools 1.0 - Test Flyer drive speed
*
* Copyright (c) 1994,1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* 01-19-95      Marty    Created original RWtest
* 01-20-95      Todd     Created DriveSpeed based on original RWtest
* 01-30-95      Todd     Added speed drop reporting
* 01-31-95      Todd     Removed all "grading" code
* 02-01-95      Todd     Removed all "target" info
* 02-15-95      Todd     Changed timing info to ms
* 03-08-95      Todd     added NOSTAT, LIST, SPEED
* 04-27-95      Todd     Changed NOSTAT to FILE
* 04-27-95      Todd     Added check for dips only
* 07-26-95      Todd     Added GUI, support for multiple tests,
*                        saving logs, Mode Page viewing.
* 07-26-95      Todd     Fixed bug clearing DriveStr on rescan, and
*                        improperly marking idle times on repeated tests
* 08-25-95		 Todd		 Changed defaults to repeat test 3 times and check
*								 for HQ5 capability
* 08-28-95      Todd     Now puts Flyer in Record Mode before testing
*								 (resets to Play Mode at end of test)
* 11-10-95		 Todd		 Added Cache parameter loading, saving, setting
*
\*********************************************************************/

/* INCLUDES */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <clib/dos_protos.h>
#include <clib/exec_protos.h>
#include <clib/gadtools_protos.h>
#include <clib/graphics_protos.h>
#include <clib/intuition_protos.h>
#include <clib/rexxsyslib_protos.h>
#include <clib/asl_protos.h>
#include <dos/dos.h>
#include <exec/types.h>
#include <exec/memory.h>
#include <intuition/gadgetclass.h>
#include <intuition/intuition.h>
#include <libraries/gadtools.h>
#include <libraries/asl.h>
#include <proto/exec.h>
#include <rexx/storage.h>
#include "flyerlib.h"
#include "flyer.h"
#include "flyerFS.h"

#define FlyerBase FlyerBase   // This will cause a warning that can be ignored

/* DEFINES */
#define	FHDTVERSION "FlyerHDTools 4.1"
#define	TESTBLKS (512)		/* Each transfer will be 512 blocks (256K) */
#define	REPEAT	(64)			/* Do 64 transfers for each test */
#define	TOTALSIZE	(TESTBLKS*REPEAT)
#define	NUMTESTS		(4)
#define	DBUG(x)			if (Debugging) { x }
#define EN_LENGTH 3
#define EN_REPEAT 4
#define EN_SPEED 4


/***** Gadget IDs for all Gadgets *****/
#define DSGAD_DRIVELIST				(0)
#define DSGAD_TESTTYPE 				(1)
#define DSGAD_LENGTH					(2)
#define DSGAD_TARGETSPD				(3)
#define DSGAD_LENGTHMIN				(4)
#define DSGAD_TARGETNUM				(5)
#define DSGAD_RESLABLOW				(6)
#define DSGAD_RESLABAVG				(7)
#define DSGAD_RESLABIDL				(8)
#define DSGAD_RESLISTLR				(9)
#define DSGAD_RESLISTAR				(10)
#define DSGAD_RESLISTIR				(11)
#define DSGAD_RESLISTLW				(12)
#define DSGAD_RESLISTAW				(13)
#define DSGAD_RESLISTIW				(14)
#define DSGAD_VIEWLOG				(15)
#define DSGAD_SAVELOG				(16)
#define DSGAD_VIEWCACHE				(17)
#define DSGAD_PERCENT				(18)
#define DSGAD_STATUS					(19)
#define DSGAD_START					(20)
#define DSGAD_RESCAN					(21)
#define DSGAD_EXIT					(22)
#define DSGAD_RESBOX					(23)
#define DSGAD_LOGLIST				(24)
#define DSGAD_LOGCONT				(25)
#define DSGAD_LOGSAVE				(26)
#define DSGAD_SELDRIVE  			(27)
#define DSGAD_AUTOSAVE  			(28)
#define DSGAD_GOFORMAT				(29) //Opens Format window
#define DSGAD_VOLNAME				(30)
#define DSGAD_AVFLAGS				(31)
#define DSGAD_DOSHORT				(32)
#define DSGAD_SSVAL					(33)
#define DSGAD_DOFORMAT				(34) //Performs format
#define DSGAD_FDEVICE				(35)
#define DSGAD_FREV					(36)
#define DSGAD_FNAME					(37)
#define DSGAD_FCAPACITY				(38)
#define DSGAD_SETCACHE				(39)
#define DSGAD_DEFCACHE				(40)
#define DSGAD_TESTSS					(41)
#define DSGAD_NOOP      			(42)
#define DSGAD_READCACHE 			(43)
#define DSGAD_READDATA  			(44)
#define DSGAD_SAVEDATA  			(45)
#define DSGAD_LONGLOG				(46)
#define DSGAD_TESTERNAME 			(47)
#define DSGAD_TESTERCONT 			(48)
#define DSGAD_TESTOPTLABEL 		(49)
#define DSGAD_TESTER 				(50)
#define DSGAD_COMMLABEL				(51)
#define DSGAD_COMMENT 				(52)
#define DSGAD_COMMCONT 				(53)
#define DSGAD_MANUFLIST				(54)
#define DSGAD_WRITECACHE 			(55)
#define DSGAD_VIEWSELCACHE 		(56)
#define DSGAD_COPYCACHE 			(57)
#define DSGAD_DELCACHE		 		(58)
#define DSGAD_EDITCOMMENT	 		(59)
#define DSGAD_SELCACHE				(60)
#define DSGAD_GOCACHE				(61)
#define DSGAD_MANUFLABEL			(62)

#define MAX_GADS        			(62)

USHORT __chip WaitPointer[36] =
{
	0x0000, 0x0000, 0x0400, 0x07C0, 0x0000, 0x07C0, 0x0100, 0x0380, 0x0000,
	0x07E0, 0x07C0, 0x1FF8, 0x1FF0, 0x3FEC, 0x3FF8, 0x7FDE, 0x3FF8, 0x7FBE,
	0x7FFC, 0xFF7F, 0x7EFC, 0xFFFF, 0x7FFC, 0xFFFF, 0x3FF8, 0x7FFE, 0x3FF8,
	0x7FFE, 0x1FF0, 0x3FFC, 0x07C0, 0x1FF8, 0x0000, 0x07E0, 0x0000, 0x0000
};

/*
 * These two defines set up and clear the WaitPointer...
 */
#define	SetWait(x)	SetPointer(x,WaitPointer,16L,16L,-6L,0L)
#define	ClearWait(x)	ClearPointer(x)

/* STRUCTURES */

struct LBAStats 
{
	struct LBAStats *prev,*next;
	ULONG lba;
	ULONG ticks;
	double speed;
	long drop;
	long drop_ms;
	int dip;
};

struct TestStats 
{
	struct TestStats 	*prev, *next;
	struct LBAStats 	*first,*last;
	int 					num_tests;
	struct LBAStats 	*lowest;
	double 				average;
	struct LBAStats	*max_drop;
};

struct TextNode 
{
	struct Node node;
	UBYTE str[128];
};

struct Monitor
{
	struct RastPort *rp;
	LONG x,y;
	LONG w,h;
	ULONG steps;
	ULONG curr_step;
	
};

struct Options 
{
	struct Options *next;
	char	optname[16];
	UBYTE	page;
	UBYTE	pos;
	UBYTE	flag;
	LONG	value;
};

struct Manuf 
{
	struct Node 	 node;
	UBYTE				 str[128];
	int				 index;
	char				 manufname[9];
	char				 modelname[17];
	char				 revision[5];
	ULONG				 blocks;
	char  			 comment[33];
	struct Options *options;
	ULONG 			 chksum;
};

struct DLNode
{
	struct Node 			node;
	UBYTE 					str[128];
	ULONG						size;
	ULONG 					ss_block;	// This is the block that the drive is
												// currently short stroked to 
	ULONG 					blklen;
	int   					drive ;
	struct FlyerVolume	FlyVol;
	struct Manuf			Manuf;
	struct List			  *Log;         // These will actually point to 
	struct List			  *CacheList;   // TextNode lists
	
};

struct AppData 
{

	/* main screen gadget data area */
	UBYTE ad_LengthIndex;      /* Set this for initial value of cycle gadget */
	char **ad_LengthLabels;

	UBYTE ad_SpeedIndex;			/* set for initial test length */
	char ad_SpeedStr[16];
	char ad_PercentStr[16];
	ULONG ad_ss_block;			// From the results of the test
	float ad_ss_val;
	char ad_ResList[6][32];
	char ad_DriveStr[80];     // Global description of the current drive
	char ad_CacheStr[80];     // Global description of the current Cache setting
	int	ad_do_read;
	int   ad_do_write;
	BOOL ad_autosave;
	BOOL ad_longlog;
	BOOL ad_changedconfig;
	struct Window *ad_StatusWin;
	struct Gadget **ad_StatusGadList;
	char ad_StatusStr[64];
	struct Monitor ad_mon;
	char ad_Tester[128];
	
	/* Format window gadget area */
	UBYTE ad_do_ss;
	ULONG ad_fmt_ss_block;
	char ad_ss_str[16];
	BOOL ad_videoflag;
	char ad_VolNameStr[128];
	char ad_SetCacheLabelStr[64];
	char ad_FormatBuffer[256];

	/* System information variables */
	int ad_GadLib;
	char *ad_version;

	/* List files for listview displays.  The drive list also holds information
		regarding current drive/cache settings and is doubly referenced through
		an array of pointers */
	struct DLNode *ad_DriveInfo[24];
	int ad_SelDriveIndex, ad_SelCacheIndex;
	struct List *ad_DriveList, *ad_CacheList, *ad_ManufList;

	/* drive unique data variables */	
	
	/* Global pointers for data file names */
	char ad_LogFile[64];
	char ad_LogDir[128];
	char ad_DataFile[64];
	char ad_DataDir[128];
	char ad_CfgFile[64];
	char ad_CfgDir[128];

	char ad_tempstr[128];    // Temporary variable for passing strings between
								 // functions indirectly.
};

/************** FUNCTION PROTOS ***************/
BOOL HandleGadgetEvent(struct Window *win, struct Gadget *gad, UWORD code,
								struct Gadget *gadlist[], UWORD menuID);
struct Gadget *CreateMainGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[]);
void ProcessWindowEvents(struct Window *win, struct Gadget *gadlist[], struct Menu *menuStrip);
void DSWindow( void );
void SetCacheWindow( void );
struct Gadget *CreateSetCacheGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[]);
void FormatWindow( struct DLNode *dln );
void UpdateFormatWindow(struct Window *win, struct Gadget *gadlist[], 
								struct DLNode *dln, char *gb);
struct Gadget *CreateFormatGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[]);
void UpdateDriveList(struct Window *win, struct Gadget **gadlist);
void GetDriveList(struct Window *win, struct Gadget **gadlist);
void AddDListNode(struct List *list, int index);
void FreeDList( struct List *list );
void GetDriveInfo(struct DLNode *dln);
int FindToaster( void );
ULONG Str2ULONG(char *str);
ULONG Hex2ULONG(char *str);
float Str2Float(char *str);
void ErrorMessage(char *msg);
void StatusMessage(char *msg);
void cleanup( void );
BOOL StartTest(struct Window *win, struct Gadget *gadlist[], struct DLNode *dln );
UBYTE WriteWarning(struct Window *win );
UBYTE Warning(struct Window *win , char *msg );
void AskReformat( struct Window *win, struct FlyerVolInfo *info,
						struct DLNode *dln);
void UpdateSummaryWindow(struct Window *win, struct Gadget **gadlist);
ULONG DoTest(ULONG lba,ULONG testblks,ULONG repeat,int writeflag, struct DLNode *dln);
ULONG ElapsedTicks(struct DateStamp *begin, struct DateStamp *end);
struct LBAStats *NewStat(struct LBAStats *old, struct TestStats *Test, ULONG lba,
								 ULONG ticks);
void DoStats(struct TestStats *test);
void UpdateShortStroke(struct Window *win, struct Gadget **gadlist, struct DLNode *dln);
ULONG	FindLastBlockAtSpeed(double spd, int read, int write);
void CreateLog( struct DLNode *dln );
void CreateCacheList( struct Manuf *man , struct List **listptr);
void ViewLog( struct List *list );
struct Gadget *CreateLogGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[], struct List *list);
void SaveLog( struct List *list );
void WriteListFile(struct List *list, char *dir, char *file);
void ViewCache( struct List *list ); 
struct Gadget *CreateCacheGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[], struct List *list);
void PurgeMsgPort(struct Window *win);
void InitStats( void );
void ValidVolName( char *name );
void FormatDrive( struct DLNode *dln );
void SetCache( struct DLNode *dln, struct Options *opt );
BOOL VerifyFormat(struct Window *win );
void clear( char *str, ULONG len );
void Init_Monitor(struct Monitor *mon, struct Window *win, LONG x, LONG y, LONG w, LONG h, ULONG steps );
void Inc_Monitor( struct Monitor *mon );
void Step_Monitor( struct Monitor *mon, ULONG steps );
void End_Monitor( struct Monitor *mon );
void InitAppData( void );
int AddDriveData(struct Window *win, char *dir, char *filepart);
void SetOption( struct DLNode *dln, struct Options *opt);
void GetManuf( struct DLNode *dln );
UWORD Menu2ID(UWORD menu, UWORD item, UWORD sub, UWORD *code);
void FetchCacheSettings( struct DLNode *dln);
int WriteDriveData(char *dir, char *filepart, struct Manuf *man);
BOOL VerifyChecksum(struct Window *win, struct Manuf *man);
void AskTester( void );
void AskComment( struct Manuf *man );
struct Gadget *CreateTesterGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[]);
struct Gadget *CreateCommentGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[], struct Manuf *man);
BOOL ReqFile( char *title, char *dirpart, char *filepart );
BOOL CopyFlyerIcon( int drive );
void AddTextNode( struct List *list, char *str );
void FreeTextList( struct List *list );
void InitManufList( void );
struct Manuf *FindIndex( struct List *list, int index);
void ErrorReq(struct Window *win , char *msg);
void IndexList( struct List *list);
void ReadDriveCache(struct DLNode *dln);
void CopyManufNode( struct Manuf *man);
void DeleteManufNode(struct Manuf *man);
void AddManufNode( struct Manuf *man );
void SortOptions(struct Manuf *man);
void SortList(struct List *list);
struct Manuf *ParamsExist(struct DLNode *dln);

/******************* GLOBALS ******************/
struct TextAttr Topaz80 = { "topaz.font", 8, 0, 0, };

struct Library *IntuitionBase = NULL;
struct Library *GfxBase = NULL;
struct Library *GadToolsBase = NULL;
struct Library *AslBase = NULL;
struct Library	*FlyerBase = NULL;

char *TypeLabels[] = {"Read","Write","Read/Write", NULL};
char *LengthLabels1[] = {"Full","5 Minutes", "10 Minutes", "Custom", "Repeat", NULL};
char *LengthLabels2[] = {"Full","10 Minutes", "20 Minutes", "Custom", "Repeat", NULL};
ULONG	LengthVals[] = { 0, 5, 10, 0, 3 };  
char *SpeedLabels[] = {"None", "Audio", "Standard", "HQ5", "Custom", NULL};
float SpeedVals[] = { 0.0, 0.0, 3.7, 4.8, 0.0 };
char *AVFlagLabels[] = {"Audio/Video","Audio Only", NULL};
char *SSLabels[] = {"None", "Percent", "Blocks", NULL};
UWORD topborder;
struct NewMenu DSMenu[] = 
{
	{NM_TITLE,	"Project",					  0, 0, 0, 0,},
	{	NM_ITEM, "Save Log File...",		"S", 0, 0, 0,},
	{	NM_ITEM, "Save Config File",		"s", 0, 0, 0,},
	{	NM_ITEM,	NM_BARLABEL,		 		  0, 0, 0, 0,},
	{	NM_ITEM,	"QUIT",				 		"q", 0, 0, 0,},
	{NM_TITLE,	"Options",					  0, 0, 0, 0,},
	{	NM_ITEM, "Long Log File?",			"l", CHECKIT | MENUTOGGLE, 0, 0,},
	{	NM_ITEM, "AutoSave Log File...", "a", CHECKIT | MENUTOGGLE, 0, 0,},
	{NM_TITLE,	"Drive Settings",			  0, 0, 0, 0,},
	{	NM_ITEM, "Add New Drive Data",	  0, 0, 0, 0,},
	{		NM_SUB, "From Drive",			"d", 0, 0, 0,},
	{		NM_SUB, "From File...", 		"f", 0, 0, 0,},
	{NM_END,		NULL, 						  0, 0, 0, 0,},
};

struct AppData *ad;


struct TestStats 		read_stats =  { NULL, NULL, 0, NULL, 0.0, NULL };
struct TestStats		write_stats = { NULL, NULL, 0, NULL, 0.0, NULL };


/************** FUNCTION BODIES ***************/

/******************************************************\
*
*	FUNCTION:
*		void main( int argc, char *argv[])
*
*	SYNOPSIS:
*		Main function: opens all libraries and initializes 
*		data to start program
*
*	INPUT:
*		none
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void main( int argc, char *argv[])
{
	BOOL usetoast = FALSE;

	InitAppData();
	ad->ad_version = "$VER:4.10 FlyerHDTools (11-10-95)\nCopyright  1995 NewTek, Inc.\n";


 //  MAKE SURE THAT THE TOASTER IS RUNNING 
	if ((argc == 2) && (!(stricmp(argv[1], "NOTOAST"))))
		usetoast = FALSE;

	if (usetoast)
	{
		if (!FindToaster()){
			printf("Toaster is not running!  Please start the Toaster and run this test again!\n");
			if (FlyerBase)
				CloseLibrary(FlyerBase);
			exit(20);
		}
	}

	if (NULL == (IntuitionBase = OpenLibrary("intuition.library", 37L)))
		ErrorMessage("Requires Version 37 intuition library");
	else
	{
		if (NULL == (GfxBase = OpenLibrary("graphics.library",37L)))
			ErrorMessage("Requires Version 37 graphics library");
		else
		{
			if (GadToolsBase = OpenLibrary("gadtools.library",39L))
				ad->ad_GadLib = 39;
			else
				if (GadToolsBase = OpenLibrary("gadtools.library", 37L))
				{
					ad->ad_GadLib = 37;
				}
				else
					ErrorMessage("Requires Version 37 gadtools library");
			if (ad->ad_GadLib > 0)
			{
				if (NULL == (FlyerBase = OpenLibrary("flyer.library",0))) 
				{
					ErrorMessage("Open flyer.library failed");
				}
				else
				{
					DSWindow();
					CloseLibrary(FlyerBase);
				}
				CloseLibrary(GadToolsBase);
			}		
			CloseLibrary(GfxBase);
		}
		CloseLibrary(IntuitionBase);
	}
}

/******************************************************\
*
*	FUNCTION:
*		void InitAppData( void )
*
*	SYNOPSIS:
*		Allocates and Initializes all members of the AppData struct, ad.
*
*	INPUT:
*		none
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void InitAppData( void )
{
	ad = AllocMem(sizeof(struct AppData), MEMF_CLEAR);
	ad->ad_GadLib = 0;
	ad->ad_LengthLabels = LengthLabels2;
	ad->ad_LengthIndex = EN_REPEAT;
	ad->ad_SpeedIndex = 3;
	ad->ad_do_ss = 0;
	ad->ad_videoflag = TRUE;
	ad->ad_fmt_ss_block = 0L;
	ad->ad_ss_val = 0.0;
	ad->ad_DriveList = NULL;
	ad->ad_CacheList = NULL;
	ad->ad_ManufList = NULL;
	ad->ad_autosave = FALSE;
	ad->ad_changedconfig = FALSE;
	/* if this is changed, make sure to update the menu... */
	ad->ad_longlog = FALSE;
	clear(ad->ad_SpeedStr, 16);
	clear(ad->ad_DriveStr, 80);
	clear(ad->ad_VolNameStr, 128);
	clear(ad->ad_SetCacheLabelStr, 64);
	sprintf(ad->ad_PercentStr, "0.0");
	clear(ad->ad_StatusStr, 64); 
	clear(ad->ad_ss_str, 16);
	ad->ad_Tester[0] = '\0';

	sprintf(ad->ad_LogFile, "FlyerHDTools.log");
	sprintf(ad->ad_LogDir, "ram:");
	sprintf(ad->ad_DataFile, "FlyerHDTools.dpf");
	sprintf(ad->ad_DataDir, "");
	sprintf(ad->ad_CfgFile, "FlyerHDTools.dpf");
	sprintf(ad->ad_CfgDir, "");
}

/******************************************************\
*
*	FUNCTION:
*		void DSWindow( void )
*
*	SYNOPSIS:
*		Creates main window and attaches gadgets
*
*	INPUT:
*		none
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void DSWindow( void )
{
	struct TextFont	*font;
	struct Screen		*scr;
	struct Window		*win;
	struct Gadget		*glist, *gadlist[MAX_GADS + 1];
	struct Menu *menuStrip;
	void 					*vi;
	int i;
	
	if (NULL == (font = OpenFont(&Topaz80)))
		ErrorMessage("Failed to Open Topaz 80");
	else
	{
		if (NULL == (scr = LockPubScreen(NULL)))
			ErrorMessage("Could not lock public screen");
		else
		{
			if (NULL ==(vi = GetVisualInfo(scr, TAG_END)))
				ErrorMessage("GetVisualInfo() failed");
			else
			{
				topborder = scr->WBorTop + (scr->Font->ta_YSize + 1);
				for(i=0;i <= MAX_GADS;i++)
					gadlist[i] = NULL;
				if (NULL == (CreateMainGadgets(&glist, vi, gadlist)))
					ErrorMessage("CreateMainGadgets() Failed");
				else
				{
					if (NULL ==(win = OpenWindowTags(NULL,
											WA_Title, 			FHDTVERSION,
											WA_Gadgets,			glist,
											WA_Left,				35,
											WA_Top,				13,
											WA_Width,			570,
											WA_Height,			192 + scr->Font->ta_YSize,
											WA_DragBar,			TRUE,
											WA_DepthGadget,	TRUE,
											WA_Activate,		TRUE,
											WA_CloseGadget,	TRUE,
											WA_SimpleRefresh,	TRUE,
											WA_IDCMP,			IDCMP_CLOSEWINDOW |
																	IDCMP_REFRESHWINDOW |
																	IDCMP_MENUPICK |
																	LISTVIEWIDCMP,
											WA_PubScreen,		scr,
											TAG_END)))
						ErrorMessage("OpenWindow() Failed");
					else
					{
						if (NULL == (menuStrip = CreateMenus(DSMenu, TAG_END)))
						{
							ErrorMessage("CreateMenus Failed!");
						}
						else
						{
							if (!LayoutMenus(menuStrip, vi, TAG_END))
							{
								ErrorMessage("LayoutMenus Failed!");
							}
							else
							{
								if (!SetMenuStrip(win, menuStrip))
								{
									ErrorMessage("SettMenuStrip Failed!");
								}
								else
								{
									ad->ad_StatusWin = win;
									ad->ad_StatusGadList = gadlist;

									GT_RefreshWindow(win, NULL);
									SetWait(win);
									GetDriveList(win, gadlist);
									ad->ad_SelDriveIndex = 0;
									StatusMessage("Loading Drive Info...");
									InitManufList();
									if (AddDriveData(win, ad->ad_CfgDir, ad->ad_CfgFile))
										StatusMessage("Unable to locate Config file!");
									else
										StatusMessage("");
									ClearWait(win);
									GT_SetGadgetAttrs(gadlist[DSGAD_TESTER], win, NULL,
																GTST_String,	ad->ad_Tester,
																TAG_END);
									ProcessWindowEvents(win, gadlist, menuStrip); 
									cleanup();
									ClearMenuStrip(win);
									CloseWindow(win);
								}
								FreeMenus(menuStrip);
							}
						}
					}
				}
				FreeGadgets(glist);
				FreeVisualInfo(vi);
			}
			UnlockPubScreen(NULL, scr);
		}
		CloseFont(font);
	}
}

/******************************************************\
*
*	FUNCTION:
*		struct Gadget *CreateMainGadgets(struct Gadget **glistptr, 
*													void *vi,
*													struct Gadget *gadlist[])
*
*	SYNOPSIS:
*		Creates all gadgets for the main window
*
*	INPUT:
*		glistptr - pointer to the gadget list to create
*		vi - pointer to the visualinfo struct
*		gadlist - pointer to the gadget list to fill in
*
*	RETURNS:
*		a pointer to the last gadget created, NULL if there was an error
*	
*	NEEDS:
*
\******************************************************/

struct Gadget *CreateMainGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[])
{
	struct NewGadget ng;
	struct Gadget *gad;

	gad = CreateContext(glistptr);

	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 4 + topborder;
	ng.ng_Width = 542;
	ng.ng_Height = 53;
	ng.ng_GadgetText = "";
	ng.ng_TextAttr = &Topaz80;
	ng.ng_VisualInfo = vi;
	ng.ng_GadgetID = DSGAD_DRIVELIST;
	ng.ng_Flags = PLACETEXT_ABOVE;
	if (ad->ad_GadLib == 39)
		gadlist[DSGAD_DRIVELIST] = gad = CreateGadget(LISTVIEW_KIND, gad, &ng,
										GTLV_ShowSelected,	NULL,
										TAG_END);
	else
		gadlist[DSGAD_DRIVELIST] = gad = CreateGadget(LISTVIEW_KIND, gad, &ng,
										TAG_END);

	ng.ng_TopEdge = topborder + 55;
	ng.ng_Height = 14;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_SELDRIVE;
	gadlist[DSGAD_SELDRIVE] = gad = CreateGadget(TEXT_KIND, gad, &ng,
									GTTX_Border, TRUE,
									TAG_END);	

	ng.ng_LeftEdge = 113;
	ng.ng_TopEdge = 114 + topborder;
	ng.ng_Width = 118;
	ng.ng_Height = 14;
	ng.ng_GadgetText = "Test Type :";
	ng.ng_GadgetID = DSGAD_TESTTYPE;
	ng.ng_Flags = PLACETEXT_LEFT;
	ad->ad_do_read = 1;
	ad->ad_do_write = 1;
	gadlist[DSGAD_TESTTYPE] = gad = CreateGadget(CYCLE_KIND, gad, &ng, 
									GTCY_Labels,	TypeLabels,
									GTCY_Active,	2,
									TAG_END);

	ng.ng_TopEdge = 128 + topborder;
	ng.ng_GadgetText = "Duration  :";
	ng.ng_GadgetID = DSGAD_LENGTH;
	
	gadlist[DSGAD_LENGTH] = gad = CreateGadget(CYCLE_KIND, gad, &ng, 
									GTCY_Labels,	ad->ad_LengthLabels,
									GTCY_Active,	ad->ad_LengthIndex,
									TAG_END);

	ng.ng_TopEdge = 142 + topborder;
	ng.ng_GadgetText = "Target Spd:";
	ng.ng_GadgetID = DSGAD_TARGETSPD;
	gadlist[DSGAD_TARGETSPD] = gad = CreateGadget(CYCLE_KIND, gad, &ng, 
									GTCY_Labels,	SpeedLabels,
									GTCY_Active,	ad->ad_SpeedIndex,
									TAG_END);

	ng.ng_LeftEdge = 232;
	ng.ng_TopEdge = 128 + topborder;
	ng.ng_Width = 56;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_LENGTHMIN;
	if ((ad->ad_LengthIndex == EN_LENGTH) || (ad->ad_LengthIndex == EN_REPEAT))
	{
		gadlist[DSGAD_LENGTHMIN] = gad = CreateGadget(INTEGER_KIND, gad, &ng, 
										STRINGA_Justification,	GACT_STRINGRIGHT,
										GA_Disabled,				(ad->ad_LengthIndex < EN_LENGTH),
										GTIN_Number,				LengthVals[ad->ad_LengthIndex],
										TAG_END);
	}
	else
	{
		gadlist[DSGAD_LENGTHMIN] = gad = CreateGadget(INTEGER_KIND, gad, &ng, 
										STRINGA_Justification,	GACT_STRINGRIGHT,
										GA_Disabled,				(ad->ad_LengthIndex < EN_LENGTH),
										GTIN_Number,				0,
										TAG_END);
	}

	ng.ng_TopEdge = 142 + topborder;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_TARGETNUM;
	sprintf(ad->ad_SpeedStr,"%.2f", SpeedVals[ad->ad_SpeedIndex]);
	gadlist[DSGAD_TARGETNUM] = gad = CreateGadget(STRING_KIND, gad, &ng, 
//									STRINGA_Justification,	GACT_STRINGRIGHT,
									GTST_String,				ad->ad_SpeedStr,
									GA_Disabled,				(ad->ad_SpeedIndex != 4),
									TAG_END);

	ng.ng_LeftEdge = 296;
	ng.ng_TopEdge = 102 + topborder;
	ng.ng_Width = 169;
	ng.ng_Height = 10;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_RESLABLOW;
	gadlist[DSGAD_RESLABLOW] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	"Lowest Spd  (MB/s):",
									TAG_END);
	ng.ng_TopEdge = 113 + topborder;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_RESLABAVG;
	gadlist[DSGAD_RESLABAVG] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	"Average Spd (MB/s):",
									TAG_END);
	ng.ng_TopEdge = 124 + topborder;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_RESLABIDL;
	gadlist[DSGAD_RESLABIDL] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	"Max Idle Time (ms):",
									TAG_END);


	sprintf(ad->ad_ResList[0],"%5.2f", 0.0);
	sprintf(ad->ad_ResList[1],"%5.2f", 0.0);
	sprintf(ad->ad_ResList[2],"%5.2f", 0.0);
	sprintf(ad->ad_ResList[3],"%5.2f", 0.0);
	sprintf(ad->ad_ResList[4],"%5ld", 0L);
	sprintf(ad->ad_ResList[5],"%5ld", 0L);
	ng.ng_LeftEdge = 453;
	ng.ng_TopEdge = 102 + topborder;
	ng.ng_Width = 51;
	ng.ng_Height = 10;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_RESLISTLR;
	gadlist[DSGAD_RESLISTLR] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ad->ad_ResList[0],
									TAG_END);

	ng.ng_TopEdge = 113 + topborder;
	ng.ng_GadgetID = DSGAD_RESLISTAR;
	gadlist[DSGAD_RESLISTAR] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ad->ad_ResList[2],
									TAG_END);

	ng.ng_TopEdge = 124 + topborder;
	ng.ng_GadgetID = DSGAD_RESLISTIR;
	gadlist[DSGAD_RESLISTIR] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ad->ad_ResList[4],
									TAG_END);

	ng.ng_LeftEdge = 504;
	ng.ng_TopEdge = 102 + topborder;
	ng.ng_GadgetID = DSGAD_RESLISTLW;
	gadlist[DSGAD_RESLISTLW] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ad->ad_ResList[1],
									TAG_END);

	ng.ng_TopEdge = 113 + topborder;
	ng.ng_GadgetID = DSGAD_RESLISTAW;
	gadlist[DSGAD_RESLISTAW] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ad->ad_ResList[3],
									TAG_END);

	ng.ng_TopEdge = 124 + topborder;
	ng.ng_GadgetID = DSGAD_RESLISTIW;
	gadlist[DSGAD_RESLISTIW] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ad->ad_ResList[5],
									TAG_END);

	ng.ng_LeftEdge = 451;
	ng.ng_TopEdge = 101 + topborder;
	ng.ng_Width = 106;
	ng.ng_Height = 34;
	ng.ng_GadgetText = "Read / Write";
	ng.ng_GadgetID = DSGAD_RESBOX;
	ng.ng_Flags = PLACETEXT_ABOVE;
	gadlist[DSGAD_RESBOX] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	"",
									GTTX_Border,TRUE,
									TAG_END);

	ng.ng_LeftEdge = 382;
	ng.ng_TopEdge = 72 + topborder;
	ng.ng_Width = 175;
	ng.ng_Height = 12;
	ng.ng_GadgetText = "View Log...";
	ng.ng_GadgetID = DSGAD_VIEWLOG;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_VIEWLOG] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									GA_Disabled,	TRUE,
									TAG_END);

	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 173 + topborder;
	ng.ng_GadgetText = "Cache Options...";
	ng.ng_GadgetID = DSGAD_GOCACHE;
	gadlist[DSGAD_GOCACHE] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 198;
	ng.ng_TopEdge = 173 + topborder;
	ng.ng_GadgetText = "Format Options...";
	ng.ng_GadgetID = DSGAD_GOFORMAT;
	gadlist[DSGAD_GOFORMAT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 505;
	ng.ng_TopEdge = 142 + topborder;
	ng.ng_Width = 52;
	ng.ng_Height = 13;
	ng.ng_GadgetText = "% of Drive Over Target:";
	ng.ng_GadgetID = DSGAD_PERCENT;
	ng.ng_Flags = PLACETEXT_LEFT;
	gadlist[DSGAD_PERCENT] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,		ad->ad_PercentStr,
									GTTX_Border,	TRUE,
									TAG_END);

	ng.ng_LeftEdge = 74;
	ng.ng_TopEdge = 158 + topborder;
	ng.ng_Width = 483;
	ng.ng_Height = 13;
	ng.ng_GadgetText = "Status:";
	ng.ng_GadgetID = DSGAD_STATUS;
	gadlist[DSGAD_STATUS] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	"",
									GTTX_Border,	TRUE,
									TAG_END);

	ng.ng_LeftEdge = 198;
	ng.ng_TopEdge = 72 + topborder;
	ng.ng_Width = 175;
	ng.ng_Height = 12;
	ng.ng_GadgetText = "Start Test";
	ng.ng_GadgetID = DSGAD_START;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_START] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 14;
	ng.ng_GadgetText = "Rescan Drives";
	ng.ng_GadgetID = DSGAD_RESCAN;
	gadlist[DSGAD_RESCAN] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 382;
	ng.ng_TopEdge = 173 + topborder;
	ng.ng_GadgetText = "QUIT";
	ng.ng_GadgetID = DSGAD_EXIT;
	gadlist[DSGAD_EXIT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 121;
	ng.ng_TopEdge = 90 + topborder;
	ng.ng_Width = 98;
	ng.ng_Height = 10;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_TESTOPTLABEL;
	gadlist[DSGAD_TESTOPTLABEL] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	"Test Options",
									TAG_END);

	ng.ng_LeftEdge = 113;
	ng.ng_TopEdge = 100 + topborder;
	ng.ng_Width = 175;
	ng.ng_Height = 14;
	ng.ng_GadgetText = "Tester    :";
	ng.ng_Flags = PLACETEXT_LEFT;
	ng.ng_GadgetID = DSGAD_TESTER;
	gadlist[DSGAD_TESTER] = gad = CreateGadget(STRING_KIND, gad, &ng, 
									GTST_String,				ad->ad_Tester,
									TAG_END);

	return(gad);
}

/******************************************************\
*
*	FUNCTION:
*		BOOL HandleGadgetEvent( struct Window *win, 
*										struct Gadget *gad, 
*										UWORD code,
*										struct Gadget *gadlist[], 
*										UWORD menuID)
*
*	SYNOPSIS:
*		Handles all gadget events
*
*	INPUT:
*		win - pointer to the window to handle events for
*		gad - pointer to the gadget that was activated
*		code - the code field from the message that the gadget sent
*		gadlist - pointer to the gadget list containing the gadgets to process
*		menuID - ID of the menu that was selected, if any
*
*	RETURNS:
*		TRUE if a user's action indicates that they want the program to close
*		FALSE if not
*
*	NEEDS:
*
\******************************************************/

BOOL HandleGadgetEvent(struct Window *win, struct Gadget *gad, UWORD code,
								struct Gadget *gadlist[], UWORD menuID)
{
	BOOL terminated = FALSE;
	UWORD ID;

	if (gad == NULL)
		ID = menuID;
	else
		ID = gad->GadgetID;

	switch(ID)
	{
		case DSGAD_NOOP:
			break;
		case DSGAD_DRIVELIST:
			ad->ad_SelDriveIndex = code + 1;  
			if (ad->ad_DriveInfo[code + 1] == NULL)
				ad->ad_SelDriveIndex = 0;  //Indicates no drive selected
				
			sprintf(ad->ad_DriveStr, "%-.79s", ad->ad_DriveInfo[ad->ad_SelDriveIndex]->str); 

			GT_SetGadgetAttrs(gadlist[DSGAD_SELDRIVE], win, NULL,
									GTTX_Text,	ad->ad_DriveStr,
									TAG_END);
			if (ad->ad_DriveInfo[ad->ad_SelDriveIndex]->Log == NULL)
			{
				GT_SetGadgetAttrs(gadlist[DSGAD_VIEWLOG], win, NULL,
										GA_Disabled,	TRUE,
										TAG_END);
			}
			else
			{
				GT_SetGadgetAttrs(gadlist[DSGAD_VIEWLOG], win, NULL,
										GA_Disabled,	FALSE,
										TAG_END);
			}
				
			GT_BeginRefresh(win);
			GT_EndRefresh(win, TRUE);

			break;
		case DSGAD_MANUFLIST:
			{
				struct Manuf *tempman;
				
				ad->ad_SelCacheIndex = code + 1;  
				
				tempman = FindIndex(ad->ad_ManufList, ad->ad_SelCacheIndex);
				if (tempman != NULL)
					sprintf(ad->ad_CacheStr, "%-.79s", tempman->str); 
				else
					ad->ad_CacheStr[0]= '\0'; 
				GT_SetGadgetAttrs(gadlist[DSGAD_SELCACHE], win, NULL,
										GTTX_Text,	ad->ad_CacheStr,
										TAG_END);
				GT_BeginRefresh(win);
				GT_EndRefresh(win, TRUE);
			}
			break;
		case DSGAD_TESTTYPE:
			switch(code)
			{
				case (0):
					ad->ad_do_read  = 1;
					ad->ad_do_write = 0;
					ad->ad_LengthLabels = LengthLabels1;
					break;
				case (1):
					ad->ad_do_read  = 0;
					ad->ad_do_write = 1;
					ad->ad_LengthLabels = LengthLabels1;
					break;
				case (2):
					ad->ad_do_read  = 1;
					ad->ad_do_write = 1;
					ad->ad_LengthLabels = LengthLabels2;
					break;
			}
			GT_SetGadgetAttrs(gadlist[DSGAD_TESTTYPE], win, NULL,
										GTCY_Active,	code,
										TAG_END);
			GT_SetGadgetAttrs(gadlist[DSGAD_LENGTH], win, NULL,
										GTCY_Labels,	ad->ad_LengthLabels,
										TAG_END);
			break;
		case DSGAD_LENGTH:
			ad->ad_LengthIndex = code;

			if ((ad->ad_LengthIndex == EN_LENGTH) || (ad->ad_LengthIndex == EN_REPEAT))
			{
				GT_SetGadgetAttrs(gadlist[DSGAD_LENGTHMIN], win, NULL,
											GTIN_Number,	LengthVals[code],
											GA_Disabled,	FALSE,
											TAG_END);
			}
			else
			{
				GT_SetGadgetAttrs(gadlist[DSGAD_LENGTHMIN], win, NULL,
											GA_Disabled,	TRUE,
											GTIN_Number,	0,
											TAG_END);
			}
			GT_SetGadgetAttrs(gadlist[DSGAD_LENGTH], win, NULL,
											GTCY_Active,	code,
											TAG_END);
			break;
		case DSGAD_TARGETSPD:
			sprintf(ad->ad_SpeedStr, "%.2f", SpeedVals[code]);
			GT_SetGadgetAttrs(gadlist[DSGAD_TARGETNUM], win, NULL,
										GTST_String,	ad->ad_SpeedStr,
										TAG_END);
			if ((ad->ad_SpeedIndex = code) == EN_SPEED)
				GT_SetGadgetAttrs(gadlist[DSGAD_TARGETNUM], win, NULL,
											GA_Disabled,	FALSE,
											TAG_END);
			else
				GT_SetGadgetAttrs(gadlist[DSGAD_TARGETNUM], win, NULL,
											GA_Disabled,	TRUE,
											TAG_END);
			GT_SetGadgetAttrs(gadlist[DSGAD_TARGETSPD], win, NULL,
											GTCY_Active,	code,
											TAG_END);
			break;
		case DSGAD_LENGTHMIN:
			if (ad->ad_LengthIndex == EN_LENGTH)
				LengthVals[EN_LENGTH] = (ULONG)
					((struct StringInfo *)(gadlist[DSGAD_LENGTHMIN]->SpecialInfo))->LongInt;
			else
				LengthVals[EN_REPEAT] = (ULONG)
					((struct StringInfo *)(gadlist[DSGAD_LENGTHMIN]->SpecialInfo))->LongInt;
			break;
		case DSGAD_TARGETNUM:
			SpeedVals[EN_SPEED] = Str2Float(((struct StringInfo *)
										(gadlist[DSGAD_TARGETNUM]->SpecialInfo))->Buffer);

			while(SpeedVals[EN_SPEED] > 100.0)
				SpeedVals[EN_SPEED] = 100.0;

			sprintf(ad->ad_SpeedStr, "%.2f", SpeedVals[EN_SPEED]);
			GT_SetGadgetAttrs(gadlist[DSGAD_TARGETNUM], win, NULL,
											GTST_String,	ad->ad_SpeedStr,
											TAG_END);
			break;
		case DSGAD_VIEWCACHE:
			if ( (ad->ad_SelDriveIndex != 0) && ( ad->ad_DriveInfo[ad->ad_SelDriveIndex]->size != 0 ) )
			{
				FetchCacheSettings(ad->ad_DriveInfo[ad->ad_SelDriveIndex]);
				CreateCacheList(&(ad->ad_DriveInfo[ad->ad_SelDriveIndex]->Manuf), &(ad->ad_CacheList));
				ViewCache(ad->ad_CacheList);
				PurgeMsgPort(win);
				StatusMessage("");
			}
			else
			{
				StatusMessage("Error: No Valid Drive Selected!");
			}
			break;
		case DSGAD_GOFORMAT:
			if ( (ad->ad_SelDriveIndex != 0) && ( ad->ad_DriveInfo[ad->ad_SelDriveIndex]->size != 0 ) )
			{
				// This could be made much smarter about only
				// updating the drive that was selected when entering the
				// panel, so that a full scan of the bus wouldn't be needed
				FormatWindow(ad->ad_DriveInfo[ad->ad_SelDriveIndex]);
				StatusMessage("Rescanning drives...");
				SetWait(win);
				ad->ad_SelDriveIndex = 0;
				sprintf(ad->ad_DriveStr, "");
				GT_SetGadgetAttrs(gadlist[DSGAD_SELDRIVE], win, NULL,
												GTTX_Text,	ad->ad_DriveStr,
												TAG_END);	
				GT_SetGadgetAttrs(gadlist[DSGAD_VIEWLOG], win, NULL,
												GA_Disabled,	TRUE,
												TAG_END);	
				GT_BeginRefresh(win);
				GT_EndRefresh(win, TRUE);
				UpdateDriveList(win, gadlist);
				PurgeMsgPort(win);
				StatusMessage("");
				ClearWait(win);
			}
			else
			{
				StatusMessage("Error: No Valid Drive Selected!");
			}
			break;
		case DSGAD_START:
			if ( (ad->ad_SelDriveIndex != 0) && ( ad->ad_DriveInfo[ad->ad_SelDriveIndex]->size != 0 ) )
			{
				sprintf(ad->ad_Tester,"%-.127s",(char *)((struct StringInfo *)
										(gadlist[DSGAD_TESTER]->SpecialInfo))->Buffer);
				while(ad->ad_Tester[0] == '\0')
				{
					AskTester();
					GT_SetGadgetAttrs(gadlist[DSGAD_TESTER], win, NULL,
													GTST_String,	ad->ad_Tester,
													TAG_END);
					PurgeMsgPort(win);
				}
				SetWait(win);
				if (StartTest(win, gadlist, ad->ad_DriveInfo[ad->ad_SelDriveIndex]))
				{
					StatusMessage("Test Complete!");
					UpdateSummaryWindow(win, gadlist);			
					UpdateShortStroke(win, gadlist, ad->ad_DriveInfo[ad->ad_SelDriveIndex]);
					CreateLog(ad->ad_DriveInfo[ad->ad_SelDriveIndex]);
					if (ad->ad_autosave)
						WriteListFile( ad->ad_DriveInfo[ad->ad_SelDriveIndex]->Log, 
											ad->ad_LogDir, ad->ad_LogFile);
					GT_SetGadgetAttrs(gadlist[DSGAD_VIEWLOG], win, NULL,
											GA_Disabled,	FALSE,
											TAG_END);
					
				}
				PurgeMsgPort(win);
				ClearWait(win);
				StatusMessage("");
			}
			else
				StatusMessage("Error: No Valid Drive Selected!");
			break;
		case DSGAD_VIEWLOG:
			if (ad->ad_SelDriveIndex)
			{
				ViewLog(ad->ad_DriveInfo[ad->ad_SelDriveIndex]->Log);
				PurgeMsgPort(win);
			}
			break;
		case DSGAD_SAVELOG:
			if (ad->ad_SelDriveIndex)
			{
				SaveLog(ad->ad_DriveInfo[ad->ad_SelDriveIndex]->Log);
				PurgeMsgPort(win);
			}
			break;
		case DSGAD_AUTOSAVE:
			ad->ad_autosave = (!(ad->ad_autosave));
			if (ad->ad_autosave)
			{
				ReqFile("Save FlyerHDTools Log As...", ad->ad_LogDir, ad->ad_LogFile);
				PurgeMsgPort(win);
			}
			break;
		case DSGAD_RESCAN:
			StatusMessage("Rescanning drives...");
			SetWait(win);
			GetDriveList(win, gadlist);
			ad->ad_SelDriveIndex = 0;

			sprintf(ad->ad_DriveStr, "");
			GT_SetGadgetAttrs(gadlist[DSGAD_VIEWLOG], win, NULL,
									GA_Disabled,	TRUE,
									TAG_END);
			GT_SetGadgetAttrs(gadlist[DSGAD_SELDRIVE], win, NULL,
											GTTX_Text,	ad->ad_DriveStr,
											TAG_END);
			StatusMessage("");
			ClearWait(win);
			break;
		case DSGAD_EXIT:
			if ((ad->ad_changedconfig == TRUE) && (win == ad->ad_StatusWin))
			{
				if(Warning(win, "Your configuration has changed.\nWould you like to save your configuration file now?"))
				{
					if (WriteDriveData(ad->ad_CfgDir, ad->ad_CfgFile, NULL))
						if (win == ad->ad_StatusWin)
							StatusMessage("Unable to save configuration file!");
						else
							ErrorReq(win, "Unable to save configuration file!");
					else
						if (win == ad->ad_StatusWin)
							StatusMessage("Saved configuration file!");
				}
			}
			PurgeMsgPort(win);
			terminated = TRUE;
			break;
		case DSGAD_LOGLIST:
			break;
		case DSGAD_LOGSAVE:
			SaveLog(ad->ad_DriveInfo[ad->ad_SelDriveIndex]->Log);
			break;
		case DSGAD_LOGCONT:
			terminated = TRUE;
			break;

		case DSGAD_VOLNAME:
			sprintf(ad->ad_VolNameStr,"%s",(char *)((struct StringInfo *)
											(gadlist[DSGAD_VOLNAME]->SpecialInfo))->Buffer);
			ValidVolName(ad->ad_VolNameStr);
			GT_SetGadgetAttrs(gadlist[DSGAD_VOLNAME], win, NULL,
											GTST_String,	ad->ad_VolNameStr,
											TAG_END);
			break;
		case DSGAD_AVFLAGS:
			ad->ad_videoflag = (!(ad->ad_videoflag));
			break;
		case DSGAD_DOSHORT:
			ad->ad_do_ss = code;
			switch(ad->ad_do_ss){
				case 0:
					// Update values in case user left DSGAD_SSVAL w/o hitting enter
					ad->ad_fmt_ss_block = Hex2ULONG((char *)((struct StringInfo *)
										(gadlist[DSGAD_SSVAL]->SpecialInfo))->Buffer);
					sprintf(ad->ad_ss_str, "0x%lx", ad->ad_fmt_ss_block);
					GT_SetGadgetAttrs(gadlist[DSGAD_SSVAL], win, NULL,
								GA_Disabled,	TRUE,
								GTST_String,	ad->ad_ss_str,
								TAG_END);
					break;
				case 1:
					sprintf(ad->ad_ss_str, "%.4f%% ", ad->ad_ss_val);
					GT_SetGadgetAttrs(gadlist[DSGAD_SSVAL], win, NULL,
								GA_Disabled,	FALSE,
								GTST_String,	ad->ad_ss_str,
								TAG_END);
					break;
				case 2:
					ad->ad_ss_val = Str2Float((char *)((struct StringInfo *)
										(gadlist[DSGAD_SSVAL]->SpecialInfo))->Buffer);
					if (ad->ad_ss_val > 100.0)
						ad->ad_ss_val = 100.0;
					sprintf(ad->ad_ss_str, "0x%lx", ad->ad_fmt_ss_block);
					GT_SetGadgetAttrs(gadlist[DSGAD_SSVAL], win, NULL,
								GTST_String,	ad->ad_ss_str,
								TAG_END);
					break;
			}
			break;
		case DSGAD_SSVAL:
			switch(ad->ad_do_ss){
				case 1:
					ad->ad_ss_val = Str2Float((char *)((struct StringInfo *)
										(gadlist[DSGAD_SSVAL]->SpecialInfo))->Buffer);
					if (ad->ad_ss_val > 100.0)
						ad->ad_ss_val = 100.0;
					sprintf(ad->ad_ss_str, "%.4f%% ", ad->ad_ss_val);
					break;
				case 2:
					ad->ad_fmt_ss_block = Hex2ULONG((char *)((struct StringInfo *)
										(gadlist[DSGAD_SSVAL]->SpecialInfo))->Buffer);
					sprintf(ad->ad_ss_str, "0x%lx", ad->ad_fmt_ss_block);
					break;
			}

			GT_SetGadgetAttrs(gadlist[DSGAD_SSVAL], win, NULL,
											GTST_String,	ad->ad_ss_str,
											TAG_END);
			break;
		case DSGAD_DOFORMAT:
			SetWait(win);
			sprintf(ad->ad_VolNameStr,"%s",(char *)((struct StringInfo *)
											(gadlist[DSGAD_VOLNAME]->SpecialInfo))->Buffer);
			ValidVolName(ad->ad_VolNameStr);
			GT_SetGadgetAttrs(gadlist[DSGAD_VOLNAME], win, NULL,
											GTST_String,	ad->ad_VolNameStr,
											TAG_END);
			if ((VerifyFormat(win)) && (ad->ad_SelDriveIndex))
			{
				FormatDrive(ad->ad_DriveInfo[ad->ad_SelDriveIndex]);
				CopyFlyerIcon(ad->ad_DriveInfo[ad->ad_SelDriveIndex]->FlyVol.SCSIdrive);
				UpdateFormatWindow(win, gadlist, ad->ad_DriveInfo[ad->ad_SelDriveIndex], ad->ad_FormatBuffer);
			}
			PurgeMsgPort(win);
			ClearWait(win);
			break;
		case DSGAD_GOCACHE:
			SetWait(win);
			if ( (ad->ad_SelDriveIndex != 0) && ( ad->ad_DriveInfo[ad->ad_SelDriveIndex]->size != 0 ) )
			{
				struct Manuf *man;
				
				FetchCacheSettings(ad->ad_DriveInfo[ad->ad_SelDriveIndex]);
				if (!(man = ParamsExist(ad->ad_DriveInfo[ad->ad_SelDriveIndex])))
				{
					if (Warning(win, 
						"The selected drive is not currently using any\nof the known cache settings.\nWould you like to add the drive's current\nsettings to the list of known settings?"))
					{
						ReadDriveCache(ad->ad_DriveInfo[ad->ad_SelDriveIndex]);
						ad->ad_changedconfig = TRUE;
						if (man = ParamsExist(ad->ad_DriveInfo[ad->ad_SelDriveIndex]))
						{
							ad->ad_SelCacheIndex = man->index;
						}
						else
						{
							ad->ad_SelCacheIndex = 0;
						}
					}
					else
					{
						ad->ad_SelCacheIndex = 0;
					}
				}
				else
				{
					ad->ad_SelCacheIndex = man->index;
				}
				SortList(ad->ad_ManufList);
				IndexList(ad->ad_ManufList);
				SetCacheWindow();
			}
			else
			{
				StatusMessage("Error: No Valid Drive Selected!");
			}
			PurgeMsgPort(win);
			ClearWait(win);
			break;
		case DSGAD_READDATA:
			SetWait(win);
			if (ReqFile("Select Drive Data File...", ad->ad_DataDir, ad->ad_DataFile))
			{
				if (gadlist[DSGAD_MANUFLIST] != NULL)
					GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
													GTLV_Labels,	~0,
													TAG_END);
					
				if (AddDriveData(win, ad->ad_DataDir, ad->ad_DataFile))
					if (gadlist[DSGAD_MANUFLIST] == NULL)
						StatusMessage("Unable to locate drive data file!");
					else
						ErrorReq(win, "Unable to locate drive data file!");
				else
				{
					StatusMessage("Drive Data Loaded!");
					ad->ad_changedconfig = TRUE;
				}

				ad->ad_SelCacheIndex = 0;
				ad->ad_CacheStr[0]= '\0'; 
				GT_SetGadgetAttrs(gadlist[DSGAD_SELCACHE], win, NULL,
										GTTX_Text,	ad->ad_CacheStr,
										TAG_END);
				SortList((struct List *)ad->ad_ManufList);
				IndexList(ad->ad_ManufList);
				if (gadlist[DSGAD_MANUFLIST] != NULL)
					GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
													GTLV_Selected,	~0,
													GTLV_Labels,	ad->ad_ManufList,
													TAG_END);
			}
			PurgeMsgPort(win);
			ClearWait(win);
			break;
		case DSGAD_COPYCACHE:
			SetWait(win);
			if (ad->ad_SelCacheIndex != 0)
			{
				struct Manuf *man;
			
				GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
												GTLV_Labels,	~0,
												TAG_END);
				man = FindIndex(ad->ad_ManufList, ad->ad_SelCacheIndex); 
				if (man != NULL)
				{
					CopyManufNode(man);
					ad->ad_changedconfig = TRUE;
					ad->ad_SelCacheIndex = 0;
					ad->ad_CacheStr[0]= '\0'; 
					GT_SetGadgetAttrs(gadlist[DSGAD_SELCACHE], win, NULL,
											GTTX_Text,	ad->ad_CacheStr,
											TAG_END);
					SortList((struct List *)ad->ad_ManufList);
					IndexList(ad->ad_ManufList);
				}
				GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
												GTLV_Selected, ~0,
												GTLV_Labels,	ad->ad_ManufList,
												TAG_END);
			}
			PurgeMsgPort(win);
			ClearWait(win);
			break;
		case DSGAD_DELCACHE:
			SetWait(win);
			{
				UBYTE reply;
				
				if (ad->ad_SelCacheIndex != 0)
				{
					if ((reply = Warning(win, "Are you sure that you want to\ndelete these settings?")) == 1)
					{
						struct Manuf *man;
				
						GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
														GTLV_Labels,	~0,
														TAG_END);
						man = FindIndex(ad->ad_ManufList, ad->ad_SelCacheIndex); 
						if (man != NULL)
						{
							DeleteManufNode(man);
							ad->ad_changedconfig = TRUE;
							ad->ad_SelCacheIndex = 0;
							ad->ad_CacheStr[0]= '\0'; 
							GT_SetGadgetAttrs(gadlist[DSGAD_SELCACHE], win, NULL,
													GTTX_Text,	ad->ad_CacheStr,
													TAG_END);
							SortList((struct List *)ad->ad_ManufList);
							IndexList(ad->ad_ManufList);
						}
						GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
														GTLV_Selected,	~0,
														GTLV_Labels,	ad->ad_ManufList,
														TAG_END);
					}
				}
				PurgeMsgPort(win);
			}
			ClearWait(win);
			break;
		case DSGAD_SAVEDATA:
			SetWait(win);
			{
				UBYTE reply;
				FILE *exists;
				char filename[192];
				
				sprintf(filename, "%s", ad->ad_CfgDir);
				AddPart(filename, ad->ad_CfgFile, 191);
				
				exists = fopen(filename, "r");
				if (exists)
				{
					reply = Warning(win, "Are you sure that you want to replace\nyour existing configuration file?");
					fclose(exists);
				}
				else
					reply = 1;

				if (reply == 1)
				{
					if (WriteDriveData(ad->ad_CfgDir, ad->ad_CfgFile, NULL))
						if (win == ad->ad_StatusWin)
							StatusMessage("Unable to save configuration file!");
						else
							ErrorReq(win, "Unable to save configuration file!");
					else
						if (win == ad->ad_StatusWin)
							StatusMessage("Saved configuration file!");
				}
				PurgeMsgPort(win);
			}
			ClearWait(win);
			break;
		case DSGAD_WRITECACHE:
			SetWait(win);
			if (ad->ad_SelCacheIndex != NULL)
			{
				struct Manuf *man;
				man = FindIndex(ad->ad_ManufList, ad->ad_SelCacheIndex);

				if (man != NULL)
				{
					if (ReqFile("Save Drive Data File as...", ad->ad_DataDir,ad->ad_DataFile))
					{
						if (WriteDriveData(ad->ad_DataDir, ad->ad_DataFile, man))
						{
							if (win == ad->ad_StatusWin)
								StatusMessage("Unable to save Data file!");
							else
								ErrorReq(win, "Unable to save Data file!");
						}
						else
						{
							if (win == ad->ad_StatusWin)
								StatusMessage("Saved Data file!");
						}
					}
				}
			}
			PurgeMsgPort(win);
			ClearWait(win);
			break;
		case DSGAD_READCACHE:
			SetWait(win);
			if (ad->ad_SelDriveIndex != 0)
			{
				if (gadlist[DSGAD_MANUFLIST] != NULL)
					GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
													GTLV_Labels,	~0,
													TAG_END);
				ReadDriveCache(ad->ad_DriveInfo[ad->ad_SelDriveIndex]);
				ad->ad_changedconfig = TRUE;
				ad->ad_SelCacheIndex = 0;
				ad->ad_CacheStr[0]= '\0'; 
				GT_SetGadgetAttrs(gadlist[DSGAD_SELCACHE], win, NULL,
										GTTX_Text,	ad->ad_CacheStr,
										TAG_END);
				SortList((struct List *)ad->ad_ManufList);
				IndexList(ad->ad_ManufList);
				if (gadlist[DSGAD_MANUFLIST] != NULL)
					GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
													GTLV_Selected,	~0,
													GTLV_Labels,	ad->ad_ManufList,
													TAG_END);
			}
			else
				StatusMessage("Error: No Drive Selected");

			PurgeMsgPort(win);
			ClearWait(win);
			break;
		case DSGAD_EDITCOMMENT:
			if (ad->ad_SelCacheIndex != 0)
			{
				struct Manuf *man;
				
				GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
												GTLV_Labels,	~0,
												TAG_END);
				man = FindIndex(ad->ad_ManufList, ad->ad_SelCacheIndex);
				if (man != NULL)
				{
					AskComment(man);
					sprintf(man->str, "%s%s%s %s",man->manufname,
															man->modelname,
															man->revision,
															man->comment);
					ad->ad_changedconfig = TRUE;
					GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
													GTLV_Labels,	ad->ad_ManufList,
													TAG_END);
				}
			}
			break;
		case DSGAD_SETCACHE:
			if (ad->ad_SelCacheIndex != NULL)
			{
				struct Manuf *man;
				SetWait(win);
				man = FindIndex(ad->ad_ManufList, ad->ad_SelCacheIndex);
				if (man != NULL)
				{
					if (stricmp(man->manufname, ad->ad_DriveInfo[ad->ad_SelDriveIndex]->Manuf.manufname) || 
							stricmp(man->modelname, ad->ad_DriveInfo[ad->ad_SelDriveIndex]->Manuf.modelname) ||
						 	stricmp(man->revision,  ad->ad_DriveInfo[ad->ad_SelDriveIndex]->Manuf.revision)) 
					{
						ErrorReq(win, "These Cache Parameters are not\nvalid for the selected drive!");
					}
					else
					{
						SetCache(ad->ad_DriveInfo[ad->ad_SelDriveIndex], man->options);
					}
				}
				ClearWait(win);
			}
			break;
		case DSGAD_VIEWSELCACHE:
			if (ad->ad_SelCacheIndex != 0)
			{
				struct Manuf *man;
				
				man = FindIndex(ad->ad_ManufList, ad->ad_SelCacheIndex);
				if (man != NULL)
				{
					CreateCacheList(man, &(ad->ad_CacheList));
					ViewCache(ad->ad_CacheList);
				}
				PurgeMsgPort(win);
				StatusMessage("");
			}
			break;
		case DSGAD_LONGLOG:
			ad->ad_longlog = !(ad->ad_longlog);
			break;
		case DSGAD_COMMCONT:
		case DSGAD_COMMENT:
			sprintf(ad->ad_tempstr,"%-.127s",(char *)((struct StringInfo *)
											(gadlist[DSGAD_COMMENT]->SpecialInfo))->Buffer);
			GT_SetGadgetAttrs(gadlist[DSGAD_COMMENT], win, NULL,
											GTST_String,	ad->ad_tempstr,
											TAG_END);
			terminated = TRUE;
			break;
		case DSGAD_TESTERCONT:
		case DSGAD_TESTERNAME:
			sprintf(ad->ad_Tester,"%-.127s",(char *)((struct StringInfo *)
											(gadlist[DSGAD_TESTERNAME]->SpecialInfo))->Buffer);
			GT_SetGadgetAttrs(gadlist[DSGAD_TESTERNAME], win, NULL,
											GTST_String,	ad->ad_Tester,
											TAG_END);
			terminated = TRUE;
			break;
		case DSGAD_TESTER:
			sprintf(ad->ad_Tester,"%-.127s",(char *)((struct StringInfo *)
											(gadlist[DSGAD_TESTER]->SpecialInfo))->Buffer);
			while(ad->ad_Tester[0] == '\0')
				AskTester();

			GT_SetGadgetAttrs(gadlist[DSGAD_TESTER], win, NULL,
											GTST_String,	ad->ad_Tester,
											TAG_END);
			break;

	}
	return(terminated);
}

/******************************************************\
*
*	FUNCTION:
*		void ProcessWindowEvents(  struct Window *win, 
*											struct Gadget *gadlist[], 
*											struct Menu *menuStrip)
*
*	SYNOPSIS:
*		Handles all messages for gadgets and menus 
*
*	INPUT:
*		win - pointer to window to handle messages for
*		gadlist - pointer to list of gadgets to handle events for
*		menuStrip - pointer to the menu to process if available, NULL if none
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void ProcessWindowEvents(struct Window *win, struct Gadget *gadlist[], struct Menu *menuStrip)
{
	struct IntuiMessage *imsg;
	ULONG imsgClass;
	UWORD imsgCode, menuID;
	struct Gadget *gad;
	UWORD menuNumber;
	struct MenuItem *item; 
	BOOL terminated = FALSE;

	while (!terminated)
	{
		Wait (1<<win->UserPort->mp_SigBit);
		while ((!terminated) && (imsg = GT_GetIMsg(win->UserPort)))
		{
			gad = (struct Gadget *)imsg->IAddress;

			imsgClass = imsg->Class;
			imsgCode = imsg->Code;

			GT_ReplyIMsg(imsg);

			switch(imsgClass)
			{
				case IDCMP_MENUPICK:
					menuNumber = imsgCode;
					while ((menuNumber != MENUNULL) && (!terminated))
					{
						item = ItemAddress(menuStrip, menuNumber);
						menuID = Menu2ID(MENUNUM(menuNumber), ITEMNUM(menuNumber), SUBNUM(menuNumber), &imsgCode);
						terminated = HandleGadgetEvent(win, NULL, imsgCode, gadlist, menuID);
						menuNumber = item->NextSelect;
					}
					break;
				case IDCMP_GADGETDOWN:
				case IDCMP_GADGETUP:
				case IDCMP_MOUSEMOVE:
					terminated = HandleGadgetEvent(win, gad, imsgCode, gadlist, NULL);
					break;
				case IDCMP_CLOSEWINDOW:
					terminated = TRUE;
					break;
				case IDCMP_REFRESHWINDOW:
					GT_BeginRefresh(win);
					GT_EndRefresh(win, TRUE);
					break;
			}
		}
	}
}

/******************************************************\
*
*	FUNCTION:
*		void UpdateDriveList(struct Window *win, struct Gadget **gadlist)
*
*	SYNOPSIS:
*		Scans Flyer SCSI buses and adds any drives it finds to 
*		the DriveList as well as the ad_DriveInfo array, but preserves
*		all logs associated with drives and re-attaches them to the drive
*		after update.
*
*	INPUT:
*		win - pointer to the window containing the DriveList gadget
*		gadlist - pointer to the gadget list containing the DriveList gadget
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void UpdateDriveList(struct Window *win, struct Gadget **gadlist)
{
	struct List *LogList[24];
	int i;

	for(i=0; i<24 ; i++)
	{
		if (ad->ad_DriveInfo[i])
		{
			LogList[i] = ad->ad_DriveInfo[i]->Log;
			ad->ad_DriveInfo[i]->Log = NULL;
		}	
	}
	GetDriveList(win, gadlist);
	for(i=0; i<24 ; i++)
		if (ad->ad_DriveInfo[i])
			if (ad->ad_DriveInfo[i]->Log == NULL)
				ad->ad_DriveInfo[i]->Log = LogList[i];
}

/******************************************************\
*
*	FUNCTION:
*		void GetDriveList(struct Window *win, struct Gadget **gadlist)
*
*	SYNOPSIS:
*		Scans Flyer SCSI buses and adds any drives it finds to 
*		the DriveList as well as the ad_DriveInfo array
*
*	INPUT:
*		win - pointer to the window containing the DriveList gadget
*		gadlist - pointer to the gadget list containing the DriveList gadget
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void GetDriveList(struct Window *win, struct Gadget **gadlist)
{
	UBYTE i,j, index;
	struct FlyerVolume FlyVol;
	UBYTE MiscBuff[18], drives;
	ULONG res;

	FlyVol.Path = NULL;
	FlyVol.Board = 0;
	FlyVol.Flags = FVF_USENUMS;
	ad->ad_SelDriveIndex = 0;

	for (i=0;i<24;i++)
		ad->ad_DriveInfo[i] = NULL;
	GT_SetGadgetAttrs(gadlist[DSGAD_DRIVELIST], win, NULL,
															GTLV_Labels,		NULL,
															TAG_END);
	FreeDList(ad->ad_DriveList);
	if (!(ad->ad_DriveList = AllocMem(sizeof(struct List), MEMF_CLEAR)))
	{
		StatusMessage("Out of Memory!");
	}
	else
	{
		NewList(ad->ad_DriveList);
		StatusMessage("Finding Flyer Drives...");
		for(i=0,index = 1;i<=2;i++)
		{
			FlyVol.SCSIdrive = (i<<3);
			res = SCSIinit(&FlyVol);
			if (res) 
			{
				StatusMessage("Error: Could not initialize SCSI channel ");
			}
			res = FindDrives(&FlyVol,MiscBuff);
			if (res) 
			{
				StatusMessage("Error: Could Not Find Drives");
			}

			drives = MiscBuff[0];	/* Bit flags for each ID on bus (1=drive present) */
		
			for(j=0;j<=6;j++)
			{
				if ((1<<j) & drives)
				{
					AddDListNode(ad->ad_DriveList, index);
					ad->ad_DriveInfo[index]->FlyVol.SCSIdrive = (i<<3) + j;
					ad->ad_DriveInfo[index]->FlyVol.Board = 0;
					ad->ad_DriveInfo[index]->FlyVol.Flags = FVF_USENUMS;
					GetDriveInfo(ad->ad_DriveInfo[index++]);
				}
			}
		}
		if (index == 1)
		{
			AddDListNode(ad->ad_DriveList,0);
			sprintf(ad->ad_DriveInfo[ad->ad_SelDriveIndex]->str, 
						"<No Flyer Drives Present>");
		}
		GT_SetGadgetAttrs(gadlist[DSGAD_DRIVELIST], win, NULL,
																GTLV_Labels, ad->ad_DriveList,
																TAG_END);		
	}
}

/******************************************************\
*
*	FUNCTION:
*		void AddDListNode(struct List *list, int index)
*
*	SYNOPSIS:
*		adds a new DLNode to list str, and copies the pointer 
*		to the new DLNode to ad->ad_DriveInfo[index]
*
*	INPUT:
*		list - pointer to the list to add the new node to
*		index - index of the ad->ad_DriveInfo array to place the address
*					of the new node into.
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void AddDListNode(struct List *list, int index)
{
	struct DLNode *Dnode;

	if (!(Dnode = AllocMem(sizeof(struct DLNode), MEMF_CLEAR)))
		StatusMessage("Out Of Memory!");
	else
	{
		Dnode->node.ln_Name = Dnode->str;
		Dnode->node.ln_Pri = 0;
		Dnode->node.ln_Type = 100;
		Dnode->Log = NULL;
		Dnode->CacheList = NULL;
		AddTail(list, (struct Node *)Dnode);
		if ((index >= 0) && (index < 24))
			ad->ad_DriveInfo[index] = Dnode;
	}
}

/******************************************************\
*
*	FUNCTION:
*		void AddTextNode( struct List *list, char *str )
*
*	SYNOPSIS:
*		Adds a TextNode containing the string str to List list
*
*	INPUT:
*		list - Pointer to the list to add the new node to the tail of
*		str - pointer to the string to copy to the new node
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void AddTextNode( struct List *list, char *str )
{
	struct TextNode *newNode;

	if (!(newNode = AllocMem(sizeof(struct TextNode), MEMF_CLEAR)))
		StatusMessage("Out Of Memory!");
	else
	{
		sprintf(newNode->str, "%s", str);
		newNode->node.ln_Name = newNode->str;
		newNode->node.ln_Pri = 0;
		newNode->node.ln_Type = 100;

		AddTail(list, (struct Node *)newNode);
	}
}

/******************************************************\
*
*	FUNCTION:
*		void FreeDList( struct List *list )
*
*	SYNOPSIS:
*		Frees the given DLNode list and all of its nodes
*
*	INPUT:
*		list - pointer to the list to be freed
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void FreeDList( struct List *list )
{
	struct DLNode *worknode;
	struct Options *opt;

	if (list)
	{
		while(worknode = (struct DLNode *)(RemHead(list)))		
		{
			while(worknode->Manuf.options)
			{
				opt = worknode->Manuf.options->next;
				FreeMem(worknode->Manuf.options, sizeof(struct Options));
				worknode->Manuf.options = opt;
			}
			FreeTextList(worknode->Log);
			FreeTextList(worknode->CacheList);

			FreeMem(worknode, sizeof(struct DLNode));
		}
		FreeMem(list, sizeof(struct List));
	}
}

/******************************************************\
*
*	FUNCTION:
*		void FreeTextList( struct List *list )
*
*	SYNOPSIS:
*		Frees the given TextNode list and all of its nodes
*
*	INPUT:
*		list - pointer to list to be freed
*	
*	RETURNS:
*		none
*
*	NEEDS:
*	
\******************************************************/

void FreeTextList( struct List *list )
{
	struct TextNode *worknode;

	if (list)
	{
		while(worknode = (struct TextNode *)(RemHead(list)))
			FreeMem(worknode, sizeof(struct TextNode));
		FreeMem(list, sizeof(struct List));
	}
}

/******************************************************\
*
*	FUNCTION:
*		void GetDriveInfo( struct DLNode *dln )
*
*	SYNOPSIS:
*		Fills in the DLNode struct, dln, with information 
*		for the drive.
*
*	INPUT:
*		dln - pointer to the DLNode struct for the drive, with
*				at least the dln->FlyVol.Board and dln->FlyVol.SCSIdrive
*				members are set
*
*	RETURNS:
*		none
*
*	NEEDS:
*		
\******************************************************/

void GetDriveInfo(struct DLNode *dln)
{
	ULONG res;
	char ss[255];
	UBYTE InqBuff[255];
	struct FlyerVolInfo info;

	dln->FlyVol.Path = NULL;
	dln->FlyVol.Flags = FVF_USENUMS;

	/* Get inquiry data from SCSI drive */
	res = Inquiry(&(dln->FlyVol),36,InqBuff);
	if (res) 
	{
		StatusMessage("Inquiry Error. Please make sure that Flyer has started");
		dln->str[0] = '\0';
		return;
	}

	/* Get FlyerVolInfo from drive... */
	info.len = sizeof(struct FlyerVolInfo);
	res = FlyerDriveInfo(&(dln->FlyVol),&info);
	if (res)
	{
		StatusMessage("Error: could not get Flyer Drive information");
		sprintf(dln->str,"%lc %1ld <Drive Information Unavailable>", 
					('A' + (dln->FlyVol.SCSIdrive >> 3)),
					(dln->FlyVol.SCSIdrive % 8));
		dln->size = 0;
		return;
	}
	
	/* Get drive size info... */	
	res = ReadSize(&(dln->FlyVol), &(dln->size), &(dln->blklen));
 	if (res)
	{
		sprintf(ss, "ReadSize Error: %ld - %ls", res, Error2String(res));
		StatusMessage(ss);
		sprintf(dln->str,"%lc %1ld <Drive Information Unavailable>", 
					('A' + (dln->FlyVol.SCSIdrive >> 3)),
					(dln->FlyVol.SCSIdrive % 8));
		dln->size = 0;
		return;
	}

	if (!(InqBuff[0] & (UBYTE)0x1F)) /* Make sure that the device is a hard-drive */
	{
		dln->ss_block = info.Blocks;
		GetManuf(dln);
		dln->Log = NULL;
		//string should be 66 chars long
		if ((info.Ident == 0x524f4f54) && (info.Version == 1))
		{
			sprintf(dln->str,"%lc %1ld %-.23s %.4s %-19.19s %6.1fGB %lc%lc", 
						('A' + (dln->FlyVol.SCSIdrive >> 3)),
						(dln->FlyVol.SCSIdrive % 8),
						&(InqBuff[8]),
						&(InqBuff[32]),
						(info.Title ? info.Title:" "),
						((double)(info.Blocks) * (double)(info.BlkSize)) / 1000000000.0,
						((info.Flags & FVIF_AUDIOREADY) ? 'A':' '),
						((info.Flags & FVIF_VIDEOREADY) ? 'V':' '));
		}
		else
		{
			sprintf(dln->str,"%lc %1ld %-.23s %.4s %-19.19s %6.1fGB %lc%lc", 
						('A' + (dln->FlyVol.SCSIdrive >> 3)),
						(dln->FlyVol.SCSIdrive % 8),
						&(InqBuff[8]),
						&(InqBuff[32]),
						" ",
						((double)(dln->size) * 512.0) / 1000000000.0,
						' ',
						' ');
		}
	}
	else
	{
		sprintf(dln->str,"%lc %1ld <Drive Information Unavailable>", 
					('A' + (dln->FlyVol.SCSIdrive >> 3)),
					(dln->FlyVol.SCSIdrive % 8));
		dln->size = 0;
	}
}


/******************************************************\
*
*	FUNCTION:
*		int FindToaster( void )
*
*	SYNOPSIS:
*		checks to see if the Video Toaster is running
*
*	INPUT:
*		none
*
*	RETURNS:
*		1 if the Toaster is running
*		0 if the Toaster is not running
*
*	NEEDS:
*
\******************************************************/

int FindToaster( void )
{
	struct MsgPort *port;
	port = FindPort("ToasterARexx.port");

	if (port)
		return(1);
	else
		return(0);
}

/******************************************************\
*
*	FUNCTION:
*		ULONG Str2ULONG(char *str)
*
*	SYNOPSIS:
*		extracts a value from a string containing a number
*		in decimal format, skipping any leading spaces and
*		ending when it encounters the end of the string or
*		any invalid characters
*
*	INPUT:
*		str - string containing an ascii representation of 
*				a decimal number
*
*	RETURNS:
*		the ULONG representation of the number
*
*	NEEDS:
*
\******************************************************/

ULONG Str2ULONG(char *str)
{
	ULONG num = 0L;
	int i;
	
	while (str[0] == 32)
		str++;

	for (i=0;str[i] != '\0';i++)
	{
		if ((str[i] >= '0') && (str[i] <= '9'))
			num = (num * 10) + (str[i] - '0');
		else
			break;
	}
	
	return(num);
}

/******************************************************\
*
*	FUNCTION:
*		ULONG Hex2ULONG(char *str)
*
*	SYNOPSIS:
*		extracts a value from a string containing a number
*		in hexadecimal format, skipping any invalid characters
*
*	INPUT:
*		str - string containing an ascii representation of 
*				a hexadecimal number, with or without a leading 0x
*
*	RETURNS:
*		the ULONG representation of the number
*
*	NEEDS:
*
\******************************************************/

ULONG Hex2ULONG(char *str)
{
	ULONG num = 0L;
	int i;
	
	for (i=0;str[i] != '\0';i++)
	{
		if ((str[i] >= '0') && (str[i] <= '9'))
			num = (num * 16) + (str[i] - '0');
		if ((str[i] >= 'a') && (str[i] <= 'f'))
			num = (num * 16) + (str[i] - 'a') + 10L;
		if ((str[i] >= 'A') && (str[i] <= 'F'))
			num = (num * 16) + (str[i] - 'A') + 10L;
	}
	
	return(num);
}

/******************************************************\
*
*	FUNCTION:
*		float Str2Float(char *str)
*
*	SYNOPSIS:
*		Extracts a floating point number from a string, 
*		skipping leading spaces and stopping when it hits
*		the end of the string or invalid character.
*
*	INPUT:
*		str - string containing the number to extract
*
*	RETURNS:
*		the floating point version of the decimal number
*		contained in the string.
*
*	NEEDS:
*
\******************************************************/

float Str2Float(char *str)
{
	float num;
	int i, sigdigs, point;
	float power;
	
	num = 0;
	sigdigs = 0;
	point = 0;
	power = 10.0;
	
	while(str[0] == 32)
		str++;

	for(i=0; ((str[i] != '\0') && (sigdigs < 6));i++)
	{
		if ((str[i] >= '0') && (str[i] <= '9'))
		{
			if (point == 0)
				num = (num * 10.0) + (float)(str[i] - '0');
			else
			{
				num += ((float)(str[i] - '0') / power);
				power *= 10.0;
			}
			sigdigs++;
		}
		else
			if (str[i] == '.')
				point = 1;
			else
				break;
	}
	return(num);
}

/******************************************************\
*
*	FUNCTION:
*		void ErrorMessage( char *msg )
*
*	SYNOPSIS:
*		Prints the given string to stdout as an error message
*
*	INPUT:
*		msg - string to print as an error
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void ErrorMessage(char *msg)
{
	if (msg)
		printf("Error: %s\n", msg);
}

/******************************************************\
*
*	FUNCTION:
*		void StatusMessage( char *msg )
*
*	SYNOPSIS:
*		Displays the string, msg, on the status bar of the main window
* 
*	INPUT:
*		msg - string to print in status bar
*
*	RETURNS:
*		none
*
*	NEEDS:
*		ad->ad_StatusWin must be a pointer to the main window
*			containing the status gadget
*		ad->ad_StatusGadList must point to the gadget list 
*			containing the status gadget
*
\******************************************************/

void StatusMessage( char *msg )
{
	strncpy(ad->ad_StatusStr, msg, 63);

	GT_SetGadgetAttrs(ad->ad_StatusGadList[DSGAD_STATUS], ad->ad_StatusWin, NULL,
													GTTX_Text,	ad->ad_StatusStr,
													TAG_END);
}

/******************************************************\
*
*	FUNCTION:
*		void cleanup( void )
*
*	SYNOPSIS:
*		Removes any lists and linked structures.  May be called
*		at anytime within the program to exit cleanly.
*
*	INPUT:
*		none
*		
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void cleanup( void ) 
{
	struct LBAStats *temp;
	struct Manuf *tempman;
	struct Options *tempopt;

	/* remove TestStats */
	while (read_stats.first != NULL)
	{
		temp = read_stats.first->next;
		FreeMem(read_stats.first, sizeof(struct LBAStats));
		read_stats.first = temp;
	}	
	while (write_stats.first != NULL)
	{
		temp = write_stats.first->next;
		FreeMem(write_stats.first, sizeof(struct LBAStats));
		write_stats.first = temp;
	}	
	
	if (ad->ad_ManufList != NULL)
	{
		while (ad->ad_ManufList->lh_TailPred != (struct Node *)ad->ad_ManufList)
		{
			tempman = (struct Manuf *)RemHead(ad->ad_ManufList);
			while (tempman->options != NULL)
			{
				tempopt = tempman->options->next;
				FreeMem(tempman->options, sizeof(struct Options));
				tempman->options = tempopt;
			}
			FreeMem(tempman, sizeof(struct Manuf));
		}
		FreeMem(ad->ad_ManufList, sizeof(struct List));
	}

	FreeDList(ad->ad_DriveList);
	FreeTextList(ad->ad_CacheList);
}

/******************************************************\
*
*	FUNCTION:
*		void InitStats( void )
*
*	SYNOPSIS:
*		Initializes read_stats and write_stats, freeing any
*		LBAStats that might be attached to them
*
*	INPUT:
*		none
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void InitStats(void)
{
	struct LBAStats *current;

	/* remove TestStats */
	while (read_stats.first != NULL){
		current = read_stats.first->next;
		FreeMem(read_stats.first, sizeof(struct LBAStats));
		read_stats.first = current;
	}	
	while (write_stats.first != NULL){
		current = write_stats.first->next;
		FreeMem(write_stats.first, sizeof(struct LBAStats));
		write_stats.first = current;
	}	

	read_stats.first = NULL;
	read_stats.last = NULL;
	read_stats.num_tests = 0;
	read_stats.lowest = NULL;
	read_stats.average = 0.0;
	read_stats.max_drop = NULL;

	write_stats.first = NULL;
	write_stats.last = NULL;
	write_stats.num_tests = 0;
	write_stats.lowest = NULL;
	write_stats.average = 0.0;
	write_stats.max_drop = NULL;
}

/******************************************************\
*
*	FUNCTION:
*		BOOL StartTest(struct Window *win, 
*							struct Gadget *gadlist[], 
*							struct DLNode *dln)
*
*	SYNOPSIS:
*		Performs a test on the selected drive with user specified
*		options.
*	
*	INPUT:
*		win - pointer to the parent window for the abort requester
*				as well as the window containing all of the test
*				options gadgets
*		gadlist - pointer to the Gadget list containing all of 
*					 the test options gadgets
*		dln - pointer to the filled DLNode for the drive to test
*
*	RETURNS:
*		TRUE if the test was performed successfully
*		FALSE if there was an error or the test was aborted by the user
*
*	NEEDS:
*		LengthVals[] must contain correct values
*		ad->ad_LengthIndex must contain the index to the current choice
*		SpeedVals[] must contain correct values
*		ad->ad_SpeedIndex must contain the index to the current choice
*		LengthLabels1 and LengthLabels2 must contain valid arrays of labels
*		topborder must hold the height of the main windows title bar
*		ad->ad_do_read and ad->ad_do_write must be set or cleared as needed
*		
\******************************************************/

BOOL StartTest(struct Window *win, struct Gadget *gadlist[], struct DLNode *dln)
{
	int first, writeflag, minutes, repeat, iteration, loop_cnt;
	struct FlyerVolInfo Buffer;
	UBYTE formatted = 0, reply, InqBuff[255];
	ULONG res, lba, max, num_tests, test_cnt, block_inc;
	struct LBAStats *current;
	char str[128];
	LONG abort = -2L;
	struct Window *ReqWin;
	struct EasyStruct ESabort = 
		{
		sizeof(struct EasyStruct), 0, NULL,
		"Performing Test...\n\nPress Left Amiga-b to Abort.",
		"ABORT",
		};

	InitStats();

// These should already be set...
//	dln->FlyVol.Board = 0;
//	dln->FlyVol.SCSIdrive = ad->ad_drive;
	dln->FlyVol.Flags = FVF_USENUMS;

	// Verify that the user's last entry in the length and speed fields
	// are used, since that may not be the case in the custom value option...
	if ((ad->ad_LengthIndex == EN_LENGTH) || (ad->ad_LengthIndex == EN_REPEAT))
		LengthVals[ad->ad_LengthIndex] = (ULONG)
			((struct StringInfo *)(gadlist[DSGAD_LENGTHMIN]->SpecialInfo))->LongInt;

	if (ad->ad_SpeedIndex == EN_SPEED)
	{
		SpeedVals[EN_SPEED] = Str2Float(((struct StringInfo *)
									(gadlist[DSGAD_TARGETNUM]->SpecialInfo))->Buffer);
		sprintf(ad->ad_SpeedStr, "%.2f", SpeedVals[EN_SPEED]);
		// Update the gadget to display the correct value...
		GT_SetGadgetAttrs(gadlist[DSGAD_TARGETNUM], win, NULL,
										GTST_String,	ad->ad_SpeedStr,
										TAG_END);
	}
	
	/* Get inquiry data from SCSI drive */
	res = Inquiry(&(dln->FlyVol),36,InqBuff);
	if (res) 
	{
		sprintf(str,"Inquiry Error %ld - %ls  Test Aborted!",res,Error2String(res));
		StatusMessage(str);
		return(FALSE);	
	}
	
	res = SCSIinit(&(dln->FlyVol)); //  flush SCSI bus 
	if (res)
	{
		sprintf(str,"SCSIinit Error %ld - %ls Test Aborted!",res,Error2String(res));
		StatusMessage(str);
		return(FALSE);	
	}

	/* Get inquiry data from SCSI drive */
	/* This command is necessary because some Flyer commands fail 
		if called following a SCSIinit...                           */
	res = Inquiry(&(dln->FlyVol),36,InqBuff);
	if (res) 
	{
		sprintf(str,"Inquiry Error %ld - %ls Test Aborted!",res,Error2String(res));
		StatusMessage(str);
		return(FALSE);	
	}

	//  CHECK DRIVE TO SEE IF FORMATTED 
	Buffer.len = sizeof(struct FlyerVolInfo);
	formatted = 0;
	res = FlyerDriveInfo(&(dln->FlyVol),&Buffer);
	if (!res && (Buffer.Ident == 0x524f4f54) && (Buffer.Version == 1))
		formatted = 1;

	if (ad->ad_do_write)
	{
		reply = WriteWarning(win); 
		if (reply == 2)  //  Chose to do a read test only...
		{
			ad->ad_do_read = 1;
			ad->ad_do_write = 0;
			GT_SetGadgetAttrs(gadlist[DSGAD_TESTTYPE], win, NULL,
											GTCY_Active,	0,
											TAG_END);
			ad->ad_LengthLabels = LengthLabels1;
			GT_SetGadgetAttrs(gadlist[DSGAD_LENGTH], win, NULL,
										GTCY_Labels,	ad->ad_LengthLabels,
										TAG_END);
		}
		else
			if (reply == 0)
			{
				StatusMessage("Test Aborted!");
				return(FALSE);
			}
	}
	else
		reply = 1;

	if (!(InqBuff[0] & (UBYTE)0x1F)) /* Make sure that the device is a hard-drive */
	{
		/* Get drive specs: # blocks, block size */
		res = ReadSize(&(dln->FlyVol),&max,&(dln->blklen));
		if (res) 
		{
			sprintf(str,"ReadSize Error %ld - %ls Aborting Test!",res,Error2String(res));
			StatusMessage(str);
			return(FALSE);
		}

		dln->size = max;
		max -= (TOTALSIZE + 1);  /* Ensure Quantum drives don't hang at end of test... */

		if (ad->ad_LengthIndex == EN_REPEAT)
		{
			minutes = 0;
			repeat  = (int)(LengthVals[ad->ad_LengthIndex]);
		}
		else 
			if (ad->ad_LengthIndex == 0)
			{
				minutes = 0;
				repeat = 1;
			}
			else
				{
					minutes = (int)(LengthVals[ad->ad_LengthIndex]);
					repeat = 0;
//					num_tests = ((minutes * 60 * speed) / (10 * TOTALSIZE * ad->ad_blklen));
					num_tests = (((ULONG)minutes * 48000) / TOTALSIZE);   //Trust me...(4.0 MB/s)
					if (num_tests < (max / (10 * TOTALSIZE)))
						block_inc = ((max - (10 * TOTALSIZE)) / num_tests) - (10 * TOTALSIZE);
					else
						block_inc = 0;
				}
	
		if (minutes > 0)
		{
			Init_Monitor( &(ad->ad_mon), win, 76, 159 + topborder, 476, 10,
									num_tests * 10 * (ad->ad_do_read + ad->ad_do_write));
		}
		else
		{
			Init_Monitor( &(ad->ad_mon), win, 76, 159 + topborder, 476, 10,
									max / TOTALSIZE * repeat * (ad->ad_do_read + ad->ad_do_write));
		}

		RecordMode(0);  //ensure that the Flyer is in the proper mode

		for (writeflag=0;writeflag <= 1;writeflag++, abort != 0) 
		{
			current = NULL;
			iteration = 0;
			test_cnt = 0;

			if((ad->ad_do_read && (!writeflag)) || (ad->ad_do_write && writeflag)) 
			{
				first = 1;
				loop_cnt = 0;

				//  START TEST 

				ReqWin = BuildEasyRequest(win, &ESabort, NULL);

				for (lba=0;
					  (abort = SysReqHandler(ReqWin, NULL, FALSE)) < 0;
					  (lba += TOTALSIZE)) 
				{ 
					/* Prevent last test from going past end of drive */
					if ((lba+TOTALSIZE) >= max) 
					{
						lba = 0;
						if (repeat > 0)
						{
							iteration++;
							if (iteration >= repeat)
								break;
						}
					}	
					//  CREATE NEW LIST ENTRY 
					if (first) 
						first = 0;
					else
					{
						if (writeflag)
							current = NewStat(current,&write_stats,lba,
											DoTest(lba,TESTBLKS,REPEAT, writeflag, dln));
						else
							current = NewStat(current,&read_stats,lba,
											DoTest(lba,TESTBLKS,REPEAT, writeflag, dln));
						if (current == NULL)
							break;
					}

					Inc_Monitor(&(ad->ad_mon));

					if (minutes > 0)
					{
						loop_cnt++;
						if (loop_cnt >= 10)
						{
							test_cnt++;
							if (test_cnt >= num_tests)
								break;
							loop_cnt = 0;
							lba += block_inc;
							first = 1;
						}
					}
				}
				FreeSysRequest(ReqWin);
			}
			if (writeflag)
				DoStats(&write_stats);
			else
				DoStats(&read_stats);
		}

		PlayMode(0);  //Reset Flyer into proper mode

		End_Monitor(&(ad->ad_mon));
		if ((ad->ad_do_write) && (formatted))
		AskReformat(win, &Buffer, dln);
	}

	if (abort == 0)
	{
		StatusMessage("Test Aborted!");
		return(FALSE);
	}
	return(TRUE);
}

/******************************************************\
*
*	FUNCTION:
*		UBYTE Warning( struct Window *win , char *msg)
*
*	SYNOPSIS:
*		Displays a requester warning the user of "msg"
*
*	INPUT:
*		win - pointer to the parent window for the requester
*		msg - pointer to the string containing the text for the requester
*
*	RETURNS:
*		0 if the user chooses the NO button
*		1 if the user chooses the YES button
*
*	NEEDS:
*
\******************************************************/

UBYTE Warning(struct Window *win , char *msg)
{
	LONG reply;

	struct EasyStruct ESWrite = 
		{
		sizeof(struct EasyStruct), 0, NULL,
				NULL,
				"YES|NO",
		};
	ESWrite.es_TextFormat = msg;
	reply = EasyRequest(win, &ESWrite, NULL );
	GT_BeginRefresh(win);
	GT_EndRefresh(win, TRUE);
	
	return((UBYTE)reply);
}

/******************************************************\
*
*	FUNCTION:
*		UBYTE WriteWarning( struct Window *win )
*
*	SYNOPSIS:
*		Displays a requester warning the user that performing
*		a write test will cause any data on the drive to be
*		destroyed and allows the user to choose to continue,
*		abort, or choose to do a non-destructive read test only.
*
*	INPUT:
*		win - pointer to the parent window for the requester
*
*	RETURNS:
*		0 if the user chooses the abort button
*		1 if the user chooses the continue button
*		2 if the user chooses the do read only button
*
*	NEEDS:
*
\******************************************************/

UBYTE WriteWarning(struct Window *win )
{
	LONG reply;

	struct EasyStruct ESWrite = 
		{
		sizeof(struct EasyStruct), 0, NULL,
					"Performing a Write Test will destroy\nall data on the selected drive.\nContinue?\n",
					"CONTINUE|Do READ Only|ABORT",
		};
	
	reply = EasyRequest(win, &ESWrite, NULL );
	GT_BeginRefresh(win);
	GT_EndRefresh(win, TRUE);
	
	return((UBYTE)reply);
}

/******************************************************\
*
*	FUNCTION:
*		void AskReformat( struct Window *win,
*								struct FlyerVolInfo *info,
*								struct DLNode *dln)
*
*	SYNOPSIS:
*		Displays a requester to ask the user if they would like to
*		reformat the drive.  If so, it is formatted.
*
*	INPUT:
*		win - pointer to the parent window for the requester
*		info - pointer to a FlyerVolInfo containing information to
*					use when formatting the drive
*		dln - pointer to the DLNode struct for the drive to be formatted
*
*	RETURNS:
*		nothing
*
*	NEEDS:
*
\******************************************************/

void AskReformat( struct Window *win, 
						struct FlyerVolInfo *info, 
						struct DLNode *dln)
{
	ULONG res;
	char str[128];

	struct EasyStruct ESFormat = 
		{
		sizeof(struct EasyStruct), 0, NULL,
		"Would you like to re-format volume\n\"%ls\"\nusing its previous settings?",
		"ReFormat|Don't Format",
		};

	if (EasyRequest(win, &ESFormat, NULL, info->Title ))
	{
		GT_BeginRefresh(win);
		GT_EndRefresh(win, TRUE);
		sprintf(str,"Formatting drive %lc %ld as \"%ls\"...", (dln->FlyVol.SCSIdrive >> 3) + 'A',
						(dln->FlyVol.SCSIdrive & 0x07), info->Title);
		StatusMessage(str);
		res = FlyerFormat(&(dln->FlyVol), info->Title, 
								&(info->DiskDate), info->Blocks, info->Flags);
		if (res)
		{
			sprintf(str,"Error %ld - %ls  Format Incomplete!",res,Error2String(res));
			StatusMessage(str);
		}
		else
		{
			sprintf(str,"Finished formatting \"%ls\".",info->Title);
			StatusMessage(str);
		}
	}
	else
	{
		GT_BeginRefresh(win);
		GT_EndRefresh(win, TRUE);
	}
}

/******************************************************\
*
*	FUNCTION:
*		void UpdateSummaryWindow( struct Window *win,
*										  struct Gadget **gadlist)
*	SYNOPSIS:
*		Updates the summary display on the main screen
*
*	INPUT:
*		win - pointer to the window containing the summary display
*		gadlist - pointer to the Gadget list containing the summary
*						display gadgetry
*
*	RETURNS:
*		none
*
*	NEEDS:
*		
\******************************************************/

void UpdateSummaryWindow(struct Window *win, struct Gadget **gadlist)
{
	if (ad->ad_do_read)
	{
		if (read_stats.lowest)
			sprintf(ad->ad_ResList[0], " %5.2f", read_stats.lowest->speed);
		sprintf(ad->ad_ResList[2], " %5.2f", read_stats.average);
		if (read_stats.max_drop)
			sprintf(ad->ad_ResList[4], " %5ld", read_stats.max_drop->drop_ms);

	}
	else
	{
		sprintf(ad->ad_ResList[0], " %5.2f", 0.0 );
		sprintf(ad->ad_ResList[2], " %5.2f", 0.0 );
		sprintf(ad->ad_ResList[4], " %5ld", 0L );
	}
	if (ad->ad_do_write)
	{
		if (write_stats.lowest)
			sprintf(ad->ad_ResList[1], " %5.2f", write_stats.lowest->speed);
		sprintf(ad->ad_ResList[3], " %5.2f", write_stats.average);
		if (write_stats.max_drop)
			sprintf(ad->ad_ResList[5], " %5ld", write_stats.max_drop->drop_ms);
	}
	else
	{
		sprintf(ad->ad_ResList[1], " %5.2f", 0.0 );
		sprintf(ad->ad_ResList[3], " %5.2f", 0.0 );
		sprintf(ad->ad_ResList[5], " %5ld", 0L );
	}
		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTLR], win, NULL,
										GTTX_Text,	ad->ad_ResList[0],
										TAG_END);
		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTAR], win, NULL,
										GTTX_Text,	ad->ad_ResList[2],
										TAG_END);
		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTIR], win, NULL,
										GTTX_Text,	ad->ad_ResList[4],
										TAG_END);

		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTLW], win, NULL,
										GTTX_Text,	ad->ad_ResList[1],
										TAG_END);
		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTAW], win, NULL,
										GTTX_Text,	ad->ad_ResList[3],
										TAG_END);
		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTIW], win, NULL,
										GTTX_Text,	ad->ad_ResList[5],
										TAG_END);
}

/******************************************************\
*
*	FUNCTION:
*		ULONG DoTest( ULONG lba, 
*						  ULONG testblks, 
*						  ULONG repeat, 
*						  int writeflag, 
*						  struct DLNode *dln)
*
*	SYNOPSIS:
*		performs a single group of read or write tests on the drive
*		referenced by dln.
*
*	INPUT:
*		lba - block to begin testing at
*		testblks - number of blocks to test each time
*		repeat - number of times to repeat the test.
*		writeflag - flag: if 0, performs read test
*								if 1, performs write test
*		dln - pointer to DLNode containing data for drive to test
*
*	RETURNS:
*		number of ticks elapsed during test
*
*	NEEDS:
*
\******************************************************/

ULONG DoTest( ULONG lba,ULONG testblks,ULONG repeat,
				  int writeflag, struct DLNode *dln)
{
	struct	DateStamp	start,end;
	ULONG		ticks,res;
	char 		str[128];

	if (writeflag) {
		Forbid();						
		DateStamp(&start);			/* Log time now */
		res = WriteTest(&(dln->FlyVol),testblks,repeat,lba,1);
		DateStamp(&end);				/* Log time now */
		Permit();
	} else {
		Forbid();	
		DateStamp(&start);			/* Log time now */
		res = ReadTest(&(dln->FlyVol),testblks,repeat,lba,1);
		DateStamp(&end);				/* Log time now */
		Permit(); 
	}
	if (res) {
		sprintf(str,"R/W Error %ld - %ls",res,Error2String(res));
		ErrorMessage(str);
		return(0);
	}
	else {
		ticks = ElapsedTicks(&start,&end);
		return(ticks);
	}
}

/******************************************************\
*
*	FUNCTION:
*		ULONG ElapsedTicks( struct DateStamp *begin,
*								  struct DateStamp *end)
*
*	SYNOPSIS:
*		returns the number of ticks between the datestamps begin and end
*
*	INPUT:
*		begin - pointer to the first datestamp
*		end - pointer to the second datestamp
* 
*	RETURNS:
*		number of ticks between the two datestamps
*
*	NEEDS:
*
\******************************************************/

ULONG ElapsedTicks(struct DateStamp *begin, struct DateStamp *end)
{
	ULONG		ticks;

	ticks = end->ds_Minute * 60 *50 + end->ds_Tick;
	ticks -= begin->ds_Minute * 60*50 + begin->ds_Tick;

	return(ticks);
}

struct LBAStats *NewStat(struct LBAStats *old, struct TestStats *Test, ULONG lba,
								 ULONG ticks)
{
	struct LBAStats *new;
	char	 str[128]; 
	new = (struct LBAStats *)AllocMem(sizeof(struct LBAStats),MEMF_ANY || MEMF_CLEAR);

	if (new) {
		new->lba = lba;
		new->ticks = ticks;
		new->next = NULL;
		if (!(new->prev = old))
			Test->first = new;
		else
			new->prev->next = new;
		Test->last = new;	
	} else {
		sprintf(str,"Sorry..Can't allocate Memory!");
		ErrorMessage(str);
	}
	return(new);	
}

/******************************************************\
*
*	FUNCTION:
*		void DoStats( struct TestStats *test )
*
*	SYNOPSIS:
*		Computes all of the stats in the linked list of stats, test, 
* 		base on the results of the drive test
*
*	INPUT:
*		test - pointer to the linked list of TestStats to compute
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void DoStats(struct TestStats *test)
{
	int i;
	double meg,total_spd;
	struct LBAStats *lowest,*current;
	
	meg = ((double)(TOTALSIZE)* 512.0)/1000000.0;
	
	lowest = test->max_drop = current = test->first;
	total_spd = (double)0.0;

	for(i=0;current != NULL;i++,current = current->next) {
		current->speed = meg/((double)current->ticks/50.0);
		total_spd += current->speed; 
		if (current->speed < lowest->speed)
			lowest = current;
		if (current->prev != NULL) {
			if (current->lba <= current->prev->lba)
				current->drop = 0L;
			else
				current->drop = (long)((long)current->ticks - (long)current->prev->ticks);
		} else 
			current->drop = 0L;	
		current->drop_ms = current->drop * 20L;
	}

	test->num_tests = i;
	test->lowest = lowest;
	test->average = (total_spd / i);
	
	/* Mark all Dips */

	current = test->first;

	while (current != NULL){
		if (current->prev == NULL)
			current->dip = 0;
		else {
			if (current->next == NULL)
				current->dip = 0;
			else {
				if ((current->drop_ms > 40L) && (current->next->drop_ms < -40L)){
					current->dip = 1;
					if (current->drop_ms > test->max_drop->drop_ms)
						test->max_drop = current;
				} else
					current->dip = 0;
			}
		}
		current = current->next;
	}
}

/******************************************************\
*
*	FUNCTION:
*		void UpdateShortStroke( struct Window *win, 
*										struct Gadget **gadlist,
*										struct DLNode *dln)
*	SYNOPSIS:
*		updates the ss_block field of the DLNode and updates the
*		gadgets reporting the short stroke value on the main screen
*
*	INPUT:
*		win - pointer to the window containing the short stroke gadgets
*				to be updated
*		gadlist - pointer to the gadget array containing the short stroke 
*					 gadgets to be updated.
*		dln - pointer to the DLNode to update
*
*	RETURNS:
*
*	NEEDS:
*
\******************************************************/

void UpdateShortStroke(struct Window *win, struct Gadget **gadlist, struct DLNode *dln)
{
	ad->ad_ss_block = ((ad->ad_SpeedIndex > 1) ?  
					FindLastBlockAtSpeed(SpeedVals[ad->ad_SpeedIndex], ad->ad_do_read, ad->ad_do_write) :
					(ULONG) 0);

	sprintf(ad->ad_PercentStr,"%.1f", (((double)ad->ad_ss_block/(double)dln->size) * 100.0));
	GT_SetGadgetAttrs(gadlist[DSGAD_PERCENT], win, NULL,
									GTTX_Text,	ad->ad_PercentStr,
									TAG_END);
}

/******************************************************\
*
*	FUNCTION:
*		ULONG FindLastBlockAtSpeed( double spd, int read, int write )
*
*	SYNOPSIS:
*		Parses the test results and determines the last contiguous
*		block that is able to sustain the target speed, spd, in both
*		reads and writes, and returns that block number.
*
*	INPUT:
*		spd - the target speed to test for
*		read - flag: if 0, then no read test was done, so skip to write
*		write - flag: if 0, then no write test was done, so skip to end
*
*	RETURNS:
*		Block number of last block to sustain spd in reads and/or writes
*
*	NEEDS:
*
\******************************************************/

ULONG	FindLastBlockAtSpeed(double spd, int read, int write )
{
	ULONG readblock,writeblock,block;
	struct LBAStats *current;

	readblock = writeblock = 0L;

	/* TCal Read Stats */	
	if (read){
		for(current = read_stats.first;current != NULL;current = current->next)
		{
			if (current->speed < spd){
				if (current->dip){
					if (current->next != NULL){
						if (current->next->speed >= spd)			 
							readblock = current->lba;
						else
							break;
					} else {
						break;
					}
				} else {
					break;
				}
			} else {
				readblock = current->lba;	
			}
		}
	}

	/* TCal Write Stats */	
	if (write){
		for(current = write_stats.first;current != NULL;current = current->next)
		{
			if (current->speed < spd){
				if (current->dip){
					if (current->next != NULL){
						if (current->next->speed >= spd)			 
							writeblock = current->lba;
						else
							break;
					} else {
						break;
					}
				} else {
					break;
				}
			} else {
				writeblock = current->lba;	
			}
		}
	}

	if (read)
		if (write)
			if (readblock < writeblock)
				block = readblock;
			else
				block = writeblock;
		else
			block = readblock;
	else
		block = writeblock;

	return(block);
}

/******************************************************\
*
*	FUNCTION:
*		void CreateLog( struct DLNode *dln )
*
*	SYNOPSIS:
*		Creates a log list from the cachelist and stats and 
*		attaches the list to the DLNode for that drive
*
*	INPUT:
*		dln - pointer to the DLNode to create the log for
*
*	RETURNS:
*		none
*
*	NEEDS:
*		
\******************************************************/

void CreateLog( struct DLNode *dln )
{
	struct LBAStats *read,*write,*dummy;
	char str[128];
	char Rstr[64];
	char Wstr[64];
	char date[32], time[32]; 
	struct TextNode *current;
	struct DateTime dtnow;

	FetchCacheSettings(dln);
	CreateCacheList(&(dln->Manuf), &(dln->CacheList));

	FreeTextList(dln->Log);

	if (!(dln->Log = AllocMem(sizeof(struct List), MEMF_CLEAR)))
	{
		StatusMessage("Out of Memory!");
	}
	else
	{
		NewList(dln->Log);
		StatusMessage("Building Log...");
	
		if (ad->ad_do_read)
			dummy = read = read_stats.first;
		if (ad->ad_do_write)
			dummy = write = write_stats.first;	

		/* Print header */
		DateStamp(&(dtnow.dat_Stamp));
		dtnow.dat_Format  = FORMAT_DOS;
		dtnow.dat_StrDay = NULL;
		dtnow.dat_StrDate = date;
		dtnow.dat_StrTime = time;
		DateToStr(&dtnow);
		sprintf(str,"Test Performed By: %-.127s", ad->ad_Tester);
		AddTextNode(dln->Log, str);
		sprintf(str,"Test Performed On: %-.32s %-.32s", date, time);
		AddTextNode(dln->Log, str);
		sprintf(str,"Manufacturer/Model: %-.23s", &(dln->str[4]));
		AddTextNode(dln->Log, str);
		sprintf(str,"Revision: %-.4s", &(dln->str[28]));
		AddTextNode(dln->Log, str);
		sprintf(str,"Size: %ld Blocks (%.2f GB)", dln->size, (((double)dln->size)*((double)dln->blklen)/(double)(1000000000.0)));
		AddTextNode(dln->Log, str);
		AddTextNode(dln->Log, "  ");

		if (ad->ad_longlog)
		{		
			sprintf(str,"LBA:       %s%s",(ad->ad_do_read  ? " READ:     READ DROP:   " : ""),
											  (ad->ad_do_write ? " WRITE:    WRITE DROP:"  : ""));
			AddTextNode(dln->Log, str);

			while (dummy != NULL)
			{
				if (ad->ad_do_read)
				{
					if (read != NULL)
					{
						sprintf(Rstr," %5.2f%lc     %4ld ms%lc    ",read->speed, 
									((read->speed == read_stats.lowest->speed)? '*':' '),
									read->drop_ms,
									(((read->drop_ms == read_stats.max_drop->drop_ms) && (read->dip))? '*':' '));
						read = read->next;
					}
					else
					{
						sprintf(Rstr,"                       ");
					}
				}

				if (ad->ad_do_write)
				{
					if (write != NULL)
					{
						sprintf(Wstr," %5.2f%lc     %4ld ms%lc    ",write->speed,
									((write->speed == write_stats.lowest->speed)? '*':' '),
									write->drop_ms,
									(((read->drop_ms == write_stats.max_drop->drop_ms) && (read->dip))? '*':' '));
						write = write->next;
					}
					else
					{
						sprintf(Wstr,"                        ");
					}
				}
				sprintf(str,"%8lx   %s%s",dummy->lba, Rstr, Wstr);
				AddTextNode(dln->Log, str);
				dummy = dummy->next;
			}
			AddTextNode(dln->Log, "  ");
		}
		sprintf(str,"                 %s%s",(ad->ad_do_read  ? "    READ "  : ""),
														(ad->ad_do_write ? "    WRITE " : ""));
		AddTextNode(dln->Log, str);

		sprintf(str,"-----------------%s%s",(ad->ad_do_read  ? "   ------" : ""),
														(ad->ad_do_write ? "   ------" : ""));
		AddTextNode(dln->Log, str);

		if (ad->ad_do_read)
			sprintf(Rstr,"    %.2f ",read_stats.lowest->speed);
		else
			sprintf(Rstr,"");
		if (ad->ad_do_write)
			sprintf(Wstr,"    %.2f ",write_stats.lowest->speed);
		else
			sprintf(Wstr,"");
		sprintf(str,"Lowest:          %s%s", Rstr, Wstr);
		AddTextNode(dln->Log, str);

		if (ad->ad_do_read)
			sprintf(Rstr,"    %.2f ",read_stats.average);
		else
			sprintf(Rstr,"");
		if (ad->ad_do_write)
			sprintf(Wstr,"    %.2f ",write_stats.average);
		else
			sprintf(Wstr,"");
		sprintf(str,"Average:         %s%s", Rstr, Wstr);
		AddTextNode(dln->Log, str);

		if (ad->ad_do_read)
			sprintf(Rstr,"   %3ld ms ",(read_stats.max_drop->drop_ms));
		else
			sprintf(Rstr,"");
		if (ad->ad_do_write)
			sprintf(Wstr,"   %3ld ms ",(write_stats.max_drop->drop_ms));
		else
			sprintf(Wstr,"");
		sprintf(str,"Idle Time 20 ms:%s%s", Rstr, Wstr);
		AddTextNode(dln->Log, str);
		AddTextNode(dln->Log, " ");

		sprintf(str,"NOTE: A MegaByte is measured as 1,000,000 bytes to be");
		AddTextNode(dln->Log, str);
		sprintf(str,"      compatible with drive manufacturer specifications.");
		AddTextNode(dln->Log, str);
		AddTextNode(dln->Log, " ");

		switch(ad->ad_SpeedIndex)
		{
			case 0:
				break;
			case 1:
				{
				/* Testing for Audio */
				double low;
				ULONG seek;

				if (ad->ad_do_read)
				{				
					if (ad->ad_do_write)
						low = ((read_stats.lowest->speed < write_stats.lowest->speed) ?
								read_stats.lowest->speed : write_stats.lowest->speed);
					else
						low = read_stats.lowest->speed;
				}
				else
					low = write_stats.lowest->speed;

				seek = (ULONG)(100.0  - (768.0 / (low * 10.0)));
				sprintf(str, "This drive will be able to support 4 simultaneous stereo");
				AddTextNode(dln->Log, str);
				sprintf(str, "audio pairs as long as the drive has a maximum full stroke");
				AddTextNode(dln->Log, str);
				sprintf(str, "seek time of %lu milliseconds or less.", seek);
				AddTextNode(dln->Log, str);
				AddTextNode(dln->Log, " ");
				}
				break;
			default:
				sprintf(str,"The last block that sustained %.2f MB/s was %ld (0x%lx)",
							SpeedVals[ad->ad_SpeedIndex], ad->ad_ss_block, ad->ad_ss_block);
				AddTextNode(dln->Log, str);
				sprintf(str,"This would allow %.2f GB of this %.2f GB drive (%.1f\%) to be",
						(((double)ad->ad_ss_block)*((double)dln->blklen)/(double)(1000000000.0)),
						(((double)dln->size)*((double)dln->blklen)/(double)(1000000000.0)),
						(((double)ad->ad_ss_block/(double)dln->size) * 100.0));
				AddTextNode(dln->Log, str);
				sprintf(str,"used in this mode.");
				AddTextNode(dln->Log, str);
				AddTextNode(dln->Log, " ");
				break;
		}

		AddTextNode(dln->Log, "Cache Parameters:  ");

		current = (struct TextNode *)(dln->CacheList->lh_Head);
		while(current)
		{
			if ((current->str) && (current->node.ln_Succ))
			{
				sprintf(str,"    %s", current->str);
				AddTextNode(dln->Log, str);
			}
			current = (struct TextNode *)(current->node.ln_Succ);
		}
	}
}

/******************************************************\
*
*	FUNCTION:
*		void ViewLog( struct List *list )
*
*	SYNOPSIS:
*		Opens a window for the user to view the current 
*		test log and cache settings for the selected drive
*
*	INPUT:
*		list - pointer to the List containing the drive and
*		cache information to view.
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void ViewLog( struct List *list )
{
	struct TextFont	*font;
	struct Screen		*scr;
	struct Window		*win;
	struct Gadget		*glist, *gadlist[MAX_GADS + 1];
	void 					*vi;
	int i;

	if (NULL == (font = OpenFont(&Topaz80)))
		ErrorMessage("Failed to Open Topaz 80");
	else
	{
		if (NULL == (scr = LockPubScreen(NULL)))
			ErrorMessage("Could not lock public screen");
		else
		{
			if (NULL ==(vi = GetVisualInfo(scr, TAG_END)))
				ErrorMessage("GetVisualInfo() failed");
			else
			{
				topborder = scr->WBorTop + (scr->Font->ta_YSize + 1);
				for(i=0;i <= MAX_GADS;i++)
					gadlist[i] = NULL;
				if (NULL == (CreateLogGadgets(&glist, vi, gadlist, list)))
					ErrorMessage("CreateLogGadgets() Failed");
				else
				{
					if (NULL ==(win = OpenWindowTags(NULL,
											WA_Title, 			FHDTVERSION,
											WA_Gadgets,			glist,
											WA_Left,				35,
											WA_Top,				13,
											WA_Width,			570,
											WA_Height,			192 + scr->Font->ta_YSize,
											WA_DragBar,			TRUE,
											WA_DepthGadget,	TRUE,
											WA_Activate,		TRUE,
											WA_CloseGadget,	TRUE,
											WA_SimpleRefresh,	TRUE,
											WA_IDCMP,			IDCMP_CLOSEWINDOW |
																	IDCMP_REFRESHWINDOW |
																	LISTVIEWIDCMP,
											WA_PubScreen,		scr,
											TAG_END)))
						ErrorMessage("OpenWindow() Failed");
					else
					{
						GT_RefreshWindow(win, NULL);
						ProcessWindowEvents(win, gadlist, NULL); 
						CloseWindow(win);
					}
				}
				FreeGadgets(glist);
				FreeVisualInfo(vi);
			}
			UnlockPubScreen(NULL, scr);
		}
		CloseFont(font);
	}
}

/******************************************************\
*
*	FUNCTION:
*		struct Gadget *CreateLogGadgets( struct Gadget **glistptr,
*														void *vi,
*														struct Gadget *gadlist
														struct List *list)
*
*	SYNOPSIS:
*		Creates all gadgets necessary for ViewLog window
*
*	INPUT:
*		glistptr - pointer to Gadget List
*		vi - pointer to a Visual Info struct
*		gadlist - an array of Gadget structs to hold all created gadgets
*		list - pointer to the list to view
*
*	RETURNS:
*		A pointer to the last gadget created.  If NULL, then the call
*		failed and some or all of the gadgets were not created.
*
*	NEEDS:
*		topborder must contain the height of the window border,
*
\******************************************************/

struct Gadget *CreateLogGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[], struct List *list)
{
	struct NewGadget ng;
	struct Gadget *gad;

	gad = CreateContext(glistptr);

	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 14 + topborder;
	ng.ng_Width = 540;
	ng.ng_Height = 152;
	ng.ng_GadgetText = "";
	ng.ng_TextAttr = &Topaz80;
	ng.ng_VisualInfo = vi;
	ng.ng_GadgetID = DSGAD_LOGLIST;
	ng.ng_Flags = PLACETEXT_ABOVE;
	gadlist[DSGAD_LOGLIST] = gad = CreateGadget(LISTVIEW_KIND, gad, &ng,
									GTLV_Labels,	list,
									GTLV_ReadOnly,	TRUE,
									
									TAG_END);	
	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 173 + topborder;
	ng.ng_Width = 175;
	ng.ng_Height = 12;
	ng.ng_Flags = PLACETEXT_IN;
	ng.ng_GadgetText = "Save Log";
	ng.ng_GadgetID = DSGAD_LOGSAVE;
	gadlist[DSGAD_LOGSAVE] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 382;
	ng.ng_GadgetText = "Main Screen";
	ng.ng_GadgetID = DSGAD_LOGCONT;
	gadlist[DSGAD_LOGCONT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);
	return(gad);
}

/******************************************************\
*
*	FUNCTION:
*		void SaveLog( struct List *list )
*
*	SYNOPSIS:
*		Opens a file requester for the user to choose or enter
*		a file name to save the log under and then saves it 
*		under that name unless the user chooses cancel
*
*	INPUT:
*		list - pointer to the List to write to the file
*
*	RETURNS:
*		none
*		upon exit, ad->ad_LogDir and ad->ad_LogFile hold the
*			new file and directory names as strings.
*	
*	NEEDS:
*
\******************************************************/

void SaveLog( struct List *list )
{

	if (list)
	{
		if (ReqFile("Save FlyerHDTools Log As...", ad->ad_LogDir, ad->ad_LogFile))
			WriteListFile(list, ad->ad_LogDir, ad->ad_LogFile);
	}
	else
		StatusMessage("No Log File to save!");
}

/******************************************************\
*
*	FUNCTION:
*		void WriteListFile(struct List *list, char *dir, char *filepart)
*
*	SYNOPSIS:
*		Outputs the text of a List to a file of name dir/filepart
*
*	INPUT:
*		list - pointer to the list to write to a file
*		dir - string containing the name of the directory to write the file to 
*		filepart - string containing the name of the file to write.
*
*	RETURNS:
*		none
*
*	NEEDS:
*		list must be a pointer to a valid initialized List struct
*
\******************************************************/

void WriteListFile(struct List *list, char *dir, char *filepart)
{
	FILE *file = NULL;
	struct TextNode *current;
	char filename[192];

	sprintf(filename, "%s", dir);
	AddPart(filename, filepart, 191);

	if (file = fopen(filename, "w"))
	{
		current = (struct TextNode *)(list->lh_Head);
		
		while(current)
		{
			if ((current->str) && (current->node.ln_Succ))
				fprintf(file, "%s\n", current->str);
			current = (struct TextNode *)(current->node.ln_Succ);
		}
		fclose(file);
		StatusMessage("Wrote File");
	}
	else
		StatusMessage("Can't Open File!"); 
}

/******************************************************\
*
*	FUNCTION:
*		struct Gadget *CreateCacheGadgets( struct Gadget **glistptr,
*														void *vi,
*														struct Gadget *gadlist
														struct List *list)
*
*	SYNOPSIS:
*		Creates all gadgets necessary for ViewCache window
*
*	INPUT:
*		glistptr - pointer to Gadget List
*		vi - pointer to a Visual Info struct
*		gadlist - an array of Gadget structs to hold all created gadgets
*		list - pointer to the list to view
*
*	RETURNS:
*		A pointer to the last gadget created.  If NULL, then the call
*		failed and some or all of the gadgets were not created.
*
*	NEEDS:
*		topborder must contain the height of the window border,
*
\******************************************************/

struct Gadget *CreateCacheGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[], struct List *list)
{
	struct NewGadget ng;
	struct Gadget *gad;

	gad = CreateContext(glistptr);

	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 14 + topborder;
	ng.ng_Width = 540;
	ng.ng_Height = 152;
	ng.ng_GadgetText = "";
	ng.ng_TextAttr = &Topaz80;
	ng.ng_VisualInfo = vi;
	ng.ng_GadgetID = DSGAD_LOGLIST;
	ng.ng_Flags = PLACETEXT_ABOVE;
	gadlist[DSGAD_LOGLIST] = gad = CreateGadget(LISTVIEW_KIND, gad, &ng,
									GTLV_Labels,	list,
									GTLV_ReadOnly,	TRUE,
									
									TAG_END);	
	ng.ng_LeftEdge = 168;
	ng.ng_TopEdge = 173 + topborder;
	ng.ng_Width = 175;
	ng.ng_Height = 12;
	ng.ng_GadgetText = "Continue";
	ng.ng_GadgetID = DSGAD_LOGCONT;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_LOGCONT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);
	return(gad);
}

/******************************************************\
*
*	FUNCTION:
*		void ViewCache( struct List *list )
*
*	SYNOPSIS:
*		Opens a window for the user to view the current 
*		cache settings for the selected drive
*
*	INPUT:
*		list - pointer to the list containing the drive and
*		cache information to view.
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void ViewCache( struct List *list ) 
{
	struct TextFont	*font;
	struct Screen		*scr;
	struct Window		*win;
	struct Gadget		*glist, *gadlist[MAX_GADS + 1];
	void 					*vi;
	int i;

	if (NULL == (font = OpenFont(&Topaz80)))
		ErrorMessage("Failed to Open Topaz 80");
	else
	{
		if (NULL == (scr = LockPubScreen(NULL)))
			ErrorMessage("Could not lock public screen");
		else
		{
			if (NULL ==(vi = GetVisualInfo(scr, TAG_END)))
				ErrorMessage("GetVisualInfo() failed");
			else
			{
				topborder = scr->WBorTop + (scr->Font->ta_YSize + 1);
				for(i=0;i <= MAX_GADS;i++)
					gadlist[i] = NULL;
				if (NULL == (CreateCacheGadgets(&glist, vi, gadlist, list)))
					ErrorMessage("CreateCacheGadgets() Failed");
				else
				{
					if (NULL ==(win = OpenWindowTags(NULL,
											WA_Title, 			"View Cache Settings",
											WA_Gadgets,			glist,
											WA_Left,				35,
											WA_Top,				13,
											WA_Width,			570,
											WA_Height,			192 + scr->Font->ta_YSize,
											WA_DragBar,			TRUE,
											WA_DepthGadget,	TRUE,
											WA_Activate,		TRUE,
											WA_CloseGadget,	TRUE,
											WA_SimpleRefresh,	TRUE,
											WA_IDCMP,			IDCMP_CLOSEWINDOW |
																	IDCMP_REFRESHWINDOW |
																	LISTVIEWIDCMP,
											WA_PubScreen,		scr,
											TAG_END)))
						ErrorMessage("OpenWindow() Failed");
					else
					{
						GT_RefreshWindow(win, NULL);
						ProcessWindowEvents(win, gadlist, NULL);
						CloseWindow(win);
					}
				}
				FreeGadgets(glist);
				FreeVisualInfo(vi);
			}
			UnlockPubScreen(NULL, scr);
		}
		CloseFont(font);
	}
}

/******************************************************\
*
*	FUNCTION:
*		struct Gadget *CreateFormatGadgets( struct Gadget **glistptr,
*														void *vi,
*														struct Gadget *gadlist)
*
*	SYNOPSIS:
*		Creates all gadgets necessary for FormatWindow
*
*	INPUT:
*		glistptr - pointer to Gadget List
*		vi - pointer to a Visual Info struct
*		gadlist - an array of Gadget structs to hold all created gadgets
*
*	RETURNS:
*		A pointer to the last gadget created.  If NULL, then the call
*		failed and some or all of the gadgets were not created.
*
*	NEEDS:
*		topborder must contain the height of the window border,
*		AVFlagLabels must all be set to the strings to be displayed
*			on the A/V cyclic gadget, 
*		ad->ad_videoflag must be set properly,
*		ad->ad_do_ss must be set properly.
*
\******************************************************/

struct Gadget *CreateFormatGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[])
{
	struct NewGadget ng;
	struct Gadget *gad;


	gad = CreateContext(glistptr);

	ng.ng_LeftEdge = 252;
	ng.ng_TopEdge = topborder + 7;
	ng.ng_Width = 226;
	ng.ng_Height = 14;
	ng.ng_GadgetText = "Device:";
	ng.ng_TextAttr = &Topaz80;
	ng.ng_VisualInfo = vi;
	ng.ng_GadgetID = DSGAD_FDEVICE;
	ng.ng_Flags = PLACETEXT_LEFT;

	gadlist[DSGAD_FDEVICE] = gad = CreateGadget(TEXT_KIND, gad, &ng,
									TAG_END);	

	ng.ng_LeftEdge = 252;
	ng.ng_TopEdge += 10;
	ng.ng_Width = 48;
	ng.ng_GadgetText = "Firmware Rev:";
	ng.ng_GadgetID = DSGAD_FREV;
	gadlist[DSGAD_FREV] = gad = CreateGadget(TEXT_KIND, gad, &ng,
									TAG_END);	

	ng.ng_LeftEdge = 252;
	ng.ng_TopEdge += 10;
	ng.ng_Width = 256;
	ng.ng_GadgetText = "Volume:";
	ng.ng_GadgetID = DSGAD_FNAME;
	gadlist[DSGAD_FNAME] = gad = CreateGadget(TEXT_KIND, gad, &ng,
									TAG_END);	

	ng.ng_LeftEdge = 252;
	ng.ng_TopEdge += 10;
	ng.ng_Width = 32;
	ng.ng_GadgetText = "Size:";
	ng.ng_GadgetID = DSGAD_FCAPACITY;
	gadlist[DSGAD_FCAPACITY] = gad = CreateGadget(TEXT_KIND, gad, &ng,
									TAG_END);	

	ng.ng_TopEdge += 28;
	ng.ng_Width = 238;
	ng.ng_GadgetText = "New Volume Name  :";
	ng.ng_GadgetID = DSGAD_VOLNAME;
	gadlist[DSGAD_VOLNAME] = gad = CreateGadget(STRING_KIND, gad, &ng, 
									TAG_END);	

	ng.ng_TopEdge += 14;
	ng.ng_Width = 118;
	ng.ng_Height = 14;
	ng.ng_GadgetText = "Format Volume For:";
	ng.ng_GadgetID = DSGAD_AVFLAGS;
	ng.ng_Flags = PLACETEXT_LEFT;
	gadlist[DSGAD_AVFLAGS] = gad = CreateGadget(CYCLE_KIND, gad, &ng, 
									GTCY_Labels,	AVFlagLabels,
									GTCY_Active,	(ad->ad_videoflag ? 0:1),
									TAG_END);


	ng.ng_TopEdge += 14;
	ng.ng_GadgetText = "Short Stroke     :";
	ng.ng_GadgetID = DSGAD_DOSHORT;
	ng.ng_Flags = PLACETEXT_LEFT;
	gadlist[DSGAD_DOSHORT] = gad = CreateGadget(CYCLE_KIND, gad, &ng, 
									GTCY_Labels,	SSLabels,
									GTCY_Active,	ad->ad_do_ss,
									TAG_END);

	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_SSVAL;
	ng.ng_LeftEdge = 372;
	ng.ng_Width = 118;
	gadlist[DSGAD_SSVAL] = gad = CreateGadget(STRING_KIND, gad, &ng, 
									GA_Disabled,				(ad->ad_do_ss == 0),
									TAG_END);


	ng.ng_TopEdge += 14;
	ng.ng_GadgetID = DSGAD_TESTSS;
	ng.ng_Width = 118;
	ng.ng_GadgetText = "Percent of drive above target:";
	gadlist[DSGAD_TESTSS] = gad = CreateGadget(TEXT_KIND, gad, &ng,
												TAG_END);

	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 173 + topborder;
	ng.ng_Width = 175;
	ng.ng_Height = 12;
	ng.ng_GadgetText = "Format";
	ng.ng_GadgetID = DSGAD_DOFORMAT;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_DOFORMAT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 382;
	ng.ng_GadgetText = "Main Screen";
	ng.ng_GadgetID = DSGAD_EXIT;
	gadlist[DSGAD_EXIT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	return(gad);
}

/******************************************************\
*
*	FUNCTION:
*		void FormatWindow( struct DLNode *dln )
*
*	SYNOPSIS:
*		Opens a window for the user to choose how to format 
*		a selected drive
*
*	INPUT:
*		dln - pointer to the DLNode struct for the drive to be formatted
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void FormatWindow( struct DLNode *dln ) 
{
	struct TextFont	*font;
	struct Screen		*scr;
	struct Window		*win;
	struct Gadget		*glist, *gadlist[MAX_GADS + 1];
	void 					*vi;
	int i;

	if (NULL == (font = OpenFont(&Topaz80)))
		ErrorMessage("Failed to Open Topaz 80");
	else
	{
		if (NULL == (scr = LockPubScreen(NULL)))
			ErrorMessage("Could not lock public screen");
		else
		{
			if (NULL ==(vi = GetVisualInfo(scr, TAG_END)))
				ErrorMessage("GetVisualInfo() failed");
			else
			{
				topborder = scr->WBorTop + (scr->Font->ta_YSize + 1);
				for(i=0;i <= MAX_GADS;i++)
					gadlist[i] = NULL;
				if (NULL == (CreateFormatGadgets(&glist, vi, gadlist)))
					ErrorMessage("CreateFormatGadgets() Failed");
				else
				{
					if (NULL ==(win = OpenWindowTags(NULL,
											WA_Title, 			"Format Flyer Drive",
											WA_Gadgets,			glist,
											WA_Left,				35,
											WA_Top,				13,
											WA_Width,			570,
											WA_Height,			192 + scr->Font->ta_YSize,
											WA_DragBar,			TRUE,
											WA_DepthGadget,	TRUE,
											WA_Activate,		TRUE,
											WA_CloseGadget,	TRUE,
											WA_SimpleRefresh,	TRUE,
											WA_IDCMP,			IDCMP_CLOSEWINDOW |
																	IDCMP_REFRESHWINDOW |
																	LISTVIEWIDCMP,
											WA_PubScreen,		scr,
											TAG_END)))
						ErrorMessage("OpenWindow() Failed");
					else
					{
						UpdateFormatWindow(win, gadlist, dln, 
																	ad->ad_FormatBuffer);
						GT_RefreshWindow(win, NULL);
						ProcessWindowEvents(win, gadlist, NULL);
						CloseWindow(win);
					}
				}
				FreeGadgets(glist);
				FreeVisualInfo(vi);
			}
			UnlockPubScreen(NULL, scr);
		}
		CloseFont(font);
	}
}

/******************************************************\
*
*	FUNCTION:
*		void PurgeMsgPort( struct Window *win )
*
*	SYNOPSIS:
*		clears all messages from the selected message port
*		to avoid extraneous mouse clicks and menu selections
*		while the window is inactive.
*
*	INPUT:
*		win - pointer to the window to clear messages from 
*
*	RETURNS:
*		nothing
*
*	NEEDS:
*
\******************************************************/

void PurgeMsgPort(struct Window *win)
{
	struct IntuiMessage *imsg;
	ULONG imsgClass;

	while (imsg = GT_GetIMsg(win->UserPort))
	{
		imsgClass = imsg->Class;
		GT_ReplyIMsg(imsg);
		switch(imsgClass)
		{
			case IDCMP_REFRESHWINDOW:
				GT_BeginRefresh(win);
				GT_EndRefresh(win, TRUE);
				break;
		}
	}
}

/******************************************************\
*
*	FUNCTION:
*		void CreateCacheList( struct Manuf *man , struct List **listptr)
*
*	SYNOPSIS:
*		Will create and fill in list with the information from man
*
*	INPUT:
*		man - pointer to the Manuf struct containing the cache parameters
*		listptr - pointer to the pointer to the list to be filled in
*
*	RETURNS:
*		none
*		upon exit, listptr will point to an exec list containing
*		a text version of the Manufacturer and Cache Setting information
*		from the man struct suitable for attaching to a ListView gadget.
*
*	NEEDS:
*		man must be a valid, filled Manuf struct
*
\******************************************************/

void CreateCacheList( struct Manuf *man, struct List **listptr )
{
	char str[128];
	struct Options *opt;
 
	if (listptr == NULL)	
		return;
	
	if (*listptr)
		FreeTextList(*listptr);

	if (!(*listptr = AllocMem(sizeof(struct List), MEMF_CLEAR)))
	{
		StatusMessage("Out of Memory!");
	}
	else
	{
		NewList(*listptr);
		StatusMessage("Getting Cache Parameters...");
		sprintf(str,"{%-8s%-16s%-4s,%.31s,%lx\0",man->manufname,man->modelname,
							man->revision, man->comment, man->blocks);
		AddTextNode(*listptr, str);
		for(opt = man->options; opt != NULL; opt = opt->next)
		{
			sprintf(str,"$%-.15s,%lx,%lx,%lx,%lx\0", opt->optname,
				(ULONG)opt->page, (ULONG)opt->pos, (ULONG)opt->flag, opt->value);
			AddTextNode(*listptr, str);
		}
		sprintf(str, "}\0");
		AddTextNode(*listptr, str);
		sprintf(str, "[%lx]\0", man->chksum);
		AddTextNode(*listptr, str);
	}
}

/******************************************************\
*
*	FUNCTION:
*		void ValidVolName( char *name )
*
*	SYNOPSIS:
*		checks name to see if it is a valid Volume Name.
*		If not, it prompts the user for a new one until a
*		valid string is entered
*		(currently unimplemented)
*
*	INPUT:
*		name - string to check as well as fill in if additional
*		user input is necessary
*
*	RETURNS:
*
*	NEEDS:
*		name must point to a valid string buffer
*
\******************************************************/

void ValidVolName( char *name )
{

}

/******************************************************\
*
*	FUNCTION:
*		void FormatDrive( struct DLNode *dln )
*
*	SYNOPSIS:
*		Formats the given drive with the audio/video flags
*		and short stroke values set in the format options panel
* 
*	INPUT:
*		dln - pointer to the DLNode struct for the drive to be formatted
*
*	RETURNS:
*		none
*
*	NEEDS:
*		dln must be initialized with at least the FlyVol and size members set.
*		ad->ad_do_ss will determine whether the drive is short stroked,
*		ad->ad_ss_val or ad->ad_fmt_ss_block will determine how much,
*		ad->ad_videoflag will determine whether the drive is a/v or audio only
*		ad->ad_VolNameStr must hold the name to format the drive as
*
\******************************************************/

void FormatDrive( struct DLNode *dln )
{
	ULONG res, blks;
	struct DateStamp now;
 	UBYTE avf;
	
	switch(ad->ad_do_ss)
	{
		case 0:
			blks = 0L;
			break;
		case 1:
			blks = (ULONG)(ad->ad_ss_val * (float)dln->size / 100.0);			
			break;
		case 2:
			blks = ad->ad_fmt_ss_block;
			break;
	}

	avf = (ad->ad_videoflag ? (FVIF_VIDEOREADY | FVIF_AUDIOREADY) : FVIF_AUDIOREADY);

	DateStamp(&now);
	res = FlyerFormat(&(dln->FlyVol), ad->ad_VolNameStr, &now, blks, avf);
// Since we are providing new options for cache setting, we will not 
// 	automatically apply cache settings when formatting.
//	SetCache(dln, opt);
}

/******************************************************\
*
*	FUNCTION:
*		BOOL VerifyFormat(struct Window *win )
*
*	SYNOPSIS:
*		Displays a requester to ask the user for confirmation before 
*		formatting the selected drive.
*
*	INPUT:
*		win - A pointer to the parent window for the requester
*
*	RETURNS:
*		TRUE if user wished to format the drive, 
*		FALSE if user wishes to abort the format process.
*
*	NEEDS:
*		win must be a valid pointer to an open window.
*
\******************************************************/

BOOL VerifyFormat(struct Window *win )
{
	LONG reply;

	struct EasyStruct ESWrite = 
		{
		sizeof(struct EasyStruct), 0, NULL,
		"Formatting will destroy all data on the selected drive.\nContinue?\n",
		"CONTINUE|ABORT",
		};
	
	reply = EasyRequest(win, &ESWrite, NULL );
	GT_BeginRefresh(win);
	GT_EndRefresh(win, TRUE);
	
	return((BOOL)reply);
}

/******************************************************\
*
*	FUNCTION:
*		void clear ( char *str, ULONG len )
*
*	SYNOPSIS:
*		clears a string of user defined length out to all NULLs
*
*	INPUT:
*		str - pointer to the first char to clear
*		len - number of chars to clear
*
*	RETURNS:
*		none
*
*	NEEDS:
*		str must be a pointer to a string that has been declared
*		as at least len bytes
\******************************************************/

void clear( char *str, ULONG len )
{
	ULONG i;

	for (i = 0 ; i < len ; i++)
	{
		str[i] = '\0';
	}
}

/******************************************************\
*
*	FUNCTION:
*		void Init_Monitor( struct Monitor *mon, 
*								 struct Window  *win,
*								 LONG				  x
*								 LONG				  y 
*								 LONG				  w
*								 LONG				  h
*								 ULONG 			  steps )
*
*	SYNOPSIS:
*		Inintializes Monitor struct to be used as fuel gauge
*
*	INPUT:
*		mon - pointer to the monitor struct to initialize
*		win - pointer to the window on which to display the monitor
*		x - x coord of the upper left hand corner of the monitor
*		y - y coord of the upper left hand corner of the monitor
*		w - width of the monitor
*		h - height of the monitor
*		steps - number of steps in the monitor
*
*	RETURNS:
*		none
*
*	NEEDS:
*		mon must point to a valid and initialized Monitor structure
*		win must be a pointer to a valid Window
*
\******************************************************/

void Init_Monitor(struct Monitor *mon, struct Window *win, 
						LONG x, LONG y, LONG w, LONG h, ULONG steps )
{
	mon->rp = win->RPort;
	mon->x = x;
	mon->y = y;
	mon->w = w;
	mon->h = h;
	mon->steps = steps;
	mon->curr_step = 0L;

		StatusMessage(" ");
}

/******************************************************\
*
*	FUNCTION:
*		void Inc_Monitor( struct Monitor *mon )
*
*	SYNOPSIS:
*		Increments fuel gauge by 1 step 
*
*	INPUT:
*		mon - pointer to the monitor struct to step
*
*	RETURNS:
*		none
*
*	NEEDS:
*		mon must point to a valid and initialized Monitor structure
*
\******************************************************/

void Inc_Monitor( struct Monitor *mon )
{
	Step_Monitor( mon, 1L );
}

/******************************************************\
*
*	FUNCTION:
*		void Step_Monitor( struct Monitor *mon, ULONG steps )
*
*	SYNOPSIS:
*		Increments fuel gauge by steps 
*
*	INPUT:
*		mon - pointer to the monitor struct to step
*		steps - number of steps to increment the monitor
*
*	RETURNS:
*		none
*
*	NEEDS:
*		mon must point to a valid and initialized Monitor structure
*
\******************************************************/

void Step_Monitor( struct Monitor *mon, ULONG steps )
{
	mon->curr_step += steps;
	if (mon->curr_step > mon->steps)
		mon->curr_step = mon->steps;

	SetAPen(mon->rp, 3);
	RectFill(mon->rp, mon->x, mon->y, 
				mon->x + (mon->curr_step * mon->w / mon->steps), 
				mon->y + mon->h);
}

/******************************************************\
*
*	FUNCTION:
*		void End_Monitor( struct Monitor *mon )
*
*	SYNOPSIS:
*		Briefly displays the fuel gauge at full before resetting
*		its monitor structure
*
*	INPUT:
*		mon - Pointer to the Monitor struct to finish and reset
*
*	RETURNS:
*		none
*		Upon exit ad->ad_mon is reset
*
*	NEEDS:
*		*mon must be a valid and initialized Monitor struct
*
\******************************************************/

void End_Monitor( struct Monitor *mon )
{
	Step_Monitor( mon, mon->steps );

	SetAPen(mon->rp, 0);
	RectFill(mon->rp, mon->x, mon->y, mon->x + mon->w, mon->y + mon->h);
	mon->x = 0L;
	mon->y = 0L;
	mon->w = 0L;
	mon->h = 0L;
	mon->steps = 0L;
	mon->curr_step = 0L;
}

/******************************************************\
*
*	FUNCTION:
*		void SetCache(	struct DLNode *dln , struct Options *opt )
*
*	SYNOPSIS:
*		Applies the list of drive options to the drive referenced
*		by dln.
*
*	INPUT:
*		dln - A pointer to the drive to set parameters for
*		opt - A pointer to the first Option in the list to be applied
*
*	RETURNS:
*		none
*
*	NEEDS:
*		dln and opt must both be valid and initialized structures
*
\******************************************************/

void SetCache( struct DLNode *dln, struct Options *opt )
{
	for(;opt != NULL;opt = opt->next)
		SetOption( dln, opt );
}

/******************************************************\
*
*	FUNCTION:
*		void GetManuf( struct DLNode *dln )
*
*	SYNOPSIS:
*		Fills in a Manuf struct with data on the selected drive.
*		Also attempts to match up the drive with a known set of 
*			parameters, and places a pointer to those settings in
*			the man->next field.
*
*	INPUT:
*		dln - A pointer to the DLNode struct for the drive
*
*	RETURNS:
*		none
*		Upon exit, *man contains data about the drive referenced by dln
*
*	NEEDS:
*		A valid and initialized DLNode struct pointed to by dln
*		A valid Manuf struct pointed to by man
*
\******************************************************/

void GetManuf( struct DLNode *dln )
{
	ULONG res;
	UBYTE InqBuff[256];

	//Fill in fields of the Manuf struct...
	res = Inquiry(&(dln->FlyVol), 36, InqBuff);
	if (res)
	{
		StatusMessage("Inquiry Error!");
		dln->Manuf.manufname[0] = '\0';
		dln->Manuf.modelname[0] = '\0';
		dln->Manuf.revision[0] = '\0';
		dln->Manuf.blocks = 0L;
		dln->Manuf.options = NULL;
	}
	else
	{
		strncpy(dln->Manuf.manufname, &(InqBuff[8]) ,  8);
		strncpy(dln->Manuf.modelname, &(InqBuff[16]), 16);
		strncpy(dln->Manuf.revision,  &(InqBuff[32]),  4);
		dln->Manuf.manufname[8]  = '\0';
		dln->Manuf.modelname[16] = '\0';
		dln->Manuf.revision[4]   = '\0';
		dln->Manuf.options = NULL;
	}	
} 

/******************************************************\
*
*	FUNCTION: 
*		void SetOption( struct DLNode *dln, struct Options *opt )
*
*	SYNOPSIS:
*		Applies a given Option to the selected drive
*
*	INPUT:
*		dln - A pointer to the DLNode struct for the drive to be affected
*		opt - A pointer to the Option struct to be applied to the drive
*
*	RETURNS:
*		none
*
*	NEEDS:
*		dln must point to a valid, initialized DLNode struct
*		opt must point to a valid, initialized Options struct
*
\******************************************************/

void SetOption( struct DLNode *dln, struct Options *opt)
{
	UBYTE pagelen, PageBuffer[256], *buffptr;
	ULONG res;
	int i;

	/* Get Length of Mode Page */
	res = ModeSense(&(dln->FlyVol), 1 , opt->page, &pagelen);
	if (res)
		 StatusMessage("ModeSense1 Error!");
	
	pagelen++;  /* Total length of data phase */

	if (pagelen > 255)
	{
		StatusMessage("Error - page length too long");
		return;
	}

	/* Read pages current settings */
	res = ModeSense(&(dln->FlyVol), pagelen, opt->page, PageBuffer);
	if (res)
	{
		StatusMessage("ModeSense2 Error!");
		return;
	}
	
	PageBuffer[0] = 0;
	buffptr = &(PageBuffer[4]);

	if (PageBuffer[3])   /* Did Drive return block descriptor? */
	{
		PageBuffer[3] = 0;
		for(i=12; i < pagelen; i++)   /* Strip it out... */
			PageBuffer[i-8] = PageBuffer[i];
		pagelen -= 8;
	}

	if (opt->flag)     /* Is the option a single bit? */
	{
		if (opt->value == 1)
			buffptr[opt->pos] |= opt->flag;     /* Set it */
		else
			buffptr[opt->pos] &= (~opt->flag);     /* Clear it */
	}
	else
	{
		if (opt->value >= 0)
			buffptr[opt->pos] = opt->value;
	}

	buffptr[0] &= 0x7F;		/* Strip off PS bit before sending back */

	/* Send updated page back to drive... */
	res = ModeSelect(&(dln->FlyVol), pagelen, PageBuffer, 0x11);    /*PF=1,SP=1 */
	if (res)
		res = ModeSelect(&(dln->FlyVol), pagelen, PageBuffer, 0x01); /*PF=0,SP=1 */
		if (res)
			StatusMessage("ModeSelect Error!");
}

/******************************************************\
*
*	FUNCTION:
*		int AddDriveData( struct Window *win,
*								char *dir,
*								char *filepart)
*
*	SYNOPSIS:
*		Reads in a file containing one or more sets of drive data,
*		and skips any extraneous data before the data, allowing it to read 
*		data in from a saved log.
*		
*	INPUT:
*		win - pointer to parent window
*		dir - string containing the name of the directory to save the file to 
*		filepart - string containing the name of the file to save
*		
*	RETURNS:
*		0 if no error occurred
*		1 if an error occurred
*		fills in the ad->ad_Tester variable if the info is in the file
*		adds a new Manuf struct to ad->ad_Manuf_List for each entry in the file
*		verifies that the Manuf is unique for a given revision of a drive
*		and asks the user if the new version should replace the older if one 
*		already exists.
*
*	NEEDS:
*
\******************************************************/

int AddDriveData(struct Window *win, char *dir, char *filepart)
{
	FILE *file = NULL;
	char filename[192], c;
	struct Manuf man, *newman;
	struct Options opt, *newopt;
	ULONG Tlong[3];
	int res;

	sprintf(filename, "%s", dir);
	AddPart(filename, filepart, 191);

	if (file = fopen(filename, "r"))
	{
		
		res = fscanf(file, "<%[^>]>", ad->ad_Tester);
		if (res == EOF)
		{
			fclose(file);
			return(0);
		}
		for (c = fgetc(file);c != EOF;c=fgetc(file))
			if (c == '{')
			{
				c= ungetc(c, file);
				break;
			}

		for(;;)
		{
			res = fscanf(file,"{%8c%16c%4c,%[^,],%lx", man.manufname, man.modelname, //}
												man.revision, man.comment, &(man.blocks));
			if (( res == EOF) || (res < 4))
				break;
			man.manufname[8] = '\0';
			man.modelname[16] = '\0';
			man.revision[4] = '\0';
			newman = AllocMem(sizeof(struct Manuf), MEMF_CLEAR);
			strncpy(newman->manufname, man.manufname, 8);
			strncpy(newman->modelname, man.modelname, 16);
			strncpy(newman->revision, man.revision, 4);
			strncpy(newman->comment, man.comment, 32);
			newman->blocks = man.blocks;
			newman->options = NULL;	

			for(;;)
			{
				clear(opt.optname, 16);  
				res = fscanf(file, "$%15[^,],%lx,%lx,%lx,%lx", opt.optname,  
							&(Tlong[0]), &(Tlong[1]), &(Tlong[2]), &(opt.value));
				if ((res == EOF) || (res < 5))
					break;
				opt.page = Tlong[0];
				opt.pos  = Tlong[1];
				opt.flag = Tlong[2];
				newopt = AllocMem(sizeof(struct Options), MEMF_CLEAR);
				strncpy(newopt->optname, opt.optname, 15);
				newopt->page = opt.page;
				newopt->pos = opt.pos;
				newopt->flag = opt.flag;
				newopt->value = opt.value;
				newopt->next = newman->options;
				newman->options = newopt;
			}
									//{
			res = fscanf(file, "}[%lx]", &(newman->chksum));
			if ((res == EOF) || (res < 1))			
				break;


			// Check to see if the file has been modified
			if (VerifyChecksum(win, newman))
			{
				SortOptions(newman);
				AddManufNode(newman);
			}
			else
			{
				while (newman->options != NULL)
				{
					newopt = newman->options->next;
					FreeMem(newman->options, sizeof(struct Options));
					newman->options = newopt;
				}
				FreeMem(newman, sizeof(struct Manuf));
				newman = NULL;
			}
		}
		fclose(file);
		return(0);
	}
	else
		return(1);
}

/******************************************************\
*
*	FUNCTION:
*		int WriteDriveData( char *dir, char *filepart )
*
*	SYNOPSIS:
*		Saves all of the currently loaded options for drive
*		settings to the config file
*
*	INPUT:
*		dir - string containing the directory to save the config file in
*		filepart - string containing the name of the file to save the config as
*
*	RETURNS:
*		0 if no error occurred
*		1 if an error occurred
*
*	NEEDS:
*		a valid name in ad->ad_Tester
*		ad->ad_Manuf_List must be a valid list of Manufs
*
\******************************************************/

int WriteDriveData(char *dir, char *filepart, struct Manuf *man)
{
	FILE *file = NULL;
	char filename[192];
	struct Manuf *currman;
	struct Options *curropt;

	sprintf(filename, "%s", dir);
	AddPart(filename, filepart, 191);

	if (file = fopen(filename, "w"))
	{
		if (man == NULL)
		{
			fprintf(file, "<%s>\n", ad->ad_Tester);

			for(currman = (struct Manuf *)ad->ad_ManufList->lh_Head; 
				 currman->node.ln_Succ != NULL; 
				 currman = (struct Manuf *)currman->node.ln_Succ)
			{
				fprintf(file,"{%-8s%-16s%-4s,%s,%lx\n", currman->manufname,
							currman->modelname, currman->revision, currman->comment, 
							currman->blocks);
				currman->chksum = 0L;

				for(curropt = currman->options; curropt != NULL; curropt = curropt->next)
				{
					currman->chksum += curropt->page + curropt->pos + curropt->flag + curropt->value; 
					fprintf(file,"$%-.15s,%lx,%lx,%lx,%lx\n", curropt->optname,
						(ULONG)curropt->page, (ULONG)curropt->pos, (ULONG)curropt->flag, curropt->value);
				}
				fprintf(file, "}\n[%lx]\n", currman->chksum);
			}
		}
		else
		{
			fprintf(file,"{%-8s%-16s%-4s,%s,%lx\n", man->manufname,
						man->modelname, man->revision, man->comment, 
						man->blocks);
			man->chksum = 0L;
			for(curropt = man->options; curropt != NULL; curropt = curropt->next)
			{
				man->chksum += curropt->page + curropt->pos + curropt->flag + curropt->value; 
				fprintf(file,"$%-.15s,%lx,%lx,%lx,%lx\n", curropt->optname,
					(ULONG)curropt->page, (ULONG)curropt->pos, (ULONG)curropt->flag, curropt->value);
			}
			fprintf(file, "}\n[%lx]\n", man->chksum);
		}
		fclose(file);
	}
	else
		return(1);
	return(0);
}

/******************************************************\
*
*	FUNCTION:
*		UWORD Menu2ID( UWORD menu, UWORD item, UWORD sub, UWORD *code)
*
*	SYNOPSIS:
*		given a number of a (sub)item, returns the gadget constant
*		that is used in HandleGadgetEvent to perform the function.
*		Thus, all gadget and menu handling is done in the same function.
*
*	INPUT:
*		menu - menu number of the selection
*		item - item number of the selection
*		sub - subitem number of the selection
*     code - pointer to the code in the MENUPICK message.
*
*	RETURNS:
*		the defined constant representing a gadget ID that the menu
*		(sub)item is connected with
*
*	NEEDS:
*
\******************************************************/

UWORD Menu2ID(UWORD menu, UWORD item, UWORD sub, UWORD *code)
{
	switch(menu)
	{
		case 0:
			switch(item)
			{
				case 0:
					return(DSGAD_SAVELOG);
					break;
				case 1:
					return(DSGAD_SAVEDATA);
					break;
				case 3:
					return(DSGAD_EXIT);
					break;
			}
			break;
		case 1:
			switch(item)
			{
				case 0:
					return(DSGAD_LONGLOG);
					break;
				case 1:
					return(DSGAD_AUTOSAVE);
					break;
			}
			break;
		case 2:
			switch(item)
			{
				case 0:
					switch(sub)
					{
						case 0:
							return(DSGAD_READCACHE);
							break;
						case 1:
							return(DSGAD_READDATA);
							break;
					}
					break;
			}
			break;
	}
	return(DSGAD_NOOP);
}

/******************************************************\
*
*	FUNCTION:
*		void FetchCacheSettings( struct DLNode *dln )
*
*	INPUT:
*		dln - pointer to the DLNode for the drive to fetch from
*
*	SYNOPSIS:
*		Reads cache settings from a drive and builds a list of
*		changeable Options which are then attached to dln->Manuf.
*
*	RETURNS:
*		none
*		Upon exit, dln->Manuf.options is a valid list of changeable Options
*		for the drive.
*
*	NEEDS:
*		dln must be filled in with necessary information from drive
*
\******************************************************/

void FetchCacheSettings( struct DLNode *dln )
{
	struct Options *opt;
	UBYTE j, pagenum, pagelen, bufflen, origlen, *maskbuff, *buffer;
	ULONG res, i, pagestart;

	while(dln->Manuf.options != NULL)
	{
		opt = dln->Manuf.options->next;
		FreeMem(dln->Manuf.options, sizeof(struct Options));
		dln->Manuf.options = opt;
	}
	dln->Manuf.chksum = 0L;

	dln->FlyVol.Flags = FVF_USENUMS;

	/* Get length of all mode pages... */
	res = ModeSense(&(dln->FlyVol), 1, 0x3f, &bufflen);
	if (res)
	{
		StatusMessage("ModeSense Error!");
		return;
	}
	bufflen++;   /* Total length of data phase */
	origlen = bufflen;

	if ((maskbuff = AllocMem(bufflen, MEMF_CLEAR)) == NULL)
	{
		StatusMessage("Out of Memory!");
		return;
	}
	if ((buffer   = AllocMem(bufflen, MEMF_CLEAR)) == NULL)
	{
		StatusMessage("Out of Memory!");
		return;
	}

	/*  Read pages CURRENT settings */
	res = ModeSense(&(dln->FlyVol), bufflen, 0x3f, buffer);
	if (res)
	{
		StatusMessage("Mode Sense Error!");
		FreeMem(buffer, origlen);
		FreeMem(maskbuff, origlen);
		return;
	}

	/*  Read pages CHANGEABLE settings */
	res = ModeSense(&(dln->FlyVol), bufflen, 0x7f, maskbuff);
	if (res)
	{
		StatusMessage("Mode Sense Error!");
		FreeMem(buffer, origlen);
		FreeMem(maskbuff, origlen);
		return;
	}

	if (buffer[3] > 0)
	{
		for(i=buffer[3] + 4;i < bufflen; i++)
		{
			buffer[i-buffer[3]] = buffer[i];
			maskbuff[i-buffer[3]] = maskbuff[i];
		}
		bufflen -= buffer[3];

		buffer[3] = 0;
		maskbuff[3] = 0;
	}

	pagestart = 4;
	while(pagestart < bufflen)
	{
		pagenum = buffer[pagestart] & 0x3f;
		pagelen = buffer[pagestart + 1];
		for(i=2;i< pagelen + 2; i++)
		{
			switch(maskbuff[pagestart + i])
			{
				case 0:
					break;
				case 255:
					if ((opt = AllocMem(sizeof(struct Options), MEMF_CLEAR)) == NULL)
					{
						StatusMessage("Out of Memory!");
						FreeMem(buffer, origlen);
						FreeMem(maskbuff, origlen);
						return;
					}
					opt->optname[0] = '\0';
					opt->page = pagenum;
					opt->pos = i;
					opt->flag = 0;
					opt->value = buffer[pagestart + i];
					opt->next = dln->Manuf.options;
					dln->Manuf.options = opt;
					dln->Manuf.chksum += opt->page + opt->pos + opt->flag + opt->value;
					break;
				default:
					for(j=0;j<8;j++)
					{
						if (maskbuff[pagestart + i] & (1<<j))
						{
							if ((opt = AllocMem(sizeof(struct Options), MEMF_CLEAR)) == NULL)
							{
								StatusMessage("Out of Memory!");
								FreeMem(buffer, origlen);
								FreeMem(maskbuff, origlen);
								return;
							}
							opt->optname[0] = '\0';
							opt->page = pagenum;
							opt->pos = i;
							opt->flag = (1<<j);
							if (buffer[pagestart + i] & (1<<j))
								opt->value = 1;
							else
								opt->value = 0;
							dln->Manuf.chksum += opt->page + opt->pos + opt->flag + opt->value;
							opt->next = dln->Manuf.options;
							dln->Manuf.options = opt;
						}
					}
					break;
			}
		}
		pagestart += pagelen + 2;
	}

	SortOptions(&(dln->Manuf));
	FreeMem(buffer, origlen);
	FreeMem(maskbuff, origlen);
}

/******************************************************\
*
*	FUNCTION:
*		BOOL VerifyChecksum( struct Window *win,
*									struct Manuf *man)
*
*	SYNOPSIS:
*		Calculates the actual checksum for a set of drive parameters
*		and compares them to the value in the Manuf struct. Returns
*		TRUE if the checksums are the same, FALSE if they differ
*	INPUT:
*		win - pointer to the parent window for the requester
*		man - pointer to the Manuf struct to be verified
*
*	RETURNS:
*		TRUE if the calculated checksum is equal to the read-in checksum
*		FALSE if they are different.
*
*	NEEDS:
*
*
\******************************************************/

BOOL VerifyChecksum(struct Window *win, struct Manuf *man)
{
	ULONG total;
	LONG reply;
	struct Options *opt;
	struct EasyStruct ESBad = 
		{
		sizeof(struct EasyStruct), 0, NULL,
		"Drive Data has bad checksum!\nThe file may have been damaged.",
		"Continue",
		};

	total = 0L;

	for(opt = man->options;opt != NULL;opt = opt->next)
		total += opt->page + opt->pos + opt->flag + opt->value;

	if(man->chksum != total)	
	{
		reply = EasyRequest(win, &ESBad, NULL);
		GT_BeginRefresh(win);
		GT_EndRefresh(win, TRUE);
		
		return(FALSE);
	}

	return(TRUE);
}

/******************************************************\
*
*	FUNCTION:
*		void AskTester( void )
*
*	SYNOPSIS:
*		Opens a requester window for the tester to input their name
*
*	INPUT:
*		none
*
*
*	RETURNS:
*		ad->ad_Tester is filled in with the current tester's name
*
*	NEEDS:
*
\******************************************************/

void AskTester( void ) 
{
	struct TextFont	*font;
	struct Screen		*scr;
	struct Window		*win;
	struct Gadget		*glist, *gadlist[MAX_GADS + 1];
	void 					*vi;
	int i;
	
	if (NULL == (font = OpenFont(&Topaz80)))
		ErrorMessage("Failed to Open Topaz 80");
	else
	{
		if (NULL == (scr = LockPubScreen(NULL)))
			ErrorMessage("Could not lock public screen");
		else
		{
			if (NULL ==(vi = GetVisualInfo(scr, TAG_END)))
				ErrorMessage("GetVisualInfo() failed");
			else
			{
				topborder = scr->WBorTop + (scr->Font->ta_YSize + 1);
				for(i=0;i <= MAX_GADS;i++)
					gadlist[i] = NULL;
				if (NULL == (CreateTesterGadgets(&glist, vi, gadlist)))
					ErrorMessage("CreateTesterGadgets() Failed");
				else
				{
					if (NULL ==(win = OpenWindowTags(NULL,
											WA_Title, 			FHDTVERSION,
											WA_Gadgets,			glist,
											WA_Left,				178,
											WA_Top,				56,
											WA_Width,			285,
											WA_Height,			78 + scr->Font->ta_YSize,
											WA_DragBar,			TRUE,
											WA_DepthGadget,	TRUE,
											WA_Activate,		TRUE,
											WA_CloseGadget,	TRUE,
											WA_SimpleRefresh,	TRUE,
											WA_IDCMP,			IDCMP_CLOSEWINDOW |
																	IDCMP_REFRESHWINDOW |
																	LISTVIEWIDCMP,
											WA_PubScreen,		scr,
											TAG_END)))
						ErrorMessage("OpenWindow() Failed");
					else
					{
						GT_RefreshWindow(win, NULL);
						ActivateGadget(gadlist[DSGAD_TESTERNAME], win, NULL);
						ProcessWindowEvents(win, gadlist, NULL);
						CloseWindow(win);
					}
				}
				FreeGadgets(glist);
				FreeVisualInfo(vi);
			}
			UnlockPubScreen(NULL, scr);
		}
		CloseFont(font);
	}
}

/******************************************************\
*
*	FUNCTION:
*		struct Gadget *CreateTesterGadgets( struct Gadget **glistptr, 
*														void *vi,
*														struct Gadget *gadlist[])
*
*	SYNOPSIS:
*		Creates all gadgets for the tester name requester window.
*
*	INPUT:
*		glistptr - pointer to a Gadget list to create
*		vi - pointer to a visual info structure from the parent window
*		gadlist - pointer to a gadlist array to be created
*
*	RETURNS:
*		gad - a pointer to the last gadget created. If this is null, 
*				then the function failed.
*		gadlist is filled in with all gadget definitions
*
*	NEEDS:
*
*
\******************************************************/

struct Gadget *CreateTesterGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[])
{
	struct NewGadget ng;
	struct Gadget *gad;

	gad = CreateContext(glistptr);

	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 20 + topborder;
	ng.ng_Width = 257;
	ng.ng_Height = 14;
	ng.ng_GadgetText = "Name of Tester";
	ng.ng_TextAttr = &Topaz80;
	ng.ng_VisualInfo = vi;
	ng.ng_GadgetID = DSGAD_TESTERNAME;
	ng.ng_Flags = PLACETEXT_ABOVE;
	gadlist[DSGAD_TESTERNAME] = gad = CreateGadget(STRING_KIND, gad, &ng,
									GTST_String,	ad->ad_Tester,
									TAG_END);	

	ng.ng_LeftEdge = 73;
	ng.ng_TopEdge = 58 + topborder;
	ng.ng_Width = 138;
	ng.ng_Height = 12;
	ng.ng_GadgetText = "Continue";
	ng.ng_GadgetID = DSGAD_TESTERCONT;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_TESTERCONT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);
	return(gad);
}

/******************************************************\
*
*	FUNCTION:
*		BOOL ReqFile(char *title, char *dirpart, char *filepart )
*
*	SYNOPSIS:
*		Puts up a file requester with a passed title bar, returning strings
*		for the directory and file parts of the selection.
*
*	INPUT:
*		title - string to be printed in titlebar of requester
*		dirpart - string containing initial directory name for requester
*		filepart - string containing initial file name for requester
*
*	RETURNS:
*		TRUE if user selected ok, FALSE if they selected cancel
*		if OK selected:
*			sets dirpart to the string containing the directory the user chose
*			sets filepart to the string containing the file the user chose
*
*	NEEDS:
*
*
\******************************************************/

BOOL ReqFile( char *title, char *dirpart, char *filepart )
{
	struct FileRequester *fr;
	BOOL ok = FALSE;

	if (AslBase = OpenLibrary("asl.library", 37L))
	{
		if (fr = (struct FileRequester *)AllocAslRequestTags(ASL_FileRequest,
																		ASL_Hail,		title,
																		ASL_Height,		180,
																		ASL_Width,		320,
																		ASL_LeftEdge,	160,
																		ASL_TopEdge,	10,
																		ASL_OKText,		"OK",
																		ASL_CancelText,"Cancel",
																		ASL_Dir,			dirpart,
																		ASL_File,		filepart,
																		TAG_DONE))
		{
			if (ok = AslRequest(fr, NULL))
			{	
				strncpy(dirpart, fr->rf_Dir,127);
				strncpy(filepart, fr->rf_File,63);
			}
		FreeAslRequest(fr);
		}	
		CloseLibrary(AslBase);
	}
	return(ok);
}

/******************************************************\
*
*	FUNCTION:
*		void UpdateFormatWindow(struct Window *win, 
*										struct Gadget *gadlist[], 
*										struct DLNode *dln,
*										char *gb)
*	
*	SYNOPSIS:
*		Updates gadgets on the FormatWindow with new info from the drive.
*		used mainly after performing a format operation.
*
*	INPUT:
*		win - Pointer to the window created by FormatWindow()
*		gadlist - Pointer to Gadget list from CreateFormatGadgets()
*		dln - pointer to the DLNode structure initialized with the values
*				for the drive to be formatted.
*		gb - pointer to a 256 byte buffer that is valid for the life of 
*				the FormatWindow() function to hold strings for gadgets
*
*	RETURNS:
*
*
*	NEEDS:
*
*
\******************************************************/

void UpdateFormatWindow(struct Window *win, struct Gadget *gadlist[], 
								struct DLNode *dln, char *gb)
{
	struct FlyerVolInfo info;
	ULONG  res;
	char temp[128];

	dln->FlyVol.Path = (char *)NULL;
	dln->FlyVol.Flags = FVF_USENUMS;
// These should have been initialized already...
//	dln->FlyVol.Board = 0;
//	dln->FlyVol.SCSIdrive = ad->ad_drive;

	info.len = sizeof(struct FlyerVolInfo);
	res = FlyerDriveInfo(&(dln->FlyVol),&info);
	if (res)
		StatusMessage("Error: could not get Flyer Drive information");

	res = ReadSize(&(dln->FlyVol), &(dln->size), &(dln->blklen));
 	if (res)
	{
		sprintf(temp, "ReadSize Error: %ld - %ls", res, Error2String(res));
		StatusMessage(temp);
	}

	strncpy(gb, dln->str, 28);
	gb[28] = '\0';
	GT_SetGadgetAttrs(gadlist[DSGAD_FDEVICE], win, NULL,
									GTTX_Text,	gb,
									TAG_END);

	strncpy(&(gb[29]), &(dln->str[28]), 4);
	gb[33] = '\0';
	GT_SetGadgetAttrs(gadlist[DSGAD_FREV], win, NULL,
									GTTX_Text,	&(gb[29]),
									TAG_END);

	if ((info.Ident == 0x524f4f54) && (info.Version == 1) && info.Title)
	{
		sprintf(&(gb[34]),"%.127s",info.Title);
	}
	else
	{
		gb[34] = '\0';
	}
	GT_SetGadgetAttrs(gadlist[DSGAD_FNAME], win, NULL,
									GTTX_Text,	&(gb[34]),
									TAG_END);

	if ((info.Ident == 0x524f4f54) && (info.Version == 1) && info.Title)
	{
		sprintf(ad->ad_VolNameStr, "%.127s", info.Title);
	}
	else
	{
		sprintf(ad->ad_VolNameStr, "Flyer%lc%lc", ad->ad_DriveStr[0], ad->ad_DriveStr[2]);
	}
	GT_SetGadgetAttrs(gadlist[DSGAD_VOLNAME], win, NULL,
									GTST_String,	ad->ad_VolNameStr,
									TAG_END);

	ad->ad_fmt_ss_block = ad->ad_ss_block;
	ad->ad_ss_val = ((float)ad->ad_fmt_ss_block/(float)dln->size) * 100.0;
	if (ad->ad_do_ss == 2)
		sprintf(ad->ad_ss_str,"0x%lx", ad->ad_fmt_ss_block);
	else	
		sprintf(ad->ad_ss_str,"%.4f%%", ad->ad_ss_val);
	GT_SetGadgetAttrs(gadlist[DSGAD_SSVAL], win, NULL,
									GTST_String,				ad->ad_ss_str,
									TAG_END);

	GT_SetGadgetAttrs(gadlist[DSGAD_TESTSS], win, NULL,
										GTTX_Text,				ad->ad_ss_str,
										TAG_END);
		
	if ((info.Ident == 0x524f4f54) && (info.Version == 1))
	{
		sprintf(&(gb[163]),"%6.1fG capacity, %5.1f%% full\0", 
						((double)(info.Blocks) * (double)(info.BlkSize)) / 1000000000.0,
						(double)(info.Blocks - info.BlksFree)/(double)info.Blocks * 100.0);
	}
	else
	{
		sprintf(&(gb[163]),"%6.1fG capacity, unformatted\0", 
						((double)dln->size * (double)dln->blklen) / 1000000000.0);
	}
	GT_SetGadgetAttrs(gadlist[DSGAD_FCAPACITY], win, NULL,
									GTTX_Text,	&(gb[163]),
									TAG_END);
}

/******************************************************\
*
*	FUNCTION:
*		BOOL CopyFlyerIcon( int drive )
*
*	SYNOPSIS:
*		Copies FlyerDrive.info to a flyer drive (mounted or unmounted)
*
*	INPUT:
*		drive - number of drive to copy icon to
*
*
*	RETURNS:
*		TRUE if copy succeeded, FALSE if it failed
*
*	NEEDS:
*		A toaster: assign to a newtek drawer containing
*			programs/flyer_support/flyerdisk.info
*
\******************************************************/


BOOL CopyFlyerIcon( int drive )
{
	BOOL success = FALSE;
	FILE *src;
	UBYTE Buffer[512];
	ULONG lenin, res;
	struct FlyerVolume FlyVol;
	struct ClipAction Action;
	ULONG fileid, actual;

	FlyVol.Board = 0;
	FlyVol.Flags = FVF_USENUMS;
	FlyVol.Path  = "disk.info";
	FlyVol.SCSIdrive = drive;

	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_STOPPED;
	Action.Grip = 0;
	Action.Access = FLYER_MODE_OUTPUT;

	if (src = fopen("Toaster:programs/flyer_support/FlyerDisk.info", "rb"))
	{
		if ((res = FlyerFileOpen(&Action)) == 0)

		{
			fileid = Action.FileID;
			FlyVol.Path = NULL;

			success = TRUE;
			while (!feof(src))
			{
				lenin = fread(Buffer, 1, 512, src);
				res = FlyerFileWrite(&FlyVol, fileid, lenin, Buffer, &actual);
				if ((actual < lenin) || ferror(src) || (res != 0))
				{
					success = FALSE;
					break;
				}
			}
			res = FlyerFileClose(&FlyVol, fileid);
			FlyerFreeGrip(&FlyVol, fileid);
		}
		fclose(src);
	}
	return(success);
}

/******************************************************\
*
*	FUNCTION:
*		void InitManufList( void )
*
*	SYNOPSIS:
*		Allocates memory for list and initializes main fields
*
*	INPUT:
*
*	RETURNS:
*		ad->ad_ManufList is a valid Exec list upon exit
*
*	NEEDS:
*
*
\******************************************************/

void InitManufList( void )
{
	if (!(ad->ad_ManufList = AllocMem(sizeof(struct List), MEMF_CLEAR)))
	{
		StatusMessage("Out of Memory!");
		return;
	}
	
	NewList(ad->ad_ManufList);
	
}

/******************************************************\
*
*	FUNCTION:
*		void AddManufNode( struct Manuf *man )
*
*	SYNOPSIS:
*		Inserts a Manuf node into a list, and asks for a 
*		comment if none exist
*
*	INPUT:
*		man - pointer to the Manuf node to insert into the list
*
*	RETURNS:
*
*	NEEDS:
*
*
\******************************************************/

void AddManufNode( struct Manuf *man )
{
	if (man != NULL)
	{
		sprintf(man->str, "%s%s%s %s",man->manufname,
												man->modelname,
												man->revision,
												man->comment);
		if (man->comment[0] == '\0')
		{
			AskComment(man);
			sprintf(man->str, "%s%s%s %s",man->manufname,
													man->modelname,
													man->revision,
													man->comment);
		}
		man->node.ln_Name = man->str;
		man->node.ln_Pri  = 0;
		man->node.ln_Type = 100;
		AddHead( ad->ad_ManufList, (struct Node *)man );
		SortList((struct List *)ad->ad_ManufList);
		IndexList(ad->ad_ManufList);
	}
}

/******************************************************\
*
*	FUNCTION:
*	void AskComment( struct Manuf *man )
*
*	SYNOPSIS:
*		Opens a requester window for inputting a comment for a set of params
*
*	INPUT:
*		none
*
*
*	RETURNS:
*		man->comment is filled in with the comment
*
*	NEEDS:
*
\******************************************************/

void AskComment( struct Manuf *man )
{
	struct TextFont	*font;
	struct Screen		*scr;
	struct Window		*win;
	struct Gadget		*glist, *gadlist[MAX_GADS + 1];
	void 					*vi;
	int i;
	
	if (man != NULL)
	{
		if (NULL == (font = OpenFont(&Topaz80)))
			ErrorMessage("Failed to Open Topaz 80");
		else
		{
			if (NULL == (scr = LockPubScreen(NULL)))
				ErrorMessage("Could not lock public screen");
			else
			{
				if (NULL ==(vi = GetVisualInfo(scr, TAG_END)))
					ErrorMessage("GetVisualInfo() failed");
				else
				{
					topborder = scr->WBorTop + (scr->Font->ta_YSize + 1);
					for(i=0;i <= MAX_GADS;i++)
						gadlist[i] = NULL;
					if (NULL == (CreateCommentGadgets(&glist, vi, gadlist, man)))
						ErrorMessage("CreateCommentGadgets() Failed");
					else
					{
						if (NULL ==(win = OpenWindowTags(NULL,
												WA_Title, 			FHDTVERSION,
												WA_Gadgets,			glist,
												WA_Left,				178,
												WA_Top,				56,
												WA_Width,			285,
												WA_Height,			94 + scr->Font->ta_YSize,
												WA_DragBar,			TRUE,
												WA_DepthGadget,	TRUE,
												WA_Activate,		TRUE,
												WA_CloseGadget,	TRUE,
												WA_SimpleRefresh,	TRUE,
												WA_IDCMP,			IDCMP_CLOSEWINDOW |
																		IDCMP_REFRESHWINDOW |
																		LISTVIEWIDCMP,
												WA_PubScreen,		scr,
												TAG_END)))
							ErrorMessage("OpenWindow() Failed");
						else
						{
							GT_RefreshWindow(win, NULL);
							ActivateGadget(gadlist[DSGAD_COMMENT], win, NULL);
							ProcessWindowEvents(win, gadlist, NULL);
							strncpy(man->comment, ad->ad_tempstr, 32);
							CloseWindow(win);
						}
					}
					FreeGadgets(glist);
					FreeVisualInfo(vi);
				}
				UnlockPubScreen(NULL, scr);
			}
			CloseFont(font);
		}
	}
		
}

/******************************************************\
*
*	FUNCTION:
*		struct Gadget *CreateCommentGadgets( struct Gadget **glistptr, 
*														void *vi,
*														struct Gadget *gadlist[]
*														struct Manuf *man)
*
*	SYNOPSIS:
*		Creates all gadgets for the comment requester window.
*
*	INPUT:
*		glistptr - pointer to a Gadget list to create
*		vi - pointer to a visual info structure from the parent window
*		gadlist - pointer to a gadlist array to be created
*		man - pointer to the Manuf struct to edit the comment for
*
*	RETURNS:
*		gad - a pointer to the last gadget created. If this is null, 
*				then the function failed.
*		gadlist is filled in with all gadget definitions
*
*	NEEDS:
*
*
\******************************************************/

struct Gadget *CreateCommentGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[], struct Manuf *man)
{
	struct NewGadget ng;
	struct Gadget *gad;
	char temp;

	gad = CreateContext(glistptr);

	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 20 + topborder;
	ng.ng_Width = 257;
	ng.ng_Height = 12;
	ng.ng_VisualInfo = vi;
	ng.ng_TextAttr = &Topaz80;
	ng.ng_GadgetText = "Enter a comment for settings for:";
	ng.ng_GadgetID = DSGAD_COMMLABEL;
	ng.ng_Flags = PLACETEXT_ABOVE;

	temp = man->str[28];
	man->str[28] = '\0';
	gadlist[DSGAD_COMMLABEL] = gad = CreateGadget(TEXT_KIND, gad, &ng,
									GTTX_CopyText,		TRUE,
									GTTX_Text,			man->str,
									TAG_END);	
	man->str[28] = temp;
	

	ng.ng_TopEdge = 36 + topborder;
	ng.ng_Height = 14;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_COMMENT;
	gadlist[DSGAD_COMMENT] = gad = CreateGadget(STRING_KIND, gad, &ng,
											GTST_String,	man->comment,
											TAG_END);	

	ng.ng_LeftEdge = 73;
	ng.ng_TopEdge = 74 + topborder;
	ng.ng_Width = 138;
	ng.ng_Height = 12;
	ng.ng_GadgetText = "Continue";
	ng.ng_GadgetID = DSGAD_COMMCONT;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_COMMCONT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);
	return(gad);
}

/******************************************************\
*
*	FUNCTION:
*		void SetCacheWindow( void )
*
*	SYNOPSIS:
*		Creates Cache Options window and attaches gadgets
*
*	INPUT:
*		none
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void SetCacheWindow( void )
{
	struct TextFont	*font;
	struct Screen		*scr;
	struct Window		*win;
	struct Gadget		*glist, *gadlist[MAX_GADS + 1];
	void 					*vi;
	int i;

	if (NULL == (font = OpenFont(&Topaz80)))
		ErrorMessage("Failed to Open Topaz 80");
	else
	{
		if (NULL == (scr = LockPubScreen(NULL)))
			ErrorMessage("Could not lock public screen");
		else
		{
			if (NULL ==(vi = GetVisualInfo(scr, TAG_END)))
				ErrorMessage("GetVisualInfo() failed");
			else
			{
				topborder = scr->WBorTop + (scr->Font->ta_YSize + 1);
				for(i=0;i <= MAX_GADS;i++)
					gadlist[i] = NULL;
				if ((CreateSetCacheGadgets(&glist, vi, gadlist)) == NULL)
					ErrorMessage("CreateSetCacheGadgets() Failed");
				else
				{
					if (NULL ==(win = OpenWindowTags(NULL,
											WA_Title, 			FHDTVERSION,
											WA_Gadgets,			glist,
											WA_Left,				35,
											WA_Top,				13,
											WA_Width,			570,
											WA_Height,			192 + scr->Font->ta_YSize,
											WA_DragBar,			TRUE,
											WA_DepthGadget,	TRUE,
											WA_Activate,		TRUE,
											WA_CloseGadget,	TRUE,
											WA_SimpleRefresh,	TRUE,
											WA_IDCMP,			IDCMP_CLOSEWINDOW |
																	IDCMP_REFRESHWINDOW |
																	LISTVIEWIDCMP,
											WA_PubScreen,		scr,
											TAG_END)))
						ErrorMessage("OpenWindow() Failed");
					else
					{
						GT_RefreshWindow(win, NULL);
						if (ad->ad_SelCacheIndex)
						{
							struct Manuf *tempman;

							GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
														GTLV_Selected,	ad->ad_SelCacheIndex - 1,
														TAG_END);
							tempman = FindIndex(ad->ad_ManufList, ad->ad_SelCacheIndex);
							if (tempman != NULL)
								sprintf(ad->ad_CacheStr, "%-.79s", tempman->str); 
							else
								ad->ad_CacheStr[0]= '\0'; 
							GT_SetGadgetAttrs(gadlist[DSGAD_SELCACHE], win, NULL,
													GTTX_Text,	ad->ad_CacheStr,
													TAG_END);
						}
						else
						{
							GT_SetGadgetAttrs(gadlist[DSGAD_MANUFLIST], win, NULL,
														GTLV_Selected,	-1,
														TAG_END);
							ad->ad_CacheStr[0]= '\0'; 
							GT_SetGadgetAttrs(gadlist[DSGAD_SELCACHE], win, NULL,
														GTTX_Text,	ad->ad_CacheStr,
														TAG_END);

						}
						ProcessWindowEvents(win, gadlist, NULL); 
						CloseWindow(win);
					}
				}
				FreeGadgets(glist);
				FreeVisualInfo(vi);
			}
			UnlockPubScreen(NULL, scr);
		}
		CloseFont(font);
	}
}

/******************************************************\
*
*	FUNCTION:
*		struct Gadget *CreateSetCacheGadgets(struct Gadget **glistptr, 
*													void *vi,
*													struct Gadget *gadlist[])
*
*	SYNOPSIS:
*		Creates all gadgets for the SetCache window
*
*	INPUT:
*		glistptr - pointer to the gadget list to create
*		vi - pointer to the visualinfo struct
*		gadlist - pointer to the gadget list to fill in
*
*	RETURNS:
*		a pointer to the last gadget created, NULL if there was an error
*	
*	NEEDS:
*
\******************************************************/

struct Gadget *CreateSetCacheGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[])
{
	struct NewGadget ng;
	struct Gadget *gad;

	gad = CreateContext(glistptr);

	sprintf(ad->ad_SetCacheLabelStr, "Current Drive: %-53.53s", ad->ad_DriveInfo[ad->ad_SelDriveIndex]->str); 

	ng.ng_TextAttr = &Topaz80;
	ng.ng_VisualInfo = vi;
	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 18 + topborder;
	ng.ng_Width = 542;
	ng.ng_Height = 12;
	ng.ng_GadgetText = ad->ad_SetCacheLabelStr;
	ng.ng_Flags = PLACETEXT_ABOVE | NG_HIGHLABEL;
	ng.ng_GadgetID = DSGAD_MANUFLABEL;
	gadlist[DSGAD_MANUFLABEL] = gad = CreateGadget( TEXT_KIND, gad, &ng,
																	GTTX_Text,	 "",
																	GTTX_Border, FALSE,
																	TAG_END);	

	ng.ng_TopEdge = 30 + topborder;
	ng.ng_Height = 73;
	ng.ng_GadgetText = "Available Cache Settings";
	ng.ng_Flags = PLACETEXT_ABOVE;
	ng.ng_GadgetID = DSGAD_MANUFLIST;
	if (ad->ad_GadLib == 39)
		gadlist[DSGAD_MANUFLIST] = gad = CreateGadget(LISTVIEW_KIND, gad, &ng,
										GTLV_Labels,			ad->ad_ManufList,
										GTLV_ShowSelected,	NULL,
										TAG_END);
	else
		gadlist[DSGAD_MANUFLIST] = gad = CreateGadget(LISTVIEW_KIND, gad, &ng,
										GTLV_Labels,			ad->ad_ManufList,
										TAG_END);

	ng.ng_TopEdge = topborder + 97;
	ng.ng_Height = 14;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_SELCACHE;
	gadlist[DSGAD_SELCACHE] = gad = CreateGadget(TEXT_KIND, gad, &ng,
										GTTX_Border, TRUE,
										TAG_END);	

	ng.ng_Width = 175;
	ng.ng_Height = 12;

	ng.ng_TopEdge = 113 + topborder;
	ng.ng_LeftEdge = 14;
	ng.ng_GadgetText = "Read From File...";
	ng.ng_GadgetID = DSGAD_READDATA;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_READDATA] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);
	ng.ng_LeftEdge = 198;
	ng.ng_GadgetText = "View Selected...";
	ng.ng_GadgetID = DSGAD_VIEWSELCACHE;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_VIEWSELCACHE] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 382;
	ng.ng_GadgetText = "Write To File...";
	ng.ng_GadgetID = DSGAD_WRITECACHE;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_WRITECACHE] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);


	ng.ng_TopEdge = 128 + topborder;
	ng.ng_LeftEdge = 14;
	ng.ng_GadgetText = "Edit Comment...";
	ng.ng_GadgetID = DSGAD_EDITCOMMENT;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_EDITCOMMENT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 198;
	ng.ng_GadgetText = "Copy Selected";
	ng.ng_GadgetID = DSGAD_COPYCACHE;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_COPYCACHE] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 382;
	ng.ng_GadgetText = "Delete Selected";
	ng.ng_GadgetID = DSGAD_DELCACHE;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_DELCACHE] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_TopEdge = 173 + topborder;

	ng.ng_LeftEdge = 14;
	ng.ng_GadgetText = "Set Drive Cache";
	ng.ng_GadgetID = DSGAD_SETCACHE;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_SETCACHE] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 198;
	ng.ng_GadgetText = "Save Config File";
	ng.ng_GadgetID = DSGAD_SAVEDATA;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_SAVEDATA] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 382;
	ng.ng_GadgetText = "Main Screen";
	ng.ng_GadgetID = DSGAD_EXIT;
	gadlist[DSGAD_EXIT] = gad = CreateGadget( BUTTON_KIND, gad, &ng, 
															TAG_END);


	return(gad);
}

/******************************************************\
*
*	FUNCTION:
*		void ErrorReq( struct Window *win , char *msg)
*
*	SYNOPSIS:
*		Displays a requester with the passed string displayed
*		and waits for the user to click OK
*
*	INPUT:
*		win - pointer to the parent window for the requester
*		msg - pointer to the string to display to the user
*
*	RETURNS:
*
*	NEEDS:
*
\******************************************************/

void ErrorReq(struct Window *win , char *msg)
{
	LONG reply;
	struct EasyStruct ESWrite = 
		{
		sizeof(struct EasyStruct), 0, FHDTVERSION,
				NULL,
				"OK",
		};

	ESWrite.es_TextFormat = msg;
	reply = EasyRequest(win, &ESWrite, NULL );
	GT_BeginRefresh(win);
	GT_EndRefresh(win, TRUE);
	
}

/******************************************************\
*
*	FUNCTION:
*		void IndexList( struct List *list )
*
*	SYNOPSIS:
*		Given a list, will set the index fields of the nodes in a
*		zero based, sequential order
*
*	INPUT:
*		list - pointer to the list of Manuf nodes to be indexed
*
*	RETURNS:
*
*	NEEDS:
*
\******************************************************/

void IndexList( struct List *list)
{
	struct Node *node;
	int i;

	for(i=1,node = list->lh_Head;node->ln_Succ != NULL;node = node->ln_Succ)
		((struct Manuf *)(node))->index = i++;
}

/******************************************************\
*
*	FUNCTION:
*		struct Manuf *FindIndex( struct List *list , int index)
*
*	SYNOPSIS:
*		Given a list, will find the Manuf node with the index 
*		field equal to index, and return a pointer to the node
*
*	INPUT:
*		list - pointer to the list of Manuf nodes to be searched
*		index - index to find
*
*	RETURNS:
*
*	NEEDS:
*
\******************************************************/

struct Manuf *FindIndex( struct List *list, int index)
{
	struct Node *node;
	int i;

	for(i=0,node = list->lh_Head;node->ln_Succ != NULL;node = node->ln_Succ)
		if (((struct Manuf *)(node))->index == index)
			break;

	if (node->ln_Succ == NULL)
		return(NULL);
	else
		return((struct Manuf *)node);
}


void ReadDriveCache(struct DLNode *dln)
{
	struct Options *opt, *opt2;
	struct Manuf *tempman;

	FetchCacheSettings(dln);

	tempman = AllocMem(sizeof(struct Manuf), MEMF_CLEAR);
	strncpy(tempman->manufname, dln->Manuf.manufname, 8);
	strncpy(tempman->modelname, dln->Manuf.modelname, 16);
	strncpy(tempman->revision,  dln->Manuf.revision,  4);
	tempman->blocks = dln->Manuf.blocks;
	tempman->options = NULL;
	tempman->chksum = dln->Manuf.chksum;

	opt = dln->Manuf.options;
	while (opt != NULL)
	{
		if ((opt2 = AllocMem(sizeof(struct Options), MEMF_CLEAR)) == NULL)
		{
			StatusMessage("Out of Memory!");
			break;
		}
		strncpy(opt2->optname, opt->optname, 15);
		opt2->next  = tempman->options;
		tempman->options = opt2;
		opt2->page  = opt->page;
		opt2->pos   = opt->pos;
		opt2->flag  = opt->flag;
		opt2->value = opt->value;
			
		opt = opt->next;
	}
	SortOptions(tempman);
	AddManufNode(tempman);
}

void CopyManufNode( struct Manuf *man)
{
	struct Manuf *newman;
	struct Options *opt, *newopt;

	newman = AllocMem(sizeof(struct Manuf), MEMF_CLEAR);
	strncpy(newman->manufname, man->manufname, 8);
	strncpy(newman->modelname, man->modelname, 16);
	strncpy(newman->revision, man->revision, 4);
	strncpy(newman->comment, man->comment, 32);
	newman->blocks = man->blocks;
	newman->options = NULL;	
	for(opt = man->options;opt != NULL;opt = opt->next)
	{
		newopt = newman->options;
		newman->options = AllocMem(sizeof(struct Options), MEMF_CLEAR);
		newman->options->next = newopt;
		strncpy(newman->options->optname, opt->optname, 15);
		newman->options->page = opt->page;
		newman->options->pos = opt->pos;
		newman->options->flag = opt->flag;
		newman->options->value = opt->value;
	}
							//{
	SortOptions(newman);
	AddManufNode(newman);
}

void DeleteManufNode(struct Manuf *man)
{
	struct Options *opt;

	if (man != NULL)
	{
		while(man->options != NULL)
		{
			opt = man->options->next;
			FreeMem(man->options, sizeof(struct Options));
			man->options = opt;
		}
		Remove((struct Node *)man);
		FreeMem(man, sizeof(struct Manuf));
	}
}

void SortOptions(struct Manuf *man)
{
	struct Options *current, *previous, *optlist, *toinsert;
	UWORD value;

	if (man != NULL)
	{
		if (man->options != NULL)
		{
			toinsert = man->options->next;
			man->options->next = NULL;
			
			while (toinsert)
			{
				optlist = toinsert->next;
				value = ((ULONG)(toinsert->page) << 8) + toinsert->pos;
				previous = NULL;
				current = man->options;
				
				while (current != NULL)
				{
					if (value <= (((ULONG)(current->page) << 8) + current->pos))
					{
						if (previous == NULL)
							man->options = toinsert;
						else
							previous->next = toinsert;
						toinsert->next = current;
						toinsert = NULL;
						break;
					}
					previous = current;
					current = current->next;
				}
				if (current == NULL)
				{
					previous->next = toinsert;
					toinsert->next = NULL;
				}
				toinsert = optlist;
			}
		}
	}
}

struct Manuf *ParamsExist(struct DLNode *dln)
{
	struct Manuf *man;
	struct Options *dlnopt, *manopt;

	for(man = (struct Manuf *)(ad->ad_ManufList->lh_Head);man != NULL; man = (struct Manuf *)(man->node.ln_Succ))
	{
		if (!(stricmp(man->manufname, dln->Manuf.manufname) || 
				stricmp(man->modelname, dln->Manuf.modelname) ||
			 	stricmp(man->revision,  dln->Manuf.revision)))
		{
			for( dlnopt = dln->Manuf.options, manopt = man->options;
				((dlnopt != NULL) && (manopt != NULL));
				dlnopt = dlnopt->next, manopt = manopt->next)
			{
				if ((dlnopt->page  != manopt->page ) || 
					 (dlnopt->pos 	!= manopt->pos  ) ||
					 (dlnopt->flag 	!= manopt->flag ) ||
					 (dlnopt->value != manopt->value))
				{
					break;
				}
					
			}
			if ((dlnopt == NULL) && (manopt == NULL))
				return(man);
					
		}
	}
	return(NULL);
}


void SortList(struct List *list)
{
	int swaps;
	struct Node *current;

	if (list != NULL)
	{
		do
		{
			swaps = 0;

			for(current = list->lh_Head;
				((current != list->lh_TailPred) && (current->ln_Succ != NULL));
				current = current->ln_Succ)
			{
				if (stricmp(current->ln_Name, current->ln_Succ->ln_Name) > 0)
				{
					swaps++;
					current->ln_Pred->ln_Succ = current->ln_Succ;
					current->ln_Succ->ln_Pred = current->ln_Pred;
					current->ln_Pred = current->ln_Succ;
					current->ln_Succ = current->ln_Succ->ln_Succ;
					current->ln_Pred->ln_Succ = current;
					current->ln_Succ->ln_Pred = current;
				}
			}	
		} while (swaps != 0);
	}
}
