/*********************************************************************\
*
* MakeDatFiles 1.0 - Compile bin files for chips into dat files
*							and create appropriate chipset.dat file
*
* Copyright (c) 1994,1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* 10-31-96		Todd		Created File
*
\*********************************************************************/

/* INCLUDES */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <clib/dos_protos.h>
#include <clib/exec_protos.h>
#include <dos/dos.h>
#include <exec/types.h>
#include <exec/memory.h>
#include <proto/exec.h>

#define FLYER_DAT		0
#define DSP_DAT		1	
#define ME_DAT			2
#define PE_DAT			3
#define MD_DAT			4
#define PD_DAT			5
#define SK_DAT			6
#define TR_DAT			7
#define DMA_DAT		8
#define AUD_DAT		9
#define AL_DAT			10

struct id_node
{
	struct id_node *prev;
	struct id_node *next;
	char 				 id[5];
};
	
void main( int argc, char *argv[] )
{
	FILE *in_file = NULL, *out_file = NULL;
	ULONG 	res;
	char 	chipID[32] = "";		// A little extra space so that we can be forgiving
	ULONG	lo_bound, hi_bound;
	int	end_of_ruleset = 0;
	struct id_node *id_list = NULL, *temp = NULL;
	int i;
	ULONG chunk_length;
	char file_list[12][32] =
		{ 	"Flyer",
			"DSP",
			"ME",
			"PE",
			"MD",
			"PD",
			"SK",
			"TR",
			"DMA",
			"AUD",
			"AL",
			""
		};

	if ( argc != 3 )
	{
		printf("Usage: %s <input file> <output file>\n", argv[0] );
		exit(10);
	}

	if( in_file = fopen( argv[1], "r" ) )
	{
		if ( out_file = fopen( argv[2], "wb" ) )
		{
			do
			{
				memset( chipID, '\0', 5 );

				res = fscanf( in_file, "%s", chipID );			
				if ( res == EOF )
				{
					if ( end_of_ruleset )
					{
						printf("Finished creating %s\n", argv[2] );
					}
					else
					{
						printf("Error: Not a valid rule file!\n");
					}
					break;
				}
				end_of_ruleset = 0;
								
				temp = AllocMem( sizeof( struct id_node ), MEMF_PUBLIC | MEMF_CLEAR );
				strncpy( temp->id, chipID, 4 );

				temp->prev = NULL;
				temp->next = id_list;
				id_list = temp;
				

				fwrite( &chipID, 1, 4, out_file );

				do
				{
					res = fscanf( in_file, "%ld %ld", &lo_bound, &hi_bound );
					if ( res != 2 )
					{
						break;
					}
					fwrite( &lo_bound, 4, 1, out_file );
					fwrite( &hi_bound, 4, 1, out_file );

					if ( hi_bound == 0 )
					{
						end_of_ruleset = 1;
						break;
					}
				} while( !end_of_ruleset );	
			} while( TRUE );
		
			fclose( out_file );
		}

		fclose( in_file );
	}	

	for( i = 0; i < 11 ; i++ )
	{
		BPTR	bin_file, dat_file;
		char bin_filename[64];
		char dat_filename[64];
		
		sprintf( dat_filename, "%s.dat", file_list[i] );

		if( dat_file = Open( dat_filename, MODE_NEWFILE ) )
		{
			for( temp = id_list ; temp != NULL ; temp = temp->next )
			{
				sprintf( bin_filename, "%s/%s.bin", temp->id, file_list[i] );

				if ( bin_file = Open( bin_filename, MODE_OLDFILE ) )
				{
					UBYTE *buffer;
					// add new chunk to .dat file
					chunk_length = Seek( bin_file, 0, OFFSET_END );
					chunk_length = Seek( bin_file, 0, OFFSET_BEGINNING );
					buffer = AllocMem( chunk_length, MEMF_PUBLIC | MEMF_CLEAR );
					if ( buffer )
					{
						res = Read( bin_file, buffer, chunk_length );
						if ( res != chunk_length )
						{
							printf( "Error reading file: %s\n", bin_filename );
						}
						else
						{
							res = Write( dat_file, temp->id, 4 );
							if ( res != 4 )
							{
								printf("Error writing to file: %s\n", dat_file );
							}
							else
							{
								res = Write( dat_file, &chunk_length, 4 );

								if ( res != 4 )
								{
									printf("Error writing to file: %s\n", dat_file );
								}
								else
								{
									res = Write( dat_file, buffer, chunk_length );
									if ( res != chunk_length )
									{
										printf("Error writing to file: %s\n", dat_file );
									}
								}
							}
						}
						FreeMem( buffer, chunk_length );
						chunk_length = 0;
					}
					else
					{
						printf("Error! Could not get memory for buffer!\n");
					}
					Close( bin_file );
				}
			}
			chunk_length = 0;
			Write( dat_file, &chunk_length, 4 );	
			Close( dat_file );
		}
	}
}
