/*********************************************************************\
*
* MakeFPGA - Converts .hex FPGA files to Flyer binary format
*
* $Id: MakeFPGA.c,v 1.0 1995/05/05 15:51:47 Flick Exp $
*
* $Log: MakeFPGA.c,v $
*Revision 1.0  1995/05/05  15:51:47  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	10/22/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <exec/libraries.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyer.h"
#include "flyerprivate.h"

ULONG ReadHexFile(char *);
ULONG ASCIItoBinary(UBYTE *,ULONG);
void FreeFile(void);
void GetArg(char *arg,int *var,int lo,int hi,char *label);

UBYTE	*Buffer;
ULONG	Length;


main(int argc,char **argv)
{
	int	chipnum,retcode=0,defnum,chiprev;
	LONG	size,speed,id;
	FILE	*fp;
	struct	FlyerChipHeader	hdr;

	if ((argc!=8)||(argv[1][0]=='?')) {
		printf("Usage: %ls <chip> <def> <id> <rev> <speed> <hexfile> <binfile>\n",argv[0]);
		printf("Chips:\n");
		printf(" 1) U42 - Clock Skew\n");
		printf(" 2) U38 - Timer/Router\n");
		printf(" 7) U92 - DMA Controller\n");
		printf(" 8) U1  - Audio Control\n");
		printf(" 9) U25 - Aligner\n");
		printf("\nDefs:\n");
		printf(" 0) Mencoder\n");
		printf(" 1) Pencoder\n");
		printf(" 2) Mdecoder\n");
		printf(" 3) Pdecoder\n");
		return(20);
	}

	GetArg(argv[1],&chipnum,0,9,"Chip number");
	GetArg(argv[2],&defnum,0,3,"Definition number");

	stch_l(argv[3],&id);
	GetArg(argv[4],&chiprev,0,0xFF,"Chip revision");
	stcd_l(argv[5],&speed);

	size = ReadHexFile(argv[6]);
	if (size) {
		fp = fopen(argv[7],"w");
		if (fp) {
			hdr.FlyerID = 0x464C5952;		/* "FLYR" */
			hdr.chipnum = chipnum;
			hdr.defnum	= defnum;
			hdr.chiprev = chiprev;
			hdr.boardrev = 40;		/* 4.0 */
			hdr.id		= id;
			hdr.speed  = speed;
			hdr.length = size;

			if (fwrite(&hdr,sizeof(struct FlyerChipHeader),1,fp)!=1)
				printf("Write error\n");
			if (fwrite(Buffer,size,1,fp)!=1)
				printf("Write error\n");
			fclose(fp);
		}
	}
	FreeFile();

	return(retcode);
}


ULONG ReadHexFile(char *name)
{
	char	filename[80];
	FILE	*fp;
	int	read;

	strcpy(filename,name);
	if (stricmp(".HEX",&filename[strlen(filename)-4]))
		strcat(filename,".HEX");

	read = 0;
	fp = fopen(filename,"r");
	if (fp) {
		fseek(fp,0,SEEK_END);
		Length = ftell(fp);
		fseek(fp,0,SEEK_SET);

		Buffer = AllocMem(Length,MEMF_PUBLIC);
		if (Buffer) {
			read = fread(Buffer,Length,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",filename);

	if (read==1)
		return(ASCIItoBinary(Buffer,Length));
	else
		return(0);
}


ULONG ASCIItoBinary(UBYTE *buff,ULONG len)
{
	char 		*from,c;
	UBYTE 	*to,v,vt;
	ULONG		binlen;
	int		state;

	binlen = 0;
	from = (char *)buff;
	to = buff;
	state = 0;

	while (len) {
		c = *from++;
		c = toupper(c);
		if (isxdigit(c)) {
			if (c>'9')
				v = ((UBYTE)c)-'A'+10;
			else
				v = ((UBYTE)c)-'0';
			if (state==0) {
				vt = 16*v;
				state=1;
			}
			else {
				vt += v;
				state=0;
				*to++ = vt;
				binlen++;
			}
		}
		len--;
	}

	return(binlen);
}


void FreeFile(void)
{
	if (Length)
		FreeMem(Buffer,Length);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
