/*********************************************************************\
*
* PatchMapRam - Write MapRam table
*
* $Id: PatchMapRam.c,v 1.0 1995/05/05 15:52:06 Flick Exp $
*
* $Log: PatchMapRam.c,v $
*Revision 1.0  1995/05/05  15:52:06  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	11/01/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

struct Library *FlyerBase;

ULONG ReadHexFile(char *);
void FreeFile(void);
ULONG ASCIItoBinary(UBYTE *,ULONG);
void GetArg(char *arg,int *var,int lo,int hi,char *label);

#define	BINSIZE	4096

UBYTE	*Buffer;
ULONG	Length;

main(int argc,char **argv)
{
	int	i,lowaddr,highaddr,bank,unit;
	ULONG	res,size;

	if ((argc!=3)||(argv[1][0]=='?')) {
		printf("Usage: %ls <file> <bank>\n",argv[0]);
		return(20);
	}

	unit = 0;

	GetArg(argv[2],&bank,0,7,"Bank number");

	printf("Writing bank %ld\n",bank);

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	size = ReadHexFile(argv[1]);
	if (size) {
		res = SBusWrite(unit,0x7F,3);		/* Put in pgm mode */
		highaddr = bank<<4;
		res = SBusWrite(unit,0x7D,highaddr);			/* Set low addr */

		printf("Starting at high address $%lx\n",highaddr);

		lowaddr = 0;
		for (i=0;i<BINSIZE;i++) {
			res = SBusWrite(unit,0x7C,lowaddr);		/* Set low addr */
			res = SBusWrite(unit,0x7E,Buffer[i]);	/* Data */
			lowaddr++;
			if (lowaddr == 256) {
				lowaddr = 0;
				highaddr++;
				res = SBusWrite(unit,0x7D,highaddr);		/* Set hi addr */
			}
		}
		res = SBusWrite(unit,0x7F,0);		/* Put in normal mode */
	}

	printf("Done\n");

	CloseLibrary(FlyerBase);
	FreeFile();

	return(0);
}


ULONG ReadHexFile(char *name)
{
	char	filename[80];
	FILE	*fp;
	int	read;
	ULONG	size;

	strcpy(filename,name);
	if (stricmp(".HEX",&filename[strlen(filename)-4]))
		strcat(filename,".HEX");

	read = 0;
	fp = fopen(filename,"r");
	if (fp) {
		fseek(fp,0,SEEK_END);
		Length = ftell(fp);
		fseek(fp,0,SEEK_SET);

		Buffer = AllocMem(Length,MEMF_PUBLIC);
		if (Buffer) {
			read = fread(Buffer,Length,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",filename);

	if (read==1) {
		size = ASCIItoBinary(Buffer,Length);
		return(size);
	}
	else
		return(0);
}


ULONG ASCIItoBinary(UBYTE *buff,ULONG len)
{
	char 		*from,c;
	UBYTE 	*to,v,vt;
	ULONG		binlen;
	int		state;

	binlen = 0;
	from = (char *)buff;
	to = buff;
	state = 0;

	while (len) {
		c = *from++;
		c = toupper(c);
		if (isxdigit(c)) {
			if (c>'9')
				v = ((UBYTE)c)-'A'+10;
			else
				v = ((UBYTE)c)-'0';
			if (state==0) {
				vt = 16*v;
				state=1;
			}
			else {
				vt += v;
				state=0;
				*to++ = vt;
				binlen++;
			}
		}
		len--;
	}

	return(binlen);
}


void FreeFile(void)
{
	if (Length)
		FreeMem(Buffer,Length);
}

void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
