/********************************************************************
* $prophelp.c$
* $Id: PropHelp.c,v 2.1 1994/11/16 14:35:12 CACHELIN4000 Exp $
* $Log: PropHelp.c,v $
*Revision 2.1  1994/11/16  14:35:12  CACHELIN4000
**** empty log message ***
*
*Revision 2.0  1994/02/17  16:24:03  Kell
*FirstCheckIn
*
*Revision 2.0  94/02/17  15:57:06  Kell
*FirstCheckIn
*
*Revision 2.0  94/02/17  14:44:19  Kell
*FirstCheckIn
*
*
* Copyright (c)1992 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	12-17-92	Steve H		Convert to use SmartStrings
*	12-17-92	Steve H		Last Update
*
*********************************************************************/
#include <exec/types.h>
#include <exec/memory.h>
#include <intuition/intuition.h>
#include <stdio.h>
#include <string.h>
#include <prophelp.h>

#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>

#ifndef PROTO_PASS
#include <prophelp.p>
#endif

// #define PROP_DEBUG

#ifdef CUSTOM_BODY
#include <customprop.p>
#endif
extern struct IntuitionBase *IntuitionBase;

char PortName[] = "PropHelpTimerPort0";

/****** PropHelp/InitPropHelp ***************************************
*
*   NAME   
*	InitPropHelp
*
*   SYNOPSIS
*	struct PropHelp *InitPropHelp(VOID)
*
*   FUNCTION
*	Each task wanting to use the PropHelp routines should call
*	this routine once at startup. The PropHelp structure returned
*	should then be passed to the other PropHelp routines.
*
*	
*	Gets preferences key repeat values (used for down/up arrows)
*	Allocates (struct PropHelp *) for use by other routines
*
*********************************************************************
*/
struct PropHelp *InitPropHelp(VOID)
{
	struct PropHelp *PropHelp;
	UBYTE PrefBuff[20];
	struct Preferences *Prefs;

	if (!MakePortNameUnique(PortName)) return(NULL);
	if (PropHelp = AllocMem(sizeof(struct PropHelp),MEMF_CLEAR)) {

	if ((PropHelp->TimerReplyPort=CreatePort(PortName,0))
	&& (PropHelp->TimeRequest=(struct timerequest *)CreateExtIO(
		PropHelp->TimerReplyPort,sizeof(struct timerequest)))
	&& (!(OpenDevice(TIMERNAME,UNIT_MICROHZ,(struct IORequest *)
		PropHelp->TimeRequest,0L)))) {

		PropHelp->TimerBase = (struct Library *)PropHelp->TimeRequest->
			tr_node.io_Device;
		PropHelp->TimeRequest->tr_node.io_Command = TR_ADDREQUEST;

		Prefs = GetPrefs((struct Preferences *)&PrefBuff[0],20);
		PropHelp->KeyRptSpeed.tv_secs = Prefs->KeyRptSpeed.tv_secs;
		PropHelp->KeyRptSpeed.tv_micro = Prefs->KeyRptSpeed.tv_micro;
		PropHelp->KeyRptDelay.tv_secs = Prefs->KeyRptDelay.tv_secs;
		PropHelp->KeyRptDelay.tv_micro = Prefs->KeyRptDelay.tv_micro;
	} else {
		FreePropHelp(PropHelp); // free partially initialized PropHelp
		PropHelp = NULL;
	}

	}
	return(PropHelp);
}

//*******************************************************************
// allows up to 10 names before failing
BOOL MakePortNameUnique(char Name[])
{
	UWORD Length;
	BOOL Result = TRUE;

	Length = strlen(Name);
	if (Length) {
		Name[Length-1] = '0';
		while (FindPort(Name)) {
			Name[Length-1] += 1;
			if (Name[Length-1] == ':') return(FALSE);
		}
	} else Result = FALSE;
	return(Result);
}

/****** PropHelp/FreePropHelp ***************************************
*
*   NAME   
*	FreePropHelp
*
*   SYNOPSIS
*	VOID FreePropHelp(struct PropHelp *PropHelp)
*
*   FUNCTION
*	Should be called by each task when through with PropHelp routines
*
*********************************************************************
*/
VOID FreePropHelp(struct PropHelp *PropHelp)
{
	if (PropHelp) {
		if (PropHelp->TimerReplyPort) DeletePort(PropHelp->TimerReplyPort);
		if (PropHelp->TimeRequest) {
			if (PropHelp->TimerBase) 
				CloseDevice((struct IORequest *)PropHelp->TimeRequest);
			DeleteExtIO((struct IORequest *)PropHelp->TimeRequest);
		}
		FreeMem(PropHelp,sizeof(struct PropHelp));
	}
}

/****** PropHelp/PropStartRepeat ************************************
*
*   NAME   
*	PropStartRepeat
*
*   SYNOPSIS
*	VOID PropStartRepeat(struct PropHelp *PropHelp)
*
*   FUNCTION
*	Starts key repeat messages after KeyRptDelay
*
*********************************************************************
*/
VOID PropStartRepeat(struct PropHelp *PropHelp)
{
#ifdef PROP_DEBUG
	printf("PropStartRepeat()\n");
#endif
	if (PropHelp) {
		PropHelp->TimeRequest->tr_time.tv_secs = PropHelp->KeyRptDelay.tv_secs;
		PropHelp->TimeRequest->tr_time.tv_micro = PropHelp->KeyRptDelay.tv_micro;
		SendIO((struct IORequest *)PropHelp->TimeRequest);
	}
}

/****** PropHelp/PropHandleRepeat ************************************
*
*   NAME   
*	PropHandleRepeat
*
*   SYNOPSIS
*	VOID PropHandleRepeat(struct PropHelp *PropHelp)
*
*   FUNCTION
*	Asks for another repeat message, this time at KeyRptSpeed
*
*********************************************************************
*/
VOID PropHandleRepeat(struct PropHelp *PropHelp)
{
	if (PropHelp) {
		PropHelp->TimeRequest->tr_time.tv_secs = PropHelp->KeyRptSpeed.tv_secs;
		PropHelp->TimeRequest->tr_time.tv_micro = PropHelp->KeyRptSpeed.tv_micro;

#ifdef PROP_DEBUG
	printf("PropHandleRepeat(): %d %d\n",
		PropHelp->TimeRequest->tr_time.tv_secs,
		PropHelp->TimeRequest->tr_time.tv_micro);
#endif

		SendIO((struct IORequest *)PropHelp->TimeRequest);
	}
}

/****** PropHelp/PropEndRepeat ************************************
*
*   NAME   
*	PropEndRepeat
*
*   SYNOPSIS
*	VOID PropEndRepeat(struct PropHelp *PropHelp)
*
*   FUNCTION
*	Aborts any pending messages
*
*********************************************************************
*/
VOID PropEndRepeat(struct PropHelp *PropHelp)
{
#ifdef PROP_DEBUG
	printf("PropEndRepeat()\n");
#endif
	if (PropHelp) {
		AbortIO((struct IORequest *)PropHelp->TimeRequest);
		WaitIO((struct IORequest *)PropHelp->TimeRequest);
	}
}

/****** PropHelp/SetPropVertBody ************************************
*
*   NAME   
*	SetPropVertBody
*
*   SYNOPSIS
*	BOOL SetPropVertBody(struct Gadget *Gadget,struct Window *Window,
*		ULONG Visible,ULONG Total,BOOL ZeroPot)
*
*   FUNCTION
*	Updates VertBody based on Visible and Total. Resets VertPot to zero
*	if ZeroPot TRUE. Re-renders gadget.
*
*********************************************************************
*/
BOOL SetPropVertBody(struct Gadget *Gadget,struct Window *Window,
	ULONG Visible,ULONG Total,BOOL ZeroPot)
{
	BOOL Result = FALSE;
	struct PropInfo *PropInfo;
	ULONG NewVertBody, NewVertPot;

	if (Gadget && Window) {
		PropInfo = Gadget->SpecialInfo;
		if (Visible >= Total) NewVertBody = MAXBODY;
		else NewVertBody = (MAXBODY * Visible) / Total;
		if (ZeroPot) NewVertPot = 0;
		else NewVertPot = PropInfo->VertPot;

#ifdef CUSTOM_BODY
		CustomModifyProp(Gadget,Window,NULL,IntuitionBase,PropInfo->Flags,
		PropInfo->HorizPot,NewVertPot,PropInfo->HorizBody,NewVertBody,
		2); // NumGad 2 so Intuition will render Body
#else
		NewModifyProp(Gadget,Window,NULL,PropInfo->Flags,
		PropInfo->HorizPot,NewVertPot,PropInfo->HorizBody,NewVertBody,
		1);
#endif

		Result = TRUE;
	}
	return(Result);
}

/****** PropHelp/SetPropVertPot ************************************
*
*   NAME   
*	SetPropVertPot
*
*   SYNOPSIS
*	BOOL SetPropVertPot(struct Gadget *Gadget,struct Window *Window,
*		ULONG Offset,ULONG Visible,ULONG Total)
*
*   FUNCTION
*	Updates VertPot based on Offset and Total. Re-renders gadget.
*	"Visible" not used, but here for compatibility
*
*********************************************************************
*/
BOOL SetPropVertPot(struct Gadget *Gadget,struct Window *Window,
	ULONG Offset,ULONG Visible,ULONG Total)
{
	BOOL Result = FALSE;
	struct PropInfo *PropInfo;
	ULONG NewVertPot;

	if (Gadget && Window) {
		if (Total) { // only need to bother if scrollable
			PropInfo = Gadget->SpecialInfo;
			NewVertPot = (MAXPOT * Offset) / Total;
			NewModifyProp(Gadget,Window,NULL,PropInfo->Flags,
				PropInfo->HorizPot,NewVertPot,PropInfo->HorizBody,
				PropInfo->VertBody,1);
		}
		Result = TRUE;
	}
	return(Result);
}

/****** PropHelp/SetPropHorizPot ************************************
*
*   NAME   
*	SetPropHorizPot
*
*   SYNOPSIS
*	BOOL SetPropHorizPot(struct Gadget *Gadget,struct Window *Window,
*		ULONG Offset,ULONG Visible,ULONG Total)
*
*   FUNCTION
*	Updates HorizPot based on Offset and Total. Re-renders gadget.
*	"Visible" not used, but here for compatibility
*
*********************************************************************
*/
BOOL SetPropHorizPot(struct Gadget *Gadget,struct Window *Window,
	ULONG Offset,ULONG Visible,ULONG Total)
{
	BOOL Result = FALSE;
	struct PropInfo *PropInfo;
	ULONG NewHorizPot;

	if (Gadget) {
		if (Total) { // only need to bother if scrollable
			PropInfo = Gadget->SpecialInfo;
			NewHorizPot = (MAXPOT * Offset) / Total;
			if (Window) NewModifyProp(Gadget,Window,NULL,PropInfo->Flags,
				NewHorizPot,PropInfo->VertPot,PropInfo->HorizBody,
				PropInfo->VertBody,1);
			else PropInfo->HorizPot = NewHorizPot;
		}
		Result = TRUE;
	}
	return(Result);
}

// end of prophelp.c
