/*********************************************************************\
*
* RWtest - Test drive speed
*
* $Id: RWtest.c,v 1.0 1995/05/05 15:52:29 Flick Exp $
*
* $Log: RWtest.c,v $
*Revision 1.0  1995/05/05  15:52:29  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	01/19/95		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>
#include <clib/dos_protos.h>

#include "flyerlib.h"
#include "flyer.h"

#define	TESTBLKS 512		/* Each transfer will be 512 blocks (256K) */
#define	REPEAT	64			/* Do 64 transfers for each test */
#define	TOTALSIZE	TESTBLKS*REPEAT
#define	NUMTESTS		4

#define	DBUG(x)			if (Debugging) { x }

double DoTest(ULONG lba,ULONG testblks,ULONG repeat,int writeflag);
ULONG ElapsedTicks(struct DateStamp *begin, struct DateStamp *end);
void GetArg(char *arg,int *var,int lo,int hi,char *label);

UBYTE	*Buffer;
ULONG	BufLen;
int	Debugging;

struct Library			*FlyerBase;
struct FlyerVolume	FlyVol;
struct ClipAction		Action;

main(int argc,char **argv)
{
	int	drive,first,last,samples,i,j,writeflag;
	ULONG	lba,blklen,max,res;
	double spd,lowest,totalspd;
	BPTR	handle;
	char	c;
	UBYTE	InqBuff[255];

	if ((argc == 3) && (stricmp(argv[2],"debug")==0)) {
		Debugging = 1;
		argc--;
	}

	if ((argc!=2)||(argv[1][0]=='?')) {
		printf("Usage: %ls <drive>\n",argv[0]);
		return(20);
	}

	FlyVol.Board = 0;		/* 1st Flyer board */
	GetArg(argv[1],&drive,0,23,"Drive");
	FlyVol.SCSIdrive = drive;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}


	/* Get inquiry data from SCSI drive */
	res = Inquiry(&FlyVol,36,InqBuff);
	if (res) {
		printf("Inquiry Error %ld - %ls\n",res,Error2String(res));
		exit(20);
	}
	else {
		printf("\nManufacturer/Model: ");
		for (j=8;j<=31;j++)					/* Print Manufacturer */
			printf("%c",InqBuff[j]);
		printf("\nRevision: ");
		for (j=32;j<=35;j++)					/* Print model # */
			printf("%c",InqBuff[j]);
		printf("\n\n");
	}

	/* Get drive specs: # blocks, block size */
	res = ReadSize(&FlyVol,&max,&blklen);
	if (res) {
		printf("ReadSize Error %ld - %ls\n",res,Error2String(res));
		exit(20);
	}
	printf("Drive has %ld blocks  (%.1f GB)\n\n",max,
		((double)max)/(double)(2000000));

	handle = Input();

	printf("This test will destroy data on the drive whose light is now flashing.\n");
	printf("Press Y and return to continue test, or anything else to abort.\n");
	for (i=0;;i++) {
		if (WaitForChar(handle,30)) {
			c = getchar();
			if (toupper(c)=='Y') {
				getchar();
				break;
			}
			else {
				printf("Aborted\n");
				getchar();

				CloseLibrary(FlyerBase);
				return(20);
			}
		}
		if (i > 5) {
			Action.Volume = &FlyVol;
			Action.ReturnTime = RT_STOPPED;
			res = Read10(&Action,2,0,0x1ffe);
			i = 0;
		}
	}

	for (writeflag=0;writeflag<=1;writeflag++) {
		printf("Doing drive %ls test...\n",(writeflag?"write":"read"));

		samples = 0;
		lowest = 9999.0;
		totalspd = 0.0;

		first = 1;
		last = 0;
		for (lba=0,i=0;!last;lba+=TOTALSIZE,first=0,i++) {

			/* Time to skip to end of drive? */
			if (i==NUMTESTS) {
				lba = max-(TOTALSIZE*NUMTESTS);
				first = 1;
			}

			if ((lba+TOTALSIZE) >= max) {
				lba = max-TOTALSIZE;
				last = 1;
			}
			spd = DoTest(lba,TESTBLKS,REPEAT,writeflag);
			if (spd < 0.1)		/* Some major failure? */
				break;

			DBUG(printf("LBA %lx ---> %.2f\n",lba,spd);)

			/* Record lowest speed (forgive first speed) */
			if ((!first) && (spd < lowest))
				lowest = spd;

			/* Keep average for entire session */
			totalspd += spd;
			samples++;
		}
		printf("   Slowest speed was %.2f MB/sec\n",lowest);
		printf("   Average speed was %.2f MB/sec\n\n",totalspd/(double)samples);
	}


	CloseLibrary(FlyerBase);

	return(0);
}


double DoTest(ULONG lba,ULONG testblks,ULONG repeat,int writeflag)
{
	struct	DateStamp	start,end;
	double	meg,speed,seconds;
	ULONG		ticks,res;

	Forbid();

	DateStamp(&start);			/* Log time now */

	if (writeflag)
		res = WriteTest(&FlyVol,testblks,repeat,lba,1);
	else
		res = ReadTest(&FlyVol,testblks,repeat,lba,1);

	DateStamp(&end);				/* Log time now */

	Permit();

	if (res) {
		printf("R/W Error %ld - %ls\n",res,Error2String(res));
		return(0);
	}
	else {
		/* Compute stats */
		meg = ((double)(testblks * repeat))/2048.0;

		ticks = ElapsedTicks(&start,&end);
		seconds = ((double)ticks)/50;

		speed = meg/((double)seconds);

#if 0
		printf("	%.2f",speed);
#endif

		return(speed);
	}
}

ULONG ElapsedTicks(struct DateStamp *begin, struct DateStamp *end)
{
	ULONG		ticks;

	ticks = end->ds_Minute * 60 *50 + end->ds_Tick;
	ticks -= begin->ds_Minute * 60*50 + begin->ds_Tick;

	return(ticks);
}

void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stcd_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
