/*********************************************************************\
*
* ReadField - Dump video field
*
* $Id: ReadField.c,v 1.0 1995/05/05 15:50:40 Flick Exp $
*
* $Log: ReadField.c,v $
*Revision 1.0  1995/05/05  15:50:40  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	09/16/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>
#include <clib/dos_protos.h>

#include "flyerlib.h"
#include "flyer.h"

void Interleave(UBYTE *buffer);
void GetArg(char *arg,int *var,int lo,int hi,char *label);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);


struct Library *FlyerBase;

struct FlyerVolume	FlyVol;
struct ClipAction		Action;

UBYTE Buffer[2][SAMPLESPERLINE],Buff2[SAMPLESPERLINE];

ULONG FSheader[] = {
	0x464F524D,0x000B0450,								/* FORM,size */

	0x494C424D,												/* ILBM */

	0x424D4844,0x00000014,								/* BMHD,size */
	0x02F001E0,												/* width,height */
	0x00000000,												/* X,Y */
	0x10000000,					/* #planes,masking,compression,pad */
	0x00000B0D,					/* Transparent color,Xaspect,Yaspect */
	0x02F001E0,												/* Page width,height */

	0x504C5450,0x00000020,								/* PLTP,size */
	0x06070606,0x06050604,0x06030602,0x06010600,	/* Plane types (fields 1,2) */
	0x07070706,0x07050704,0x07030702,0x07010700,	/* Plane types (fields 3,4) */

	0x424F4459,0x000B0400								/* BODY,size */
};

main(int argc,char **argv)
{
	int	unit,drive,i,j,line,fldnum,firstfield,lastfield,cntr,which,oper;
	ULONG	field,res,ticks;
	BPTR	handle;
	struct	DateStamp	start,end;

	if ((argc<6)||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <drive> <name> <field> <oper> [fs.name]\n",argv[0]);
		printf("Operations:\n");
		printf("   1) Dump first line of field\n");
		printf("   2) Pull in complete field (ignore)\n");
		printf("   3) Read complete color frame, convert to framestore\n");
		printf("   4) Pull in every 3rd line, convert to framestore\n");
		printf("   5) Pull in every 3rd line (ignore)\n");
		printf("   6) Special\n");
		printf("   7) Decomp field to binary file\n");
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	GetArg(argv[2],&drive,0,23,"SCSI Drive");
	GetLongArg(argv[4],&field,0,0xFFFFFFFF,"Field number");
	GetArg(argv[5],&oper,1,7,"Operation");

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	FlyVol.Board = unit;
	FlyVol.SCSIdrive = drive;
	FlyVol.Path = argv[3];

	Action.Volume = &FlyVol;

	handle = 0;
	firstfield = field;
	lastfield = field;

	if ((oper==3)||(oper==4)||(oper==7)) {
		if (argc<7) {
			printf("Missing file name\n");
			CloseLibrary(FlyerBase);
			exit(10);
		}

		handle = Open(argv[6],MODE_NEWFILE);
		if (handle==0) {
			printf("Couldn't create file!\n");
			CloseLibrary(FlyerBase);
			exit(10);
		}
		if ((oper==3)||(oper==4)) {
			Write(handle,FSheader,sizeof(FSheader));
			firstfield = (field>>2) * 4;
			lastfield = firstfield+3;
		}
	}

	DateStamp(&start);			/* Log time now */

	for (field=firstfield;field<=lastfield;field++) {
		cntr = field-firstfield;

		if ((oper==3)||(oper==4)) {
			switch (cntr) {
#if 0
				case (0): fldnum = 1; break;
				case (1): fldnum = 3; break;
				case (2): fldnum = 2; break;
				case (3): fldnum = 0; break;
#endif
				case (0): fldnum = 0; break;
				case (1): fldnum = 2; break;
				case (2): fldnum = 1; break;
				case (3): fldnum = 3; break;
			}
		}
		else
			fldnum = 0;

		fldnum += firstfield;

		Action.ReturnTime = RT_STOPPED;
		res = OpenReadField(&Action,fldnum,0);
		if (res) {
			printf("\nOpen Error %ld - %ls\n",res,Error2String(res));
			printf("Didn't open field %ld\n",fldnum);
			break;
		}
		else {
			printf("Found field %ld\n",fldnum);

			if (oper==6)
				break;
#if 0
			/* Tests fill color and line skipping on read */
			Action.ReturnTime = RT_STOPPED;
			SkipLines(&Action,120);
			Action.MatteY = 127;		/* Green */
			Action.MatteI = -27;
			Action.MatteQ = -51;
			SetFillColor(&Action);
#endif

			/* For 2,3,4th passes, skip back to start, offset to currect spot */
			if (((oper==3)||(oper==4)) && (field!=firstfield))
				Seek(handle,sizeof(FSheader) + cntr*SAMPLESPERLINE,OFFSET_BEGINNING);

			for (which=0,line=21;line<=264;line++,which=1-which) {
				if ((oper != 7) && (line > 261))
					break;
				if (line != 21)
					WaitAction(&Action);				/* Make sure we have data ready */

				if (line<=260) {
					Action.ReturnTime = RT_IMMED;
					res = FlyerReadLine(&Action,Buffer[1-which]);
					if (res) {
						printf("\nError %ld - %ls\n",res,Error2String(res));
						break;
					}
					if ((oper==4)||(oper==5))
						SkipLines(&Action,2);
				}

				if (line == 21)				/* Do not have data first time thru */
					continue;

				if (((((line-1)-21)%10) == 0) && (oper!=5))
					printf("Line %ld...\n",line-1);

				if ((oper==3)||(oper==4)) {
					Interleave(Buffer[which]);
					Write(handle,Buff2,SAMPLESPERLINE);
					if (field==firstfield) {
						Write(handle,Buff2,SAMPLESPERLINE);
						Write(handle,Buff2,SAMPLESPERLINE);
						Write(handle,Buff2,SAMPLESPERLINE);
					}
					else {
						Seek(handle,3 * SAMPLESPERLINE,OFFSET_CURRENT);
					}
				}
				else if (oper==7) {
					Write(handle,Buffer[which],SAMPLESPERLINE);
				}
				else if (oper==1) {
					if ((line-1)<=22) {
						printf("(line=%ld)\n",line-1);
						for (i=0;i<SAMPLESPERLINE;i+=16) {
							for (j=0;j<16;j++)
								printf("%02lx ",Buffer[which][i+j]);
							printf("\n");
						}
					}
					else
						break;
				}
			}
			Action.ReturnTime = RT_STOPPED;
			CloseField(&Action);
		}
	}

	DateStamp(&end);				/* Log time now */

	/* Finish writing FrameStore, if begun */
	if (handle)
		Close(handle);

	CloseLibrary(FlyerBase);

	ticks = end.ds_Minute * 60 *50 + end.ds_Tick;
	ticks -= start.ds_Minute * 60*50 + start.ds_Tick;
	printf("Elapsed time was %ld and %ld/50 seconds\n",
		ticks/50,ticks % 50);

	return(0);
}


void Interleave(UBYTE *srcbuf)
{
	register	UBYTE	collect,testimage;
	int	i,j;
	register UBYTE *b2ptr,*srcptr;
	int	p;

	b2ptr = Buff2;
	for (p=7;p>=0;p--) {
		testimage = 1<<p;
		for (srcptr=srcbuf,i=0;i<SAMPLESPERLINE;i+=8) {
			for (collect=0,j=8;j;j--) {
				collect <<= 1;
				if ((*srcptr++) & testimage)
					collect |= 1;
			}
			*b2ptr++ = collect;
		}
	}
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}

void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
