/*********************************************************************\
*
* ReadField2 - Dump video field
*
* $Id: ReadField2.c,v 1.0 1995/05/05 15:50:44 Flick Exp $
*
* $Log: ReadField2.c,v $
*Revision 1.0  1995/05/05  15:50:44  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	09/16/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>
#include <clib/dos_protos.h>

#include "flyerlib.h"
#include "flyer.h"

void Interleave(UBYTE *srcbuf,UBYTE *destbuf);
void Inverse(UBYTE *srcbuf,UBYTE *destbuf);
void GetArg(char *arg,int *var,int lo,int hi,char *label);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);


struct Library *FlyerBase;

struct FlyerVolume	FlyVol;
struct ClipAction		Action;

UBYTE Buffer[SAMPLESPERLINE],ILBMbuff[SAMPLESPERLINE];
UBYTE BufferI[SAMPLESPERLINE],ILBMbuffI[SAMPLESPERLINE];

ULONG FSheader[] = {
	0x464F524D,0x000B0450,								/* FORM,size */

	0x494C424D,												/* ILBM */

	0x424D4844,0x00000014,								/* BMHD,size */
	0x02F001E0,												/* width,height */
	0x00000000,												/* X,Y */
	0x10000000,					/* #planes,masking,compression,pad */
	0x00000B0D,					/* Transparent color,Xaspect,Yaspect */
	0x02F001E0,												/* Page width,height */

	0x504C5450,0x00000020,								/* PLTP,size */
	0x06070606,0x06050604,0x06030602,0x06010600,	/* Plane types (fields 1,2) */
	0x07070706,0x07050704,0x07030702,0x07010700,	/* Plane types (fields 3,4) */

	0x424F4459,0x000B0400								/* BODY,size */
};

main(int argc,char **argv)
{
	int	unit,drive,line;
	ULONG	field,res,ticks;
	BPTR	handle;
	struct	DateStamp	start,end;

	if ((argc!=6)||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <drive> <name> <field> <fs.name>\n",argv[0]);
		printf("Operations:\n");
		printf("   1) Read 1 field, convert to framestore\n");
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	GetArg(argv[2],&drive,0,23,"SCSI Drive");
	GetLongArg(argv[4],&field,0,0xFFFFFFFF,"Field number");

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	FlyVol.Board = unit;
	FlyVol.SCSIdrive = drive;
	FlyVol.Path = argv[3];

	Action.Volume = &FlyVol;

	handle = Open(argv[5],MODE_NEWFILE);
	if (handle==0) {
		printf("Couldn't create framestore file!\n");
		CloseLibrary(FlyerBase);
		exit(10);
	}
	Write(handle,FSheader,sizeof(FSheader));

	DateStamp(&start);			/* Log time now */

	Action.ReturnTime = RT_STOPPED;
	res = OpenReadField(&Action,field,0);
	if (res) {
		printf("\nOpen Error %ld - %ls\n",res,Error2String(res));
		printf("Didn't open field %ld\n",field);
	}
	else {
		printf("Found field %ld\n",field);

#if 0
		/* Tests fill color and line skipping on read */
		Action.ReturnTime = RT_STOPPED;
		SkipLines(&Action,50);
		Action.MatteY = 99;		/* Red */
		Action.MatteI = 58;
		Action.MatteQ = 21;
		Action.MatteY = 99;		/* Burst vector??? */
		Action.MatteI = 30;
		Action.MatteQ = -46;
		SetFillColor(&Action);
#endif

		for (line=21;line<=260;line++) {
			Action.ReturnTime = RT_STOPPED;
			res = FlyerReadLine(&Action,Buffer);
			if (res) {
				printf("\nError %ld - %ls\n",res,Error2String(res));
				break;
			}

			if (((line-21)%10) == 0)
				printf("Line %ld...\n",line);

			Interleave(Buffer,ILBMbuff);
			Inverse(Buffer,BufferI);
			Interleave(BufferI,ILBMbuffI);

			/* Order 1,3,2,0 */
			Write(handle,ILBMbuff,SAMPLESPERLINE);
			Write(handle,ILBMbuffI,SAMPLESPERLINE);
			Write(handle,ILBMbuffI,SAMPLESPERLINE);
			Write(handle,ILBMbuff,SAMPLESPERLINE);
		}
		Action.ReturnTime = RT_STOPPED;
		CloseField(&Action);
	}

	DateStamp(&end);				/* Log time now */

	/* Finish writing FrameStore, if begun */
	if (handle)
		Close(handle);

	CloseLibrary(FlyerBase);

	ticks = end.ds_Minute * 60 *50 + end.ds_Tick;
	ticks -= start.ds_Minute * 60*50 + start.ds_Tick;
	printf("Elapsed time was %ld and %ld/50 seconds\n",
		ticks/50,ticks % 50);

	return(0);
}


void Interleave(UBYTE *srcbuf,UBYTE *destbuf)
{
	register	UBYTE	collect,testimage;
	int	i,j;
	register UBYTE *b2ptr,*srcptr;
	int	p;

	b2ptr = (UBYTE *)destbuf;
	for (p=7;p>=0;p--) {
		testimage = 1<<p;
		for (srcptr=srcbuf,i=0;i<SAMPLESPERLINE;i+=8) {
			for (collect=0,j=8;j;j--) {
				collect <<= 1;
				if ((*srcptr++) & testimage)
					collect |= 1;
			}
			*b2ptr++ = collect;
		}
	}
}


void Inverse(UBYTE *srcbuf,UBYTE *destbuf)
{
	register UBYTE *dptr,*sptr;
	int	i,s1;

	dptr = destbuf;
	sptr = srcbuf+2;
	for (i=0;i<(SAMPLESPERLINE-4);i++) {
		*dptr++ = *sptr++;		// Old way

//		s2 = *(sptr+2);
//		*(dptr+2) = *sptr++;
//		*dptr++ = s2;
//
//		s2 = *(sptr+2);
//		*(dptr+2) = *sptr++;
//		*dptr++ = s2;
//
//		sptr += 2;
//		dptr += 2;
	}

	for (i=0;i<2;i++) {
		s1 = *sptr;
		*dptr = *(sptr+2);
		*(dptr+2) = s1;

		sptr++;
		dptr++;
	}
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}

void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
