/*********************************************************************\
*
* SavePatchFile - Writes a patch file (microcode or FPGA) to special
*						patch area on a Flyer drive
*
* $Id: SavePatchFile.c,v 1.3 95/09/28 14:41:18 Flick Exp $
*
* $Log:	SavePatchFile.c,v $
*Revision 1.3  95/09/28  14:41:18  Flick
*Uses save SRAM area now (still not very safe)
*
*Revision 1.2  1995/07/06  14:21:46  Flick
*Removed hack which forced always reading patch from drive!
*
*Revision 1.1  1995/06/20  09:17:08  Flick
*Added option to read patch in and save to a file
*
*Revision 1.0  1995/05/05  15:51:10  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	03/29/95		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>
#include <clib/dos_protos.h>

#include "flyerlib.h"
#include "flyer.h"

ULONG doit(ULONG revision, ULONG loadaddr);
void GetArg(char *arg,int *var,int lo,int hi,char *label);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);
ULONG SCSI_Read(ULONG lba, ULONG size);
ULONG SCSI_Write(ULONG lba, ULONG size);
ULONG ReadBinFile(char *,ULONG);
void FreeFile(void);
ULONG readitbackin(ULONG bytes,char *filename);

#define	CHUNK		64			//32K
#define	DMABLK	0xA00
#define	RAMBLK	0x20		// Use EC4000 - ECC000

struct Library *FlyerBase;
struct FlyerVolume FlyVol;
struct ClipAction	 Action;
struct FlyerVolInfo	FVIbuff;

UBYTE	*Buffer;
ULONG	Length;

struct DATESTAMP {
	ULONG		days;
	ULONG		minutes;
	ULONG		ticks;
};

#define	ROOTENTRIES		100

struct ROOTHDR {
	ULONG		id;					/* Identifies this header */
	UBYTE		flags;				/* Drive characteristics */
	UBYTE		diskokay;			/* Disk structures all valid? */
	UWORD		blksize;				/* Logical block size */
	ULONG		parent;				/* Always 0 */
	char		volname[32];		/* Name of volume */
		
	ULONG		freelist;			/* Location of FreeList table */
	ULONG		headlist;			/* Location of HeadList table */
	ULONG		hdrsarea;			/* File/Dir headers */
	ULONG		unused;				/* Pointer past used hdrs */
	ULONG		dataarea;			/* Start of data area (fixed) */
	ULONG		headspac;			/* Space used for heads */
	ULONG		videoend;			/* End of fast area of drive */
	ULONG		blksfree;			/* Data blocks free -- OBSOLETE */

	UBYTE		version;				/* FileSystem version */
	UBYTE		FStype;				/* Type (root) */
	struct DATESTAMP thedate;	/* Date formatted */

	ULONG		userblks;			/* Drive size (user data blks) */
	ULONG		newheadlist;		/* Location of NewHeadList table */

	UBYTE		pad1[10];			/* Future */

	ULONG		entries[ROOTENTRIES];	/* Hashed children */
	ULONG		checksum;			/* Balance to 0 */
};


struct PATCHHDR {
	ULONG		id;					// Identifies this header
	ULONG		size;					// Data length in bytes
	ULONG		type;					// Type code of file
	ULONG		rev;					// Revision
	ULONG		speed;				// Clk speed for FPGA's
	ULONG		loadaddr;			// Load/run address of microcode
	ULONG		blks;					// Blocks needed to contain data
	UBYTE		pad1[484];			/* Future */
};

#define	ID_PATCH	0x50544348


main(int argc,char **argv)
{
	int	drive;
	ULONG	res,revision,loadaddr,size;
	BOOL	readin;

	if (((argc!=5)&&(argc!=6))||(argv[1][0]=='?')) {
		printf("Usage: %ls <filename> <drive> <rev> <address> [READ]\n",argv[0]);
		return(20);
	}

	FlyVol.Board = 0;
	GetArg(argv[2],&drive,0,23,"Drive");
	GetLongArg(argv[3],&revision,0,0x7FFFFFFF,"Revision");
	GetLongArg(argv[4],&loadaddr,0xC00000,0xC3FFFF,"Load Address");

	if ((argc==6) && (stricmp(argv[5],"READ")==0))
		readin = TRUE;
	else
		readin = FALSE;

	FlyVol.SCSIdrive = drive;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return(20);
	}

	printf("Just a moment...\n");

	res = PlayMode(0);			// Ensure Flyer is in play mode
	res = RecordMode(0);
	res = PlayMode(0);

	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_STOPPED;

	if (readin)
	{
		printf("Reading patch from drive...\n");
		readitbackin(revision,argv[1]);
		printf("Done\n");
	}
	else
	{
		size = ReadBinFile(argv[1],0);
		if (size) {
	
			printf("Writing patch to drive...\n");
	
			res = doit(revision,loadaddr);
			if (res)
				printf("Error %ld - %ls\n",res,Error2String(res));
			else
				printf("Done\n");
	
		}
	}

	CloseLibrary(FlyerBase);
	FreeFile();

	if (res)
		return(10);
	else
		return(0);
}


ULONG doit(ULONG revision, ULONG loadaddr)
{
	ULONG	lba,size,res,maxblock,blklen,blks,datalen,bytesize;
	UBYTE	*buffer,*dataptr;
	struct ROOTHDR		*root;
	struct PATCHHDR	*phdr;

	buffer = (UBYTE *)0xEC0000 + (512 * RAMBLK);

	FVIbuff.len = sizeof(struct FlyerVolInfo);
	res = FlyerDriveInfo(&FlyVol,&FVIbuff);
	if (res) {
		printf("FVI ");
		return(res);
	}

	res = SCSI_Read(0,1);		// Read ROOT info in
	if (res) {
		printf("Root ");
		return(res);
	}

	root = (struct ROOTHDR *)buffer;

	if (root->id != 0x524F4F54) {
		printf("This drive is not Flyer-formatted\n");
		return(FERR_UNFORMATTED);
	}

	lba = root->videoend;
//	printf("Placing at lba %lx\n",lba);


	/* Get drive specs: # blocks, block size */
	res = ReadSize(&FlyVol,&maxblock,&blklen);
	if (res) {
		printf("ReadSize ");
		return(res);
	}

	blks = (Length+511)>>9;

	if ((lba + 1 + blks) > maxblock) {
		printf("That file will not fit in the available patch area\n");
		return(FERR_OKAY);
	}

	phdr = (struct PATCHHDR *)buffer;

	memset(phdr,0,sizeof(struct PATCHHDR));

	phdr->id = ID_PATCH;
	phdr->size = Length;
	phdr->type = 100;			// 68000 Microcode
	phdr->rev	= revision;
	phdr->speed = 0;
	phdr->loadaddr = loadaddr;
	phdr->blks = blks;

	res = SCSI_Write(lba++,1);		// Write Patch header
	if (res) {
		printf("Patch Header ");
		return(res);
	}

	for (dataptr=Buffer,datalen=Length ; blks ; lba+=size,blks-=size) {
		size = CHUNK;
		if (size > blks)
			size = blks;

//		if ((lba+size) > maxblock)
//			size = (maxblock - lba);

		bytesize = size*512;
		if (bytesize > datalen)
			bytesize = datalen;

//		printf("Copying from %lx (%lx)\n",dataptr,bytesize);
		memcpy(buffer,dataptr,bytesize);		// Copy data out of binary area
		dataptr += bytesize;
		datalen -= bytesize;

//		printf("Writing to %lx (%lx)\n",lba,size);
		res = SCSI_Write(lba,size);			// Write next chunk of data
		if (res) {
			printf("Write ");
			return(res);
		}
	}
	return(FERR_OKAY);
}


ULONG readitbackin(ULONG bytes,char *filename)
{
	ULONG	lba,res,blks;
	UBYTE	*buffer;
	struct ROOTHDR		*root;
	FILE	*fp;

	fp = fopen(filename,"w");
	if (!fp) {
		printf("File open error\n");
		return(12);
	}

	buffer = (UBYTE *)0xEC0000 + (512 * RAMBLK);

	res = SCSI_Read(0,1);		// Read ROOT info in
	if (res) {
		printf("Root ");
		return(res);
	}

	root = (struct ROOTHDR *)buffer;

	if (root->id != 0x524F4F54) {
		printf("This drive is not Flyer-formatted\n");
		return(FERR_UNFORMATTED);
	}

	lba = root->videoend+1;
//	printf("Reading from lba %lx\n",lba);

	blks = (bytes+511)>>9;

	for (;blks;blks--) {
		res = SCSI_Read(lba++,1);			// Read block of data
		if (res) {
			printf("Read ");
			fclose(fp);
			return(res);
		}

		if (fwrite(buffer,512,1,fp)!=1)
		{
			printf("Write ");
			fclose(fp);
			return(11);
		}
	}

	fclose(fp);

	return(FERR_OKAY);
}


ULONG SCSI_Read(ULONG lba, ULONG size)
{
	ULONG	 res;

	res = Read10(&Action,size,lba,DMABLK);
	if (res)
		printf("Read10 Error %ld - %ls\n",res,Error2String(res));
	else {
		res = CPUDMA(0,RAMBLK,DMABLK,size,0);		/* Write to CPU */
		if (res)
			printf("CPUDMA Error %ld - %ls\n",res,Error2String(res));
	}
	return(res);
}


ULONG SCSI_Write(ULONG lba, ULONG size)
{
	ULONG	 res;

	res = CPUDMA(0,RAMBLK,DMABLK,size,1);		/* Read from CPU */
	if (res)
		printf("CPUDMA Error %ld - %ls\n",res,Error2String(res));
	else {
		res = Write10(&Action,size,DMABLK,lba);
		if (res)
			printf("Write10 Error %ld - %ls\n",res,Error2String(res));
	}
	return(res);
}


ULONG ReadBinFile(char *name,ULONG offset)
{
	char	filename[80];
	FILE	*fp;
	int	read;

	strcpy(filename,name);
	if (stricmp(".BIN",&filename[strlen(filename)-4]))
		strcat(filename,".BIN");

	read = 0;
	fp = fopen(filename,"r");
	if (fp) {
		fseek(fp,0,SEEK_END);
		Length = ftell(fp)-offset;
		fseek(fp,offset,SEEK_SET);

		Buffer = AllocMem(Length,MEMF_PUBLIC);
		if (Buffer) {
			read = fread(Buffer,Length,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",filename);

	if (read==1)
		return(Length);
	else
		return(0);
}


void FreeFile(void)
{
	if (Length)
		FreeMem(Buffer,Length);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}

void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
