/********************************************************************
* $ScrollGrid.c$
* $Id: ScrollGrid.c,v 2.0 1994/02/17 16:24:09 Kell Exp $
* $Log: ScrollGrid.c,v $
*Revision 2.0  1994/02/17  16:24:09  Kell
*FirstCheckIn
*
*Revision 2.0  94/02/17  15:57:13  Kell
*FirstCheckIn
*
*Revision 2.0  94/02/17  14:44:25  Kell
*FirstCheckIn
*
*
* Copyright (c)1992 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	9-22-92		Steve H		Changed to 640 width
*	12-17-92	Steve H		Convert to use SmartStrings
*	12-17-92	Steve H		Last Update
*
*********************************************************************/
#include <exec/types.h>
#include <exec/memory.h>
#include <intuition/intuition.h>
#include <stdio.h>
#include <string.h>
#include <crouton_all.h>

#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <proto/layers.h>

#ifndef PROTO_PASS
#include <ScrollGrid.p>
#include <graphichelp.p>
#endif

#define ICON_WIDTH 80
#define ICONS_PER_ROW 8
#define SG_WIDTH 640

// ScrollGrid's private data structures (shared with ScrollFast.a)
struct IconRow {
	struct Node Node;
	UWORD ValidIcons; // from zero to IconsPerRow
	UBYTE *RowBuffer; // Row data in weird movem-friendly format
};

struct ScrollGrid {
	struct BitMap *ChipBitMap;
	struct Layer_Info *ChipLayerInfo;
	struct Layer *ChipLayer;
	struct List IconRows;
	ULONG IconRowPlaneSize; // in bytes
	UWORD IconCount;	// always = total number of icons
	UWORD IconsPerRow;
	UWORD IconWidth,IconHeight;
};

// in ScrollFast.a
extern ULONG DrawRow(struct ScrollGrid *SG,ULONG Row,ULONG Scroll,
	ULONG Height,
	struct BitMap *DestinationBitMap,ULONG LeftEdge,ULONG TopEdge);

extern VOID SaveRow(struct BitMap *SourceBM,UBYTE *FastBuffer);

extern VOID SoftSpriteOn(VOID);
extern VOID SoftSpriteOff(VOID);

/****** ScrollGrid/AllocScrollGrid **********************************
*
*   NAME   
*	AllocScrollGrid
*
*   SYNOPSIS
*	struct ScrollGrid *AllocScrollGrid(UWORD InitialIconCount,
*		UWORD IconsPerRow,UWORD IconWidth,UWORD IconHeight,UWORD IconDepth,
*		struct TextFont *Font)
*
*   FUNCTION
*	Setup for new Scrolling Grid buffers. Returns NULL if out of memory.
*
*	NOTES
*	Will return structure, even if unable to allocate all InitialIconCount
*	structures! Up to caller to call CurrentIconCount() to verify that
*	all necessary structures were allocated.
*
*	Current ScrollGrid limits:
*	BitMap width is rounded up to word width rows
*	Number of rows is UWORD
*	Height of full size fast BitMap plane is ULONG
*	Total number of icons in list is UWORD
*	Allocated bitmap is always SG_WIDTH, regardless of IconWidth,IconsPerRow
*
*********************************************************************
*/
struct ScrollGrid *AllocScrollGrid(UWORD InitialIconCount,UWORD IconsPerRow,
	UWORD IconWidth,UWORD IconHeight,UWORD IconDepth,struct TextFont *Font)
{
	struct ScrollGrid *SG;
	UWORD A;
	BOOL Success = FALSE;
	struct BitMap *BM;
	struct Layer_Info *LI;
	ULONG PlaneSize;

	if (SG = SafeAllocMem(sizeof(struct ScrollGrid),MEMF_CLEAR|MEMF_PUBLIC)) {
		NewList(&SG->IconRows); // must do before immediately
		SG->IconsPerRow = IconsPerRow;
		SG->IconWidth = IconWidth;
		SG->IconHeight = IconHeight;

// Setup Chip BM
		if (!(BM = (struct BitMap *)SafeAllocMem(sizeof(struct BitMap),MEMF_CLEAR)))
			goto Exit;
		SG->ChipBitMap = BM;
		InitBitMap(BM,IconDepth,SG_WIDTH,IconHeight);
		PlaneSize = BM->BytesPerRow*BM->Rows;
		SG->IconRowPlaneSize = PlaneSize; // same for fast memory buffers
		if (!(HelpAllocBitMapPlanes(SG->ChipBitMap,MEMF_CHIP|MEMF_PUBLIC)))
			goto Exit;

// Setup initial rows (not required for routine to succeed)
		for (A=0; A < ((InitialIconCount+IconsPerRow-1)/IconsPerRow); A++) {
			if (!AppendIconRow(SG)) break;
		}

// Setup Layer
		if (!(LI = NewLayerInfo())) goto Exit;
		SG->ChipLayerInfo = LI;
		if (!(SG->ChipLayer=CreateUpfrontLayer(LI,BM,0,0,(BM->BytesPerRow*8)-1,
			BM->Rows-1,LAYERSIMPLE,NULL))) goto Exit;

		SetFont(SG->ChipLayer->rp,Font);

		Success = TRUE;
Exit:
		if (!Success) FreeScrollGrid(SG);
	}
	return(SG);
}

/****** ScrollGrid/AppendIconRow ************************************
*
*   NAME   
*	AppendIconRow
*
*   SYNOPSIS
*	BOOL AppendIconRow(struct ScrollGrid *SG)
*
*   FUNCTION
*	Adds a Row to an existing Scroll Grid
*
*********************************************************************
*/
BOOL AppendIconRow(struct ScrollGrid *SG)
{
	struct IconRow *IR = NULL;
	BOOL Success = FALSE;
	ULONG ByteSize;

	if (SG && SG->IconRowPlaneSize) {
		if (IR = (struct IconRow *)SafeAllocMem(sizeof(struct IconRow),
			MEMF_CLEAR|MEMF_PUBLIC)) {
			ByteSize = SG->IconRowPlaneSize*SG->ChipBitMap->Depth;
			if (!(IR->RowBuffer = SafeAllocMem(ByteSize,MEMF_PUBLIC))) {
				FreeIconRow(SG,IR);
				IR = NULL;
			} else {
				AddTail(&SG->IconRows,&IR->Node);
				SG->IconCount += SG->IconsPerRow;
				Success = TRUE;
			}
		}
	}
	return(Success);
}

/****** ScrollGrid/FreeScrollGrid **********************************
*
*   NAME   
*	FreeScrollGrid
*
*   SYNOPSIS
*	VOID FreeScrollGrid(struct ScrollGrid *SG)
*
*   FUNCTION
*	When done with Scroll Grid, frees it all
*
*********************************************************************
*/
VOID FreeScrollGrid(struct ScrollGrid *SG)
{
	struct IconRow *IconRow,*NextIR;

	if (SG) {
		if (SG->ChipLayer) DeleteLayer(NULL,SG->ChipLayer);
		if (SG->ChipLayerInfo) DisposeLayerInfo(SG->ChipLayerInfo);
		WaitBlit(); // DeleteLayer clears bitmap

		IconRow = (struct IconRow *)SG->IconRows.lh_Head;
		while (NextIR=(struct IconRow *)IconRow->Node.ln_Succ) {
			FreeIconRow(SG,IconRow);
			IconRow = NextIR;
		}

		// Don't free ChipBitMap until all IconRows freed
		if (SG->ChipBitMap) HelpFreeBitMap(SG->ChipBitMap);


		FreeMem(SG,sizeof(struct ScrollGrid));
	}
}

/****** ScrollGrid/FreeIconRow **************************************
*
*   NAME   
*	FreeIconRow
*
*   SYNOPSIS
*	VOID FreeIconRow(struct ScrollGrid *SG,struct IconRow *IR)
*
*   FUNCTION
*	Frees one IconRow (Better Remove() it from the list FIRST!)
*
*********************************************************************
*/
VOID FreeIconRow(struct ScrollGrid *SG,struct IconRow *IR)
{
	ULONG ByteSize;

	if (IR) {
		if (IR->RowBuffer && SG && SG->IconRowPlaneSize) {
			SG->IconCount -= SG->IconsPerRow;
			ByteSize = SG->IconRowPlaneSize*SG->ChipBitMap->Depth;
			FreeMem(IR->RowBuffer,ByteSize);
		}
		FreeMem(IR,sizeof(struct IconRow));
	}
}

/****** ScrollGrid/GridChipToFast ***********************************
*
*   NAME   
*	GridChipToFast
*
*   SYNOPSIS
*	BOOL GridChipToFast(struct ScrollGrid *SG,UWORD DestinationRow)
*
*   FUNCTION
*	Copies rendered ChipRowBM into fast buffers for later scrolling
*
*********************************************************************
*/
BOOL GridChipToFast(struct ScrollGrid *SG,UWORD DestinationRow)
{
	BOOL Success = FALSE;
	struct IconRow *IR;

	IR = (struct IconRow *)GetNode(&SG->IconRows,DestinationRow);
	if (IR) {
		SaveRow(SG->ChipBitMap,IR->RowBuffer);
		Success = TRUE;
	}
	return(Success);
}

/****** ScrollGrid/DrawScrollGrid ***********************************
*
*   NAME   
*	DrawScrollGrid
*
*   SYNOPSIS
*	ULONG DrawScrollGrid(struct ScrollGrid *SG,UWORD Row,UWORD Scroll,
*		ULONG Height,
*		struct BitMap *DestinationBitMap,UWORD LeftEdge,UWORD TopEdge)
*
*   FUNCTION
*	Draws ScrollGrid, synchronized with display beam to look good
*
*	INPUTS
*	SG - ScrollGrid returned by AllocScrollGrid()
*	Row - ordinal row #
*	Scroll - from 0..ICON_HEIGHT-1
*	Height - how many lines to copy
*	DestinationBitMap - where to put it
*	LeftEdge - offset into DestinationBitMap
*	TopEdge - offset into DestinationBitMap
*
*	RESULT
*	returns number of lines rendered, up to Height
*
*********************************************************************
*/
ULONG DrawScrollGrid(struct ScrollGrid *SG,UWORD Row,UWORD Scroll,
		ULONG Height,
		struct BitMap *DestinationBitMap,UWORD LeftEdge,UWORD TopEdge)
{
	ULONG L;

	SoftSpriteOff();
	L = DrawRow(SG,Row,Scroll,Height,DestinationBitMap,LeftEdge,TopEdge);
	SoftSpriteOn();
	return(L);
}

/****** ScrollGrid/GetRow *******************************************
*
*   NAME   
*	GetRow
*
*   SYNOPSIS
*	ULONG GetRow(struct ScrollGrid *SG,UWORD IconOffset)
*
*   FUNCTION
*	Returns ordinal number of Row containing given IconOffset
*
*********************************************************************
*/
ULONG GetRow(struct ScrollGrid *SG,UWORD IconOffset)
{
	ULONG Row = 0;

	if (SG && IconOffset && SG->IconsPerRow) {
		Row = ((IconOffset-1)/SG->IconsPerRow)+1;
	}
	return(Row);
}

/****** ScrollGrid/GetChipRastPort **************************************
*
*   NAME   
*	GetChipRastPort
*
*   SYNOPSIS
*	struct RastPort *GetChipRastPort(struct ScrollGrid *SG)
*
*   FUNCTION
*	Returns RastPort for rendering Row data into
*
*********************************************************************
*/
struct RastPort *GetChipRastPort(struct ScrollGrid *SG)
{
	struct RastPort *RP = NULL;

	if (SG && SG->ChipLayer) RP = SG->ChipLayer->rp;
	return(RP);
}

/****** ScrollGrid/GetIconCount *********************************
*
*   NAME   
*	GetIconCount
*
*   SYNOPSIS
*	UWORD GetIconCount(struct ScrollGrid *SG)
*
*   FUNCTION
*	Returns number of icon rows presently allocated
*
*********************************************************************
*/
UWORD GetIconCount(struct ScrollGrid *SG)
{
	if (SG) return(SG->IconCount);
	else return(NULL);
}

/****** ScrollGrid/FixIconRowCount *********************************
*
*   NAME   
*	FixIconRowCount
*
*   SYNOPSIS
*	BOOL FixIconRowCount(struct ScrollGrid *SG,ULONG NumberIcons)
*
*   FUNCTION
*	adds or removes icon rows to match NumberIcons/IconsPerRow
*	returns 0 on success, otherwise # of first icon unable to
*	alloc mem for (starting w/1 for first icon)
*
*********************************************************************
*/
UWORD FixIconRowCount(struct ScrollGrid *SG,ULONG NumberIcons)
{
	UWORD A,Result=0;
	struct IconRow *IconRow;

	if (SG) {
		A = (NumberIcons + SG->IconsPerRow - 1);
		A = A - (A % SG->IconsPerRow); // round A to full row

		while (SG->IconCount < NumberIcons) {
			if (!AppendIconRow(SG)) {
				Result = SG->IconCount + 1;
				goto Exit;
			}
		}
		while (SG->IconCount > A) {
			IconRow = (struct IconRow *)SG->IconRows.lh_Head;
			Remove(&IconRow->Node);
			FreeIconRow(SG,IconRow);
		}
	}
Exit:
	return(Result);
}

// end of scrollgrid.c
