/*********************************************************************\
*
* Sequence - Flyer test sequencer
*
* $Id: Sequence.c,v 1.0 1995/05/05 15:50:24 Flick Exp $
*
* $Log: Sequence.c,v $
*Revision 1.0  1995/05/05  15:50:24  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	10/21/94		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

ULONG ReadScriptFile(char *name);
void FreeFile(void);
int WaitHere(int);
void GetArg(char *arg,int *var,int lo,int hi,char *label);

#define	THEMAX	100

struct Entry {
	ULONG	clock;
	ULONG	drive;
	char	name[80];
	ULONG	vidflag;
	ULONG	chan;
	ULONG	vstartfld;
	ULONG	vfldlen;
	ULONG	audchans;
	ULONG	astartfld;
	ULONG	afldlen;
	ULONG	atotalfld;
	ULONG	atotallen;
	ULONG vol1,vol2;
	ULONG pan1,pan2;
	ULONG	attack;
	ULONG	decay;
	UBYTE	audhead;
};

struct Entry Sequence[THEMAX];

struct Library *FlyerBase;
struct FlyerVolume FlyVol[2];
struct ClipAction	 Action[2];

struct ClipInfo	clipinfo;

UBYTE	*FileBuffer;
ULONG	FileLength;
BPTR	Handle;

main(int argc,char **argv)
{
	int	i,ndx,heads,stepmode;
	ULONG	res,clock,baseclock,back1clock,back2clock,endofprev,endofthis;
	LONG	size;
	char *scriptptr,*endptr;
	struct Entry *entptr,*prev;
	struct FlyerVolume *volptr;
	struct ClipAction	 *actptr;

	if ((argc!=4)||(argv[1][0]=='?')) {
		printf("Usage: %ls <script> <heads> <stepmode>\n",argv[0]);
		return(20);
	}

	GetArg(argv[2],&heads,0,1,"Heads switch");
	GetArg(argv[3],&stepmode,0,1,"Step mode");

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	Handle = Input();

	size = ReadScriptFile(argv[1]);
	if (size) {
		endptr = FileBuffer+FileLength;
		for (ndx=0,scriptptr=FileBuffer;(ndx<THEMAX)&&(scriptptr<endptr);) {
			if ((*scriptptr == '!')||(*scriptptr == '>')) {
				entptr = &Sequence[ndx];
				if (*scriptptr == '!')
					entptr->audhead = 1;
				else
					entptr->audhead = 0;
				scriptptr++;
				sscanf(scriptptr,"%ld %ld %ls %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %ld %lx %lx %lx %lx\n",
					&entptr->clock,
					&entptr->drive,
					entptr->name,
					&entptr->vidflag,
					&entptr->chan,
					&entptr->vstartfld,
					&entptr->vfldlen,
					&entptr->audchans,
					&entptr->astartfld,
					&entptr->afldlen,
					&entptr->atotalfld,
					&entptr->atotallen,
					&entptr->attack,
					&entptr->decay,
					&entptr->vol1,
					&entptr->vol2,
					&entptr->pan1,
					&entptr->pan2
				);
				ndx++;
			}

	      do {
				if (scriptptr >= endptr)
					break;
			} while (*scriptptr++ != 10);
		}

		for (i=0;i<ndx;i++) {
			entptr = &Sequence[i];
			printf("Step %ld at %ld: %ls from drive %ld\n",i+1,entptr->clock,entptr->name,entptr->drive);
			printf("          vflag=%ld, vchan=%ld, vflds=%ld/%ld\n",
				entptr->vidflag,entptr->chan,entptr->vstartfld,entptr->vfldlen);
		}

		if (heads) {
			/* Send any needed heads to Flyer to make now */
			printf("Making needed heads...\n");
			res = StartHeadList(0);

			for (i=0;i<ndx;i++) {
				entptr = &Sequence[i];
				if ((i==0)&&(entptr->audhead==0))		/* First one will have no conflicts */
					continue;

				prev = &Sequence[i-1];

				/* Test for overlap from same drive */
				if ((entptr->audhead)||((entptr->drive == prev->drive) &&
					 (entptr->clock < (prev->clock + prev->vfldlen)))) {

					volptr = &FlyVol[0];
					volptr->Flags = FVF_USENUMS;
					volptr->Board = 0;
					volptr->Path = entptr->name;
					volptr->SCSIdrive = entptr->drive;

					actptr = &Action[0];
					actptr->Volume = volptr;
					actptr->ReturnTime = RT_STOPPED;
					if (entptr->audhead) {
						actptr->AudStartField = entptr->astartfld;
						actptr->AudFieldCount = entptr->afldlen;
						actptr->VidStartField = 0;
						actptr->VidFieldCount = 0;
					}
					else {
						actptr->VidStartField = entptr->vstartfld;
						endofprev = prev->clock + prev->vfldlen;
						endofthis = entptr->clock + entptr->vfldlen;
						if (endofprev < endofthis)
							actptr->VidFieldCount = endofprev - entptr->clock;
						else
							actptr->VidFieldCount = endofthis - entptr->clock;
						actptr->AudStartField = 0;
						actptr->AudFieldCount = 0;
					}

					printf("Need Head! Step %ld (%ld,%ld) -- ",i+1,
						actptr->VidStartField,actptr->VidFieldCount);

					if (WaitHere(stepmode))
						goto abort;
	
					res = MakeClipHead(actptr);
				}
			}

			printf("Ready to end list -- ");
			if (WaitHere(stepmode))
				goto abort;

			res = EndHeadList(0,1);

/* Start sequence now  */
			printf("Ready to play sequence -- ");
			if (WaitHere(stepmode))
				goto abort;

		}

		baseclock = 0;
		back1clock = back2clock = 0;

		for (i=0;i<ndx;i++) {
			entptr = &Sequence[i];

			volptr = &FlyVol[i & 1];
			actptr = &Action[i & 1];

			back2clock = back1clock;

			/* Start clip playing */

			volptr->Board = 0;
			volptr->Flags = FVF_USENUMS;
			volptr->Path = entptr->name;
			volptr->SCSIdrive = entptr->drive;

			actptr->Channel = entptr->chan;
			actptr->Flags = 0;
			if (entptr->vidflag)
				actptr->Flags |= CAF_VIDEO;
			if (entptr->audchans == 1)
				actptr->Flags |= CAF_AUDIOL;
			if (entptr->audchans == 2)
				actptr->Flags |= CAF_AUDIOL | CAF_AUDIOR;
#if 0
			if (i>0)
				actptr->Flags |= CAF_NOPREROLL;
#endif
//			actptr->Flags |= CAF_USEMATTE;
			actptr->PermissFlags = /* CAPF_STEALOURVIDEO | */ CAPF_KILLOTHERVIDEO
				| CAPF_USEHEADS;

			actptr->GoClock = baseclock+entptr->clock;
			actptr->VidStartField = entptr->vstartfld;
			actptr->VidFieldCount = entptr->vfldlen;
			actptr->AudStartField = entptr->astartfld;
			actptr->AudFieldCount = entptr->afldlen;
			actptr->TotalAudStart = entptr->atotalfld;
			actptr->TotalAudLength = entptr->atotallen;
			actptr->VolSust1 = entptr->vol1;
			actptr->VolSust2 = entptr->vol2;
			actptr->AudioPan1 = entptr->pan1;
			actptr->AudioPan2 = entptr->pan2;
			actptr->VolAttack = entptr->attack;
			actptr->VolDecay = entptr->decay;
			back1clock = actptr->GoClock + actptr->VidFieldCount;

			actptr->Volume = volptr;
			if (i==0)
				actptr->ReturnTime = RT_STARTED;
			else
				actptr->ReturnTime = RT_IMMED;

			printf("Playing %ls...(@ %lx)\n",entptr->name,actptr->GoClock);
			res = FlyerPlay(actptr);
			if (res) {
				printf("Play Error %ld at step %ld - %ls\n",res,i+1,
					Error2String(res));
				break;
			}

			if (i==0) {
				back1clock = actptr->VidFieldCount;
				/* Get clock at the instant the sequence starts */
				res = GetFieldClock(&baseclock);
				baseclock += 8;		/* Starts in 2 frames */
				if (res)
					printf("Error %ld - %ls\n",res,Error2String(res));
				else
					printf("BaseClk = %lx\n",baseclock);

				back1clock += baseclock;	/* When this should end */
			}
			else if (entptr->audhead == 0) {
				printf("Waiting...(for %lx)\n",back2clock);
				do {
					res = GetFieldClock(&clock);
					if (res) {
						printf("Error %ld - %ls\n",res,Error2String(res));
						break;
					}
				}
				while (clock < back2clock);
			}
			else {
				printf("Shove on!\n");
			}
		}
	}

abort:
	CloseLibrary(FlyerBase);
	FreeFile();

	return(0);
}


ULONG ReadScriptFile(char *name)
{
	FILE	*fp;
	int	read;

	read = 0;
	fp = fopen(name,"r");
	if (fp) {
		fseek(fp,0,SEEK_END);
		FileLength = ftell(fp);
		fseek(fp,0,SEEK_SET);

		FileBuffer = AllocMem(FileLength,MEMF_PUBLIC);
		if (FileBuffer) {
			read = fread(FileBuffer,FileLength,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",name);

	if (read==1)
		return(FileLength);
	else
		return(0);
}

void FreeFile(void)
{
	if (FileLength)
		FreeMem(FileBuffer,FileLength);
}


int WaitHere(int stepmode)
{
	if (stepmode == 0) {
		printf("\n");
	}
	else {
		printf("Hit ENTER to proceed...\n");
		for (;;) {
			if (WaitForChar(Handle,30)) {
				if (getchar()=='q') {
					printf("Aborted\n");
					getchar();
					return(1);
				}
				printf("Here we go...\n");
				break;
			}
		}
	}
	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
