/*********************************************************************\
*
* SetupDrives - Setup Flyer hard drives
*
* $Id: SetupDrives.c,v 1.0 1995/05/05 15:53:00 Flick Exp $
*
* $Log: SetupDrives.c,v $
*Revision 1.0  1995/05/05  15:53:00  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	10/21/94		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

void SetBlkSize(int unit,int chan,int size);
void CacheOn(int unit,int chan);
void GetArg(char *arg,int *var,int lo,int hi,char *label);

struct DriveParam {
	UBYTE	scsiver,inqlen;
	ULONG	size;
};

struct Options {
	char	*optname;
	UBYTE	page;
	UBYTE	pos;
	UBYTE	flag;
	UBYTE	value;
};

struct Manuf {
	char	*manufname;
	struct Options *options;
};

struct Options IBMOptions[] = {
	"WCE",8,2,0x04,1,			/* WCE on */
	"MF",8,2,0x02,0,			/* MF off */
	"RCD",8,2,0x01,0,			/* RCD off */
	"Pri",8,3,0,0x11,			/* Pri = 11 */
	"Segs",8,13,0,4,			/* Segs = 4 */
	"ADC",0,8,0x40,0,			/* ADC off */
	0,0,0,0,0
};

struct Manuf Manufs[] = {
	"IBM     ",&IBMOptions[0],
	"IBM OEM ",&IBMOptions[0],
	0,0
};

struct Library *FlyerBase;
struct FlyerVolume FlyVol;

main(int argc,char **argv)
{
	int	unit,chan,i,j,len;
	ULONG	res,blkcnt,blklen;
	UBYTE	MiscBuff[18],drives,InqBuff[255];
	struct	DriveParam	Stats[8];

	if ((argc!=3)||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <channel>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	FlyVol.Board = unit;
	GetArg(argv[2],&chan,0,2,"Channel");
	FlyVol.SCSIdrive = (chan<<3)+0;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	res = Inquiry(&FlyVol,16,InqBuff);
	if (res) {
		printf("Inquiry Error %ld - %ls\n",res,Error2String(res));
	}
	InqBuff[16] = 0;

	for (manptr=&Manufs[0];manptr->manufname;manptr++) {
		if (stricmp(manptr->manufname,&InqBuff[8])==0)
			break;
	}
	if (manptr->manufname == 0) {
		printf("Error, drive '%ls' not supported\n",&InqBuff[8]);
		printf("Supported drives:\n");
		for (manptr=&Manufs[0];manptr->manufname;manptr++)
			printf("   %ls\n",manptr->manufname);
	}
	else {
		for (optptr=manptr->options;optptr->optname;optptr++) {
			ChangeItem(optptr,value);
		}
	}


	printf("Initializing SCSI channel\n");
	res = SCSIinit(&FlyVol);
	if (res) {
		printf("Error %ld - %ls\n",res,Error2String(res));
	}
	printf("Finding drives on bus\n");
	res = FindDrives(&FlyVol,MiscBuff);
	if (res) {
		printf("Error %ld - %ls\n",res,Error2String(res));
	}

	drives = MiscBuff[0];
	for (i=0;i<8;i++) {
		Stats[i].scsiver = MiscBuff[2+i];
		Stats[i].inqlen  = MiscBuff[10+i]+5;
	}

	for (i=0;i<7;i++) {
		printf("Drive %d\n",i);
		if ((1<<i)&drives) {			/* Drive was present? */
			FlyVol.SCSIdrive = (chan<<3)+i;

			len = Stats[i].inqlen;
			res = Inquiry(&FlyVol,Stats[i].inqlen,InqBuff);
			if (res) {
				printf("Inquiry Error %ld - %ls\n",res,Error2String(res));
			}

			printf("   ");
			for (j=8;j<=31;j++)
				printf("%c",InqBuff[j]);
			printf("\n   Rev: ");
			for (j=32;j<=35;j++)
				printf("%c",InqBuff[j]);
			if (len>=43) {
				printf("   Date: ");
				for (j=36;j<=43;j++)
					printf("%c",InqBuff[j]);
			}
			if (len>=55) {
				printf("   SerNum: ");
				for (j=44;j<=55;j++)
					printf("%c",InqBuff[j]);
			}
			printf("\n");

			res = ReadSize(&FlyVol,&blkcnt,&blklen);
			if (res) {
				printf("ReadSize Error %ld - %ls\n",res,Error2String(res));
			}
			Stats[i].size = blkcnt;
			printf("   %lx Blocks\n",blkcnt);
			printf("   Size is %ld\n",blklen);

#if 0
			if (blklen != 512) {
				SetBlkSize(unit,chan,512);
				res = ReadSize(&FlyVol,&blkcnt,&blklen);
				if (res) {
					printf("ReadSize Error %ld - %ls\n",res,Error2String(res));
				}
				printf("   Changed to: %ld\n",blklen);
			}
#endif

			/* Turn on (and setup) Drive Write Cache */
			CacheOn(unit,chan);
		}
		else
			printf("   Not Present\n");
	}

	CloseLibrary(FlyerBase);

	return(0);
}


struct CachePage {
	UBYTE	pagecode;
	UBYTE	paramlen;
	UBYTE	flags;
	UBYTE	pri;
	UWORD	disprelen;
	UWORD	minprefetch;
	UWORD	maxprefetch;
	UWORD	maxpreceil;
};

UBYTE	Buffer[256];

void CacheOn(int unit,int chan)
{
	UBYTE	pagelen;
	ULONG	res;
	int	i;
	UBYTE	*ptr;
	struct CachePage *CacheBuffer;

	/* Get length of Cache Page */
	res = ModeSense(&FlyVol,1,8,&pagelen);
	if (res) {
		printf("ModeSense1 Error %ld - %ls\n",res,Error2String(res));
	}

	pagelen++;

	/* Read Cache Page */
	res = ModeSense(&FlyVol,pagelen,8,&Buffer);
	if (res) {
		printf("ModeSense2 Error %ld - %ls\n",res,Error2String(res));
	}
	else {
		ptr = (UBYTE *)&Buffer;
		for (i=0;i<pagelen;i++)
			printf("%02lx ",*ptr++);
		printf("\n");

		CacheBuffer = (struct CachePage *)&Buffer[4];

		if (Buffer[3]) {		/* Returned block desc? */
			printf("Stripping block descriptor\n");
			Buffer[3] = 0;
			for (i=12;i<pagelen;i++)
				Buffer[i-8] = Buffer[i];
			pagelen -= 8;
		}

		if ((CacheBuffer->pagecode & 0x3F)==8) {
			printf("Flags = %lx\n",CacheBuffer->flags);
			printf("Priorities = %lx\n",CacheBuffer->pri);
			printf("Disable Prefetch Transfer Length = %lx\n",CacheBuffer->disprelen);
			printf("Min Prefetch = %lx\n",CacheBuffer->minprefetch);
			printf("Max Prefetch = %lx\n",CacheBuffer->maxprefetch);
			printf("Max Prefetch Ceil = %lx\n",CacheBuffer->maxpreceil);

			Buffer[0] = 0;
			CacheBuffer->pagecode &= 0x7F;	/* Strip off PS bit */
#if 0
			CacheBuffer->flags = 0x04;			/* Enable Write Caching */
			CacheBuffer->pri    = 0x11;		/* Set Used Cache Data to Lowest Priority */
			CacheBuffer->disprelen = 0xFFFF;	/* Disable prefetch length to max */
			CacheBuffer->minprefetch = 0x0000;		/* Min prefetch - none */
			CacheBuffer->maxprefetch = 0xFFFF;		/* Max prefetch - maximum */
			CacheBuffer->maxpreceil = 0xFFFF;		/* Max prefetch ceil - maximum */
#endif

			ptr = (UBYTE *)&Buffer;
			for (i=0;i<pagelen;i++)
				printf("%02lx ",*ptr++);
			printf("\n");

			res = ModeSelect(&FlyVol,pagelen,Buffer);
			if (res) {
				printf("ModeSelect Error %ld - %ls\n",res,Error2String(res));
			}
		}
		else
			printf("Did not return page 8!\n");
	}
}


void SetBlkSize(int unit,int chan,int size)
{
	UBYTE	blkdesc[12];
	ULONG	res;

	/* Get length of Cache Page */
	res = ModeSense(&FlyVol,12,0x3F,blkdesc);
	if (res) {
		printf("ModeSense512 Error %ld - %ls\n",res,Error2String(res));
	}
	else if (blkdesc[3]==0)
		printf("No block descriptor returned!\n");
	else {
		printf("Mode Sense done\n");
		blkdesc[0] = 0;
		blkdesc[9] = (size>>16) & 0xFF;
		blkdesc[10] = (size>>8) & 0xFF;
		blkdesc[11] = size & 0xFF;

		res = ModeSelect(&FlyVol,12,blkdesc);
		if (res) {
			printf("ModeSelect512 Error %ld - %ls\n",res,Error2String(res));
		}
		else
			printf("Mode Select done\n");
	}
}


void ChangeItem(struct	Options	*optptr,int value)
{
	UBYTE	pagelen,*pagebuffer;
	ULONG	res;
	int	i;
	UBYTE	*ptr;

	/* Get length of Page */
	res = ModeSense(&FlyVol,1,optptr->page,&pagelen);
	if (res) {
		printf("ModeSense1 Error %ld - %ls\n",res,Error2String(res));
	}

	pagelen++;
	if (pagelen > 255) {
		printf("Error - page length too long\n");
		return;
	}

	/* Read Page */
	res = ModeSense(&FlyVol,pagelen,optptr->page,&Buffer);
	if (res) {
		printf("ModeSense2 Error %ld - %ls\n",res,Error2String(res));
	}
	else {
		printf("Before: ");
		ptr = (UBYTE *)&Buffer;
		for (i=0;i<pagelen;i++)
			printf("%02lx ",*ptr++);
		printf("\n");

		Buffer[0] = 0;

		pagebuffer = &Buffer[4];

		if (Buffer[3]) {		/* Returned block desc? */
			Buffer[3] = 0;
			for (i=12;i<pagelen;i++)
				Buffer[i-8] = Buffer[i];
			pagelen -= 8;
		}

		if (optptr->flag) {
			if (value>0)
				pagebuffer[optptr->pos] |= optptr->flag;
			else if (value==0)
				pagebuffer[optptr->pos] &= (~optptr->flag);
			else {
				printf("%ls is ",optptr->optname);
				if (pagebuffer[optptr->pos] & optptr->flag)
					printf("TRUE\n");
				else
					printf("FALSE\n");
			}
		}
		else {
			if (value>=0)
				pagebuffer[optptr->pos] = value;
			else
				printf("%ls is currently %lx",
					optptr->optname,pagebuffer[optptr->pos]);

		}

		pagebuffer[0] &= 0x7F;

		printf("After: ");
		ptr = (UBYTE *)&Buffer;
		for (i=0;i<pagelen;i++)
			printf("%02lx ",*ptr++);
		printf("\n");

		res = ModeSelect(&FlyVol,pagelen,Buffer);
		if (res) {
			printf("ModeSelect Error %ld - %ls\n",res,Error2String(res));
		}
		else
			printf("Done\n");
	}

}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
