/*********************************************************************\
*
* TCalTest - Tests hard drives for Thermal Calibration delays
*
* $Id: TCalTest.c,v 1.0 1995/05/05 15:53:26 Flick Exp $
*
* $Log: TCalTest.c,v $
*Revision 1.0  1995/05/05  15:53:26  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	01/18/95		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>

#include "flyerlib.h"
#include "flyer.h"

#define	DBUG(x)			if (Debugging) { x }

// #define	TESTBLKS 128		/* Each transfer will be 512 blocks (256K) */
#define	REPEAT	64			/* Do 16 transfers for each test */
#define	TOTALSIZE	TestBlks*REPEAT

double DoTest(ULONG lba,ULONG blklen,ULONG repeat);
ULONG ElapsedTicks(struct DateStamp *begin, struct DateStamp *end);
void GetArg(char *arg,int *var,int lo,int hi,char *label);

struct	Library			*FlyerBase;
struct	FlyerVolume	FlyVol;

struct	DateStamp	TestStart,Now;
int		WriteFlag;
int		Debugging;
int		TestBlks;

main(int argc,char **argv)
{
	int	drive,first,last,samples,minutes,WriteFlag;
	ULONG	lba,blklen,max,res;
	double spd,lowest,totalspd;

	if ((argc == 6) && (stricmp(argv[5],"debug")==0)) {
		Debugging = 1;
		argc--;
	}

	if ((argc!=5)||(argv[1][0]=='?')) {
		printf("Usage: %ls <drive> <minutes> <READ|WRITE> <blocks/xfer>\n",argv[0]);
		return(20);
	}

	FlyVol.Board = 0;		/* 1st Flyer board */
	GetArg(argv[1],&drive,0,23,"Drive");
	FlyVol.SCSIdrive = drive;
	GetArg(argv[2],&minutes,0,999,"Minutes");
	GetArg(argv[4],&TestBlks,1,768,"Blocks/xfer");

	if (stricmp("write",argv[3])==0)
		WriteFlag = 1;
	else
		WriteFlag = 0;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	/* Get drive specs: # blocks, block size */
	res = ReadSize(&FlyVol,&max,&blklen);
	if (res) {
		printf("ReadSize Error %ld - %ls\n",res,Error2String(res));
		exit(20);
	}

	printf("Drive has %ld blocks\n",max);

	printf("Doing drive %ls test...\n",(WriteFlag?"write":"read"));

	samples = 0;
	lowest = 9999.0;
	totalspd = 0.0;

	DateStamp(&TestStart);			/* Log time we started test */

	first = 1;
	last = 0;
	for (lba=0;!last;lba+=TOTALSIZE,first=0) {
		/* Prevent last test from going past end of drive */
		if ((lba+TOTALSIZE) >= max) {
			lba = max-TOTALSIZE;
			last = 1;		/* This is the last test */
		}
		spd = DoTest(lba,TestBlks,REPEAT);
		if (spd < 0.1)			/* Some major problem? */
			break;

		DBUG(printf("LBA %lx ---> %.2f\n",lba,spd);)

		/* Record lowest speed (forgive first speed) */
		if ((!first) && (spd < lowest))
			lowest = spd;

		/* Keep average for entire session */
		totalspd += spd;
		samples++;

		/* If number of minutes specified has elapsed, quit */
		DateStamp(&Now);				/* Log time now */
		if ((ElapsedTicks(&TestStart,&Now)/(60*50)) >= minutes)
			break;
	}

	if (last)
		printf("Reached end of drive\n");
	else
		printf("%ld minute(s) elapsed\n",minutes);

	printf("Performed %ld tests...\n",samples);
	printf("   Slowest speed was %.2f MB/sec\n",lowest);
	printf("   Average speed was %.2f MB/sec\n",totalspd/(double)samples);

	CloseLibrary(FlyerBase);

	return(0);
}


double DoTest(ULONG lba,ULONG testblks,ULONG repeat)
{
	struct	DateStamp	start,end;
	double	meg,speed,seconds;
	ULONG		ticks,res;

//	Forbid();

	DateStamp(&start);			/* Log time now */

	if (WriteFlag)
		res = WriteTest(&FlyVol,testblks,repeat,lba,1);
	else
		res = ReadTest(&FlyVol,testblks,repeat,lba,1);

	DateStamp(&end);				/* Log time now */

//	Permit();

	if (res) {
		printf("R/W Error %ld - %ls\n",res,Error2String(res));
		return(0);
	}
	else {
		/* Compute stats */
		meg = ((double)(testblks * repeat))/2048.0;

		ticks = ElapsedTicks(&start,&end);
		seconds = ((double)ticks)/50;

		speed = meg/((double)seconds);

#if 0
		printf("%06lx --> %.2f\n",lba,speed);
#endif

		return(speed);
	}
}


ULONG ElapsedTicks(struct DateStamp *begin, struct DateStamp *end)
{
	ULONG		ticks;

	ticks = end->ds_Minute * 60 *50 + end->ds_Tick;
	ticks -= begin->ds_Minute * 60*50 + begin->ds_Tick;

	return(ticks);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stcd_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
