/*********************************************************************\
*
* TestDSPram - Does a R/W test on the DSP SRAM
*
* $Id: TestDSPram.c,v 1.0 1995/11/06 16:41:58 Flick Exp $
*
* $Log: TestDSPram.c,v $
*Revision 1.0  1995/11/06  16:41:58  Flick
*FirstCheckIn
*
*
* Copyright (c) 1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	10/19/95		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

UBYTE Pattern(int input);

struct Library *FlyerBase;
int	pattern;

//#define	DSPSIZE	16384
#define	SETADDR	0

main(int argc,char **argv)
{
	int	iters,pass,i,delay,depth;
	ULONG	errors,total=0;
	UBYTE	data;
	BOOL	showall=FALSE;

	if (stricmp(argv[argc-1],"SHOW")==0)
	{
		showall = TRUE;
		argc--;
	}

	if ((argc!=5)||(argv[1][0]=='?')) {
		printf("Usage: %ls <iterations> <depth> <delay ms> <pattern> [SHOW]\n",argv[0]);
		return(20);
	}

	stcd_i(argv[1],&iters);
	stcd_i(argv[2],&depth);
	stcd_i(argv[3],&delay);
	stch_i(argv[4],&pattern);

	delay /= 20;		// Convert ms to ticks

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	printf("\nThis test uses the Flyer high-speed serial bus to do read/write\n");
	printf("tests of the DSP SRAM memory block.  Errors detected could be\n");
	printf("indicative of problems with the SRAM or the serial bus used\n");
	printf("to accomplish the test.\n");

	printf("\nTesting...\n");

	SBusWrite(0,0xEB,0);		// Reset DSP

//	Delay(delay);				// This doesn't prevent errors

	SBusWrite(0,0xEB,2);		// Now engage SRAM addressing

	// If we omit this delay, we will get some errors on our first pass (only if
	// DSP had been running).  Delay after DSP reset above does not seem to fix
	// the problem, so what are we fighting against here???
	Delay(delay);

	SBusWrite(0,0xEB,3);		// Now engage SRAM addressing


	for (pass=1; pass<=iters; pass++)
	{
		SBusWrite(0,0xE8,0);		// Reset address generator to base of SRAM
		SBusWrite(0,0xE9,0);

		for (i=0; i<depth; i++)
		{
#if SETADDR
			SBusWrite(0,0xE8,i & 0xFF);		// Set address each byte
			SBusWrite(0,0xE9,i>>8);
			Delay(1);
#endif
			SBusWrite(0,0xEA,Pattern(i));		// Write simple SRAM pattern
		}

		SBusWrite(0,0xE8,0);		// Reset address generator to base of SRAM
		SBusWrite(0,0xE9,0);

		errors = 0;
		for (i=0; i<depth; i++)
		{
#if SETADDR
			SBusWrite(0,0xE8,i & 0xFF);		// Set address each byte
			SBusWrite(0,0xE9,i>>8);
			Delay(1);
#endif

			SBusRead(0,0xEA,&data);			// Read back from SRAM
			if (data != Pattern(i))
			{
				errors++;
				if (showall)
					printf("%04lx: %02lx (%02lx)\n",i,data,Pattern(i));
			}

//			SBusRead(0,0xEB,&data);			// Hack to trigger increment
		}

		printf("Pass %ld -- %ld errors\n",pass,errors);
		total += errors;
	}

	SBusWrite(0,0xEB,0);		// Disengage SRAM addressing

	printf("----------\nTotal errors: %ld\n",total);

	printf("\nNow use 'BootDSP' to reload DSP software\n");

	CloseLibrary(FlyerBase);

	return(0);
}


UBYTE Pattern(int input)
{
	UBYTE	output;

	if (pattern)
		output = pattern;
	else
		output = (~(input+57)) & 0xFF;
//	output = 0xC2;

	return(output);
}
