/*********************************************************************\
*
* TestDirect - Test SCSI direct commands
*
* $Id: TestDirect.c,v 1.1 1995/05/10 15:15:33 Flick Exp $
*
* $Log: TestDirect.c,v $
*Revision 1.1  1995/05/10  15:15:33  Flick
*Added 4 block read test, improved data dump with some formatting
*
*Revision 1.0  1995/05/05  15:51:03  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	09/05/94		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>
#include <clib/dos_protos.h>
#include <devices/scsidisk.h>

#include "flyerlib.h"
#include "flyer.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);


struct Library *FlyerBase;
struct FlyerVolume FlyVol;

struct SCSICmd scsiinfo;

UBYTE BigBuffer[2048];
UBYTE SenseBuffer[20];
UBYTE CDB[12];

main(int argc,char **argv)
{
	int	unit,drive,test,i,dumpbuffer;
	ULONG	res;
	struct SCSICmd *scsiptr;

	if ((argc!=4)||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <drive> <test>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	FlyVol.Board = unit;
	GetArg(argv[2],&drive,0,23,"Drive");
	FlyVol.SCSIdrive = drive;
	GetArg(argv[3],&test,0,99,"Test");

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	scsiptr = &scsiinfo;

	scsiptr->scsi_Data = (UWORD *)BigBuffer;
	scsiptr->scsi_Length = 0;
	scsiptr->scsi_Command = CDB;
	scsiptr->scsi_CmdLength = 6;
	scsiptr->scsi_SenseData = SenseBuffer;
	scsiptr->scsi_SenseLength = 20;

	for (i=0;i<12;i++)
		CDB[i] = 0;

	dumpbuffer = 0;

	switch (test) {
	case (1):
		scsiptr->scsi_Flags = SCSIF_AUTOSENSE | SCSIF_READ;
		printf("Doing TUR...");
		break;
	case (2):
		CDB[0] = 0x12;
		CDB[4] = 60;
		scsiptr->scsi_Length = 60;
		scsiptr->scsi_Flags = SCSIF_AUTOSENSE | SCSIF_READ;
		printf("Doing Inquiry...");
		dumpbuffer = 1;
		break;
	case (3):
		CDB[0] = 0x28;
		CDB[2] = 0x0;
		CDB[3] = 0x00;
		CDB[4] = 0x00;
		CDB[5] = 0x10;

		CDB[8] = 1;

		scsiptr->scsi_CmdLength = 10;

		scsiptr->scsi_Length = 2048;
		scsiptr->scsi_Flags = SCSIF_AUTOSENSE | SCSIF_READ;
		printf("Doing ReadExt...");
		dumpbuffer = 1;
		break;
	case (4):
		CDB[0] = 0x8;
		CDB[1] = 0x0;
		CDB[2] = 0x00;
		CDB[3] = 0x00;

		CDB[4] = 4;

		scsiptr->scsi_CmdLength = 6;

		scsiptr->scsi_Length = 2048;
		scsiptr->scsi_Flags = SCSIF_AUTOSENSE | SCSIF_READ;
		printf("Doing Read 4...");
		dumpbuffer = 1;
		break;
	case (5):
		CDB[0] = 0x49;
		scsiptr->scsi_CmdLength = 10;
		scsiptr->scsi_Length = 16;
		scsiptr->scsi_Flags = SCSIF_AUTOSENSE | SCSIF_READ;
		printf("Doing weird...");
		break;
	default:
		printf("Illegal item\n");
	}

	res = FlyerSCSIdirect(unit,drive,&scsiinfo,sizeof(struct SCSICmd));
	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
	else
		printf("Okay\n");

	printf("Sense length = %ld\n",scsiptr->scsi_SenseActual);
	for (i=0;i<scsiptr->scsi_SenseActual;i++)
		printf("%02lx ",SenseBuffer[i]);
	printf("\n");

	if (dumpbuffer) {
		printf("Actual length = %ld\n",scsiptr->scsi_Actual);
		for (i=0;i<scsiptr->scsi_Actual;i++) {
			if ((i % 16) == 0)
				printf("\n%lx: ",i);
			printf("%02lx ",BigBuffer[i]);
		}
		printf("\n");
	}

	CloseLibrary(FlyerBase);

	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
