/*********************************************************************\
*
* TestDirect2 - Test SCSI direct bridge
*
* $Id: TestDirect2.c,v 1.0 1995/05/05 15:51:06 Flick Exp $
*
* $Log: TestDirect2.c,v $
*Revision 1.0  1995/05/05  15:51:06  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	09/05/94		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>
#include <clib/dos_protos.h>
#include <devices/scsidisk.h>

void GetArg(char *arg,int *var,int lo,int hi,char *label);

int OpenFlyer(int unit);
void CloseFlyer(void);
struct SCSICmd scsiinfo;

UBYTE BigBuffer[2048];
UBYTE SenseBuffer[20];
UBYTE CDB[12];

struct	MsgPort		*MyPort;
struct	IOStdReq		*Req;
int		OpenError = 1;

main(int argc,char **argv)
{
	int	unit,test,i,dumpbuffer;
	struct SCSICmd *scsiptr;

	if ((argc!=3)||(argv[1][0]=='?')) {
		printf("Usage: %ls <unit> <test>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,999,"Unit");
	GetArg(argv[2],&test,0,99,"Test");

	if (OpenFlyer(unit)) {
		printf("Couldn't open flyerscsi.device\n");
		CloseFlyer();
		exit(20);
	}

	scsiptr = &scsiinfo;

	scsiptr->scsi_Data = (UWORD *)BigBuffer;
	scsiptr->scsi_Length = 0;
	scsiptr->scsi_Command = CDB;
	scsiptr->scsi_CmdLength = 6;
	scsiptr->scsi_SenseData = SenseBuffer;
	scsiptr->scsi_SenseLength = 20;

	for (i=0;i<12;i++)
		CDB[i] = 0;

	dumpbuffer = 0;

	switch (test) {
	case (1):
		scsiptr->scsi_Flags = SCSIF_AUTOSENSE | SCSIF_READ;
		printf("Doing TUR...");
		break;
	case (2):
		CDB[0] = 0x12;
		CDB[4] = 60;
		scsiptr->scsi_Length = 60;
		scsiptr->scsi_Flags = SCSIF_AUTOSENSE | SCSIF_READ;
		printf("Doing Inquiry...");
		dumpbuffer = 1;
		break;
	case (3):
		CDB[0] = 0x28;
		CDB[2] = 0x0;
		CDB[3] = 0x00;
		CDB[4] = 0x00;
		CDB[5] = 0x00;

		CDB[8] = 1;

		scsiptr->scsi_CmdLength = 10;

		scsiptr->scsi_Length = 2048;
		scsiptr->scsi_Flags = SCSIF_AUTOSENSE | SCSIF_READ;
		printf("Doing ReadExt...");
		dumpbuffer = 1;
		break;
	case (4):
		CDB[0] = 0x49;
		scsiptr->scsi_CmdLength = 10;
		scsiptr->scsi_Length = 16;
		scsiptr->scsi_Flags = SCSIF_AUTOSENSE | SCSIF_READ;
		printf("Doing weird...");
		break;
	case (5):
		CDB[0] = 3;
		CDB[4] = 14;
		scsiptr->scsi_Length = 14;
		scsiptr->scsi_Flags = SCSIF_AUTOSENSE | SCSIF_READ;
//		scsiptr->scsi_Flags = SCSIF_READ;
		printf("Doing ReqSense...");
		dumpbuffer = 1;
		scsiptr->scsi_SenseData = BigBuffer;
		scsiptr->scsi_SenseLength = 20;
		break;
	default:
		printf("Illegal item\n");
	}

	Req->io_Command = 28;
	Req->io_Data = (APTR)&scsiinfo;
	Req->io_Length = sizeof(struct SCSICmd);

	DoIO((struct IORequest *)Req);					/* Do it! */

	if (Req->io_Error)
		printf("Error %ld\n",Req->io_Error);
	else
		printf("Okay\n");

	printf("Sense length = %ld\n",scsiptr->scsi_SenseActual);
	for (i=0;i<scsiptr->scsi_SenseActual;i++)
		printf("%02lx ",SenseBuffer[i]);
	printf("\n");

	if (dumpbuffer) {
		printf("Actual length = %ld\n",scsiptr->scsi_Actual);
		for (i=0;i<scsiptr->scsi_Actual;i++)
			printf("%02lx ",BigBuffer[i]);
		printf("\n");
	}

	CloseFlyer();

	return(0);
}


int OpenFlyer(int unit)
{
	OpenError = 1;

	MyPort = (struct MsgPort *)CreatePort(0,0);
	if (!MyPort)
		return(-1);
	Req = (struct IOStdReq *)CreateStdIO(MyPort);
	if (!Req) {
		DeletePort(MyPort);
		return(-1);
	}

	OpenError = OpenDevice("flyerscsi.device",unit,
		(struct IORequest *)Req,0);

	return(OpenError);
}


void CloseFlyer(void)
{
	if (!OpenError)
		CloseDevice((struct IORequest *)Req);
	if (Req)
		DeleteStdIO(Req);
	if (MyPort)
		DeletePort(MyPort);

	OpenError = 1;
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
