/*********************************************************************\
*
* TestFS - Test FileSystem functions
*
* $Id: testfs.c,v 1.3 1997/01/31 18:03:57 Hayes Exp $
*
* $Log: testfs.c,v $
*Revision 1.3  1997/01/31  18:03:57  Hayes
*changed to work with 64bit FlyerFileSeek
*
*Revision 1.2  1995/11/06  13:46:43  Flick
*Added test 10 for reading first 32 bytes of a file
*
*Revision 1.1  1995/08/11  12:55:35  Flick
*Prevent FlyerBase to FS->FlyBase (see FlyerFS.h)
*Added test 9: set a file's date
*
*Revision 1.0  1995/05/05  15:53:30  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	08/05/94		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <dos/dos.h>
#include <dos/datetime.h>
#include <intuition/intuition.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"
#include "flyerFS.h"

#define	FlyerBase	FlyerBase

void GetArg(char *arg,int *var,int lo,int hi,char *label);

struct FileInfoBlock fib;
struct DateTime	stamp;
struct DateStamp	now;
BPTR	Handle;

UBYTE Buffer[32];
struct Library *FlyerBase;
struct FlyerVolume FlyVol;
struct ClipAction	 Action;

UBYTE TestString[] = "This is a test file that TestFS wrote...";
UBYTE	ReadBuff[64];

main(int argc,char **argv)
{
	int	succ,test,drive,i;
	BPTR	lock;
	ULONG	grip,fileid,actual,res;
	char	DateBuff[20],TimeBuff[20];
	LONG	oldpos,act2;

	if ((argc<4)||(argv[1][0]=='?')) {
		printf("Usage: %ls <test> <drive> <name> [<name2>] [<name3>]\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&test,0,99,"Test code");
	GetArg(argv[2],&drive,0,23,"Drive");

	FlyVol.Board = 0;
	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	FlyVol.Flags = FVF_USENUMS;
	FlyVol.Path = argv[3];
	FlyVol.SCSIdrive = drive;
	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_STOPPED;
	Action.Grip = 0;

	switch (test) {
	case 0:		/* NameFromLock */
		lock = Lock(argv[3],ACCESS_READ);
		if (lock) {
			succ = NameFromLock(lock,Buffer,32);
			printf("Result: %ld '%ls'\n",succ,Buffer);
			UnLock(lock);
		}
		res = 0;
		break;
	case 1:		/* Create file in sub-directory */
		FlyVol.Path = argv[3];
		Action.Grip = 0;
		res = FlyerLocate(&Action);
		if (res==0) {
			grip = Action.Grip;
			printf("Located %ls, grip = %lx\n",argv[3],grip);

			FlyVol.Path = argv[4];
			Action.Access = FLYER_MODE_OUTPUT;
			res = FlyerFileOpen(&Action);
			if (res==0) {
				fileid = Action.FileID;
				FlyVol.Path = 0;
				res = FlyerFileWrite(&FlyVol,fileid,
					sizeof(TestString),TestString,&actual);
				if (res)
					printf("Error writing file %ld\n",res);

				res = FlyerFileClose(&FlyVol,fileid);

				printf("Wrote %ls, grip = %lx\n",argv[4],fileid);
				FlyerFreeGrip(&FlyVol,fileid);
			}
			FlyerFreeGrip(&FlyVol,grip);
		}
		break;
	case 2:		/* Create dir in sub-directory */
		FlyVol.Path = argv[3];
		Action.Grip = 0;
		res = FlyerLocate(&Action);
		if (res==0) {
			grip = Action.Grip;
			printf("Located %ls, grip = %lx\n",argv[3],grip);

			FlyVol.Path = argv[4];
			res = FlyerCreateDir(&Action);
			if (res==0) {
				printf("Created %ls, grip = %lx\n",argv[4],Action.Grip);
				FlyerFreeGrip(&FlyVol,Action.Grip);
			}
			FlyerFreeGrip(&FlyVol,grip);
		}
		break;
	case 3:		/* Open file */
		FlyVol.Path = argv[3];
		Action.Grip = 0;
		res = FlyerLocate(&Action);
		if (res==0) {
			grip = Action.Grip;
			printf("Located %ls, grip = %lx\n",argv[3],grip);

			FlyVol.Path = argv[4];
			Action.Access = FLYER_MODE_INPUT;
			res = FlyerFileOpen(&Action);
			if (res==0) {
				fileid = Action.FileID;

				res = FlyerFileClose(&FlyVol,fileid);

				printf("Opened file %ls, id = %lx\n",argv[4],fileid);
				FlyerFreeGrip(&FlyVol,fileid);
			}
			FlyerFreeGrip(&FlyVol,grip);
		}
		break;
	case 4:		/* Rename */
		FlyVol.Path = argv[3];
		Action.Grip = 0;
		res = FlyerLocate(&Action);
		if (res==0) {
			grip = Action.Grip;
			printf("Located %ls, grip = %lx\n",argv[3],grip);

			FlyVol.Path = argv[4];
			res = FlyerRename(&Action,grip,argv[5]);
			if (res==0) {
				printf("Renamed (%lx)%ls to (%lx)%ls\n",grip,argv[4],grip,argv[5]);
			}
			FlyerFreeGrip(&FlyVol,grip);
		}
		break;
	case 5:		/* Lock, get date */
		lock = Lock(argv[3],ACCESS_READ);
		if (lock) {
			succ = Examine(lock,&fib);
			stamp.dat_Stamp = fib.fib_Date;
			stamp.dat_Format = FORMAT_USA;
			stamp.dat_StrDate = DateBuff;
			stamp.dat_StrTime = TimeBuff;
			DateToStr(&stamp);
			printf("Result: %ld '%ls, %ls'\n",succ,DateBuff,TimeBuff);
			UnLock(lock);
		}
		res = 0;
		break;
	case 6:		/* Set Flyer's real-time clock */
		DateStamp(&now);
		res = SetFlyerTime(&now);
		break;
	case 7:		/* Open,seek end-x,read */
#if 0
		FlyVol.Path = argv[3];
		Action.Grip = 0;
		Action.Access = FLYER_MODE_INPUT;
		res = FlyerFileOpen(&Action);
		if (res==0) {
			fileid = Action.FileID;
			res = FlyerFileSeek(&FlyVol,fileid,-336,0x01,&newpos,&oldpos);
			printf("Err = %ld, Newpos = %ld, Oldpos = %ld\n",res,newpos,oldpos);
			res = FlyerFileSeek(&FlyVol,fileid,16,0x00,&newpos,&oldpos);
			printf("Err = %ld, Newpos = %ld, Oldpos = %ld\n",res,newpos,oldpos);

			FlyVol.Path = 0;
			res = FlyerFileRead(&FlyVol,fileid,sizeof(ReadBuff),ReadBuff,&actual);
			if (res)
				printf("Error reading file %ld\n",res);
			else {
				printf("Read %ls, grip = %lx\n",argv[3],fileid);
				for (i=0;i<sizeof(ReadBuff);i++)
					printf("%02lx ",ReadBuff[i]);
			}
			res = FlyerFileClose(&FlyVol,fileid);

			FlyerFreeGrip(&FlyVol,fileid);
		}
		else
			printf("Did not open successfully\n");
#endif
		Handle = Open(argv[3],MODE_OLDFILE);
		if (Handle) {
			oldpos = Seek(Handle,-336,OFFSET_END);
			printf("Oldpos = %ld\n",oldpos);
			oldpos = Seek(Handle,16,OFFSET_CURRENT);
			printf("Oldpos = %ld\n",oldpos);

			actual = Read(Handle,ReadBuff,sizeof(ReadBuff));
			printf("Result = %ld\n",actual);
			for (i=0;i<sizeof(ReadBuff);i++)
				printf("%02lx ",ReadBuff[i]);

			Close(Handle);
		}
		else
			printf("Did not open successfully\n");
		res = 0;
		break;
	case 8:		/* Find size by seeking */
		Handle = Open(argv[3],MODE_OLDFILE);
		if (Handle) {
			oldpos = Seek(Handle,0,OFFSET_END);
			printf("1st seek returned %lx\n",oldpos);
			oldpos = Seek(Handle,0,OFFSET_BEGINNING);
			printf("Size = %lx\n",oldpos);

			Close(Handle);
		}
		else
			printf("Did not open successfully\n");
		res = 0;
		break;
	case 9:		/* Set File's date */
		DateStamp(&now);
		res = SetFileDate(argv[3],&now);
		printf("Returned %ld\n",res);
		res = 0;
		break;
	case 10:		/* Dump first 32 bytes of file */
		Handle = Open(argv[3],MODE_OLDFILE);
		if (Handle)
		{
			act2 = Read(Handle,ReadBuff,32);
			if (act2 == -1)
			{
				printf("Failed, error code = %ld,%ld\n",act2,IoErr());
			}
			else
			{
				printf("Actual = %ld\n",act2);
				for (i=0; i<act2; i++)
					printf("%02lx ",ReadBuff[i]);
			}
			Close(Handle);
		}
		else
			printf("Did not open successfully\n");
		res = 0;
		break;
	default:
		break;
	}

	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
	else {
		printf("Okay\n");
	}

	CloseLibrary(FlyerBase);

	return(0);
}

void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
