/*********************************************************************\
*
* TestFlyer - Hardware checkout
*
* $Id: TestFlyer.c,v 1.1 1995/07/19 11:13:02 Flick Exp $
*
* $Log: TestFlyer.c,v $
*Revision 1.1  1995/07/19  11:13:02  Flick
*Revised for FlyerQuit() f'n name change
*
*Revision 1.0  1995/05/05  15:53:37  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	11/13/94		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <exec/libraries.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"
#include "flyerprivate.h"

void	GetArg(char *arg,int *var,int lo,int hi,char *label);
void	Cleanup(int code);
int	YesOrNo(char *prompt);
void	PromptWait(char *prompt);
int	TestSharedSRAM(int brd);
int	TestEEPROM(int brd);
int	TestDSP_SRAM(int brd);
int	TestDSP_SRAM2(int brd);
int	SerTestDSP_SRAM(int brd);
int	TestFIRcoef(int brd);
int	TestRS232(int brd);
int	TestCpuDma(int brd);

struct Library		*FlyerBase;
struct FlyerLib	*FlyLib;
struct FlyerUnit	*FlyUnit;

BPTR	handle;
int	arg1,arg2;

#define	TEST_SHAREDSRAM	0x01
#define	TEST_EEPROM			0x02
#define	TEST_SRAM			0x04
#define	TEST_CPUDMA			0x08
#define	TEST_SERIAL			0x10


main(int argc,char **argv)
{
	int	unit,brd,errors,tests;
	ULONG	res;
	BPTR	lock;

	errors = 0;

	handle = Input();

	if ((argc!=2)&&(argc!=4)) {
		printf("Usage: %ls <tests> [<arg1> <arg2>]\n",argv[0]);
		Cleanup(10);
	}

	GetArg(argv[1],&tests,0,0xFF,"Tests");
	if (argc==4) {
		GetArg(argv[2],&arg1,0,0xFFFF,"Arg 1");
		GetArg(argv[3],&arg2,0,0xFFFF,"Arg 2");
	}
	else {
		arg1 = 0x0000;
		arg2 = 0x37F0;
	}

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		Cleanup(10);
	}

	FlyLib = (struct FlyerLib *)FlyerBase;

	brd = -1;
	for (unit = 0;unit<4;unit++) {
		FlyUnit = &FlyLib->fl_Units[unit];
		if (FlyUnit->fu_SRAMbase) {
			if (brd<0) {
				printf("Flyer found at address %lx\n",FlyUnit->fu_SRAMbase);
				brd = unit;
			}
			else {
				printf("!!!Another Flyer found!!! This one at address %lx\n",FlyUnit->fu_SRAMbase);
			}
		}
	}

	/**************** Pre-init tests *****************/

	if (tests & TEST_SHAREDSRAM) {
		if (TestSharedSRAM(brd))
			Cleanup(10);
	}

	PromptWait("\nHit a key to boot test Flyer code");

	/* Try to Init */
	printf("Initializing Flyer...\n");
	lock = Lock("Flyer_Test",SHARED_LOCK);
	if (lock) {
		res = InitFlyers(lock);				/* Do it! */
		if (res) {
			printf("Flyer Init Error %ld\n",res);
			Cleanup(10);
		}
		else {
			printf("Okay\n");
		}
		UnLock(lock);
	}
	else {
		printf("Couldn't find 'Flyer_Test' directory\n");
		Cleanup(10);
	}

	if (tests & TEST_EEPROM) {
		if (TestEEPROM(brd))
			errors++;
	}

	if (tests & TEST_SRAM) {
		if (TestDSP_SRAM2(brd))
			errors++;
	}

#if 0
	if (TestFIRcoef(brd))
		errors++;
#endif

#if 0
	if (SerTestDSP_SRAM(brd))
		errors++;
#endif

	if (tests & TEST_CPUDMA) {
		if (TestCpuDma(brd))
			errors++;
	}

	if (tests & TEST_SERIAL) {
		if (TestRS232(brd))
			errors++;
	}

	printf("\nFinal report --- %ld errors occurred\n",errors);

	/**************** Shut down Flyer *****************/
	PromptWait("\nHit a key when ready to reset Flyer");

	res = FlyerQuit(brd);
	if (res) {
		printf("Quit Error %ld - %ls\n",res,Error2String(res));
	}
	else
		printf("Test complete\n");

	Cleanup(0);

	return(0);
}


void Cleanup(int code)
{
	if (FlyerBase)
		CloseLibrary(FlyerBase);

	if (code)
		exit(code);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}


int YesOrNo(char *prompt)
{
	char	c;

#if 0
	while (WaitForChar(handle,1000)) {			/* Flush any that are queued */
		getchar();
	}
#endif

#if 0
	do {
		c = getch();
		printf("(%ld) ",c);
		if (c == 0)
			break;
	} while (c == EOF);
#endif

	printf("%ls?\n",prompt);

#if 0
	while (!WaitForChar(handle,1000)) {}		/* Wait for new */
#endif

	c = getch();

	if ((c=='Y')||(c=='y'))
		return(1);
	else
		return(0);
}


void PromptWait(char *prompt)
{
#if 0
	while (WaitForChar(handle,30)) {			/* Flush any that are queued */
		getchar();
	}
#endif

	printf("%ls\n",prompt);

#if 0
	while (!WaitForChar(handle,30)) {}		/* Wait for new */
#endif
	getch();
}


int TestSharedSRAM(int brd)
{
	UWORD *ramlo,*ramhi;
	register UWORD *addr;
	UWORD	ptn;
	int	err;

	FlyUnit = &FlyLib->fl_Units[brd];
	ramlo = FlyUnit->fu_SRAMbase;

	/* Don't clobber ROM code variables, stack, vector table, etc. */
	ramhi = (UWORD *)(((ULONG)ramlo) + 0x3F000);

	/* Don't clobber 68000 vectors, avoid 1st command slot */
	ramlo = (UWORD *)(((ULONG)ramlo) + 0x00710);

	printf("\nTesting shared SRAM...\n");

	/* Write pattern to SRAM */
	ptn = 0x1234;
	addr = ramlo;
	while (addr < ramhi)
		*addr++ = ptn++;

	/* Check pattern in SRAM */
	ptn = 0x1234;
	addr = ramlo;
	err = 0;
	while (addr < ramhi) {
		if (*addr++ != ptn++) {
			err = 1;
			break;
		}
	}

	if (err) {
		printf("failed!\n");
		if (YesOrNo("Show errors")) {
			ptn = 0x1234;
			addr = ramlo;
			while (addr < ramhi) {
				if (*addr != ptn) {
					printf("%lx: is %04lx not %04lx\n",addr,*addr,ptn);
				}
				addr++;
				ptn++;
			}
		}
	}
	else {
		printf("Okay\n");
	}

	/* Clear SRAM back to 0's */
	addr = ramlo;
	while (addr < ramhi)
		*addr++ = 0;

	if (err)
		return(1);
	else
		return(0);
}


int TestEEPROM(int brd)
{
	ULONG	res,result;

	PromptWait("\nHit a key when ready to test EEPROM");

	printf("Testing EEPROM...\n");
	res = FlyerSelfTest(brd,1,0,0,&result);
	if (res)
		printf("Error %ld, result = %lx\n",res,result);
	else
		printf("Okay\n");

	if (res)
		return(1);
	else
		return(0);
}


int TestDSP_SRAM(int brd)
{
	ULONG	res,result;

	printf("Testing DSP SRAM...\n");
	printf("(Hit RETURN to exit test)\n");

	for (;;) {
		if (WaitForChar(handle,30)) {
			getchar();
			break;
		}

		res = FlyerSelfTest(brd,2,arg1,arg2,&result);
		if (res)
			printf("Error %ld, count = %lx\n",res,result);
		else
			printf("Okay\n");
	}

	if (res)
		return(1);
	else
		return(0);
}


int TestDSP_SRAM2(int brd)
{
	ULONG	res,result;

	printf("Testing DSP SRAM...\n");
	printf("(Hit RETURN to exit test)\n");

	for (;;) {
		if (WaitForChar(handle,30)) {
			getchar();
			break;
		}

		res = FlyerSelfTest(brd,7,arg1,arg2,&result);
		if (res)
			printf("Error, count = %lx\n",result);
		else
			printf("Okay\n");
	}

	if (res)
		return(1);
	else
		return(0);
}


int SerTestDSP_SRAM(int brd)
{
	ULONG	res,result;

	PromptWait("\nHit a key when ready to serially test DSP SRAM");

	printf("Testing...(Hit RETURN to abort)\n");
	res = FlyerSelfTest(brd,8,0,0,&result);
	if (res)
		printf("Error %ld, result = %lx\n",res,result);
	else
		printf("Okay\n");

	if (res)
		return(1);
	else
		return(0);
}


int TestFIRcoef(int brd)
{
	ULONG	res,result;

	PromptWait("\nHit a key when ready to test FIR coefficients");

	printf("Testing FIR coefficients...\n");
	res = FlyerSelfTest(brd,3,0,0,&result);
	if (res)
		printf("Error %ld, result = %lx\n",res,result);
	else
		printf("Okay\n");

	if (res)
		return(1);
	else
		return(0);
}


int TestRS232(int brd)
{
	ULONG	res,result;

	PromptWait("\nInstall RS232 loop cable and hit a key to test Serial Ports");

	do {
		printf("Testing Serial Ports...\n");
		res = FlyerSelfTest(brd,6,0,0,&result);
		if (res)
			printf("Error %ld, result = %lx\n",res,result);
		else {
			printf("Okay\n");
			break;
		}
	} while (YesOrNo("Test again"));


	if (res)
		return(1);
	else
		return(0);
}


int TestCpuDma(int brd)
{
	ULONG	res,result;

	PromptWait("\nHit a key when ready to test CPU DMA");

	printf("Testing...(Hit RETURN to abort)\n");
	res = FlyerSelfTest(brd,9,0,0,&result);
	if (res)
		printf("Error %ld, result = %lx\n",res,result);
	else
		printf("Okay\n");

	if (res)
		return(1);
	else
		return(0);
}
