/*********************************************************************\
*
* Tweak - Tweak mode page data
*
* $Id: Tweak.c,v 1.0 1995/05/05 15:53:47 Flick Exp $
*
* $Log: Tweak.c,v $
*Revision 1.0  1995/05/05  15:53:47  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	06/28/94		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);
void ChangeItem(struct	Options	*optptr,int value);
void DoModeSense(int page);
void SetBlkSize(int size);

struct Library *FlyerBase;
struct FlyerVolume FlyVol;


struct Options {
	char	*optname;
	UBYTE	page;
	UBYTE	pos;
	UBYTE	flag;
};

struct Manuf {
	char	*manufname;
	struct Options *options;
};


struct Options MicropOptions[] = {
	"WCE",8,2,0x04,
	"MinHi",8,6,0,
	"MinLo",8,7,0,
	0,0,0,0
};

struct Options QuantumOptions[] = {
	"AWRE",1,2,0x80,
	"RC",1,2,0x10,
	"DCR",1,2,0x01,
	"CorSpan",1,4,0,
	"HeadOff",1,5,0,
	"ServOff",1,6,0,
	"BufFull",2,2,0,
	"BufEmpty",2,3,0,
	"PE",0x37,2,0x02,
	"CE",0x37,2,0x01,
	"Segs",0x37,3,0,
	"RCD",8,2,0x01,
	"WCE",8,2,0x04,
	"Pri",8,3,0,
	"DisPreH",8,4,0,
	"DisPreL",8,5,0,
	"MinPreH",8,6,0,
	"MinPreL",8,7,0,
	"MaxPreH",8,8,0,
	"MaxPreL",8,9,0,
	0,0,0,0
};

struct Options SeagateOptions[] = {
	"IC",8,2,0x80,
	"CAP",8,2,0x20,
	"DISC",8,2,0x10,
	"SIZE",8,2,0x08,
	"WCE",8,2,0x04,
	"RCD",8,2,0x01,
	"Pri",8,3,0,
	"MaxHi",8,8,0,
	"MaxLo",8,9,0,
	"FSW",8,12,0x80,
	"DRA",8,12,0x20,
	"Segs",8,13,0,
	0,0,0,0
};

struct Options IBMOptions[] = {
	"WCE",8,2,0x04,
	"MF",8,2,0x02,
	"RCD",8,2,0x01,
	"Pri",8,3,0,
	"Segs",8,13,0,
	"ADC",0,8,0x40,
	"WPEN",0,5,0x04,
	0,0,0,0
};

struct Options HPOptions[] = {
	"WCE",8,2,0x04,
	"RCD",8,2,0x01,
	"Pri",8,3,0,
	0,0,0,0
};

struct Manuf Manufs[] = {
	"QUANTUM ",&QuantumOptions[0],
	"SEAGATE ",&SeagateOptions[0],
	"IBM     ",&IBMOptions[0],
	"IBM OEM ",&IBMOptions[0],
	"MICROP  ",&MicropOptions[0],
	"HP      ",&HPOptions[0],
	0,0
};

main(int argc,char **argv)
{
	int	unit,drive,value;
	ULONG	res;
	struct	Manuf	*manptr;
	struct	Options	*optptr;

	UBYTE	InqBuff[255];

	if (((argc!=4)&&(argc!=5))||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <drive> <option> [<value>]\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	FlyVol.Board = unit;
	GetArg(argv[2],&drive,0,23,"Drive");
	FlyVol.SCSIdrive = drive;
	if (argc==5)
		GetArg(argv[4],&value,0,2048,"Value");
	else
		value = -1;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	if (stricmp(argv[3],"DUMP")==0) {		/* Dump contents of mode page? */
		DoModeSense(value);
	}
	else if (stricmp(argv[3],"BLKSIZE")==0) {		/* Set new block size? */
		SetBlkSize(value);
	}
	else {													/* Misc parameters */
		/* Get Inquiry data from SCSI drive */
		res = Inquiry(&FlyVol,16,InqBuff);
		if (res) {
			printf("Inquiry Error %ld - %ls\n",res,Error2String(res));
		}
		InqBuff[16] = 0;

		for (manptr=&Manufs[0];manptr->manufname;manptr++) {
			/* Match of Manufacturer name? */
			if (stricmp(manptr->manufname,&InqBuff[8])==0)
				break;
		}
		if (manptr->manufname == 0) {
			printf("Error, no options table for drive '%ls'\n",&InqBuff[8]);
			printf("Supported drives:\n");
			for (manptr=&Manufs[0];manptr->manufname;manptr++)
				printf("   %ls\n",manptr->manufname);
		}
		else {
			for (optptr=manptr->options;optptr->optname;optptr++) {
				if (stricmp(optptr->optname,argv[3])==0)
					break;
			}
			if (optptr->optname == 0) {
				printf("Error, no option '%ls' found for drive '%ls'\n",
					argv[3],&InqBuff[8]);
				printf("Supported options:\n");
				for (optptr=manptr->options;optptr->optname;optptr++)
					printf("   %ls\n",optptr->optname);
			}
			else {
				ChangeItem(optptr,value);
			}
		}
	}

	CloseLibrary(FlyerBase);

	return(0);
}


UBYTE	Buffer[256];

void ChangeItem(struct	Options	*optptr,int value)
{
	UBYTE	pagelen,*pagebuffer;
	ULONG	res;
	int	i;
	UBYTE	*ptr;

	/* Get length of Mode Page */
	res = ModeSense(&FlyVol,1,optptr->page,&pagelen);
	if (res) {
		printf("ModeSense1 Error %ld - %ls\n",res,Error2String(res));
	}

	pagelen++;			/* Total length of data phase */

	if (pagelen > 255) {
		printf("Error - page length too long\n");
		return;
	}

	/* Read Page's current settings */
	res = ModeSense(&FlyVol,pagelen,optptr->page,&Buffer);
	if (res) {
		printf("ModeSense2 Error %ld - %ls\n",res,Error2String(res));
	}
	else {
		printf("Before: ");
		ptr = (UBYTE *)&Buffer;
		for (i=0;i<pagelen;i++)
			printf("%02lx ",*ptr++);
		printf("\n");

		Buffer[0] = 0;

		pagebuffer = &Buffer[4];

		if (Buffer[3]) {		/* Drive returned block descriptor? */
			Buffer[3] = 0;
			for (i=12;i<pagelen;i++)		/* Strip it out if so */
				Buffer[i-8] = Buffer[i];
			pagelen -= 8;
		}

		if (optptr->flag) {			/* Is the item to change a single bit? */
			if (value>0)
				pagebuffer[optptr->pos] |= optptr->flag;		/* Set it */
			else if (value==0)
				pagebuffer[optptr->pos] &= (~optptr->flag);	/* Clr it */
			else {
				printf("%ls is ",optptr->optname);
				if (pagebuffer[optptr->pos] & optptr->flag)
					printf("TRUE\n");
				else
					printf("FALSE\n");
			}
		}
		else {							/* Item to change is a byte */
			if (value>=0)
				pagebuffer[optptr->pos] = value;
			else
				printf("%ls is currently %lx",
					optptr->optname,pagebuffer[optptr->pos]);

		}

		pagebuffer[0] &= 0x7F;		/* Strip off PS bit before sending back */

		printf("After: ");
		ptr = (UBYTE *)&Buffer;
		for (i=0;i<pagelen;i++)
			printf("%02lx ",*ptr++);
		printf("\n");

		/* Send updated page back to drive */
		res = ModeSelect(&FlyVol,pagelen,Buffer,0x11);		//PF=1, SP=1
		if (res) {
			res = ModeSelect(&FlyVol,pagelen,Buffer,0x01);	//PF=0, SP=1
			if (res) {
				printf("ModeSelect Error %ld - %ls\n",res,Error2String(res));
			}
		}
		else
			printf("Done\n");
	}
}


void DoModeSense(int page)
{
	UBYTE	pagelen;
	ULONG	res;
	int	i;
	UBYTE	*ptr;

	/* Get length of Page */
	res = ModeSense(&FlyVol,1,page,&pagelen);
	if (res) {
		printf("ModeSense1 Error %ld - %ls\n",res,Error2String(res));
	}
	else {
		pagelen++;
		if (pagelen > 255) {
			printf("Error - page length too long\n");
			return;
		}

		/* Read Page */
		res = ModeSense(&FlyVol,pagelen,page,Buffer);
		if (res) {
			printf("ModeSense2 Error %ld - %ls\n",res,Error2String(res));
		}
		else {
			ptr = (UBYTE *)Buffer;
			for (i=0;i<pagelen;i++)
				printf("%02lx ",*ptr++);
			printf("\n");
		}
	}
}

void SetBlkSize(int size)
{
	ULONG	res;
	int	i;
	UBYTE	*ptr;

	/* Read header/blkdesc */
	res = ModeSense(&FlyVol,12,0x3F,&Buffer);
	if (res) {
		printf("ModeSense512 Error %ld - %ls\n",res,Error2String(res));
	}
	else if (Buffer[3]==0)
		printf("No block descriptor returned!\n");
	else {
		printf("Mode Sense done\n");
		Buffer[0] = 0;
		Buffer[2] = 0;
		Buffer[4] = 0;
		Buffer[5] = 0;
		Buffer[6] = 0;
		Buffer[7] = 0;
		Buffer[8] = 0;
		Buffer[9] = (size>>16) & 0xFF;
		Buffer[10] = (size>>8) & 0xFF;
		Buffer[11] = size & 0xFF;

		ptr = (UBYTE *)&Buffer;
		for (i=0;i<12;i++)
			printf("%02lx ",*ptr++);
		printf("\n");

		res = ModeSelect(&FlyVol,12,Buffer,0);
		if (res) {
			printf("ModeSelect512 Error %ld - %ls\n",res,Error2String(res));
		}
		else
			printf("Mode Select done\n");
	}
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
