/*********************************************************************\
*
* XPgm - Downloads FPGA chip data
*
* $Id: XPgm.c,v 1.0 1995/05/05 15:54:07 Flick Exp $
*
* $Log: XPgm.c,v $
*Revision 1.0  1995/05/05  15:54:07  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/23/94		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <exec/libraries.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"
#include "flyerprivate.h"

ULONG ReadBinFile(char *);
void FreeFile(void);
void GetArg(char *arg,int *var,int lo,int hi,char *label);
ULONG ASCIItoBinary(UBYTE *,ULONG);

struct Library *FlyerBase;

UBYTE	*Buffer;
ULONG	Length;
int	HexFile;

main(int argc,char **argv)
{
	int	unit,chipnum,retcode=0,rev;
	UBYTE	teachflag,*buff;
	struct 	FlyerChipHeader	*hdr;
	ULONG	res;
	LONG	size;

	if ((argc<4)||(argc>5)||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <chip> <file> [TEACH | rev]\n",argv[0]);
		printf("Chips:\n");
		printf(" 1) U42 - Clock Skew\n");
		printf(" 2) U38 - Timer/Router\n");
		printf(" 3) U29 - PCoder 1\n");
		printf(" 4) U35 - PCoder 2\n");
		printf(" 5) U31 - MCoder 1\n");
		printf(" 6) U37 - MCoder 2\n");
		printf(" 7) U92 - DMA Controller\n");
		printf(" 8) U1  - Audio Control\n");
		printf(" 9) U25 - Aligner\n");
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	GetArg(argv[2],&chipnum,0,9,"Chip number");

	if ((argc==5)&&(stricmp(argv[4],"TEACH")==0))
		teachflag = 1;
	else
		teachflag = 0;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	size = ReadBinFile(argv[3]);
	if (size) {
		hdr = (struct FlyerChipHeader *)Buffer;
		buff = Buffer;
		if (teachflag)
			res = TeachFPGA(chipnum,size,buff);
		else {
			if (HexFile == 0) {
				/* Make adjustments to trim off header data */
				/* Not used internally yet */
				buff += sizeof(struct FlyerChipHeader);
				size -= sizeof(struct FlyerChipHeader);
				rev = hdr->chiprev;
			}
			else {
				if ((argc==5)&&(!teachflag))
					stcd_i(argv[4],&rev);
				else
					rev = 0;
			}
			res = PgmFPGA(unit,chipnum,size,buff,rev);
		}
		if (res) {
			printf("Error %ld - %ls\n",res,Error2String(res));
			retcode = 10;
		}
		else
			printf("Chip download okay\n");
	}
	CloseLibrary(FlyerBase);
	FreeFile();

	return(retcode);
}


ULONG ReadBinFile(char *name)
{
	char	filename[80];
	FILE	*fp;
	int	read;
	ULONG	size;

	strcpy(filename,name);
	if (stricmp(".HEX",&filename[strlen(filename)-4])==0)
		HexFile = 1;
	else {
		HexFile = 0;
		if (stricmp(".BIN",&filename[strlen(filename)-4]))
			strcat(filename,".BIN");
	}

	read = 0;
	fp = fopen(filename,"r");
	if (fp) {
		fseek(fp,0,SEEK_END);
		Length = ftell(fp);
		fseek(fp,0,SEEK_SET);

		Buffer = AllocMem(Length,MEMF_PUBLIC);
		if (Buffer) {
			read = fread(Buffer,Length,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",filename);

	if (read==1) {
		if (HexFile)
			size = ASCIItoBinary(Buffer,Length);
		else
			size = Length;
		return(size);
	}
	else
		return(0);
}


ULONG ASCIItoBinary(UBYTE *buff,ULONG len)
{
	char 		*from,c;
	UBYTE 	*to,v,vt;
	ULONG		binlen;
	int		state;

	binlen = 0;
	from = (char *)buff;
	to = buff;
	state = 0;

	while (len) {
		c = *from++;
		c = toupper(c);
		if (isxdigit(c)) {
			if (c>'9')
				v = ((UBYTE)c)-'A'+10;
			else
				v = ((UBYTE)c)-'0';
			if (state==0) {
				vt = 16*v;
				state=1;
			}
			else {
				vt += v;
				state=0;
				*to++ = vt;
				binlen++;
			}
		}
		len--;
	}

	return(binlen);
}


void FreeFile(void)
{
	if (Length)
		FreeMem(Buffer,Length);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
