/*********************************************************************\
*
* BootFlyer - Downloads/runs Flyer firmware
*
* $Id: bootflyer.c,v 1.1 1995/08/04 10:54:40 Flick Exp $
*
* $Log: bootflyer.c,v $
*Revision 1.1  1995/08/04  10:54:40  Flick
*Added verify option to compare loaded code against .bin file
*
*Revision 1.0  1995/05/05  15:48:06  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/24/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "Flyer.h"

ULONG ReadBinFile(char *,ULONG);
void FreeFile(void);
void GetArg(char *arg,int *var,int lo,int hi,char *label);
void HomebrewVerify(ULONG size,UBYTE *Buffer, ULONG offset);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);

struct Library *FlyerBase;

UBYTE	*Buffer;
ULONG	Length;

BPTR	handle;

main(int argc,char **argv)
{
	int	unit;
	LONG	size;
	ULONG	res,offset;
	BOOL	verify = FALSE;

	if ((argc>3) && (!stricmp(argv[argc-1],"VERIFY")))
	{
		verify = TRUE;
		argc--;
	}

	offset = 0x20000;
	if (argc>3)
	{
		GetLongArg(argv[argc-1],&offset,0,0x3FFFF,"SRAM offset");
		argc--;
	}

	if ((argc!=3) || (argv[1][0]=='?'))
	{
		printf("Usage: %ls <board> <file> [<offset>] [VERIFY]\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	handle = Input();

	size = ReadBinFile(argv[2],0);
	if (size) {
		if (verify)
		{
			HomebrewVerify(size,Buffer,offset);
		}
		else
		{
			printf("Downloading program.  Size: %ld bytes\n",size);
			res = Firmware(unit,size,Buffer,offset);
			if (res)
				printf("Error %ld - %ls\n",res,Error2String(res));
			else
				printf("Program Downloaded\n");
		}
	}
	CloseLibrary(FlyerBase);
	FreeFile();

	return(0);
}


ULONG ReadBinFile(char *name,ULONG offset)
{
	char	filename[80];
	FILE	*fp;
	int	read;

	strcpy(filename,name);
	if (stricmp(".BIN",&filename[strlen(filename)-4]))
		strcat(filename,".BIN");

	read = 0;
	fp = fopen(filename,"r");
	if (fp) {
		fseek(fp,0,SEEK_END);
		Length = ftell(fp)-offset;
		fseek(fp,offset,SEEK_SET);

		Buffer = AllocMem(Length,MEMF_PUBLIC);
		if (Buffer) {
			read = fread(Buffer,Length,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",filename);

	if (read==1)
		return(Length);
	else
		return(0);
}


void FreeFile(void)
{
	if (Length)
		FreeMem(Buffer,Length);
}


void HomebrewVerify(ULONG size,UBYTE *Buffer, ULONG offset)
{
	UBYTE	*src,*dest;
	ULONG	count=0;

	src = Buffer;
	dest = (UBYTE *)(0xEC0000+offset);
	printf("Verifying data at %lx\n",dest);

	while (size--)
	{
		if ((*src++) != (*dest++))
		{
			printf("%lx: %02lx <> %02lx\n",dest-1,*(dest-1),*(src-1));
			count++;
		}
	}
	printf("Total of %ld errors\n",count);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}


void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
