/*********************************************************************\
*
* CalibNew - Flyer hardware calibration
*
* $Id: calibnew.c,v 1.0 1995/05/05 15:48:14 Flick Exp $
*
* $Log: calibnew.c,v $
*Revision 1.0  1995/05/05  15:48:14  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	11/01/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <dos/dos.h>
#include <clib/dos_protos.h>
#include <clib/MathIeeeSingTrans_protos.h>
#include <pragmas/MathIeeeSingTrans_pragmas.h>
#include <libraries/mathieeesp.h>

#if 0
#include <clib/alib_protos.h>
#include <clib/alib_stdio_protos.h>
#endif


#include "flyerlib.h"
#include "flyer.h"

#define	DBUGOPT(x)		if (debug & 1) { x }
#define	DBUG(x)			if (debug & 2) { x }
#define	DBUGACAL(x)		if (debug & 4) { x }
#define	DBUGVERB(x)		if (debug & 8) { x }
#define	DBUGRESULT(x)	if (debug & 16) { x }
#define	DBUGMATH(x)		if (debug & 32) { x }

/* Ped/Gain test */
#define	PGLINES			2		/* Number of lines to read when avging */
#define	PGPIXELSIN		40		/* Pixels in from edges to take samples */
#define	MAXCHROMA		2		/* Max chroma allowed for luma-only readings */
#define	NTSC_BLANK		60
#define	NTSC_BLACK		71
#define	NTSC_WHITE		200

/* Auto-hue test */
#define	TARGVECTOR		123.0		/* Target vector for auto-hue (+I vector) */
#define	SAMPLEFIELDS	1			/* Fields to average per sample */
#define	SAMPLELINES		2			/* Lines per field to average per sample */
#define	RECPIXELSIN		248		/* Start pixel for averaging */
#define	RECQUADS			64			/* Number of quads to sample */
#define	BURSTTHRESHLO	-4			/* Allowable delta avg.lum (D2 units) */
#define	BURSTTHRESHHI	4
#define	LOWOFFSET		19			/* Lowest pixel offset during record */
#define	SKEWSETTLETIME	4			/* Ticks for new skew setting to settle */
#define	GOODENUFF		0.2		/* Stop when we get this or better */
#define	JITTER			1.0		/* Allowable jitter (+/-) */

#define	SLOWDOWN			1

int	NoiseThreshLo,NoiseThreshHi;
int	Lowest,Highest,debug,Optimize;

int EnsureStable(FLOAT err);
int CheckRange(int edgelo, int coarselo, int finelo,
					int edgehi, int coarsehi, int finehi,
					FLOAT close);
void GetArg(char *arg,int *var,int lo,int hi,char *label);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);
ULONG OpenLiveField(UBYTE field);
ULONG CloseLiveField(void);
ULONG GetNextLine(UBYTE *buffer);
int ReturnHit(void);
int AvgQuadLuma(UBYTE *ptr,int *badvideo);
void YIQ(UBYTE *ptr);
int  AverageQuads(LONG *minI,LONG *minQ,LONG *plsI,LONG *plsQ);
void Doit(int value,int saveit);
void Doit3(int which,int value,int saveit);
void PlayClip(int channel,char *name,int length);
FLOAT CalcBurstPhase(int skipframes,int *badvidflag);

#define	POSSPAN	752*2
int		PosBase;

struct	Window			*Window;
struct	RastPort			*RP;

struct	IntuiMessage	*IntMsg;
ULONG		MIClass;
USHORT	theID;
int		AdjType,Toggle;

char		ValueBuffer[30];

struct Library	*MathIeeeSingTransBase;
struct Library *FlyerBase;

struct FlyerVolume	FlyVol;
struct ClipAction		Action;

UBYTE Buffer[SAMPLESPERLINE];


BPTR	handle;


struct TestType {
	char *typename;
	int	typenum;
};

#define	TEST_PedGain	1
#define	TEST_AutoRec	2
#define	TEST_ManualRec	3
#define	TEST_D2			4
#define	TEST_BadVid		5
#define	TEST_Genlock	6

struct TestType TestTypes[] = {
	"PG",TEST_PedGain,
	"AR",TEST_AutoRec,
	"MR",TEST_ManualRec,
	"D2",TEST_D2,
	"BV",TEST_BadVid,
	"GC",TEST_Genlock,
	0,0
};

main(int argc,char **argv)
{
	int	lo,hi,i,unit,test,do1,do2,do3;
	ULONG	res;
	FLOAT berr,besterr,result;
	int	offset,edge,coarse,fine;
	int	bestoffset,bestedge,bestcoarse,bestfine;
	int	badvideo,abort;
	UBYTE	data;
	FILE	*fp;
	UWORD	eeword;
/*	int	j,badval,totalval; */

#if 0
	stdin = Input();
	stdout = Output();
#endif

	test = 0;
	if (argc >= 4) {
		stcd_i(argv[3],&debug);
		argc--;
	}

	if (argc >= 3) {
		stcd_i(argv[2],&Optimize);
		argc--;
	}

	if (argc >= 2) {
		for (i=0;TestTypes[i].typename;i++) {
			if (stricmp(TestTypes[i].typename,argv[1])==0) {
				test = TestTypes[i].typenum;
				break;
			}
		}
	}

	if ((argc!=2)||(argv[1][0]=='?')||(test==0)) {
		printf("Usage: %ls <test> [<opt>]\n",argv[0]);
		printf("where 'test' is\n");
		printf("PG  - Calibrate Pedestal & Gain pots\n");
		printf("AR  - Auto-calibrate Record Phase\n");
		printf("Requires special test pattern input\n");
		return(20);
	}

	if((MathIeeeSingTransBase = OpenLibrary("mathieeesingtrans.library", 0)) == 0) {
		printf("Can't open MathIeeeSingTrans.library\n");
		exit(20);
	}

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	unit = 0;

	FlyVol.Board = unit;
	FlyVol.SCSIdrive = 0;
	FlyVol.Path = 0;
/*	Action.ReturnTime = RT_STOPPED; */

#if 0
	res = FIRcoef(unit,0,FIR100);
	Delay(25);
#endif

	handle = Input();

#if 0
	/* Digitize Toaster main out -- Sync from Toaster Main too */
	res = FlyerInputSel(unit,FI_ToasterMain,FS_ToasterMain);
#endif

	Delay(10);		/* Wait for new input to settle */
	
	switch (test) {
	case TEST_Genlock:
		/* Unlock Flyer */
		SBusWrite(unit,0x44,0xD);
		Delay(10);

		if (res = SBusRead(unit,0x44,&data),(data & 0x80)) {
			printf("Waiting...\n");
			while (res = SBusRead(unit,0x44,&data),(data & 0x80)) { }
			Delay(50);
		}

		res = SetFlooby(unit,0,4,0);		/* Tell PLL CPU to auto-cal and save */

		res = ReadEEreg(0,12,&eeword);
		if (eeword == 0xFFFF) {
			printf("Failed\n");
			result = 99.9;
		}
		else {
			printf("Done\n");
			result = 0.0;
		}

		/* Lock Flyer */
		SBusWrite(unit,0x44,0x5);
		Delay(80);

		break;
	case TEST_PedGain:
		res = RecordMode(unit);
		AdjType = 2;		/* Record */

		/* Calibrate input pedestal and gain */
		printf("Adjust gain and pedestal pots -- hit a key to exit\n\n");

		printf("Gain Error   Ped Error\n");

		for (;;) {
			if (WaitForChar(handle,30)) {
				getchar();
				break;
			}

			DBUG(printf("*** CAPTURE ***\n");)

			res = OpenLiveField(0);
#if 0
			res = GetNextLine(Buffer);
			lo = AvgQuadLuma(Buffer+PGPIXELSIN,&badvideo);
			hi = AvgQuadLuma(Buffer+SAMPLESPERLINE-PGPIXELSIN,&badvideo);
#endif
			res = GetNextLine(Buffer);		/* Skip opposite phase line */
			lo = hi = 0;
			for (i=0;i<PGLINES;i++) {
#if 0
				res = GetNextLine(Buffer);		/* Skip opposite phase line */
#endif
				res = GetNextLine(Buffer);
				lo += AvgQuadLuma(Buffer+PGPIXELSIN,&badvideo);
				if (badvideo)
					break;

				hi += AvgQuadLuma(Buffer+SAMPLESPERLINE-PGPIXELSIN,&badvideo);
				if (badvideo)
					break;
			}
			res = CloseLiveField();

			hi /= PGLINES;
			lo /= PGLINES;

			if (badvideo) {
				printf("   ---          ---   \n");

				DBUGVERB(
				for (i=0;i<32;i+=4)
					YIQ(&Buffer[i]);
				for (i=SAMPLESPERLINE-32;i<SAMPLESPERLINE;i+=4)
					YIQ(&Buffer[i]);
				)
			}
			else {
				printf("   %3ld         %3ld   \n",
					(hi-lo)-(NTSC_WHITE-NTSC_BLACK),
					lo-NTSC_BLACK);
			}
		}

		result = 0.0;
		break;
	case TEST_AutoRec:
		res = RecordMode(unit);
		AdjType = 2;	/* Record */

#if 0
		/* This causes Flyer to set baselines for 2 skew channels */
		res = SetFlooby(0,0,0x10,0);
#endif

		DBUG(printf("Auto calibrating record phase...(Hit RETURN to abort early)\n");)

		NoiseThreshLo = BURSTTHRESHLO;
		NoiseThreshHi = BURSTTHRESHHI;

		abort = 0;
		besterr = 9999.9;

		for (offset=LOWOFFSET;offset<=(LOWOFFSET+3);offset++) {
			if (WaitForChar(handle,30)) {
				getchar();
				abort = 1;
				break;
			}

			res = FlyerWriteCalib(0,CALIB_HRECOFFSETA,offset,0);

			/* Check 'edge' range to see if we can skip entire offset */
			DBUGOPT(printf("[%ld,xx,xx,xx]",offset);)
			do1 = CheckRange(0,0,0, 3,9,7, (float)100.0);

			for (edge=0;(do1)&&(edge<4);edge++) {
				if (WaitForChar(handle,30)) {
					getchar();
					abort = 1;
					break;
				}

				res = FlyerWriteCalib(0,CALIB_ADC_PHASE_EDGE,edge,0);

				/* Check 'coarse' range to see if we can skip entire edge */
				DBUGOPT(printf("[%ld,%ld,xx,xx]",offset,edge);)
				do2 = CheckRange(-1,0,0, -1,9,7, (float)50.0);

				for (coarse=0;(do2)&&(coarse<10);coarse++) {
					if (WaitForChar(handle,30)) {
						getchar();
						abort = 1;
						break;
					}

					res = FlyerWriteCalib(0,CALIB_ADC_PHASE_COURSE,coarse,0);

					/* Check 'fine' range to see if we can skip entire coarse */
					DBUGOPT(printf("[%ld,%ld,%ld,xx]",offset,edge,coarse);)
					do3 = CheckRange(-1,-1,0, -1,-1,7, (float)20.0);

					for (fine=0;(do3)&&(fine<8);fine++) {
						if (WaitForChar(handle,30)) {
							getchar();
							abort = 1;
							break;
						}

						res = FlyerWriteCalib(0,CALIB_ADC_PHASE_FINE,fine,0);

						DBUGRESULT(printf("[%ld,%ld,%ld,%ld]",offset,edge,coarse,fine);)
						berr = CalcBurstPhase(0,&badvideo)-TARGVECTOR;
						if (badvideo) {		/* Skip this one if bad */
							DBUGRESULT(printf("(bad)\n");)
							berr = 0.0;
							continue;
						}
						else {
							DBUGRESULT(printf("(%.1f)\n",berr);)

							if (berr < 0.0)
								berr = -berr;

							if (berr < besterr) {
								if (EnsureStable(berr)) {	/* Stable enough to use? */
									besterr = berr;

									bestoffset = offset;
									bestedge = edge;
									bestcoarse = coarse;
									bestfine = fine;

									DBUG(printf("New best score: %.1f    \n",berr);)

									if (berr <= GOODENUFF) {
										abort = 1;
										break;
									}
								}
							}
						}
#if 0
						if (berr > 40.0)		/* More than 2 coarse LSB's off? */
							break;				/* Then skip all 'fine' values */
#endif
					}
					if ((Optimize==1)&&	/* Gross pixel error optimization? */
						(berr > 100.0))	/* More than 1.5 edge LSB's off? */
						break;				/* Then skip all 'coarse' values */
					if (abort)
						break;
				}
				if ((Optimize==1)&&	/* Gross pixel error optimization? */
					(berr > 100.0))	/* More than 90 off? */
					break;				/* Then skip all 'edge' values */
				if (abort)
					break;
			}
			if (abort)
				break;
		}

		if (besterr < 50) {
			/* Save to EEPROM */
			res = FlyerWriteCalib(0,CALIB_HRECOFFSETA,bestoffset,1);
			res = FlyerWriteCalib(0,CALIB_ADC_PHASE_EDGE,bestedge,1);
			res = FlyerWriteCalib(0,CALIB_ADC_PHASE_COURSE,bestcoarse,1);
			res = FlyerWriteCalib(0,CALIB_ADC_PHASE_FINE,bestfine,1);
			DBUG(printf("Saved!\n");)
		}

		DBUG(printf("Best is %ld/%ld/%ld/%ld (err = %.1f)\n",
		bestoffset,bestedge,bestcoarse,bestfine,besterr);)

		result = besterr;
		break;
#if 0
	case TEST_ManualRec:
		res = RecordMode(unit);
		AdjType = 2;	/* Record */

		printf("Manually calibrating record phase...(Hit RETURN to abort)\n");

		NoiseThreshLo = BURSTTHRESHLO;
		NoiseThreshHi = BURSTTHRESHHI;

		for (;;) {
			if (WaitForChar(handle,30)) {
				getchar();
				break;
			}

			berr = CalcBurstPhase(1,&badvideo)-TARGVECTOR;
			if (badvideo) {		/* Skip this one if bad */
				DBUGRESULT(printf("(bad)\n");)
				berr = 0.0;
				continue;
			}
			else {
				DBUGRESULT(printf("(%.1f)\n",berr);)
			}
		}
		break;
	case TEST_D2:
		res = OpenLiveField(0);

		for (j=1;j<=2;j++) {
			printf("Line %ld:\n",j);
			res = GetNextLine(Buffer);
			for (i=0;i<SAMPLESPERLINE;i+=4)
				YIQ(&Buffer[i]);
		}

		res = CloseLiveField();

		result = 0.0;
		break;
	case TEST_BadVid:
		res = RecordMode(unit);
		AdjType = 2;	/* Record */

		printf("Counting usable skew values...(Hit RETURN to abort early)\n");

		NoiseThreshLo = BURSTTHRESHLO;
		NoiseThreshHi = BURSTTHRESHHI;

		abort = 0;
		totalval = badval = 0;

		for (offset=LOWOFFSET;offset<=(LOWOFFSET+3);offset++) {
			res = FlyerWriteCalib(0,CALIB_HRECOFFSETA,offset,0);
			for (edge=0;(edge<4);edge++) {
				res = FlyerWriteCalib(0,CALIB_ADC_PHASE_EDGE,edge,0);
				for (coarse=0;(coarse<10);coarse++) {
					res = FlyerWriteCalib(0,CALIB_ADC_PHASE_COURSE,coarse,0);
					for (fine=0;(fine<8);fine++) {
						res = FlyerWriteCalib(0,CALIB_ADC_PHASE_FINE,fine,0);

						if (WaitForChar(handle,30)) {
							getchar();
							abort = 1;
							break;
						}

						berr = CalcBurstPhase(1,&badvideo)-TARGVECTOR;
						if (badvideo) {
							printf("Edge=%ld, Crs=%ld, Fine=%ld---",
							edge,coarse,fine);
							res = SBusRead(unit,0xF1,&data);
							printf("Skew4=%ld ",data>>4);
							res = SBusRead(unit,0xFE,&data);
							printf("Aln4=%ld\n",data>>4);
							badval++;
						}

						totalval++;
					}
					if (abort)
						break;
				}
				if (abort)
					break;
			}
			if (abort)
				break;
		}

		printf("%ld bad out of %ld (%ld\%)\n",badval,totalval,badval*100/totalval);

		result = besterr;
		break;
#endif
	}

#if 0
	res = FIRcoef(unit,0,FIR050);
#endif

#if 0
	res = FlyerInputSel(unit,FI_Toaster1,FS_ToasterMain);
#endif

	/* Save result in environment var */
	fp = fopen("ENV:CalibResult","w");
	if (fp) {
		sprintf(ValueBuffer,"%.1f",result);
		fwrite(ValueBuffer,strlen(ValueBuffer),1,fp);
		fclose(fp);
	}

	CloseLibrary(FlyerBase);
	CloseLibrary(MathIeeeSingTransBase);

	return(0);
}


int EnsureStable(FLOAT err)
{
	int	i,badvideo;
	FLOAT	berr,diff;

	if (err > 3.0)		/* If not really close yet, don't worry about stability */
		return(1);

	for (i=1;i<5;i++) {
		berr = CalcBurstPhase(0,&badvideo)-TARGVECTOR;
		if (badvideo) {
			DBUGRESULT(printf("Very Unstable!\n");)
			return(0);
		}
		if (berr<0)
			berr = -berr;


		diff = berr-err;
		if (diff<0)
			diff = -diff;

		if (diff > JITTER) {
			DBUGRESULT(printf("Unstable! (%.1f,%.1f)\n",berr,err);)
			return(0);
		}
	}

	DBUGRESULT(printf("(Stable)\n");)
	return(1);
}


int CheckRange(int edgelo, int coarselo, int finelo,
					int edgehi, int coarsehi, int finehi,
					FLOAT close)
{
	FLOAT	loerr,hierr;
	ULONG	res;
	int	badvideo;

	if (Optimize<2)		/* Range optimization enabled?  If not, do range */
		return(1);

	if (edgelo>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_EDGE,edgelo,0);
	if (coarselo>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_COURSE,coarselo,0);
	if (finelo>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_FINE,finelo,0);

	loerr = CalcBurstPhase(1,&badvideo)-TARGVECTOR;
	DBUGOPT(printf("Lo side:");)
	if (badvideo) {
		DBUGOPT(printf("bad video\n");)
		return(1);		/* Do range */
	}
	DBUGOPT(printf("%.1f\n",loerr);)

	if (edgehi>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_EDGE,edgehi,0);
	if (coarsehi>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_COURSE,coarsehi,0);
	if (finehi>=0)
		res = FlyerWriteCalib(0,CALIB_ADC_PHASE_FINE,finehi,0);

	hierr = CalcBurstPhase(1,&badvideo)-TARGVECTOR;
	DBUGOPT(printf("Hi side:");)
	if (badvideo) {
		DBUGOPT(printf("bad video\n");)
		return(1);		/* Do range */
	}
	DBUGOPT(printf("%.1f\n",hierr);)

	if (loerr<0.0) {
		if (hierr>=0.0)
			return(1);			/* Do range if sign changes */
	}
	else {
		if (hierr<0.0)
			return(1);			/* Do range if sign changes */
	}

	if (loerr < 0.0)
		loerr = -loerr;
	if (hierr < 0.0)
		hierr = -hierr;

	if ((loerr < close) || (hierr < close))
		return(1);			/* Do range if either is fairly close */

	return(0);			/* Made it thru all tests!  Skip entire range! */
}


/* Average quads (xxx in from edge) for x lines from each of flds 0 and 1 */
int AverageQuads(LONG *minI,LONG *minQ,LONG *plsI,LONG *plsQ)
{
	register	UBYTE	*ptr;
	int	i,j,k,avglum,lastlum,badflag,deltalum;
	ULONG	res;

	*minI = *minQ = *plsI = *plsQ = 0;

	badflag = 0;

	Lowest = Highest = 0;

	for (i=0;i<SAMPLEFIELDS;i++) {
		Delay(SLOWDOWN);

		res = OpenLiveField(i);

		Delay(SLOWDOWN);

		if (i & 1) {
			res = GetNextLine(Buffer);		/* Skip whole line to be in phase */
		}
		else {		/* Do it here too! */
			res = GetNextLine(Buffer);		/* Skip half lines */
			res = GetNextLine(Buffer);		/* Skip whole line to be in phase?!?!? */
		}
		for (j=1;j<=SAMPLELINES;j++) {
			DBUG(printf("Field %ld, line %ld:\n",i,j);)
			res = GetNextLine(Buffer);
			ptr = &Buffer[RECPIXELSIN];	/* Skip into burst area of pattern */
			lastlum = 999;
			for (k=1;k<=RECQUADS;k++) {
				if ((j & 1) == 0) {			/* ++-- */
					DBUGVERB(printf("+");)
				}
				else {							/* --++ */
					DBUGVERB(printf("-");)
				}
				DBUGVERB(YIQ(ptr);)

				/* On first line, check for crappy video */
				if (j==1) {
					avglum = (ptr[0]+ptr[1]+ptr[2]+ptr[3]);
					DBUGVERB(printf("(%ld)",avglum);)
					if (lastlum != 999) {
						deltalum = (avglum-lastlum)>>2;		/* Into D2 units */

						if ((deltalum>NoiseThreshHi) || (deltalum<NoiseThreshLo)) {
							DBUGACAL(printf("broke on this: %ld\n",deltalum);)
							badflag = 1;
							break;
						}
					}
					lastlum = avglum;
				}

				if ((j & 1) == 0) {			/* ++-- */
					*plsI += *ptr++;
					*plsQ += *ptr++;
					*minI += *ptr++;
					*minQ += *ptr++;
				}
				else {							/* --++ */
					*minI += *ptr++;
					*minQ += *ptr++;
					*plsI += *ptr++;
					*plsQ += *ptr++;
				}
			}
			if (badflag)
				break;
		}
		res = CloseLiveField();
	}

	return(badflag);
}


ULONG OpenLiveField(UBYTE field)
{
	ULONG	res;

	FlyVol.Board = 0;
	FlyVol.SCSIdrive = 0;
	FlyVol.Path = 0;

	Action.Volume = &FlyVol;

	Action.ReturnTime = RT_STOPPED;
	res = OpenReadField(&Action,field,0);
	if (res)
		printf("\nOLF error %ld - %ls\n",res,Error2String(res));

	return(res);
}


ULONG CloseLiveField(void)
{
	ULONG	res;

	Action.ReturnTime = RT_STOPPED;
	res = CloseField(&Action);
	if (res)
		printf("\nCLF error %ld - %ls\n",res,Error2String(res));

	return(res);
}


ULONG GetNextLine(UBYTE *buffer)
{
	ULONG	res;

	Action.ReturnTime = RT_STOPPED;
	res = FlyerReadLine(&Action,Buffer);
	if (res)
		printf("\nGLL error %ld - %ls\n",res,Error2String(res));

	return(res);
}


int ReturnHit(void)
{
	if (WaitForChar(handle,30)) {
		getchar();
		return(1);
	}
	return(0);
}


int AvgQuadLuma(UBYTE *ptr,int *badvideo)
{
	int	avg,i1,i2,q1,q2;

	i1 = *ptr++;
	q1 = *ptr++;
	i2 = *ptr++;
	q2 = *ptr++;

	avg = (i1 + q1 + i2 + q2)>>2;

	DBUGVERB(printf("%ld (%ld,%ld,%ld,%ld)\n",avg,i1-avg,q1-avg,i2-avg,q2-avg);)

	if (abs(i1 - avg) > MAXCHROMA)
		*badvideo = 1;
	else if (abs(q1 - avg) > MAXCHROMA)
		*badvideo = 1;
	else if (abs(i2 - avg) > MAXCHROMA)
		*badvideo = 1;
	else if (abs(q2 - avg) > MAXCHROMA)
		*badvideo = 1;
	else
		*badvideo = 0;

	return(avg);
}

void YIQ(UBYTE *ptr)
{
	int	p1,p2,p3,p4,Y,I,Q;

	p1 = *ptr++;
	p2 = *ptr++;
	p3 = *ptr++;
	p4 = *ptr++;

	Y = (p1 + p2 + p3 + p4)/4;
	I = ((Y - p1) + (p3 - Y))/2;
	Q = ((Y - p2) + (p4 - Y))/2;

	printf("%ld %ld %ld %ld     Y=%ld, I=%ld, Q=%ld\n",p1,p2,p3,p4,Y,I,Q);
}

void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}

void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}


void Doit(int val,int saveit)
{
	ULONG	res;

	res = FlyerWriteCalib(0,AdjType+CALIB_HPLAYOFFSETA,val,saveit);
	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
}


void Doit3(int which,int val,int saveit)
{
	ULONG	res;

	res = FlyerWriteCalib(0,AdjType*3+which,val,saveit);
	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
}


void PlayClip(int channel,char *name,int length)
{
#if 0
	ULONG res;
	struct FlyerVolume FlyVol;
	struct ClipAction	 Action;

	FlyVol.Path = name;
	FlyVol.Board = 0;
	FlyVol.SCSIdrive = 0;

	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_STOPPED;

	Action.Channel = channel;
	Action.Flags = CAF_VIDEO;
	Action.VidStartField = 0;
	Action.VidFieldCount = length;
	Action.AudStartField = 0;
	Action.AudFieldCount = length;
	Action.GoClock = 0;

	res = FlyerPlay(&Action);
	if (res)
		printf("Bars Error %ld - %ls\n",res,Error2String(res));
#endif
}



FLOAT CalcBurstPhase(int skipframes,int *badvidflag)
{
	LONG	mI,mQ,pI,pQ,I,Q;
	FLOAT	tan,base;
	ULONG	res;
	int	i;

	Delay(SKEWSETTLETIME);

	for (i=0;i<skipframes;i++) {
		res = OpenLiveField(0);			/* Throw away 'n' color frames */
		res = CloseLiveField();
	}

	*badvidflag = AverageQuads(&mI,&mQ,&pI,&pQ);
	DBUGACAL(printf("Lo/Hi errors: %ld %ld\n",Lowest,Highest);)
	if (*badvidflag==0) {

		DBUGACAL(printf("-I,-Q,+I,+Q = %ld,%ld,%ld,%ld\n",mI,mQ,pI,pQ);)

		I = (pI - mI)/2;
		Q = (pQ - mQ)/2;

		DBUGACAL(printf("I,Q = %ld,%ld ",
		I/(SAMPLEFIELDS*SAMPLELINES*RECQUADS),
		Q/(SAMPLEFIELDS*SAMPLELINES*RECQUADS));)

		if (Q<0.0) {
			base = 180.0;
		}
		else {
			base = 0.0;
		}
		DBUGMATH(printf("base = %.1f\n",base);)

		tan = ((FLOAT)I)/((FLOAT)Q);
		DBUGMATH(printf("ratio = %.1f\n",tan);)

		tan = IEEESPAtan(tan)*180.0/(PI;		/* Imbalanced ')' in include */
		DBUGMATH(printf("tan = %.1f\n",tan);)

		tan += base+33;
		DBUGMATH(printf("based = %.1f\n",tan);)

		if (tan<0.0)
			tan += 360.0;
		if (tan>=360.0)
			tan -= 360.0;
		DBUGMATH(printf("final = %.1f\n",tan);)
	}
	else
		tan = 999.9;

	return(tan);
}
