/*********************************************************************\
*
* ClipStats - Scan thru clip and dump stats
*
* $Id: clipstats.c,v 1.2 1996/08/13 15:28:53 Holt Exp $
*
* $Log: clipstats.c,v $
*Revision 1.2  1996/08/13  15:28:53  Holt
*added hq6 to posable video grades.
*
*Revision 1.1  1995/10/16  17:21:06  Flick
*Added recognition of HQ5 clips
*
*Revision 1.0  1995/07/11  10:54:34  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	07/11/95		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>
#include <clib/dos_protos.h>
#include <dos.h>

#include "flyerlib.h"
#include "flyer.h"
#include "flyerFSfmt.h"

ULONG SuckInBlock(UBYTE drive, ULONG lba, UBYTE *dest);
void GetArg(char *arg,int *var,int lo,int hi,char *label);


struct Library *FlyerBase;
struct FlyerVolume	FlyVol;
struct ClipAction		Action;
struct ClipInfo		Buffer;
struct ClipHeader		ClipHeader;
struct FrameHeader	FrameHeader;

int Breaker(void)
{
   printf("Aborted\n");

	if (FlyerBase)
		CloseLibrary(FlyerBase);
   exit(10);

	return(0);
}


main(int argc,char **argv)
{
	int	unit=0,drive,fld,tol;
	ULONG	res,lba,meg10,sec10;
	char	*name;
	struct Field *fldptr;

   onbreak(Breaker);

	if ((argc!=3)||(argv[1][0]=='?')) {
		printf("Usage: %ls <drive> <name>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&drive,0,23,"SCSI Drive");
	name = argv[2];

	FlyVol.Board = unit;
	FlyVol.SCSIdrive = drive;
	FlyVol.Path = name;
//	FlyVol.Flags = FVF_USENUMS;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	Buffer.len = sizeof(struct ClipInfo);
	res = GetClipInfo(&FlyVol,&Buffer);
	if (res)
	{
		printf("ClipInfo Error %ld - %ls\n",res,Error2String(res));
		goto abort;
	}

	if (Buffer.Type != FLYER_TYPE_FILE)
	{
		printf("%ls is a sub-directory\n",name);
		goto abort;
	}

	printf("Information for clip '%ls'\n",Buffer.Name);
	sec10 = Buffer.Fields / 6;
	printf("  Fields = %ld (%ld.%ld seconds)\n",Buffer.Fields,sec10/10,sec10%10);
	meg10 = (Buffer.Length / 100000) + (Buffer.LengthExt * 42950);
	printf("  Total size = %ld.%ld MB\n",meg10/10,meg10%10);
	printf("  Contents = ");
	if (Buffer.Flags & CIF_HASVIDEO)
	{
		printf("video ");
		if (Buffer.VideoGrade == VG_HQ5)
			printf("(HQ5) ");
		else
			if (Buffer.VideoGrade >= VG_HQ5)
				printf("(HQ6) ");
			else 
				printf("(Standard) "); 
	}
	if (Buffer.Flags & CIF_HASAUDIO)
		printf("audio -- %ld channel(s)",Buffer.NumAudChans);
	printf("\n\n");

	lba = Buffer.Start;

	res = SuckInBlock(drive,lba++,(UBYTE *)&ClipHeader);
//	printf("ID = %lx\n",ClipHeader.ID);
	if ((res != FERR_OKAY) || (ClipHeader.ID != ID_CLIP))
	{
		printf("Not a Flyer clip\n");
		goto abort;
	}

	do {
		res = SuckInBlock(drive,lba,(UBYTE *)&FrameHeader);
		if ((res != FERR_OKAY) || (FrameHeader.ID != ID_CFRM))
		{
			printf("Frame header error at %lx\n",lba);
			break;
		}

		fldptr = &FrameHeader.fields[0];
		for (fld=1 ; fld<=4 ; fld++,fldptr++)
		{
			tol = fldptr->Tolerance;
			if (fldptr->Flags & 1)
				tol += 4;

			meg10 = fldptr->VidLength * 192 / 625;		// Convert blocks into meg/sec * 10

			printf("Fld %ld: %ld (%ld.%ld M/s)\n",
				FrameHeader.FrameNumber*4+fld,tol,meg10/10,meg10%10);
		}

		lba += FrameHeader.NextFrame;

	} while (FrameHeader.NextFrame);			// 0 link stops us

	printf("--- end of clip ---\n");

abort:
	CloseLibrary(FlyerBase);

	return(0);
}


ULONG SuckInBlock(UBYTE drive, ULONG lba, UBYTE *dest)
{
	ULONG	res,addr;
	UWORD	*ptr;
	int	i,unit=0;

//	printf("LBA=%lx\n",lba);

	FlyVol.Board = unit;
	FlyVol.SCSIdrive = drive;

	Action.Flags = 0;
	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_STOPPED;

	res = Read10(&Action,1,lba,0x1FFF);					// Move from SCSI to DMA mem
	if (res) {
		printf("Read10 Error %ld - %ls\n",res,Error2String(res));
		return(res);
	}

	res = CPUDMA(unit,0x40,0x1FFF,1,0);		// Move from DMA to CPU mem
	if (res) {
		printf("CPUDMA Error %ld - %ls\n",res,Error2String(res));
		return(res);
	}

	ptr = (UWORD *)dest;
	addr = 0xC08000;
	for (i=0 ; i<256 ; i++,ptr++,addr+=2)
	{
//		printf("%lx->%lx\n",addr,ptr);
		res = CPUread(unit,addr,ptr);
		if (res) {
			printf("CPUread Error %ld - %ls\n",res,Error2String(res));
			return(res);
		}
	}

	return(FERR_OKAY);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
