/*********************************************************************\
*
* DirList - List contents of directory
*
* $Id: dirlist.c,v 1.4 1996/12/16 13:52:36 Hayes Exp $
*
* $Log: dirlist.c,v $
*Revision 1.4  1996/12/16  13:52:36  Hayes
*no changes
*
*Revision 1.3  1996/08/13  15:29:39  Holt
*added cr to end of flyerdrive line
*
*Revision 1.2  1996/06/17  18:31:59  Holt
*Fixed up the sloppie spacing of the output of this program/
*
*Revision 1.1  1995/06/15  17:56:49  Flick
*Handles clips with bytes lengths > 32 bits
*
*Revision 1.0  1995/05/05  15:48:52  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/24/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);
void ListGripDir(int depth,ULONG grip);

struct Library *FlyerBase;
struct FlyerVolume FlyVol;
struct ClipAction Action;

struct ClipInfo	Cinfo;
ULONG	LastEnd;

main(int argc,char **argv)
{
	int	unit,drive;
	ULONG	res,grip;

	if ((argc<3)||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <drive> [<path>]\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	FlyVol.Board = unit;
	GetArg(argv[2],&drive,0,23,"Drive");

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	LastEnd = 0;
	
	FlyVol.SCSIdrive = drive;

	if (argc==4) {
		FlyVol.Path = argv[3];
		Action.Volume = &FlyVol;
		Action.ReturnTime = RT_STOPPED;
		Action.Grip = 0;
		res = FlyerLocate(&Action);
		if (res==0)
			grip = Action.Grip;
	}
	else
		grip = 0;

	Cinfo.len = sizeof(struct ClipInfo);
	res = FlyerDirList(&FlyVol,grip,&Cinfo,1,0);
	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
	else
	{
		printf("FlyerDrive # %lx \n",drive);
		ListGripDir(0,grip);
	}		

	if (grip)
		FlyerFreeGrip(&FlyVol,grip);

	printf("Last block used is %lx\n",LastEnd);

	CloseLibrary(FlyerBase);

	return(0);
}


void ListGripDir(int depth,ULONG grip)
{
	int	i;
	ULONG	res,meg10,subgrip,endblk;

	for (;;) {
		Cinfo.len = sizeof(struct ClipInfo);

		res = FlyerDirList(&FlyVol,grip,&Cinfo,0,0);
		if (res) {
			if (res!=FERR_EXHAUSTED)
				printf("Error %ld - %ls\n",res,Error2String(res));
			break;
		}

		//for (i=0;i<depth;i++)
		//	printf("   ");

		if (Cinfo.Type == FLYER_TYPE_FILE) {
			meg10 = (Cinfo.Length / 100000) + (Cinfo.LengthExt * 42950);

			switch(depth){
				case (0):
					printf("%-20ls ---- %7ld.%ld MB",Cinfo.Name,meg10/10,meg10%10);
			 		break;
				case (1):
					printf("  %-18ls ---- %7ld.%ld MB",Cinfo.Name,meg10/10,meg10%10);
			 		break;
				case (2):
					printf("    %-16ls ---- %7ld.%ld MB",Cinfo.Name,meg10/10,meg10%10);
			 		break;
				case (3):
					printf("      %-14ls ---- %7ld.%ld MB",Cinfo.Name,meg10/10,meg10%10);
			 		break;
				case (4):
					printf("        %-12ls ---- %7ld.%ld MB",Cinfo.Name,meg10/10,meg10%10);
			 		break;
				default:
					printf("%-20ls ---- %7ld.%ld MB",Cinfo.Name,meg10/10,meg10%10);
			}

/*			printf("  Bits:%lx",Cinfo.Bits); */
			printf("  Start:%10lx",Cinfo.Start);
			endblk = Cinfo.Start + (Cinfo.LengthExt * 8388608) + ((Cinfo.Length+511)>>9)-1;
			printf("  End:%10lx",endblk);
/*			printf("  Fields:%ld  (%ld frames)",Cinfo.Fields,Cinfo.Fields/4); */
/*			printf("  Index:%lx",Cinfo.IndexBlk); */
			printf("\n");

			if (endblk > LastEnd)
				LastEnd = endblk;
		}
		else {
			printf("\n");
			printf("%ls  (dir)\n",Cinfo.Name);

			FlyVol.Path = Cinfo.Name;
			Action.Volume = &FlyVol;
			Action.ReturnTime = RT_STOPPED;
			Action.Grip = grip;
			res = FlyerLocate(&Action);
			if (res==0) {
				subgrip = Action.Grip;

				Cinfo.len = sizeof(struct ClipInfo);
				res = FlyerDirList(&FlyVol,subgrip,&Cinfo,1,0);
				if (res)
					printf("SubDir Error %ld - %ls\n",res,Error2String(res));
				else
					ListGripDir(depth+1,subgrip);

				FlyerFreeGrip(&FlyVol,subgrip);
			}
		}
	}
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
