/*********************************************************************\
*
* DriveSpeed2.0 - Test Flyer drive speed
*
* Copyright (c) 1994,1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* 01-19-95      Marty    Created original RWtest
* 01-20-95      Todd     Created DriveSpeed based on original RWtest
* 01-30-95      Todd     Added speed drop reporting
* 01-31-95      Todd     Removed all "grading" code
* 02-01-95      Todd     Removed all "target" info
* 02-15-95      Todd     Changed timing info to ms
* 03-08-95      Todd     added NOSTAT, LIST, SPEED
* 04-27-95      Todd     Changed NOSTAT to FILE
* 04-27-95      Todd     Added check for dips only
\*********************************************************************/

/* INCLUDES */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <clib/dos_protos.h>
#include <clib/exec_protos.h>
#include <clib/gadtools_protos.h>
#include <clib/graphics_protos.h>
#include <clib/intuition_protos.h>
#include <clib/rexxsyslib_protos.h>
#include <clib/asl_protos.h>
#include <dos/dos.h>
#include <exec/types.h>
#include <exec/memory.h>
#include <intuition/gadgetclass.h>
#include <intuition/intuition.h>
#include <libraries/gadtools.h>
#include <libraries/asl.h>
#include <proto/exec.h>
#include <rexx/storage.h>
#include "flyerlib.h"
#include "flyer.h"

/* DEFINES */

#define	TESTBLKS 512		/* Each transfer will be 512 blocks (256K) */
#define	REPEAT	64			/* Do 64 transfers for each test */
#define	TOTALSIZE	TESTBLKS*REPEAT
#define	NUMTESTS		4

#define	DBUG(x)			if (Debugging) { x }

/***** Gadget IDs for all Gadgets *****/
#define DSGAD_DRIVELIST	(0)
#define DSGAD_TESTTYPE 	(1)
#define DSGAD_LENGTH		(2)
#define DSGAD_TARGETSPD	(3)
#define DSGAD_LENGTHMIN	(4)
#define DSGAD_TARGETNUM	(5)
#define DSGAD_RESLABLOW	(6)
#define DSGAD_RESLABAVG	(7)
#define DSGAD_RESLABIDL	(8)
#define DSGAD_RESLISTLR	(9)
#define DSGAD_RESLISTAR	(10)
#define DSGAD_RESLISTIR	(11)
#define DSGAD_RESLISTLW	(12)
#define DSGAD_RESLISTAW	(13)
#define DSGAD_RESLISTIW	(14)
#define DSGAD_VIEWLOG	(15)
#define DSGAD_SAVELOG	(16)
#define DSGAD_VIEWCACHE	(17)
#define DSGAD_PERCENT	(18)
#define DSGAD_STATUS		(19)
#define DSGAD_START		(20)
#define DSGAD_RESCAN		(21)
#define DSGAD_EXIT		(22)
#define DSGAD_RESBOX		(23)
#define DSGAD_LOGLIST	(0)
#define DSGAD_LOGCONT	(1)
#define DSGAD_LOGSAVE	(2)

USHORT __chip WaitPointer[36] =
{
	0x0000, 0x0000, 0x0400, 0x07C0, 0x0000, 0x07C0, 0x0100, 0x0380, 0x0000,
	0x07E0, 0x07C0, 0x1FF8, 0x1FF0, 0x3FEC, 0x3FF8, 0x7FDE, 0x3FF8, 0x7FBE,
	0x7FFC, 0xFF7F, 0x7EFC, 0xFFFF, 0x7FFC, 0xFFFF, 0x3FF8, 0x7FFE, 0x3FF8,
	0x7FFE, 0x1FF0, 0x3FFC, 0x07C0, 0x1FF8, 0x0000, 0x07E0, 0x0000, 0x0000
};

/*
 * These two defines set up and clear the WaitPointer...
 */
#define	SetWait(x)	SetPointer(x,WaitPointer,16L,16L,-6L,0L)
#define	ClearWait(x)	ClearPointer(x)

/* STRUCTURES */

struct LBAStats {
	struct LBAStats *prev,*next;
	ULONG lba;
	ULONG ticks;
	double speed;
	long drop;
	long drop_ms;
	int dip;
};

struct TestStats {
	struct LBAStats *first,*last;
	int num_tests;
	struct LBAStats *lowest;
	double average;
	struct LBAStats *max_drop;
};

struct DLNode
{
	struct Node node;
	UBYTE 		str[128];
	int			index;
	int   		drive;
};

/************** FUNCTION PROTOS ***************/
BOOL HandleGadgetEvent(struct Window *win, struct Gadget *gad, UWORD code,
								struct Gadget *gadlist[]);
struct Gadget *CreateAllGadgets(struct Gadget **glistptr, void *vi,
					UWORD topborder, struct Gadget *gadlist[]);
void ProcessWindowEvents(struct Window *win, struct Gadget *gadlist[],
									BOOL (*HandleEvent)(struct Window *, 
																struct Gadget *, 
																UWORD,
																struct Gadget **));
void DSWindow( void );
void GetDriveList(struct Window *win, struct Gadget **gadlist);
void AddDListNode(struct List *list, char *str, int index, 
						struct Window *win, struct Gadget **gadlist);
void FreeList( struct List *list );
void GetDriveInfo(UBYTE ID, char *strbuf, struct Window *win, struct Gadget **gadlist);
int FindToaster( void );
float Str2Float(char *str);
void errorMessage(char *msg);
void StatusMessage(char *msg);
void cleanup( void );
void StartTest(struct Window *win, struct Gadget *gadlist[]);
UBYTE WriteWarning(struct Window *win, struct Gadget **gadlist);
void AskReformat(struct Window *win, struct Gadget **gadlist, 
						struct FlyerVolInfo *Buffer);
void UpdateSummaryWindow(struct Window *win, struct Gadget **gadlist);
ULONG DoTest(ULONG lba,ULONG testblks,ULONG repeat,int writeflag);
ULONG ElapsedTicks(struct DateStamp *begin, struct DateStamp *end);
struct LBAStats *NewStat(struct LBAStats *old, struct TestStats *Test, ULONG lba,
								 ULONG ticks);
void DoStats(struct TestStats *test);
void UpdateShortStroke(struct Window *win, struct Gadget **gadlist);
ULONG	FindLastBlockAtSpeed(double spd, int read, int write);
void CreateLog(struct Window *win, struct Gadget **gadlist);
void ViewLog( void );
BOOL HandleLogEvent(struct Window *win, struct Gadget *gad, UWORD code,
								struct Gadget *gadlist[]);
struct Gadget *CreateLogGadgets(struct Gadget **glistptr, void *vi,
					UWORD topborder, struct Gadget *gadlist[]);
void SaveLog(struct Window *win, struct Gadget **gadlist);
void ReqFile(char *dir, char *file);
void WriteListFile(struct List *list, char *filename);
void ViewCache(struct Window *win, struct Gadget **gadlist); 
void PurgeMsgPort(struct Window *win);

/******************* GLOBALS ******************/
struct TextAttr Topaz80 = { "topaz.font", 8, 0, 0, };

struct Library *IntuitionBase = NULL;
struct Library *GfxBase = NULL;
struct Library *GadToolsBase = NULL;
struct Library *AslBase = NULL;

char *TypeLabels[] = {"Read","Write","Read/Write", NULL};

#define EN_LENGTH 5
char *LengthLabels[] = {"Full","1 Minute","2 Minutes", 
								"5 Minutes", "10 Minutes", "Custom", NULL};
ULONG	LengthVals[6] = { 0, 1, 2, 5, 10, 0 };
UBYTE LengthIndex = 0;

#define EN_SPEED 3
char *SpeedLabels[] = {"None", "Standard", "HQ5", "Custom", NULL};
float SpeedVals[4] = { 0.0, 3.7, 4.8, 0.0 };
char SpeedStr[16];
char PercentStr[16] = "0.0";
UBYTE TargetIndex = 0;

struct List *DriveList = NULL, *LogList = NULL;
char ResList[6][32];
struct Window *StatusWin;
struct Gadget **StatusGadList;
char StatusStr[64] = "";

ULONG	BufLen,size,ss_block, max, blklen;
int	do_read, do_write, drive, good_tcal = 1;
char *version;
char status[]="-\\|/";
char stat_n = 0;
FILE *file = NULL;
struct Library			*FlyerBase;
struct FlyerVolume	FlyVol;
struct ClipAction		Action;
struct DateStamp 		TestStart,Now;
struct TestStats 		tcal_read_stats =  { NULL, NULL, 0, NULL, 0.0, NULL };
struct TestStats		tcal_write_stats = { NULL, NULL, 0, NULL, 0.0, NULL };

UBYTE	PageBuffer[256];
UBYTE InqBuff[255];
char LogFile[64] = "DriveSpeed.log";
char LogDir[128] = "ram:";

/************** FUNCTION BODIES ***************/

BOOL HandleGadgetEvent(struct Window *win, struct Gadget *gad, UWORD code,
								struct Gadget *gadlist[])
{
	char buff[256];
	struct DLNode *node;
	BOOL terminated = FALSE;

	switch(gad->GadgetID)
	{
		case DSGAD_DRIVELIST:
			drive = -1;  //Indicates no drive selected
			for(node = (struct DLNode *)DriveList->lh_Head;node->node.ln_Succ;
												node = (struct DLNode *)node->node.ln_Succ )
				if (node->index == code)
					drive = node->drive;
			break;
		case DSGAD_TESTTYPE:
			switch(code)
			{
				case (0):
					do_read  = 1;
					do_write = 0;
					break;
				case (1):
					do_read  = 0;
					do_write = 1;
					break;
				case (2):
					do_read  = 1;
					do_write = 1;
					break;
			}
			break;
		case DSGAD_LENGTH:

			GT_SetGadgetAttrs(gadlist[DSGAD_LENGTHMIN], win, NULL,
										GTIN_Number,	LengthVals[code],
										TAG_END);
			if ((LengthIndex = code) == EN_LENGTH)
				GT_SetGadgetAttrs(gadlist[DSGAD_LENGTHMIN], win, NULL,
											GA_Disabled,	FALSE,
											TAG_END);
			else
				GT_SetGadgetAttrs(gadlist[DSGAD_LENGTHMIN], win, NULL,
											GA_Disabled,	TRUE,
											TAG_END);
			break;
		case DSGAD_TARGETSPD:
			sprintf(SpeedStr, "%.2f", SpeedVals[code]);
			GT_SetGadgetAttrs(gadlist[DSGAD_TARGETNUM], win, NULL,
										GTST_String,	SpeedStr,
										TAG_END);
			if ((TargetIndex = code) == EN_SPEED)
				GT_SetGadgetAttrs(gadlist[DSGAD_TARGETNUM], win, NULL,
											GA_Disabled,	FALSE,
											TAG_END);
			else
				GT_SetGadgetAttrs(gadlist[DSGAD_TARGETNUM], win, NULL,
											GA_Disabled,	TRUE,
											TAG_END);
			break;
		case DSGAD_LENGTHMIN:
			LengthVals[EN_LENGTH] = (ULONG)
				((struct StringInfo *)(gadlist[DSGAD_LENGTHMIN]->SpecialInfo))->LongInt;
			break;
		case DSGAD_TARGETNUM:
			SpeedVals[EN_SPEED] = Str2Float(((struct StringInfo *)
										(gadlist[DSGAD_TARGETNUM]->SpecialInfo))->Buffer);
			sprintf(SpeedStr, "%.2f", SpeedVals[EN_SPEED]);
			GT_SetGadgetAttrs(gadlist[DSGAD_TARGETNUM], win, NULL,
											GTST_String,	SpeedStr,
											TAG_END);
			break;
		case DSGAD_VIEWCACHE:
			ViewCache(win, gadlist);
			break;
		case DSGAD_START:
			if (drive != -1)
			{
				SetWait(win);
				StartTest(win, gadlist);
				UpdateSummaryWindow(win, gadlist);			
				UpdateShortStroke(win, gadlist);
				CreateLog(win, gadlist);
				ClearWait(win);
			}
			break;
		case DSGAD_VIEWLOG:
			ViewLog();
			PurgeMsgPort(win);
			break;
		case DSGAD_SAVELOG:
			SaveLog(win, gadlist);
			break;
		case DSGAD_RESCAN:
			SetWait(win);
			GetDriveList(win, gadlist);
			ClearWait(win);
			break;
		case DSGAD_EXIT:
			terminated = TRUE;
			break;
	}
	StatusMessage(buff);
	return(terminated);
}

struct Gadget *CreateAllGadgets(struct Gadget **glistptr, void *vi,
					UWORD topborder, struct Gadget *gadlist[])
{
	struct NewGadget ng;
	struct Gadget *gad;

	gad = CreateContext(glistptr);

	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 14 + topborder;
	ng.ng_Width = 345;
	ng.ng_Height = 63;
	ng.ng_GadgetText = "Drive To Test";
	ng.ng_TextAttr = &Topaz80;
	ng.ng_VisualInfo = vi;
	ng.ng_GadgetID = DSGAD_DRIVELIST;
	ng.ng_Flags = PLACETEXT_ABOVE;
	drive = 0;
	gadlist[DSGAD_DRIVELIST] = gad = CreateGadget(LISTVIEW_KIND, gad, &ng,
									GTLV_ShowSelected,	NULL,
									TAG_END);	

	ng.ng_LeftEdge = 125;
	ng.ng_TopEdge = 81 + topborder;
	ng.ng_Width = 118;
	ng.ng_Height = 14;
	ng.ng_GadgetText = "Test Type:";
	ng.ng_GadgetID = DSGAD_TESTTYPE;
	ng.ng_Flags = PLACETEXT_LEFT;
	do_read = 1;
	do_write = 1;
	gadlist[DSGAD_TESTTYPE] = gad = CreateGadget(CYCLE_KIND, gad, &ng, 
									GTCY_Labels,	TypeLabels,
									GTCY_Active,	2,
									TAG_END);

	ng.ng_TopEdge = 95 + topborder;
	ng.ng_GadgetText = "Test Length:";
	ng.ng_GadgetID = DSGAD_LENGTH;
	
	gadlist[DSGAD_LENGTH] = gad = CreateGadget(CYCLE_KIND, gad, &ng, 
									GTCY_Labels,	LengthLabels,
									TAG_END);

	ng.ng_TopEdge = 109 + topborder;
	ng.ng_GadgetText = "Target Speed:";
	ng.ng_GadgetID = DSGAD_TARGETSPD;
	gadlist[DSGAD_TARGETSPD] = gad = CreateGadget(CYCLE_KIND, gad, &ng, 
									GTCY_Labels,	SpeedLabels,
									TAG_END);

	ng.ng_LeftEdge = 244;
	ng.ng_TopEdge = 95 + topborder;
	ng.ng_Width = 56;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_LENGTHMIN;
	gadlist[DSGAD_LENGTHMIN] = gad = CreateGadget(INTEGER_KIND, gad, &ng, 
									STRINGA_Justification,	GACT_STRINGRIGHT,
									GA_Disabled,				TRUE,
									GTIN_Number,				LengthVals[0],
									TAG_END);

	ng.ng_TopEdge = 109 + topborder;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_TARGETNUM;
	sprintf(SpeedStr,"%.2f", SpeedVals[0]);
	gadlist[DSGAD_TARGETNUM] = gad = CreateGadget(STRING_KIND, gad, &ng, 
//									STRINGA_Justification,	GACT_STRINGRIGHT,
									GTST_String,				SpeedStr,
									GA_Disabled,				TRUE,
									TAG_END);

	ng.ng_LeftEdge = 366;
	ng.ng_TopEdge = 15 + topborder;
	ng.ng_Width = 88;
	ng.ng_Height = 10;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_RESLABLOW;
	gadlist[DSGAD_RESLABLOW] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	"Low(MB/s):",
									TAG_END);
	ng.ng_TopEdge = 25 + topborder;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_RESLABAVG;
	gadlist[DSGAD_RESLABAVG] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	"Avg(MB/s):",
									TAG_END);
	ng.ng_TopEdge = 35 + topborder;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_RESLABIDL;
	gadlist[DSGAD_RESLABIDL] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	" Idle(ms):",
									TAG_END);


	sprintf(ResList[0],"%5.2f", 0.0);
	sprintf(ResList[1],"%5.2f", 0.0);
	sprintf(ResList[2],"%5.2f", 0.0);
	sprintf(ResList[3],"%5.2f", 0.0);
	sprintf(ResList[4],"%5ld", 0L);
	sprintf(ResList[5],"%5ld", 0L);
	ng.ng_LeftEdge = 449;
	ng.ng_TopEdge = 15 + topborder;
	ng.ng_Width = 50;
	ng.ng_Height = 10;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = DSGAD_RESLISTLR;
	gadlist[DSGAD_RESLISTLR] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ResList[0],
									TAG_END);

	ng.ng_TopEdge = 25 + topborder;
	ng.ng_GadgetID = DSGAD_RESLISTAR;
	gadlist[DSGAD_RESLISTAR] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ResList[2],
									TAG_END);

	ng.ng_TopEdge = 35 + topborder;
	ng.ng_GadgetID = DSGAD_RESLISTIR;
	gadlist[DSGAD_RESLISTIR] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ResList[4],
									TAG_END);

	ng.ng_LeftEdge = 504;
	ng.ng_TopEdge = 15 + topborder;
	ng.ng_GadgetID = DSGAD_RESLISTLW;
	gadlist[DSGAD_RESLISTLW] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ResList[1],
									TAG_END);

	ng.ng_TopEdge = 25 + topborder;
	ng.ng_GadgetID = DSGAD_RESLISTAW;
	gadlist[DSGAD_RESLISTAW] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ResList[3],
									TAG_END);

	ng.ng_TopEdge = 35 + topborder;
	ng.ng_GadgetID = DSGAD_RESLISTIW;
	gadlist[DSGAD_RESLISTIW] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	ResList[5],
									TAG_END);

	ng.ng_LeftEdge = 446;
	ng.ng_TopEdge = 14 + topborder;
	ng.ng_Width = 110;
	ng.ng_Height = 31;
	ng.ng_GadgetText = "  Read  Write";
	ng.ng_GadgetID = DSGAD_RESBOX;
	ng.ng_Flags = PLACETEXT_ABOVE;
	gadlist[DSGAD_RESBOX] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	"",
									GTTX_Border,TRUE,
									TAG_END);

	ng.ng_LeftEdge = 381;
	ng.ng_TopEdge = 53 + topborder;
	ng.ng_Width = 87;
	ng.ng_Height = 12;
	ng.ng_GadgetText = "View Log";
	ng.ng_GadgetID = DSGAD_VIEWLOG;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_VIEWLOG] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 469;
	ng.ng_GadgetText = "Save Log";
	ng.ng_GadgetID = DSGAD_SAVELOG;
	gadlist[DSGAD_SAVELOG] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 381;
	ng.ng_TopEdge = 65 + topborder;
	ng.ng_Width = 175;
	ng.ng_GadgetText = "View Cache Settings";
	ng.ng_GadgetID = DSGAD_VIEWCACHE;
	gadlist[DSGAD_VIEWCACHE] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 504;
	ng.ng_TopEdge = 109 + topborder;
	ng.ng_Width = 52;
	ng.ng_Height = 13;
	ng.ng_GadgetText = "% of Drive Over Target:";
	ng.ng_GadgetID = DSGAD_PERCENT;
	ng.ng_Flags = PLACETEXT_LEFT;
	gadlist[DSGAD_PERCENT] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,		PercentStr,
									GTTX_Border,	TRUE,
									TAG_END);
	ng.ng_LeftEdge = 73;
	ng.ng_TopEdge = 127 + topborder;
	ng.ng_Width = 483;
	ng.ng_Height = 13;
	ng.ng_GadgetText = "Status:";
	ng.ng_GadgetID = DSGAD_STATUS;
	gadlist[DSGAD_STATUS] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
									GTTX_Text,	"",
									GTTX_Border,	TRUE,
									TAG_END);

	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 144 + topborder;
	ng.ng_Width = 138;
	ng.ng_Height = 12;
	ng.ng_GadgetText = "Start Test";
	ng.ng_GadgetID = DSGAD_START;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_START] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 216;
	ng.ng_GadgetText = "Rescan Drives";
	ng.ng_GadgetID = DSGAD_RESCAN;
	gadlist[DSGAD_RESCAN] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	ng.ng_LeftEdge = 418;
	ng.ng_GadgetText = "EXIT";
	ng.ng_GadgetID = DSGAD_EXIT;
	gadlist[DSGAD_EXIT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);

	return(gad);
}

void ProcessWindowEvents(struct Window *win, struct Gadget *gadlist[],
									BOOL (*HandleEvent)(struct Window *, 
																struct Gadget *, 
																UWORD,
																struct Gadget **))
{
	struct IntuiMessage *imsg;
	ULONG imsgClass;
	UWORD imsgCode;
	struct Gadget *gad;
	BOOL terminated = FALSE;

	while (!terminated)
	{
		Wait (1<<win->UserPort->mp_SigBit);
		while ((!terminated) && (imsg = GT_GetIMsg(win->UserPort)))
		{
			gad = (struct Gadget *)imsg->IAddress;

			imsgClass = imsg->Class;
			imsgCode = imsg->Code;

			GT_ReplyIMsg(imsg);

			switch(imsgClass)
			{
				case IDCMP_GADGETDOWN:
				case IDCMP_GADGETUP:
				case IDCMP_MOUSEMOVE:
					terminated = (*HandleEvent)(win, gad, imsgCode, gadlist);
					break;
				case IDCMP_CLOSEWINDOW:
					terminated = TRUE;
					break;
				case IDCMP_REFRESHWINDOW:
					GT_BeginRefresh(win);
					GT_EndRefresh(win, TRUE);
					break;
			}
		}
	}
}

void DSWindow( void )
{
	struct TextFont	*font;
	struct Screen		*scr;
	struct Window		*win;
	struct Gadget		*glist, *gadlist[25];
	void 					*vi;
	UWORD					 topborder;
	
	if (NULL == (font = OpenFont(&Topaz80)))
		errorMessage("Failed to Open Topaz 80");
	else
	{
		if (NULL == (scr = LockPubScreen(NULL)))
			errorMessage("Could not lock public screen");
		else
		{
			if (NULL ==(vi = GetVisualInfo(scr, TAG_END)))
				errorMessage("GetVisualInfo() failed");
			else
			{
				topborder = scr->WBorTop + (scr->Font->ta_YSize + 1);
				if (NULL == (CreateAllGadgets(&glist, vi, topborder, gadlist)))
					errorMessage("CreateAllGadgets() Failed");
				else
				{
					if (NULL ==(win = OpenWindowTags(NULL,
											WA_Title, 			"DriveSpeed 2.00",
											WA_Gadgets,			glist,
											WA_Left,				35,
											WA_Top,				13,
											WA_Width,			570,
											WA_Height,			173,
											WA_DragBar,			TRUE,
											WA_DepthGadget,	TRUE,
											WA_Activate,		TRUE,
											WA_CloseGadget,	TRUE,
											WA_SimpleRefresh,	TRUE,
											WA_IDCMP,			IDCMP_CLOSEWINDOW |
																	IDCMP_REFRESHWINDOW |
																	LISTVIEWIDCMP,
											WA_PubScreen,		scr,
											TAG_END)))
						errorMessage("OpenWindow() Failed");
					else
					{
						StatusWin = win;
						StatusGadList = gadlist;

						GT_RefreshWindow(win, NULL);
						SetWait(win);
						GetDriveList(win, gadlist);
						ClearWait(win);
						ProcessWindowEvents(win, gadlist, 
												(BOOL (*)(struct Window *,
															 struct Gadget *,
															 UWORD,
							 								 struct Gadget **))HandleGadgetEvent);
						CloseWindow(win);
					}
				}
				FreeGadgets(glist);
				FreeVisualInfo(vi);
			}
			UnlockPubScreen(NULL, scr);
		}
		CloseFont(font);
	}
}

void main( void )
{
	version = "$VER:2.0 Flyer DriveSpeed (7-5-95)\nCopyright  1995 NewTek, Inc.\n";

 //  MAKE SURE THAT THE TOASTER IS RUNNING 
	if (!FindToaster()){
		printf("Toaster is not running!  Please start the Toaster and run this test again!");
		CloseLibrary(FlyerBase);
		exit(20);
	}

	if (NULL == (IntuitionBase = OpenLibrary("intuition.library", 37L)))
		errorMessage("Requires Version 37 intuition library");
	else
	{
		if (NULL == (GfxBase = OpenLibrary("graphics.library",37L)))
			errorMessage("Requires Version 37 graphics library");
		else
		{
			if (NULL == (GadToolsBase = OpenLibrary("gadtools.library",37L)))
				errorMessage("Requires Version 37 gadtools library");
			else
			{
				if (NULL == (FlyerBase = OpenLibrary("flyer.library",0))) 
				{
					errorMessage("Open flyer.library failed");
				}
				else
				{
					DSWindow();
					CloseLibrary(FlyerBase);
				}
				CloseLibrary(GadToolsBase);
			}		
			CloseLibrary(GfxBase);
		}
		CloseLibrary(IntuitionBase);
	}
}

void GetDriveList(struct Window *win, struct Gadget **gadlist)
{
	UBYTE i,j, index;
	char str[62] = "";
	UBYTE MiscBuff[18], drives;
	ULONG res;

	FlyVol.Board = 0;

	GT_SetGadgetAttrs(gadlist[DSGAD_DRIVELIST], win, NULL,
															GTLV_Labels,		~0,
															TAG_END);
	FreeList(DriveList);
	if (!(DriveList = AllocMem(sizeof(struct List), MEMF_CLEAR)))
	{
		StatusMessage("Out of Memory!");
	}
	else
	{
		NewList(DriveList);
		StatusMessage("Finding Flyer Drives...");
		for(i=0, index = 0;i<=2;i++)
		{
			FlyVol.SCSIdrive = (i<<3);
			res = SCSIinit(&FlyVol);
			if (res) {
				StatusMessage("Error: Could not initialize SCSI channel ");
			}
			res = FindDrives(&FlyVol,MiscBuff);
			if (res) {
				StatusMessage("Error: Could Not Find Drives");
			}

			drives = MiscBuff[0];	/* Bit flags for each ID on bus (1=drive present) */
			
			for(j=0;j<=6;j++)
			{
				if ((1<<j)&drives)
				{
					GetDriveInfo((i<<3) + j, str, win, gadlist);
					if (str[0])
						AddDListNode(DriveList, str, index++, win, gadlist);
				}
			}
		}
		if (!index)
		{
			sprintf(str, "<No Flyer Drives Present>");
			AddDListNode(DriveList, str, -1, win, gadlist);
		}
		GT_SetGadgetAttrs(gadlist[DSGAD_DRIVELIST], win, NULL,
																GTLV_Labels, DriveList,
																GTLV_Selected,		0,
																TAG_END);		
	}
}

void AddDListNode(struct List *list, char *str, int index, 
						struct Window *win, struct Gadget **gadlist)
{
	struct DLNode *Dnode;

	if (!(Dnode = AllocMem(sizeof(struct DLNode), MEMF_CLEAR)))
		StatusMessage("Out Of Memory!");
	else
	{
		strcpy(Dnode->str, str);
		Dnode->node.ln_Name = Dnode->str;
		Dnode->node.ln_Pri = 0;
		Dnode->node.ln_Type = 100;
		Dnode->index = index;
		if ((list == DriveList)&&(str[0] != '<'))
		{
			Dnode->drive = ((str[0] - 'A') * 8) + (str[2] - '0');
		}
		AddTail(list, (struct Node *)Dnode);
	}
}

void FreeList( struct List *list )
{
	struct DLNode *worknode, *nextnode;

	if (list)
	{
		worknode = (struct DLNode *)(list->lh_Head);
		while (nextnode = (struct DLNode *)(worknode->node.ln_Succ))
		{
			FreeMem(worknode, sizeof(struct DLNode));
			worknode = nextnode;
		}
		FreeMem(list, sizeof(struct List));
	}
}

void GetDriveInfo(UBYTE ID, char *strbuf, struct Window *win, struct Gadget **gadlist)
{
	ULONG res, blkcnt;
	struct FlyerVolume FV;
	char ss[255];

	FV.Path = (char *)NULL;
	FV.Board = 0;
	FV.SCSIdrive = ((ID >> 3) << 3);
	FV.Flags = FVF_USENUMS;

//	sprintf(ss, "Getting info for drive %ld", ID);
//	StatusMessage(ss);

	/* Get inquiry data from SCSI drive */
	FV.SCSIdrive = ID;
	res = Inquiry(&FV,36,InqBuff);
	if (res) {
		StatusMessage("Inquiry Error. Please make sure that Flyer has started");
		strbuf[0] = '\0';
	}
	else
	{
		if (!(InqBuff[0] & (UBYTE)0x1F)) /* Make sure that the device is a hard-drive */
		{
			res = ReadSize(&FV, &blkcnt, &blklen);
			if (res)
			{
				sprintf(ss, "ReadSize Error: %ld - %ls", res, Error2String(res));
				StatusMessage(ss);
				strbuf[0] = '\0';
			}
			else
				sprintf(strbuf,"%lc %1ld %-.23s %.4s", ('A' + (ID >> 3)),
							(ID % 8),
							&(InqBuff[8]),
							&(InqBuff[32]));
		}
		else
		{
			strbuf[0] = '\0';
		}
	}
}

int FindToaster( void )
{
	struct MsgPort *port;
	port = FindPort("ToasterARexx.port");

	if (port)
		return(1);
	else
		return(0);
}

float Str2Float(char *str)
{
	float num;
	int i;
	float power;
	
	num = 0.0;

	for(i=0;str[i] != '\0';i++)
	{
		if ((str[i] >= '0') && (str[i] <= '9'))
			num = (num * 10.0) + (float)(str[i] - '0');
		else
			if (str[i] == '.')
			{
				i++;
				break;
			}
		while (num >= 100.0)
		{
			num -= 100.0;
		}
	}

	for(power = 10.0;str[i] != '\0', power < 1000.0;i++)
	{
		if ((str[i] >= '0') && (str[i] <= '9'))
		{
			num += (float)(str[i] - '0') / power;
			power *= 10.0;
		} 
	}
	return(num);
}

void errorMessage(char *msg)
{
	if (msg)
		printf("Error: %s\n", msg);
}

void StatusMessage( char *msg )
{
	static char StatusStr[128];

	strncpy(StatusStr, msg, 127);

	GT_SetGadgetAttrs(StatusGadList[DSGAD_STATUS], StatusWin, NULL,
													GTTX_Text,	StatusStr,
													TAG_END);
}

void cleanup( void ) 
{
	struct LBAStats *temp;

	/* remove TestStats */
	while (tcal_read_stats.first != NULL){
		temp = tcal_read_stats.first->next;
		FreeMem(tcal_read_stats.first, sizeof(struct LBAStats));
		tcal_read_stats.first = temp;
	}	
	while (tcal_write_stats.first != NULL){
		temp = tcal_write_stats.first->next;
		FreeMem(tcal_write_stats.first, sizeof(struct LBAStats));
		tcal_write_stats.first = temp;
	}	
	if (file != NULL)
		fclose(file);
	if (LogList)
		FreeList(LogList);
	if (DriveList)
		FreeList(DriveList);
}

void StartTest(struct Window *win, struct Gadget *gadlist[])
{
	int first, last, writeflag, minutes;
	struct FlyerVolInfo Buffer;
	UBYTE formatted = 0, reply;
	ULONG res, lba;
	struct LBAStats *current;
	char str[128];

	/* remove TestStats */
	while (tcal_read_stats.first != NULL){
		current = tcal_read_stats.first->next;
		FreeMem(tcal_read_stats.first, sizeof(struct LBAStats));
		tcal_read_stats.first = current;
	}	
	while (tcal_write_stats.first != NULL){
		current = tcal_write_stats.first->next;
		FreeMem(tcal_write_stats.first, sizeof(struct LBAStats));
		tcal_write_stats.first = current;
	}	

	tcal_read_stats.first = NULL;
	tcal_read_stats.last = NULL;
	tcal_read_stats.num_tests = 0;
	tcal_read_stats.lowest = NULL;
	tcal_read_stats.average = 0.0;
	tcal_read_stats.max_drop = NULL;

	tcal_write_stats.first = NULL;
	tcal_write_stats.last = NULL;
	tcal_write_stats.num_tests = 0;
	tcal_write_stats.lowest = NULL;
	tcal_write_stats.average = 0.0;
	tcal_write_stats.max_drop = NULL;

	/*  May want to add an abort option using BuildEasyRequest and SysReqHandler... */

	FlyVol.Board = 0;
	FlyVol.SCSIdrive = drive;
	FlyVol.Flags = FVF_USENUMS;

	minutes = (int)(LengthVals[LengthIndex]);

	// Verify that the user's last entry in the length and speed fields
	// are used, since that may not be the case in the custom value option...
	if (LengthIndex == EN_LENGTH)
		LengthVals[EN_LENGTH] = (ULONG)
			((struct StringInfo *)(gadlist[DSGAD_LENGTHMIN]->SpecialInfo))->LongInt;

	if (TargetIndex == EN_SPEED)
	{
		SpeedVals[EN_SPEED] = Str2Float(((struct StringInfo *)
									(gadlist[DSGAD_TARGETNUM]->SpecialInfo))->Buffer);
		sprintf(SpeedStr, "%.2f", SpeedVals[EN_SPEED]);
		// Update the gadget to display the correct value...
		GT_SetGadgetAttrs(gadlist[DSGAD_TARGETNUM], win, NULL,
										GTST_String,	SpeedStr,
										TAG_END);
	}
	
	/* Get inquiry data from SCSI drive */
	res = Inquiry(&FlyVol,36,InqBuff);
	if (res) {
		sprintf(str,"Inquiry Error %ld - %ls  Test Aborted!",res,Error2String(res));
		StatusMessage(str);
	}
	else
	{
		res = SCSIinit(&FlyVol); //  flush SCSI bus 
		if (res)
		{
			sprintf(str,"SCSIinit Error %ld - %ls Test Aborted!",res,Error2String(res));
			StatusMessage(str);
		}
		else
		{
			/* Get inquiry data from SCSI drive */
			/* This command is necessary because some Flyer commands fail 
				if called following a SCSIinit...                           */
			res = Inquiry(&FlyVol,36,InqBuff);
			if (res) {
				sprintf(str,"Inquiry Error %ld - %ls Test Aborted!",res,Error2String(res));
				StatusMessage(str);
			}
			else
			{
				Buffer.len = sizeof(struct FlyerVolInfo);
				//  CHECK DRIVE TO SEE IF FORMATTED 
				formatted = 0;
				res = FlyerDriveInfo(&FlyVol,&Buffer);
				if (!res && (Buffer.Ident == 0x524f4f54) && (Buffer.Version == 1))
					formatted = 1;
				if (do_write)
				{
					if ((reply = WriteWarning(win, gadlist)) == 2)  //  Chose to do a read test only...
					{
						do_read = 1;
						do_write = 0;
						GT_SetGadgetAttrs(gadlist[DSGAD_TESTTYPE], win, NULL,
														GTCY_Active,	0,
														TAG_END);
					}
				}
				else
					reply = 1;
				if (reply)
				{
					if (!(InqBuff[0] & (UBYTE)0x1F)) /* Make sure that the device is a hard-drive */
					{
						/* Get drive specs: # blocks, block size */
						res = ReadSize(&FlyVol,&max,&blklen);
						if (res) {
							sprintf(str,"ReadSize Error %ld - %ls Aborting Test!",res,Error2String(res));
							StatusMessage(str);
						}
						else
						{
							size = max;
							max -= (TOTALSIZE + 1);  /* Ensure Quantum drives don't hang at end of test... */
							tcal_read_stats.num_tests = 0;
							tcal_write_stats.num_tests = 0;
							for (writeflag=0;writeflag <= 1;writeflag++) {
								current = NULL;
								if((do_read && (!writeflag)) || (do_write && writeflag)) 
								{
									sprintf(str,"Doing drive %ls test...",(writeflag?"write":"read"));
									StatusMessage(str);

									DateStamp(&TestStart);		/* Log time we started test */
									first = 1;
									last = 0;
									//  START TEST 
									for (lba=0;!last;lba+=TOTALSIZE,first=0) { 
										/* Prevent last test from going past end of drive */
										if ((lba+TOTALSIZE) >= max) {
											lba = max-TOTALSIZE;
											last = 1;	/* This is the last test */
											break; //  MAKE SURE THAT THE LAST TEST AREA IS NOT USED 
										}	
										//  CREATE NEW LIST ENTRY 
										if (!first) {
											if (writeflag)
												current = NewStat(current,&tcal_write_stats,lba,
																DoTest(lba,TESTBLKS,REPEAT, writeflag));
											else
												current = NewStat(current,&tcal_read_stats,lba,
																DoTest(lba,TESTBLKS,REPEAT, writeflag));
											if (current == NULL)
												break;
										}
										/* If number of minutes specified has elapsed, quit */
										DateStamp(&Now);				/* Log time now */
										if ((minutes > 0) && ((ElapsedTicks(&TestStart,&Now)/(60*50)) >= minutes))
											break;
									}
									if (writeflag)
										DoStats(&tcal_write_stats);
									else
										DoStats(&tcal_read_stats);
								}
							}
						if ((do_write) && (formatted))
							AskReformat(win, gadlist, &Buffer);
						}
					}
				}
				else
				{
					StatusMessage("Test Aborted!");
				}
			}
		}
	}
}

UBYTE WriteWarning(struct Window *win, struct Gadget **gadlist)
{
	LONG reply;
	struct EasyStruct ESWrite = 
		{
		sizeof(struct EasyStruct), 0, NULL,
		"Performing a Write Test will\ndestroy all data on this\ndrive. Continue?\n",
		"CONTINUE|Do READ Only|ABORT",
		};

	reply = EasyRequest(win, &ESWrite, NULL );
	GT_BeginRefresh(win);
	GT_EndRefresh(win, TRUE);
	
	return((UBYTE)reply);

/*  Need to add these lines later...

		printf("Press Y and return to continue test, or anything else to abort.\n");
		for (i=0;;i++) {
			if (WaitForChar(handle,30)) {
				c = getchar();
				if (toupper(c)=='Y') {
					getchar();
					break;
				}
				else {
					printf("Aborted\n");
						getchar();
	
					CloseLibrary(FlyerBase);
					return(20);
				}
			}
			if (i > 5) {
				Action.Volume = &FlyVol;
				Action.ReturnTime = RT_STOPPED;
				res = Read10(&Action,2,0,0x1ffe);
				i = 0;
			}
		}
*/
}

void AskReformat(struct Window *win, struct Gadget **gadlist, 
						struct FlyerVolInfo *Buffer)
{
	ULONG res;
	char str[128];

	struct EasyStruct ESFormat = 
		{
		sizeof(struct EasyStruct), 0, NULL,
		"Would you like to re-format volume \"%ls\"? ",
		"ReFormat|Cancel",
		};

	if (EasyRequest(win, &ESFormat, NULL, Buffer->Title ))
	{
		GT_BeginRefresh(win);
		GT_EndRefresh(win, TRUE);
		sprintf(str,"Formatting drive %lc %ld as \"%ls\"...", (drive >> 3) + 'A',
						(drive & 0x07), Buffer->Title);
		StatusMessage(str);
		res = FlyerFormat(&FlyVol,Buffer->Title,&(Buffer->DiskDate),Buffer->Blocks,Buffer->Flags);
		if (res)
		{
			sprintf(str,"Error %ld - %ls  Format Incomplete!",res,Error2String(res));
			StatusMessage(str);
		}
		else
		{
			sprintf(str,"Finished formatting \"%ls\".",Buffer->Title);
			StatusMessage(str);
		}
	}
	else
	{
		GT_BeginRefresh(win);
		GT_EndRefresh(win, TRUE);
	}
}

void UpdateSummaryWindow(struct Window *win, struct Gadget **gadlist)
{
	if (do_read)
	{
		if (tcal_read_stats.lowest)
			sprintf(ResList[0], " %5.2f", tcal_read_stats.lowest->speed);
		sprintf(ResList[2], " %5.2f", tcal_read_stats.average);
		if (tcal_read_stats.max_drop)
			sprintf(ResList[4], " %5ld", tcal_read_stats.max_drop->drop_ms);

	}
	else
	{
		sprintf(ResList[0], " %5.2f", 0.0 );
		sprintf(ResList[2], " %5.2f", 0.0 );
		sprintf(ResList[4], " %5ld", 0L );
	}
	if (do_write)
	{
		if (tcal_write_stats.lowest)
			sprintf(ResList[1], " %5.2f", tcal_write_stats.lowest->speed);
		sprintf(ResList[3], " %5.2f", tcal_write_stats.average);
		if (tcal_write_stats.max_drop)
			sprintf(ResList[5], " %5ld", tcal_write_stats.max_drop->drop_ms);
	}
	else
	{
		sprintf(ResList[1], " %5.2f", 0.0 );
		sprintf(ResList[3], " %5.2f", 0.0 );
		sprintf(ResList[5], " %5ld", 0L );
	}
		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTLR], win, NULL,
										GTTX_Text,	ResList[0],
										TAG_END);
		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTAR], win, NULL,
										GTTX_Text,	ResList[2],
										TAG_END);
		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTIR], win, NULL,
										GTTX_Text,	ResList[4],
										TAG_END);

		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTLW], win, NULL,
										GTTX_Text,	ResList[1],
										TAG_END);
		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTAW], win, NULL,
										GTTX_Text,	ResList[3],
										TAG_END);
		GT_SetGadgetAttrs(gadlist[DSGAD_RESLISTIW], win, NULL,
										GTTX_Text,	ResList[5],
										TAG_END);
}

ULONG DoTest(ULONG lba,ULONG testblks,ULONG repeat,int writeflag)
{
	struct	DateStamp	start,end;
	ULONG		ticks,res;
	char 		str[128];

	if (writeflag) {
		Forbid();						
		DateStamp(&start);			/* Log time now */
		res = WriteTest(&FlyVol,testblks,repeat,lba,1);
		DateStamp(&end);				/* Log time now */
		Permit();
	} else {
		Forbid();	
		DateStamp(&start);			/* Log time now */
		res = ReadTest(&FlyVol,testblks,repeat,lba,1);
		DateStamp(&end);				/* Log time now */
		Permit(); 
	}
	if (res) {
		sprintf(str,"R/W Error %ld - %ls",res,Error2String(res));
		errorMessage(str);
		return(0);
	}
	else {
		ticks = ElapsedTicks(&start,&end);
		return(ticks);
	}
}

ULONG ElapsedTicks(struct DateStamp *begin, struct DateStamp *end)
{
	ULONG		ticks;

	ticks = end->ds_Minute * 60 *50 + end->ds_Tick;
	ticks -= begin->ds_Minute * 60*50 + begin->ds_Tick;

	return(ticks);
}

struct LBAStats *NewStat(struct LBAStats *old, struct TestStats *Test, ULONG lba,
								 ULONG ticks)
{
	struct LBAStats *new;
	char	 str[128]; 
	new = (struct LBAStats *)AllocMem(sizeof(struct LBAStats),MEMF_ANY || MEMF_CLEAR);

	if (new) {
		new->lba = lba;
		new->ticks = ticks;
		new->next = NULL;
		if (!(new->prev = old))
			Test->first = new;
		else
			new->prev->next = new;
		Test->last = new;	
	} else {
		sprintf(str,"Sorry..Can't allocate Memory!");
		errorMessage(str);
	}
	return(new);	
}

void DoStats(struct TestStats *test)
{
	int i;
	double meg,total_spd;
	struct LBAStats *lowest,*current;
	
	meg = ((double)(TOTALSIZE)* 512.0)/1000000.0;
	
	lowest = test->max_drop = current = test->first;
	total_spd = (double)0.0;

	for(i=0;current != NULL;i++,current = current->next) {
		current->speed = meg/((double)current->ticks/50.0);
		total_spd += current->speed; 
		if (current->speed < lowest->speed)
			lowest = current;
		if (current->prev != NULL) {
			if ((current->lba - current->prev->lba) <= TOTALSIZE)
				current->drop = (long)((long)current->ticks - (long)current->prev->ticks);
			else
				current->drop = 0L;
		} else 
			current->drop = 0L;	
		current->drop_ms = current->drop * 20L;
	}

	test->num_tests = i;
	test->lowest = lowest;
	test->average = (total_spd / i);
	
	/* Mark all Dips */

	current = test->first;

	while (current != NULL){
		if (current->prev == NULL)
			current->dip = 0;
		else {
			if (current->next == NULL)
				current->dip = 0;
			else {
				if ((current->drop_ms > 40L) && (current->next->drop_ms < -40L)){
					current->dip = 1;
					if (current->drop_ms > test->max_drop->drop_ms)
						test->max_drop = current;
				} else
					current->dip = 0;
			}
		}
		current = current->next;
	}
}

void UpdateShortStroke(struct Window *win, struct Gadget **gadlist)
{
	ss_block = (TargetIndex ?  
					FindLastBlockAtSpeed(SpeedVals[TargetIndex], do_read, do_write) :
					(ULONG) 0);

	sprintf(PercentStr,"%.1f", (((double)ss_block/(double)size) * 100.0));
	GT_SetGadgetAttrs(gadlist[DSGAD_PERCENT], win, NULL,
									GTTX_Text,	PercentStr,
									TAG_END);
}

ULONG	FindLastBlockAtSpeed(double spd, int read, int write)
{
	ULONG readblock,writeblock,block;
	struct LBAStats *current;

	readblock = writeblock = 0L;

	/* TCal Read Stats */	
	if (read){
		for(current = tcal_read_stats.first;current != NULL;current = current->next)
		{
			if (current->speed < spd){
				if (current->dip){
					if (current->next != NULL){
						if (current->next->speed >= spd)			 
							readblock = current->lba;
						else
							break;
					} else {
						break;
					}
				} else {
					break;
				}
			} else {
				readblock = current->lba;	
			}
		}
	}

	/* TCal Write Stats */	
	if (write){
		for(current = tcal_write_stats.first;current != NULL;current = current->next)
		{
			if (current->speed < spd){
				if (current->dip){
					if (current->next != NULL){
						if (current->next->speed >= spd)			 
							writeblock = current->lba;
						else
							break;
					} else {
						break;
					}
				} else {
					break;
				}
			} else {
				writeblock = current->lba;	
			}
		}
	}

	if (read)
		if (write)
			if (readblock < writeblock)
				block = readblock;
			else
				block = writeblock;
		else
			block = readblock;
	else
		block = writeblock;

	return(block);
}

void CreateLog(struct Window *win, struct Gadget **gadlist)
{
	struct LBAStats *read,*write,*dummy;
	char str[128];
	char Rstr[64] = "";
	char Wstr[64] = ""; 

	FreeList(LogList);

	if (!(LogList = AllocMem(sizeof(struct List), MEMF_CLEAR)))
	{
		StatusMessage("Out of Memory!");
	}
	else
	{
		NewList(LogList);
		StatusMessage("Building Log...");
	
		if (do_read)
			dummy = read = tcal_read_stats.first;
		if (do_write)
			dummy = write = tcal_write_stats.first;	

		/* Print header */
		
		sprintf(str,"Manufacturer/Model: %-.23s", &InqBuff[8]);
		AddDListNode(LogList, str, 0, win, gadlist);
		sprintf(str,"Revision: %-.4s", &InqBuff[32]);
		AddDListNode(LogList, str, 0, win, gadlist);
		sprintf(str,"Size: %ld Blocks (%.2f GB)", size, (((double)size)*((double)blklen)/(double)(1000000000.0)));
		AddDListNode(LogList, str, 0, win, gadlist);
		AddDListNode(LogList, "  ", 0, win, gadlist);

		sprintf(str,"LBA:       %s%s",(do_read  ? " READ:     READ DROP:   " : ""),
											  (do_write ? " WRITE:    WRITE DROP:"  : ""));
		AddDListNode(LogList, str, 0, win, gadlist);

		while (dummy != NULL)
		{
			if (do_read)
			{
				if (read != NULL)
				{
					sprintf(Rstr," %5.2f%lc     %4ld ms%lc    ",read->speed, 
								((read->speed == tcal_read_stats.lowest->speed)? '*':' '),
								read->drop_ms,
								((read->lba == tcal_read_stats.max_drop->lba)? '*':' '));
					read = read->next;
				}
				else
				{
					sprintf(Rstr,"                       ");
				}
			}

			if (do_write)
			{
				if (write != NULL)
				{
					sprintf(Wstr," %5.2f%lc     %4ld ms%lc    ",write->speed,
								((write->speed == tcal_write_stats.lowest->speed)? '*':' '),
								write->drop_ms,
								((write->lba == tcal_write_stats.max_drop->lba)? '*':' '));
					write = write->next;
				}
				else
				{
					sprintf(Wstr,"                        ");
				}
			}
			sprintf(str,"%8lx   %s%s",dummy->lba, Rstr, Wstr);
			AddDListNode(LogList, str, 0, win, gadlist);
			dummy = dummy->next;
		}
		AddDListNode(LogList, "  ", 0, win, gadlist);

		sprintf(str,"                 %s%s",(do_read  ? "    READ "  : ""),
														(do_write ? "    WRITE " : ""));
		AddDListNode(LogList, str, 0, win, gadlist);

		sprintf(str,"-----------------%s%s",(do_read  ? "   ------" : ""),
														(do_write ? "   ------" : ""));
		AddDListNode(LogList, str, 0, win, gadlist);

		if (do_read)
			sprintf(Rstr,"    %.2f ",tcal_read_stats.lowest->speed);
		else
			sprintf(Rstr,"");
		if (do_write)
			sprintf(Wstr,"    %.2f ",tcal_write_stats.lowest->speed);
		else
			sprintf(Wstr,"");
		sprintf(str,"Lowest:          %s%s", Rstr, Wstr);
		AddDListNode(LogList, str, 0, win, gadlist);

		if (do_read)
			sprintf(Rstr,"    %.2f ",tcal_read_stats.average);
		else
			sprintf(Rstr,"");
		if (do_write)
			sprintf(Wstr,"    %.2f ",tcal_write_stats.average);
		else
			sprintf(Wstr,"");
		sprintf(str,"Average:         %s%s", Rstr, Wstr);
		AddDListNode(LogList, str, 0, win, gadlist);

		if (do_read)
			sprintf(Rstr,"   %3ld ms ",(tcal_read_stats.max_drop->drop_ms));
		else
			sprintf(Rstr,"");
		if (do_write)
			sprintf(Wstr,"   %3ld ms ",(tcal_write_stats.max_drop->drop_ms));
		else
			sprintf(Wstr,"");
		sprintf(str,"Idle Time 20 ms:%s%s", Rstr, Wstr);
		AddDListNode(LogList, str, 0, win, gadlist);
		AddDListNode(LogList, " ", 0, win, gadlist);

		sprintf(str,"NOTE: A MegaByte is measured as 1,000,000 bytes to be");
		AddDListNode(LogList, str, 0, win, gadlist);
		sprintf(str,"      compatible with drive manufacturer specifications.");
		AddDListNode(LogList, str, 0, win, gadlist);
		AddDListNode(LogList, " ", 0, win, gadlist);

		if (SpeedVals[TargetIndex] != 0.0){
			sprintf(str,"The last block that sustained %.2f MB/s was %ld (0x%lx)",
						SpeedVals[TargetIndex], ss_block, ss_block);
			AddDListNode(LogList, str, 0, win, gadlist);
			sprintf(str,"This would allow %.2f GB of this %.2f GB drive (%.1f\%)",
					(((double)ss_block)*((double)blklen)/(double)(10000000000.)),
					(((double)size)*((double)blklen)/(double)(1000000000.0)),
					(((double)ss_block/(double)size) * 100.0));
			AddDListNode(LogList, str, 0, win, gadlist);
			AddDListNode(LogList, " ", 0, win, gadlist);
		}

		if (!good_tcal) {
			sprintf(str,"Please note: The test was not performed on the entire surface");
			AddDListNode(LogList, str, 0, win, gadlist);
			sprintf(str,"of the drive, so this may not have been an accurate evaluation ");
			AddDListNode(LogList, str, 0, win, gadlist);
			sprintf(str,"of the drive. To be certain that the test is an accurate");
			AddDListNode(LogList, str, 0, win, gadlist);
			sprintf(str,"evaluation of the drive, please re-run the test using a value");
			AddDListNode(LogList, str, 0, win, gadlist);
			sprintf(str,"of 0 to test the entire surface of the drive.");
			AddDListNode(LogList, str, 0, win, gadlist);
			AddDListNode(LogList, " ", 0, win, gadlist);
		}
	}
}

void ViewLog( void )
{
	struct TextFont	*font;
	struct Screen		*scr;
	struct Window		*win;
	struct Gadget		*glist, *gadlist[25];
	void 					*vi;
	UWORD					 topborder;
	
	if (NULL == (font = OpenFont(&Topaz80)))
		errorMessage("Failed to Open Topaz 80");
	else
	{
		if (NULL == (scr = LockPubScreen(NULL)))
			errorMessage("Could not lock public screen");
		else
		{
			if (NULL ==(vi = GetVisualInfo(scr, TAG_END)))
				errorMessage("GetVisualInfo() failed");
			else
			{
				topborder = scr->WBorTop + (scr->Font->ta_YSize + 1);
				if (NULL == (CreateLogGadgets(&glist, vi, topborder, gadlist)))
					errorMessage("CreateLogGadgets() Failed");
				else
				{
					if (NULL ==(win = OpenWindowTags(NULL,
											WA_Title, 			"DriveSpeed 2.00 Log",
											WA_Gadgets,			glist,
											WA_Left,				35,
											WA_Top,				13,
											WA_Width,			570,
											WA_Height,			173,
											WA_DragBar,			TRUE,
											WA_DepthGadget,	TRUE,
											WA_Activate,		TRUE,
											WA_CloseGadget,	TRUE,
											WA_SimpleRefresh,	TRUE,
											WA_IDCMP,			IDCMP_CLOSEWINDOW |
																	IDCMP_REFRESHWINDOW |
																	LISTVIEWIDCMP,
											WA_PubScreen,		scr,
											TAG_END)))
						errorMessage("OpenWindow() Failed");
					else
					{
						GT_RefreshWindow(win, NULL);
						ProcessWindowEvents(win, gadlist, 
												(BOOL (*)(struct Window *,
															 struct Gadget *,
															 UWORD,
							 								 struct Gadget **))HandleLogEvent);
						CloseWindow(win);
					}
				}
				FreeGadgets(glist);
				FreeVisualInfo(vi);
			}
			UnlockPubScreen(NULL, scr);
		}
		CloseFont(font);
	}
}

BOOL HandleLogEvent(struct Window *win, struct Gadget *gad, UWORD code,
								struct Gadget *gadlist[])
{
	BOOL terminated = FALSE;

	switch(gad->GadgetID)
	{
		case DSGAD_LOGLIST:
			break;
		case DSGAD_LOGSAVE:
			SaveLog(win, gadlist);
			break;
		case DSGAD_LOGCONT:
			terminated = TRUE;
			break;
	}
	return(terminated);
}

struct Gadget *CreateLogGadgets(struct Gadget **glistptr, void *vi,
					UWORD topborder, struct Gadget *gadlist[])
{
	struct NewGadget ng;
	struct Gadget *gad;

	gad = CreateContext(glistptr);

	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 14 + topborder;
	ng.ng_Width = 540;
	ng.ng_Height = 123;
	ng.ng_GadgetText = "";
	ng.ng_TextAttr = &Topaz80;
	ng.ng_VisualInfo = vi;
	ng.ng_GadgetID = DSGAD_LOGLIST;
	ng.ng_Flags = PLACETEXT_ABOVE;
	gadlist[DSGAD_LOGLIST] = gad = CreateGadget(LISTVIEW_KIND, gad, &ng,
									GTLV_Labels,	LogList,
									GTLV_ReadOnly,	TRUE,
									
									TAG_END);	
	ng.ng_LeftEdge = 14;
	ng.ng_TopEdge = 144 + topborder;
	ng.ng_Width = 138;
	ng.ng_Height = 12;
	ng.ng_GadgetText = "Continue";
	ng.ng_GadgetID = DSGAD_LOGCONT;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[DSGAD_LOGCONT] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);
	ng.ng_LeftEdge = 418;
	ng.ng_GadgetText = "Save Log";
	ng.ng_GadgetID = DSGAD_LOGSAVE;
	gadlist[DSGAD_LOGSAVE] = gad = CreateGadget(BUTTON_KIND, gad, &ng, 
									TAG_END);
	return(gad);
}

void SaveLog(struct Window *win, struct Gadget **gadlist)
{
	char filename[192] = "";

	if (LogDir[0] == '\0')
		strcpy(LogDir,"ram:");
	if (LogFile[0] == '\0')
		strcpy(LogFile, "DriveSpeed.log");

	ReqFile(LogDir, LogFile);

	strcpy(filename,LogDir);
	AddPart(filename, LogFile, 192);
	WriteListFile(LogList, filename);
}

void ReqFile(char *dir, char *file)
{
	struct TagItem frtags[] =
	{
		ASL_Hail,		(ULONG)"Save DriveSpeed Log As...",
		ASL_Height,		180,
		ASL_Width,		320,
		ASL_LeftEdge,	160,
		ASL_TopEdge,	10,
		ASL_OKText,		(ULONG)"OK",
		ASL_CancelText,(ULONG)"Cancel",
		ASL_File,		"DriveSpeed.log",
		TAG_DONE
	};

	struct FileRequester *fr;

	if (AslBase = OpenLibrary("asl.library", 37L))
	{
		if (fr = (struct FileRequester *)AllocAslRequest(ASL_FileRequest, frtags))
		{
			fr->rf_Dir = dir;
			fr->rf_File = file;

			if (AslRequest(fr, NULL))
			{	
				strcpy(dir, fr->rf_Dir);
				strcpy(file, fr->rf_File);
			}
		FreeAslRequest(fr);
		}	
		CloseLibrary(AslBase);
	}
}

void WriteListFile(struct List *list, char *filename)
{
	FILE *file = NULL;
	struct DLNode *current;

	if (file = fopen(filename, "w"))
	{
		if (list)
		{
			current = (struct DLNode *)(list->lh_Head);
			
			while(current)
			{
				if ((current->str) && (current->node.ln_Succ))
					fprintf(file, "%s\n", current->str);
				current = (struct DLNode *)(current->node.ln_Succ);
			}
		}
		fclose(file);
	}
	else
		errorMessage("Can't Open File!"); 
}

void ViewCache(struct Window *win, struct Gadget **gadlist) 
{

}

void PurgeMsgPort(struct Window *win)
{
	struct IntuiMessage *imsg;
	ULONG imsgClass;
	UWORD imsgCode;
	BOOL terminated = FALSE;

	while (imsg = GT_GetIMsg(win->UserPort))
	{
		imsgClass = imsg->Class;
		GT_ReplyIMsg(imsg);
		switch(imsgClass)
		{
			case IDCMP_REFRESHWINDOW:
				GT_BeginRefresh(win);
				GT_EndRefresh(win, TRUE);
				break;
		}
	}
}