/* $Flyer.c$
* $Id: flyer.c,v 1.5 1996/07/19 14:52:19 Holt Exp $
* $Log: flyer.c,v $
*Revision 1.5  1996/07/19  14:52:19  Holt
*finished up cfx.
*
*Revision 1.4  1995/10/06  16:12:01  Flick
*Added Flyer_ClipInfo fn
*
*Revision 1.3  1995/10/05  18:36:28  Flick
*Moved all record/pause/stop functions to flyer.library, not crappy ES stuff!
*
*Revision 1.2  1995/10/03  18:05:43  Flick
*Made DHD_Reorg abortable
*
*Revision 1.1  1995/09/28  10:07:28  Flick
*Superficial change to DHD_AddClip
*
*Revision 1.0  1995/09/13  13:13:31  Flick
*FirstCheckIn
*
*
* Copyright (c)1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved.
*
*	9-6-95	Marty F		Created this file
*********************************************************************/
#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>

#include <flyerlib.h>
#include <flyer.h>
#include <editswit.h>
#include <panel.h>

#include <proto/exec.h>

#ifndef PROTO_PASS
#include <proto.h>
#endif

//#define SERDEBUG	1
#include <serialdebug.h>


extern struct Library 	 *FlyerBase;
extern ULONG CurCompMode;


static char ClipPathName[CLIP_PATH_MAX];
UBYTE global_LeftGain=1,global_RightGain=1;


//=============================================================
// PlayFlyerEvent
//		Send a simple play event request to Flyer
//		Currently waits for completion, although this is not
//		important for current clients of this function
//=============================================================
ULONG PlayFlyerEvent(UBYTE chan, char *name, ULONG fld, ULONG dur)
{
	struct FlyerVolume volume;
	struct ClipAction	 action;
	ULONG	err = FERR_OKAY;

	if (FlyerBase)
	{
		volume.Board = 0;
		volume.Flags = 0;			// Not FVF_USENUMS, since we don't know drive numbers !
		volume.SCSIdrive = 0;	// We let Flyer worry about silly things like drive #'s
		volume.Path = name;		// Name of Flyer file

		// Clear all fields of this structure
		memset(&action, 0, sizeof(struct ClipAction));

		action.Channel = chan;		// Video channel (NC for audio at this time)

//		action.PermissFlags = /* CAPF_STEALOURVIDEO | */ CAPF_KILLOTHERVIDEO | CAPF_USEHEADS;

		action.Flags = 0;
		action.Flags |= CAF_VIDEO;			// Video
//		action.Flags |= CAF_USEMATTE;		// Go to black when done (NOT)

		action.VidStartField = fld;		// Start field
		action.VidFieldCount = dur;		// Duration

		action.AudStartField = fld;		// Same as video
		action.AudFieldCount = dur;		// Same as video

		action.Volume = &volume;			// Link together
		action.ReturnTime = RT_STOPPED;	// Wait for complete

		err = FlyerPlay(&action);			// DO IT!
	}
	return(err);
}


//=============================================================
// GetAudioMeters
//		Read L/R audio peak info from Flyer in record mode
//=============================================================
UWORD GetAudioMeters(void)
{
	struct	FlyAudCtrl		MyAudCtrl;
	ULONG	err = FERR_OKAY;
	UWORD	s=0;

	if (FlyerBase)
	{
		err = FlyerAudioCtrl(0,&MyAudCtrl,FACOF_SENSE8);
		if (err==FERR_OKAY)
		{
			s = (MyAudCtrl.LeftSense<<8);
			s |= MyAudCtrl.RightSense;
		}
	}

	return(s);
}


void	ClearFlyerStatus(void)
{
	if (FlyerBase)
		GetClrSeqError(0,1,NULL,NULL,NULL);					// Clear status
}

ULONG	CheckFlyerStatus(UBYTE *done, ULONG *moreinfo)
{
	ULONG	err;

	if (!FlyerBase)
	{
		*done=TRUE;
		return(FERR_OKAY);
	}

	// Retrieve current play status (done ptr) and dropped fields (moreinfo ptr)
	// Either can be NULL if info is not desired
	err = GetClrSeqError(0,0,done,NULL,moreinfo);	// Get play/record status/info
	return(err);
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//~~~~~~~~~~~~~~~~~~~~~~~~ Those still sent thru ES system ~~~~~~~~~~~~~~~~~~~~~~~~
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

extern struct ESParams1 ESparams1;
extern struct ESParams2 ESparams2;
extern struct ESParams3 ESparams3;
extern struct ESParams4 ESparams4;

void	__inline DHD_Abort(ULONG Unit)
{
	DUMPUDECL("DHD_Abort( ",Unit,"); \\ ");
	SendSwitcherReply(ES_Stop,NULL);
	return;
}

void	__inline DHD_Play(ULONG Unit)
{
	DUMPUDECL("DHD_Play( ",Unit,"); \\ ");
	SendSwitcherReply(ES_Auto,NULL);
	return;
}

void	__inline DHD_Reorganize(char *name)
{
	struct FlyerVolume	volume;
	struct ClipAction		action;
	ULONG	err = FERR_NOCARD;

	DUMPSTR("DHD_Reorganize( ");
	DUMPSTR(name);
	DUMPMSG(" )");

//	ESparams1.Data1 =(LONG) name;
//	SendSwitcherReply(ES_DefragFlyer,&ESparams1);

	if (FlyerBase)
	{
//Don't do this, is it takes a LONG time and it potentially nasty.  Flyer does its own
//internal system-wide CleanupHeads() now when we request a defrag
//		VoidCardHeads(0);

		volume.Board = 0;
		volume.Flags = 0;			// Not FVF_USENUMS, since we don't know drive numbers !
		volume.SCSIdrive = 0;	// We let Flyer worry about silly things like drive #'s
		volume.Path = name;		// Name of Flyer volume

//OLD WAY (not abortable)
//		err = FlyerDeFrag(&volume);		// Do it!

		/* MUST Be initialized to zeroes */
		memset(&action,0,sizeof(action));

		action.Volume = &volume;
		action.ReturnTime = RT_IMMED;

		DUMPSTR("Flyer defrag...");
		err = FlyerDeFragNew(&action);		// Start defrag...
		if (err == FERR_OKAY)
		{
			DUMPSTR("Waiting...");
			do
			{
				if (CheckNoticeCancel())				// If CANCEL button pressed...
				{
					DUMPSTR("Aborting...");
					AbortAction(&action);				// ...abort defrag
					DUMPMSG("Aborted");
					err = FERR_ABORTED;
				}
				else
					err = CheckAction(&action);		// Still busy defrag'ing?
			} while (err == FERR_BUSY);				// Loop until done
			DUMPSTR("Done");
		}

	}
}

void	__inline DHD_StartClipList(char *Name,ULONG Destruct)
{
	DUMPUDECL("DHD_StartClipList( ",Destruct,"); \\ ");
	ESparams2.Data1 =(LONG)Name ;
	ESparams2.Data2 =(LONG)Destruct ;
	SendSwitcherReply(ES_StartClipCutList,&ESparams2);
}

ULONG	__inline DHD_EndClipList(ULONG DoIt)
{
	ULONG	err;

	DisplayWaitSprite();
	DUMPUDECL("DHD_EndClipList( ",DoIt,") \\ ");
	ESparams1.Data1 =(LONG)DoIt ;
	err = (ULONG)SendSwitcherReply(ES_EndClipCutList,&ESparams1);
	DisplayNormalSprite();

	if (err)
	{
		DUMPHEXIL("EndClipList failed!  Error = ",err,"\\");
	}

	return(err);
}

// NOTA BENE!!!  Assumes clip in/out relative to raw clip start==0
ULONG	__inline DHD_AddClip(struct NewClip	*cl,char *PathPre,ULONG Flags)
{
	ULONG	err;

	strncpy(ClipPathName,PathPre,CLIP_PATH_MAX);
	strncat(ClipPathName,cl->Name,CLIP_PATH_MAX);
	DUMPSTR("Subclip name is ");
	DUMPMSG(ClipPathName);

//	DUMPHEXIL("DHD_AddClip( ",(LONG)cl,") \\ ");
	ESparams4.Data1 =(LONG)ClipPathName;
	ESparams4.Data2 =(LONG)cl->in<<1;
	ESparams4.Data3 =(LONG) (cl->out - cl->in + 2)<<1;
	ESparams4.Data4 =(LONG)Flags;
	err = (ULONG)SendSwitcherReply(ES_AddClipCut,&ESparams4);

	if (err)
	{
		DUMPHEXIL("AddClip failed!  Error = ",err,"\\");
	}

	return(err);
}


// Quiets audio looping after a jog/shuttle
void	DHD_Quiet(ULONG frame)
{
	DUMPMSG("Quiet");

	ESparams2.Data1=NULL;		// FG=NULL for "quiet"
	ESparams2.Data2=Frms2Flds(frame);			// Frame #
	SendSwitcherReply(ES_Jump,&ESparams2);		// Mute audio looping
}


/************** NEED TO PHASE THIS DINOSAUR OUT! *****************/
struct ClipInfo *DHD_ClipInfo(char *name)
{
	struct ClipInfo *ci=NULL;
	if(*name)
	{
		DUMPSTR("FlyerClipInfo( ");
		ESparams2.Data1=(LONG)name;
		DUMPSTR((char *)ESparams2.Data1);
		ESparams2.Data2=NULL;
		ci=(struct ClipInfo *)SendSwitcherReply(ES_FlyerClipInfo,&ESparams2);
		DUMPHEXIL(" ) = ",(LONG)ci,"\\");
		DUMPUDECL("ci->Fields =  ",(LONG)ci->Fields,"\\");
		DUMPBITSL("ci->Flags =  ",(LONG)ci->Flags,"\\");
		return(ci);
	}
	return(NULL);
}


struct ClipInfo *Flyer_ClipInfo(char *name)
{
	struct FlyerVolume	volume;
	ULONG	res;
	static struct ClipInfo	MyClipInfo;
	struct ClipInfo *ci=NULL;

	DUMPSTR("Flyer_ClipInfo(");
	DUMPSTR(name);
	DUMPMSG(")");

	if ((FlyerBase) && (*name))
	{
		volume.Board = 0;
		volume.Flags = 0;			// Not FVF_USENUMS, since we don't know drive numbers !
		volume.SCSIdrive = 0;	// We let Flyer worry about silly things like drive #'s
		volume.Path = name;		// Name of Flyer clip/file

		MyClipInfo.len = sizeof(struct ClipInfo);	// MUST be done!
		res = GetClipInfo(&volume,&MyClipInfo);		// Fill in structure
		if (res == FERR_OKAY)
		{
			ci = &MyClipInfo;

			DUMPSTR("FlyerClipInfo: ");
			DUMPUDECL("Fields =  ",(LONG)ci->Fields," ");
			DUMPBITSL("Flags =  ",(LONG)ci->Flags,"\\");
		}
	}

	return(ci);
}


struct ClipAction	RecAction;
struct FlyerVolume RecVolume;

ULONG	Flyer_Record(char *name, ULONG RecFields)
{
	ULONG	flags,res=FERR_NOCARD;
	struct ClipAction	*action;
	struct FlyerVolume *volume;

	DUMPSTR("Flyer_Record( ");
	DUMPSTR(name);
	DUMPSTR(" )");

/* DEH 7/18/96 See Video while recording audio
	if(CurCompMode==AUDIO_COMP_MODE)
	{
		ESparams1.Data1 = 0;
		SendSwitcherReply(ES_RecordSource,&ESparams1);
	}
*/

// Old ugly ES method (yuck!)
//	ESparams5.Data1=(LONG)name;
//	ESparams5.Data2=(LONG)RecFields;
//	ESparams5.Data3=NULL;		// Start func
//	ESparams5.Data4=NULL;		// Stop func
//	ESparams5.Data5=(LONG)0;	// Flags (unused)
//	return((ULONG)SendSwitcherReply(ES_Record,&ESparams5));

	if (FlyerBase)
	{
		AbortAction(0);		// Kill everything that might be occurring

		memset(&RecAction,0,sizeof(struct ClipAction));		// Clear struct

		action = &RecAction;
		volume = &RecVolume;

		volume->Board = 0;
		volume->Flags = 0;			// Not FVF_USENUMS, since we don't know drive numbers !
		volume->SCSIdrive = 0;		// We let Flyer worry about silly things like drive #'s
		volume->Path = name;			// Name of file to record

		action->Volume = volume;
		action->Channel = 0;

		flags = 0;
		if (CurCompMode!=AUDIO_COMP_MODE)
			flags |= CAF_VIDEO;
		if (global_LeftGain>0)
			flags |= CAF_AUDIOL;
		if (global_RightGain>0)
			flags |= CAF_AUDIOR;
		action->Flags = flags;

		action->VidFieldCount = action->AudFieldCount = RecFields;
		action->ReturnTime = RT_IMMED;	// Go async

		if (action->Flags & (CAF_VIDEO | CAF_AUDIOL | CAF_AUDIOR))		// Skip if nada to do!
			res = FlyerRecord(action);		// Start recording...
		else
			res = FERR_BADPARAM;
	}

	return(res);
}


ULONG	Flyer_Pause(ULONG PauseOn)
{
	ULONG	res=FERR_NOCARD;

	DUMPUDECL("DHD_Pause( ",PauseOn,"); \\ ");

//Old, grungy ES method
//	ESparams1.Data1 =(LONG) PauseOn;
//	SendSwitcherReply(ES_Pause,&ESparams1);

	if (FlyerBase)
	{
		res = CheckAction(&RecAction);
		if (res == FERR_BUSY)									// Do not do if stopped on its own
			res = PauseAction(&RecAction,PauseOn);
	}

	return(res);
}


ULONG Flyer_CheckRecord(void)
{
	ULONG	res=FERR_NOCARD;

//Old, grungy ES method
//	return(	SendSwitcherReply(ES_CheckRecord,NULL) );

	if (FlyerBase)
		res = CheckAction(&RecAction);

	return(res);
}


ULONG Flyer_AbortRecord(void)
{
	ULONG	res=FERR_NOCARD;

	DUMPUDECL("DHD_Abort( ",Unit,"); \\ ");

//	SendSwitcherReply(ES_Stop,NULL);

	if (FlyerBase)
		res = AbortAction(&RecAction);

	return(res);
}

// end of Flyer.c
