/*********************************************************************\
*
* Flyer FileSystem - Interfaces AmigaDOS to the Flyer card(s)
*							Module 1 - AmigaDOS packet interface
*
* $Id: flyerfs1.c,v 1.5 1997/04/18 16:54:16 Holt Exp $
*
* $Log: flyerfs1.c,v $
*Revision 1.5  1997/04/18  16:54:16  Holt
*changed to use NewFileWrite.
*
*Revision 1.4  1997/04/12  15:12:34  Holt
*now using filehsread
*
*Revision 1.3  1997/03/26  16:40:28  Holt
*no real changes to speak of.
*
*Revision 1.2  1997/01/31  16:49:44  Hayes
*changed seek_pkt to handle 64-bit file seeks
*
*Revision 1.1  1995/08/11  13:08:33  Flick
*Big cleanup!  Removed all globals, Eliminated Dispatch()/Abort() system for
*a more traditional method of calling packet handlers.  No longer uses temp
*string buffers on stack to turn BSTR's into CSTR's, BSTRtoCSTR now allocates
*a temp buffer.  This is MUCH safer for big strings (e.g. lengthy paths), we
*definitely crashed on a path > 34 on several packets (rename, for ex.).
*Many other changes too numerous to remember.  Diff it yourself.
*
*Revision 1.0  1995/05/05  15:49:33  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/22/94		Marty	created
*	06/12/94		Marty	Fixed DiskInfoSub so it wouldn't set id_DiskState
*							to ID_WRITE_PROTECTED on unformatted drives
\*********************************************************************/
// ToDo:
//
// Fix/test FH_FROM_LOCK (doesn't look like it works)
// Test requesters again

#include "exec/types.h"
#include "exec/io.h"
#include "exec/memory.h"
#include "exec/execbase.h"
#include "dos/dos.h"
#include "dos/dosextens.h"
#include "workbench/workbench.h"
#include "devices/timer.h"
#include "proto/exec.h"
#include "proto/dos.h"
#include "string.h"
#include "stdlib.h"

#include "flyerlib.h"
#include "flyer.h"
#include "FlyerFS.h"

#include "FlyerFS1.ps"
#include "FlyerFS1.p"
#include "FlyerFS2.p"
#include "FlyerFS3.p"

#define	DEBUGGING	0

#if DEBUGGING
#define	DBUG(x)	x
void kprintf(char *, ... );
#else
#define	DBUG(x)	/* nada */
#endif


void __stdargs __main(char *unused)
{
	register struct 	FSbase *FS;
	register	PktFunc	func;			// Pointer to packet handler function
				ULONG		diskID;
				UWORD		action;
				struct	Message	*msg;

	FS = AllocMem(sizeof(struct FSbase),MEMF_PUBLIC|MEMF_CLEAR);
	if (!FS)
		Suicide(FS);

	if ((OpenFlyer(FS))==0)		// Flyer software not installed?
	{
		DBUG(kprintf("No Flyer!  Idling...\n");)
		FS->FlyerState = FLYST_NONE;		// No Flyer, go dead (don't keep checking)
		FS->ID = ID_NO_DISK_PRESENT;
//		Suicide(FS);
	}
	else if (FlyerRunning(0) != FERR_OKAY)
	{
		DBUG(kprintf("Flyer not up yet!  Waiting...\n");)
		FS->FlyerState = FLYST_WAIT;		// Check back periodically
		FS->ID = ID_NO_DISK_PRESENT;
	}
	else
		FS->FlyerState = FLYST_OKAY;

	FS->myProcess = (struct Process *)FindTask(0);
	FS->myMsgPort = &FS->myProcess->pr_MsgPort;

	// Wait for startup message
	WaitPort(FS->myMsgPort);
	msg = GetMsg(FS->myMsgPort);
	FS->DosPkt = (struct DosPacket *)msg->mn_Node.ln_Name;

	GetMuchoStuffOnDrive(FS);

	DBUG(kprintf("ID:%lx\n",FS->ID);)

	diskID = PublicID(FS);
	ReturnArgs(FS,diskID,FS->myVolume);

	if (diskID == NULL)
		Suicide(FS);						// Quit if major failure on init

	RingMeBack(FS,1);						// Start sending myself timer events

	for (;;)
	{
//		DBUG(kprintf("(wait)");)

		// Wait for message packets from DOS
		Wait(1<<FS->myProcess->pr_MsgPort.mp_SigBit);
		while (msg = GetMsg(FS->myMsgPort))
		{
			DBUG(kprintf("<p>");)

			FS->DosPkt = (struct DosPacket *)msg->mn_Node.ln_Name;
			if ((FS->DosPkt)&&(FS->DosPkt->dp_Link == msg))	// Verify true Packet
			{
				action = (UWORD) FS->DosPkt->dp_Action;		// Extract action code

				func = PacketFunction(action);					// Lookup handler function for it
				if (func)
				{
					func(FS);											// Perform handler for packet
				}
				else
				{
					DBUG(kprintf("\n*** UNKNOWN PACKET %ld ***\n",action);)

					ReturnArgs(FS,DOSFALSE,ERROR_ACTION_NOT_KNOWN);
				}
			}
			else
			{
				DBUG(kprintf("Bogus msg received!\n");)
			}
		}
	}
}


void Suicide(struct FSbase *FS)
{
	DBUG(kprintf("Suicide!\n");)

	CloseFlyer(FS);
	if (FS)
		FreeMem(FS,sizeof(struct FSbase));

	__exit(10);		// Quit FileSystem -- dealloc and RemTask myself
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Packet Handlers Begin ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void ExamineObject_pkt(struct FSbase *FS)
{
	register	struct	DosPacket		*dp = FS->DosPkt;
	struct	FileInfoBlock	*fib;
	BPTR		lock;

	lock = (BPTR)dp->dp_Arg1;
	fib = (struct FileInfoBlock *)BADDR(dp->dp_Arg2);

	DBUG(kprintf("\n*** ExamineObject at lock %lx (FIB %lx)",lock,fib);)

	if (!ExamineLock(FS,lock,fib))
		ReturnError(FS);						// Error code set for us
	else
	{
		FS->DOSerror = 0;
		ReturnArgs(FS,DOSTRUE,0);			// Okay!
	}
}


void ExamineFH_pkt(struct FSbase *FS)
{
	register	struct	DosPacket		*dp = FS->DosPkt;
	register struct	FileOpen			*fo;
	register	struct	FileInfoBlock	*fib;

	DBUG(kprintf("\n*** ExamineFH (FO:%lx  FIB %lx)",dp->dp_Arg1,dp->dp_Arg2);)

	fo = (struct FileOpen *)dp->dp_Arg1;	// Extract 'fo' ptr
	if (fo==0)
		ReturnNewError(FS,ERROR_OBJECT_NOT_FOUND);
	else
	{
		fib = (struct FileInfoBlock *)BADDR(dp->dp_Arg2);

		if (ExamineLock(FS,fo->Lock,fib)==FALSE)
			ReturnError(FS);						// Error code set for us
		else
		{
			FS->DOSerror = 0;
			ReturnArgs(FS,DOSTRUE,0);			// Okay!
		}
	}
}


BOOL ExamineLock(struct FSbase *FS, BPTR lock, struct FileInfoBlock *fib)
{
	ULONG	grip,block;

	if (GetLocksInfo(FS,lock,&grip,&block)==FALSE)
		return(FALSE);										// Various errors set for us

	if (!FillInInfo(FS,grip,TRUE,fib))				// Try to fill in FIB
	{
		FS->DOSerror = ERROR_OBJECT_NOT_FOUND;		// Set error code
		return(FALSE);
	}

	return(TRUE);
}


void ExamineNext_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	struct	FileInfoBlock	*fib;
	ULONG		grip,block;
	BPTR		lock;

	DBUG(kprintf("\n*** ExamineNext %lx %lx",dp->dp_Arg1,dp->dp_Arg2);)

	lock = (BPTR)dp->dp_Arg1;
	fib = (struct FileInfoBlock *)BADDR(dp->dp_Arg2);

	if (GetLocksInfo(FS,lock,&grip,&block)==FALSE)
		ReturnError(FS);			// Various errors set for us
	else
	{
		if (!FillInInfo(FS,grip,FALSE,fib))
			ReturnNewError(FS,ERROR_NO_MORE_ENTRIES);
		else
		{
			FS->DOSerror = 0;
			ReturnArgs(FS,DOSTRUE,0);			// Okay!
		}
	}
}


BOOL FillInInfo(struct FSbase *FS,ULONG grip,BOOL firstflag,struct FileInfoBlock *fib)
{
	struct	ClipInfo	ThisInfo;
	ULONG		err;

	if (FS->FlyerState != FLYST_OKAY)			// Flyer not ready yet?
		return(FALSE);

	FS->FlyVol.Path = 0;

	// Get info from Flyer on specified grip
	ThisInfo.len = sizeof(struct ClipInfo);			// IMPORTANT!!
	// Ask for FileSystem-related stuff only (goes faster)
	err = FlyerDirList(&FS->FlyVol,grip,&ThisInfo,firstflag?1:0,1);
	if (err)
		return(FALSE);

	fib->fib_Date = ThisInfo.Date;		// Copy date

	strcpy(fib->fib_FileName+1,ThisInfo.Name);
	fib->fib_FileName[0] = strlen(fib->fib_FileName+1);	// to BSTR

	DBUG(kprintf("Name from Flyer: %ls\n",ThisInfo.Name);)

	if (ThisInfo.Type == FLYER_TYPE_FILE)				// File?
	{
		fib->fib_DirEntryType = fib->fib_EntryType = -3;
		fib->fib_Size = ThisInfo.Length;
		fib->fib_NumBlocks = (ThisInfo.Length+FS->FSinfo.BlkSize-1)>>FS->LogBlkShift;
	}
	else
	{
		fib->fib_DirEntryType = fib->fib_EntryType = 2;
		fib->fib_Size = 0;
		fib->fib_NumBlocks = 0;
	}

	strcpy(fib->fib_Comment+1,ThisInfo.Comment);
	fib->fib_Comment[0] = strlen(fib->fib_Comment+1);		// to BSTR

	fib->fib_Protection = ThisInfo.Bits;		// User-defined

//	if (grip==0)
//	{
//		strncpy(fib->fib_FileName,&FS->FSinfo.LTitle,VOLNAMEMAXSIZE);
//
//		DBUG(kprintf("Copied volume name: %ls\n",FS->FSinfo.Title);)
//	}

	DBUG(
		kprintf("\nEntryType:%ld  NumBlocks:%ld  Size:%ld  Prot:%lx",
			fib->fib_EntryType,fib->fib_NumBlocks,fib->fib_Size,fib->fib_Protection);
		kprintf(" Comment:");
		BSTR2Terminal(fib->fib_Comment);
		kprintf("\n  Filename:");
		BSTR2Terminal(fib->fib_FileName);
		kprintf("\n--Done");
	)

	return(TRUE);
}


void LocateObject_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	register	BSTR		fullpath;
	register	BPTR		lock;
	register	int		access;

	lock = (BPTR)dp->dp_Arg1;
	fullpath = dp->dp_Arg2;
	access = dp->dp_Arg3;

	DBUG(
		kprintf("\n*** LocateObject DadLock:%lx Nam:",lock);
		BSTR2Terminal((char *)BADDR(fullpath));
		kprintf(" Mode:%ld",access);
	)

	lock = Locate(FS,lock,fullpath,access);
	if (lock == 0)
		FS->DOSerror = ERROR_OBJECT_NOT_FOUND;

	ReturnArgs(FS,lock,FS->DOSerror);
}


BPTR Locate(struct FSbase *FS,BPTR lock,BSTR fullpath,int access)
{
	ULONG		grip,newgrip,block,err;
	char		*cname;

	cname = BSTRtoCSTR(fullpath);
	if (cname==NULL)
		return(0);

	if (GetLocksInfo(FS,lock,&grip,&block)==FALSE)
	{
		FreeVec(cname);				// Free string, if any
		return(0);
	}

	lock = NULL;						// Default, in case anything fails

	FS->ClipAct.Grip = grip;
	FS->ClipAct.Access = access;
	FS->ClipAct.ReturnTime = RT_STOPPED;			// Synchronous
	FS->FlyVol.Path = cname;

	err = FlyerLocate(&FS->ClipAct);
	if (err==FERR_OKAY)
	{
		newgrip = FS->ClipAct.Grip;
		block = FS->ClipAct.StartBlk;

		if (newgrip)
		{
			lock = RequestLock(FS,block,access,newgrip);

			DBUG(kprintf("\nMade lock %lx: Block:%ld  Grip:%lx",lock,block,newgrip);)
		}
	}
	FreeVec(cname);				// Free string, if any
	return(lock);
}


void FreeLock_pkt(struct FSbase *FS)
{
	LONG	stat;

	DBUG(kprintf("\n*** FreeLock %lx",FS->DosPkt->dp_Arg1);)

	stat = RemoveLock(FS,FS->DosPkt->dp_Arg1);
	ReturnArgs(FS,stat,FS->DOSerror);
}


void Parent_pkt(struct FSbase *FS)
{
	register	BPTR		lock;
				ULONG		block,grip;

	lock = (BPTR)FS->DosPkt->dp_Arg1;

	DBUG(kprintf("\n*** Parent %lx",lock);)

	if (GetLocksInfo(FS,lock,&grip,&block)==FALSE)
		ReturnError(FS);					// Sets various error codes for us
	else
	{
		lock = GetParent(FS,lock);

		ReturnArgs(FS,lock,FS->DOSerror);
	}
}


void ParentFH_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	register	struct	FileOpen		*fo;
	register	BPTR		lock;
				ULONG		grip,block;

	DBUG(kprintf("\n*** ParentFH %lx",dp->dp_Arg1);)

	fo = (struct FileOpen *)dp->dp_Arg1;

	if ((fo->Volume)&&(fo->Volume != FS->myVolume))
		ReturnArgs(FS,DOSFALSE,ERROR_DEVICE_NOT_MOUNTED);
	else
	{
		if (GetLocksInfo(FS,fo->Lock,&grip,&block)==FALSE)
			ReturnError(FS);
		else
		{
			lock = GetParent(FS,fo->Lock);

			ReturnArgs(FS,lock,FS->DOSerror);
		}
	}
}


void CurrentVolume_pkt(struct FSbase *FS)
{
	register struct	FileOpen		*fo;
	register struct	DosPacket	*dp = FS->DosPkt;

	DBUG(kprintf("\n*** CurrentVolume %lx",dp->dp_Arg1);)

	if (dp->dp_Arg1)
	{
		fo = (struct FileOpen *)dp->dp_Arg1;
		ReturnArgs(FS,fo->Volume,FS->FlyVol.SCSIdrive);
	}
	else
		ReturnArgs(FS,FS->myVolume,FS->FlyVol.SCSIdrive);
}


/* This routine called once every time period (3 seconds) */
void Timer_pkt(struct FSbase *FS)
{
	DBUG(kprintf("-3-");)

	if (FS->FlyerState == FLYST_WAIT)					// Keep trying to open Flyer?
	{
		if (FlyerRunning(0) == FERR_OKAY)				// Flyer running now?
		{
			FS->FlyerState = FLYST_OKAY;
			DBUG(kprintf("Flyer awakened!\n");)

			FS->FlyVol.Path = 0;

			/* Let flyer.library mount a volume for drive, it it can find one */
			FS->FSinfo.len = sizeof(struct FlyerVolInfo);
			FlyerDriveInfo(&FS->FlyVol,&FS->FSinfo);
		}
	}

	if (FS->FlyerState == FLYST_OKAY)
		DiskChanges(FS);

	RingMeBack(FS,3);				// Send myself another wake-up call in 3 seconds
}


void Flush_pkt(struct FSbase *FS)
{
	DBUG(kprintf("\n*** Flush");)

	// Not currently buffering any write data

	ReturnArgs(FS,DOSTRUE,0);			// Okay!
//	ReplyToDOS(FS);
}


void CopyDir_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	register	BPTR		lock;

	DBUG(kprintf("\n*** CopyDir %lx",dp->dp_Arg1);)

	lock = DuplicateLock(FS,dp->dp_Arg1);
	ReturnArgs(FS,lock,FS->DOSerror);
}


void CopyDirFH_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	register	struct	FileOpen		*fo;
	register	BPTR		lock;

	DBUG(kprintf("\n*** CopyDirFH %lx",dp->dp_Arg1);)

	fo = (struct FileOpen *)dp->dp_Arg1;

	if ((fo->Volume)&&(fo->Volume != FS->myVolume))
		ReturnArgs(FS,DOSFALSE,ERROR_DEVICE_NOT_MOUNTED);
	else
	{
		lock = DuplicateLock(FS,fo->Lock);
		ReturnArgs(FS,lock,FS->DOSerror);
	}
}


// Question: should Info prepare an InfoData structure for a lock that is on a
//	removed volume?  Docs say yes, but FFS returns ERROR_DEVICE_NOT_MOUNTED

void Info_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
				ULONG		blk;
				ULONG		grip;

	DBUG(kprintf("\n*** Info %lx",dp->dp_Arg1);)

	if (GetLocksInfo(FS,dp->dp_Arg1,&grip,&blk)==FALSE)
		ReturnArgs(FS,DOSFALSE,FS->DOSerror);
	else
		DiskInfoSub(FS,(BPTR)dp->dp_Arg2);
}


void DiskInfo_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;

	DBUG(kprintf("\n*** DiskInfo %lx",dp->dp_Arg1);)

	DiskInfoSub(FS,(BPTR)dp->dp_Arg1);
}


void DiskInfoSub(struct FSbase *FS,BPTR info)
{
	register	struct	InfoData	*id;
	UBYTE	protflag;
	int	error;

	FS->FlyVol.Path = 0;

//	FlyerWriteProt(&FS->FlyVol,0,0,&protflag);

	if (FS->FlyerState != FLYST_OKAY)						// Flyer not yet open?
		error = 0;
	else
	{
		FS->FSinfo.len = sizeof(struct FlyerVolInfo);
		error = FlyerDriveInfo(&FS->FlyVol,&FS->FSinfo);
	}

	if ((error==0) || (error==FERR_SELTIMEOUT))
	{
		id = (struct InfoData *)BADDR(info);
		id->id_UnitNumber = FS->FlyVol.SCSIdrive;
		id->id_NumSoftErrors = FS->SoftErrCnt;
		id->id_NumBlocks = FS->FSinfo.Blocks;	/* Total size */
		id->id_NumBlocksUsed = FS->FSinfo.Blocks - FS->FSinfo.BlksFree; /* used */
		id->id_BytesPerBlock = FS->FSinfo.BlkSize;	/* BLOCKSIZE */
		id->id_VolumeNode = FS->myVolume;
		id->id_InUse = FS->MasterLockList;
		id->id_DiskType = PublicID(FS);		/* Return public version of ID type */

		/* Scale down sizes by 1024 to keep from breaking AmigaDOS math */
//		id->id_NumBlocks >>= 10;
//		id->id_NumBlocksUsed >>= 10;

		DBUG(kprintf(" NumBlocks = %lx\n",id->id_NumBlocks);)
		DBUG(kprintf(" NumBlocksUsed = %lx\n",id->id_NumBlocksUsed);)
	
		DBUG(kprintf("ID:%lx\n",id->id_DiskType);)

		id->id_DiskState = ID_VALIDATED;

		protflag = FS->FSinfo.Flags & FVIF_WRITEPROT;
		if ((FS->ID != ID_NOT_REALLY_DOS) && (protflag))
			id->id_DiskState = ID_WRITE_PROTECTED;

		// This conversion is awful, but must be done on a pre-2.0 system
		// Convert block figures for software that assumes 512 bytes/block
		if (FS->OSversion < 37)
		{
			id->id_NumBlocks <<= (FS->LogBlkShift-9);		/* Total blocks */
			id->id_NumBlocksUsed <<= (FS->LogBlkShift-9);	/* All blocks used */
			id->id_BytesPerBlock = 512;					/* BLOCKSIZE */
		}
	}
	ReturnArgs(FS,DOSTRUE,0);
}


void MoreCache_pkt(struct FSbase *FS)
{
	register	LONG		num;
	register	struct	DosPacket	*dp = FS->DosPkt;

	DBUG(kprintf("\n*** MoreCache %ld",dp->dp_Arg1);)

	num = dp->dp_Arg1;

	ReturnArgs(FS,1,0);		/* Always just 1 buffer (not really) */
}


void Inhibit_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;

	DBUG(kprintf("\n*** Inhibit %ld",dp->dp_Arg1);)

	if (dp->dp_Arg1)
	{
		UndoVolume(FS);
		FS->ID = ID_DISK_BUSY;
		FS->Inhibited = 1;
	}
	else
	{
		FS->Inhibited = 0;
		if (FS->ID == ID_DISK_BUSY)
			GetVolumeInfo(FS);
	}
	ReturnArgs(FS,DOSTRUE,0);
}


void ProtLocker_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	int	prot,err;

	prot = dp->dp_Arg1;

	DBUG(kprintf("\n*** ProtLocker %ld",prot);)

	FS->FlyVol.Path = 0;
	err = FlyerWriteProt(&FS->FlyVol,1,prot,0);
	ReturnFlyerError(FS,err);
}


void RenameDisk_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	char	*volname;
	int	err,i;
	char	*mem,*name,*temp;
	BYTE	*old;
	struct	DeviceList	*vol;

	volname = BSTRtoCSTR(dp->dp_Arg1);
	if (volname==NULL)
		ReturnNewError(FS,ERROR_NO_FREE_STORE);
	else
	{
		name = (char *)BADDR(dp->dp_Arg1);

		DBUG(kprintf("\n*** RenameDisk '%ls'",volname);)

//		if (SetStatusError()) Return???Error();

		if (CheckName(FS,name)==FALSE)
			ReturnError(FS);
		else
		{
			mem = (char *)AllocMem(*name + 2,MEMF_PUBLIC|MEMF_CLEAR);
			if (mem == NULL)
				ReturnNewError(FS,ERROR_NO_FREE_STORE);
			else
			{
				temp = mem;

				for (i=*name+1;i;i--)
					*temp++ = *name++;            // make own copy of name

				vol = (struct DeviceList *)BADDR(FS->myVolume);
				old = (BYTE *)BADDR(vol->dl_Name);
				FreeMem(old,*old+2);					// throw away old name
				vol->dl_Name = (BSTR)MKBADDR(mem);

				FS->FlyVol.Path = 0;
				err = FlyerRenameDisk(&FS->FlyVol,volname);
				ReturnFlyerError(FS,err);
			}
		}
		FreeVec(volname);					// Free temp string, if any
	}
}


void SetProtect_pkt(struct FSbase *FS)
{
	register	BSTR		fullpath;
	register	struct	DosPacket	*dp = FS->DosPkt;
	register	BPTR		lock;
	LONG	prot;
	ULONG	grip;
	int	err;
	BPTR	templock;
//	char	*cname;

	lock = (BPTR)dp->dp_Arg2;
	fullpath = dp->dp_Arg3;
	prot = dp->dp_Arg4;

//	cname = BSTRtoCSTR(fullpath);

	DBUG(
	kprintf("\n*** SetProt  Lock:%lx ",lock);
	BSTR2Terminal((char *)BADDR(fullpath));
	kprintf(" Bits:%lx",prot);
	)

	templock = Locate(FS,lock,fullpath,SHARED_LOCK);
	if (templock)
	{
		grip = Lock2Grip(templock);
		FS->FlyVol.Path = 0;
		err = FlyerSetBits(&FS->FlyVol,grip,prot);
		RemoveLock(FS,templock);
		ReturnFlyerError(FS,err);
	}
	else
		ReturnArgs(FS,DOSFALSE,ERROR_OBJECT_NOT_FOUND);
}


void SetComment_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	register	BSTR		fullpath;
	register	BPTR		lock;
	ULONG	grip;
	int	err;
	BPTR	templock;
	char	*cname;

	lock = (BPTR)dp->dp_Arg2;
	fullpath = dp->dp_Arg3;
	cname = BSTRtoCSTR(dp->dp_Arg4);
	if (cname==NULL)
		ReturnNewError(FS,ERROR_NO_FREE_STORE);
	else
	{
		DBUG(
			kprintf("\n*** SetComment  Lock:%lx ",lock);
			BSTR2Terminal((char *)BADDR(fullpath));
			kprintf(" Comment:%ls",cname);
		)

		templock = Locate(FS,lock,fullpath,SHARED_LOCK);
		if (templock)
		{
			grip = Lock2Grip(templock);
			FS->FlyVol.Path = 0;
			err = FlyerSetComment(&FS->FlyVol,grip,cname);
			RemoveLock(FS,templock);
			ReturnFlyerError(FS,err);
		}
		else
			ReturnArgs(FS,DOSFALSE,ERROR_OBJECT_NOT_FOUND);

		FreeVec(cname);					// Free string, if any
	}
}


void SetDate_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	register	BPTR		lock;
	BSTR		fullpath;
	ULONG	grip;
	int	err;
	struct DateStamp *date;
	BPTR	templock;
	char	*cname;

	lock = (BPTR)dp->dp_Arg2;
	fullpath = dp->dp_Arg3;
	date = (struct DateStamp *)dp->dp_Arg4;

	cname = BSTRtoCSTR(fullpath);
	if (cname==NULL)
		ReturnNewError(FS,ERROR_NO_FREE_STORE);
	else
	{
		DBUG(kprintf("\n*** SetDate  Lock:%lx Name:%ls Date:%lx",lock,cname,date);)
		DBUG(kprintf("\n    Days:%lx Mins:%lx Ticks%lx",
				date->ds_Days,date->ds_Minute,date->ds_Tick);)

		templock = Locate(FS,lock,fullpath,SHARED_LOCK);
		if (templock)
		{
			grip = Lock2Grip(templock);
			FS->FlyVol.Path = 0;
			err = FlyerSetDate(&FS->FlyVol,grip,date->ds_Days,date->ds_Minute,date->ds_Tick);
			RemoveLock(FS,templock);
			ReturnFlyerError(FS,err);
		}
		else
			ReturnArgs(FS,DOSFALSE,ERROR_OBJECT_NOT_FOUND);

		FreeVec(cname);						// Free string, if any
	}
}


void DeleteObject_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	BPTR	lock;
	ULONG	grip,err;
	char	*cname;

	lock = (BPTR)dp->dp_Arg1;
	grip = Lock2Grip(lock);
	cname = BSTRtoCSTR(dp->dp_Arg2);
	if (cname==NULL)
		ReturnNewError(FS,ERROR_NO_FREE_STORE);
	else
	{
		DBUG(kprintf("\n*** DeleteObject  Lock:%lx  Grip:%lx  Name:%ls",lock,grip,cname);)

		FS->ClipAct.Grip = grip;
		FS->ClipAct.ReturnTime = RT_STOPPED;			// Synchronous
		FS->FlyVol.Path = cname;

		err = FlyerDelete(&FS->ClipAct);
		ReturnFlyerError(FS,err);

		FreeVec(cname);								// Free string, if any
	}
}


void RenameObject_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	BPTR	lock1,lock2;
	ULONG	grip1,grip2,err;
	char	*cname1,*cname2;

	lock1 = (BPTR)dp->dp_Arg1;
	grip1 = Lock2Grip(lock1);
	cname1 = BSTRtoCSTR(dp->dp_Arg2);

	lock2 = (BPTR)dp->dp_Arg3;
	grip2 = Lock2Grip(lock2);
	cname2 = BSTRtoCSTR(dp->dp_Arg4);

	if ((cname1==NULL) || (cname2==NULL))
		ReturnNewError(FS,ERROR_NO_FREE_STORE);
	else
	{
		DBUG(kprintf("\n*** RenameObject  Lock:%lx Grip:%lx Name:%ls ---> Lock:%lx Grip:%lx Name:%ls)",
			lock1,grip1,cname1,lock2,grip2,cname2);)

		FS->ClipAct.Grip = grip1;
		FS->ClipAct.ReturnTime = RT_STOPPED;			// Synchronous
		FS->FlyVol.Path = cname1;

		err = FlyerRename(&FS->ClipAct,grip2,cname2);
		ReturnFlyerError(FS,err);
	}

	FreeVec(cname1);										// Free strings, if any
	FreeVec(cname2);
}


void CreateDir_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	BPTR	lock;
	ULONG	grip,block,err;
	char	*cname;

	lock = (BPTR)dp->dp_Arg1;
	grip = Lock2Grip(lock);
	cname = BSTRtoCSTR(dp->dp_Arg2);
	if (cname==NULL)
		ReturnNewError(FS,ERROR_NO_FREE_STORE);
	else
	{
		DBUG(kprintf("\n*** CreateDir  Lock:%lx Grip:%lx Name:%ls",lock,grip,cname);)

		FS->ClipAct.Grip = grip;
		FS->ClipAct.ReturnTime = RT_STOPPED;			// Synchronous
		FS->FlyVol.Path = cname;

		err = FlyerCreateDir(&FS->ClipAct);
		if (!err)
		{
			block = FS->ClipAct.StartBlk;
			lock = RequestLock(FS,block,SHARED_LOCK,FS->ClipAct.Grip);
			ReturnArgs(FS,lock,0);
		}
		else
			ReturnNewError(FS,Flyer2AmigaErr(err));

		FreeVec(cname);						// Free string, if any
	}
}


void Truncate_pkt(struct FSbase *FS)
{
	DBUG(kprintf("\n*** Truncate");)

	ReturnArgs(FS,DOSFALSE,ERROR_DISK_WRITE_PROTECTED);
}


/****** New 2.0 packets *******/

void  IsFileSystem_pkt(struct FSbase *FS)
{
	DBUG(kprintf("\n*** IsFileSystem");)

	ReturnArgs(FS,DOSTRUE,0);		// Yes, I am a filesystem
}


// DON'T LISTEN TO THE DOCS!!! This version responds just like the 2.0 FFS
void SameLock_pkt(struct FSbase *FS)
{
	register struct	LongLock		*l1,*l2;
	register struct	DosPacket	*dp = FS->DosPkt;

	l1 = (struct LongLock *)BADDR(dp->dp_Arg1);
	l2 = (struct LongLock *)BADDR(dp->dp_Arg2);

	DBUG(
	kprintf("\n*** SameLock %lx %lx",dp->dp_Arg1,dp->dp_Arg2);
	kprintf("Lock1: key %lx, volume %lx\n",
		l1->ll_Key,l1->ll_Volume);
	kprintf("Lock2: key %lx, volume %lx\n",
		l2->ll_Key,l2->ll_Volume);
	)

	if (l1 == l2)			/* Same exact address? */
		ReturnArgs(FS,DOSTRUE,0);
	else if (l1->ll_Volume == l2->ll_Volume)
	{
		if (l1->ll_Key == l2->ll_Key)
			ReturnArgs(FS,DOSTRUE,0);		/* Same locked objects */
		else
			ReturnArgs(FS,DOSFALSE,0);		/* Different objects on same volume */
	}
	else
		ReturnArgs(FS,DOSFALSE,ERROR_DEVICE_NOT_MOUNTED);	/* Totally different */
}


void Format_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	struct DateStamp now;
	int	err;
	char	*volname;

	volname = BSTRtoCSTR(dp->dp_Arg1);
	if (volname==NULL)
		ReturnNewError(FS,ERROR_NO_FREE_STORE);
	else
	{
		DBUG(kprintf("\n*** Format '%ls'",volname);)

		GetDateStamp(FS,&now);		/* Get date */

		if (FS->FlyerState != FLYST_OKAY)						/* Flyer not yet open? */
			ReturnNewError(FS,ERROR_DEVICE_NOT_MOUNTED);
		else
		{
			FS->FlyVol.Path = 0;
			err = FlyerFormat(&FS->FlyVol,volname,&now,0,
				FVIF_VIDEOREADY | FVIF_AUDIOREADY);

			ReturnFlyerError(FS,err);
		}

		FreeVec(volname);								// Free string, if any
	}
}


void Notify_pkt(struct FSbase *FS)
{
	DBUG(kprintf("\n*** Add/Remove Notify");)

	ReturnArgs(FS,DOSTRUE,0);		// Fake notification start/stop
}


void MakeLink_pkt(struct FSbase *FS)
{
	DBUG(kprintf("\n*** Make Link");)

	ReturnArgs(FS,DOSFALSE,ERROR_DISK_WRITE_PROTECTED);
}


void Records_pkt(struct FSbase *FS)
{
	DBUG(kprintf("\n*** Lock/Free Record");)

	ReturnArgs(FS,DOSTRUE,0);			// Fake record locking
}


void ChangeMode_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	register	struct	LongLock		*llp;
	register struct	FileOpen		*fo;
				LONG		mode;
				ULONG		grip;
				int		err;
				BPTR		lock;

	if (dp->dp_Arg1==CHANGE_FH)
	{
		fo = (struct FileOpen *)dp->dp_Arg2;	/* Extract 'fo' ptr */
		if (fo==0)
		{
			ReturnNewError(FS,ERROR_OBJECT_NOT_FOUND);
			return;
		}
		lock = fo->Lock;
	}
	else							/* Change lock */
		lock = dp->dp_Arg2;

	mode = dp->dp_Arg3;

	DBUG(kprintf("\n*** Change Mode Type:%ld Object:%lx Mode:%ld",
		dp->dp_Arg1,dp->dp_Arg2,mode);)

	grip = Lock2Grip(lock);
	FS->FlyVol.Path = 0;
	err = FlyerChangeMode(&FS->FlyVol,grip,mode);
	if (!err)
	{
		llp = (struct LongLock *)BADDR(lock);
		llp->ll_Access = mode;
	}
	ReturnFlyerError(FS,err);
}


/****** Open/Close/Read/Write packets ******/

void Open_pkt(struct FSbase *FS)
{
	register	struct	FileOpen		*fo;
	register	struct	DosPacket	*dp = FS->DosPkt;
	struct	FileHandle	*handle;
	BPTR		lock;
	int		mode;
	ULONG		grip,err,block;
	char		*cname;

	DBUG(
		if (dp->dp_Action == ACTION_FH_FROM_LOCK)
			kprintf("*** FH_FROM_LOCK is non functional!!!***\n");
	)

	DBUG(
		if (dp->dp_Action == ACTION_FH_FROM_LOCK)
			kprintf("\n*** Open FH from Lock FH:%lx Lock:%lx",dp->dp_Arg1,dp->dp_Arg2);
		else
		{
			kprintf("\n*** Open FH:%lx Lock:%lx Name:",dp->dp_Arg1,dp->dp_Arg2);
			BSTR2Terminal((char *)BADDR(dp->dp_Arg3));
		}
	)

	fo = (struct FileOpen *)AllocMem(sizeof(struct FileOpen),MEMF_PUBLIC|MEMF_CLEAR);
	if (fo == NULL)
	{
		ReturnNewError(FS,ERROR_NO_FREE_STORE);
		return;
	}

	fo->pad = 0xCD;		// Debugging
	fo->wrote = 0;			// No writes to file yet

	if (dp->dp_Action == ACTION_FINDOUTPUT)
		mode = FLYER_MODE_OUTPUT;
	else if (dp->dp_Action == ACTION_FINDUPDATE)
		mode = FLYER_MODE_UPDATE;
	else
		mode = FLYER_MODE_INPUT;				// FINDINPUT & FH_FROM LOCK

	if (dp->dp_Action == ACTION_FH_FROM_LOCK)
		lock = dp->dp_Arg2;
	else
	{
		lock = dp->dp_Arg2;
// 	lock = Locate(FS,lock,dp->dp_Arg3,FLYER_ACC_SHARED);		// Open takes grip/name
	}

	grip = Lock2Grip(lock);
	cname = BSTRtoCSTR(dp->dp_Arg3);
	if (cname==NULL)
	{
		ReturnNewError(FS,ERROR_NO_FREE_STORE);
		return;
	}

	FS->ClipAct.Grip = grip;
	FS->ClipAct.Access = mode;
	FS->ClipAct.ReturnTime = RT_STOPPED;			// Synchronous
	FS->FlyVol.Path = cname;

	err = FlyerFileOpen(&FS->ClipAct);
	if (err == FERR_WRONGTYPE)					// Trying to open root or a directory maybe?
	{
//		if (dp->dp_Action != ACTION_FH_FROM_LOCK)
//			RemoveLock(FS,lock);
		ReturnNewError(FS,ERROR_OBJECT_WRONG_TYPE);
		// Deallocate structure
	}
	else if (err)
	{
		ReturnArgs(FS,DOSFALSE,Flyer2AmigaErr(err));
		// Deallocate structure
	}
	else
	{
		fo->FileID = FS->ClipAct.FileID;			// (from Flyer)
		block = FS->ClipAct.StartBlk;
//		DBUG(kprintf("open blk %lx\n",block);)

		fo->Volume = FS->myVolume;

		fo->Lock = lock = RequestLock(FS,block,FLYER_ACC_SHARED,fo->FileID);
		if (lock == NULL)
		{
			ReturnNewError(FS,ERROR_OBJECT_NOT_FOUND);
			// Deallocate structure
		}
		else
		{
			handle = (struct FileHandle *)BADDR(dp->dp_Arg1);
			handle->fh_Type = FS->myMsgPort;  	// New: required per docs
			handle->fh_Arg1 = (ULONG)fo;

			DBUG(kprintf("\nResults: Vol:%lx",fo->Volume);)
			DBUG(kprintf("\n(fo=%lx, fh=%lx)",fo,handle);)

			ReturnArgs(FS,DOSTRUE,0);
			FreeVec(cname);								// Free string, if any

			return;						// Leave structure allocated
		}
	}

	// Failure -- throw away 'FileOpen' structure
	if (fo)
		FreeMem((UBYTE *)fo,sizeof(struct FileOpen));

	FreeVec(cname);								// Free string, if any
}


void Close_pkt(struct FSbase *FS)
{
	register	struct	DosPacket	*dp = FS->DosPkt;
	register	struct	FileOpen		*fo;
							int			err;
				struct	DateStamp	now;

	DBUG(kprintf("\n*** Close %lx",dp->dp_Arg1);)

	fo = (struct FileOpen *)dp->dp_Arg1;

	if ((fo->Volume)&&(fo->Volume != FS->myVolume))
	{
		ReturnArgs(FS,DOSFALSE,ERROR_DEVICE_NOT_MOUNTED);
		return;
	}

	FS->FlyVol.Path = 0;

	err = FlyerFileClose(&FS->FlyVol,fo->FileID);
	if (err==FERR_OKAY)
	{
		if (fo->wrote)
		{
			GetDateStamp(FS,&now);		/* Get date */
			FS->FlyVol.Path = 0;
			FlyerSetDate(&FS->FlyVol,Lock2Grip(fo->Lock),
				now.ds_Days,now.ds_Minute,now.ds_Tick);
		}
		ReturnArgs(FS,RemoveLock(FS,fo->Lock),FS->DOSerror);
		FreeMem((UBYTE *)fo,sizeof(struct FileOpen));
	}
	else
		ReturnArgs(FS,DOSFALSE,Flyer2AmigaErr(err));
}


void Seek_pkt(struct FSbase *FS)
{
	register	struct	FileOpen		*fo;
	register	struct	DosPacket	*dp = FS->DosPkt;
	int		mode,err;
	LONG		pos,pos_ext;
	struct DLONG 	newpos,oldpos;

	fo = (struct FileOpen *)dp->dp_Arg1;

	DBUG(kprintf("\n*** Seek %lx %lx %ld\n",dp->dp_Arg1,dp->dp_Arg2,dp->dp_Arg3);)

	if ((fo->Volume)&&(fo->Volume != FS->myVolume))
	{
		ReturnArgs(FS,-1,ERROR_DEVICE_NOT_MOUNTED);
	}
	else
	{
		pos = dp->dp_Arg2;
		if ( pos < 0 )
		{
			pos_ext = -1;
		}
		else
		{
			pos_ext = 0;
		}

		mode = dp->dp_Arg3;
DBUG(kprintf("*** position: %lx;%lx mode: %ld\n",pos_ext,pos,mode);)

		FS->FlyVol.Path = 0;

		err = FlyerFileSeek(&FS->FlyVol,fo->FileID,pos_ext,pos,mode,&newpos,&oldpos);
DBUG(kprintf("*** FlyerFileSeek Result: %lx\n",err);)
DBUG(kprintf("\n*** Seek newpos: %lx;%lx oldpos: %lx;%lx",newpos.high, newpos.low, oldpos.high, oldpos.low);)
		if (err)
		{
			DBUG(kprintf("\n*** SeekError %lx, newpos: %lx;%lx oldpos: %lx;%lx",err,newpos.high, newpos.low, oldpos.high, oldpos.low);)
			ReturnArgs(FS,-1,Flyer2AmigaErr(err));
		}
		else
		{
// 		oldpos.low = fo->Position;
//			fo->Position = pos.low;
			DBUG(kprintf("\n*** Seek success newpos: %lx;%lx oldpos: %lx;%lx",newpos.high, newpos.low, oldpos.high, oldpos.low);)
			ReturnArgs(FS,oldpos.low,0);
		}
	}
}


void RW_pkt(struct FSbase *FS)
{
	register	struct	FileOpen		*fo;
	register	struct	DosPacket	*dp = FS->DosPkt;
				int		wflg,err;
				ULONG		actual,length,left,total;
				UBYTE		*buffer;

	DBUG(kprintf("\n*** RW (%lx) buf:%lx len:%ld",dp->dp_Arg1,dp->dp_Arg2,dp->dp_Arg3);)

	fo = (struct FileOpen *)dp->dp_Arg1;	/* Retrieved from the Open */

	if ((fo->Volume)&&(fo->Volume != FS->myVolume))
	{
		ReturnArgs(FS,-1,ERROR_DEVICE_NOT_MOUNTED);
		return;
	}

	buffer = (UBYTE *)dp->dp_Arg2;
	left = length = dp->dp_Arg3;
	wflg = (dp->dp_Action == ACTION_WRITE)?1:0;

	for (total=0 ; total<length ; )
	{
		if (wflg)
		{
			FS->FlyVol.Path = 0;

			//err = FlyerFileWrite(&FS->FlyVol,fo->FileID,left,buffer,&actual);
			err = FlyerNewFileWrite(&FS->FlyVol,fo->FileID,left,buffer,&actual);
			if ((err==FERR_OKAY)&&(actual>0))
				fo->wrote = 1;					/* Will need a new date on this */
		}
		else
		{
			FS->FlyVol.Path = 0;
			err = FlyerNewFileRead(&FS->FlyVol,fo->FileID,left,buffer,&actual);
			//err = FlyerFileRead(&FS->FlyVol,fo->FileID,left,buffer,&actual);
		}

		if (err)
		{
			ReturnArgs(FS,-1,Flyer2AmigaErr(err));
			return;
		}

		if (actual==0)
			break;

		total += actual;
		buffer += actual;
		left -= actual;

		DBUG(kprintf("(%ld)(+%lx=%lx)",err,actual,total);)
	}
	ReturnArgs(FS,total,0);			/* Maybe should be DOSTRUE if not at EOF */
}


/*****************\
**  Subroutines  **
\*****************/


void ReturnNewError(struct FSbase *FS,LONG err)
{
	FS->DOSerror = err;
	ReturnArgs(FS,DOSFALSE,FS->DOSerror);
}


void ReturnError(struct FSbase *FS)
{
	ReturnArgs(FS,DOSFALSE,FS->DOSerror);
}


void ReturnArgs(struct FSbase *FS, LONG res1, LONG res2)
{
	register	struct	DosPacket	*dp;

	dp = FS->DosPkt;
	if (dp)
	{
		dp->dp_Res1 = res1;
		dp->dp_Res2 = res2;
		ReplyToDOS(FS);
	}
}


void ReplyToDOS(struct FSbase *FS)
{
	register	struct	DosPacket	*dp;
	register	struct	MsgPort		*port;
	register	struct	Message		*msg;

	dp = FS->DosPkt;

	DBUG(kprintf(" [%lx,%lx]",dp->dp_Res1,dp->dp_Res2);)

	msg = (struct Message *)dp->dp_Link;				// get ptr to EXEC message
	if (msg)
	{
		port = dp->dp_Port;									// get ptr to reply port
		dp->dp_Port = FS->myMsgPort;
		msg->mn_Node.ln_Name = (char *)dp;
		if (port)
			PutMsg(port,msg);
	}

	FS->DosPkt = NULL;		// Force enforcer hit if we use this again
}


ULONG PublicID(struct FSbase *FS)
{
	ULONG	theID;

#if 0
//	This fixes the FA0:FLY icons on WB when CD-ROM mounted early, but
//	causes Editor to miss Flyer drives!

	if (FS->ID == ID_FLYER_DISK)
		return(ID_DOS_DISK);
#endif

	theID = FS->ID & 0xFFFFFF00;
	if (theID != ID_DOS_DISK)		/* Not a normal data disk? */
		theID = FS->ID;

	return(theID);
}


int GetOSversion(void)
{
	register struct	ExecBase	*eb,**ebp;

	ebp = (struct ExecBase **)4;
	eb = *ebp;

	return(eb->LibNode.lib_Version);
}


//void BSTRtoCSTR(BSTR bname,char *cname)
//{
//	char *name;
//
//	name = (char *)BADDR(bname);
//	strncpy(cname,name+1,name[0]);
//	cname[name[0]] = 0;
//}

char *BSTRtoCSTR(BSTR bname)
{
	char *name,*cname;
	int	len;

	name = (char *)BADDR(bname);
	len = name[0];

	cname = AllocVec(len+1,MEMF_PUBLIC|MEMF_CLEAR);		// Allow room for NULL!
	if (cname)
	{
		strncpy(cname,name+1,len);			// Copy body of string
		cname[len] = 0;						// NULL-terminate copy
	}

	return(cname);
}


static const LONG ErrorConversion[] = {
/*	0,		*/								// FERR_OKAY
	ERROR_OBJECT_NOT_FOUND,			// FERR_NOTFOUND
	ERROR_DISK_FULL,					// FERR_FULL
	ERROR_DISK_FULL,					// FERR_DIRFULL
	ERROR_NO_MORE_ENTRIES,			// FERR_EXHAUSTED
	ERROR_DISK_NOT_VALIDATED,		// FERR_FAILED
	ERROR_OBJECT_WRONG_TYPE,		// FERR_WRONGTYPE
	ERROR_NOT_A_DOS_DISK,			// FERR_UNFORMATTED
	ERROR_OBJECT_IN_USE,				// FERR_EXCLUDED
	ERROR_SEEK_ERROR,					// FERR_OUTOFRANGE
	ERROR_OBJECT_TOO_LARGE,			// FERR_CANNOTEXTEND
	ERROR_DISK_WRITE_PROTECTED,	// FERR_PROTECTED
	-1,									// FERR_DIFFERENT
	ERROR_OBJECT_EXISTS,				// FERR_EXISTS
	ERROR_NO_FREE_STORE,				// FERR_NOMEM
	ERROR_DELETE_PROTECTED,			// FERR_DELPROT
	ERROR_READ_PROTECTED,			// FERR_READPROT
	ERROR_WRITE_PROTECTED,			// FERR_WRITEPROT
	ERROR_OBJECT_IN_USE,				// FERR_INUSE
	ERROR_DIRECTORY_NOT_EMPTY,		// FERR_DIRNOTEMPTY
};


LONG Flyer2AmigaErr(int error)
{
	if ((error >= FIRSTFSERR) && (error <= LASTFSERR))
		return(ErrorConversion[error-FIRSTFSERR]);
	else
		return(error);
}

void ReturnFlyerError(struct FSbase *FS,int error)
{
	if (error==FERR_OKAY)
		ReturnArgs(FS,DOSTRUE,0);			// Okay!
	else
		ReturnArgs(FS,DOSFALSE,Flyer2AmigaErr(error));
}


PktFunc PacketFunction(UWORD action)
{
	register PktFunc	func;

	switch (action)
	{
		case (ACTION_READ):
		case (ACTION_WRITE):
			func = RW_pkt;
			break;
		case (ACTION_SEEK):
			func = Seek_pkt;
			break;
		case (ACTION_EXAMINE_NEXT):
			func = ExamineNext_pkt;
			break;
		case (ACTION_EXAMINE_OBJECT):
			func = ExamineObject_pkt;
			break;
		case (ACTION_LOCATE_OBJECT):
			func = LocateObject_pkt;
			break;
		case (ACTION_FINDINPUT):
		case (ACTION_FINDOUTPUT):
		case (ACTION_FINDUPDATE):
			func = Open_pkt;
			break;
		case (ACTION_END):
			func = Close_pkt;
			break;
		case (ACTION_FREE_LOCK):
			func = FreeLock_pkt;
			break;
		case (ACTION_PARENT):
			func = Parent_pkt;
			break;
		case (ACTION_CURRENT_VOLUME):
			func = CurrentVolume_pkt;
			break;
		case (ACTION_TIMER):
			func = Timer_pkt;
			break;
		case (ACTION_FLUSH):
			func = Flush_pkt;
			break;
		case (ACTION_COPY_DIR):
			func = CopyDir_pkt;
			break;
		case (ACTION_INFO):
			func = Info_pkt;
			break;
		case (ACTION_DISK_INFO):
			func = DiskInfo_pkt;
			break;
		case (ACTION_MORE_CACHE):
			func = MoreCache_pkt;
			break;
		case (ACTION_INHIBIT):
			func = Inhibit_pkt;
			break;

		case (ACTION_WRITE_PROTECT):
			func = ProtLocker_pkt;
			break;
		case (ACTION_RENAME_DISK):
			func = RenameDisk_pkt;
			break;
		case (ACTION_SET_DATE):
			func = SetDate_pkt;
			break;
		case (ACTION_SET_PROTECT):
			func = SetProtect_pkt;
			break;
		case (ACTION_SET_COMMENT):
			func = SetComment_pkt;
			break;
		case (ACTION_DELETE_OBJECT):
			func = DeleteObject_pkt;
			break;
		case (ACTION_RENAME_OBJECT):
			func = RenameObject_pkt;
			break;
		case (ACTION_CREATE_DIR):
			func = CreateDir_pkt;
			break;
		case (ACTION_SET_FILE_SIZE):
			func = Truncate_pkt;
			break;

/* New packets for 2.0 */

		case (ACTION_IS_FILESYSTEM):
			func = IsFileSystem_pkt;
			break;
		case (ACTION_SAME_LOCK):
			func = SameLock_pkt;
			break;
		case (ACTION_FORMAT):
			func = Format_pkt;
			break;
		case (ACTION_ADD_NOTIFY):
		case (ACTION_REMOVE_NOTIFY):
			func = Notify_pkt;
			break;
		case (ACTION_MAKE_LINK):
			func = MakeLink_pkt;
			break;
		case (ACTION_LOCK_RECORD):
		case (ACTION_FREE_RECORD):
			func = Records_pkt;
			break;
		case (ACTION_PARENT_FH):
			func = ParentFH_pkt;
			break;
		case (ACTION_COPY_DIR_FH):
			func = CopyDirFH_pkt;
			break;
		case (ACTION_FH_FROM_LOCK):
			func = Open_pkt;
			break;
		case (ACTION_EXAMINE_FH):
			func = ExamineFH_pkt;
			break;
		case (ACTION_CHANGE_MODE):
			func = ChangeMode_pkt;
			break;
		default:
			func = NULL;
	}

	return(func);
}

#if DEBUGGING

void BSTR2Terminal(string)
char  *string;
{
	register	int	len;
	register	char	*p;

	kprintf("\x22");
	if (*string == 0)
		kprintf("NULL");
	else
	{
		p = string+1;
		for (len = *string;len;len--)
			kprintf("%lc",*p++);
	}
	kprintf("\x22");
}

#endif
