**********************************************************************
*
* flyer.library - VTASC algorithm support
*
* $Id: flyerlibvtasc.asm,v 1.5 1995/11/22 12:56:34 Flick Exp $
*
* $Log: flyerlibvtasc.asm,v $
*Revision 1.5  1995/11/22  12:56:34  Flick
*Replaced some inner-loop div and mul's with shifts
*Enabled special code for '020s and up
*This makes VTASC encoding/decoding an average of 20% faster
*
*Revision 1.4  1995/09/01  16:25:34  Flick
*Improved Sub-Nyquist regeneration to get luma from line and chroma from 2 pixels
*each from lines above and below (not quite the 3-pixel version yet, but it
*looks MUCH better than the OLD regen algorithm)
*
*Revision 1.3  1995/08/25  16:19:19  Flick
*Now am stashing user open modes and CPU type for field r/w
*Changed CodeMagSize to lookup table, added NOFIR option for field r/w
*Added improved VTASC comp/decomp using '020 bit fields (not turned on yet)
*
*Revision 1.2  1995/08/04  10:51:44  Flick
*Added include needed for new flyerprivate.i
*
*Revision 1.1  1995/07/19  11:38:37  Flick
*Changed OpenReadField2 to use new LocateField call, not Begin/Do/EndFindField
*
*Revision 1.0  1995/05/05  15:50:50  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* 09/22/94	Marty	created
**********************************************************************
* VTASC compression/decompress, FIR filtering, pseudo-random dithering
**********************************************************************

	include "exec/types.i"
	include "exec/lists.i"
	include "exec/libraries.i"
	include "exec/execbase.i"
	include "exec/memory.i"
	include "asmsupp.i"
	include "Flyer.i"
	include "FlyerPrivate.i"
	include "serialdebug.i"

OPTVER_020	equ	1


******* Exported *******************************************************

	XDEF	EndCode

	XDEF	OpenReadField2,OpenWriteField2,CloseField2
	XDEF	ReadLine2,WriteLine2
	XDEF	SetFillColor2,SkipLines2

******* Imported *******************************************************

	XREF	Read10,Write10,CPUDMA,FastCopy,GetCompInfo,GetCardUnit,FIRquery
	XREF	LocateField
	XREF	OpenWriteField1,CloseField1


*********************************************************************
* OpenReadField2 -- Locate name/field, prepare to read data
*
* Entry:
*	a0:struct ClipAction *ptr
*	d0:field
*	d1:mode flags (FWF_)
*	a6:Flyerbase (library)
*
* Exit:
*********************************************************************
OpenReadField2
	movem.l	d1-d3/a0-a2/a5,-(sp)
	move.l	d0,d2			;Keep field

	move.b	#0,d3			;Not in direct DMA mode (by default)
	clr.l	ca_FldHandle(a0)

;If NULL name ptr provided, assume we want to capture live video
	move.l	ca_Volume(a0),a1	;Get volume
	tst.l	fv_Path(a1)		;Any name provided?
	bne.s	.notdirect
	move.b	#1,d3			;"Direct" mode
	clr.l	ca_StartBlk(a0)		;lba=0 for "direct"
	bra.s	.skipfinder

.notdirect
	move.b	#RT_STOPPED,ca_ReturnTime(a0)
	move.l	d2,ca_VidStartField(a0)
	clr.b	ca_Flags(a0)		;Want no video/audio
	bsr	LocateField		;Find the frame (->ca_StartBlk)
	tst.l	d0			;An error?
	bne	.exit			;If failed, exit now

;	clr.b	ca_Flags(a0)		;Want no video/audio
;	bsr	BeginFindField		;Ready to find the field
;	tst.l	d0
;	bne	.exit
;
;	move.l	d2,ca_AudStartField(a0)
;	clr.b	ca_Flags(a0)		;Want no video/audio
;	bsr	DoFindField		;Find the frame
;
;	move.l	d0,-(sp)
;	bsr	EndFindField		;Shut down finder
;	move.l	(sp)+,d0		;Get 'find' result again
;	bne	.exit			;If failed, exit now

.skipfinder

	lea.l	VtascCtrl(pc),a5

	IFD	DEBUGVTASC
	DUMPHEXI.L <Allocated VTASC = >,a5,<\>
	ENDC

	move.b	d1,vt_ModeFlags(a5)	;Keep these during compresssion/decompression

	move.l	a5,ca_FldHandle(a0)	;Ctrl structure for future calls
	move.l	a6,vt_FlyerBase(a5)	;Need for Flyer calls

	bsr	FindSRAMbase		;Find Flyer's SRAM, keep in vt_

	clr.l	vt_RefNum(a5)		;No Flyer reference when reading

	and.b	#3,d2			;Field 0..3
	move.b	d2,d0
	move.l	ca_StartBlk(a0),d1	;lba of frame header
	move.l	ca_Volume(a0),a0	;Get volume only
	lea.l	MyVidComp(pc),a1	;Where to put results
	bsr	GetCompInfo
	tst.l	d0
	bne	.exit			;Failed?

	tst.b	d3			;Direct mode?
	beq.s	.SCSI
.direct
	pea	1			;Direct (from DRAM)
	move.l	d2,-(sp)		;Field #
	pea	0			;drive (unused)
	bra.s	.joinup

.SCSI
	pea	0			;Not direct (from SCSI)
	move.l	d2,-(sp)		;field (0..3)
	moveq.l	#0,d0
	move.b	fv_SCSIdrive(a0),d0
	move.l	d0,-(sp)		;drive
.joinup
	pea	0			;Decompression
	lea.l	MyVidComp(pc),a2
	bsr	StartVTASC		;Set things up
	lea.l	$10(sp),sp
	moveq.l	#FERR_OKAY,d0		;Okay

.exit
	movem.l	(sp)+,d1-d3/a0-a2/a5
	rts


*********************************************************************
* OpenWriteField2 -- Prepare to write field data to a clip
*
* Entry:
*	a0:struct ClipAction *ptr
*	d0:field.l
*	d1:mode.b
*	a1:struct VidCompInfo *compinfo
*	a6:Flyerbase (library)
*
* Exit:
*********************************************************************
OpenWriteField2
	movem.l	d1-d4/a0-a2/a5,-(sp)

	move.b	d1,d4				;Keep til later

	clr.l	ca_FldHandle(a0)		;Not successfully open yet
	move.b	#RT_STOPPED,ca_ReturnTime(a0)	;Must be synchronous!

	cmp.w	#0,a1			;Did user not care about compression?
	bne.s	.pickedVCI
	lea.l	MyVidComp(pc),a1	;Use my choices then
	bsr	DefaultCompression	;Use good defaults
.pickedVCI

	bsr	OpenWriteField1		;Have Flyer pick start/end points
	tst.l	d0			;Error? (e.g. FERR_FULL)
	bne	.exit			;(d1 = field,d2 = refnum)

*** Need to pick the proper seed for this field ***
	move.l	d1,d3
	divu	#255,d3			;Get field % 255
	swap	d3
	move.b	d3,vci_RndSeed(a1)	;Seed = (field % 255)

	lea.l	VtascCtrl(pc),a5

	IFD	DEBUGVTASC
	DUMPHEXI.L <Allocated VTASC = >,a5,<\>
	ENDC

	move.b	d4,vt_ModeFlags(a5)	;Keep these during compresssion/decompression

	move.l	a5,ca_FldHandle(a0)	;Ctrl structure for future calls
	move.l	a6,vt_FlyerBase(a5)	;Need for Flyer calls

	move.l	d2,vt_RefNum(a5)

	bsr	FindSRAMbase		;Find Flyer's SRAM, keep in vt_

	pea	0			;Not direct (to SCSI)
	move.l	d1,-(sp)		;field (0..3)
	moveq.l	#0,d0
	move.l	ca_Volume(a0),a0	;Get volume only
	move.b	fv_SCSIdrive(a0),d0
	move.l	d0,-(sp)		;drive
	pea	1			;Compression
	move.l	a1,a2			;Caller's VidCompInfo
	bsr	StartVTASC		;Set things up
	lea.l	$10(sp),sp
	moveq.l	#FERR_OKAY,d0		;Okay

.exit
	movem.l	(sp)+,d1-d4/a0-a2/a5
	rts


*********************************************************************
* DefaultCompression -- Select default compression parameters
*
* Entry:
*	a1:struct VidCompInfo *ptr (to fill in)
*
*********************************************************************
DefaultCompression
	move.b	#ALGO_D2,vci_Algo(a1)
	clr.b	vci_Tolerance(a1)	;0
	move.b	#2,vci_FIRcomp(a1)	;33% precompensation
	clr.b	vci_RndFreq(a1)
	move.b	#1,vci_RndSeed(a1)
	clr.b	vci_Flags(a1)
	move.w	#VIDSIZE_0,vci_DataSize(a1)	;Grade 0
	rts


*********************************************************************
* CloseField2 -- Finish reading/writing field data
*
* Entry:
*	a0:struct ClipAction *ptr (contains fhandle)
*	a6:Flyerbase (library)
*
* Exit:
*	d0:error code or 0
*********************************************************************
CloseField2
	movem.l	d1-d2/a5,-(sp)
	moveq.l	#FERR_BADFLDHAND,d0
	move.l	ca_FldHandle(a0),a5
	cmp.w	#0,a5
	beq.s	.exit

*** If a partial field was written, fill out with fill color ***
	tst.b	vt_comp(a5)		;(On compression only)
	beq.s	.no_fill
.fill_loop
	move.w	vt_y(a5),d0
	cmp.w	#LINESPERFIELD,d0	;Stop if/when full
	bcc.s	.no_fill

	IFD	DEBUGVTTEST
	DUMPMSG	<----pad---->
	ENDC
	move.l	a0,-(sp)		;Save ClipAction!
	lea.l	vt_SpareLine(a5),a0	;Line ptr
	bsr	FillLineWide		;Make line of fill color
	lea.l	vt_SpareLine(a5),a0	;Line ptr
	bsr	CompLine		;Compress into clip
	move.l	(sp)+,a0
	tst.b	d0			;Error?
	beq.s	.fill_loop		;If not, keep filling
	bra.s	.fill_fail
.no_fill
	moveq.l	#FERR_OKAY,d0
.fill_fail
	move.l	d0,-(sp)		;Save return value

	bsr	StopVTASC
	move.l	d0,-(sp)		;Save return value

	move.l	vt_RefNum(a5),d0	;Get Flyer reference number
	moveq.l	#1,d1			;Yes, we wrote stuff (implement this
					;for real, soon)
	move.l	vt_lba(a5),d2		;Final LBA (just past end)
	bsr	CloseField1		;Let Flyer wrap things up

	move.l	(sp)+,d1		;Retrieve StopVTASC return code
	beq.s	.keepprev1		;If okay, return Close err code
	move.l	d1,d0			;Return StopVTASC err code
.keepprev1

	move.l	(sp)+,d1		;Retrieve fill return code
	beq.s	.exit			;If okay, return Stop/Close err code
	move.l	d1,d0			;Return fill err code

.exit

	IFD	DEBUGVTTEST
	DUMPHEXI.L <CloseField returning >,d0,<\>
	ENDC

	movem.l	(sp)+,d1-d2/a5
	rts

*********************************************************************
* ReadLine2 -- Read a line of video data from open field
*
* Entry:
*	a0:struct ClipAction *ptr (contains fhandle)
*	a1:UBYTE *buffer (to receive)
*	a6:Flyerbase (library)
*
* Exit:
*	d0:error code or 0
*********************************************************************
ReadLine2
	movem.l	a1/a5,-(sp)

	moveq.l	#FERR_BADFLDHAND,d0
	move.l	ca_FldHandle(a0),a5
	cmp.w	#0,a5
	beq	RLIexit

	IFD	DEBUGVTASC
	DUMPHEXI.L <RL >,a1,<\>
	ENDC
	IFD	DEBUGVTLOTS
	DUMPHEXI.L <(VTASC = >,a5,<)\>
	ENDC

	move.w	vt_y(a5),d0
;;; This needs to account for users choice of Toaster/NTSC field size
	cmp.w	#TOASTERLINES,d0	;Still in legal Y range?
	bcc.s	.pastend

	lea.l	vt_SpareLine(a5),a0	;Line ptr to hold wide line
	bsr	DecompLine		;Do it!

	lea.l	vt_SpareLine(a5),a0	;My line buffer
	lea.l	4(a0),a0		;Skip past pad pixels
					;a1 contains user's buffer
	move.l	#SAMPLESPERLINE,d0	;Size of copy
	jsr	FastCopy		;Copy final line of active pixels

	moveq.l	#FERR_OKAY,d0
	bra.s	RLIexit
.pastend
	move.l	a1,a0			;Line ptr
	bsr	FillLineActive
	add.w	#1,vt_y(a5)		;Y++
	moveq.l	#FERR_OKAY,d0
RLIexit
	movem.l	(sp)+,a1/a5
	rts


*********************************************************************
* WriteLine2 -- Write a line of video data to open field
*
* Entry:
*	a0:struct ClipAction *ptr (contains fhandle)
*	a1:UBYTE *buffer (source data)
*	a6:Flyerbase (library)
*
* Exit:
*	d0:error code or 0
*********************************************************************
WriteLine2
	movem.l	a1/a5,-(sp)

	moveq.l	#FERR_BADFLDHAND,d0
	move.l	ca_FldHandle(a0),a5
	cmp.w	#0,a5
	beq	.exit

	IFD	DEBUGVTASC
	DUMPHEXI.L <WL >,a1,<\>
	DUMPHEXI.W <Y = >,vt_y(a5),<)\>

;	DUMPHEXI.L <LBAs = >,vt_lba(a5),< >
;	DUMPHEXI.L <- >,vt_lbastop(a5),<\>
	ENDC

	IFD	DEBUGVTLOTS
	DUMPHEXI.L <(VTASC = >,a5,<)\>
	ENDC

	move.w	vt_y(a5),d0
;;; This needs to account for users choice of Toaster/NTSC field size
	cmp.w	#TOASTERLINES,d0	;Still in legal Y range?
	bcc.s	.pastend

**** Copy line of video to my spare line buffer ****
	move.l	a1,a0			;Line ptr
	lea.l	vt_SpareLine+4(a5),a1	;My line buffer (skip past pad pixels)
	move.l	#SAMPLESPERLINE,d0	;Size of copy
	jsr	FastCopy		;Copy provided line of active pixels

	lea.l	vt_SpareLine(a5),a0	;Source data to work with
	bsr	CompLine		;Do it!
	bra.s	.exit			;Returns its own error code
.pastend
	add.w	#1,vt_y(a5)		;Y++
	moveq.l	#FERR_OKAY,d0
.exit
	movem.l	(sp)+,a1/a5
	rts

*********************************************************************
* SetFillColor2 -- Set fill color to use for blankness during field r/w
*
* Entry:
*	a0:struct ClipAction *ptr (contains fhandle, YIQ)
*	a6:Flyerbase (library)
*
* Exit:
*	d0:error code or 0
*********************************************************************
SetFillColor2
	movem.l	a5,-(sp)

	moveq.l	#FERR_BADFLDHAND,d0
	move.l	ca_FldHandle(a0),a5
	cmp.w	#0,a5
	beq.s	.exit

	move.w	ca_MatteY(a0),d0
	move.b	d0,vt_FillY(a5)			;Y
	move.b	ca_MatteI(a0),vt_FillI(a5)	;I
	move.b	ca_MatteQ(a0),vt_FillQ(a5)	;Q

	moveq.l	#FERR_OKAY,d0
.exit
	movem.l	(sp)+,a5
	rts

*********************************************************************
* SkipLines2 -- Skip specified number of lines in field read/write
*
* Entry:
*	a0:struct ClipAction *ptr (contains fhandle)
*	d0:lines.w
*	a6:Flyerbase (library)
*
* Exit:
*	d0:error code or 0
*********************************************************************
SkipLines2
	movem.l	d1/d6-d7/a0-a1/a3-a5,-(sp)

	IFD	DEBUGVTASC
	DUMPHEXI.W <SkipLineCount = >,d0,<\>
	ENDC

	move.w	d0,d1			;Line counter

	moveq.l	#FERR_BADFLDHAND,d0
	move.l	ca_FldHandle(a0),a5
	cmp.w	#0,a5
	beq	.exit

	IFD	DEBUGVTLOTS
	DUMPHEXI.L <(VTASC = >,a5,<)\>
	ENDC

	bra	.intoit

.again

	IFD	DEBUGVTASC
	DUMPHEXI.W <Y = >,vt_y(a5),<)\>
	ENDC

	move.w	vt_y(a5),d0
;;; This needs to account for users choice of Toaster/NTSC field size
	cmp.w	#TOASTERLINES,d0	;Abort if we skip outside legal Y
	bcc.s	.exit

	tst.b	vt_comp(a5)		;Compression or Decompression?
	beq.s	.notcomp
	lea.l	vt_SpareLine(a5),a0	;Line ptr
	bsr	FillLineWide		;Make line of blank color
	lea.l	vt_SpareLine(a5),a0	;Line ptr
	bsr	CompLine		;Compress into clip
	tst.b	d0			;Error?
	bne.s	.exit			;If so, abort
	bra.s	.intoit
.notcomp
	sub.l	a0,a0			;No line ptr
	bsr	DecompLine		;Decompress line, throw away

.intoit
	dbf	d1,.again		;Do all lines

	moveq.l	#FERR_OKAY,d0
.exit
	movem.l	(sp)+,d1/d6-d7/a0-a1/a3-a5
	rts

*********************************************************************
* StartVTASC -- Start field (de)compress
*
* Entry:
*	a5:VTASC ctrl structure
*	a2:VidCompInfo structure
*	stack:comp,drive,field,direct
*	      8    C     10    14
*
* Exit:
*********************************************************************
StartVTASC
	link	a6,#0
	movem.l	d0-d2/a0-a1,-(sp)

	moveq.l	#FERR_OKAY,d0
	move.b	d0,vt_Error(a5)		;All is well so far

	move.l	a6,-(sp)
	move.l	(_AbsExecBase).w,a6
	move.w	AttnFlags(a6),vt_CPU(a5)	;Optimize code for CPU type
	move.l	(sp)+,a6

	move.b	$B(a6),d0		;(comp)
	move.b	d0,vt_comp(a5)

	move.b	$13(a6),d0		;(field)
	move.b	d0,vt_field(a5)

	IFD	DEBUGVTSETUP
	DUMPHEXI.B <*** StartVTASC *** (field = >,d0,<)\>
	ENDC

****** Random noise stuff
	move.b	vci_Tolerance(a2),d0		;(toler)
	cmp.b	#MAXTOLER,d0	;Max tolerance
	bls.s	.tolerokay
	move.b	#MAXTOLER,d0
.tolerokay
	move.b	d0,vt_tolbits(a5)
	move.b	d0,vt_RndDither(a5)
;;	move.b	#2,vt_RndDither(a5)	;REMOVE!!!  Testing only

	IFD	DEBUGVTSETUP
	DUMPHEXI.B <Tolerance = >,d0,<\>
	ENDC

	move.w	d0,vt_tolmag(a5)	;Save shift bits
;	moveq.l	#1,d1		;Calc tolerance magnitude from tolbits
;	lsl.l	d0,d1
;	move.w	d1,vt_tolmag(a5)

	move.b	vci_RndFreq(a2),d0		;(freq)
	addq.l	#1,d0				;P-Coders do (period+1)
	move.b	d0,vt_RndFreq(a5)

	IFD	DEBUGVTSETUP
	DUMPHEXI.B <RndFreq = >,d0,<\>
	ENDC

	move.b	vci_RndSeed(a2),d0		;(seed)
	move.b	d0,vt_RndSeed(a5)

	IFD	DEBUGVTSETUP
	DUMPHEXI.B <Seed = >,d0,<\>
	ENDC

****** Set hard drive storage interface
	move.b	$F(a6),d0		;(drive)
	move.b	d0,vt_drive(a5)

	IFD	DEBUGVTSETUP
	DUMPHEXI.B <Drive = >,d0,<\>
	ENDC

	move.l	vci_DataStart(a2),d0	;(lba/DRAMblk)

	tst.b	$17(a6)			;Direct to/from DRAM memory?
	beq	.talktoSCSI

	IFD	DEBUGVTSETUP
	DUMPHEXI.L <DRAM = >,d0,<\>
	ENDC

	move.l	d0,vt_DRAMblk(a5)	;Move data to/from DRAM memory...
	clr.l	vt_lba(a5)		; ...not to/from SCSI
	bra	.meethere

.talktoSCSI
	move.l	d0,vt_lba(a5)		;Move data to/from SCSI
	IFD	DEBUGVTSETUP
	DUMPHEXI.L <LBA = >,d0,<\>
	ENDC

	move.l	vci_DataEnd(a2),d0	;lba
	move.l	d0,vt_lbastop(a5)	;SCSI limit for writing
	IFD	DEBUGVTSETUP
	DUMPHEXI.L <LBASTOP = >,d0,<\>
	ENDC
.meethere

****** Set algorithm specs
	move.b	vci_Algo(a2),d0		;(algo)
	cmp.b	#ALGO_OLDD2,d0
	beq.s	.oldd2
	cmp.b	#ALGO_D2,d0
	beq.s	.newd2
	cmp.b	#ALGO_SN,d0
	beq.s	.subnyq
.oldd2
	clr.b	vt_SNflag(a5)
	clr.b	vt_pcoder(a5)		;Old version predictor
	bra.s	.algomerge
.newd2
	clr.b	vt_SNflag(a5)
	move.b	#1,vt_pcoder(a5)	;New version predictor
	bra.s	.algomerge
.subnyq
	move.b	#1,vt_SNflag(a5)	;Sub-nyquist
	move.b	#1,vt_pcoder(a5)	;New version predictor
.algomerge
	IFD	DEBUGVTSETUP
	DUMPHEXI.B <Algo = >,d0,<\>
	ENDC


****** Get FIR info from Flyer
	move.l	a6,-(sp)
	move.l	vt_FlyerBase(a5),a6

	moveq.l	#0,d0			;Flyer board
	move.b	vci_FIRcomp(a2),d1	;Coef set

	IFD	DEBUGVTSETUP
	DUMPHEXI.B <FIR set used >,d1,<\>
	ENDC

	CLEAR	d2			;Pre-comp (record)
	tst.b	vt_comp(a5)
	bne.s	.yepusepre
	moveq.l	#1,d2			;Play, so use post-comp
.yepusepre				;d2=Pre(0)/Post(1)
	lea.l	vt_fscale(a5),a0	;UWORD *scale
	lea.l	vt_Coefs(a5),a1		;UWORD Coefs[8]
	bsr	FIRquery		;Get FIR coefs/scale
	move.l	(sp)+,a6
;	tst.b	d0			;Error?
;	bne	.error

****************** End of FIR stuff

;	move.l	$14(a6),d0		;(scale)
;	move.w	d0,vt_fscale(a5)

	IFD	DEBUGVTSETUP
	DUMPHEXI.W <FIR Scale set to >,vt_fscale(a5),<\>
	ENDC

****** Reword FIR coefficients, sign extending from 10 bits to 16 bits
;	move.l	$10(a6),a0		;(coefs)
	lea.l	vt_Coefs(a5),a1
	moveq.l	#8-1,d1
.coefcopy
;	move.w	(a0)+,d0
	move.w	(a1),d0
	and.w	#$3FF,d0	;Strip off any above 10 bits!

	btst	#9,d0		;Negative?
	beq.s	.ispos
	sub.w	#1024,d0	;Sign-extend to 16
.ispos
	move.w	d0,(a1)+

	IFD	DEBUGVTSETUP
	DUMPHEXI.W <Coef = >,d0,<\>
	ENDC

	dbf	d1,.coefcopy

****** Set fill color to black
	CLEAR	d0
	move.b	#NTSC_BLACK,vt_FillY(a5)
	move.b	d0,vt_FillI(a5)
	move.b	d0,vt_FillQ(a5)

	move.b	#BORDERVALUE,d0		;For old D2 coding only
	move.b	d0,vt_C4(a5)
	move.b	d0,vt_C5(a5)
	move.b	d0,vt_C6(a5)


****** Fill 2 line memory with border value
	lea.l	vt_Lines(a5),a0
	move.b	#BORDERVALUE,d0
	move.w	#(PIXELSPERLINE*2)-1,d1
.fillup2
	move.b	d0,(a0)+
	dbf	d1,.fillup2

	move.b	#1,vt_topline(a5)
	move.b	#2,vt_extraline(a5)

	move.b	vt_RndSeed(a5),d0
	bsr	InitRndSeed

****** Get ready for first line
	move.w	#8,vt_msize(a5)		;msize starts at 8
	clr.b	vt_preverr(a5)
	clr.w	vt_y(a5)		;Y=0

****** Init pseudo-random number generation stuff
	move.b	#1,vt_rndcntr(a5)
	clr.b	vt_RndNum(a5)

;THE NEW P-CODERS DO NOT DO THIS -- THEY USE RND # 00 FOR FIRST GROUP OF PIXELS
;THEN GENERATES FIRST NUMBER JUST BEFORE SECOND GROUP (based on vt_RndFreq)
;****** Get our first random value
;	move.b	vt_RndDither(a5),d0
;	bsr	GenRndNum
;	move.b	d0,vt_RndNum(a5)

****** Do mode-specific setup (comp/decomp)
	tst.b	vt_comp(a5)
	beq.s	.decompmode

	bsr	InitBitOut	;Ready for compression

;Was commented out
;	tst.b	vt_SNflag(a5)	;Sub-nyquist?
;	beq.s	.noSNsetup
;	subq.w	#1,vt_y(a5)	;Y=-1
;	sub.l	a0,a0		;No line ptr
;	bsr	CompLine	;Compress no line, but cache first one
;.noSNsetup

;Test if fields 2/4
	move.b	vt_field(a5),d0
	btst	#0,d0		;Field = 2/4?
	beq.s	.notTst24	;(Skip if 1/3)
;;; Skip this section if in NTSC video mode
	IFD	DEBUGVTTEST
	DUMPMSG	<----NTSC comp pad---->
	ENDC
	lea.l	vt_SpareLine(a5),a0	;Line ptr
	bsr	FillLineWide		;Make line of fill color
	lea.l	vt_SpareLine(a5),a0	;Line ptr
	bsr	CompLine		;Compress into clip
;Shouldn't we handle an error code here???  Yes we should!!!
	subq.w	#1,vt_y(a5)		;Pretend we're still on 1st line
.notTst24
	bra.s	.notdecomp

.decompmode
	bsr	InitBitIn	;Ready for decompression

	tst.b	vt_SNflag(a5)	;Sub-nyquist?
	beq.s	.noSNsetup
	subq.w	#1,vt_y(a5)	;Y--
.noSNsetup
;Test if fields 2/4
	move.b	vt_field(a5),d0
	btst	#0,d0		;Field = 2/4?
	beq.s	.notToaster24	;(Skip if 1/3)
; Skip this section if in NTSC video mode
	subq.w	#1,vt_y(a5)	;y--
.notToaster24

.pullinglines
	tst.w	vt_y(a5)	;Pull lines until y=0
	beq.s	.iszero
	IFD	DEBUGVTTEST
	DUMPMSG	<----decomp toss---->
	ENDC
	sub.l	a0,a0		;No line ptr
	bsr	DecompLine	;Decompress (and cache line)
	bra.s	.pullinglines
.iszero	

;	tst.b	vt_SNflag(a5)	;Sub-nyquist?
;	beq.s	.noSNsetup
;	subq.w	#1,vt_y(a5)	;Y=-1
;	sub.l	a0,a0		;No line ptr
;	bsr	DecompLine	;Decompress and cache first line
;.noSNsetup
;;Test if fields 2/4
;	move.b	vt_field(a5),d0
;	btst	#0,d0		;Field = 2/4?
;;	beq.s	.notToaster24	;(Skip if 1/3)
;;;; Skip this section if in NTSC video mode
;;	sub.l	a0,a0		;No line ptr
;;	bsr	DecompLine	;Toaster video: throw away first (half) line
;;	subq.w	#1,vt_y(a5)	;Pretend we're still on 1st line
;;.notToaster24
.notdecomp

.exit	movem.l	(sp)+,d0-d2/a0-a1
	unlk	a6
	rts


*********************************************************************
* StopVTASC -- End field (de)compress
*
* Entry:
*	a5:VTASC ctrl structure
*
* Exit:
*	d0:Error code (FERR_FULL) or 0
*********************************************************************
StopVTASC
	tst.b	vt_comp(a5)		;In compress mode?
	beq.s	.notcomp
	bsr	PurgeOut		;Flush any residual data
	bra.s	.exit
.notcomp
	nop
.exit
	rts

*********************************************************************
* InitRndSeed -- Initialize random number generator with a seed value
*		 $FF is an illegal seed value
*
* Entry:
*	a5:VTASC ctrl structure
*	d0:seed.b
*
* Exit:
*********************************************************************
InitRndSeed
;	IFD	DEBUGVTTEST
;	DUMPHEXI.B <Seed24 = >,d0,<\>
;	ENDC

	movem.l	d1-d3,-(sp)
	cmp.b	#$FF,d0
	bne.s	.notbadseed
	addq.b	#1,d0
.notbadseed
	moveq.l	#7,d2
	moveq.l	#0,d1
	moveq.l	#8-1,d3
.loop
	lsr.b	#1,d0
	bcc.s	.no3bits
	add.l	d2,d1
.no3bits
	lsl.l	#3,d2		;shift group '111' to next 3 bits
	dbf	d3,.loop

	move.l	d1,vt_RndData(a5)	;Store as 24 PRN bits

	movem.l	(sp)+,d1-d3
	rts



*********************************************************************
* GenRndNum -- Generate a signed random number of a given length
*
* Entry:
*	a5:VTASC ctrl structure
*	d0:bit length of random number desired (.b)
*
* Exit:
*	d0:Signed byte value
*********************************************************************
GenRndNum
	movem.l	d2-d3,-(sp)
	moveq	#0,d3
	move.b	d0,d3			;Keep length

	move.l	vt_RndData(a5),d0

;	IFD	DEBUGVTTEST
;	DUMPHEXI.L <(RND24in = >,d0,<)\>
;	ENDC

	moveq.l	#4-1,d2
.loop
	btst	#23,d0
	beq.s	.not23
	btst	#16,d0
	beq.s	.bit0
	bra.s	.bit1
.not23
	btst	#16,d0
	bne.s	.bit0
.bit1
	lsl.l	d0		;Recirculate 1 back into shift register
	addq.l	#1,d0
	bra.s	.bitmerge
.bit0
	lsl.l	d0		;Recirculate 0 back into shift register
.bitmerge
	dbf	d2,.loop	;Gather 4 bit code

	move.l	d0,vt_RndData(a5)	;Keep new shift register bits

	moveq.l	#8-1,d2
	bra.s	.intoit
.takeNbits
	ror.b	d0		;Move number of desired bits to top of byte
	subq.w	#1,d2
.intoit
	dbf	d3,.takeNbits

.signextend
	asr.b	d0		;Pad remainder of byte with sign bits
	dbf	d2,.signextend

;	IFD	DEBUGVTTEST
;	DUMPHEXI.B <[>,d0,<]\>
;	ENDC

	movem.l	(sp)+,d2-d3
	rts



*********************************************************************
* InitBitOut -- Prepare bit buffering mechanism to write codes
*
* Entry:
*	a5:VTASC ctrl structure
*
* Exit:
*********************************************************************
InitBitOut
	IFD	OPTVER_020
	btst	#AFB_68020,vt_CPU+1(a5)		;68020 or better?
	bne.s	InitBitOut020
	ENDC

InitBitOut000
	move.l	a0,-(sp)
	lea.l	vt_BitBuffer(a5),a0
	move.l	a0,vt_BitBufPtr(a5)
	clr.w	vt_BitBufAmt(a5)	;Buffer is empty
	clr.b	vt_NextBit(a5)		;Start with LSB
	clr.b	vt_BitBucket(a5)	;Empty
	move.l	(sp)+,a0
	rts

	IFD	OPTVER_020
InitBitOut020
	move.w	#8+BITBUFSIZE*8,vt_BitBufAmt(a5)	;Current offset position
							;(+8: ref'd from pre-byte)
	rts
	ENDC

*********************************************************************
* OutputCode -- Write out a variable length code to disk
*
* Entry:
*	a5:VTASC ctrl structure
*	d0:Code
*	d1:Length of code (in bits) .b
*
* Exit:
*	(none -- get error from vt_Error(a5))
*********************************************************************
OutputCode
	IFD	OPTVER_020
	btst	#AFB_68020,vt_CPU+1(a5)		;68020 or better?
	bne	OutputCode020
	ENDC

OutputCode000
	movem.l	d0-d4/a0,-(sp)

	move.b	vt_BitBucket(a5),d4	;Working value
	move.b	vt_NextBit(a5),d3	;Working value

	and.w	#$00FF,d1
	subq.w	#1,d1		;Prepare bit counter

.bitloop
	bclr	d3,d4
	btst	#0,d0		;Check next LSB of code
	beq.s	.not1
	bset	d3,d4		;If 1, set next bit in bucket
.not1
	lsr.b	d0		;Toss bit read from code
	addq.b	#1,d3		;Bump nextbit
***** If BitBucket full, move to output buffer
	cmp.b	#8,d3
	bne.s	.bitsleft
	move.l	vt_BitBufPtr(a5),a0	;Get fill pointer
	move.b	d4,(a0)+		;Place new byte
	move.l	a0,vt_BitBufPtr(a5)	;Keep new ptr
	addq.w	#1,vt_BitBufAmt(a5)	;One more byte in buffer
	clr.b	d3			;Start anew at LSB
***** If Buffer is now full, write to disk
	move.w	vt_BitBufAmt(a5),d2
	cmp.w	#BITBUFSIZE,d2		;Maxed out?
	bne.s	.notfull

;	move.l	d0,-(sp)		;Save code
	bsr	EmptyBuffer		;Write it out to disk
;	cmp.b	#FERR_FULL,d0		;Ran out of room?
;	bne.s	.goinstrong
;	move.b	d0,vt_Error(a5)		;Stop after line!
;.goinstrong
;	move.l	(sp)+,d0		;Restore code

	lea.l	vt_BitBuffer(a5),a0
	move.l	a0,vt_BitBufPtr(a5)	;Pointer to start again
	clr.w	vt_BitBufAmt(a5)	;None in buffer
.notfull
.bitsleft
	dbf	d1,.bitloop

	move.b	d4,vt_BitBucket(a5)	;Store off for next time
	move.b	d3,vt_NextBit(a5)	;Store off for next time

	movem.l	(sp)+,d0-d4/a0
	rts

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	IFD	OPTVER_020
	OPT	P=68020

OutputCode020		;d0=code, d1=len
	movem.l	d1-d2,-(sp)

	moveq.l	#0,d2
	move.b	d1,d2			;Promote code length to a long

	moveq.l	#0,d1
	move.w	vt_BitBufAmt(a5),d1	;Get current bit field position
	sub.l	d2,d1			;Move offset to place next
	bfins	d0,vt_BitBuffPre(a5){d1:d2}	;Drop in the bits!

	cmp.w	#8,d1			;Wrote into the "pre" overflow byte?
	bhi.s	.dontempty		;(or filled buffer entirely?)

;	move.l	d0,-(sp)		;Save code
	bsr	EmptyBuffer		;Write it out to disk
;	cmp.b	#FERR_FULL,d0		;Ran out of room?
;	bne.s	.goinstrong
;	move.b	d0,vt_Error(a5)		;Stop after line!
;.goinstrong
;	move.l	(sp)+,d0		;Restore code

	move.b	vt_BitBuffPre(a5),vt_BitBuffer+BITBUFSIZE-1(a5)	;Move overflow into next grp
	add.w	#BITBUFSIZE*8,d1	;New current pos

.dontempty
	move.w	d1,vt_BitBufAmt(a5)	;Save current pos

	movem.l	(sp)+,d1-d2
	rts

	OPT	P=68000
	ENDC

*********************************************************************
* PurgeOut -- Write out any residual unwritten data
*
* Entry:
*	a5:VTASC ctrl structure
*
* Exit:
*	d0:Error code (FERR_FULL or 0)
*********************************************************************
PurgeOut
	IFD	OPTVER_020
	btst	#AFB_68020,vt_CPU+1(a5)		;68020 or better?
	bne.s	PurgeOut020
	ENDC

PurgeOut000
	move.l	d1,-(sp)
**** Flush out any residual bits left in bitbucket -- pad to end of block
	bra.s	.test
.padout
	moveq.l	#0,d0		;Code = 0
	moveq.l	#1,d1		;Length = 1
	bsr	OutputCode
	tst.b	vt_Error(a5)		;Error?
	bne.s	.fail
.test
	tst.b	vt_NextBit(a5)		;Any residual bits?
	bne.s	.padout
	tst.w	vt_BitBufAmt(a5)	;Any residual bytes?
	bne.s	.padout

.fail
	CLEAR	d0
	move.b	vt_Error(a5),d0		;Error?

	IFD	DEBUGVTTEST
	DUMPHEXI.L <PurgeOut returning >,d0,<\>
	ENDC

	move.l	(sp)+,d1
	rts


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	IFD	OPTVER_020

PurgeOut020
	move.l	d1,-(sp)
**** Flush out any residual bits left in bitbuffer -- pad to end of block
	bra.s	.test
.padout
	moveq.l	#0,d0		;Code = 0
	moveq.l	#1,d1		;Length = 1
	bsr	OutputCode
	tst.b	vt_Error(a5)		;Error?
	bne.s	.fail
.test
	move.w	vt_BitBufAmt(a5),d1	;Any residual bits?
	cmp.w	#8+BITBUFSIZE*8,d1	;Target value for current bit pos
	bne.s	.padout

.fail
	CLEAR	d0
	move.b	vt_Error(a5),d0		;Error?

	IFD	DEBUGVTTEST
	DUMPHEXI.L <PurgeOut returning >,d0,<\>
	ENDC

	move.l	(sp)+,d1
	rts

	ENDC

*********************************************************************
* InitBitIn -- Prepare bit buffering mechanism to read codes
*
* Entry:
*	a5:VTASC ctrl structure
*
* Exit:
*********************************************************************
InitBitIn
	IFD	OPTVER_020
	btst	#AFB_68020,vt_CPU+1(a5)		;68020 or better?
	bne.s	InitBitIn020
	ENDC

InitBitIn000
	move.w	#0,vt_BitBufAmt(a5)	;Have no data
	move.b	#8,vt_NextBit(a5)	;Ready to reload
	rts

	IFD	OPTVER_020
	OPT	P=68020

InitBitIn020
	move.w	#0,vt_BitBufAmt(a5)	;Bit field offset at 0 = Have no data
	rts

	OPT	P=68000
	ENDC


*********************************************************************
* InputCode -- Read in code of the given size from bit buffering
*
* Entry:
*	a5:VTASC ctrl structure
*	d0:length of code in bits (.b)
*
* Exit:
*	d0:code (.b)
*********************************************************************
InputCode
	IFD	OPTVER_020
	btst	#AFB_68020,vt_CPU+1(a5)		;68020 or better?
	bne	InputCode020
	ENDC

InputCode000
	movem.l	d1-d4/a0,-(sp)

	CLEAR	d1
	move.b	d0,d1		;Bit counter
	subq.l	#1,d1
	moveq.l	#1,d2		;Bit weight
	CLEAR	d0		;Build code here

	move.b	vt_BitBucket(a5),d4	;Working value
	move.b	vt_NextBit(a5),d3	;Working value

.bitloop
***** If BitBucket is exhausted, get another byte
	cmp.b	#8,d3
	bne	.havebits

***** If BitBuffer is exhausted, read some more bytes in
	tst.w	vt_BitBufAmt(a5)
	bne	.somedataleft

	bsr	ReloadBuffer		;Read in from disk
;Handle SCSI error?!?!?

	move.w	#BITBUFSIZE,vt_BitBufAmt(a5)
	lea.l	vt_BitBuffer(a5),a0
	move.l	a0,vt_BitBufPtr(a5)
.somedataleft
	move.l	vt_BitBufPtr(a5),a0
	move.b	(a0)+,d4		;Grab next 8 bits
	move.l	a0,vt_BitBufPtr(a5)
	sub.w	#1,vt_BitBufAmt(a5)	;One less in buffer
	CLEAR	d3			;Ready for lsb
.havebits
	btst	d3,d4			;1 or 0?
	beq.s	.not1bit
	or.b	d2,d0			;Set proper bit
.not1bit
	addq.b	#1,d3
	lsl.b	d2

	dbf	d1,.bitloop

	move.b	d4,vt_BitBucket(a5)	;Store off for next time
	move.b	d3,vt_NextBit(a5)	;Store off for next time

;	IFD	DEBUGVTSETUP
;	DUMPHEXI.B <==== >,d0,<\>
;	ENDC

	movem.l	(sp)+,d1-d4/a0
	rts

;~~~~~~~~~~~~~~~~~~
	IFD	OPTVER_020
	OPT	P=68020

InputCode020
	movem.l	d1-d2,-(sp)

	moveq.l	#0,d1
	move.w	vt_BitBufAmt(a5),d1	;Get current bit field offset
	cmp.w	#8,d1			;Need to load more data?
	bhi.s	.noreload

	move.b	vt_BitBuffer(a5),vt_BitBuffPost(a5)	;Shove fragment to end
	bsr	ReloadBuffer		;Read more from disk
;Handle SCSI error?!?!?
	add.w	#BITBUFSIZE*8,d1	;Advance bit field offset to end of fragment moved

.noreload
	moveq.l	#0,d2
	move.b	d0,d2			;Promote code size to long
	sub.l	d2,d1			;Adjust bit offset to get code size
	move.w	d1,vt_BitBufAmt(a5)	;Save as current bit position

	moveq.l	#0,d0
	bfextu	vt_BitBuffer(a5){d1:d2},d0	;Get code

	movem.l	(sp)+,d1-d2
	rts

	OPT	P=68000
	ENDC

*********************************************************************
* ReloadBuffer -- Load more data from Flyer into BitBuffer
*
* Entry:
*	a5:VTASC ctrl structure
*
* Exit:
*********************************************************************
ReloadBuffer
	movem.l	d0-d2/a0-a2/a6,-(sp)

	IFD	DEBUGVTLOTS
	DUMPHEXI.L <(VTASC = >,a5,<) >
	ENDC
	IFD	DEBUGVTASC
	DUMPMSG	< Reload\>
	ENDC

	move.l	vt_FlyerBase(a5),a6

**** Read next BITBUFBLKS (at vt_drive) into my BitBuffer

	lea.l	MyVolume(pc),a1
	clr.b	fv_Board(a1)
	move.b	vt_drive(a5),fv_SCSIdrive(a1)

	lea.l	MyAction(pc),a0
	move.l	a1,ca_Volume(a0)
	move.b	#RT_STOPPED,ca_ReturnTime(a0)

	tst.l	vt_lba(a5)		;From SCSI?
	beq.s	.directmem

***** Read from drive to special DRAM area
	lea.l	MyAction(pc),a0
	move.l	vt_lba(a5),d1
	moveq.l	#BITBUFBLKS,d0
	add.l	d0,vt_lba(a5)		;Ready for next time
	move.l	#VTASCDRAM,d2
	bsr	Read10
	tst.b	d0			;Error?
	bne	.error

***** DMA from DRAM to SRAM
	move.l	#VTASCDRAM,a1	;DMA RAM
	bra.s	.dmait

.directmem
	move.l	vt_DRAMblk(a5),a1	;Source DRAM block(s)
	moveq.l	#BITBUFBLKS,d0
	add.l	d0,vt_DRAMblk(a5)	;Ready for next time

.dmait
;	lea.l	MyVolume(pc),a0
	move.l	#VTASCSRAM,a0	;CPU RAM
	moveq.l	#0,d0		;Flyer board
	move.w	#BITBUFBLKS,d1
	CLEAR	d2		;Write to CPU RAM
	bsr	CPUDMA		;Move to SRAM
	tst.b	d0		;Error?
	bne	.error

***** Copy from special block-aligned SRAM area to my private buffer
	IFD	OPTVER_020
	btst	#AFB_68020,vt_CPU+1(a5)		;68020 or better?
	bne	.copy020
	ENDC
.copy000
	move.l	#VTASCSRAMADDR,a0	;From SRAM
	adda.l	vt_SRAMbase(a5),a0	;Offset into Flyer SRAM
	lea.l	vt_BitBuffer(a5),a1	;To BitBuffer
	move.l	#BITBUFSIZE,d0
	bsr	FastCopy
	bra.s	.exit

	IFD	OPTVER_020
.copy020
	move.l	#VTASCSRAMADDR,a0	;From SRAM
	adda.l	vt_SRAMbase(a5),a0	;Offset into Flyer SRAM
	lea.l	vt_BitBuffer+BITBUFSIZE(a5),a1	;End of BitBuffer
	move.l	#BITBUFSIZE-1,d0
.rvscopy
	move.b	(a0)+,-(a1)		;Must copy backwards for '020 bit field stuff
	dbf	d0,.rvscopy
	bra.s	.exit
	ENDC

***** Error occurred, just fill BitBuffer with 0's
.error	lea.l	vt_BitBuffer(a5),a0
	move.w	#(BITBUFSIZE/4)-1,d1
	moveq.l	#0,d0
.clrloop
	move.l	d0,(a0)+		;Blank out
	dbf	d1,.clrloop

.exit	movem.l	(sp)+,d0-d2/a0-a2/a6
	rts


*********************************************************************
* EmptyBuffer -- Save data from BitBuffer to Flyer disk
*
* Entry:
*	a5:VTASC ctrl structure
*
* Exit:
*	d0:Error code if out of room (FERR_FULL)
*********************************************************************
EmptyBuffer
	movem.l	d0-d3/a0-a2/a6,-(sp)

	IFD	DEBUGVTLOTS
	DUMPHEXI.L <(VTASC = >,a5,<) >
	ENDC
	IFD	DEBUGVTASC
	DUMPMSG	<EmptyBuf>
	ENDC

	move.l	vt_FlyerBase(a5),a6

**** Write next BITBUFBLKS (at vt_drive) from my BitBuffer

	lea.l	MyVolume(pc),a1
	clr.b	fv_Board(a1)
	move.b	vt_drive(a5),fv_SCSIdrive(a1)

	lea.l	MyAction(pc),a0
	move.l	a1,ca_Volume(a0)
	move.b	#RT_STOPPED,ca_ReturnTime(a0)

***** Copy from my private buffer to special block-aligned SRAM area
	IFD	OPTVER_020
	btst	#AFB_68020,vt_CPU+1(a5)		;68020 or better?
	bne.s	.copy020
	ENDC

.copy000
	lea.l	vt_BitBuffer(a5),a0	;From BitBuffer
	move.l	#VTASCSRAMADDR,a1	;To SRAM
	adda.l	vt_SRAMbase(a5),a1	;Offset into Flyer SRAM
	move.l	#BITBUFSIZE,d0
	bsr	FastCopy

	IFD	OPTVER_020
	bra.s	.merge

.copy020
	lea.l	vt_BitBuffer(a5),a0	;From BitBuffer
	move.l	#VTASCSRAMADDR+BITBUFSIZE,a1	;To SRAM
	adda.l	vt_SRAMbase(a5),a1	;Offset to Flyer SRAM (end of area)
	move.w	#BITBUFSIZE-1,d0
.rvscopy
	move.b	(a0)+,-(a1)
	dbf	d0,.rvscopy
.merge
	ENDC

***** DMA from SRAM to DRAM (special area)
	move.l	#VTASCSRAM,a0	;CPU RAM
	move.l	#VTASCDRAM,a1	;DMA RAM
	moveq.l	#0,d0		;Flyer board
	move.w	#BITBUFBLKS,d1
	moveq.l	#1,d2		;Read from CPU RAM
	bsr	CPUDMA		;Move from SRAM

***** Write to drive
	lea.l	MyAction(pc),a0
	move.l	#VTASCDRAM,d1
	moveq.l	#BITBUFBLKS,d0
	move.l	vt_lba(a5),d2
	move.l	d2,d3
	add.l	d0,d3			;Compute lba for next time
	cmp.l	vt_lbastop(a5),d3	;Fits within limits for field?
	bls.s	.doesfit

	sub.l	vt_lbastop(a5),d3	;Trim amount
	sub.l	d3,d0			;New size
	move.l	vt_lbastop(a5),d3
	move.l	d3,vt_lba(a5)		;Up against the limit
	bsr	Write10
	moveq.l	#FERR_FULL,d0		;Show that it didn't fit!
	move.b	d0,vt_Error(a5)		;Log error to be handled later
	bra.s	.exit

.doesfit
	move.l	d3,vt_lba(a5)		;Ready for next time
	bsr	Write10

.exit
	movem.l	(sp)+,d0-d3/a0-a2/a6
	rts

*********************************************************************
* CodeMagSize -- Determine magnitude of code in d0, return bit size
*
* Entry:
*	d0:input code (.b)
*	a5:VTASC ctrl
*
* Exit:
*	d0:code size (.w)
*********************************************************************
CodeMagSize:
	and.w	#$00FF,d0
	move.b	CodeMagTable(pc,d0.w),d0
	rts


CodeMagTable
	dc.b	1,2,3,3,4,4,4,4,5,5,5,5,5,5,5,5	;0
	dc.b	6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6	;1
	dc.b	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7	;2
	dc.b	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7	;3
	dc.b	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8	;4
	dc.b	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8	;5
	dc.b	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8	;6
	dc.b	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8	;7
	dc.b	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8	;8
	dc.b	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8	;9
	dc.b	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8	;A
	dc.b	8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8	;B
	dc.b	8,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7	;C
	dc.b	7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7	;D
	dc.b	7,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6	;E
	dc.b	6,5,5,5,5,5,5,5,5,4,4,4,4,3,3,2	;F



*********************************************************************
* FIRfilterLine -- Do an FIR filter on the provided line using
*	the parameters provided
*
* Entry:
*	d0:Advance flag
*	a1:Coef ptr
*	a5:VTASC ctrl
*	a6:ptr to line
*
* Exit:
*********************************************************************
FIRfilterLine:
	btst	#FWB_NOFIR,vt_ModeFlags(a5)	;No FIR wanted?
	beq.s	.yesFIR
	rts
.yesFIR
	link	a4,#-PIXELSPERLINE
	movem.l	d1-d7/a0/a2,-(sp)
	CLEAR	d2
	move.w	vt_fscale(a5),d2	;Scale
	lsr.l	#8,d2			;Use integral part scale only

	lea.l	-PIXELSPERLINE(a4),a2	;Temp line buffer

	IFD	DEBUGVTLOTS
	DUMPHEXI.L <FIR @>,a6,<\>
	ENDC

**** If advance then range = x-7 to x+8, advancing output by 1/2 sample
**** Otherwise,      range = x-8 to x+7, retarding output by 1/2 sample
	moveq.l	#0,d4
	tst.b	d0		;Advance flag
	bne.s	.noretard
	subq.l	#1,d4
.noretard

	move.w	#PIXELSPERLINE-1,d3
.xloop
**** Calculate one value
	moveq.l	#0,d5		;sum = 0
	moveq.l	#8-1,d6		;d=7..0
.dloop
**** Sum from small values to large values

**** If advance then x+(1..8)
**** Otherwise       x+(0..7)
	move.w	d3,d1
	add.w	d6,d1
	add.w	d4,d1
	addq.l	#1,d1		;d1 = x+(d+1)+adv

	moveq.l	#0,d7

**** Clip to right end of line
	cmp.w	#PIXELSPERLINE,d1
	bcs.s	.noclipright
;	move.b	#BORDERVALUE,d7
	move.b	#NTSC_BLANK,d7
	bra.s	.pastright
.noclipright
	move.b	0(a6,d1.w),d7
.pastright

**** If advance then x-(0..7)
**** Otherwise       x-(1..8)
	move.w	d3,d1
	sub.w	d6,d1
	add.w	d4,d1		;d1 = x-(d+1)+adv+1

	moveq.l	#0,d0

**** Clip to left end of line
	tst.w	d1
	bpl.s	.noclipleft
;	move.b	#BORDERVALUE,d0
	move.b	#NTSC_BLANK,d0
	bra.s	.pastleft
.noclipleft
	move.b	0(a6,d1.w),d0
.pastleft

**** Multiply and accumulate (Harris filter must be symmetrical)
	add.w	d7,d0
	moveq.l	#7,d1
	sub.w	d6,d1
	asl.w	#1,d1
	move.w	0(a1,d1.w),d1
	muls.w	d1,d0
	add.l	d0,d5

	dbf	d6,.dloop

	asr.l	#2,d5		;Divide / 4 (make fit in word)
	muls.w	d2,d5		;Scale the accumulated sum
	asr.l	#7,d5		;Divide / 128 (/512 total)

**** Clip result to 0-FF
	cmp.w	#$FF,d5
	bls.s	.noclip
	move.b	#$FF,d5
.noclip
	move.b	d5,0(a2,d3.w)	;Store off result in temp buffer

	dbf	d3,.xloop

**** Copy temp buffer back to source buffer
	move.l	a6,a0		;Line data ptr
	move.w	#(PIXELSPERLINE/4)-1,d0
.copyloop
	move.l	(a2)+,(a0)+
	dbf	d0,.copyloop

	movem.l	(sp)+,d1-d7/a0/a2
	unlk	a4
	rts



*****************************************************************************
* FillLineActive -- Fill line buffer with fill color
* FillLineWide   -- Fill a wide line buffer with fill color (center 752 only)
*
* Entry:
*	a5:VTASC ctrl structure
*	a0:ptr to line of data
*
* Exit:
*********************************************************************
FillLineWide
	movem.l	d0-d4/a0,-(sp)
	lea.l	4(a0),a0		;Skip past pad pixels
	bra.s	fullfill
FillLineActive
	movem.l	d0-d4/a0,-(sp)
fullfill
	move.b	vt_FillY(a5),d1		;Y
	move.b	vt_FillI(a5),d2		;I
	move.b	vt_FillQ(a5),d3		;Q

	move.w	vt_y(a5),d0
	btst	#0,d0			;Odd line?
	beq.s	.evenline
	neg.b	d2			;If so, flip subcarrier phase
	neg.b	d3
.evenline

* If full NTSC, skip this adjustment for fields II and IV
* This is needed because we throw away the half lines
	move.b	vt_field(a5),d0
	btst	#0,d0			;field 2/4?
	beq.s	.flds13
	neg.b	d2			;If so, flip subcarrier phase
	neg.b	d3
.flds13

	move.b	vt_field(a5),d0
	and.b	#3,d0			;Modulo 4
	cmp.b	#2,d0			;field 3/4?
	bcs.s	.firstpair
	neg.b	d2			;If so, flip subcarrier phase
	neg.b	d3
.firstpair

	move.b	d1,d0
	sub.b	d2,d0	;y-i

	lsl.w	#8,d0

	move.b	d1,d0
	sub.b	d3,d0	;y-q

	swap	d0

	move.b	d1,d0
	add.b	d2,d0	;y+i

	lsl.w	#8,d0

	move.b	d1,d0
	add.b	d3,d0	;y+q

	IFD	DEBUGVTASC
	DUMPHEXI.L <Fill quad = >,d0,<\>
	ENDC

	move.w	#(SAMPLESPERLINE/4)-1,d1
.xloop
	move.l	d0,(a0)+		;Write a pixel-quad at a time
	dbf	d1,.xloop

	movem.l	(sp)+,d0-d4/a0
	rts



*********************************************************************
* CompLine -- Compress a line of composite video using VTASC
*
* Entry:
*	a5:VTASC ctrl structure
*	a0:ptr to line of data (wide760)
*
* Exit:
*	d0:Error code (FERR_OKAY or FERR_FULL)
*********************************************************************
* Register usage:
*  d0 				a0 
*  d1 error/dither		a1 lineptr
*  d2 t7/rndnum			a2 escapecodes table
*  d3 guess/c7/rndcntr		a3
*  d4 tolmag			a4 up2ptr
*  d5 nextsize			a5 vt
*  d6 x				a6 fixed lineptr
*  d7 msize
*********************************************************************
CompLine
	IFD	DEBUGVTLOTS
	DUMPHEXI.L <CompLine>,a0,<\>
	ENDC

	movem.l	d1-d7/a1-a2/a4-a6,-(sp)
	move.l	a0,a6			;Ptr to line of data

****** Strip off/pad 4 pixels on each edge to fit NTSC pixels into wide760
	move.l	a0,a1
	move.b	#NTSC_BLACK,d0
	move.b	d0,(a1)+		;Pad w/ 4 pixels of black on left edge
	move.b	d0,(a1)+
	move.b	d0,(a1)+
	move.b	d0,(a1)+
	lea.l	SAMPLESPERLINE(a1),a1	;Skip past active pixels
	move.b	d0,(a1)+		;Pad w/ 4 pixels of black on right edge
	move.b	d0,(a1)+
	move.b	d0,(a1)+
	move.b	d0,(a1)+

	IFD	DEBUGVTASC
	DUMPHEX	<# Pixels #>,(a6),#16	;32
	ENDC

	move.w	vt_msize(a5),d7		;Keep in reg for fast access

****** Do FIR filter, advancing by 1/2 sample
	cmp.w	#0,a6
	beq	.noFIR
	moveq.l	#1,d0			;Advance
	lea.l	vt_Coefs(a5),a1
	bsr	FIRfilterLine		;Do it!
.noFIR


****** Add fixed size PRN to line, at frequency indicated by RndFreq
	cmp.w	#0,a6
	beq	.noRnd
	move.b	vt_RndDither(a5),d1
	beq	.noRnd

	move.b	vt_RndNum(a5),d2	;Get fast working copies
	move.b	vt_rndcntr(a5),d3

;	IFD	DEBUGVTTEST
;	DUMPHEXI.B <(RndNum = >,d2,<)\>
;	ENDC

	CLEAR	d6			;x=0
	move.l	a6,a1
.xloop2
	add.b	d2,(a1)+		;Add in rnd value
	cmp.b	vt_RndFreq(a5),d3	;Time for a new value?
	bne.s	.keepsame

	move.b	d1,d0			;Size of random value
	bsr	GenRndNum
	move.b	d0,d2

	CLEAR	d3			;Reset freq counter
.keepsame
	addq.l	#1,d3

	addq.l	#1,d6			;x++
	cmp.w	#PIXELSPERLINE,d6
	bcs.s	.xloop2

	IFD	DEBUGVTASC
	DUMPHEX	<# +noise #>,(a6),#16	;32
	ENDC

	move.b	d2,vt_RndNum(a5)	;Store off for later
	move.b	d3,vt_rndcntr(a5)	;Store off for later
.noRnd


***** Now compress using VTASC algorithm
	CLEAR	d0
	move.b	vt_topline(a5),d0
	subq.l	#1,d0			;Which line
	move.w	#PIXELSPERLINE,d1	;Make index
	mulu.w	d1,d0
	lea.l	vt_Lines(a5),a4
	add.l	d0,a4			;Ptr to "up2" buffer

	lea.l	EscapeCodes(pc),a2
	move.w	vt_tolmag(a5),d4	;Keep tolmag here

	CLEAR	d6			;x=0
	move.l	a6,a1
.xloop1
****** Is this pixel on screen?  Need this due to SN overrun
	cmp.w	#PIXELSPERLINE,d6	;X on screen?
	bhs	.xend
	move.w	vt_y(a5),d0
	cmp.w	#LINESPERFIELD,d0	;Y on screen?
	bhs	.xend

****** Is this an active pixel?
	tst.b	vt_SNflag(a5)		;Sub-nyquist?
	beq.s	.isactive
	move.w	vt_y(a5),d0
	eor.b	d6,d0

;If Full NTSC flag is true, skip this step, which is needed because
;we skip the half lines on fields 2 & 4
	move.b	vt_field(a5),d2
	eor.b	d2,d0

	lsr.b	d0
	bcc.s	.isactive
					;Not an active SN pixel, skip
	;Dont really need to keep this!
	move.b	(a1)+,d3		;Get inactive pixel
	move.b	d3,(a4)+		;Copy to "2 up" buffer
	bra	.xend

.isactive

******                t5      t7  t8   (top line of kernal)
******
******            m4      m6      m8   (middle line of kernal)
******
******            c4  c5  c6  x         (bottom line of kernal)
******
****** Predictor value for x is calculated as follows:
****** Old D2:		guess = c4 - c5 + c6
****** New D2:		guess = t7 + preverr
****** Sub-nyquist:	guess = t7 + preverr

****** Get sample 2 lines above this pixel
	move.b	(a4),d2			;t7 = *up2ptr

****** Use proper predictor to make pixel guess
	tst.b	vt_pcoder(a5)
	bne.s	.neweralgo
	move.b	vt_C4(a5),d3
	sub.b	vt_C5(a5),d3
	add.b	vt_C6(a5),d3		;c4 - c5 + c6
	bra.s	.haveguess
.neweralgo
	move.b	vt_preverr(a5),d3
	add.b	d2,d3			;t7 + vt_preverr
.haveguess

****** Calculate error (prediction from actual)
	move.b	(a1)+,d1		;Get next pixel
	sub.b	d3,d1			;Error amount
;	ext.w	d1
;	ext.l	d1			;Extend error to signed long
;	divs.w	d4,d1			;Scaled error
	asr.b	d4,d1			;Divide by power of 2

;	ext.w	d1			;Extend signed byte to signed word
;	muls.w	d1,d0			;(d0: tolmag)
	move.b	d1,d0
	lsl.b	d4,d0			;Mult by power of 2

	add.b	d0,d3			;c7 = guess + (error * tolmag)

	move.b	d3,d0
	sub.b	d2,d0
	move.b	d0,vt_preverr(a5)	;preverr = c7 - t7

***** Calculate code size needed to represent the error magnitude
	move.b	d1,d0
	bsr	CodeMagSize		;Measure size of code
	move.w	d0,d5			;Size needed

***** Output escape codes as needed to make big enough code for error
	move.w	d1,-(sp)
.escloop
	cmp.w	d5,d7			;Big enough already?
	bcc.s	.willfit
	move.b	0(a2,d7.w),d0		;Get escape code
	move.b	d7,d1			;Size of ESC
	bsr	OutputCode		;Make ESC code
	addq.w	#1,d7			;Up 1 size now
	bra.s	.escloop
.willfit
	move.w	(sp)+,d1

****** Now output error code in current or increased size
	move.b	d1,d0			;Error value
	move.b	d7,d1			;Size of code
	bsr	OutputCode		;Send to disk

	move.w	d5,d7		;Use 'nextsize' next time

****** Store new pixel back in line buffer for 2 lines from now
	move.b	d3,(a4)+

****** For old D2 coding only
	move.b	vt_C5(a5),vt_C4(a5)
	move.b	vt_C6(a5),vt_C5(a5)
	move.b	d3,vt_C6(a5)

.xend
	addq.l	#1,d6			;x++
	cmp.w	#PIXELSPERLINE,d6
	bcs	.xloop1

****** Toggle previous line buffers
	move.b	vt_topline(a5),d0
	move.b	vt_extraline(a5),vt_topline(a5)
	move.b	d0,vt_extraline(a5)

	add.w	#1,vt_y(a5)		;Y++
	move.w	d7,vt_msize(a5)		;Store away for later

	CLEAR	d0
	move.b	vt_Error(a5),d0		;Get error code

;	IFD	DEBUGVTLOTS
	IFD	DEBUGVTTEST
	DUMPHEXI.B <CompLine returning >,d0,<\>
	ENDC

	movem.l	(sp)+,d1-d7/a1-a2/a4-a6
	rts


*********************************************************************
* DecompLine -- Decompress a line of composite video using VTASC
*
* Entry:
*	a5:VTASC ctrl structure
*	a0:ptr to line of data (wide760)  (or 0 to not collect data)
*
* Exit:
*********************************************************************
* Register usage:
*  d0 				a0 down1ptr
*  d1 error/dither		a1 lineptr
*  d2 t7/rndnum			a2 escapecodes table
*  d3 guess/c7/rndcntr		a3 signextend tables
*  d4 tolmag			a4 up1ptr
*  d5 				a5 vt
*  d6 x				a6 fixed lineptr
*  d7 msize
*********************************************************************
DecompLine
	movem.l	d0-d4/d6-d7/a0-a4/a6,-(sp)
	move.l	a0,a6			;Ptr to line of data
	move.w	vt_msize(a5),d7		;Keep in reg for fast access

	CLEAR	d0
	move.b	vt_topline(a5),d0
	subq.l	#1,d0			;Which line
	move.w	#PIXELSPERLINE,d1	;Make index
	mulu.w	d1,d0
	lea.l	vt_Lines(a5),a4
	add.l	d0,a4			;Ptr to "up1" buffer

	CLEAR	d0
	move.b	vt_extraline(a5),d0
	subq.l	#1,d0			;Which line
	move.w	#PIXELSPERLINE,d1	;Make index
	mulu.w	d1,d0
	lea.l	vt_Lines(a5),a0
	add.l	d0,a0			;Ptr to "down1" buffer

	lea.l	EscapeCodes(pc),a2
	lea.l	SgnExtBits(pc),a3
	move.w	vt_tolmag(a5),d4	;Keep tolmag here

	CLEAR	d6			;x=0
	move.l	a6,a1
.xloop1
****** Special Sub-Nyquist stuff
	tst.b	vt_SNflag(a5)		;Sub-nyquist?
	beq	.getcode		;No, just get next pixel code

****** Is this an active SN pixel?
	move.w	vt_y(a5),d0
	eor.b	d6,d0			;x XOR y

;If Full NTSC flag is true, skip this step, which is needed because
;we skip the half lines on fields 2 & 4
	move.b	vt_field(a5),d2
	eor.b	d2,d0

	lsr.b	d0
	bcc.s	.isactive

****** Handle inactive pixels
	cmp.w	#PIXELSPERLINE,d6	;X past end?
	beq	.xend			;Skip extra pixel since inactive

	move.w	vt_y(a5),d0
	cmp.w	#LINESPERFIELD-1,d0	;Building last line in field?
	bne	.getcode		;If not, be fetching next line
	move.b	#BORDERVALUE,d3		;Otherwise, don't decode pixels...
	bra	.SNalgo			; ...just use BORDER values for line

****** Active SN pixel -- Get from line below, regenerate pixel to left
.isactive
	CLEAR	d3		;(will need sample.w)
	move.b	#BORDERVALUE,d3
	cmp.w	#PIXELSPERLINE,d6	;X past end?
	bne.s	.normregen		;No, do this pixel and previous

					;Yes, just to previous pixel
	cmpa.w	#0,a1			;User provided buffer?
	beq.s	.tossSN			;If not, nothing to do for this pixel
	bra.s	.justregen		;Dont store this pixel, just regen left

.normregen
	move.b	(a4),d3		;Get pixel (read last line)
	move.b	d3,(a0)		;Keep in 'up1' line
	cmpa.w	#0,a1		;User provided buffer?
	beq.s	.tossSN

	move.b	d3,(a1)		;If so, place pixel in buffer

	tst.w	d6		;Only regen pixel to left if x > 0
	beq.s	.noRegen
.justregen
	CLEAR	d1		;(will need left.w)
	cmp.w	#1,d6		;x = 1?
	beq.s	.xis1
	move.b	-2(a1),d1	;Pickup pixel to left of missing pixel
	bra.s	.gotleft
.xis1	move.b	#BORDERVALUE,d1	;If no left pixel, use border
.gotleft
	add.w	d3,d1		;2Y (9 bits) from pixel to left/right
	CLEAR	d2
	move.b	-3(a4),d2	;Get pixel above and left 2
	sub.b	-1(a4),d2	;Get 2C from line above
	bcc.s	.notneg1
	ext.w	d2		;Sign extend to -word
.notneg1
	CLEAR	d0
	move.b	-3(a0),d0	;Get pixel below and left 2
	sub.b	-1(a0),d0	;Get 2C from line below
	bcc.s	.notneg2
	ext.w	d0		;Sign extend to -word
.notneg2
	add.w	d2,d0		;4C from lines above/below
	asr.w	#1,d0		;/2 to get 2C
	add.w	d1,d0		;2Y + 2C
	asr.w	#1,d0		;/2

	cmp.w	#$0100,d0	;Under/Overflow?
	bcs.s	.regenokay
	lsr.w	#1,d1		;Y2/2 (avg the luma of the L/R pixels)
	move.b	d1,d0
.regenokay
	move.b	d0,-1(a1)	;Store regenerated previous pixel	
.noRegen
	addq.l	#1,a1		;Bump ptr to user buffer
.tossSN
	addq.l	#1,a4		;Bump ptr to up1
	addq.l	#1,a0		;Bump ptr to down1
	bra	.xend

****** Is this pixel on screen?  Need this due to SN overrun
	cmp.w	#PIXELSPERLINE,d6	;X on screen?
	bhs	.xend
;	move.w	vt_y(a5),d0
;	cmp.w	#LINESPERFIELD,d0	;Y on screen?
;	bhs	.xend

****** Get next error code (maybe adjust msize upward)
.getcode
	move.b	d7,d0			;Size of code needed
	bsr	InputCode		;Get it
	cmp.b	0(a2,d7.w),d0		;An escape code?
	bne.s	.goodcode
	addq.w	#1,d7			;Up 1 size
	bra.s	.getcode
.goodcode
	move.b	d0,d1			;Keep code here

****** Sign-extend error value
	move.b	0(a3,d7.w),d0
	btst	d0,d1			;Is this error code negative?
	beq.s	.ispositive
	add.b	SgnExtVals-SgnExtBits(a3,d7.w),d1	;Sign-extend it
.ispositive

	move.b	d1,d0
	bsr	CodeMagSize		;Measure size of code
	move.w	d0,d7			;Use as next size


******                t5      t7  t8   (top line of kernal)
******
******            m4      m6      m8   (middle line of kernal)
******
******            c4  c5  c6  x         (bottom line of kernal)
******
****** Predictor value for x is calculated as follows:
****** Old D2:		guess = c4 - c5 + c6
****** New D2:		guess = t7 + preverr
****** Sub-nyquist:	guess = t7 + preverr

****** Get sample 2 lines above this pixel
	move.b	(a4),d2			;t7 = *up1ptr

****** Use proper predictor to make pixel guess
	tst.b	vt_pcoder(a5)
	bne.s	.neweralgo
	move.b	vt_C4(a5),d3
	sub.b	vt_C5(a5),d3
	add.b	vt_C6(a5),d3		;c4 - c5 + c6
	bra.s	.haveguess
.neweralgo
	move.b	vt_preverr(a5),d3
	add.b	d2,d3			;t7 + vt_preverr
.haveguess

;	ext.w	d1
;	move.w	vt_tolmag(a5),d0
;	muls.w	d1,d0
	lsl.b	d4,d1

	add.b	d1,d3			;c7 = guess + (error * tolmag)

	move.b	d3,d0
	sub.b	d2,d0
	move.b	d0,vt_preverr(a5)	;preverr = c7 - t7

****** Determine what to do with new pixel, depending on compression mode
	tst.b	vt_SNflag(a5)		;SN or D2 modes?
	beq.s	.D2algo

************************ Sub-Nyquist pixel handling **********************
.SNalgo
****** Store decoded pixel in 'down1' buffer (for me to use next time)
	move.b	d3,(a0)+		;Store in next line
	addq.l	#1,a4			;Don't touch 'up1' data here
	cmpa.w	#0,a1			;User provided buffer?
	beq.s	.staynull
	addq.l	#1,a1			;If so, skip this pixel
.staynull
	bra.s	.xend

************************ D2 (old/new) pixel handling **********************
.D2algo
****** Put pixel in caller's buffer, if provided
	cmpa.w	#0,a1
	beq.s	.tossdata
	move.b	d3,(a1)+		;Place pixel in buffer
.tossdata

****** Store new pixel back in line buffer for 2 lines from now
	move.b	d3,(a4)+

****** For old D2 coding only
	move.b	vt_C5(a5),vt_C4(a5)
	move.b	vt_C6(a5),vt_C5(a5)
	move.b	d3,vt_C6(a5)


.xend
	addq.l	#1,d6			;x++
	cmp.w	#PIXELSPERLINE,d6
	bcs	.xloop1			;If still on line, keep going
	bne.s	.waydone
	tst.b	vt_SNflag(a5)		;If Sub-nyquist, do 1 extra
	bne	.xloop1			;to regen last pixel on line
.waydone


****** Subtract fixed size PRN from line, at frequency indicated by RndFreq
	cmp.w	#0,a6
	beq	.noRnd
	move.b	vt_RndDither(a5),d1
	beq	.noRnd

	move.b	vt_RndNum(a5),d2
	move.b	vt_rndcntr(a5),d3

;	IFD	DEBUGVTTEST
;	DUMPHEXI.B <(RndNum = >,d2,<)\>
;	ENDC

	CLEAR	d6			;x=0
	move.l	a6,a1
.xloop2
	sub.b	d2,(a1)+		;Subtract out rnd value
	cmp.b	vt_RndFreq(a5),d3	;Time for a new value?
	bne.s	.keepsame

	move.b	d1,d0			;Size of random value
	bsr	GenRndNum
	move.b	d0,d2

	CLEAR	d3			;Reset freq counter
;	bra.s	.freqplus1		;(Rnd frequency is value+1)
.keepsame
	addq.l	#1,d3
;.freqplus1
	addq.l	#1,d6			;x++
	cmp.w	#PIXELSPERLINE,d6
	bcs.s	.xloop2

	move.b	d2,vt_RndNum(a5)	;Store off for later
	move.b	d3,vt_rndcntr(a5)	;Store off for later
.noRnd


****** Do FIR (inverse) filter, delaying by 1/2 sample
	cmp.w	#0,a6
	beq	.noFIR
	moveq.l	#0,d0			;No advance (retard)
	lea.l	vt_Coefs(a5),a1
	bsr	FIRfilterLine		;Do it!
.noFIR

****** Toggle previous line buffers
	move.b	vt_topline(a5),d0
	move.b	vt_extraline(a5),vt_topline(a5)
	move.b	d0,vt_extraline(a5)

	add.w	#1,vt_y(a5)		;Y++
	move.w	d7,vt_msize(a5)		;Store away for later

	movem.l	(sp)+,d0-d4/d6-d7/a0-a4/a6
	rts




*********************************************************************
* FindSRAMbase -- Locate base of Flyer's SRAM, store in vt_SRAMbase
*
* Entry:
*	a5:VTASC ctrl structure
*	a6:Flyerbase (library)
*
* Exit:
*********************************************************************
FindSRAMbase
	movem.l	d0/a4-a5,-(sp)
	move.l	a5,a4

	moveq.l	#0,d0			;Hard coded: Flyer # 0!!!
	bsr	GetCardUnit		;Get unit structure for card 'd0'
	bne	.exit			;Failed? Exit

	move.l	unit_SRAMbase(a5),d0	;Get from unit structure
	move.l	d0,vt_SRAMbase(a4)	;Store in Vtasc structure

	IFD	DEBUGVTLOTS
	DUMPHEXI.L <Flyer SRAM found at >,d0,<\>
	ENDC
.exit
	movem.l	(sp)+,d0/a4-a5
	rts


	IFNE	SERDEBUG
	HXDUMP
	HDUMP
	ENDC

EscapeCodes
	dc.b	$00	;00000000
	dc.b	$01	;00000001
	dc.b	$02	;00000010
	dc.b	$04	;00000100
	dc.b	$08	;00001000
	dc.b	$10	;00010000
	dc.b	$20	;00100000
	dc.b	$40	;01000000
	dc.b	$80	;10000000

SgnExtBits		;Array for detecting sign of various size codes
	dc.b	0
	dc.b	0
	dc.b	1
	dc.b	2
	dc.b	3
	dc.b	4
	dc.b	5
	dc.b	6
	dc.b	7

SgnExtVals		;Array for sign-extending various size codes
	dc.b	0
	dc.b	0
	dc.b	$FC	;11111100
	dc.b	$F8	;11111000
	dc.b	$F0	;11110000
	dc.b	$E0	;11100000
	dc.b	$C0	;11000000
	dc.b	$80	;10000000
	dc.b	$00	;00000000


	cnop	0,2
MyAction	ds.b	CA_sizeof

	cnop	0,2
MyVolume	ds.b	FV_sizeof

	cnop	0,2
MyVidComp	ds.b	VCI_sizeof

	cnop	0,2
VtascCtrl	ds.b	VTASCinfo_Sizeof

EndCode:

	END
