/********************************************************************
* $flyerstream.c$
* $Id: flyerstream.c,v 5.4 1995/02/09 17:01:13 pfrench Exp $
* $Log: flyerstream.c,v $
*Revision 5.4  1995/02/09  17:01:13  pfrench
*Removed total kbytes transferred, now goes to scsistream
*
*Revision 5.3  1995/02/01  15:23:57  pfrench
*Now maintains total bytes communicated
*
*Revision 5.2  1995/01/26  09:15:36  pfrench
*Fixed same drive number check where zero would fail
*
*Revision 5.1  1995/01/17  19:32:05  pfrench
*Now correctly computes the scsidrive for flyer-flyer calls
*
*Revision 5.0  1995/01/13  21:47:44  pfrench
*FirstCheckIn
*
*
* Copyright (c)1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved.
*
*********************************************************************/
#include <string.h>
#include <exec/types.h>
#include <exec/memory.h>

#ifndef	EXEC_IO_H
#include <exec/io.h>
#endif

#ifndef	CLASSES_H
#include "classes.h"
#endif

#ifndef	CLASSBASE_H
#include "classbase.h"
#endif

#ifndef	PROOF_LIB_H
#include <proof_lib.h>
#endif

#ifndef	PROOF_STREAM_H
#include "stream.h"
#endif

#ifndef	INC_FLYER_H
#include "flyer.h"
#endif

#ifndef	FLYERSTREAM_H
#include "flyerstream.h"
#endif

#include "flyerlib.h"

#include <clib/exec_protos.h>
#include <pragmas/exec_pragmas.h>

#include <clib/alib_protos.h>

#include <clib/alib_stdio_protos.h>

#ifndef PROTO_PASS
#include <proto.h>
#endif

struct Class *flyerstream_AddClass( void );

/**********************************************************/

extern struct Library	*FlyerBase;
extern struct Library	*ProofBase;
extern struct ClassBase	*ClassBase;

static ULONG __saveds __asm fs_Dispatch(
	REG(a0) struct Class *cl,
	REG(a2) APTR ob,
	REG(a1) Msg msg );

struct FlyerStreamData
{
	struct FlyerVolume	 fs_fv;
};

/**********************************************************/

struct Class *flyerstream_AddClass( void )
{
	struct Class *cl;

	if ( cl = cl_MakeClass( FLYERSTMCLASS,
									SCSISTMCLASS, NULL,
									sizeof(struct FlyerStreamData), 0) )
	{
		cl->cl_Dispatch.h_Entry = (HOOKFUNC) fs_Dispatch;

		if ( !pb_AddClass(cl) )
		{
			cl_FreeClass(cl);
			cl = NULL;
		}
	}

	return(cl);
}

static ULONG __saveds __asm fs_Dispatch(
	REG(a0) struct Class *cl,
	REG(a2) APTR ob,
	REG(a1) Msg msg )
{
	ULONG					 retval = 0;

	switch ( msg->MethodID )
	{
		case OM_NEW:
		{
			if ( ob = (APTR)ob_DoSuperMethodA(cl,ob,msg) )
			{
				struct FlyerStreamData	*fs = (struct FlyerStreamData *)INST_DATA(cl,ob);
				struct opSet				*ops = (struct opSet *)msg;
				LONG							 drivenum;

				drivenum = (LONG)
					ti_GetTagData(SCSISTMA_Unit,(ULONG)-1,ops->ops_Tags);

				if ( drivenum >= 0 )
				{
					fs->fs_fv.SCSIdrive = ((drivenum / 100) * 8) + (drivenum % 10);

					fs->fs_fv.Flags = FVF_USENUMS;

					retval = (ULONG)ob;
				}

				if ( !retval )
				{
					ob_CoerceMethod(cl,ob,OM_DISPOSE);
				}
			}
		}
		break;

		case OM_DISPOSE:
		{
			// struct FlyerStreamData	*fs = (struct FlyerStreamData *)INST_DATA(cl,ob);

			retval = ob_DoSuperMethodA(cl,ob,msg);
		}
		break;

		case FLYSTMM_WriteBlocks:
		{
			struct FlyerStreamData	*fs = (struct FlyerStreamData *)INST_DATA(cl,ob);
			struct opFSWriteBlocks	*fsw = (struct opFSWriteBlocks *)msg;

			// High-speed transfer of blocks from flyer drive to tape
			// via flyer's internal SCSI bus.

			/* Flush any buffered information before doing transfer */
			if ( !(retval = ob_DoMethod(ob,STREAMM_Cleanup)) )
			{
				retval = CopyData( fsw->fsw_fv, &fs->fs_fv,
								fsw->fsw_block, fsw->fsw_len, -1 );

				if ( !retval )
				{
					ULONG			kbytes = 0;

					ob_GetSuperAttrs( cl,ob,
						SCSISTMA_TotalKbytes, &kbytes,
						TAG_DONE );

					/* Round up */
					kbytes += (ULONG) fsw->fsw_len / 2;
					if ( fsw->fsw_len & 0x01 )
						kbytes++;

					ob_SetSuperAttrs( cl,ob,
						SCSISTMA_TotalKbytes, kbytes,
						TAG_DONE );
				}
			}
		}
		break;

		case FLYSTMM_ReadBlocks:
		{
			struct FlyerStreamData	*fs = (struct FlyerStreamData *)INST_DATA(cl,ob);
			struct opFSWriteBlocks	*fsw = (struct opFSWriteBlocks *)msg;

			// High-speed transfer of blocks from tape to flyer drive
			// via flyer's internal SCSI bus.

			/* Flush any buffered information before doing transfer */
			if ( !(retval = ob_DoMethod(ob,STREAMM_Cleanup)) )
			{
				retval = CopyData( &fs->fs_fv, fsw->fsw_fv,
								-1, fsw->fsw_len, fsw->fsw_block );

				if ( !retval )
				{
					ULONG			kbytes = 0;

					ob_GetSuperAttrs( cl,ob,
						SCSISTMA_TotalKbytes, &kbytes,
						TAG_DONE );

					/* Round up */
					kbytes += (ULONG) fsw->fsw_len / 2;
					if ( fsw->fsw_len & 0x01 )
						kbytes++;

					ob_SetSuperAttrs( cl,ob,
						SCSISTMA_TotalKbytes, kbytes,
						TAG_DONE );
				}
			}
		}
		break;

		default:
			retval = ob_DoSuperMethodA(cl,ob,msg);
		break;
	}

	return(retval);
}
