/*********************************************************************\
*
* FSViewer - View Flyer File system on a drive
*
* Copyright (c) 1994,1995,1996,1997 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* 02-05-97      Hayes    Created original File
*
\*********************************************************************/

/* INCLUDES */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <clib/dos_protos.h>
#include <clib/exec_protos.h>
#include <clib/gadtools_protos.h>
#include <clib/graphics_protos.h>
#include <clib/intuition_protos.h>
#include <clib/asl_protos.h>
#include <dos/dos.h>
#include <exec/types.h>
#include <exec/memory.h>
#include <intuition/gadgetclass.h>
#include <intuition/intuition.h>
#include <libraries/gadtools.h>
#include <libraries/asl.h>
#include <proto/exec.h>

#include "flyerlib.h"
#include "flyer.h"
#include "flyerFS.h"
#include "intfsprivate.h"
/* DEFINES */

#define FlyerBase FlyerBase   // This will cause a warning that can be ignored

#define	DMABLK	0xA00
#define	RAMBLK	0x20		// Use EC4000 - ED4000

#define Debugging 0
#define DBUG(x)	if (Debugging) { x }
#define DOUT(x)   { printf( x ); fflush( NULL ); }
#define DOUT2(x, y)   { printf( x, y ); fflush( NULL ); }
#define RAW_INDEX			0
#define ROOT_INDEX 		1
#define DIR_INDEX 		2
#define FILE_INDEX 		3
#define MAPBLOCK_INDEX	4
#define FSHEADS_INDEX 	5
#define MAX_INDEX			5

/*
 * These two defines set up and clear the WaitPointer...
 */
#define	SetWait(x)	SetPointer(x,WaitPointer,16L,16L,-6L,0L)
#define	ClearWait(x)	ClearPointer(x)

/***** Gadget IDs for all Gadgets *****/
#define GAD_VIEWAS		( 0)
#define GAD_BLOCKNUM 	( 1)
#define GAD_JUMP			( 2)
#define GAD_LISTVIEW		( 3)
#define GAD_NEXT			( 4)
#define GAD_PREV			( 5)
#define GAD_STATUS		( 6)
#define GAD_VIEWREPORT	( 7)
#define GAD_NOOP			( 8)
#define GAD_SELNODE		( 9)
#define GAD_EXIT			(10)
#define MAX_GADS			(10)

/* STRUCTURES */

struct BlockFieldNode
{
	struct Node node;
	UBYTE			str[128];
	ULONG			value;
	UBYTE			address;  /* Boolean value... is value an address */
	UBYTE			block_type;
};

struct FSVData
{
	char  fsv_BlockString[32];
	ULONG fsv_BlockValue;
	int	fsv_ViewType;
	struct BlockFieldNode *fsv_SelectedNode;
	struct List *fsv_FieldList;
	ULONG fsv_DriveSize;
	struct FlyerVolume FlyVol;
	UBYTE fsv_BlockBuffer[512];
	int fsv_GadLib;
	char fsv_StatusStr[64];
	struct Window *fsv_StatusWin;
	struct Gadget **fsv_StatusGadList;
	char fsv_SelString[128];
};

/* FUNCTION PROTOS */
int InitFSVData( void );
void FSVWindow( void );
struct Gadget *CreateMainGadgets(struct Gadget **glistptr, 
											void *vi,
											struct Gadget *gadlist[]);
BOOL HandleGadgetEvent(struct Window *win, struct Gadget *gad, UWORD code,
								struct Gadget *gadlist[], UWORD menuID);
void GetNewBlockData( void );
void RefreshBlockDisplay( struct Window *win, struct Gadget *gadlist[] );
void CreateRootList( void );
void CreateDirList( void );
void CreateFileList( void );
void CreateMapBlockList( void );
void CreateFSHeadsList( void );
void CreateRawList( void );
struct BlockFieldNode *GetListNode( struct List *list, UWORD index );
void ProcessWindowEvents(struct Window *win, struct Gadget *gadlist[], struct Menu *menuStrip);
ULONG Str2ULONG(char *str);
ULONG Hex2ULONG(char *str);
float Str2Float(char *str);
ULONG SCSI_Read(ULONG lba, ULONG size);
void StatusMessage( char *msg );
UWORD Menu2ID(UWORD menu, UWORD item, UWORD sub, UWORD *code);
void PurgeMsgPort(struct Window *win);

/* GLOBALS */

USHORT __chip WaitPointer[36] =
{
	0x0000, 0x0000, 0x0400, 0x07C0, 0x0000, 0x07C0, 0x0100, 0x0380, 0x0000,
	0x07E0, 0x07C0, 0x1FF8, 0x1FF0, 0x3FEC, 0x3FF8, 0x7FDE, 0x3FF8, 0x7FBE,
	0x7FFC, 0xFF7F, 0x7EFC, 0xFFFF, 0x7FFC, 0xFFFF, 0x3FF8, 0x7FFE, 0x3FF8,
	0x7FFE, 0x1FF0, 0x3FFC, 0x07C0, 0x1FF8, 0x0000, 0x07E0, 0x0000, 0x0000
};

struct TextAttr Topaz80 = { "topaz.font", 8, 0, 0, };

struct Library *IntuitionBase = NULL;
struct Library *GfxBase = NULL;
struct Library *GadToolsBase = NULL;
struct Library *AslBase = NULL;
struct Library	*FlyerBase = NULL;

char *WindowTitle 	= "FlyerFS Viewer";
char *JumpLabel		= "Jump To Selected";
char *BlockLabel		= "Block Number:";
char *ViewTitle	  	= "View Block As:";
char *ViewLabels[] 	= {	"Raw",
									"ROOTHDR", 
									"DIRHDR", 
									"FILEHDR", 
									"MAPBLOCK", 
									"FSHEADSBLOCK", 
									NULL };
struct NewMenu FSVMenu[] =
{
	{NM_TITLE,	"Project",				0, 0, 0, 0,},
	{	NM_ITEM, "View FS Report",	 "V", 0, 0, 0,},
	{NM_END,		NULL,						0, 0, 0, 0,},
};

UWORD	topborder;

struct FSVData *fd;

/************** FUNCTION BODIES ***************/

/******************************************************\
*
*	FUNCTION:
*		void main( int argc, char *argv[])
*
*	SYNOPSIS:
*		Main function: opens all libraries and initializes 
*		data to start program
*
*	INPUT:
*		none
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void main( int argc, char *argv[])
{
	ULONG drive_num;
	
	if ( argc != 2 )
	{
		printf( "Written by Todd Hayes\nCopyright Newtek, Inc. 1996\n\nUsage: %s <drive>\n", argv[0]);
		return;
	}
	else
	{
		drive_num = Str2ULONG( argv[1] );
		if ( ( drive_num < 0 ) || ( drive_num > 23 ) )
		{
			printf("Error: Drive number must be in range 0 to 23\n");
			return;
		}
	}

	if (NULL == (IntuitionBase = OpenLibrary("intuition.library", 37L)))
		printf("Requires Version 37 intuition library");
	else
	{
		if (NULL == (GfxBase = OpenLibrary("graphics.library",37L)))
			printf("Requires Version 37 graphics library");
		else
		{
			if (GadToolsBase = OpenLibrary("gadtools.library",39L))
				fd->fsv_GadLib = 39;
			else
				if (GadToolsBase = OpenLibrary("gadtools.library", 37L))
				{
					fd->fsv_GadLib = 37;
				}
				else
					printf("Requires Version 37 gadtools library");
			if (fd->fsv_GadLib > 0)
			{
				if (NULL == (FlyerBase = OpenLibrary("flyer.library",0))) 
				{
					printf("Open flyer.library failed");
				}
				else
				{
					ULONG blklen;
					ULONG res;

					fd = ( struct FSVData * )AllocMem(  sizeof( struct FSVData ), 
																	MEMF_CLEAR | MEMF_ANY );
					if ( fd )
					{
						if ( !InitFSVData() )
						{
							fd->FlyVol.SCSIdrive = drive_num;
							/* Get drive size info... */	
							res = ReadSize(&(fd->FlyVol), &(fd->fsv_DriveSize), &blklen);
						 	if (res)
							{
								printf("ReadSize Error: %ld - %ls", res, Error2String(res));
							}
							else
							{
								FSVWindow();
							}
						}

						if ( fd->fsv_FieldList )
						{
							struct BlockFieldNode *current;

							while( !IsListEmpty( fd->fsv_FieldList ) )
							{	
								current = ( struct BlockFieldNode * )RemHead( fd->fsv_FieldList );
								FreeMem( current, sizeof( struct BlockFieldNode ) );
							}

							FreeMem( fd->fsv_FieldList, sizeof( struct List ) );
							fd->fsv_FieldList = NULL;
						}

						FreeMem( fd, sizeof( struct FSVData ) );
						fd = NULL;
					}
					CloseLibrary(FlyerBase);
				}
				CloseLibrary(GadToolsBase);
			}		
			CloseLibrary(GfxBase);
		}
		CloseLibrary(IntuitionBase);
	}
}

/******************************************************\
*
*	FUNCTION:
*		void InitFSVData( void )
*
*	SYNOPSIS:
*		Initializes data in the FSVData structure
*
*	INPUT:
*		none
*
*	RETURNS:
*		0 if successful, 1 if not.
*
*	NEEDS:
*
\******************************************************/
int InitFSVData( void )
{
	int result = 0;

	sprintf( fd->fsv_BlockString, "0x00000000");
	fd->fsv_BlockValue = 0L;
	fd->fsv_DriveSize = 0L;
	fd->fsv_ViewType = ROOT_INDEX;
	fd->fsv_SelectedNode = NULL;
	fd->FlyVol.Path = NULL;
	fd->FlyVol.Flags = FVF_USENUMS;
	fd->FlyVol.Board = 0;
	fd->fsv_FieldList = AllocMem( sizeof ( struct List ), MEMF_CLEAR | MEMF_ANY );
	if ( fd->fsv_FieldList )
	{
		NewList( fd->fsv_FieldList );
	}
	else
	{
		result = 1;
	}

	return( result );

}

/******************************************************\
*
*	FUNCTION:
*		void FSVWindow( void )
*
*	SYNOPSIS:
*		Creates main window and attaches gadgets
*
*	INPUT:
*		none
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void FSVWindow( void )
{
	struct TextFont	*font;
	struct Screen		*scr;
	struct Window		*win;
	struct Gadget		*glist, *gadlist[MAX_GADS + 1];
	struct Menu *menuStrip;
	void 					*vi;
	int i;
	
	if (NULL == (font = OpenFont(&Topaz80)))
		printf("Failed to Open Topaz 80");
	else
	{
		if (NULL == (scr = LockPubScreen(NULL)))
			printf("Could not lock public screen");
		else
		{
			if (NULL ==(vi = GetVisualInfo(scr, TAG_END)))
				printf("GetVisualInfo() failed");
			else
			{
				topborder = scr->WBorTop + (scr->Font->ta_YSize + 1);
				for(i=0;i <= MAX_GADS;i++)
					gadlist[i] = NULL;
				if (NULL == (CreateMainGadgets(&glist, vi, gadlist)))
					printf("CreateMainGadgets() Failed");
				else
				{
					if (NULL == (win = OpenWindowTags(NULL,
											WA_Title, 			WindowTitle,
											WA_Gadgets,			glist,
											WA_Left,				35,
											WA_Top,				13,
											WA_Width,			296,
											WA_Height,			228 + scr->Font->ta_YSize,
											WA_DragBar,			TRUE,
											WA_DepthGadget,	TRUE,
											WA_Activate,		TRUE,
											WA_CloseGadget,	TRUE,
											WA_SimpleRefresh,	TRUE,
											WA_IDCMP,			IDCMP_CLOSEWINDOW |
																	IDCMP_REFRESHWINDOW |
																	IDCMP_MENUPICK |
																	LISTVIEWIDCMP,
											WA_PubScreen,		scr,
											TAG_END)))
					{
						printf("OpenWindow() Failed");
					}
					else
					{
						fd->fsv_StatusWin = win;
						fd->fsv_StatusGadList = gadlist;

						if (NULL == (menuStrip = CreateMenus(FSVMenu, TAG_END)))
						{
							printf("CreateMenus Failed!");
						}
						else
						{
							if (!LayoutMenus(menuStrip, vi, TAG_END))
							{
								printf("LayoutMenus Failed!");
							}
							else
							{
								if (!SetMenuStrip(win, menuStrip))
								{
									printf("SetMenuStrip Failed!");
								}
								else
								{
									GT_RefreshWindow( win, NULL );
									GetNewBlockData();
									RefreshBlockDisplay( win, gadlist );
									ProcessWindowEvents(win, gadlist, menuStrip); 
									ClearMenuStrip(win);
									CloseWindow(win);
								}
							}
							FreeMenus(menuStrip);
						}
					}
				}
				FreeGadgets(glist);
				FreeVisualInfo(vi);
			}
			UnlockPubScreen(NULL, scr);
		}
		CloseFont(font);
	}
}

/******************************************************\
*
*	FUNCTION:
*		struct Gadget *CreateMainGadgets(struct Gadget **glistptr, 
*													void *vi,
*													struct Gadget *gadlist[])
*
*	SYNOPSIS:
*		Creates all gadgets for the main window
*
*	INPUT:
*		glistptr - pointer to the gadget list to create
*		vi - pointer to the visualinfo struct
*		gadlist - pointer to the gadget list to fill in
*
*	RETURNS:
*		a pointer to the last gadget created, NULL if there was an error
*	
*	NEEDS:
*
\******************************************************/

struct Gadget *CreateMainGadgets(struct Gadget **glistptr, void *vi,
					struct Gadget *gadlist[])
{
	struct NewGadget ng;
	struct Gadget *gad;

	
	gad = CreateContext(glistptr);
	ng.ng_LeftEdge = 128;
	ng.ng_TopEdge = 4 + topborder;
	ng.ng_Width = 136;
	ng.ng_Height = 12;
	ng.ng_GadgetText = ViewTitle;
	ng.ng_TextAttr = &Topaz80;
	ng.ng_VisualInfo = vi;
	ng.ng_GadgetID = GAD_VIEWAS;
	ng.ng_Flags = PLACETEXT_LEFT;
	gadlist[GAD_VIEWAS] = gad = CreateGadget(CYCLE_KIND, gad, &ng, 
								GTCY_Labels,	ViewLabels,
								GTCY_Active,	1,
								TAG_END);
	
	ng.ng_LeftEdge = 244;
	ng.ng_TopEdge = 20 + topborder;
	ng.ng_Width = 10;
	ng.ng_GadgetText = "<";
	ng.ng_GadgetID = GAD_PREV;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[GAD_PREV] = gad = CreateGadget( BUTTON_KIND, gad, &ng, 
															TAG_END);
	ng.ng_LeftEdge = 254;
	ng.ng_GadgetText = ">";
	ng.ng_GadgetID = GAD_NEXT;
	gadlist[GAD_NEXT] = gad = CreateGadget( BUTTON_KIND, gad, &ng, 
															TAG_END);


	ng.ng_LeftEdge = 128;
	ng.ng_GadgetText = BlockLabel;
	ng.ng_Width = 116;
	ng.ng_GadgetID = GAD_BLOCKNUM;
	ng.ng_Flags = PLACETEXT_LEFT;
	gadlist[GAD_BLOCKNUM] = gad = CreateGadget(STRING_KIND, gad, &ng, 
									STRINGA_Justification,	GACT_STRINGRIGHT,
									GTST_String,				fd->fsv_BlockString,
									TAG_END);

	ng.ng_TopEdge = 32 + topborder;
	ng.ng_GadgetText = JumpLabel;
	ng.ng_Width = 136;
	ng.ng_GadgetID = GAD_JUMP;
	ng.ng_Flags = PLACETEXT_IN;
	gadlist[GAD_JUMP] = gad = CreateGadget( BUTTON_KIND, gad, &ng, 
															TAG_END);
	
	ng.ng_TopEdge = 60 + topborder;
	ng.ng_GadgetText = "";
	ng.ng_LeftEdge = 8;
	ng.ng_Width = 280;
	ng.ng_Height = 128;
	ng.ng_GadgetID = GAD_LISTVIEW;
	ng.ng_Flags = PLACETEXT_ABOVE;
	if (fd->fsv_GadLib == 39)
	{
		gadlist[GAD_LISTVIEW] = gad = CreateGadget(LISTVIEW_KIND, gad, &ng,
										GTLV_ShowSelected,	NULL,
										TAG_END);
	}
	else
	{
		gadlist[GAD_LISTVIEW] = gad = CreateGadget(LISTVIEW_KIND, gad, &ng,
										TAG_END);
	}

	ng.ng_TopEdge = topborder + 183;
	ng.ng_Height = 14;
	ng.ng_GadgetText = "";
	ng.ng_GadgetID = GAD_SELNODE;
	gadlist[GAD_SELNODE] = gad = CreateGadget(TEXT_KIND, gad, &ng,
															GTTX_Text, "",
															GTTX_Border, TRUE,
															TAG_END);	

	ng.ng_TopEdge = 204 + topborder;
	ng.ng_Height = 14;
	ng.ng_GadgetID = GAD_STATUS;
	gadlist[GAD_STATUS] = gad = CreateGadget(TEXT_KIND, gad, &ng, 
								GTTX_Text,	"",
								GTTX_Border,	TRUE,
								TAG_END);

	return( gad );
}

/******************************************************\
*
*	FUNCTION:
*		BOOL HandleGadgetEvent( struct Window *win, 
*										struct Gadget *gad, 
*										UWORD code,
*										struct Gadget *gadlist[], 
*										UWORD menuID)
*
*	SYNOPSIS:
*		Handles all gadget events
*
*	INPUT:
*		win - pointer to the window to handle events for
*		gad - pointer to the gadget that was activated
*		code - the code field from the message that the gadget sent
*		gadlist - pointer to the gadget list containing the gadgets to process
*		menuID - ID of the menu that was selected, if any
*
*	RETURNS:
*		TRUE if a user's action indicates that they want the program to close
*		FALSE if not
*
*	NEEDS:
*
\******************************************************/

BOOL HandleGadgetEvent(struct Window *win, struct Gadget *gad, UWORD code,
								struct Gadget *gadlist[], UWORD menuID)
{
	BOOL terminated = FALSE;
	UWORD ID;

	if (gad == NULL)
		ID = menuID;
	else
		ID = gad->GadgetID;

	switch(ID)
	{
		case GAD_NOOP:
			break;
		case GAD_VIEWAS:		/* Cycle Gadget */
			{
				fd->fsv_ViewType = code;
				RefreshBlockDisplay( win, gadlist );
			}
			break;
		case GAD_BLOCKNUM:	/* String Gadget */
			{
				strcpy( fd->fsv_BlockString, ( char * )( ( ( struct StringInfo * )( gadlist[GAD_BLOCKNUM]->SpecialInfo ) )->Buffer ) );
				fd->fsv_BlockValue = Hex2ULONG( fd->fsv_BlockString );
				if ( fd->fsv_BlockValue >= fd->fsv_DriveSize )
				{
					fd->fsv_BlockValue = fd->fsv_DriveSize - 1;
				}
				if ( fd->fsv_BlockValue < 0 )
				{
					fd->fsv_BlockValue = 0;
				}

				sprintf( fd->fsv_BlockString, "%#8.8lx", fd->fsv_BlockValue );
				GetNewBlockData();
				RefreshBlockDisplay( win, gadlist );
			}
			break;
		case GAD_NEXT:
			{
				fd->fsv_BlockValue++;
				if ( fd->fsv_BlockValue >= fd->fsv_DriveSize )
				{
					fd->fsv_BlockValue = fd->fsv_DriveSize - 1;
				}
				sprintf( fd->fsv_BlockString, "%#8.8lx", fd->fsv_BlockValue );
				GT_SetGadgetAttrs(gadlist[GAD_BLOCKNUM], win, NULL,
										GTST_String,	fd->fsv_BlockString,
										TAG_END);
				GetNewBlockData();
				RefreshBlockDisplay( win, gadlist );
			}
			break;
		case GAD_PREV:
			{
				if ( fd->fsv_BlockValue > 0 )
				{
					fd->fsv_BlockValue--;
				}
				sprintf( fd->fsv_BlockString, "%#8.8lx", fd->fsv_BlockValue );
				GT_SetGadgetAttrs(gadlist[GAD_BLOCKNUM], win, NULL,
										GTST_String,	fd->fsv_BlockString,
										TAG_END);
				GetNewBlockData();
				RefreshBlockDisplay( win, gadlist );
			}
			break;
		case GAD_JUMP:			/* Button */
			{
				if ( fd->fsv_SelectedNode )
				{
					if ( fd->fsv_SelectedNode->address )
					{
						fd->fsv_BlockValue = fd->fsv_SelectedNode->value;
						if ( fd->fsv_BlockValue >= fd->fsv_DriveSize )
						{
							fd->fsv_BlockValue = fd->fsv_DriveSize - 1;
						}

						fd->fsv_ViewType = fd->fsv_SelectedNode->block_type;
						GT_SetGadgetAttrs(gadlist[GAD_VIEWAS], win, NULL,
												GTCY_Active,	fd->fsv_ViewType,
												TAG_END);

						sprintf( fd->fsv_BlockString, "%#8.8lx", fd->fsv_BlockValue );
						GT_SetGadgetAttrs(gadlist[GAD_BLOCKNUM], win, NULL,
												GTST_String,	fd->fsv_BlockString,
												TAG_END);
						GetNewBlockData();
						RefreshBlockDisplay( win, gadlist );
					}
					else
					{
						/* Print an error message... */
						StatusMessage( "Selected field is not an address" );
					}
				}
			}
			break;
		case GAD_LISTVIEW:	/* ListView Gadget */
			{
				fd->fsv_SelectedNode = ( struct BlockFieldNode * ) GetListNode( fd->fsv_FieldList, code );
				if ( fd->fsv_SelectedNode )
				{
					strcpy( fd->fsv_SelString, fd->fsv_SelectedNode->str );
					GT_SetGadgetAttrs(gadlist[GAD_SELNODE], win, NULL,
											GTTX_Text,	fd->fsv_SelString,
											TAG_END);
				}
			}
			break;
		case GAD_EXIT:
			{
				PurgeMsgPort( win );
				terminated = TRUE;
			}
			break;
	}		
	return( terminated );
}

/******************************************************\
*
*	FUNCTION:
*		void GetNewBlockData( void )
*
*	SYNOPSIS:
*		Reads a block from the drive into fd->fsv_BlockBuffer
*
*	INPUT:
*
*	RETURNS:
*
*	NEEDS:
*
\******************************************************/
void GetNewBlockData( void )
{
	ULONG res;
	UBYTE *buffer;

	buffer = (UBYTE *)0xEC0000 + (512 * RAMBLK);
	res = SCSI_Read( fd->fsv_BlockValue, 1 );
	memcpy( fd->fsv_BlockBuffer, buffer, 512 );
		
}

void RefreshBlockDisplay( struct Window *win, struct Gadget *gadlist[] )
{
	struct BlockFieldNode *current;
	
	/* detach previous list */
	GT_SetGadgetAttrs(gadlist[GAD_LISTVIEW], win, NULL,
															GTLV_Labels,		NULL,
															TAG_END);

	/* free previous list */
	while( !IsListEmpty( fd->fsv_FieldList ) )
	{
		current = ( struct BlockFieldNode * )RemHead( fd->fsv_FieldList );
		FreeMem( current, sizeof( struct BlockFieldNode ) );
	}

	/* create new list */

	switch( fd->fsv_ViewType )
	{
		case RAW_INDEX:
			CreateRawList();
			break;
		case ROOT_INDEX:
			CreateRootList();
			break;
		case DIR_INDEX:
			CreateDirList();
			break;
		case FILE_INDEX:
			CreateFileList();
			break;
		case MAPBLOCK_INDEX:
			CreateMapBlockList();
			break;
		case FSHEADS_INDEX:
			CreateFSHeadsList();
			break;
	}

	/* attach new list */
	GT_SetGadgetAttrs(gadlist[GAD_LISTVIEW], win, NULL,
							GTLV_Labels, fd->fsv_FieldList,
							TAG_END);		
}

void CreateRootList( void )
{
	struct BlockFieldNode *current;
	struct ROOTHDR *buffer;
	int i;
	
	buffer = ( struct ROOTHDR * )( fd->fsv_BlockBuffer );

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->id;
		sprintf( current->str, "Drive ID: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = ( ULONG )( buffer->flags );
		sprintf( current->str, "Drive Flags: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = (ULONG )( buffer->diskokay );
		if ( current->value)
		{
			sprintf( current->str, "Disk Okay: Yes" );
		}
		else
		{
			sprintf( current->str, "Disk Okay: No" );
		}
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = ( ULONG )( buffer->blksize );
		sprintf( current->str, "Block Size: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->parent;
		sprintf( current->str, "Parent: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Volume Name: %.32ls", buffer->volname);
		current->address = 0;
		current->block_type = ROOT_INDEX; // Block 0
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->freelist;
		sprintf( current->str, "FreeList Block: %#lx", current->value);
		current->address = 1;
		current->block_type = MAPBLOCK_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->headlist;
		sprintf( current->str, "Heads List: %#lx", current->value);
		current->address = 1;
		current->block_type = FSHEADS_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->hdrsarea;
		sprintf( current->str, "File/Dirs Headers: %#lx", current->value);
		current->address = 1;
		current->block_type = FILE_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->unused;
		sprintf( current->str, "Unused area: %#lx", current->value);
		current->address = 1;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->dataarea;
		sprintf( current->str, "Data Area: %#lx", current->value);
		current->address = 1;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->headspace;
		sprintf( current->str, "Heads Space: %#lx", current->value);
		current->address = 1;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->videoend;
		sprintf( current->str, "Video End: %#lx", current->value);
		current->address = 1;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->blksfree;
		sprintf( current->str, "Blocks Free: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = ( ULONG )( buffer->version );
		sprintf( current->str, "Version: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = ( ULONG )( buffer->type );
		sprintf( current->str, "Type: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->date.days;
		sprintf( current->str, "Days since 1/1/78: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->date.minutes;
		sprintf( current->str, "Minutes Past Midnight: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->date.ticks;
		sprintf( current->str, "Ticks Past Minute: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->userblks;
		sprintf( current->str, "User Blocks: %lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->newheadlist;
		sprintf( current->str, "New Heads List: %#lx", current->value);
		current->address = 1;
		current->block_type = FSHEADS_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Pad: ");
		for ( i = 0 ; i < 8 ; i++)
		{
			sprintf(current->str, "%s %02.2x", current->str, buffer->pad1[i] );
		}		
		current->address = 1;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Pad: ");
		for ( i = 8 ; i < 10 ; i++)
		{
			sprintf(current->str, "%s %02.2x", current->str, buffer->pad1[i] );
		}		
		current->address = 1;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Children:" );
		current->address = 0;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	for ( i = 0 ; i < ROOTENTRIES ; i++ )
	{
		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			current->value = buffer->entries[i];
			sprintf( current->str, "Entry #%ld: %#lx", i, current->value);
			current->address = 1;
			current->block_type = FILE_INDEX;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->checksum;
		sprintf( current->str, "CheckSum: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

}

void CreateDirList( void )
{
	struct BlockFieldNode *current;
	struct DIRHDR *buffer;
	int i;
	
	buffer = ( struct DIRHDR * ) ( fd->fsv_BlockBuffer );

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->id;
		sprintf( current->str, "Header ID: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->nexthash;
		sprintf( current->str, "Next Hash: %#lx", current->value);
		current->address = 1;
		current->block_type = FILE_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->parent;
		sprintf( current->str, "Parent: %#lx", current->value);
		current->address = 1;
		current->block_type = DIR_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Name: %.32ls", buffer->name);
		current->address = 0;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Comment: %.32ls", buffer->comment);
		current->address = 0;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = ( ULONG )( buffer->flags );
		sprintf( current->str, "Flags: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = ( ULONG )buffer->type;
		sprintf( current->str, "Type: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->date.days;
		sprintf( current->str, "Days since 1/1/78: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->date.minutes;
		sprintf( current->str, "Minutes Past Midnight: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->date.ticks;
		sprintf( current->str, "Ticks Past Minute: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->bits;
		sprintf( current->str, "Bits: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Pad: ");
		for ( i = 0 ; i < 8 ; i++)
		{
			sprintf(current->str, "%s %02.2x", current->str, *(buffer + i) );
		}		
		current->address = 1;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Pad: ");
		for ( i = 8 ; i < 14 ; i++)
		{
			sprintf(current->str, "%s %02.2x", current->str, *(buffer + i) );
		}		
		current->address = 1;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Children:" );
		current->address = 0;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	for ( i = 0 ; i < ROOTENTRIES ; i++ )
	{
		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			current->value = buffer->entries[i];
			sprintf( current->str, "Entry #%ld: %#lx", i, current->value);
			current->address = 1;
			current->block_type = FILE_INDEX;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->checksum;
		sprintf( current->str, "CheckSum: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}
}

void CreateFileList( void )
{
	struct BlockFieldNode *current;
	struct FILEHDR *buffer;
	int i;
	
	buffer = ( struct FILEHDR * ) ( fd->fsv_BlockBuffer );

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->id;
		sprintf( current->str, "Header ID: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}
	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->nexthash;
		sprintf( current->str, "Next Hash: %#lx", current->value);
		current->address = 1;
		current->block_type = FILE_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->parent;
		sprintf( current->str, "Parent: %#lx", current->value);
		current->address = 1;
		current->block_type = DIR_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Name: %.32ls", buffer->name);
		current->address = 0;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Comment: %.32ls", buffer->comment);
		current->address = 0;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = ( ULONG )( buffer->pad1 );
		sprintf( current->str, "Pad: %#02.2lx", current->value );
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = ( ULONG )( buffer->type );
		sprintf( current->str, "Type: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->date.days;
		sprintf( current->str, "Days since 1/1/78: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->date.minutes;
		sprintf( current->str, "Minutes Past Midnight: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->date.ticks;
		sprintf( current->str, "Ticks Past Minute: %ld", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = ( ULONG ) ( buffer->bits );
		sprintf( current->str, "Bits: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Pad: ");
		for ( i = 0 ; i < 8 ; i++)
		{
			sprintf(current->str, "%s %02.2x", current->str, buffer->pad2[i] );
		}		
		current->address = 1;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->start;
		sprintf( current->str, "Start: %#lx", current->value);
		current->address = 1;
		current->block_type = RAW_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->bytes;
		sprintf( current->str, "Bytes Low-16 bits: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->bytes_hi;
		sprintf( current->str, "Bytes High-16 bits: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}


	for ( i = 0 ; i < 49 ; i++ )
	{
		int j;


		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			current->value = 0;
			sprintf( current->str, "Pad:");
			for ( j = 0 ; j < 8 ; j++)
			{
				sprintf(current->str, "%s %02.2x", current->str, buffer->pad3[i * 8 + j] );
			}		
			current->address = 1;
			current->block_type = ROOT_INDEX;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Pad:");
		for ( i = 392 ; i < 394 ; i++)
		{
			sprintf(current->str, "%s %02.2x", current->str, buffer->pad3[i] );
		}		
		current->address = 1;
		current->block_type = ROOT_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->checksum;
		sprintf( current->str, "CheckSum: %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

}

void CreateMapBlockList( void )
{
	struct BlockFieldNode *current;
	struct MAPBLOCK *buffer;
	int i;
	
	buffer = ( struct MAPBLOCK * ) ( fd->fsv_BlockBuffer );

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->mb_ID;
		sprintf( current->str, "MapBlock ID(FLST): %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->mb_Next;
		sprintf( current->str, "Next MapBlock: %#lx", current->value);
		current->address = 1;
		current->block_type = MAPBLOCK_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "Map Entries:" );
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	for ( i = 0 ; i < MAPENTRIES ; i++ )
	{
		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			current->value = 0;
			sprintf( current->str, " Map Entry #%ld", i);
			current->address = 0;
			current->block_type = 0;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}

		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			current->value = buffer->mb_Entries[i].me_Block;
			sprintf( current->str, "  Block: %#lx", current->value);
			current->address = 1;
			current->block_type = RAW_INDEX;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}

		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			current->value = buffer->mb_Entries[i].me_Length;
			sprintf( current->str, "  Length(Blocks): %#lx", current->value);
			current->address = 0;
			current->block_type = 0;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}
	}
}


void CreateFSHeadsList( void )
{
	struct BlockFieldNode *current;
	struct FSHEADSBLOCK *buffer;
	int i;
	
	buffer = ( struct FSHEADSBLOCK * ) ( fd->fsv_BlockBuffer );

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->fsh_ID;
		sprintf( current->str, "FSHeads Block ID(HLST): %#lx", current->value);
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = buffer->fsh_Next;
		sprintf( current->str, "Next MapBlock: %#lx", current->value);
		current->address = 1;
		current->block_type = MAPBLOCK_INDEX;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
	{
		StatusMessage( "Out of Memory!");
	}
	else
	{
		current->value = 0;
		sprintf( current->str, "FS Heads Entries:" );
		current->address = 0;
		current->block_type = 0;
		current->node.ln_Name = current->str;
		current->node.ln_Pri = 0;
		current->node.ln_Type = 100;
		AddTail( fd->fsv_FieldList, ( struct Node * )current );
	}

	for ( i = 0 ; i < FSHEADENTRIES ; i++ )
	{
		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			current->value = 0;
			sprintf( current->str, " Map Entry #%ld", i);
			current->address = 0;
			current->block_type = 0;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}

		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			current->value = buffer->fsh_Heads[i].fshe_start;
			sprintf( current->str, "  Start Block: %#lx", current->value);
			current->address = 1;
			current->block_type = RAW_INDEX;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}

		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			current->value = buffer->fsh_Heads[i].fshe_length;
			sprintf( current->str, "  Length(Blocks): %#lx", current->value);
			current->address = 0;
			current->block_type = 0;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}

		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			current->value = buffer->fsh_Heads[i].fshe_ID;
			sprintf( current->str, "  Sub Group ID: %#lx", current->value);
			current->address = 0;
			current->block_type = 0;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}
	}
}

void CreateRawList( void )
{
	struct BlockFieldNode *current;
	UBYTE *buffer;
	int i;
	
	buffer = fd->fsv_BlockBuffer;

	for ( i = 0 ; i < 64 ; i++ )
	{
		if (!(current = AllocMem( sizeof( struct BlockFieldNode ), MEMF_CLEAR | MEMF_PUBLIC ) ) )
		{
			StatusMessage( "Out of Memory!");
		}
		else
		{
			int offset = i<<3;

			current->value = 0;
			sprintf( current->str, 
						"%#4lx: %02.2x %02.2x %02.2x %02.2x %02.2x %02.2x %02.2x %02.2x", 
						i,
						buffer[offset],
						buffer[offset | 1],
						buffer[offset | 2],
						buffer[offset | 3],
						buffer[offset | 4],
						buffer[offset | 5],
						buffer[offset | 6],
						buffer[offset | 7] );
			current->address = 0;
			current->block_type = 0;
			current->node.ln_Name = current->str;
			current->node.ln_Pri = 0;
			current->node.ln_Type = 100;
			AddTail( fd->fsv_FieldList, ( struct Node * )current );
		}
	}
}

/******************************************************\
*
*	FUNCTION:
*		struct Node *GetListNode( struct List *list, UWORD index )
*
*	SYNOPSIS:
*		Returns a pointer to the (index)th node in list
*
*	INPUT:
*		list - The list to find the Node in
*		index - The number of the Node to find
*
*	RETURNS:
*		pointer to the corresponding Node
*
*	NEEDS:
*
\******************************************************/

struct BlockFieldNode *GetListNode( struct List *list, UWORD index )
{
	struct Node *current = list->lh_Head;
	int i;
	
	if ( IsListEmpty( list ) )
	{
		current = NULL;
	}
	else
	{
		for ( i = 0; i < index ; current = current->ln_Succ, i++ )
		{
			if ( current->ln_Succ == NULL )
			{
				current = NULL;
				break;
			}
		}
	}
	return( ( struct BlockFieldNode * )current );
}

/******************************************************\
*
*	FUNCTION:
*		void ProcessWindowEvents(  struct Window *win, 
*											struct Gadget *gadlist[], 
*											struct Menu *menuStrip)
*
*	SYNOPSIS:
*		Handles all messages for gadgets and menus 
*
*	INPUT:
*		win - pointer to window to handle messages for
*		gadlist - pointer to list of gadgets to handle events for
*		menuStrip - pointer to the menu to process if available, NULL if none
*
*	RETURNS:
*		none
*
*	NEEDS:
*
\******************************************************/

void ProcessWindowEvents(struct Window *win, struct Gadget *gadlist[], struct Menu *menuStrip)
{
	struct IntuiMessage *imsg;
	ULONG imsgClass;
	UWORD imsgCode, menuID;
	struct Gadget *gad;
	UWORD menuNumber;
	struct MenuItem *item; 
	BOOL terminated = FALSE;

	while (!terminated)
	{
		Wait (1<<win->UserPort->mp_SigBit);
		while ((!terminated) && (imsg = GT_GetIMsg(win->UserPort)))
		{
			gad = (struct Gadget *)imsg->IAddress;

			imsgClass = imsg->Class;
			imsgCode = imsg->Code;

			GT_ReplyIMsg(imsg);

			switch(imsgClass)
			{
				case IDCMP_MENUPICK:
					menuNumber = imsgCode;
					while ((menuNumber != MENUNULL) && (!terminated))
					{
						item = ItemAddress(menuStrip, menuNumber);
						menuID = Menu2ID(MENUNUM(menuNumber), ITEMNUM(menuNumber), SUBNUM(menuNumber), &imsgCode);
						terminated = HandleGadgetEvent(win, NULL, imsgCode, gadlist, menuID);
						menuNumber = item->NextSelect;
					}
					break;
				case IDCMP_GADGETDOWN:
				case IDCMP_GADGETUP:
				case IDCMP_MOUSEMOVE:
					terminated = HandleGadgetEvent(win, gad, imsgCode, gadlist, NULL);
					break;
				case IDCMP_CLOSEWINDOW:
					terminated = TRUE;
					break;
				case IDCMP_REFRESHWINDOW:
					GT_BeginRefresh(win);
					GT_EndRefresh(win, TRUE);
					break;
			}
		}
	}
}

/******************************************************\
*
*	FUNCTION:
*		ULONG Str2ULONG(char *str)
*
*	SYNOPSIS:
*		extracts a value from a string containing a number
*		in decimal format, skipping any leading spaces and
*		ending when it encounters the end of the string or
*		any invalid characters
*
*	INPUT:
*		str - string containing an ascii representation of 
*				a decimal number
*
*	RETURNS:
*		the ULONG representation of the number
*
*	NEEDS:
*
\******************************************************/

ULONG Str2ULONG(char *str)
{
	ULONG num = 0L;
	int i;
	
	while (str[0] == 32)
		str++;

	for (i=0;str[i] != '\0';i++)
	{
		if ((str[i] >= '0') && (str[i] <= '9'))
			num = (num * 10) + (str[i] - '0');
		else
			break;
	}
	
	return(num);
}

/******************************************************\
*
*	FUNCTION:
*		ULONG Hex2ULONG(char *str)
*
*	SYNOPSIS:
*		extracts a value from a string containing a number
*		in hexadecimal format, skipping any invalid characters
*
*	INPUT:
*		str - string containing an ascii representation of 
*				a hexadecimal number, with or without a leading 0x
*
*	RETURNS:
*		the ULONG representation of the number
*
*	NEEDS:
*
\******************************************************/

ULONG Hex2ULONG(char *str)
{
	ULONG num = 0L;
	int i;
	
	for (i=0;str[i] != '\0';i++)
	{
		if ((str[i] >= '0') && (str[i] <= '9'))
			num = (num * 16) + (str[i] - '0');
		if ((str[i] >= 'a') && (str[i] <= 'f'))
			num = (num * 16) + (str[i] - 'a') + 10L;
		if ((str[i] >= 'A') && (str[i] <= 'F'))
			num = (num * 16) + (str[i] - 'A') + 10L;
	}
	
	return(num);
}

/******************************************************\
*
*	FUNCTION:
*		float Str2Float(char *str)
*
*	SYNOPSIS:
*		Extracts a floating point number from a string, 
*		skipping leading spaces and stopping when it hits
*		the end of the string or invalid character.
*
*	INPUT:
*		str - string containing the number to extract
*
*	RETURNS:
*		the floating point version of the decimal number
*		contained in the string.
*
*	NEEDS:
*
\******************************************************/

float Str2Float(char *str)
{
	float num;
	int i, sigdigs, point;
	float power;
	
	num = 0;
	sigdigs = 0;
	point = 0;
	power = 10.0;
	
	while(str[0] == 32)
		str++;

	for(i=0; ((str[i] != '\0') && (sigdigs < 6));i++)
	{
		if ((str[i] >= '0') && (str[i] <= '9'))
		{
			if (point == 0)
				num = (num * 10.0) + (float)(str[i] - '0');
			else
			{
				num += ((float)(str[i] - '0') / power);
				power *= 10.0;
			}
			sigdigs++;
		}
		else
			if (str[i] == '.')
				point = 1;
			else
				break;
	}
	return(num);
}

ULONG SCSI_Read(ULONG lba, ULONG size)
{
	ULONG	 res;
	char ss[128];
	struct ClipAction Action;

	Action.Flags = 0;
	Action.Volume =  &(fd->FlyVol);
	Action.ReturnTime = RT_STOPPED;
	
	res = Read10(&Action,size,lba,DMABLK);
	if (res)
	{
		sprintf(ss,"Read10 Error %ld - %ls\n",res,Error2String(res));
		StatusMessage(ss);
	}
	else 
	{
		res = CPUDMA(0,RAMBLK,DMABLK,size,0);		/* Write to CPU */
		if (res)
		{
			sprintf(ss, "CPUDMA Error %ld - %ls\n",res,Error2String(res));
			StatusMessage(ss);
		}
	}
	return(res);
}

/******************************************************\
*
*	FUNCTION:
*		void StatusMessage( char *msg )
*
*	SYNOPSIS:
*		Displays the string, msg, on the status bar of the main window
* 
*	INPUT:
*		msg - string to print in status bar
*
*	RETURNS:
*		none
*
*	NEEDS:
*		fd->fsv_StatusWin must be a pointer to the main window
*			containing the status gadget
*		fd->fsv_StatusGadList must point to the gadget list 
*			containing the status gadget
*
\******************************************************/

void StatusMessage( char *msg )
{
	strncpy(fd->fsv_StatusStr, msg, 63);

	GT_SetGadgetAttrs(fd->fsv_StatusGadList[GAD_STATUS], fd->fsv_StatusWin, NULL,
													GTTX_Text,	fd->fsv_StatusStr,
													TAG_END);
}

/******************************************************\
*
*	FUNCTION:
*		UWORD Menu2ID( UWORD menu, UWORD item, UWORD sub, UWORD *code)
*
*	SYNOPSIS:
*		given a number of a (sub)item, returns the gadget constant
*		that is used in HandleGadgetEvent to perform the function.
*		Thus, all gadget and menu handling is done in the same function.
*
*	INPUT:
*		menu - menu number of the selection
*		item - item number of the selection
*		sub - subitem number of the selection
*     code - pointer to the code in the MENUPICK message.
*
*	RETURNS:
*		the defined constant representing a gadget ID that the menu
*		(sub)item is connected with
*
*	NEEDS:
*
\******************************************************/

UWORD Menu2ID(UWORD menu, UWORD item, UWORD sub, UWORD *code)
{
	switch(menu)
	{
		case 0:
			switch(item)
			{
				case 0:
					return(GAD_VIEWREPORT);
					break;
			}
			break;
	}
	return(GAD_NOOP);
}

/******************************************************\
*
*	FUNCTION:
*		void PurgeMsgPort( struct Window *win )
*
*	SYNOPSIS:
*		clears all messages from the selected message port
*		to avoid extraneous mouse clicks and menu selections
*		while the window is inactive.
*
*	INPUT:
*		win - pointer to the window to clear messages from 
*
*	RETURNS:
*		nothing
*
*	NEEDS:
*
\******************************************************/

void PurgeMsgPort(struct Window *win)
{
	struct IntuiMessage *imsg;
	ULONG imsgClass;

	while (imsg = GT_GetIMsg(win->UserPort))
	{
		imsgClass = imsg->Class;
		GT_ReplyIMsg(imsg);
		switch(imsgClass)
		{
			case IDCMP_REFRESHWINDOW:
				GT_BeginRefresh(win);
				GT_EndRefresh(win, TRUE);
				break;
		}
	}
}
