/*********************************************************************\
*
* NewSequence - Flyer test sequencer (4.1)
*
* $Id: newseq.c,v 1.4 1995/08/11 12:57:30 Flick Exp $
*
* $Log: newseq.c,v $
*Revision 1.4  1995/08/11  12:57:30  Flick
*Now uses new Flyer timing error detection, will report timing errors
*Added option to wait for sequence completion
*
*Revision 1.3  1995/06/20  09:12:06  Flick
*Can sequence matte black using name=(none)
*
*Revision 1.2  1995/05/24  17:14:53  Flick
*Added "again" option
*
*Revision 1.1  1995/05/23  13:17:02  Flick
*Improved error handling (aborts)
*
*Revision 1.0  1995/05/11  15:37:23  Flick
*FirstCheckIn
*
*
* Copyright (c) 1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	05/11/94		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"


#define	THEMAX	100

struct Entry {
	ULONG	clock;
	ULONG	drive;
	char	name[80];
	ULONG	vidflag;
	ULONG	chan;
	ULONG	vstartfld;
	ULONG	vfldlen;
	ULONG	audchans;
	ULONG	astartfld;
	ULONG	afldlen;
	ULONG vol1,vol2;
	ULONG pan1,pan2;
	ULONG	attack;
	ULONG	decay;
};

struct Entry Sequence[THEMAX];

struct Library *FlyerBase;

struct ClipInfo	clipinfo;

UBYTE	*FileBuffer;
ULONG	FileLength;
BPTR	Handle;


/*** Prototypes ***/
ULONG ReadScriptFile(char *name);
void FreeFile(void);
int WaitHere(int);
ULONG AddStep(struct Entry *ent,int ID);


main(int argc,char **argv)
{
	int	i,ndx,stepmode=0,waitmode=0;
	ULONG	baseclock,err,userID,moreinfo;
	LONG	size;
	char *scriptptr,*endptr;
	struct Entry *entptr;
	BOOL	showederror = FALSE;
	UBYTE	done;

	if ((argc > 2) && (stricmp(argv[argc-1],"STEP")==0))
	{
		stepmode = 1;
		argc--;
	}
	if ((argc > 2) && (stricmp(argv[argc-1],"WAIT")==0))
	{
		waitmode = 1;
		argc--;
	}

	if ((argc!=2)||(argv[1][0]=='?'))
	{
		printf("Usage: %ls <script> [WAIT] [STEP]\n",argv[0]);
		return(20);
	}

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0)
	{
		printf("Open flyer.library failed\n");
		return;
	}

	Handle = Input();

	if (!stricmp(argv[1],"AGAIN"))
		goto redoit;

	size = ReadScriptFile(argv[1]);
	if (size)
	{
		endptr = FileBuffer+FileLength;
		for (ndx=0,scriptptr=FileBuffer;(ndx<THEMAX)&&(scriptptr<endptr);)
		{
			if (*scriptptr == '>')
			{
				entptr = &Sequence[ndx];
				scriptptr++;
				sscanf(scriptptr,"%ld %ld %ls %ld %ld %ld %ld %ld %ld %ld %ld %ld %lx %lx %lx %lx\n",
					&entptr->clock,
					&entptr->drive,
					entptr->name,
					&entptr->vidflag,
					&entptr->chan,
					&entptr->vstartfld,
					&entptr->vfldlen,
					&entptr->audchans,
					&entptr->astartfld,
					&entptr->afldlen,
					&entptr->attack,
					&entptr->decay,
					&entptr->vol1,
					&entptr->vol2,
					&entptr->pan1,
					&entptr->pan2
				);

				if (!stricmp(entptr->name,"(none)"))		// Turn "(none)" into NULL string
					entptr->name[0] = 0;

				ndx++;
			}

			// Skip to next line
	      do
			{
				if (scriptptr >= endptr)
					break;
			} while (*scriptptr++ != 10);
		}

		for (i=0;i<ndx;i++)
		{
			entptr = &Sequence[i];
			printf("Step %ld at %ld: %ls from drive %ld\n",i+1,entptr->clock,entptr->name,entptr->drive);
			printf("          vflag=%ld, vchan=%ld, vflds=%ld/%ld\n",
				entptr->vidflag,entptr->chan,entptr->vstartfld,entptr->vfldlen);
		}

		printf("Ready to open sequence -- ");
		if (WaitHere(stepmode))
			goto abort;

		err = NewSequence(0);
		if (err)
			printf("Open Error %ld - %ls\n",err,Error2String(err));


		for (i=0;i<ndx;i++)
		{
			printf("Ready to add clip %ld -- ",i+1);
			if (WaitHere(stepmode))
			{
				err = EndSequence(0,0);
				goto abort;
			}

			entptr = &Sequence[i];

			/* Send clip def */
//			printf("Def %ls...\n",entptr->name);
			err = AddStep(entptr,i+1);
			if (err)
			{
				printf("AddStep Error %ld at step %ld - %ls\n",err,i+1,
					Error2String(err));
				err = EndSequence(0,0);
				goto abort;
			}
		}

		printf("Ready to close sequence -- ");
		if (WaitHere(stepmode))
		{
			err = EndSequence(0,0);
			goto abort;
		}

		err = EndSequence(0,1);
		if (err)
		{
			printf("Close Error %ld - %ls\n",err,Error2String(err));
			goto abort;
		}

		printf("Ready to play sequence -- ");
		if (WaitHere(stepmode))
			goto abort;

redoit:

		/* Get current clock value */
		err = GetFieldClock(&baseclock);
		if (err)
			printf("Error %ld - %ls\n",err,Error2String(err));
		else
			printf("BaseClk = %lx\n",baseclock);

		err = PlaySequence(0,baseclock + 60);
		if (err)
			printf("Play Error %ld - %ls\n",err,Error2String(err));

		if (waitmode)
		{
			do
			{
				err = GetClrSeqError(0,0,&done,&userID,&moreinfo);		// Get sequencing error
				if ((err != FERR_OKAY) && (!showederror))
				{
					printf("*** Sequencing Error -- %ls ***\n",Error2String(err));
					printf("    Failed on clip #%ld (%ld)\n",userID,moreinfo);
					showederror = TRUE;
				}

				Delay(10);				// Don't busy wait!
			} while (done==0);		// Wait until stopped
		}
	}

abort:
	CloseLibrary(FlyerBase);
	FreeFile();

	return(0);
}


ULONG ReadScriptFile(char *name)
{
	FILE	*fp;
	int	read;

	read = 0;
	fp = fopen(name,"r");
	if (fp)
	{
		fseek(fp,0,SEEK_END);
		FileLength = ftell(fp);
		fseek(fp,0,SEEK_SET);

		FileBuffer = AllocMem(FileLength,MEMF_PUBLIC);
		if (FileBuffer)
		{
			read = fread(FileBuffer,FileLength,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",name);

	if (read==1)
		return(FileLength);
	else
		return(0);
}

void FreeFile(void)
{
	if (FileLength)
		FreeMem(FileBuffer,FileLength);
}


int WaitHere(int stepmode)
{
	if (stepmode == 0)
	{
		printf("\n");
	}
	else
	{
		printf("Hit ENTER to proceed...\n");
		for (;;)
		{
			if (WaitForChar(Handle,30))
			{
				if (getchar()=='q')
				{
					printf("Aborted\n");
					getchar();
					return(1);
				}
				printf("Here we go...\n");
				break;
			}
		}
	}
	return(0);
}


ULONG AddStep(struct Entry *ent, int ID)
{
	struct FlyerVolume volume;
	struct ClipAction	 action;

	volume.Board = 0;
	volume.Flags = FVF_USENUMS;
	volume.Path = ent->name;
	volume.SCSIdrive = ent->drive;

	action.Channel = ent->chan;
	action.Flags = 0;
	if (ent->vidflag)
		action.Flags |= CAF_VIDEO;
	if (ent->audchans == 1)
		action.Flags |= CAF_AUDIOL;
	if (ent->audchans == 2)
		action.Flags |= CAF_AUDIOL | CAF_AUDIOR;
//	action.Flags |= CAF_USEMATTE;
//	action.PermissFlags = /* CAPF_STEALOURVIDEO | */ CAPF_KILLOTHERVIDEO | CAPF_USEHEADS;

	action.GoClock = ent->clock;
	action.VidStartField = ent->vstartfld;
	action.VidFieldCount = ent->vfldlen;
	action.AudStartField = ent->astartfld;
	action.AudFieldCount = ent->afldlen;
	action.TotalAudStart = 0;
	action.TotalAudLength = 0;
	action.VolSust1 = ent->vol1;
	action.VolSust2 = ent->vol2;
	action.AudioPan1 = ent->pan1;
	action.AudioPan2 = ent->pan2;
	action.VolAttack = ent->attack;
	action.VolDecay = ent->decay;

	action.UserID = ID;					// Shows us who failed for sequencing errors

	action.Volume = &volume;			// Link together
	action.ReturnTime = RT_STOPPED;

	return(AddSeqClip(&action));
}
