/********************************************************************
* $oras.c - Offscreen rasters$
*
* $Id: oras.c,v 1.0 1995/10/09 16:36:50 Flick Exp $
*
* $Log: oras.c,v $
*Revision 1.0  1995/10/09  16:36:50  Flick
*FirstCheckIn
*
*
* Copyright (c)1993-1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved.
*
* Original code donated by Stuart F.
*
*********************************************************************
*
* This module creates and manages offscreen rastPorts with and without clipping.
*/

#include <exec/types.h>
#include <exec/memory.h>
#include <graphics/clip.h>

#include <proto/exec.h>
#include <proto/graphics.h>

#ifndef PROTO_PASS
#include <proto.h>
#endif

/*
 * My internal extended bitmap structure.  Since the bitmap is all I can
 * ever garantee allocating, this is where I cache all my stuff.  I cast
 * the bitmaps to this and check cookie.
 */
#define COOKIE		0x04567654
typedef struct st_ExtBitMap {
	struct BitMap		map;
	struct Layer_Info	*li;
	struct RastPort	*rp;
	long					cookie;
	int					w, h, depth;
} ExtBitMap;


/*
 * Create an extended bitmap of given size.
 */
static ExtBitMap *AllocExtBM(int w, int h, int depth)
{
	ExtBitMap	*bm;
	long			bytes;
	int			i;

	bm = (ExtBitMap *)AllocMem(sizeof(ExtBitMap),MEMF_PUBLIC|MEMF_CLEAR);
	if (bm)
	{
		InitBitMap(&bm->map, (LONG)depth, (LONG)w, (LONG)h);
		bytes = bm->map.BytesPerRow * bm->map.Rows;			// Bitmap size in bytes

		for (i=0; i<depth; i++)
		{
			if ((bm->map.Planes[i] = AllocMem(bytes, MEMF_CHIP))==NULL)
			{
				while (--i >= 0)
					FreeMem(bm->map.Planes[i], bytes);
				FreeMem(bm,sizeof(ExtBitMap));
				return(NULL);
			}
		}

		bm->cookie = COOKIE;			// Mark as valid
		bm->li = NULL;					// No Layer_Info
		bm->rp = NULL;					// No RastPort
		bm->w = w;						// Record dimensions (w,h,d)
		bm->h = h;
		bm->depth = depth;
	}
	return(bm);
}


/*
 * Free a bitmap thus allocated.
 */
static void FreeExtBM(ExtBitMap *bm)
{
	long	bytes;
	int	i;

	bytes = bm->map.BytesPerRow * bm->map.Rows;

//	ASSERT (bm->cookie == COOKIE);

	for (i=0; i<bm->map.Depth; i++)
		FreeMem(bm->map.Planes[i], bytes);

	FreeMem(bm,sizeof(ExtBitMap));
}


/*
 * Create an offscreen RastPort w/ BitMap
 */
struct RastPort *OSAllocRastPort(int w, int h, int depth)
{
	ExtBitMap	*bm;

	if (bm = AllocExtBM(w, h, depth))
	{
		bm->rp = (struct RastPort *)AllocMem(sizeof(struct RastPort),MEMF_PUBLIC|MEMF_CLEAR);
		if (bm->rp)
		{
			InitRastPort(bm->rp);
			bm->rp->BitMap = &bm->map;		// Tie BitMap to RastPort

			return(bm->rp);
		}
		FreeExtBM(bm);
	}

	return(NULL);
}


/*
 * Destroy an offscreen RastPort.
 */
void OSFreeRastPort(struct RastPort *rp)
{
	ExtBitMap	*bm;

	bm = (ExtBitMap *) rp->BitMap;

//	ASSERT (bm->cookie == COOKIE);
	FreeMem(rp,sizeof(struct RastPort));
	FreeExtBM(bm);
}


/*
 * Make a clone of a previously allocated offscreen bitmap
 */
struct RastPort *OSCloneRastPort(struct RastPort *rp)
{
	ExtBitMap		*bm;

	bm = (ExtBitMap *)rp->BitMap;

//	ASSERT (bm->cookie == COOKIE);
	return(OSAllocRastPort (bm->w, bm->h, bm->depth));
}
