/*********************************************************************\
*
* PlayClip2 - Play video clip
*
* $Id: playclip2.c,v 1.0 1995/05/05 15:52:16 Flick Exp $
*
* $Log: playclip2.c,v $
*Revision 1.0  1995/05/05  15:52:16  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/24/94		Marty		Created
*	05/04/94		Marty		Added fast frame-indexing
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);

struct Library *FlyerBase;
struct FlyerVolume FlyVol;
struct ClipAction	 Action;

BPTR	handle;

main(int argc,char **argv)
{
	int	vidchan,drive,steal,kill,wait,audio,matte,nopreroll,useheads;
	ULONG	res,frame,clock,goclock,frames;
	char	*ptr;

	if ((argc!=8)||(argv[1][0]=='?')) {
		printf("Usage: %ls <vchan> <drive> <name> <frame> <S/K/W/A/M/N/H> <clock> <frames>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&vidchan,0,0xFF,"Video channel");
	GetArg(argv[2],&drive,0,23,"SCSI drive");
	GetLongArg(argv[4],&frame,0,0x7FFFFFFF,"Start Frame");
	GetLongArg(argv[6],&goclock,0,0x7FFFFFFF,"Go Clock");
	GetLongArg(argv[7],&frames,0,0x7FFFFFFF,"Frame Count");

	steal = kill = wait = audio = matte = nopreroll = useheads = 0;

	for (ptr=argv[5];*ptr;ptr++) {
		switch (toupper(*ptr)) {
			case 'S': steal = 1; break;
			case 'K': kill  = 1; break;
			case 'W': wait  = 1; break;
			case 'A': audio = 1; break;
			case 'M': matte = 1; break;
			case 'N': nopreroll = 1; break;
			case 'H': useheads = 1; break;
		}
	}

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	handle = Input();

	FlyVol.Path = argv[3];
	FlyVol.Board = 0;
	FlyVol.SCSIdrive = drive;

	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_STARTED;

	Action.Channel = vidchan;
	Action.Flags = CAF_VIDEO;
	if (audio)
		Action.Flags |= CAF_AUDIOL | CAF_AUDIOR;
	if (matte) {
		Action.Flags |= CAF_USEMATTE;
		Action.MatteY = 0;
	}
	if (nopreroll) {
		Action.Flags |= CAF_NOPREROLL;
	}

	Action.PermissFlags = 0;
	if (steal)
		Action.PermissFlags |= CAPF_STEALOURVIDEO;
	if (kill)
		Action.PermissFlags |= CAPF_KILLOTHERVIDEO;
	if (!wait)
		Action.PermissFlags |= CAPF_ERRIFBUSY;
	if (useheads)
		Action.PermissFlags |= CAPF_USEHEADS;

	Action.VidStartField = frame * 4;
	Action.VidFieldCount = frames * 4;
	Action.TotalAudStart = Action.AudStartField = frame * 4;
	Action.TotalAudLength = Action.AudFieldCount = frames * 4;
	Action.GoClock = goclock;
	Action.VolSust1 = 0xFFFF;
	Action.VolSust2 = 0xFFFF;
	Action.AudioPan1 = -0x8000;
	Action.AudioPan2 = 0x7FFF;


	res = FlyerPlay(&Action);
	if (res==FERR_OKAY) {
		res = GetFieldClock(&clock);
		printf("Started at clock %lx\n",clock);

		printf("Press RETURN to abort\n");

		printf("Used channel %lx\n",Action.Channel);
		for (;;) {
			if (WaitForChar(handle,30)) {
				if (getchar()=='a') {
					Action.Flags ^= (CAF_AUDIOL | CAF_AUDIOR);		/* Toggle audio */
					printf("Audio turned %ls\n",(Action.Flags & (CAF_AUDIOL | CAF_AUDIOR))?"on":"off");
					res = ChangeAudio(&Action);
					getchar();
				}
				else {
					res = AbortAction(&Action);
					printf("Aborted\n");
					break;
				}
			}
			res = CheckAction(&Action);
			if (res != FERR_BUSY) {
				printf("Complete\n");
				break;
			}
		}
	}

	printf("Last frame played = %ld\n",Action.LastFieldDone>>2);

	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
	else
		printf("Okay\n");

	CloseLibrary(FlyerBase);

	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}


void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
