/********************************************************************
* $popup.c - A popup menu thing$
*
* $Id: popup.c,v 1.0 1995/10/09 16:36:35 Flick Exp $
*
* $Log: popup.c,v $
*Revision 1.0  1995/10/09  16:36:35  Flick
*FirstCheckIn
*
*
* Copyright (c)1993-1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved.
*
* Original code donated by Stuart F.
*
*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <graphics/rastport.h>

#include <popup.h>
#include <graphichelp.p>

#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>

#ifndef PROTO_PASS
#include <proto.h>
#endif

// mask for all operations
extern UBYTE IntPlanePick;

/*
 * Formatting stuff.
 */
extern struct TextExtent LastExtent;
extern struct TextFont *EditFont;

/*
 * Multi Choice Pop Up Menu.  A menu has a number of choices, a current
 * choice, and a call-back function to allow clients to specify the text
 * string for each choice.
 */
typedef struct st_PopUpChoice {
	NameFunc				*name;		// Converts an item index to its string pointer
	void					*userData;	// Passed to name function
	PopupRender			*format;		// Holds formatting info
	struct IntuiText	itext;		// Used to render item strings
	int					num;			// Number of items in popup
	int					current;		// Current item in popup

	// Display state parms
	long			w, h;			// Imagery area for list
	int			numShown;	// Number of items shown in list at once
	int			curShown;	// Current selection or -1
	int			topItem;		// item at top of list
	int			itemH;		// Pixel height of each item in list (font + gap)

	// Windowing display state
	long					x, y;		// Top-left corner of list
	struct RastPort	*disp;	// RastPort/BitMap for composition
	struct RastPort	scrrp;	// screen rastport
	struct BitMap		*sbm;		// screen bitmap
	struct RastPort	*trp;		// Pattern buffer (holds original screen data)
	struct Window		*win;		// host window
	struct Layer_Info	*li;		// screen layers (locked)

	// SH: added 5-23-93 for BorderBox corners
	struct RastPort	*ScreenRP;
	ULONG					last_qual;	// IEQUAL_* on mouseup
} PopUpChoice;


/*
 * Init render info to default values.
 */
void PUCDefaultRender(PopupRender *format)
{
	format->bgColor		= 2;
	format->textColor		= 3;
	format->xMargin		= 7;
	format->yMargin		= 12;
	format->textGap		= 2;
	format->toggleMargin	= 2;

	format->edgeColor		= 1;
	format->arrowColor	= 3;
	format->edgeWidth		= 2;
	format->edgeHeight	= 8;

	format->font			= EditFont;
	format->drawBG			= NULL;
	format->textPosition	= 1;			// Center text in items
}


/*
 * Create a popup choice.
 */
PopUpID PUCCreate(NameFunc *name, void *userData, PopupRender *format)
{
	PopUpChoice		*puc;
	static PopupRender	 defaultFormat;

	puc = AllocMem(sizeof(PopUpChoice),MEMF_PUBLIC|MEMF_CLEAR);
	if (puc)
	{
		puc->num = 0;			// No items in popup (yet)
		puc->current = 0;
		puc->name = name;
		puc->userData = userData;

		if (!format)			// If no format struct provided, setup/use a default
		{
			if (!defaultFormat.font)
				PUCDefaultRender (&defaultFormat);
			format = &defaultFormat;
		}

		puc->format = format;
		puc->itext.FrontPen = format->textColor;
		puc->itext.DrawMode = JAM2;
		puc->itext.TopEdge  = format->textGap / 2;
		puc->itext.ITextFont = NULL;		// format->font;
	}
	return(puc);
}


/*
 * Destroy a popup choice.
 */
ULONG PUCDestroy(PopUpID puc)
{
	ULONG	last_qual = puc->last_qual;

	FreeMem(puc,sizeof(PopUpChoice));
	return(last_qual);
}


/*
 * Set the number of items in the choice.
 */
void PUCSetNumItems(PopUpID puc, int numItems)
{
	puc->num = numItems;
}


/*
 * Set the current choice.
 */
void PUCSetCurItem(PopUpID puc, int current)
{
	puc->current = current;
}


/*
 * Return the current choice.
 */
int PUCCurItem(PopUpID puc)
{
	return(puc->current);
}


/*
 * Swap the contents of the composition rastport up on the window at the
 * calculated coordinates.
 */
static BOOL SwapBegin(PopUpChoice *puc)
{
	struct BitMap	*sbm;
	BOOL	success = FALSE;

	sbm = puc->win->RPort->BitMap;
	if (sbm)
	{
		puc->trp = OSCloneRastPort(puc->disp);		// Make RP same size as offscreen one
		if (puc->trp)
		{
			InitRastPort(&puc->scrrp);
			puc->scrrp.BitMap = sbm;
			puc->scrrp.BgPen = puc->format->bgColor;
			puc->sbm = sbm;

			puc->li = puc->win->WLayer->LayerInfo;
			//LockLayers (puc->li);

			// Copy screen to temp
			BltBitMap (sbm, puc->x, puc->y, puc->trp->BitMap, 0L, 0L,
			   puc->w, puc->h, 0xC0L, IntPlanePick, NULL);
			// Copy list to screen
			BltBitMap (puc->disp->BitMap, 0L, 0L, sbm, puc->x, puc->y,
			   puc->w, puc->h, 0xC0L, IntPlanePick, NULL);

			WaitBlit();

			//	OSFreeRastPort(puc->trp);		// Only if something failed

			success = TRUE;
		}
	}

	return(success);
}


/*
 * Take down the Swap display and restore original bitmap.  Reverse of the
 * above.  SwapState is freed.
 */
static void SwapEnd(PopUpChoice *puc)
{
	// Move original screen data back
	BltBitMap (puc->trp->BitMap, 0L, 0L, puc->sbm, puc->x, puc->y,
		   puc->w, puc->h, 0xC0L, IntPlanePick, NULL);
	WaitBlit();

	//UnlockLayers (puc->li);

	OSFreeRastPort(puc->trp);		// Free off-screen rastport/bitmap
}


/*
 * Compute menu size and initial position.  Returns FALSE if this is not
 * possible.  This sets all the display state variables to their initial
 * values.
 */
static BOOL CalcStartState(PopUpChoice *puc, struct Window *win, LONG x0, LONG y0)
{
	struct Screen		*scn = win->WScreen;
	struct IntuiText	*it = &puc->itext;
	PopupRender		*fmt = puc->format;
	long			 w, h, x, y, z;
	int			 i;

	puc->win = win;

	// Calculate menu image width from all item strings
	w = 0;
	for (i=0; i<puc->num; i++)
	{
		it->IText = (UBYTE *) (*puc->name) (puc->userData, i);
		z = IntuiTextLength(it);
		if (z > w)
			w = z;
	}

	w += fmt->xMargin * 2;		// Plus left/right margins

	if (w > scn->Width)		// Too wide for screen?
		return(FALSE);

	/*
	 * Set number of items to show and the height to match.  If the
	 * whole list can be shown in 4/5 screen height, show it all,
	 * otherwise show part of list in 2/3 screen height.  This
	 * prevents having like one or two items in the scroll area.
	 */
	puc->itemH = fmt->font->tf_YSize + fmt->textGap;

	z = (4 * scn->Height / 5 - fmt->yMargin * 2) / puc->itemH;		// # items in 4/5?
	if (z >= puc->num)
		puc->numShown = puc->num;			// Will show all
	else
	{
		z = (2 * scn->Height / 3 - fmt->yMargin * 2) / puc->itemH;	// # items in 2/3?
		puc->numShown = z;
	}
	h = puc->numShown * puc->itemH + fmt->yMargin * 2;			// Actual height of list

	/*
	 * Determine menu X-position.  Just as close to centered on x0
	 * as is possible on the screen.
	 */
	x = x0 - w/2;
	if (x < 0)								// Hit left edge?
		x = 0;
	else if ((x+w) > scn->Width)		// Hit right edge?
		x = scn->Width - w;

	/*
	 * Determine menu Y-position.  Ideally, the top item would be
	 * item 0 for small menus, or the item that centers the menu
	 * box around the current choice for large menus.  This must
	 * be varied, however, to make the box fit on screen.  (z is
	 * the top item number.)
	 */
	z = puc->current - puc->numShown / 2;			// Try to center around current
	if (z < 0)
		z = 0;
	else if (z + puc->numShown > puc->num)
		z = puc->num - puc->numShown;

	while (1)
	{
		/*
		 * Calculate Y based on current top item and iterate if it
		 * is not on screen.
		 */
		y = y0 - (puc->current - z) * puc->itemH - fmt->yMargin - puc->itemH/2;
		if (y < 0)
			z++;						// Off top of screen, so bump top item to next
		else if ((y+h) > scn->Height)
			z--;						// Off bot of screen, so bump top item to prev
		else
			break;
	}

	puc->topItem = z;				// Index of first item shown in list
	puc->x = x;						// Position & size info
	puc->y = y;
	puc->w = w;
	puc->h = h;

	puc->curShown = -1;			// None shown yet

	return(TRUE);
}


/*
 * Draw an arrow of the given height, start width, width increment,
 * direction and x-center.  Drawing is done with current pen.
 */
static void PaintWedge(struct RastPort *rp,
	long			 x,
	long			 y,
	int			 w,
	int			 dw,
	int			 h,
	int			 dir)
{
	while (h--)
	{
		RectFill (rp, x-w+1, y, x+w-1, y);
		y += dir;
		w += dw;
	}
}


static void PaintArrow (struct RastPort *rp, PopupRender *fmt,
	long			 x0,
	long			 y0,
	int			 dir)
{
	int			 h;

	h = fmt->edgeHeight - 4;
	if (h > 1)
	{
		SetAPen (rp, fmt->arrowColor);
		PaintWedge(rp, x0, y0 + 2 * dir, 1, 1, h, dir);
	}
}


/*
 * Create menu imagery (to composition rastport)
 */
static void CalcPUCImage(PopUpChoice *puc)
{
	struct RastPort	*d = puc->disp;
	struct IntuiText	*it = &puc->itext;
	PopupRender		*fmt = puc->format;
	int	i, k,len, arrowT,arrowB;
	long	x0;

	// Determine if we need up/down arrows
	arrowT = (puc->topItem > 0);
	arrowB = (puc->topItem + puc->numShown < puc->num);

	if (fmt->drawBG)				// Have user-supplied fn to draw background of list box?
	{
		//SetRast (d, fmt->bgColor);

		//*** Fetch corner pixels underneath so we can round corners of image
		SaveBorder(puc->ScreenRP, puc->x, puc->y, puc->x+puc->w-1, puc->y+puc->h-1);
		//*** Draw user-defined box with user-defined up/down arrows (as needed)
		(*fmt->drawBG) (fmt, d, puc->w, puc->h, arrowT, arrowB);
	}
	else
	{
		SetRast (d, fmt->edgeColor);		// Set whole box to edge color
		SetAPen (d, fmt->bgColor);
		RectFill (d, fmt->edgeWidth, fmt->edgeHeight,
			  puc->w - fmt->edgeWidth - 1,
			  puc->h - fmt->edgeHeight - 1);

		x0 = puc->w / 2;
		if (arrowT)
			PaintArrow (d, fmt, x0, 0L, 1);
		if (arrowB)
			PaintArrow (d, fmt, x0, puc->h - 1, -1);
	}

	for (i=0; i<puc->numShown; i++)
	{
		k = i + puc->topItem;			// k is actual item index
		if ((k<0) || (k >= puc->num))
			continue;						// Prevent illegal things

		it->IText = (UBYTE *) (*puc->name) (puc->userData, k);		// Get name of item

#if OLDWAY_FIXED_FONTS
		if (fmt->textPosition == 0)
			x0 = fmt->xMargin;
		else if (fmt->textPosition == 1)
			x0 = (puc->w - IntuiTextLength (it)) / 2;
		else
			x0 = puc->w - fmt->xMargin - IntuiTextLength (it);
		PrintIText (d, it, x0, fmt->yMargin + i * puc->itemH);
#else
// SH: change for proportional fonts
		SetFont(d,EditFont);
		len = SafeFitText(d,it->IText,strlen(it->IText),puc->w-8,FALSE);

		if (fmt->textPosition == 0)					// Left
			x0 = fmt->xMargin;
		else if (fmt->textPosition == 1)				// Center
			x0 = (puc->w-8 - LastExtent.te_Width)/2 +4;
		else													// Right
			x0 = puc->w-8 - LastExtent.te_Width;

		Move(d,x0,(fmt->yMargin + i*puc->itemH)+9);		// Align baseline
		SafeColorText(d,it->IText,len);
#endif
	}
}


/*
 * Toggle highlight on item number i.
 */
static void ToggleItem(PopUpChoice *puc, int i)
{
	PopupRender		*fmt = puc->format;
	LONG			 x, y;

	if (i == -1)
		return;

	x = puc->x + fmt->xMargin - fmt->toggleMargin;
	y = puc->y + fmt->yMargin + (i - puc->topItem) * puc->itemH;
	SetDrMd(&puc->scrrp, (LONG)COMPLEMENT);
	puc->scrrp.Mask = 0x7;							// Complement low 3 planes
	//puc->scrrp.Mask = fmt->textColor ^ fmt->bgColor;
	RectFill (&puc->scrrp, x, y,
		  puc->x + puc->w - fmt->xMargin + fmt->toggleMargin - 1,
		  y + puc->itemH - 1);
	WaitBlit();
	puc->scrrp.Mask = IntPlanePick;
}


/*
 * Track the mouse as it moves around the menu.  This returns true if the
 * last event resulted in scrolling.
 */
static BOOL TrackMenu(PopUpChoice *puc, LONG x, LONG y)
{
	PopupRender	*fmt = puc->format;
	int			new, dir;

	// Calculate which choice the mouse has landed on
	x -= puc->x;
	y -= puc->y + fmt->yMargin;
	new = -1;	// None selected by default
	dir = 0;		// No scrolling by default
	if ((y>=0) && (y < (puc->h - fmt->yMargin * 2)))		// In valid Y range?
	{
		// Mouse in Y-range -- get choice if X is valid
		if ((x >= 0) && (x < puc->w))
		{
			new = y / puc->itemH + puc->topItem;
			if ((new < 0) || (new >= puc->num))
				new = -1;
		}
	}
	else
	{
		// Mouse outside Y-range -- Set scrolling request
		if (y > 0)
		{
			dir = 1;			// Scroll down in list
			y -= puc->h - fmt->yMargin * 2;		// Dist from edge of list
		}
		else
		{
			dir = -1;		// Scroll up in list
			y = -y;			// Dist from edge of list
		}
	}

	/*
	 * Update display if selection state has changed.
	 */
	if (new != puc->curShown)
	{
		ToggleItem(puc, puc->curShown);		// Remove old highlight
		puc->curShown = new;						// New current
		ToggleItem(puc, puc->curShown);		// Highlight new one
	}

	/*
	 * Scrolling requested?
	 */
	if (dir == 0)
		return(FALSE);

	new = puc->topItem + dir;					// New top item
	if (((new < 0) && (puc->topItem > new))
	|| ((new > puc->num - puc->numShown) && (puc->topItem < new)))
		return(FALSE);

	/*
	 * Compute new menu raster and blt it to menu spot.
	 */
	puc->topItem = new;
	CalcPUCImage (puc);						// Draw new menu into composition area
	BltBitMap (puc->disp->BitMap, 0L,0L,		// Copy to screen
		puc->sbm, puc->x, puc->y, puc->w, puc->h,
		0xC0L, IntPlanePick, NULL);

	return(TRUE);
}


/*
 * Activate the popup menu at the given location on the given window.
 */
int PUCActivate(PopUpID puc, struct Window *win,
	LONG x0, LONG y0,		// Center of popup gadget
	LONG x, LONG y)		// Location of mouse at activation
{
	struct IntuiMessage	*im, *imp;
	long			class;
	int			track, scroll;
	BOOL	done;

	//Ensure that parent window has REPORTMOUSE, MOUSEBUTTONS, MOUSEMOVE events coming

	done = FALSE;

	// Translate to screen-relative coordinates.
	x0 += win->LeftEdge;
	y0 += win->TopEdge;
	x += win->LeftEdge;
	y += win->TopEdge;

	// Compute initial conditions.
	puc->ScreenRP = &win->WScreen->RastPort;

	// Calc size and position of item list, which item is at top, etc.
	CalcStartState(puc, win, x0, y0);			// No error checking!!!

	// Create an offscreen rastport
	puc->disp = OSAllocRastPort ((int)puc->w, (int)puc->h, (int)(win->RPort->BitMap->Depth));
	if (puc->disp)
	{
		CalcPUCImage(puc);		// Draw popup list into composition rastport

		// Swap in menu image and give it primal event.
		if (SwapBegin(puc))		// Swap menu image in for screen data underneath
		{
			TrackMenu(puc, x, y);	// Do (un)highlighting, detect need to scroll

			scroll = 0;
			imp = NULL;
			while (!done)
			{
				/*
				 * Pull next ready message.  This can be the pending message
				 * (imp), or one from the queue if none pending.
				 */
				if (imp)
				{
					im = imp;
					imp = NULL;
				}
				else
				{
					im = (struct IntuiMessage *) GetMsg(win->UserPort);
					if (!im && !scroll)
					{
						WaitPort(win->UserPort);
						continue;
					}
				}

				/*
				 * Process message if there was one.
				 */
				track = scroll;
				if (im)
				{
					class = im->Class;
					x = im->MouseX;
					y = im->MouseY;
					puc->last_qual = im->Qualifier;
					ReplyMsg((struct Message *) im);

					switch (class)
					{
						case MOUSEBUTTONS:
						done = TRUE;			// Must have gotten left button up,
						break;					// but right button down would end us too

					   case MOUSEMOVE:
						/*
						 * Consume any messages that are just more
						 * moves waiting in the input stream.  If we
						 * hit a message that is not a mousemove,
						 * leave it as pending for the next loop.
						 */
						while (1)
						{
							imp = (struct IntuiMessage *) GetMsg(win->UserPort);
							if (!imp || (imp->Class != MOUSEMOVE))
								break;
							x = imp->MouseX;
							y = imp->MouseY;
							ReplyMsg((struct Message *) imp);
						}
						x += win->LeftEdge;
						y += win->TopEdge;
						track = 1;
						break;
					}
				}

				if (track)
					scroll = TrackMenu(puc, x, y);
			}
//			ASSERT (!imp);

			SwapEnd (puc);							// Swap original screen data back
		}
		OSFreeRastPort(puc->disp);		// Destroy offscreen rastport & bitmap
	}

//	CkptEnd ();
	return(puc->curShown);
}
