/*********************************************************************\
*
* FileInfo - List info on a Flyer file
*
* $Id: FileInfo.c,v 1.2 1995/10/16 17:18:21 Flick Exp $
*
* $Log: FileInfo.c,v $
*Revision 1.2  1995/10/16  17:18:21  Flick
*Added recognition of HQ5 clips
*
*Revision 1.1  1995/06/15  17:53:59  Flick
*Handles clips with bytes lengths > 32 bits
*
*Revision 1.0  1995/05/05  15:49:10  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/24/94		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);

struct Library *FlyerBase;
struct FlyerVolume FlyVol;

struct ClipInfo	Buffer;
struct SMPTEinfo		SmpteInfo;

main(int argc,char **argv)
{
	int	unit,drive;
	ULONG	res,meg10,sec10;

	if ((argc!=4)||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <drive> <name>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	GetArg(argv[2],&drive,0,23,"Drive");

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	FlyVol.Board = unit;
	FlyVol.Flags = FVF_USENUMS;
	FlyVol.Path = argv[3];
	FlyVol.SCSIdrive = drive;

	Buffer.len = sizeof(struct ClipInfo);
	res = GetClipInfo(&FlyVol,&Buffer);
	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
	else {
		printf("Name: %ls\n",Buffer.Name);
		if (Buffer.Type == FLYER_TYPE_FILE) {
			meg10 = (Buffer.Length / 100000) + (Buffer.LengthExt * 42950);
			printf("Size: $%lx blocks (%ld.%ld MB)\n",(Buffer.Length/512) + (Buffer.LengthExt * 8388608),
				meg10/10,meg10%10);
			printf("Start: $%lx\n",Buffer.Start);
			sec10 = Buffer.Fields / 6;
			printf("Fields: %ld (%ld.%ld seconds)\n",Buffer.Fields,
				sec10/10,sec10%10);
			printf("Index blk: %lx\n",Buffer.IndexBlk);
			printf("Clip contains: ");
			if (Buffer.Flags & CIF_HASVIDEO)
			{
				printf("video ");
				if (Buffer.VideoGrade == VG_HQ5)
					printf("(HQ5) ");
			}
			if (Buffer.Flags & CIF_HASAUDIO)
				printf("audio -- %ld channel(s)",Buffer.NumAudChans);
			printf("\n");

/*			printf("Length is now %ld\n",Buffer.len); */

			/* See if field had SMPTE info */
			res = GetSMPTE(FlyVol.Board,&SmpteInfo);
			if ((res==FERR_OKAY)&&(SmpteInfo.SMPTEvalid)) {
				printf("SMPTE start time: %02ld:%02ld:%02ld:%02ld %ls\n",
					SmpteInfo.SMPTEhours,SmpteInfo.SMPTEminutes,
					SmpteInfo.SMPTEseconds,SmpteInfo.SMPTEframes,
					(SmpteInfo.SMPTEflags & SIF_DropFrame)?"(DF)":" ");
			}
			else {
				printf("No SMPTE start time\n");
			}
		}
		else
			printf("Is a sub-directory\n");
	}

	CloseLibrary(FlyerBase);

	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
