.MODULE Predictor;
 
{
        This routine computes the LPC coefficients for the input data.

        Calling Parameters
                I0 --> Input Buffer             L0 = 0
                I1 --> Output Buffer            L1 = 0
                L2,L3,L4,L5,L6,L7 = 0
 
        Return Values
                Output buffer filled
                k[10].....k[1], PITCH

        Altered Registers
                I0,I1,I2,I4,I5,I6,M0,M1,M2,M4,M5,M6,M7
                AX0,AY0,AX1,AY1,AR,AF
                MX0,MY0,MX1,MY1,MR,MF
                SI,SE,SR

        Computation Time
                34,000 cycles (approximately)
}

.INCLUDE        <divide.mac>;
.INCLUDE        <lpcconst.h>;
 
.VAR/DM/RAM     a_ping[p], a_pong[p], dmhold[p];
.VAR/DM/RAM     e, i, in_a, out_a;
 
.VAR/PM/RAM     hold[length], zeroed[p], r[ptchlength];
.VAR/PM/RAM     ra[p], re[ptchlength];
 
.EXTERNAL       correlate, pitch_detect;

.INIT           zeroed: <zero.dat>;
 
.ENTRY          l_p_analysis;

l_p_analysis:   CALL correlation;
                CALL levinson;
                CALL pitch_decision;
                RTS;
 
correlation:    AY0=I1;I1=I0;M1=1;
                I4=^hold;M5=1;
                CNTR=length;
                DO trans UNTIL CE;      {copy signal into PM}
                        AX0=DM(I0,M1);
trans:                  PM(I4,M5)=AX0;
                SE=5;CNTR=ptchlength;   {set parameters for correlate}
                M6=1;M2=-1;M4=1;M0=1;
                I5=^hold;I2=length;I6=^r;
                CALL correlate;
                RTS;

levinson:       CALL initialize;
                CALL pass_1;
                CALL recursion;
                RTS;
 
pitch_decision: AY0=^r; I0=^a_ping;
                CALL pitch_detect;
                RTS;
 
initialize:     M0=0;M4=0;M6=-1;        {set up pointers for recursion}
                AX0=^a_ping;DM(in_a)=AX0;
                AX0=^a_pong;DM(out_a)=AX0;
                SE=3;
                AX0=p-1;
                AR=AX0+AY0;
                I1=AR;                  {point to k buffer}
                AX0=p;
                AR=AX0+AY0;
                SI=AR;                  {save pitch pointer}
                AR=H#1000;MY1=H#8000;
                MF=AR*MY1 (SU);         {MF = formatted one}
                I4=^r;AX0=PM(I4,M5);
                DM(e)=AX0;              {E = r(0)}
                RTS;
 
pass_1:         AY1=PM(I4,M6);          {compute k}
                SR1=DM(e);
                SR=LSHIFT SR1 (HI);
                AX0=SR1;AY0=SR0;
                divide(AX0,AY1);
                MY0=AY0;
                I0=DM(out_a);
                MX0=AY0, MR=AR*MF (SS);
                MY1=AX0, MR=MR-MX0*MY0 (SS);
                AR=-AY0;
                DM(I0,M0)=AY0, SR=LSHIFT MR1 (HI);      {compute next E}
                DM(I1,M2)=AR, MR=SR1*MY1 (SS);
                DM(e)=MR1;              {store next E}
                SR1=DM(in_a);
                SR0=DM(out_a);
                DM(out_a)=SR1;
                DM(in_a)=SR0;
                RTS;
 
recursion:      CNTR=p-1;
                AX1=1;
                DM(i)=AX1;
                AX0=H#1000;
                DO durbin UNTIL CE;
                        I2=DM(in_a);I6=DM(in_a);
                        I4=^r+1;I5=DM(out_a);M7=DM(i);
                        MX0=PM(I4,M7), AR=PASS AX0;
                        MY1=PM(I4,M6);
                        MR=AR*MY1 (SS), MY0=PM(I4,M6), MX0=DM(I2,M1);
                        CNTR=DM(i);
                        DO loop_1 UNTIL CE;     {compute k values}
loop_1:                         MR=MR-MX0*MY0 (SS),MX0=DM(I2,M1),MY0=PM(I4,M6);
                        SR=LSHIFT MR1 (HI);
                        AY1=SR1;
                        SR1=DM(e);
                        AY0=SR0, SR=LSHIFT SR1 (HI);
                        AX1=SR1;
                        divide(AX1,AY1);        {divide by E}
                        I4=DM(in_a);
                        MX1=DM(I4,M5);
                        MODIFY(I6,M7);
                        MODIFY(I6,M6);
                        CNTR=DM(i);
                        MY0=AY0;
                        DO loop_2 UNTIL CE;     {compute new a values}
                                MR=MX1*MF(SS), MX0=DM(I6,M6);
                                MR=MR-MX0*MY0 (SS);
                                SR=LSHIFT MR1 (HI), MX1=DM(I4,M5);
loop_2:                         DM(I5,M5)=SR1;
                        MY1=DM(e);
                        I6=DM(out_a);
                        MX0=MY0, MR=AR*MF (SS);                 {MR = 1}
                        SR0=DM(I6,M7), MR=MR-MX0*MY0 (SS);      {MR = 1-k2}
                        AR=-AY0;
                        DM(I1,M2)=AR, SR=LSHIFT MR1 (HI);
                        DM(I6,M6)=AY0, MR=SR1*MY1 (SS);
                        SR=LSHIFT MR1 (HI);
                        DM(e)=SR1;
                        AY0=DM(i);
                        AR=AY0+1;
                        DM(i)=AR;
                        SR1=DM(in_a);
                        SR0=DM(out_a);
                        DM(in_a)=SR0;
durbin:                 DM(out_a)=SR1;
                RTS;
.ENDMOD;
