.MODULE pitching;

{       This routine computes the pitch period for a speech sample.
        It is used in conjunction with a Linear Predictive Coder.

        Calling Parameters
                AY0 --> Autocorrelation buffer
                I0 --> LPC Coefficient buffer          L0 = 0
                SI --> Pitch buffer
                L1,L2,L3,L4,L5,L6,L7 = 0

        Return Values
                Pitch buffer filled

        Altered Registers
                I0,I1,I2,I4,I5,I6,M1,M2,M4,M5,M6,AX0,AX1,AY0,AY1,AR,AF
                MX0,MX1,MY0,MY1,MR,MF,SI,SE,SR

        Computation Time
                approximately 1800 cycles

}

.INCLUDE        <lpcconst.h>;
.VAR/DM/RAM     rahold[p];
.VAR/PM/RAM     hold[p], ra[p], re[ptchlength];
.GLOBAL         pitch_detect;
.EXTERNAL       correlate;

pitch_detect:   CALL coeff_corr;
                CALL error_corr;
                CALL pitch_period;
                RTS;
 
coeff_corr:     M1=1;M5=1;M2=-1;
                I1=I0;I4=^hold;
                CNTR=p;
                DO move_coeff UNTIL CE;            {copy coeff. to PM}
                        AX0=DM(I0,M1);
move_coeff:             PM(I4,M5)=AX0;
                I5=^hold;CNTR=p;
                M0=1;M4=1;M6=1;I2=p;I6=^ra;
                CALL correlate;
                RTS;
 
error_corr:     I0=^rahold;I4=^ra;
                CNTR=p;
                DO move_ra UNTIL CE;               {copy ra to DM}
                        AX0=PM(I4,M5);
move_ra:                DM(I0,M1)=AX0;
                SE=5;I1=^rahold;I5=AY0;CNTR=ptchlength;
                M4=-1;M2=0;I2=p;I6=^re;
                CALL correlate;
                RTS;
 
pitch_period:   I5=^re;                            {point to re}
                M7=ptchstrt;
                MX0=M7;
                AX0=PM(I5,M7);
                AX1=PM(I5,M5), AR=ABS AX0;
                CNTR=wndolength;
                AY0=PM(I5,M5);
                IF POS JUMP max;
                DO nomaxabs UNTIL CE;              {find largest neg. number}
                        AR=AX1-AY0;
                        IF LT JUMP nomaxabs;
                        AX1=AY0;                   {find peak value}
                        AX0=I5;
                        AY0=^re;
                        AF=AX0-AY0;
                        AR=AF-1;
                        MX0=AR;                    {MX0 = peak value}
nomaxabs:               AY0=PM(I5,M5);
                JUMP pitch_compute;
max:            DO nomax UNTIL CE;
                        AR=AX1-AY0;                {find peak value}
                        IF GT JUMP nomax;
                        AX1=AY0;
                        AX0=I5;
                        AY0=^re;
                        AF=AX0-AY0;
                        AR=AF-1;
                        MX0=AR;                   {MX0 = peak value}
nomax:                  AY0=PM(I5,M5);
pitch_compute:  M0=0;
                I5=^re;
                I1=SI;
                AX0=PM(I5,M5), AR=PASS 0;
                DM(I1,M0)=AR;
                AY0=AR, AR=ABS AX0;
                SR0=AR, AF=ABS AX1;
                AR=SR0-AF;                        {r(0) < r(peak)?}
                IF LE JUMP compute;               {yes, compute pitch}
                AF=PASS AX1;                      {no, find r(peak)/r(0)}
                AX1=3;
                DIVS AF,AX0;
                DIVQ AX0;
                DIVQ AX0;
                AR=AX1 AND AY0;                   {r(peak)r(0) < 0.25?}
                IF EQ JUMP done_compute;          {yes, pitch = 0}
compute:                MY0=iperiodh;             {no, compute pitch}
                MR=MX0*MY0 (ss);
                MR1=MR0;
                MR0=0;
                MY0=iperiodl;
                MR=MR+MX0*MY0 (su);
                SR=LSHIFT MR1 BY -1 (LO);
                DM(I1,M0)=SR0;
done_compute:   RTS;
 
.ENDMOD;
