.MODULE cfir_sub;

{
        Single-Precision Complex FIR Filter Subroutine

        Calling Parameters
                I0 --> Oldest data value in real delay line (Xr's)
                L0 = filter length (N)
                I1 --> Oldest data value in imaginary delay line (Xi's)
                L1 = filter length (N)
                I4 --> Beginning of real coefficient table (Hr's)
                L4 = filter length (N)
                I5 --> Beginning of imaginary coefficient table (Hi's)
                L5 = filter length (N)
                M0,M4 = 1
                AX0 = filter length minus one (N-1)
                CNTR = filter length minus one (N-1)

        Return Values
                I0 --> Oldest data value in real delay line
                I1 --> Oldest data value in imaginary delay line
                I4 --> Beginning of real coefficient table
                I5 --> Beginning of imaginary coefficient table
                SR1 = real output (rounded and conditionally saturated)
                MR1 = imaginary output (rounded and conditionally saturated)

        Altered Registers
                MX0,MY0,MR,SR1

        Computation Time
                2*(N-1) + 2*(N-1) + 13 + 8 cycles

        All coefficients and data values are assumed to be in 1.15 format.
}

.ENTRY  cfir;

cfir:   MR=0, MX0=DM(I1,M0), MY0=PM(I5,M4);
        DO realloop UNTIL CE;
            MR=MR-MX0*MY0(SS), MX0=DM(I0,M0), MY0=PM(I4,M4);        {Xi * Hi}
realloop:   MR=MR+MX0*MY0(SS), MX0=DM(I1,M0), MY0=PM(I5,M4);        {Xr * Hr}
        MR=MR-MX0*MY0(SS), MX0=DM(I0,M0), MY0=PM(I4,M4);       {last Xi * Hi}
        MR=MR+MX0*MY0(RND);                                    {last Xr * Hr}
        IF MV SAT MR;
        SR1=MR1;                                                   {Store Yr}
        MR=0, MX0=DM(I0,M0), MY0=PM(I5,M4);
        CNTR=AX0;
        DO imagloop UNTIL CE;
            MR=MR+MX0*MY0(SS), MX0=DM(I1,M0), MY0=PM(I4,M4);        {Xr * Hi}
imagloop:   MR=MR+MX0*MY0(SS), MX0=DM(I0,M0), MY0=PM(I5,M4);        {Xi * Hr}
        MR=MR+MX0*MY0(SS), MX0=DM(I1,M0), MY0=PM(I4,M4);       {last Xr * Hi}
        MR=MR+MX0*MY0(RND);                                    {last Xi * Hr}
        IF MV SAT MR;                                                {MR1=Yi}
        RTS;
.ENDMOD;
