{
	DTMF Signal Generator using ADSP-2100

	written by: Christoph D. Cavigioli (617) 461-3058
	April 1987 - modified Oct 1987
	Analog Devices, DSP Division, P.O. Box 9106, Norwood, MA 02062
}

.module/ram/abs=0		DTMF_Dialer;

.var/dm/ram	row0,row1,row2,row3,col0,col1,col2,col3; { for reference only }
.var/dm/ram	hertz1, hertz2;		{ used by software }
.var/dm/ram	sum1, sum2;		{ used by software }
.var/dm/ram	sin1, sin2;		{ used by software }
.var/dm/ram	scale;			{ divisor for sine waves }
.var/dm/ram	state;			{ dialing state machine }
.var/dm/ram	sign_dura_ms;		{ signal duration time in millisec }
.var/dm/ram	interdigit_ms;		{ interdigit time in millisec }
.var/dm/ram	time_on;		{ used by software }
.var/dm/ram	time_off;		{ used by software }
.var/dm/ram	digits[32];		{ lookup table for digit row,col freqs }
.var/dm/ram	dial_list[100];		{ store dial sequence here }
{	
	store values in dial_list as follows:
								(E=*, F=#)
	DTMF digits	==> h#000y, where y=0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F
	quiet space	==> h#00yX, where y is non-zero, X is don't care
	redial 		==> h#0yXX, where y is non-zero, X is don't care
	stop		==> h#yXXX, where y is non-zero, X is don't care
}

.port		dac;
.port		codec;
.port		cntl_port;

.init		row0:		h#02B9;		{  697 Hz }
.init		row1:		h#0302;		{  770 Hz }
.init		row2:		h#0354;		{  852 Hz }
.init		row3:		h#03AD;		{  941 Hz }
.init		col0:		h#04B9;		{ 1209 Hz }
.init		col1:		h#0538;		{ 1336 Hz }
.init		col2:		h#05C5;		{ 1477 Hz }
.init		col3:		h#0661;		{ 1633 Hz }

.init	digits[00]: h#03AD,h#0538, h#02B9,h#04B9, h#02B9,h#0538, h#02B9,h#05C5;
.init	digits[08]: h#0302,h#04B9, h#0302,h#0538, h#0302,h#05C5, h#0354,h#04B9;
.init	digits[16]: h#0354,h#0538, h#0354,h#05C5, h#02B9,h#0661, h#0302,h#0661;
.init	digits[24]: h#0354,h#0661, h#03AD,h#0661, h#03AD,h#04B9, h#03AD,h#05C5;

.init		scale:		h#FFFC;
.init		sign_dura_ms:	h#0064;		{ 100 ms }
.init		interdigit_ms:	h#0064;		{ 100 ms }

.init		dial_list[0]:	h#00F0;		{ dials author: 1-617-461-3058 }
.init		dial_list[1]:	h#0001;
.init		dial_list[2]:	h#00F0;
.init		dial_list[3]:	h#0006;
.init		dial_list[4]:	h#0001;
.init		dial_list[5]:	h#0007;
.init		dial_list[6]:	h#00F0;
.init		dial_list[7]:	h#0004;
.init		dial_list[8]:	h#0006;
.init		dial_list[9]:	h#0001;
.init		dial_list[10]:	h#00F0;
.init		dial_list[11]:	h#0003;
.init		dial_list[12]:	h#0000;
.init		dial_list[13]:	h#0005;
.init		dial_list[14]:	h#0008;
.init		dial_list[15]:	h#FFFF;

.external	u_compress;
.external	sin;

IRQ0:		rti; 
IRQ1:		rti; 
IRQ2:		jump next_state; 
IRQ3:		jump eight_khz;

setup:		si=0;
		dm(cntl_port)=si;
		dm(state)=si;
		call reset;
		l0=0; l1=0; l2=0; l3=0; l4=0; l5=0; l6=0; l7=0; 
		i0=^dial_list;
		m0=1; 
		m3=1; l3=0; 		{ used by sine routine }
		icntl=b#01111;
		imask= b#1100;
wait_int:	jump wait_int;

eight_khz:	ay0=2;
		ax0=dm(state);
		ar=ax0-ay0;
		if eq jump state2;
		af=ay0-1;
		ar=ax0-af;
		if eq jump state1;
state0:		rti;
state1:		ax0=350;
		dm(hertz1)=ax0;
		ax0=440;
		dm(hertz2)=ax0;
		jump maketones;
state2:		ay0=dm(time_on);
		ar=pass ay0;
		if eq jump quiet;
		ar=ay0-1;
		dm(time_on)=ar;

maketones:	se=dm(scale);

tone1:		ay0=dm(sum1);
		si=dm(hertz1);
		sr=ashift si by +3 (hi);
		my0=h#4189;			{ mult Hz by .512 * 2 }
		mr=sr1*my0(rnd);		{ i.e. mult by 1.024 }
		sr=ashift mr1 by +1 (hi);
		ar=sr1+ay0;
		dm(sum1)=ar;
		ax0=ar;
		call sin;
		sr=ashift ar (hi);		{ scale value in SE }
		dm(sin1)=sr1;

tone2:		ay0=dm(sum2);
		si=dm(hertz2);
		sr=ashift si by +3 (hi);
		my0=h#4189;			{ mult Hz by .512 * 2 }
		mr=sr1*my0(rnd);		{ i.e. mult by 1.024 }
		sr=ashift mr1 by +1 (hi);
		ar=sr1+ay0;
		dm(sum2)=ar;
		ax0=ar;
		call sin;
		sr=ashift ar (hi);		{ scale value in SE }
		dm(sin2)=sr1;

add_em:		ax0=dm(sin1);
		ay0=dm(sin2);
		ar=ax0+ay0;

sound:		ay0=h#8000;
		ar=ar xor ay0;
		dm(dac)=ar;
		ar=ar xor ay0;
		call u_compress;
		dm(codec)=ar;
		rti;

quiet:		ay0=dm(time_off);
		ar=pass ay0;
		if eq jump nextdigit;
		ar=ay0-1;
		dm(time_off)=ar;

		ay0=h#8000;
		ar=h#7FFF;
		dm(dac)=ar;
		ar=ar xor ay0;
		call u_compress;
		dm(codec)=ar;
		rti;

nextdigit:	call reset;
		ax0=dm(i0,m0);			{ read next digit out of list }
		ay0=h#F000;
		ar=ax0 and ay0;
		if eq jump notstop;
stop:		ar=0;
		dm(state)=ar;
		rti;
notstop:	ay0=h#0F00;
		ar=ax0 and ay0;
		if eq jump notredial;
redial:		i0=^dial_list;
		rti;
notredial:	ay0=h#00F0;
		ar=ax0 and ay0;
		if eq jump newdigit;
space:		ax0=dm(time_on);
		ay0=dm(time_off);
		ar=ax0+ay0;
		dm(time_off)=ar;
		ar=0;
		dm(time_on)=ar;
		rti;
newdigit:	ay0=h#000F;
		ar=ax0 and ay0;
		sr=lshift ar by +1 (hi);
		ay0=^digits;
		ar=sr1+ay0;
		i1=ar;
		ax0=dm(i1,m0);			{ look up row freq for digit }
		dm(hertz1)=ax0;
		ax0=dm(i1,m0);			{ look up col freq for digit }
		dm(hertz2)=ax0;
		rti;

reset:		si=0;
		dm(sum1)=si;
		dm(sum2)=si;

		si=dm(sign_dura_ms);
		sr=ashift si by +3 (hi);
		ay0=sr1;
		ar=ay0-1;
		dm(time_on)=ar;

		si=dm(interdigit_ms);
		sr=ashift si by +3 (hi);
		ay0=sr1;
		ar=ay0-1;
		dm(time_off)=ar;
		rts;

next_state:	i0=^dial_list;
		call reset;
		ay0=dm(state);
		ar=ay0+1;
		dm(state)=ar;
		ay0=3;
		ar=ar-ay0;		{ modulus 3, i.e. no state 3 exists }
		if ne rti;
		ar=0;
		dm(state)=ar;
		rti;

.endmod;
