.module/ram	DTMF_LUT;
{
	This subroutine reads data from a memory-mapped 16-key keypad
	encoder (74C922).  The 16-key keypad is organized as shown in 
	the matrix on the left below, but interpreted by the software
	as a DTMF keypad as shown in the matrix on the right below:

PHYSICAL KEY MAPPINGS:
	16-key:	3 2 1 0		DTMF:	1 2 3 A
		7 6 5 4			4 5 6 B
		B A 9 8			7 8 9 C
		F E D C			* 0 # D

USAGE:
	.var hertz1, hertz2;
	.global hertz1, hertz2;
	.external readpad;
	m6=1;
	l6=0;
	call readpad;

INPUT PARAMETERS:
	< input data read from keypad via 74C922 >

OUTPUT PARAMETERS:
	DM variables 'hertz1' and 'hertz2' filled with tone frequency value

REGISTERS AFFECTED:
	AX0, AX1, AY0, AY1, AR, SR1, SR0, I6

EXECUTION TIME:
	12 instruction cycles

TONE LOOK-UP TABLE ORGANIZATION:
	74C922	DTMF	LUT_OFFSET	FREQUENCY
	
	0	A	0		 697 Hz
			1		1633 Hz
	1	3	2		 697 Hz
			3		1477 Hz
	2	2	4		 697 Hz
			5		1336 Hz
	3	1	6		 697 Hz
			7		1209 Hz
	4	B	8		 770 Hz
			9		1633 Hz
	5	6	10		 770 Hz
			11		1477 Hz
	6	5	12		 770 Hz
			13		1336 Hz
	7	4	14		 770 Hz
			15		1209 Hz
	8	C	16		 852 Hz
			17		1633 Hz
	9	9	18		 852 Hz
			19		1477 Hz
	A	8	20		 852 Hz
			21		1336 Hz
	B	7	22		 852 Hz
			23		1209 Hz
	C	D	24		 941 Hz
			25		1633 Hz
	D	#	26		 941 Hz
			27		1477 Hz
	E	0	28		 941 Hz
			29		1336 Hz
	F	*	30		 941 Hz
			31		1209 Hz
}
.var/dm/ram	Tone_LUT[32];
.port		keypad;
.external	hertz1;
.external	hertz2;
.entry		readpad;

.init	Tone_LUT[00]:	697, 1633;
.init	Tone_LUT[02]:	697, 1477;
.init	Tone_LUT[04]:	697, 1336;
.init	Tone_LUT[06]:	697, 1209;

.init	Tone_LUT[08]:	770, 1633;
.init	Tone_LUT[10]:	770, 1477;
.init	Tone_LUT[12]:	770, 1336;
.init	Tone_LUT[14]:	770, 1209;

.init	Tone_LUT[16]:	852, 1633;
.init	Tone_LUT[18]:	852, 1477;
.init	Tone_LUT[20]:	852, 1336;
.init	Tone_LUT[22]:	852, 1209;

.init	Tone_LUT[24]:	941, 1633;
.init	Tone_LUT[26]:	941, 1477;
.init	Tone_LUT[28]:	941, 1336;
.init	Tone_LUT[30]:	941, 1209;

readpad:	ax0=dm(keypad);		{ read keypad via 74C922 }
		ay0=h#000F;
		ar=ax0 and ay0;		{ need four LSBs only }
		sr=lshift ar by 1 (hi);	{ multiply by two for LUT_OFFSET }
		ay1=^Tone_LUT;
		ar=sr1+ay1;		{ add table base address to offset }
		i6=ar;
		ax1=dm(i6,m6);
		dm(hertz1)=ax1;		{ remember ROW tone frequency }
		ax1=dm(i6,m6);
		dm(hertz2)=ax1;		{ remember COL tone frequency }
		rts;

.endmod;
