{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{                                                                              }
{	Multi Channel DTMF Decoder (six channels shown here)                   }
{                                                                              }
{	c. Analog Devices, Inc. 1987,1988 --- Norwood, MA 02062                }
{	written by Christoph D. Cavigioli - DSP Applications                   }
{                                                                              }
{	INPUT:	one voiceband telephone codec per channel                      }
{	OUTPUT:	hexadecimal code for DTMF digit detected at each channel       }
{		is written to that channel's output port (in this example,     }
{		output port is a D/A converter for display on oscilloscope)    }
{                                                                              }
{	This example runs on the ADSP-2100 Evaluation board, where Channel A   }
{	is implemented by on-board hardware, and channels B-F are processed    }
{	but fictitious.                                                        }
{                                                                              }
.module/ram/abs=0	Multi_CH_DTMF_Dec;

.const	channels	= 6;		{ must be 2 or more !!! }
	{	edit this for more channels
	}
.const	channels_x_32	= 192;
	{	edit this for more channels
	}
.const	baddigitcode	= h#FFFF;	{ output code for non-digit }
.const	pass_posttests	= 0;
.const	fail_minsig	= 1;
.const	fail_relpeak	= 2;
.const	fail_twist	= 3;
.const	fail_2ndharm	= 4;

{ *** housekeeping variables *** }
		{ the next variables are 16.0 fixed point integers }
.var	count201;		{ counts samples 1, 2, 3, ..., 201  }
.var	count4;			{ counts samples 202, 203, 204, 205 }
		{ the next variables are 1.15 fixed point fractions }
.var	min_tone_level[8];   	{ min "tone-present" mnsqr level }
.var	max_notone_level[8];   	{ max "tone-not-present" mnsqr level }
.var	max_2ndharm_level;	{ mnsqr of 2nd harm must be lt this level }
.var	maxfortwist;		{ quotient row/col  must be gt this value }
.var	maxrevtwist;		{ quotient col/row  must be gt this value }
.var	mu_lookup_table[256];	{ mu-law to linear tbl(scaled down 8 bits)}
.var/circ	in_samples[channels];	{ input samples (scaled down 8 bits) }
.var/circ	A_Q1Q2_buff[32],{ Goertzel feedback loop storage elements }
	B_Q1Q2_buff[32],
	C_Q1Q2_buff[32],
	D_Q1Q2_buff[32],
	E_Q1Q2_buff[32],
	F_Q1Q2_buff[32];
	{	etc
		etc	edit this for more channels
	}
.var	A_mnsqr[8],	{ 1.15 mnsqr Goertzel result values }
	B_mnsqr[8],
	C_mnsqr[8],
	D_mnsqr[8],
	E_mnsqr[8],
	F_mnsqr[8];
	{	etc
		etc	edit this for more channels
	}
.var/pm/ram/circ
	coefs[16];	{ 2.14 Goertzel coefs: 2*cos(2*PI*k/N) }

{ *** individual channel variables *** }
.var	A_maxrowval;		{ 1.15 mnsqr value of max row fundamental }
.var	A_maxcolval;		{ 1.15 mnsqr value of max col fundamental }
.var	A_whichrow;	{ 0,1,2,3,4 means: invalid, row1, row2, row3, row4 }
.var	A_whichcol;	{ 0,1,2,3,4 means: invalid, col1, col2, col3, col4 }
.var	A_fortwistflag;		{ 1,0 "boolean" 1 means forward twist exists }
.var	A_revtwistflag;		{ 1,0 "boolean" 1 means reverse twist exists }
.var	A_twistval;		{ 1.15 mnsqr quotient row/col or col/row }
.var	A_rowharm;		{ 1.15 mnsqr value of row second harmonic }
.var	A_colharm;		{ 1.15 mnsqr value of col second harmonic }
.var	A_digit_history[2];	{ remember past 2 digits (non-digits) }
.var	A_failurecode;		{ see .const definitions above }

.var	B_maxrowval;
.var	B_maxcolval;
.var	B_whichrow;
.var	B_whichcol;
.var	B_fortwistflag;
.var	B_revtwistflag;
.var	B_twistval;
.var	B_rowharm;
.var	B_colharm;
.var	B_digit_history[2];
.var	B_failurecode;

.var	C_maxrowval;
.var	C_maxcolval;
.var	C_whichrow;
.var	C_whichcol;
.var	C_fortwistflag;
.var	C_revtwistflag;
.var	C_twistval;
.var	C_rowharm;
.var	C_colharm;
.var	C_digit_history[2];
.var	C_failurecode;

.var	D_maxrowval;
.var	D_maxcolval;
.var	D_whichrow;
.var	D_whichcol;
.var	D_fortwistflag;
.var	D_revtwistflag;
.var	D_twistval;
.var	D_rowharm;
.var	D_colharm;
.var	D_digit_history[2];
.var	D_failurecode;

.var	E_maxrowval;
.var	E_maxcolval;
.var	E_whichrow;
.var	E_whichcol;
.var	E_fortwistflag;
.var	E_revtwistflag;
.var	E_twistval;
.var	E_rowharm;
.var	E_colharm;
.var	E_digit_history[2];
.var	E_failurecode;

.var	F_maxrowval;
.var	F_maxcolval;
.var	F_whichrow;
.var	F_whichcol;
.var	F_fortwistflag;
.var	F_revtwistflag;
.var	F_twistval;
.var	F_rowharm;
.var	F_colharm;
.var	F_digit_history[2];
.var	F_failurecode;

	{	etc
		etc	edit this for more channels
	}

{ *** individual channel I/O ports *** }
.port	A_codec;
.port	A_cntl_port;	
.port	A_dac;

.port	B_codec;
.port	B_cntl_port;
.port	B_dac;

.port	C_codec;
.port	C_cntl_port;
.port	C_dac;

.port	D_codec;
.port	D_cntl_port;
.port	D_dac;

.port	E_codec;
.port	E_cntl_port;
.port	E_dac;

.port	F_codec;
.port	F_cntl_port;
.port	F_dac;

	{	etc
		etc	edit this for more channels
	}

{ *** variable initializations *** }
.init	coefs[00]: h#6D0200, h#68B200, h#63FD00, h#5EE700;
.init	coefs[04]: h#4A7100, h#409100, h#329100, h#23CE00;
.init	coefs[08]: h#3ABC00, h#2C1700, h#1CC300, h#0CFB00;
.init	coefs[12]: h#D5CB00, h#C00000, h#A97700, h#94D000;

.init	mu_lookup_table:	< MU255.Q8 >;
.init	min_tone_level:	h#0003,h#0003,h#0003,h#0003,h#0003,h#0003,h#0003,h#0003;
.init max_notone_level:	h#0002,h#0002,h#0002,h#0002,h#0002,h#0002,h#0002,h#0002;
.init	max_2ndharm_level:	h#0100;
.init	maxfortwist:		h#32F5;
.init	maxrevtwist:		h#1449;

{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{                                                                              }
{	8-bit mu-law sample read from specified codec, then                    }
{	converted to linear (1.15 scaled down 8 bits) via look-up table        }
{                                                                              }
{	INPUT:	channel codec to read                                          }
{	OUTPUT:	scaled linear sample written into "in_samples" circular buffer }
{                                                                              }
.macro		get_sample( %0 );	{ make sure I3,M0,L3,M4,L6 initialized }
		ay0=dm(%0);		{ read codec, mu-law data }
		af=ax0 and ay0;		{ make sure AX0 initialized to h#00FF }
		ar=ax1+af;		{ make sure AX1 initialized to LUT base}
		i6=ar;
		si=dm(i6,m4);		{ look-up scaled, linear value }
		dm(i3,m0)=si;		{ store input sample }
.endmacro;

{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{                                                                              }
{	pick out the largest row and col mnsqr, remember which ones, how big   }
{                                                                              }
{	INPUT:	pointer to top of channel's mnsqr buffer                       }
{	OUTPUT:	index of detected row and col tones                            }
{		value of detected row and col tones                            }
{                                                                              }
.macro		maxrowcol( %0, %1, %2, %3, %4 );
			{ ^mnsqr, maxrowval, whichrow, maxcolval, whichcol }
.local		findmaxrow;
.local		findmaxcol;

		i2=%0;		

		ax1=0;		{ set up variables in case nothing found }
		ay1=0;		{ set up variables in case nothing found }
		ay0=0;		{ initialize BIGGEST-VALUE-SO-FAR to zero }
		cntr=4;
		do findmaxrow until ce;
			ax0=dm(i2,m0);	{ read CURRENT-MNSQR-VALUE }
			ar=ax0-ay0;	{ compare to BIGGEST-VALUE-SO-FAR }
			if le jump findmaxrow;
			ax1=5;
			ay0=ax0;	{ if CURRENT is bigger, remember value }
			ay1=cntr;	{ if CURRENT is bigger, remember index }
findmaxrow:		nop;
		dm(%1)=ay0;	{ remember the largest mnsqr value }
		ar=ax1-ay1;
		dm(%2)=ar;	{ remember index of the biggest row (1,2,3,4) }

		ax1=0;
		ay1=0;
		ay0=0;
		cntr=4;
		do findmaxcol until ce;
			ax0=dm(i2,m0);
			ar=ax0-ay0;
			if le jump findmaxcol;
			ax1=5;
			ay0=ax0;
			ay1=cntr;
findmaxcol:		nop;
		dm(%3)=ay0;
		ar=ax1-ay1;
		dm(%4)=ar;
.endmacro;

{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{                                                                              }
{	checks whether tones detected are greater than min signal power level  }
{                                                                              }
{	INPUT:	index of detected row and col tones                            }
{		value of detected row and col tones                            }
{	OUTPUT:	failurecode set if test fails                                  }
{                                                                              }
.macro		minsiglevel( %0, %1, %2, %3, %4 );
			{ whichrow, maxrowval, whichcol, maxcolval, failurecode}
.local		failsiglevel;
.local		done;

		ax0=^min_tone_level;
		ay0=dm(%0);
		af=ay0-1;
		ar=ax0+af;
		i5=ar;		{ i5 points to ^min_tone_level+whichrow-1 }
		ax1=dm(%1);
		ay1=dm(i5,m4);
		ar=ax1-ay1;
		if lt jump failsiglevel;

		ay0=3;
		ar=ax0+ay0;
		ay0=dm(%2);
		ar=ar+ay0;
		i5=ar;		{ i5 points to ^min_tone_level+4+whichcol-1 }
		ax1=dm(%3);
		ay1=dm(i5,m4);
		ar=ax1-ay1;
		if ge jump done;

failsiglevel:	ax0=0;
		dm(%0)=ax0;
		dm(%2)=ax0;
		ay0=fail_minsig;
		dm(%4)=ay0;

done:		nop;
.endmacro;

{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{                                                                              }
{	verifies that only one valid row tone and only one col tone is present }
{                                                                              }
{	INPUT:	pointer to top of channel's mnsqr buffer                       }
{		index of detected row and col tones                            }
{	OUTPUT:	failurecode set if test fails                                  }
{                                                                              }
.macro		no_other_peaks( %0, %1, %2, %3 );
			{ ^mnsqr, whichrow, whichcol, failurecode }
.local		looper;
.local		failrelpeak;
.local		done;

		i2=%0;
		i6=^max_notone_level;
		af=pass 0;
		cntr=8;
		do looper until ce;
			ax0=dm(i6,m4);
			ay0=dm(i2,m0);
			ar=ax0-ay0;
looper:			if lt af=af+1;
		ax1=2;			{ check if only 2 tones are over their }
		ar=ax1-af;		{ max notone level thresholds }
		if eq jump done;
failrelpeak:	ax0=0;			{ clear whichrow,col }
		dm(%1)=ax0;
		dm(%2)=ax0;
		ay0=fail_relpeak;
		dm(%3)=ay0;		{ set failurecode }
done:		nop;
.endmacro;

{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{                                                                              }
{	checks difference between row tone level and col tone level (twist)    }
{                                                                              }
{	INPUT:	index of detected row and col tones                            }
{		value of detected row and col tones                            }
{	OUTPUT:	forward twist flag or reverse twist flag set appropriately     }
{		twist amount as a ratio ( row/col [fwd] or col/row [rev] )     }
{		failurecode set if test fails                                  }
{                                                                              }
.macro		twisttests( %0, %1, %2, %3, %4, %5, %6, %7 );
			{ maxrowval, maxcolval, fortwistflag, revtwistflag,
			  whichrow, whichcol, twistval, failurecode }
.local		standard;
.local		reverse;
.local		failtwist;
.local		done;

		mx0=0;
		mx1=1;
		ax0=dm(%0);
		ay0=dm(%1);
		ar=ax0-ay0;
		if gt jump reverse;
standard:	dm(%2)=mx1;		{ column tone is stronger }
		dm(%3)=mx0;
		ax0=dm(%1);
		ay0=dm(%0);
		af=pass ay0;
		ay0=0;
		divs af,ax0;
		divq ax0; divq ax0; divq ax0; divq ax0; divq ax0; 
		divq ax0; divq ax0; divq ax0; divq ax0; divq ax0; 
		divq ax0; divq ax0; divq ax0; divq ax0; divq ax0; 
		dm(%6)=ay0;		{ ay0 = maxrowval / maxcolval }
		ax0=dm(maxfortwist);
		ar=ax0-ay0;
		if gt jump failtwist;
		jump done;
reverse:	dm(%2)=mx0;
		dm(%3)=mx1;		{ row tone is stronger }
		af=pass ay0;
		ay0=0;
		divs af,ax0;
		divq ax0; divq ax0; divq ax0; divq ax0; divq ax0; 
		divq ax0; divq ax0; divq ax0; divq ax0; divq ax0; 
		divq ax0; divq ax0; divq ax0; divq ax0; divq ax0; 
		dm(%6)=ay0;		{ ay0 = maxcolval / maxrowval }
		ax0=dm(maxrevtwist);
		ar=ax0-ay0;
		if gt jump failtwist;
		jump done;
failtwist:	dm(%4)=mx0;
		dm(%5)=mx0;
		ay0=fail_twist;
		dm(%7)=ay0;
done:		nop;
.endmacro;

{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{                                                                              }
{	checks for excessive energy in second harmonics of detected tones      }
{                                                                              }
{	INPUT:	pointer to top of channel's Goertzel feedback buffer           }
{		index of detected row and col tones                            }
{		pointers to variables which hold channel's 2nd harmonic levels }
{	OUTPUT:	value of channel's row and col 2nd harmonic levels             }
{		failurecode set if test fails                                  }
{                                                                              }
.macro		check2ndharm( %0, %1, %2, %3, %4, %5, %6, %7 );
			{ ^Q1Q2_buff, whichrow, ^rowharm, rowharm, 
			  whichcol, ^colharm, colharm, failurecode }
.local		fail2ndharm;
.local		done;

		ax0=%0;
		ay0=dm(%1);
		ar=ay0-1;			{ range: 1,2,3,4 ==> 0,1,2,3 }
		sr=ashift ar by +1 (hi);	{ range: 0,1,2,3 ==> 0,2,4,6 }
		ay1=16;
		af=sr1+ay1;
		ar=ax0+af;
		i0=ar;		{ i0 points to ^Q1Q2_buff+16+2*(whichrow-1) }
		ax0=^coefs;
		af=ax0+ay0;
		ax0=7;
		ar=ax0+af;
		i4=ar;		{ i4 points to ^coefs+8+whichrow-1 }
		i2=%2;		{ i2 points to ^rowharm }
		call mnsqr;
		ax0=dm(%3);
		ay0=dm(max_2ndharm_level);
		ar=ax0-ay0;
		if gt jump fail2ndharm;

		ax0=%0;
		ay0=dm(%4);
		ar=ay0-1;			{ range: 1,2,3,4 ==> 0,1,2,3 }
		sr=ashift ar by +1 (hi);	{ range: 0,1,2,3 ==> 0,2,4,6 }
		ay1=24;
		af=sr1+ay1;
		ar=ax0+af;
		i0=ar;		{ i0 points to ^Q1Q2_buff+16+8+2*(whichcol-1) }
		ax0=^coefs;
		af=ax0+ay0;
		ax0=11;
		ar=ax0+af;
		i4=ar;		{ i4 points to ^coefs+8+4+whichcol-1 }
		i2=%5;		{ i2 points to ^colharm }
		call mnsqr;
		ax0=dm(%6);
		ay0=dm(max_2ndharm_level);
		ar=ax0-ay0;
		if lt jump done;
fail2ndharm:	ax0=0;
		dm(%1)=ax0;
		dm(%4)=ax0;
		ay0=fail_2ndharm;
		dm(%7)=ay0;
done:		nop;
.endmacro;

{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{                                                                              }
{	the hexadecimal value to output for a given DTMF digit is generated    }
{	and output if necessary, digit_history updated, failurecode cleared    }
{                                                                              }
{	INPUT:	index of detected row and col tones                            }
{		failurecode                                                    }
{	OUTPUT:	digit_history list updated with latest hex output code         }
{		hex output code written to output port if the current code is  }
{			the same as the previous code, but different from the  }
{			one before that                                        }
{		failurecode cleared for next DTMF decode operation             }
{                                                                              }
.macro		outputcode( %0, %1, %2, %3, %4 );
			{ whichrow, whichcol, digit_history, failurecode, dac }
.local		checkfailures;
.local		digitdetected;
.local		nodigit;
.local		readlist;
.local		pushlist;

checkfailures:	ay0=dm(%3);	ar=pass ay0;	if ne jump nodigit;
digitdetected:	ay0=dm(%1);	ar=ay0-1;	sr=lshift ar by +12 (hi);
		ay0=dm(%0);	ar=ay0-1;	sr=sr or lshift ar by +14 (hi);
		jump readlist;
nodigit:	sr1=baddigitcode;
readlist:	ay0=dm(%2);
		ay1=dm(%2+1);
		ar=sr1-ay1;	if eq jump pushlist;
		ar=sr1-ay0;	if ne jump pushlist;
		dm(%4)=sr1;
pushlist:	dm(%2+1)=ay0;
		dm(%2)=sr1;
		ay0=pass_posttests;	dm(%3)=ay0;
.endmacro;

{------------------------------------------------------------------------------}
{-------------------- M A I N   C O D E ---------------------------------------}
{------------------------------------------------------------------------------}
		rti; rti; rti; jump sample;

		call setup;
		call restart;
		imask=b#1000;
here:		jump here;

{------------------------------------------------------------------------------}
{-------------------- I N T E R R U P T   S E R V I C E    R O U T I N E ------}
{------------------------------------------------------------------------------}
sample:	get_sample( A_codec );
	get_sample( B_codec );
	get_sample( C_codec );
	get_sample( D_codec );
	get_sample( E_codec );
	get_sample( F_codec );
	{	etc
		etc	edit this for more channels
	}

dec201:	ay0=dm(count201);
	ar=ay0-1;
	dm(count201)=ar;
	if lt jump dec4;

in201:	cntr=channels;		{ number of channels }
	do chan201 until ce;
	  ay1=dm(i3,m0);	{ get sample for channel          AY1=1.15 }
	  cntr=16;		{ 8 fundamentals, 8 2nd_harmonics per chan }
	  do freq201 until ce;
	    mx0=dm(i0,m0), my0=pm(i4,m4);   {get Q1 and COEF Q1=1.15, COEF=2.14}
	    mr=mx0*my0(rnd), ay0=dm(i0,m1); {mult, get Q2    MR=2.30,   Q2=1.15}
	    sr=ashift mr1 by +1 (hi);	    {change 2.30 to 1.15               }
	    ar=sr1-ay0;			    {Q1*COEF - Q2		AR=1.15}
	    ar=ar+ay1;			    {Q1*COEF - Q2 + input	AR=1.15}
	    dm(i0,m0)=ar;		    {result = new Q1                   }
freq201:    dm(i0,m0)=mx0;		    {old Q1 = new Q2                   }
chan201:  nop;			{ do next channel }
	rti;

dec4:	ay0=dm(count4);
	ar=ay0-1;
	dm(count4)=ar;
	if lt jump last;

in4:	cntr=channels;		{ number of channels }
	do chan4 until ce;
	  ay1=dm(i3,m0);
	  cntr=8;		{ 8 fundamentals only }
	  do freq4 until ce;
	    mx0=dm(i0,m0), my0=pm(i4,m4);
	    mr=mx0*my0(rnd), ay0=dm(i0,m1);
	    sr=ashift mr1 by +1 (hi);
	    ar=sr1-ay0;
	    ar=ar+ay1;
	    dm(i0,m0)=ar;
freq4:	    dm(i0,m0)=mx0;
	  modify(i0,m2);	{ skip 2nd harmonic Q1Q2s }
chan4:	  modify(i4,m5);	{ skip 2nd harmonic COEFs }
	rti;

last:	cntr=channels;
	do chanlast until ce;
	  cntr=8;
	  do freqlast until ce;
	    call mnsqr;
freqlast:   nop;
	  modify(i0,m2);	{ skip 2nd harmonic Q1Q2s }
chanlast: modify(i4,m5);	{ skip 2nd harmonic COEFs }

 maxrowcol( ^A_mnsqr, A_maxrowval, A_whichrow, A_maxcolval, A_whichcol );
 maxrowcol( ^B_mnsqr, B_maxrowval, B_whichrow, B_maxcolval, B_whichcol );
 maxrowcol( ^C_mnsqr, C_maxrowval, C_whichrow, C_maxcolval, C_whichcol );
 maxrowcol( ^D_mnsqr, D_maxrowval, D_whichrow, D_maxcolval, D_whichcol );
 maxrowcol( ^E_mnsqr, E_maxrowval, E_whichrow, E_maxcolval, E_whichcol );
 maxrowcol( ^F_mnsqr, F_maxrowval, F_whichrow, F_maxcolval, F_whichcol );
 {	etc
	etc	edit this for more channels
 }

{-------------------- START OF DIGIT VALIDATION TESTS -------------------------}
minsiglevel( A_whichrow, A_maxrowval, A_whichcol, A_maxcolval, A_failurecode );
minsiglevel( B_whichrow, B_maxrowval, B_whichcol, B_maxcolval, B_failurecode );
minsiglevel( C_whichrow, C_maxrowval, C_whichcol, C_maxcolval, C_failurecode );
minsiglevel( D_whichrow, D_maxrowval, D_whichcol, D_maxcolval, D_failurecode );
minsiglevel( E_whichrow, E_maxrowval, E_whichcol, E_maxcolval, E_failurecode );
minsiglevel( F_whichrow, F_maxrowval, F_whichcol, F_maxcolval, F_failurecode );
{	etc
	etc	edit this for more channels
}
no_other_peaks( ^A_mnsqr, A_whichrow, A_whichcol, A_failurecode );
no_other_peaks( ^B_mnsqr, B_whichrow, B_whichcol, B_failurecode );
no_other_peaks( ^C_mnsqr, C_whichrow, C_whichcol, C_failurecode );
no_other_peaks( ^D_mnsqr, D_whichrow, D_whichcol, D_failurecode );
no_other_peaks( ^E_mnsqr, E_whichrow, E_whichcol, E_failurecode );
no_other_peaks( ^F_mnsqr, F_whichrow, F_whichcol, F_failurecode );
{	etc
	etc	edit this for more channels
}
twisttests( A_maxrowval, A_maxcolval, A_fortwistflag, A_revtwistflag, 
			A_whichrow, A_whichcol, A_twistval, A_failurecode );

twisttests( B_maxrowval, B_maxcolval, B_fortwistflag, B_revtwistflag, 
			B_whichrow, B_whichcol, B_twistval, B_failurecode );

twisttests( C_maxrowval, C_maxcolval, C_fortwistflag, C_revtwistflag, 
			C_whichrow, C_whichcol, C_twistval, C_failurecode );

twisttests( D_maxrowval, D_maxcolval, D_fortwistflag, D_revtwistflag, 
			D_whichrow, D_whichcol, D_twistval, D_failurecode );

twisttests( E_maxrowval, E_maxcolval, E_fortwistflag, E_revtwistflag, 
			E_whichrow, E_whichcol, E_twistval, E_failurecode );

twisttests( F_maxrowval, F_maxcolval, F_fortwistflag, F_revtwistflag, 
			F_whichrow, F_whichcol, F_twistval, F_failurecode );
{	etc
	etc	edit this for more channels
}
check2ndharm( ^A_Q1Q2_buff, A_whichrow, ^A_rowharm, A_rowharm, A_whichcol, 
			^A_colharm, A_colharm, A_failurecode );

check2ndharm( ^B_Q1Q2_buff, B_whichrow, ^B_rowharm, B_rowharm, B_whichcol, 
			^B_colharm, B_colharm, B_failurecode );

check2ndharm( ^C_Q1Q2_buff, C_whichrow, ^C_rowharm, C_rowharm, C_whichcol, 
			^C_colharm, C_colharm, C_failurecode );

check2ndharm( ^D_Q1Q2_buff, D_whichrow, ^D_rowharm, D_rowharm, D_whichcol, 
			^D_colharm, D_colharm, D_failurecode );

check2ndharm( ^E_Q1Q2_buff, E_whichrow, ^E_rowharm, E_rowharm, E_whichcol, 
			^E_colharm, E_colharm, E_failurecode );

check2ndharm( ^F_Q1Q2_buff, F_whichrow, ^F_rowharm, F_rowharm, F_whichcol, 
			^F_colharm, F_colharm, F_failurecode );
{	etc
	etc	edit this for more channels
}
{-------------------- END OF DIGIT VALIDATION TESTS ---------------------------}

outputcode( A_whichrow, A_whichcol, A_digit_history, A_failurecode, A_dac );
outputcode( B_whichrow, B_whichcol, B_digit_history, B_failurecode, B_dac );
outputcode( C_whichrow, C_whichcol, C_digit_history, C_failurecode, C_dac );
outputcode( D_whichrow, D_whichcol, D_digit_history, D_failurecode, D_dac );
outputcode( E_whichrow, E_whichcol, E_digit_history, E_failurecode, E_dac );
outputcode( F_whichrow, F_whichcol, F_digit_history, F_failurecode, F_dac );
{	etc
	etc	edit this for more channels
}
	call restart;
	rti;



{------------------------------------------------------------------------------}
{-------------------- S U B R O U T I N E S -----------------------------------}
{------------------------------------------------------------------------------}

{%%%%%%%%%%%%%%%%%%%% O N E   T I M E   O N L Y   S E T U P %%%%%%%%%%%%%%%%%%%}
{                                                                              }
{	initializes TP3051 codec control ports (eval bd), digit_history lists, }
{	M and L registers in address generators, and sets ICNTL to edge-sens.  }
{                                                                              }
setup:		si=0;
		dm(A_cntl_port)=si;
		dm(B_cntl_port)=si;
		dm(C_cntl_port)=si;
		dm(D_cntl_port)=si;
		dm(E_cntl_port)=si;
		dm(F_cntl_port)=si;
		{	etc
			etc	edit this for more channels
		}

		si=baddigitcode;
		dm(A_digit_history)=si;		dm(A_digit_history+1)=si;
		dm(B_digit_history)=si;		dm(B_digit_history+1)=si;
		dm(C_digit_history)=si;		dm(C_digit_history+1)=si;
		dm(D_digit_history)=si;		dm(D_digit_history+1)=si;
		dm(E_digit_history)=si;		dm(E_digit_history+1)=si;
		dm(F_digit_history)=si;		dm(F_digit_history+1)=si;
		{	etc
			etc	edit this for more channels
		}

		l0 = channels_x_32;
		l1 =   0;
		l2 =   0;
		l3 = channels;
		l4 =  16;
		l5 =   0;
		l6 =   0;
		l7 =   0;

		m0 =  1;
		m1 = -1;
		m2 = 16;
		m4 =  1;
		m5 =  8;

		icntl=b#01111;
		rts;

{%%%%%%%%%%%%%%%%%%%% E V E R Y   T I M E   S E T U P %%%%%%%%%%%%%%%%%%%%%%%%%}
{                                                                              }
{	resets pointers to top of buffers, resets counter values,              }
{	clears Goertzel feedback buffers to zero, etc                          }
{                                                                              }
restart:	i0=^A_Q1Q2_buff;
		cntr=channels_x_32;
		do zloop until ce;
zloop:			dm(i0,m0)=0;
		i2=^A_mnsqr;
		i3=^in_samples;
		i4=^coefs;
		ax0=201;	dm(count201)=ax0;
		ax0=4;		dm(count4)=ax0;
		ax0=h#00FF;
		ax1=^mu_lookup_table;
		rts;

{%%%%%%%%%%%%%%%%%%%% C A L C   M A G N I T U D E   S Q U A R E D %%%%%%%%%%%%%}
{                                                                              }
{	calculates magnitude-squared (mnsqr) from Goertzel feedback results    }
{                                                                              }
mnsqr:	mx0=dm(i0,m0);			{ get two copies of Q1 		1.15 }
	my0=mx0;
	mx1=dm(i0,m0);			{ get two copies of Q2 		1.15 }
	my1=mx1;
	ar=pm(i4,m4);			{ get COEF 			2.14 }
	mr=0;
	mf=mx0*my1(rnd);		{  Q1*Q2 			1.15 }
	mr=mr-ar*mf(rnd);  		{ -Q1*Q2*COEF 			2.14 }
	sr=ashift mr1 by +1 (hi);	{ 2.14 -> 1.15 format conv. 	1.15 }
	mr=0;
	mr1=sr1;
	mr=mr+mx0*my0(ss);		{ Q1*Q1 + -Q1*Q2*COEF 		1.15 }
	mr=mr+mx1*my1(rnd);		{ Q1*Q1 + Q2*Q2 + -Q1*Q2*COEF 	1.15 }
	dm(i2,m0)=mr1;			{ store in mnsqr buffer		1.15 }
	rts;

.endmod;
