{*******************************************************************************
This is the source code listing for the master processor firmware in the sonar
beamformer (ADSP-2100 Family Applications Handbook Vol.3,Chapter 6)

For questions, contact:		Kapriel Karagozyan
				DSP Applications
				Tel:(617)461-4029
				Fax:(617)461-3010

************************************************************************8/24/89}
.module/rom/abs=0	master_code;

{The following are declarations for ports that are used to set and clear
several flags.  Data memory mapped D-flipflops are used to generate flags}

.port			setbmoutrdy;	{Beam output ready flag}
.port			clrbmoutrdy;	
.port			pcwe;		{PC output port}
.port			setsbr;		{Slave bus request flag}
.port			clrsbr;	
.port			setslhalt;	{Slave halt flag}
.port			clrslhalt;
.port			setbpoe;	{Backplane output enable flag}
.port			clrbpoe;
.port			clrbmtaken;
.port			setsmemrd;	{Slave memory read flag}
.port			clrsmemrd;
.port			setslint;	{Slave interrupt}
.port			pcrd;		{PC input port}
.port			beamsend;	{Interrupt the external processor}

{The following are variables that contain the system configuration info.}

.var/dm/ram/abs=8160	sensor_num;
.var/dm/ram/abs=8161	beam_num;
.var/dm/ram/abs=8162	index_num;
.var/dm/ram/abs=8163    slave_num;
.var/dm/ram/abs=8164	beams_per_slave; 
.var/dm/ram/abs=8165	last_slave_beam_num;

{The following is the main body of the master program}

{Interrupt vectors occupy the first four PM locations}
			jump adcomplete;  	{Vectored addr. for interrupt 0}
			rti;
			jump beamtaken;		{For IRQ2}
			rti;

{Program execution starts here}
			imask=b#0000;
			icntl=b#00000;
			ay0=h#0fff;
			dm(clrbmoutrdy)=mx0;	{Clear all flags by writing a}
			dm(clrbmtaken)=mx0;	{value into their respective}
			dm(clrsbr)=mx0;		{DM mapped port locations}
			dm(clrslhalt)=mx0;
			dm(clrbpoe)=mx0;
	 		dm(clrsmemrd)=mx0;

pc_init_wait:		ax0=dm(pcrd);		{Wait for the ready message}
			ar=ax0-ay0;		{from the PC}
			if eq jump pc_comm;
			jump pc_init_wait;

pc_comm:		mx0=h#ff0f;		{Initial set up before the}
			ay0=5;			{PC communications}
			af=ay0+1;		{A message is sent to the PC}
			ay0=h#1fff;		{at the end of this routine}
			ax1=h#ffff;		{signaling that the master is}
			mx1=h#ff0f;		{ready}
			i1=8160;
			m1=1;
			l1=6;
			i2=0;
			m2=1;
			l2=8160;		
			m3=-1;	
			i4=8166;	
			m4=1;
			i5=9000;
			m5=0;
			l5=1;
			i6=0;
			m6=1;
			l6=2055;
			dm(setsbr)=ay0;		{Set slave bus request flag}
			dm(pcwe)=mx0;		{Send ready message to the PC}
			jump pc_wait1;		

			
pc_comm_end_check:	ay1=i3;			{Counter register to check for}
			ar=ay1-1;		{the end of the index}
			if lt jump pc_end;	{downloading}


pc_wait1:		ay1=dm(pcrd);		{Wait for ffff from the PC}
			ar=ay1-ax1;
			if eq jump pc_read;
			jump pc_wait1;


pc_read:		modify(i6,m6);
			af=af-1;		{Write the initial 6 pieces}
			if lt jump index_store;	{of data and then start the}
init_read:		ax0=dm(pcrd);		{index storage}
			ay1=h#e000;
			ar=ax0 and ay1;		{Mask out the bottom 13 bits}
			ay1=h#a000;
			ar=ar-ay1;
			if eq jump init_sto;	{Compare to h#a000}
			jump init_read;
init_sto:		ar=ax0 and ay0;		{Mask out the top 3 bits}
			dm(setbpoe)=mx0;	{Enable backplane drivers}
			dm(i1,m1)=ar;	
			dm(setsmemrd)=mx0;	{Enable reading from slave DM}
			l4=dm(beam_num);
			i3=dm(index_num);
			dm(clrsmemrd)=mx0;	{Disable reading from slave DM}
			l3=i3;
			dm(clrbpoe)=mx0;	{Disable backplane drivers}
			jump pc_wait1;		

index_store:		ax0=dm(pcrd);		{Read the pc output}
			ay1=h#e000;
			ar=ax0 and ay1;
			ay1=h#a000;
			ar=ar-ay1;
			if eq jump sto;		{Compare to h#a000}
			jump index_store;
sto:			ar=ax0 and ay0;
			dm(setbpoe)=mx0;
			dm(i2,m2)=ar;		{Store index}
			modify(i3,m3);	
			dm(clrbpoe)=mx0;
			jump pc_comm_end_check;

pc_end:			ax1=h#f0ff;
pc_end_loop:		ay1=dm(pcrd);		{Read PC output and decide}
			ar=ax1-ay1;		{whether communication should}
			if eq jump checksum;	{be completed or not}
			jump pc_end_loop;
checksum:		ax1=i6;			{Perform checksum operation}
checksum_loop:		ay1=dm(pcrd);
			ar=ax1-ay1;
			if eq jump comm_end;
			jump checksum_loop;

comm_end:		mx0=h#f0;		{This routine ends the PC}
			dm(pcwe)=mx0;		{communications and does some}
			dm(setbpoe)=mx1;	{reads from the slave DM in}
			ay0=h#0201;		{order to prepare for the}
			af=ay0-1;		{beamforming tasks}
			ay0=0;
			dm(setsmemrd)=mx0;
			si=dm(sensor_num);	{Read #of sensors from slave}
			dm(clrsmemrd)=mx0;	{DM and do the necessary}
			ay0=si;			{division to obtain a scaled}
			ar=ay0-1;		{magnitude value for the}
			if eq jump onesensor;	{rectangular shading window}
			ay0=0;
			sr=lshift si by 9 (lo);
			ax1=sr0;
			astat=0;
			divq ax1;
			divq ax1;divq ax1;divq ax1;
			divq ax1;divq ax1;divq ax1;
			divq ax1;divq ax1;divq ax1;
			divq ax1;divq ax1;divq ax1;
			divq ax1;divq ax1;divq ax1;
			dm(8192)=ay0;		{Shading coefficient}
			jump allsensor;
onesensor:		ay0=h#7fff;		{This is shading coefficient}
			dm(8192)=ay0;		{in case of a single sensor}
allsensor:		dm(setsmemrd)=mx1;
			ay0=dm(slave_num);		{Do the rest of the}
			ax1=dm(beams_per_slave);	{reads before releasing}
			my1=dm(last_slave_beam_num);	{slave BR flag}
			mx0=dm(sensor_num);
			dm(clrsmemrd)=mx0; 	{Clear the slave DM read flag}	
			my0=255;		{and set up some pointers to be}
			mr=mx0*my0(uu);		{used later on}
			si=mr0;			{# of samples to be placed}
			sr=lshift si by -1 (hi);{into the sample buffer}
			l2=sr1;
			ar=ay0-1;
			dm(clrsbr)=mx1;
			ax0=ar;
			i1=10240;
			m1=1;
			l1=mx0;
			i2=0;
			m2=1;
			if eq jump fix_base;
			i3=h#3800;		{Base addr. for the FIFOs}
			m3=1;			{with multiple slaves}
			ar=ay0+1;
			l3=ar;
			jump normal;
fix_base:		i3=h#3801;		{Base addr. for the FIFO}
			m3=0;			{with single slave}
			l3=0;			
normal:			dm(setslhalt)=mx1;	{HALT the slaves, this will}
			dm(clrslhalt)=mx1;	{cause them to get out of}
			dm(clrbpoe)=mx1;	{their TRAP state}
			cntr=1000;
			do slave_wait until ce;	{Wait until all slaves are}
slave_wait:		nop;			{ready to go}
			ar=1;
			ay1=1;
			icntl=b#00101;
			imask=b#0001;		{Enable sampling interrupt IRQ0}

wait:			jump wait;

adcomplete:		ar=ar-ay1;			{Ignore the first}
			if eq jump first_adcomp;	{IRQ0 by using this}
			dm(setsbr)=mx1;
			af=ay0-1;
			dm(setbpoe)=mx1;		{This routine is used}
			cntr=mx0;			{to fill up the sample}
			do sample_store until ce;	{memory after each}
			mx1=dm(i1,m1);			{A/D conversion}
sample_store:		dm(i2,m2)=mx1;
			dm(clrsbr)=mx1;
			dm(setslint)=mx1;
			dm(clrbpoe)=mx1;



sendbeam:		if eq jump oneslave;	{Read out the beams in a}
			m3=1;			{sequential manner in each FIFO}
			modify(i3,m3);		{in order of numbering}
			m3=0;
			cntr=ax0;
			do beamout until ce;	{Send beam outputs via the}
			cntr=ax1;		{digital output port}
			do fifo_out until ce;
			dm(setbpoe)=mx1;
			mx1=dm(i3,m3);
			dm(clrbpoe)=mx1;
			dm(beamsend)=mx1;
			imask=b#0100;		{Enable data receive interrupt}
			dm(setbmoutrdy)=mx1;	{Set a flag for the external}
			cntr=6;			{processor}
			do resp_wait until ce;
resp_wait:		nop;
fifo_out:		nop;
			m3=1;
			modify(i3,m3);
beamout:		m3=0;


oneslave:		cntr=my1;
			do endfifo until ce;
			dm(setbpoe)=mx1;	{This routine is for single}
			mx1=dm(i3,m3);		{slave configurations and}
			dm(clrbpoe)=mx1;	{is also used for handling}
			dm(beamsend)=mx1;	{the last FIFO to be read out}
			imask=b#0100;		{It handles the irregularity}
			dm(setbmoutrdy)=mx1;	{of the last set of beams}
			cntr=6;			{i.e. possibly fewer beams}
			do resp_wt until ce;
resp_wt:		nop;
endfifo:		nop;
first_adcomp:		ar=0;
			rti;	

beamtaken:		dm(clrbmoutrdy)=mx1;	{Interrupt routine to handle}
			dm(clrbmtaken)=mx1;	{the data receive confirmation}
			rti;			{from the external processor}
		
.endmod;						

