{*******************************************************************************
This is the source code listing for the slave processor firmware in the sonar
beamformer (ADSP-2100 Family Applications Handbook Vol.3, Chapter 6)

For questions, contact:		Kapriel Karagozyan
				DSP Applications
				Tel:(617)461-4029
				Fax:(617)461-3010

************************************************************************8/24/89}
.module/rom/abs=0     	slave_code;

{The following are declarations for data memory mapped ports.  One is used to
clear a flag, while others are used to write data to the DAC, FIFO and read the
hardware switches on the slave board}

.port                 	beamout;	{FIFO}
.port			beamdac;	{DAC}
.port			clrslint;	{Clear the slave interrupt}
.port                   slave_id;	{Slave identity dipswitch}
.port			dac_beam_sel;	{Analog output selection switch}

{The following are variables that contain the system configuration info.}

.var/dm/ram/abs=8160	sensor_num;
.var/dm/ram/abs=8161    beam_num;
.var/dm/ram/abs=8162 	index_num;
.var/dm/ram/abs=8163	slave_num;
.var/dm/ram/abs=8164	beams_per_slave;
.var/dm/ram/abs=8165	last_slave_beam_num;

{The following is the main body of the slave program}

			jump beam_form;		{Vectored addr. for IRQ0}
                      	rti;
                     	rti;
                      	rti;
 
                      	imask=b#0000;		{Disable interrupts}
                      	icntl=b#00000;
			dm(clrslint)=mx0;
			trap;			{TRAP until pc_comm ends}

			i1=0;			{This initial routine is used}
			m1=1;			{to transfer the indexes from}
			l1=dm(index_num);	{the sample_mem into the}
			i4=2048;		{index_mem in PM}
			m4=1;
			l4=l1;
			cntr=l1;
			do index_store until ce;
			mx0=dm(i1,m1);
index_store:		pm(i4,m4)=mx0;
			
                      	mx0=dm(sensor_num);       
                      	my0=255;
                      	mr=mx0*my0(uu);
			si=mr0;
			sr=lshift si by -1 (hi);	{SR1 contains the}
			i1=mx0;				{length of the circular}
			m1=mx0;				{sample buffer}
                      	l1=sr1;            
			si=dm(slave_id);		{Determine this slave's}
			sr=lshift si by -12 (hi);	{ID# and the starting}
			ax0=dm(slave_num);		{location of the first}
			ay0=sr1;			{index.Also determine}
			af=ax0-ay0;			{the # of indexes}
			af=af-1;			{for this slave}
			if eq jump last_slave;	
			se=dm(beams_per_slave);
			jump all_slave;
last_slave:		se=dm(last_slave_beam_num);
all_slave:		mx0=dm(sensor_num);		{This routine}
			my0=dm(beams_per_slave);	{calculates the}
			mr=mx0*my0(uu);			{starting address of}
			si=mr0;				{this slave's indexes}
			sr=lshift si by -1 (hi);
			mx0=sr1;
			ar=ax0-ay0;
			ay1=ar;
			ar=ay1-1;
			my0=ar;
			mr=mx0*my0(uu);
			si=mr0;
			sr=lshift si by -1 (hi);
			ax1=2048;
			ay1=sr1;
			ar=ax1+ay1;
			i5=ar; 	   		{Starting address of the}
                      	m5=1;			{indexes for this slave}
                        m7=-1;
			mx0=se;
			my0=dm(sensor_num);  	{# of indexes per beam}
			mr=mx0*my0(uu);
			si=mr0;
			sr=lshift si by -1 (hi);
			l5=sr1;			{Total # of indexes to be used}
                      	m3=0;			{by this slave}
                      	l3=l1;
                      	i6=8192;
                      	m6=0;
                      	l6=1;
                      	my1=dm(i6,m6);     	{Shading coefficient; there}
                      	ax1=dm(sensor_num);	{is only for now since it is}
			icntl=b#00001;		{a rectangular window}
			imask=b#0001;		{Enable slave interrupt IRQ0}

wait:			si=dm(dac_beam_sel);	{Setup the down counter to be}
			sr=lshift si by -12(hi);{used in deciding which beam to}
			ay1=sr1;		{send out to the DAC}
			af=ay1+1;
			jump wait;		{Wait for sample buffer update}

beam_form:		cntr=se;
                      	do beam_end until ce;
	                  mr=0,ay0=pm(i5,m5);		{Read index}
        	          m3 = i1;
                	  i3 = ay0;
			  modify(i3,m3);		{Modify index}
			  mx0=dm(i3,m3);		{Get the desired sample}
	                  cntr=ax1;
        	          do single_beam_sample until ce;	{Beamform}
                           i3 = ay0;
                           modify(i3,m3);
single_beam_sample:        mr = mr+mx0*my1(ss),mx0 = dm(i3,m3), ay0 = pm(i5,m5);
                	  mr = mr+mx0*my1(rnd), ay0 = pm(i5,m7);
			  af=af-1;		{Check which beam to}
			  if eq jump dac_write;	{send out to the DAC}
beam_end:             	  dm(beamout)=mr1;      	{Write result to FIFO}
			modify(i1,m1);	  	{Advance the circular sample}
			dm(clrslint)=mx0;	{buffer pointer}
			rti;

dac_write:		dm(beamdac)=mr1;	{Write result to DAC}
			jump beam_end;

.endmod;

