.MODULE cvt_ieee_float_to_fixed;

{
        Convert 32-bit IEEE 754 floating-point to 1.15 fixed-point
 
        Calling Parameters
                MR1 = MSW of IEEE 754 floating point number
                MR0 = LSW of IEEE 754 floating point number
 
        Return Values
                MX1 = 1.15 fixed point number
                MX0 = ERROR word
 
        Altered Registers
                AX0,AX1,AY0,AY1,AF,AR,MX0,MX1,SR,SE

        Computation Time
                51 cycles (maximum)

}
 
.ENTRY  ieeefix;
 
ieeefix:AY0=H#00FF;                      {mask to extract exp, lo_bit check}
        AY1=H#0100;                      {mask for overflow sign}
        MX1=H#0000;                      {clear fixed result}
        MX0=H#0000;                      {clear ERROR status}
        AX1=H#0000;                      {clear UNDER/OVERFLOW flag}
 
zero:   SR=LSHIFT MR1 BY 0 (LO);         {check for flt_num = 0}
        SR=SR OR LSHIFT MR0 BY 0 (LO);
        AR=PASS SR0;
        IF EQ RTS;
 
negone: AF=PASS MR0;                     {check for flt_num = -1}
        IF NE JUMP checkexpo;
        AF=PASS MR1;
        AX0=H#BF80;
        AR=AX0-AF;
        IF NE JUMP checkexpo;
        MX1=H#8000;
        MX0=H#0000;
        RTS;
 
checkexpo:SR=LSHIFT MR1 BY -7 (LO);      {extract exponent}
        AF=SR0 AND AY0;                  {extract exponent}
        AX0=H#007E;                      {upper valid expo -- 126 decimal}
        AR=AF-AX0;                       {is expo .gt. max valid ???}
        IF GT JUMP overflow;             {if YES, then overflow}
        AX0=H#0070;                      {lower valid expo -- 112 decimal}
        AR=AF-AX0;                       {is expo .lt. min valid ???}
        IF LT JUMP underflow;            {if YES, then underflow}
        JUMP eightlsb;                   {go check if 8 LSBs are set}
 
overflow:AX1=H#FFFF;                     {set UNDER/OVERFLOW FLAG true}
        AF=SR0 AND AY1;                  {extract sign bit with AND mask}
        IF NE JUMP negover;
posover:MX0=H#F000;                      {ERROR = "positive overflow"}
        MX1=H#7FFF;                      {make fixed result max. pos. value}
        JUMP eightlsb;
negover:MX0=H#000F;                      {ERROR = "negative overflow"}
        MX1=H#8001;                      {make fixed result max. neg. value}
        JUMP eightlsb;
 
underflow:AX1=H#FFFF;                    {set UNDER/OVERFLOW FLAG true}
        AF=SR0 AND AY1;                  {extract sign bit with AND mask}
        IF NE JUMP negunder;
posunder:MX0=H#0F00;                     {ERROR = "positive underflow"}
        JUMP eightlsb;                   {fixed result remains zero}
negunder:MX0=H#00F0;                     {ERROR = "negative underflow"}
        JUMP eightlsb;                   {fixed result remains zero}
 
eightlsb:SR=LSHIFT MR0 BY 0 (LO);        {get 16 LSBs of flt_num}
        AF=SR0 AND AY0;                  {extract lower 8 LSBs with AND mask}
        IF EQ JUMP endlsb;
        AR=MX0;                          {if any are set, toggle ERROR}
        AR=NOT AR;
        MX0=AR;                          {ERROR value stored in MX0}
endlsb: AR=PASS AX1;                     {check for under/overflow situation}
        IF NE RTS;                       {do not convert if under/overflow}
 
convert:SR=LSHIFT MR1 BY -7 (LO);        {set up exponent field to mask}
        AF=SR0 AND AY0;                  {extract exponent field by AND}
        AX0=H#007F;                      {exponent bias -- 127 decimal}
        AR=AF-AX0;                       {subtract bias from exponent}
        SE=AR;                           {unbiased expo into SE register}
        SR=LSHIFT MR1 BY 8 (HI);         {paste hi mantissa word}
        SR=SR OR LSHIFT MR0 BY 8 (LO);   {paste lo mantissa word}
        AY1=H#8000;                      {hidden "1" bit}
        AR=SR1 OR AY1;                   {paste hidden "1" bit}
        SR=LSHIFT AR (HI);               {denormalize mantissa}
        AF=PASS MR1;                     {set sign bit if orig. was neg.}
        AR=SR1;                          {use only 16 MSBs of result}
        IF LT AR=-SR1;                   {if neg. orig. do twos complement}
        MX1=AR;
        RTS;

.ENDMOD;

