.MODULE cvt_fixed_to_ieee_floating;

{
        Converts 1.15 fixed-point to 32-bit IEEE 754 floating-point
 
        Calling Parameters
                AX0 = 1.15 fixed-point number
 
        Return Values
                SR1 = MSW of IEEE 754 floating-point number
                SR0 = LSW of IEEE 754 floating-point number
 
        Altered Registers
                AX1,AY0,AY1,AF,AR,SR,SE,SI

        Computation Time
                20 cycles (maximum)

}
 
 
.ENTRY  ieeeflt;
 
ieeeflt:AY0=H#8000;                          {neg. one = H#8000}
        AY1=126;                             {-127 bias +1 for hidden bit shift}
        AR=AX0-AY0;                          {AY0 = -1? (H#8000)}
        IF NE JUMP numok;                    {no, do conversion}
        AR=H#BF80;                           {if neg one, do float right now}
        SR=LSHIFT AR BY 0 (HI);              {this is IEEE float for -1}
        RTS;                                 {skip conversion, output right now}
 
numok:  AR=PASS AX0;                         {1.15 number to convert is in AR}
        AF=ABS AR;                           {make positive}
        IF NE JUMP notzero;                  {if not zero, do conversion}
        SR=LSHIFT AR BY 0 (HI);              {special case for zero}
        RTS;                                 {exit if zero}

notzero:IF NEG JUMP itisneg;
itispos:SI=H#0000;                           {CLEAR sign bit flag if positive}
        JUMP cvt;
itisneg:SI=H#8000;                           {SET sign bit flag if negative}
 
cvt:    AR=PASS AF;                          {use abs(orig_number) }
        SE=EXP AR (HI);                      {derive exponent}
        SR=NORM AR (HI);                     {normalize fraction}
        AX1=SE;                              {load AX1 with exponent}
        AR=AX1 + AY1;                        {add 126 to exponent}
        SR=LSHIFT SR1 BY +2 (HI);            {remove sign & hidden bit}
        SR=LSHIFT SR1 BY -9 (HI);            {open sign bit and expo field}
        SR=SR OR LSHIFT SI BY 0 (HI);        {paste sign bit}
        SR=SR OR LSHIFT AR BY +7 (HI);       {paste exponent}
        RTS;

.ENDMOD;

