{=============================================================}
{		GAM 7-24-87	fft_noshift.dsp		      }
{		Analog Devices DSP Division		      }
{							      }
{		1024 point FFT routine			      }
{		Decimation in time, Radix 2		      }
{		no scaling... input data must be scaled       }
{							      }
{		FFT routine				      }
{		called from fft_main_noshift.dsp              }
{=============================================================}
.module/ram    fft;


.const         log2N=8,N=256,nover2 =128,nover4 =64;

.external      twid_real,twid_imag,inplacereal,inplaceimag;
.external      inputreal,inputimag;
.external      groups,bflys_per_group,node_space;
.entry         fft_strt;

fft_strt:     cntr=log2N - 2;         {cntr=# stages in fft -2 }
              m0=0;                   {m0= in place modifier}
              m1=1;                   {m1= advance 1 modifier}
	      m3=-1;
	      m5=1;
              l1=0;
              l2=0;
              l3=0;
              l4=%twid_real;	      { length reg. for twiddle factor }
              l5=%twid_imag;          { tables                         }
	      l6 = 0;
{-----------------------------------------------------------------}
{	First stage of 1024 point FFT.... broken out of loop      }
{	for increased performance........	                  }
{	Twiddle factors = 1, therefore no multiplies necessary    }
{-----------------------------------------------------------------}

	      i0 = ^inplacereal;                 { I0 -> x0 }
	      i1 = ^inplacereal + 1;             { I1 -> x1 }
	      i2 = ^inplaceimag;                 { I2 -> y0 }
	      i3 = ^inplaceimag + 1;             { I3 -> y1 }
	      m2 = 2;                            { modifier for first stage }
					         { node space } 
	      cntr = nover2;		         { number of groups }

		  	ax0 = dm(i0,m0);	 { ax0 = x0 }
			ay0 = dm(i1,m0);         { ay0 = x1 }
			ay1 = dm(i3,m0);	 { ay1 = y1 }

	      do group_lp until ce;
		ar = ax0 + ay0,	ax1 = dm(i2,m0); { ar=x1+y1 , ax1=y0} 
		dm(i0,m2) = ar, ar = ax0 - ay0;  { store x0', ar=x0-x1 }
		dm(i1,m2) = ar, ar = ax1 + ay1;  { store x1', ar=y0+y1 }
		dm(i2,m2) = ar, ar = ax1 - ay1;  { store y0', ar=y0-y1 }
		dm(i3,m2) = ar;			 { store y1' }
		ax0 = dm(i0,m0);		 { ax0 = next x0 }
		ay0 = dm(i1,m0);		 { ay0 = next x1 }
group_lp:	ay1 = dm(i3,m0);		 { ay1 = next y1 }


{------               END STAGE 1                           ------}
{-----------------------------------------------------------------}
{------               STAGES 2 THRU 9                       ------}

              do stage_loop until ce;            { begin looping through }
                                                 { stage 2 thru stage 9. }
                 i0=^inplacereal;                { I0 -> x0 }
                 i2=^inplaceimag;                { I2 -> y0 }
                 si=dm(groups);                  {si=# groups in previous}
                                                 {stage                  }
                 sr=ashift si by -1(lo);         {sr0=groups/2           }
                 cntr=sr0;                       {cntr=# groups in current}
                                                 {stage.                  }
                 dm(groups)=sr0;                 {update "groups"         }
                 m4=sr0;                         {Initialize twid pointer }
                                                 {modifier                }
                 m2=dm(node_space);              {m2=dual node spacing    }
                 i1=i0;                          {i1 points to first x_real}
                 modify(i1,m2);                  {i1 points to first y_real}
                 i3=i2;                          {i3 points to first x_imag}
                 modify(i3,m2);                  {i3 points to first y_imag}
		 i6=i3;		                 {i6->y1}

                 do group_loop until ce; 
                    i4=^twid_real;               { I4 -> real twiddle factor }
                    i5=^twid_imag;               { I5 -> imaginary twiddle   }
                    cntr=dm(bflys_per_group);    { # of butterflies per group}
                    my0=pm(i4,m4),mx0=dm(i1,m0); { my0 = C, mx0 = x1 } 
                    my1=pm(i5,m4);               { my1 = (-S) }

                    do bfly_loop until ce;
		     mr=mx0*my0(ss),mx1=dm(i6,m5);     {mr=x(C), mx1=y1}
                     mr=mr-mx1*my1(ss),ay0=dm(i0,m0); {mr=x1(C)-y1(-S),ay0=x0}
		     ar=mr1+ay0,ay1=dm(i2,m0);         {ar=x0+[x1(C)-y1(-S)]}
						       {ay1=y0 } 
                     dm(i0,m1)=ar,ar=ay0-mr1;          {store x0', calc. x1'}
		     mr=mx0*my1(ss),dm(i1,m1)=ar;      {mr= x1(-S), store x1'}
		     mr=mr+mx1*my0(rnd),mx0=dm(i1,m0),my1=pm(i5,m4);
						       {mr=x1(-S)+y1(C)}
						       {mx0=x1, my1=(-S)}
		     ar=ay1-mr1,my0=pm(i4,m4);         {calc y1', my0=(C)}
		     dm(i3,m1)=ar,ar=mr1+ay1;          {store y1',calc y0'}
 bfly_loop:	     dm(i2,m1)=ar;                     {store y0'}


                  modify(i0,m2);                       {I0 -> x0 in next group}
                  modify(i1,m2);                       {I1 -> x1 in next group}
                  modify(i2,m2);                       {I2 -> y0 in next group}
group_loop:       modify(i3,m2);                       {I3 -> y1 in next group}
     
               si=dm(bflys_per_group);
               sr=ashift si by 1(lo);                  {sr=bflys_per_group * 2}
               dm(node_space)=sr0;
stage_loop:    dm(bflys_per_group)=sr0;                {update bflys_per_group}

{---------------------------------------------------------------}
{-----                    LAST STAGE                  ----------}

	      i0 = ^inplacereal;                 { I0 -> x0 }
	      i1 = ^inplacereal + nover2;        { I1 -> x1 }
	      i2 = ^inplaceimag;                 { I2 -> y0 }
	      i3 = ^inplaceimag + nover2;        { I3 -> y1 }
	      cntr = nover2;                     { # of butterflies }
	      m2 = dm(node_space);               { node space modifier }
	      m4 = 1;
      	      i6 = i3;
	      i4 = ^twid_real;                   { real twiddle pointer }
	      i5 = ^twid_imag;                   { imag. twiddle pointer }

						 { butterfly loop as above }
                    my0=pm(i4,m4),mx0=dm(i1,m0);  
                    my1=pm(i5,m4);
                    do bfly_lp until ce;
			mr=mx0*my0(ss),mx1=dm(i6,m5);
			mr=mr-mx1*my1(rnd),ay0=dm(i0,m0);
			ar=mr1+ay0,ay1=dm(i2,m0);
			dm(i0,m1)=ar,ar=ay0-mr1;
			mr=mx0*my1(ss),dm(i1,m1)=ar;
			mr=mr+mx1*my0(rnd),mx0=dm(i1,m0),my1=pm(i5,m4);
			ar=ay1-mr1,my0=pm(i4,m4);
			dm(i3,m1)=ar,ar=mr1+ay1;
 bfly_lp:		dm(i2,m1)=ar;

            rts;				 { return to main }


.endmod;                    

