{_____________________________________________________________________
               FFT4_1024.DSP

        Optimized complex 1024-Point Radix-4 DIF FFT Program for the
        Analog Devices ADSP-210X Digital Signal Processor Family
                               
                                       Written : Steve Cox 5/5/89      
                                       Revised : Fares Eidi 5/31/89

 
        Complex data stored X0,Y0, X1, Y1,... (real, imag, real, imag)

        The following terminology will be used to describe
        the  "legs" of the radix-4 butterfly:
        xa = 1st real input leg        x'a = 1st real output leg
        xb = 2nd real input leg        x'b = 2nd real output leg
        xc = 3rd real input leg        x'c = 3rd real output leg 
        xd = 4th real input leg        x'd = 4th real output leg
        ya = 1st imag input leg        y'a = 1st imag output leg
        yb = 2nd imag input leg        y'b = 2nd imag output leg
        yc = 3rd imag input leg        y'c = 3rd imag output leg
        yd = 4th imag input leg        y'd = 4th imag output leg

        The index registers of the 210X will be used as pointers
        to the following:
        I0 --> xa,xc
        I1 --> xb,xd
        I2 --> ya,yc
        I3 --> yb,yd                
               w0 (= Ca = Sa = 0)
        I5 --> w1 (1st Cb, - pi/4 for Sc)
        I6 --> w2 (2nd Cc, - pi/4 for Sc)
        I7 --> w3 (3rd Cd, - pi/4 for Sd)

        input: inplace[2*N] normal order, interleaved real,imag.
        output: inplace[2*N] digit reversed, interleaved real,imag.

        Benchmark:
          setup  =     9
          stage 1=     7699 = 19+256(30)
          stage 2=     7749 = 17+4(13+64(30))
          stage 3=     7905 = 17+16(13+16(30))
          stage 4=     8529 = 17+64(13+4(30))
          stage 5=     5130 = 10+256(20)

          Total       37021 cycles * 80ns = 2.96168ms

_____________________________________________________________________}

.MODULE/boot=0   fft_sub;
.CONST           N=1024;
.CONST           Nx2=2048;
.CONST           Nov2=512;
.CONST           Nov4=256;
.CONST           N3ov8=384;
.VAR/DM/CIRC     cos_table[N];
.VAR/DM          bfy_count;
.INIT            cos_table:<cos1024.dat>;        {N cosine values}
.ENTRY           fft;
.EXTERNAL        inplace;

fft: M4=-Nov4;          {-N/4 = -90 degrees for sine}
     L0=0;
     L1=0;
     L2=0;
     L3=0;
     L5=%cos_table;
     L6=%cos_table;
     L7=%cos_table;
     SE=0;

{_____________________________Stage 1_______________________________}



stage1: I0=^inplace;                   {in       ->Xa,Xc}
        I1=^inplace+Nov2;              {in+N/2   ->Xb,Xd}
        I2=^inplace+1;                 {in+1     ->Ya,Yc}
        I3=^inplace+Nov2+1;            {in+N/2+1 ->Yb,Yd}
        I5=^cos_table;
        I6=^cos_table;
        I7=^cos_table;

        M0=N;               {N,    skip forward to dual node}
        M1=-N;              {-N,   skip back to primary node}
        M2=-N+2;            {-N+2, skip to next butterfly}
        M5=Nov4+1;          {N/4 + groups/stage*1, Cb Sb offset}
        M6=Nov4+2;          {N/4 + groups/stage*2, Cc Sc offset}
        M7=Nov4+3;          {N/4 + groups/stage*3, Cd Sc offset}
        AX0=DM(I0,M0);                        {get first Xa}
        AY0=DM(I0,M1);                        {get first Xc}
        AR=AX0-AY0, AX1=DM(I2,M0);            {Xa-Xc,get first Ya}
        SR=LSHIFT AR(LO), AY1=DM(I2,M1);      {SR1=Xa-Xc,get first Yc}
        CNTR=Nov4;                            {Bfly/group, stage one}
        DO stg1bfy  UNTIL CE;
                 AR=AX0+AY0, AX0=DM(I1,M0);
                           {AR=xa+xc, AX0=xb }
                 MR0=AR, AR=AX1+AY1;
                           {MR0=xa+xc, AR=ya+yc}
                 MR1=AR, AR=AX1-AY1;
                           {MR1=ya+yc, AR=ya-yc}
                 SR=SR OR LSHIFT AR (HI), AY0=DM(I1,M1);
                           {SR1=ya-yc, AY0=xd}
                 AF=AX0+AY0, AX1=DM(I3,M0);
                           {AF=xb+xd, AX1=yb}
                 AR=MR0+AF, AY1=DM(I3,M1);
                           {AR=xa+xb+xc+xd, AY1=yd}
                 DM(I0,M0)=AR, AR=MR0-AF;
                           {output x'a=(xa+xb+xc+xd), AR=xa+xc-xb-xd}
                 AF=AX1+AY1, MX0=AR;
                           {AF=yb+yd, MX0=xa+xc-xb-xd}
                 AR=MR1+AF, MY0=DM(I6,M4);
                           {AR=ya+yc+yb+yd, MY0=(Cc)}
                 DM(I2,M0)=AR, AR=MR1-AF;
                           {output y'a, AR=ya+yc-yb-yd}
                 MR=MX0*MY0(SS), MY1=DM(I6,M6);
                           {MR=(xa+xc-xb-xd)(Cc), MY1=(Sc)}
                 MR=MR+AR*MY1(RND);
                           {MR=(xa-xb+xc-xd)(Cc)+(ya-yb+yc-yd)(Sc)}
                 DM(I0,M2)=MR1, MR=AR*MY0(SS);
                           {output x'c=xa-xb+xc-xd)(Cc)+(ya-yb+yc-yd)(Sc)}
                           {MR=(ya+yc-yb-yd)(Cc)}
                 MR=MR-MX0*MY1(RND), MY0=DM(I5,M4);
                           {MR=(ya+yc-yb-yd)(Cc)-(xa+xc-xb-xd)(Sc), MY0=(Cb)}
                 DM(I2,M2)=MR1, AR=AX0-AY0;
                           {output y'c=(ya+yc-yb-yd)(Cc)-(xa+xc-xb-xd)(Sc)}
                           {AR=xb-xd}
                 AY0=AR, AF=AX1-AY1;
                           {AY0=xb-xd, AF=yb-yd}
                 AR=SR0-AF, MY1=DM(I5,M5);
                           {AR=xa-xc-(yb-yd), MY1=(Sb)}
                 MX0=AR, AR=SR0+AF;
                           {MX0=xa-xc-yb+yd, AR=xa-xc+yb-yd}
                 SR0=AR, AR=SR1+AY0;
                           {SR0=xa-xc+yb-yd, AR=ya-yc+xb-xd}
                 MX1=AR, AR=SR1-AY0;
                           {MX1=ya-yc+xb-xd, AR=ya-yc-(xb-xd)}
                 MR=SR0*MY0(SS), AX0=DM(I0,M0);
                           {MR=(xa-xc+yb-yd)(Cb), AX0= xa of next bfly}
                 MR=MR+AR*MY1(RND), AY0=DM(I0,M1);
                           {MR=(xa-xc+yb-yd)(Cb)+ (ya-yc-xb+xd)(Sb)}
                           {AY0= xc of next bfly}
                 DM(I1,M0)=MR1, MR=AR*MY0(SS);
                           {ouput x'b=(xa-xc+yb-yd)(Cb)+ (ya-yc-xb+xd)(Sb)}
                           {MR=ya-yc-xb+xd)(Cb)}
                 MR=MR-SR0*MY1(RND), MY0=DM(I7,M4);
                           {MR=(ya-yc-xb+xd)(Cb)-(xa-xc+yb-yd)(Sb)}
                           {MY0=(Cd)}
                 DM(I3,M0)=MR1, MR=MX0*MY0(SS);
                           {ouput y'b= (ya-yc-xb+xd)(Cb)-(xa-xc+yb-yd)(Sb)}
                           {MR=(xa-yb-xc+yd)(Cd)}
                 MY1=DM(I7,M7), AR=AX0-AY0;
                            {MY1=(Sd), AR=xa-xc }  
                 MR=MR+MX1*MY1(RND), AX1=DM(I2,M0);
                           {MR=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)}
                           {AX1=ya of next bfly}
                 DM(I1,M2)=MR1, MR=MX1*MY0(SS);
                           {output x'd=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)}
                           {MR=(ya+yb-yc-yd)(Cd)}
                 MR=MR-MX0*MY1(RND), AY1=DM(I2,M1);
                           {MR=(ya+yb-yc-yd)(Cd)-(xa-xc-yb+yd)Sd}
                           {yc of next bfly}
stg1bfy:         DM(I3,M2)=MR1, SR=LSHIFT AR(LO);
                           {output y'd=(ya+xb-yc-xd)Cd-(xa-xc-yb+yd)Sd}
                           {SR0=ya-yc of next bfly}



{_____________________________Stage 2_______________________________}



stage2:     I0=^inplace;       {in       -> Xa,Xc}
            I1=^inplace+128;   {in+N/8   -> Xb,Xd}
            I2=^inplace+1;     {in+1     -> Ya,Yc}
            I3=^inplace+129;   {in+N/8+1 -> Yb,Yd}
            M0=Nov4;           {N/4,    skip forward to dual node}
            M1=-Nov4;          {-N/4,   skip back to primary node}
            M2=-Nov4+2;        {-N/4+2, skip to next butterfly}
            M3=384;            {N*3/8,  skip to next group}
            M5=Nov4+4;         {N/4 +groups/stage*1, Cb Sb offset}
            M6=Nov4+8;         {N/4 +groups/stage*2, Cc Sc offset}
            M7=Nov4+12;        {N/4 +groups/stage*3, Cd Sd offset}
            SI=64;             {Bfy/group, save counter for inner loop} 
            DM(bfy_count)=SI;
            CNTR=4;            {groups/stage}
            CALL mid_stg;      {do stage 2}

{_____________________________Stage 3_______________________________}

stage3:     I0=^inplace;        {in         -> Xa,Xc}
            I1=^inplace+32;     {in+N/32    -> Xb,Xd}
            I2=^inplace+1;      {in+1       -> Ya,Yc}
            I3=^inplace+33;     {in+N/32+1  -> Yb,Yd}
            M0=64;              {N/16,    skip forward to dual node}
            M1=-64;             {-N/16,   skip back to primary node}
            M2=-62;             {-N/16+2, skip to next butterfly}
            M3=96;              {N*3/32,  skip to next group}
            M5=Nov4+16;         {N/4 +groups/stage*1, Cb Sb offset}
            M6=Nov4+32;         {N/4 +groups/stage*2, Cc Sc offset}
            M7=Nov4+48;         {N/4 +groups/stage*3, Cd Sd offset}
            SI=16;              {Bfy/group, save counter for inner loop} 
            DM(bfy_count)=SI;
            CNTR=16;            {groups/stage}
            CALL mid_stg;       {do stage 3}

{_____________________________Stage 4_______________________________}

stage4:     I0=^inplace;       {in          -> Xa,Xc}
            I1=^inplace+8;     {in+N/128    -> Xb,Xd}
            I2=^inplace+1;     {in+1        -> Ya,Yc}
            I3=^inplace+9;     {in+N/128+1  -> Yb,Yd}
            M0=16;             {N/64,    skip forward to dual node}
            M1=-16;            {-N/64,   skip back to primary node}
            M2=-14;            {-N/64+2, skip to next butterfly}
            M3=24;             {N*3/128, skip to next group}
            M5=Nov4+64;        {N/4 +groups/stage*1, Cb Sb offset}
            M6=Nov4+128;       {N/4 +groups/stage*2, Cc Sc offset}
            M7=Nov4+192;       {N/4 +groups/stage*3, Cd Sd offset}
            SI=4;              {Bfly/group, save counter inner loop}
            DM(bfy_count)=SI;
            CNTR=64;           {groups/stage}
            CALL mid_stg;      {do stage 4}



{_________________Last Stage__No Multiplies__________________________}


                            
laststage:  I0=^inplace;       {in         ->Xa,Xc}
            I1=^inplace+2;     {in+N/512   ->Xb,Xd}
            I2=^inplace+1;     {in+1       ->Ya,Yc}
            I3=^inplace+3;     {in+N/512+1 ->Yb,Yd}
            M0=4;              {N/256,  skip forward to dual node}
            M1=-4;             {-N/256, skip back to primary node}
            AX0=DM(I0,M0);     {first Xa} 
            AY0=DM(I0,M1);     {first Xc}
            CNTR=Nov4;         {groups/stage}
            DO laststgbfy UNTIL CE;
                 AR=AX0-AY0, AX1=DM(I2,M0);       {AR=xa-xc, AX1=ya}
                 SR=LSHIFT AR(LO), AY1=DM(I2,M1); {SR0=xa-xc, AY1=yc}
                 AR=AX0+AY0, AX0=DM(I1,M0);       {AR=xa+xc, AX0=xb}
                 MR0=AR, AR=AX1+AY1;              {MR0=xa+xc, AR=ya+yc}
                 MR1=AR, AR=AX1-AY1;              {MR1=ya+yc, AR=ya-yc}
                 SR=SR OR LSHIFT AR (HI), AY0=DM(I1,M1);
                                                  {SR1=ya-yc, AY0 xd}
                 AF=AX0+AY0, AX1=DM(I3,M0);       {AF=xb+xd, AX1=yb}
                 AR=MR0+AF, AY1=DM(I3,M1);        {AR=xa+xc+xb+xd, AY1=yd}
                 DM(I0,M0)=AR, AR=MR0-AF;         {output x'a=xa+xc+xb+xd}
                                                  {AR=xa+xc-(xb+xd)}
                 DM(I0,M0)=AR, AF=AX1+AY1;        {output x'c=xa+xc-(xb+xd)}
                                                  {AF=yb+yd}
                 AR=MR1+AF;                       {AR=ya+yb+yc+yd}
                 DM(I2,M0)=AR, AR=MR1-AF;         {output y'a=ya+yc+yb+yd}
                                                  {AR=ya+yc-(yb+yd)}
                 DM(I2,M0)=AR, AR=AX0-AY0;        {output y'c=ya+yc-(yb+yd)}
                                                  {AR=xb-xd}
                 AX0=DM(I0,M0);                   {AX0=xa of next group}
                 AF=AX1-AY1, AY1=AR;              {AF=yb-yd, AY1=xb-xd}
                 AR=SR0+AF, AY0=DM(I0,M1);        {AR=xa-xc+yb-yd}
                                                  {AY0=xc of next group}
                 DM(I1,M0)=AR, AR=SR0-AF;         {output x'b=xa-xc+yb-yd}
                                                  {AR=xa-xc-(yb-yd)}
                 DM(I1,M0)=AR, AR=SR1-AY1;        {output x'd=xa-xc-(yb-yd)}
                                                  {AR=ya-yc+(xb-xd)}
                 DM(I3,M0)=AR, AR=SR1+AY1;        {output y'b=ya-yc+(xb-xd)}
                                                  {AR=ya-yc-(xb-xd)}
laststgbfy:      DM(I3,M0)=AR;                    {output y'd=ya-yc-(xb-xd)}
                
                 RTS;        {end and exit from FFT subroutine} 



{_____________________Subroutine for middle stages___________________}


mid_stg:    DO midgrp UNTIL CE;
                 I5=^cos_table;
                 I6=^cos_table;
                 I7=^cos_table;
                 AX0=DM(I0,M0);                        {get first Xa}
                 AY0=DM(I0,M1);                        {get first Xc}
                 AR=AX0-AY0, AX1=DM(I2,M0);            {Xa-Xc,get first Ya}
                 SR=LSHIFT AR (LO), AY1=DM(I2,M1);     {SR1=Xa-Xc,get first Yc}
                 CNTR=DM(bfy_count);                   {butterflies/group}
                     DO midbfy UNTIL CE;
                     AR=AX0+AY0, AX0=DM(I1,M0);
                                   {AR=xa+xc, AX0=xb }
                     MR0=AR, AR=AX1+AY1;
                               {MR0=xa+xc, AR=ya+yc}
                     MR1=AR, AR=AX1-AY1;
                               {MR1=ya+yc, AR=ya-yc}
                     SR=SR OR LSHIFT AR (HI), AY0=DM(I1,M1);
                               {SR1=ya-yc, AY0=xd}
                     AF=AX0+AY0, AX1=DM(I3,M0);
                               {AF=xb+xd, AX1=yb}
                     AR=MR0+AF, AY1=DM(I3,M1);
                               {AR=xa+xb+xc+xd, AY1=yd}
                     DM(I0,M0)=AR, AR=MR0-AF;
                               {output x'a=(xa+xb+xc+xd), AR=xa+xc-xb-xd}
                     AF=AX1+AY1, MX0=AR;
                               {AF=yb+yd, MX0=xa+xc-xb-xd}
                     AR=MR1+AF, MY0=DM(I6,M4);
                               {AR=ya+yc+yb+yd, MY0=(Cc)}
                     DM(I2,M0)=AR, AR=MR1-AF;
                               {output y'a, AR=ya+yc-yb-yd}
                     MR=MX0*MY0(SS), MY1=DM(I6,M6);
                               {MR=(xa+xc-xb-xd)(Cc), MY1=(Sc)}
                     MR=MR+AR*MY1(RND);
                               {MR=(xa-xb+xc-xd)(Cc)+(ya-yb+yc-yd)(Sc)}
                     DM(I0,M2)=MR1, MR=AR*MY0(SS);
                               {output x'c=xa-xb+xc-xd)(Cc)+(ya-yb+yc-yd)(Sc)}
                               {MR=(ya+yc-yb-yd)(Cc)}
                     MR=MR-MX0*MY1(RND), MY0=DM(I5,M4);
                               {MR=(ya+yc-yb-yd)(Cc)-(xa+xc-xb-xd)(Sc), MY0=(Cb)}
                     DM(I2,M2)=MR1, AR=AX0-AY0;
                               {output y'c=(ya+yc-yb-yd)(Cc)-(xa+xc-xb-xd)(Sc)}
                               {AR=xb-xd}
                     AY0=AR, AF=AX1-AY1;
                               {AY0=xb-xd, AF=yb-yd}
                     AR=SR0-AF, MY1=DM(I5,M5);
                               {AR=xa-xc-(yb-yd), MY1=(Sb)}
                     MX0=AR, AR=SR0+AF;
                               {MX0=xa-xc-yb+yd, AR=xa-xc+yb-yd}
                     SR0=AR, AR=SR1+AY0;
                               {SR0=xa-xc+yb-yd, AR=ya-yc+xb-xd}
                     MX1=AR, AR=SR1-AY0;
                               {MX1=ya-yc+xb-xd, AR=ya-yc-(xb-xd)}
                     MR=SR0*MY0(SS), AX0=DM(I0,M0);
                               {MR=(xa-xc+yb-yd)(Cb), AX0= xa of next bfly}
                     MR=MR+AR*MY1(RND), AY0=DM(I0,M1);
                               {MR=(xa-xc+yb-yd)(Cb)+ (ya-yc-xb+xd)(Sb)}
                               {AY0= xc of next bfly}
                     DM(I1,M0)=MR1, MR=AR*MY0(SS);
                               {ouput x'b=(xa-xc+yb-yd)(Cb)+ (ya-yc-xb+xd)(Sb)}
                               {MR=ya-yc-xb+xd)(Cb)}
                     MR=MR-SR0*MY1(RND), MY0=DM(I7,M4);
                               {MR=(ya-yc-xb+xd)(Cb)-(xa-xc+yb-yd)(Sb)}
                               {MY0=(Cd)}
                     DM(I3,M0)=MR1, MR=MX0*MY0(SS);
                               {ouput y'b= (ya-yc-xb+xd)(Cb)-(xa-xc+yb-yd)(Sb)}
                               {MR=(xa-yb-xc+yd)(Cd)}
                     MY1=DM(I7,M7), AR=AX0-AY0;
                                {MY1=(Sd), AR=xa-xc }  
                     MR=MR+MX1*MY1(RND), AX1=DM(I2,M0);
                               {MR=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)}
                               {AX1=ya of next bfly}
                     DM(I1,M2)=MR1, MR=MX1*MY0(SS);
                               {output x'd=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)}
                               {MR=(ya+yb-yc-yd)(Cd)}
                     MR=MR-MX0*MY1(RND), AY1=DM(I2,M1);
                               {MR=(ya+yb-yc-yd)(Cd)-(xa-xc-yb+yd)Sd}
                               {yc of next bfly}
      midbfy:        DM(I3,M2)=MR1, SR=LSHIFT AR(LO);
                               {output y'd=(ya+xb-yc-xd)Cd-(xa-xc-yb+yd)Sd}
                               {SR0=ya-yc of next bfly}
                 MODIFY (I0,M3);
                 MODIFY (I1,M3);                { point to next group }
                 MODIFY (I2,M3);                { of butterflies }
midgrp:          MODIFY (I3,M3);
        RTS;         { return to middle stage calling code}
{_____________________________End code_______________________________}
.endmod;


