{
                        TWO DIMENSIONAL FFT                          
                        Main Module                                  
                                                                    
                ENVIRONMENT:    ADSP 2100

                DESCRIPTION:
                        This program performs a two
                        dimensional FFT using the row column
                        decomposition method described in
                        "Multidimensional Digital Signal
                        Processing" by Dugeon and Mersereau.
                        Input data is stored in the variables
                        realdata and imaginarydata.
                        One dimensional FFTs are performed first    
                        on each row. The pointers rrowbase and
                        irowbase keep track of row bounds.  One
                        dimensional FFTs are then performed on 
                        each column.  The pointers rcolbase and
                        icolbase keep track of the column data.
                        The row FFTs are decimation in frequency    
                        because ths type of FFT expects normal 
                        ordered input.  Output of the row FFTs are   
                        in bit-reversed order.  The column FFTs are 
                        decimation in time.  Input is bit-reversed   
                        and output is normal ordered.  

                VARIABLES:
                        realdata          array (N X N) of real data   
                        imaginarydata     array (N X N) of imagnary data
                        twid_real         REAL part of Twiddle factors 
                        twid_imag         Imaginary part of Twiddles   
                        groups            # of groups in current stage 
                        node_space        spacing between dual node pts
                        bflys_per_group   # of butterflys per group    
                        rrowbase          real data row pointer
                        irowbase          imaginary data row pointer   
                        rcolbase          real data col. pointer
                        icolbase          imaginary data col. pointer  
                        fft_start         entry point row FFTs
                        col_fft_strt      entry point column FFTs
                        shift_strt        entry point normalizer
}
.MODULE                 FFT_2D;

.CONST                  N = 64, N_X_2 = 128;
.CONST                  N_div_2 = 32, log2N = 6;

.VAR/DM/RAM             realdata[4096];
.VAR/DM/RAM/ABS=h#1000  imaginarydata[4096];
.VAR/DM/RAM/ABS=h#2000  tempdata[4096];

.VAR/PM/RAM/CIRC        twid_real[N_div_2];
.VAR/PM/RAM/CIRC        twid_imag[N_div_2];
.VAR/DM                 row_exponents[N];
.VAR/DM                 col_exponents[N];
.VAR/PM/RAM             padding[8];
.VAR/DM/RAM             groups, node_space, bflys_per_group;
.VAR/DM/RAM             offset, rrowbase, irowbase, rcolbase, icolbase;
.VAR/DM/RAM             blk_exponent;
.VAR/DM/RAM             real_br_pointers[N];
.VAR/DM/RAM             imag_br_pointers[N];
.VAR/DM/RAM             c_real_br_pointers[N];
.VAR/DM/RAM             c_imag_br_pointers[N];
.VAR/DM/RAM             current_rrow;
.VAR/DM/RAM             current_irow;
.VAR/DM/RAM             current_rcol;
.VAR/DM/RAM             current_icol;


.init                   realdata: <realdata.dat>;
.init                   imaginarydata: <imagdata.dat>;
.init                   twid_real:  <twid_real.dat>;
.init                   twid_imag:  <twid_imag.dat>;
.init                   real_br_pointers: <real_ptr.dat>;
.init                   imag_br_pointers: <imag_ptr.dat>;
.init                   c_real_br_pointers: <c_real_p.dat>;
.init                   c_imag_br_pointers: <c_imag_p.dat>;


.init                   padding: 0,0,0,0,0,0,0,0;
.init                   groups:  1;
.init                   blk_exponent: 0;
.init                   node_space: N_div_2;
.init                   bflys_per_group:  N_div_2;

.global                 realdata, imaginarydata, twid_real, twid_imag;
.global                 groups, bflys_per_group, node_space, offset;
.global                 rrowbase, irowbase, icolbase, rcolbase, blk_exponent;
.global                 real_br_pointers, imag_br_pointers;
.global                 c_real_br_pointers, c_imag_br_pointers;
.global                 current_rrow, current_irow, tempdata;
.global                 current_rcol, current_icol;
.global                 row_exponents, col_exponents;

.external               fft_start, col_fft_strt, unscr_start;
.external               col_unscr_start, row_final_adj, col_final_adj;

                        nop;        nop;
                        nop;        nop;

                        call initialize;
                        CNTR = N;              {row counter}
                        Do rowloop until CE;    { do row FFTs }
                           call fft_start;
rowloop:                   nop;

                        CNTR = N;               {bit reverses rows}
                        I6 = ^real_br_pointers;
                        I7 = ^imag_br_pointers;
                        DO unscramble until CE; {real data -> tempdata}
                           call unscr_start;    {imaginary -> realdata}
unscramble:                nop;

                        CALL row_final_adj;
                        I7 = ^col_exponents;                        
                        CNTR = N;              { column counter }
                        DO colloop until CE;   { do column FFTs }
                           call col_init;
                           call col_fft_strt;
colloop:                   nop;

                        CNTR = N;
                        I6 = ^c_imag_br_pointers;
                        I7 = ^c_real_br_pointers;
                        DO col_unscramble until CE;
                           call col_unscr_start;
col_unscramble:            nop;

                        CALL col_final_adj;     { final BFP adjust}

                        trap;


{=========================================================================}
{=====                 INIT SUBROUTINE                               =====}
{=====          One time only initialization.  Pointers and modifiers=====}
{=====          and length registers.                                =====}
{=========================================================================}
initialize:             L0 = 0;
                        L1 = 0;
                        L2 = 0;
                        L3 = 0;
                        L4 = N_div_2;
                        L5 = N_div_2;
                        L6 = 0;
                        L7 = 0;

                        I0 = ^realdata;
                        I2 = ^imaginarydata;
                        I3 = ^tempdata;
                        I4 = ^twid_real;
                        I5 = ^twid_imag;
                        I6 = ^row_exponents;

                        dm(current_rrow) = i0;
                        dm(current_irow) = i2;

                        dm(current_icol) = I0;
                        dm(current_rcol)=  I3;

                        M0 = 0;
                        M1 = 1;
                        M3 = 0;
                        M7 = 1;

                        AF = PASS 0;
                        AX0 = 0;
                        AY0 = 0;
                        AY1 = 4096;
                        DM(offset) = AY0;       { base add. for rows }
                        DM(rrowbase) = AY0;
                        DM(irowbase) = AY1;
                        ay1 = ^tempdata;
                        DM(rcolbase) = ay1;
                        DM(icolbase) = ay0;
                        rts;

{==========================================================================}
{=====                  COL INIT SUBROUTINE                           =====}
{=====          Initialization for column DIF FFTs                    =====}
{==========================================================================}
col_init:               ay1 = N_DIV_2;
                        dm(bflys_per_group) = ay1;  { bflys = 1 }
                        ay1 = 1;
                        dm(groups) = ay1;           { groups = 32 }
                        ax1 = 2048;
                        dm(node_space) = ax1;       { node_space = 64 } 
                        m1 = 64;                    { modifier for col. points }
                        L4 = N_DIV_2;
                        I4 = ^twid_real;
                        I5 = ^twid_imag;

                        rts;

.endmod;

