{
                        TWO DIMENSIONAL FFT
                        DIF section for Row FFTs
                        ADSP 2100
                        BLOCK FLOATING POINT

        Calling Parameters:
                realdata = Real input data normally ordered
                imaginarydata = Imaginary data normally ordered
                twid_real = Twiddle factor cosine values
                twid_imag = Twiddle factor sine values
                groups = 1
                bflys_per_group = N/2
                node_space = N/2
                rrowbase = 0
                irowbase = 4096

        Return Values
                realdata = row FFT results in bit reversed order
                imaginarydata = column FFT results bit reversed         

        Altered Registers
                I0,I1,I2,I3,I4,I5,L0,L1,L2,L3,L4,L5
                M0,M1,M2,M4,M5
                AX0,AX1,AY0,AY1,AR,AF
                MX0,MX1,MY0,MY1,MR,SB,SE,SR,SI

        Altered Memory
                realdta,imaginarydata, groups, node_space
                bflys_per_group, irowbase, icolbase, offset   
}

.MODULE   col_dif_fft;

.CONST  N=64, N_div_2=32, log2N=6;

.EXTERNAL       realdata,imaginarydata, twid_real, twid_imag;
.EXTERNAL       offset, icolbase, rcolbase;
.EXTERNAL       groups,bflys_per_group,node_space;
.EXTERNAL       col_bfp, blk_exponent;

.ENTRY  col_fft_strt;
              
col_fft_strt:   CNTR=log2N;                       {Initialize stage counter}
                DO stage_loop UNTIL CE; 
                   SB = -2;                       {block exponent}
                   I0 = dm(rcolbase);
                   I2 = dm(icolbase);
                   M2 = DM(node_space);     
                   I1=I0;
                   MODIFY(I1,M2);                 { I1 -> x1 }
                   I3=I2;
                   MODIFY(I3,M2);                 { I3 -> y1 }

                   CNTR=DM(groups);             {Initialize group counter}
                   M5=CNTR;                     {Init. twiddle factor modifier}

                DO group_loop UNTIL CE; 
                        CNTR=DM(bflys_per_group);    {Init. butterfly counter}
                        AX0=DM(I0,M0);                  {AX0=x0}
                        AY0=DM(I1,M0);                  {AY0=x1}
                        AY1=DM(I3,M0);                  {AY1=y1}
                        DO bfly_loop UNTIL CE;
                           AR=AX0+AY0, AX1=DM(I2,M0), MY0=PM(I4,M5);
                                                         {AR=x0+x1,AX1=y0,MY0=C}
                           SB = EXPADJ AR;
                           DM(I0,M1)=AR, AR=AX1+AY1;      {x0=x0+x1,AR=y0+y1}
                           SB = EXPADJ AR;
                           DM(I2,M1)=AR, AR=AX0-AY0;       {y0=y0+y1,AR=x0-x1}
                           MX0=AR, AR=AX1-AY1;             {MX0=x0-x1,AR=y0-y1}

                           MR=MX0*MY0 (SS), AX0=DM(I0,M0), MY1=PM(I5,M5);
                                           {MR=(x0-x1)C,AX0=next x0,MY1=(-S)}
                           MR=MR-AR*MY1 (SS);  MR = MR(RND);
                                           {MR=(x0-x1)C-(y0-y1)(-S),AY0=next x1}
                           SB = EXPADJ MR1;
                           DM(I1,M1)=MR1, MR=AR*MY0 (SS);
                                           {x1=(x0-x1)C-(y0-y1)(-S),MR=(y0-y1)C}
                           MR=MR+MX0*MY1 (RND), AY0=DM(I1,M0);  { ay0 = new x1 }
                                   {MR=(y0-y1)C+(x0-x1)(-S),AY1=next y1}
                           DM(I3,M1)=MR1, SB = EXPADJ MR1;	{ ay1 = new y1 }
                                   {y1=(y0-y1)C+(x0-x1)(-S),check bit growth}
bfly_loop:                 AY1=DM(I3,M0);

                   MODIFY(I0,M2);  {I0->x0 of first butterfly in next group}
                   MODIFY(I1,M2);  {I1->x1 of first butterfly in next group}
                   MODIFY(I2,M2);  {I2->y0 of first butterfly in next group}
GROUP_LOOP:        MODIFY(I3,M2);  {I3->y1 of first butterfly in next group}

                CALL col_bfp;
                SI=DM(groups);  
                SR=LSHIFT SI BY 1 (LO); 
                DM(groups)=SR0;                 {groups=groups X 2}
                SI=DM(node_space);      
                SR=LSHIFT SI BY -1 (LO);        
                DM(node_space)=SR0;             {node_space=node_space / 2}
                SI=DM(bflys_per_group);
                SR=LSHIFT SI BY -1 (LO);        
stage_loop:     DM(bflys_per_group)=SR0;

                ay0 = dm(rcolbase);
                ar = ay0 + 1;
                dm(rcolbase) = ar;
                ay0 = dm(icolbase);                
                ar = ay0 + 1;
                dm(icolbase) = ar;

                ax0 = dm(blk_exponent);         {take blk exponent for this}
                dm(i7,m7) = ax0;                {fft and store in array}
                ax0 = 0;
                dm(blk_exponent) = ax0;

        RTS;
.ENDMOD;
