{
                TWO DIMENSIONAL FFT
                Column Block Floating Point Module

        Description:
                This module does the final adjusting of the column
                block floating point exponents, to normalize the 
                entire array.  Each column has an associated BFP exponent.
                This module finds the greatest magnitude exponent, 
                then shifts each column by the difference of the individual
                BFP exponent and the greatest magnitude exponent.
}
.module         all_col_bfp;
.var/dm         largest_ce;
.const          N=64;
.external       col_exponents, realdata, imaginarydata;
.external       current_rcol, current_icol;
.entry          col_final_adj;

{find the greatest of the row exponents}

col_final_adj:  i0 = ^col_exponents;
                m0 = 0;
                m1 = 1;
                m2 = N;                 {increment by N for columns}
                ax1 = 0;
                ay1 = h#1000;
                af = pass ax1;
                dm(current_rcol) = ax1;
                dm(current_icol) = ay1;

                ay0 = dm(i0,m1);        {find max of 1st two values}
                ax0 = dm(i0,m1);
                ar = ax0 - ay0;         {see which is greater}

                if ge af = pass ax0;    {af gets greatest of 1st two}
                if lt af = pass ay0;
                ax0 = dm(i0,m1);

                cntr = N-2;             {buffer size less first two values}
                do findcolmax until ce;    {find greatest in the buffer}
                  ar = ax0 - af;
                  if ge af = pass ax0;
findcolmax:       ax0 = dm(i0,m1);
                ar = pass af;           {put the largest row exponent}
                dm(largest_ce) = ar;    {in memory}


{  shift each row by the difference between the greatest and the individual
   row exponent.  this puts all of the row fft outputs in the same range}
                
                i1=dm(current_rcol);  {address of post scrambled real data}
                i2=dm(current_icol);  {address of post scrambled imag data}
                i0 = ^col_exponents;

                ay0 = dm(largest_ce);
                cntr = N;
                ax0=dm(i0,m1);         {get indiv. row BFP}

                do col UNTIL ce;
                 ar = ay0 - ax0;        {diff between greatest and row BFP}
                 se = ar;
                 af = pass 0;           {clear AC bit}
                 cntr = N;
                   do col_shift UNTIL ce;  {shift the row}
                     si = dm(i1,m0);    {get next value leave pointer}
                     sr = ashift si (hi);
                     dm(i1,m2) = sr1;   {put shifted value back, inc pointer}
                     si = dm(i2,m0);    {do same for imag values}
                     sr = ashift si (hi);
col_shift:           dm(i2,m2) = sr1;   
                 ay1=dm(current_rcol);  {increment col pointer real}
                 ar = ay1 + 1;
                 i1 = ar;
                 dm(current_rcol) = ar;
                 ay1=dm(current_icol);  {increment col pointer imag}
                 ar = ay1 + 1;
                 i2 = ar;
                 dm(current_icol) = ar;
col:             ax0=dm(i0,m1);         {get next indiv. row BFP}

                rts;

.endmod;

