.MODULE        matmul;

{       
        Single-Precision Matrix Multiplication

                           S
                Z(i,j) =  sum  [X(i,k) * Y(k,j)]   i = 0 to R; j = 0 to T
                          k=0

                X is an RxS matrix
                Y is an SxT matrix
                Z is an RxT matrix

        Calling Parameters
                I1 --> Z buffer in data memory                  L1 = 0
                I2 --> X, stored by rows in data memory         L2 = 0
                I6 --> Y, stored by rows in program memory      L6 = 0
                M0 = 1          M1 = S
                M4 = 1          M5 = T
                L0,L4,L5 = 0;
                SE = Appropriate scale value
                CNTR = R

        Return Values
                Z Buffer filled by rows

        Altered Registers
                I0,I1,I2,I4,I5,MR,MX0,MY0,SR

        Computation Time
                ((S + 8)*T + 4)*R + 2 + 2 cycles

}

.ENTRY  spmm;

spmm:      DO row_loop UNTIL CE;
             I5=I6;                                 {I5 = start of Y}
             DO column_loop UNTIL CE;
               I0=I2;                               {Set I0 to current X row}
               I4=I5;                               {Set I4 to current Y col}
               CNTR=M1;
               MR=0, MX0=DM(I0,M0), MY0=PM(I4,M5);  {Get 1st data}
               DO element_loop UNTIL CE;
element_loop:    MR=MR+MX0*MY0 (SS), MX0=DM(I0,M0), MY0=PM(I4,M5);
               SR=ASHIFT MR1 (HI), MY0=DM(I5,M4);   {Update I5}
               SR=SR OR LSHIFT MR0 (LO);            {Finish shift}
column_loop:   DM(I1,M0)=SR1;                       {Save output}
row_loop:    MODIFY(I2,M1);                         {Update I2 to next X row}
           RTS;
.ENDMOD;
