.MODULE/RAM/ABS=h#0000   graphics;

.VAR/PM/RAM/CIRC     xfm_array[h#0009];
.VAR/PM/RAM          coeff[128];

.VAR/DM/RAM/CIRC     xbuff[128];
.VAR/DM/RAM/CIRC     ybuff[128];
.VAR/DM/RAM          sin_table[h#0100];
.VAR/DM/RAM          cos_table[h#0100];
.VAR/DM/RAM          src_array[h#0150];
.VAR/DM/RAM          line_list[h#0132];
.VAR/DM/RAM          wcs_array[h#0150];
.VAR/DM/RAM          ecs_array[h#00E0];
.VAR/DM/RAM          xpntr;
.VAR/DM/RAM          ypntr;
.VAR/DM/RAM          oldx;
.VAR/DM/RAM          oldy;
.VAR/DM/RAM          newx;
.VAR/DM/RAM          newy;
.VAR/DM/RAM          dx;
.VAR/DM/RAM          dy;
.VAR/DM/RAM          mode;
.VAR/DM/RAM          rotation;
.VAR/DM/RAM          xyorzflag;

.CONST          numpoints=112;
.CONST          numpoints_2=224;
.CONST          half_scale=128;
.CONST          sin_angle=h#2121;                {sin(15 deg)*32767}
.CONST          cos_angle=h#7BA2;                {cos(15 deg)*32767}

.PORT           adx, ady, dax, day, daz;

.INIT           sin_table: <sin.dat>;
.INIT           cos_table: <cos.dat>;
.INIT           src_array: <src.dat>;
.INIT           line_list: <lin.dat>;
.INIT           coeff:     <coeff.dat>;

.MACRO          nops;
                NOP; NOP; NOP; NOP; NOP; NOP;
.ENDMACRO;

.MACRO          beam_me_up_scotty;
.LOCAL          dwell;
                CNTR=10;
                DM(daz)=AX1;                     {phasers on stun, gentelman}
                DO dwell UNTIL CE;               {lock on target}
dwell:             NOP;                          {fire!}
                DM(daz)=AX0;                     {hold your fire!}
.ENDMACRO;

{initializations...}
                RTI; RTI; JUMP ir2_serve; RTI;
{default to linear addressing}
                L0=0; L1=0; L2=0; L3=0; L4=0; L5=0; L6=0; L7=0;
                PX=0;               {clear bus exchange register}
                ENA SEC_REG;        {init secondary registers with constants}
                MY0=1;              {used in the line segment drawing routines}
                MY1=-1;
                AX0=h#00FF;         {to turn off the beam}
                AX1=h#0000;         {to turn it on}
                DIS SEC_REG;

                AY0=0;              {initialization value}
                DM(rotation)=AY0;   {init autorotation counter}
                DM(mode)=AY0;       {init display mode}
                DM(xyorzflag)=AY0;  {init x y or z flag}

                I0=^xbuff;          {init xbuff and ybuff delay lines}
                I1=^ybuff;
                M0=1;
                CNTR=128;           {clear samples of 128 tap filter}
                DO initloop UNTIL CE;
                   DM(I0,M0)=AY0;
initloop:          DM(I1,M0)=AY0;

                AY0=^xbuff;
                DM(xpntr)=AY0;      {init new x sample pointer}
                AY0=^ybuff;
                DM(ypntr)=AY0;      {init new y sample pointer}

                ICNTL=h#0004;       {make IRQ2 edge-sensitive}
                IMASK=h#0004;       {enalbe IRQ2}

{begin actual code}
mainloop:       CALL bldxfm;        {read adcs and build new transform matrix}
                CALL doxfm;         {transform source data by new matrix}
                CALL doproj;        {calculate 2d projection of 3d object}
                CALL display;       {drive xyz axes on scope using quad dac}
                JUMP mainloop;      {display until we have a new rotation}

{interrupt routine to sequence the display mode through
autorotate, unfiltered joystick and filtered joystick control}
ir2_serve:      AX0=DM(mode);
                AY0=0;
                AR=AX0 XOR AY0;
                IF EQ JUMP make1;
                AY0=1;
                AR=AX0 XOR AY0;
                IF EQ JUMP make2;
                AY0=2;
                AR=AX0 XOR AY0;
                IF EQ JUMP make3;
                AY0=3;
                AR=AX0 XOR AY0;
                IF EQ JUMP makez;
make1:          AX0=1;
                DM(mode)=AX0;
                RTI;
make2:          AX0=2;
                DM(mode)=AX0;
                RTI;
make3:          AX0=3;
                DM(mode)=AX0;
                RTI;
makez:          AX0=0;
                DM(mode)=AX0;
                RTI;
                
    

{BLDXFM
Module to build the master transformation matrix from the three rotational
axes components as sampled by the AD7824 quad ADC.

The following registers are blown away by this routine,
depending upon the display mode:
    I0, I4
    L0
    M0-M3, M4
    AX0, AY0, AR
    MX0, MY0, MF, MR
}

bldxfm:      AX0=DM(mode);          {check out display mode}
             AY0=0;
             AR=AX0 XOR AY0;
             IF EQ JUMP autoxyz;
             AY0=1;
             AR=AX0 XOR AY0;
             IF EQ JUMP autorotate;
             AY0=2;
             AR=AX0 XOR AY0;
             IF EQ JUMP averaged;
             AY0=3;
             AR=AX0 XOR AY0;
             IF EQ JUMP filtered;
             TRAP;                  {should never get here}

autoxyz:     AY0=DM(rotation);
             AR=AY0+1;
             AY0=255;
             AR=AR AND AY0;
             DM(rotation)=AR;
             M1=AR;
             M2=AR;
             M3=AR;

             AX0=DM(xyorzflag);     {get xy or z flag}
             AY0=0;
             AR=AX0 XOR AY0;        {check for zero ==> rotate only x}
             IF EQ JUMP xonly;
             AY0=1;
             AR=AX0 XOR AY0;        {check for zero ==> rotate only y}
             IF EQ JUMP yonly;
             AY0=2;
             AR=AX0 XOR AY0;        {check for zero ==> rotate only z}
             IF EQ JUMP zonly;

xonly:       AX0=M1;                {get current x rotation}
             M2=128;                {zero out y}
             M3=128;                {zero out z}
             AY0=128;               {check current rotation}
             AR=AX0 XOR AY0;        {against zero}
             IF NE JUMP calculate;  {if not zero, keep going with x}
             AR=1;                  {otherwise, change axis of rotation}
             DM(xyorzflag)=AR;      {to y before going on}
             JUMP calculate;

yonly:       AX0=M2;                {get current y rotation}
             M1=128;                {zero out x}
             M3=128;                {zero out z}
             AY0=128;               {check current rotation}
             AR=AX0 XOR AY0;        {against zero}
             IF NE JUMP calculate;  {if not zero, keep going with y}
             AR=2;                  {otherwise, change axis of rotation}
             DM(xyorzflag)=AR;      {to z before going on}
             JUMP calculate;

zonly:       AX0=M3;                {get current z rotation}
             M1=128;                {zero out x}
             M2=128;                {zero out y}
             AY0=128;               {check current rotation}
             AR=AX0 XOR AY0;        {against zero}
             IF NE JUMP calculate;  {if not zero, keep going with z}
             AR=0;                  {otherwise, change axis of rotation}
             DM(xyorzflag)=AR;      {to x before going on}
             JUMP calculate;

autorotate:  AY0=DM(rotation);
             AR=AY0+1;
             AY0=255;
             AR=AR AND AY0;
             DM(rotation)=AR;
             M1=AR;
             M2=AR;
             M3=AR;
             JUMP calculate;

{average both x and y axis joysticks over 256 samples}
averaged:    AX1=h#00FF;            {mask bits for a/d samples}
             AY1=h#00FF;
             nops;
             AX0=DM(adx);           {get 1st sample}
             NOP;
             AR=AX0 AND AY1;
             AX0=AR;
             nops;
             AY0=DM(adx);           {get 2nd sample}
             NOP;
             AR=AX1 AND AY0;
             AY0=AR;
             AF=AX0+AY0;            {add 1st and 2nd samples}
             CNTR=126;
             DO xaverage UNTIL CE;
                nops;
                AX0=DM(adx);
                NOP;
                AR=AX0 AND AY1;
                AX0=AR;
xaverage:       AF=AX0+AF;          {add in 126 more samples}
             AR=PASS AF;
             SI=AR;
             SR=LSHIFT SI BY -7 (HI);            {divide by 128 to give average}
             AX0=SR1;
             AR=AX0 AND AY1;
             M1=AR;

             nops;
             AX0=DM(ady);
             NOP;
             AR=AX0 AND AY1;
             AX0=AR;
             nops;
             AY0=DM(ady);
             NOP;
             AR=AX1 AND AY0;
             AY0=AR;
             AF=AX0+AY0;
             CNTR=126;
             DO yaverage UNTIL CE;
                nops;
                AX0=DM(ady);
                NOP;
                AR=AX0 AND AY1;
                AX0=AR;
yaverage:       AF=AX0+AF;
             AR=PASS AF;
             SI=AR;
             SR=LSHIFT SI BY -7 (HI);
             AX0=SR1;
             AR=AX0 AND AY1;
             M2=AR;

             M3=128;                {no z on manual rotation}
             JUMP calculate;

{filter x axis}
filtered:    L0=%xbuff;             {setup circular buffer for sample buffer}
             M0=1;                  {init buffer increment}
             M4=1;                  {init coeff increment}
             I0=DM(xpntr);          {get current buffer pointer}
             I4=^coeff;             {init coeff pointer}
             nops;
             AX0=DM(adx);           {get a sample}
             NOP;
             AY0=h#00FF;
             AR=AX0 AND AY0;        {mask out upper bits}
             MX0=AR;                {load it into multiplier}
             DM(I0,M0)=MX0;         {add it to the delay line}
             MY0=PM(I4,M4);         {load 1st coeff}
             MR=0;
             CNTR=127;
             DO xfilter UNTIL CE;
xfilter:        MR=MR+MX0*MY0(SS), MX0=DM(I0,M0), MY0=PM(I4,M4);
             MR=MR+MX0*MY0(SS);
             AX0=MR1;
             AR=AX0 AND AY0;        {mask out any wraparound}
             MODIFY(I0,M0);         {inc sample buffer once more}
             DM(xpntr)=I0;          {save buffer pointer}
             DM(newx)=AR;           {save filtered x}

{filter y axis}
             I0=DM(ypntr);          {get current buffer pointer}
             I4=^coeff;             {init coeff pointer}
             nops;
             AX0=DM(ady);           {get a sample}
             NOP;
             AY0=h#00FF;
             AR=AX0 AND AY0;        {mask out upper bits}
             MX0=AR;                {load it into multiplier}
             DM(I0,M0)=MX0;         {add it to the delay line}
             MY0=PM(I4,M4);         {load 1st coeff}
             MR=0;
             CNTR=127;
             DO yfilter UNTIL CE;
yfilter:        MR=MR+MX0*MY0(SS), MX0=DM(I0,M0), MY0=PM(I4,M4);
             MR=MR+MX0*MY0(SS);
             AX0=MR1;
             AR=AX0 AND AY0;        {mask out any wraparound}
             MODIFY(I0,M0);         {inc sample buffer once more}
             DM(ypntr)=I0;          {save buffer pointer}
             DM(newy)=AR;           {save filtered x}

             L0=0;                  {restore linear addressing}
             M1=DM(newx);           {load filtered x value}
             M2=DM(newy);           {load filtered y value}
             M3=128;                {no z on manual rotation}

calculate:   I4=^xfm_array;         {reset xfm pointer}
             M4=1;                  {to walk through xfm array}
             M0=0;                  {no modify after trig table lookup}

{calculate element xfm(11)...}
             I0=^cos_table;
             MODIFY(I0,M2);
             MX0=DM(I0,M0);         {cy}
             I0=^cos_table;
             MODIFY(I0,M3);
             MY0=DM(I0,M0);         {cz}
             MR=MX0*MY0(RND);       {cy*cz}
             PM(I4,M4)=MR1;

{calculate element xfm(21)...}
             I0=^sin_table;
             MODIFY(I0,M1);
             MX0=DM(I0,M0);         {sx}
             I0=^sin_table;
             MODIFY(I0,M2);
             MY0=DM(I0,M0);         {sy}
             MF=MX0*MY0(RND);       {sx*sy}
             I0=^cos_table;
             MODIFY(I0,M3);
             MX0=DM(I0,M0);         {cz}
             MR=MX0*MF(SS);         {sx*sy*cz}
             I0=^cos_table;
             MODIFY(I0,M1);
             MX0=DM(I0,M0);         {cx}
             I0=^sin_table;
             MODIFY(I0,M3);
             MY0=DM(I0,M0);         {sz}
             MR=MR-MX0*MY0(SS);     {sx*sy*cz-cx*sz}
             MR=MR(RND);            {loose round bug by not rounding above}
             PM(I4,M4)=MR1;

{calculate element xfm(31)...}
             I0=^cos_table;
             MODIFY(I0,M1);
             MX0=DM(I0,M0);         {cx}
             I0=^sin_table;
             MODIFY(I0,M2);
             MY0=DM(I0,M0);         {sy}
             MF=MX0*MY0(RND);       {cx*sy}
             I0=^cos_table;
             MODIFY(I0,M3);
             MX0=DM(I0,M0);         {cz}
             MR=MX0*MF(SS);         {sx*sy*cz}
             I0=^sin_table;
             MODIFY(I0,M1);
             MX0=DM(I0,M0);         {sx}
             I0=^sin_table;
             MODIFY(I0,M3);
             MY0=DM(I0,M0);         {sz}
             MR=MR+MX0*MY0(RND);    {cx*sy*cz+sx*sz}
             PM(I4,M4)=MR1;

{calculate element xfm(12)...}
             I0=^cos_table;
             MODIFY(I0,M2);
             MX0=DM(I0,M0);         {cy}
             I0=^sin_table;
             MODIFY(I0,M3);
             MY0=DM(I0,M0);         {sz}
             MR=MX0*MY0(RND);       {cy*sz}
             PM(I4,M4)=MR1;

{calculate element xfm(22)...}
             I0=^sin_table;
             MODIFY(I0,M1);
             MX0=DM(I0,M0);         {sx}
             I0=^sin_table;
             MODIFY(I0,M2);
             MY0=DM(I0,M0);         {sy}
             MF=MX0*MY0(RND);       {sx*sy}
             I0=^sin_table;
             MODIFY(I0,M3);
             MX0=DM(I0,M0);         {sz}
             MR=MX0*MF(SS);         {sx*sy*sz}
             I0=^cos_table;
             MODIFY(I0,M1);
             MX0=DM(I0,M0);         {cx}
             I0=^cos_table;
             MODIFY(I0,M3);
             MY0=DM(I0,M0);         {cz}
             MR=MR+MX0*MY0(RND);    {sx*sy*sz+cx*cz}
             PM(I4,M4)=MR1;

{calculate element xfm(32)...}
             I0=^cos_table;
             MODIFY(I0,M1);
             MX0=DM(I0,M0);         {cx}
             I0=^sin_table;
             MODIFY(I0,M2);
             MY0=DM(I0,M0);         {sy}
             MF=MX0*MY0(RND);       {cx*sy}
             I0=^sin_table;
             MODIFY(I0,M3);
             MX0=DM(I0,M0);         {sz}
             MR=MX0*MF(SS);         {cx*sy*sz}
             I0=^sin_table;
             MODIFY(I0,M1);
             MX0=DM(I0,M0);         {sx}
             I0=^cos_table;
             MODIFY(I0,M3);
             MY0=DM(I0,M0);         {cz}
             MR=MR-MX0*MY0(SS);     {cx*sy*sz-sx*cz}
             MR=MR(RND);            {loose round bug by not rounding above}
             PM(I4,M4)=MR1;

{calculate element xfm(13)...}
             I0=^sin_table;
             MODIFY(I0,M2);
             MR1=DM(I0,M0);         {sy}
             AR=-MR1;               {-sy}
             PM(I4,M4)=AR;

{calculate element xfm(23)...}
             I0=^sin_table;
             MODIFY(I0,M1);
             MX0=DM(I0,M0);         {sx}
             I0=^cos_table;
             MODIFY(I0,M2);
             MY0=DM(I0,M0);         {cy}
             MR=MX0*MY0(RND);       {sx*cy}
             PM(I4,M4)=MR1;

{calculate element xfm(33)...}
             I0=^cos_table;
             MODIFY(I0,M1);
             MX0=DM(I0,M0);         {cx}
             I0=^cos_table;
             MODIFY(I0,M2);
             MY0=DM(I0,M0);         {cy}
             MR=MX0*MY0(RND);       {cx*cy}
             PM(I4,M4)=MR1;

             RTS;

{DOXFM
Module to perform the actual transformation of the raw source data
by the master transformation matrix.

The transformation, xfm_array, is organized sequentially, first by columns,
then by rows:

                        | 11 12 13 |
        xfm_array =     | 21 22 23 |
                        | 31 32 33 |

i.e., xfm_array is stored in a nine location buffer in PMD as follows:

        xfm_array[1..9] = (11, 21, 31, 12, 22, 32, 13, 23, 33)

The following registers are blown away by this routine:
        I0, I1, I4
        L4
        M0, M1, M5
        MX0, MY0, MR
}

doxfm:
        I0=^src_array;              {get source dm array pointer}
        I1=^wcs_array;              {get wcs dm array pointer}
        I4=^xfm_array;              {get transform pm array pointer}

        L4=%xfm_array;   {to run modulo9 through the transform array}

        M0=2;            {to get the next xyz for each new transform}
        M2=-2;           {retard src_array pointer by 2 for each column}
        M1=1;            {general purpose for simple incrementing}
        M5=-1;           {special decrement for xfm at end of point loop}

        CNTR=numpoints;                          {transform all point vectors}
        DO points UNTIL CE;
           MX0=DM(I0,M1), MY0=PM(I4,M4);         {load 1st set of operands}
           CNTR=3;                               {do three columns}
              DO columns UNTIL CE;
                 MR=MX0*MY0(SS), MX0=DM(I0,M1), MY0=PM(I4,M4); {1st mul clears}
                 MR=MR+MX0*MY0(SS), MX0=DM(I0,M2), MY0=PM(I4,M4);
                 MR=MR+MX0*MY0(SS), MX0=DM(I0,M1), MY0=PM(I4,M4);
columns:         DM(I1,M1)=MR1;     {store the transformed component}
           MODIFY(I4,M5);           {retard xfm pointer by one}
points:    MODIFY(I0,M0);           {pick up next set of vectors}

        L4=0;

        RTS;                        {jump back, Jack!}

{DOPROJ
Module to do the 3D to 2D object projection

The following registers are blown away by this routine:
     I0-I3, M0, M1
     AX0, AY0, MX0, MX1, MY0
     MR, AR
}

doproj:
                I0=^wcs_array;      {x-component pointer}
                I1=^wcs_array+1;    {y-component pointer}
                I2=^wcs_array+2;    {z-component pointer}
                I3=^ecs_array;      {2D result pointer}

                M0=0;               {for no-modify access}
                M1=1;               {simple inc for ecs}
                M3=3;               {skip thru wcs by 3s}

                MX0=sin_angle;      {load constants}
                MX1=cos_angle;

                MY0=DM(I2,M3);      {preload z to start pipeline}

                CNTR=numpoints;
                ENA AR_SAT;         {slamm over/underflows}
           DO project UNTIL CE;
                MR=MX1*MY0(RND), AY0=DM(I0,M3);  {z*cos(angle), get x}
                AR=MR1+AY0;                      {x+z*cos(angle)}
                DM(I3,M1)=AR;                    {store x projection}

                MR=MX0*MY0(RND), AY0=DM(I1,M3);  {z*sin(angle), get y}
                AR=MR1+AY0, MY0=DM(I2,M3);       {y+z*sin(angle), get next z}
project:        DM(I3,M1)=AR;                    {store y projection}
                DIS AR_SAT;                      {restore normal ALU operation}

{in-place adjust ecs data for fullscale dac range}
                MX0=half_scale;     {load scale factor}
                AY0=half_scale;     {load axis offset}
                I3=^ecs_array;      {init 2D result pointer}
                CNTR=numpoints_2;   {twice numpoints for x&y}
                DO scale UNTIL CE;
                   MY0=DM(I3,M0);
                   MR=MX0*MY0(RND); {applly scaling}
                   AR=MR1+AY0;      {shift axis}
scale:             DM(I3,M1)=AR;    {save scaled x&y}

                RTS;

{DISPLAY
Module to display the 2D image in ecs_array on scope by writeing
to the dacs the point vectors and z data (for beamon/beamoff).

The line segment drawing routines for each octant are derrived from the
Bresenham Algorithum which may be found in any computer graphics text.

The backround registers are used here during the actual line segment
drawing routines.

The following primary registers are blown away by this routine:
     AX0, AX1, AY0, AR, AF
     I0, I1, I4
}

display:        IMASK=0;            {disable interrupts}
                AX0=^ecs_array-2;   {-2 to adjust for 0/1 starting}
                I1=^line_list;

repeat:
                AX1=DM(I1,M1);      {load next linelist value}
                AR=PASS AX1;
                IF LT JUMP done;    {watch for -1 flag to get out}
                IF EQ JUMP moveto;  {move to new line with beam off}
                JUMP lineto;        {draw line from old to new vector}

moveto:         SI=DM(I1,M1);       {do a left shift by one to account}
                SR=LSHIFT SI BY 1 (HI);   {for xy interleave of ecs array}
                AY0=SR1;
                AR=AX0+AY0;         {add in base address of ecs-2}
                I0=AR;              {transfer to ag1}
                AR=DM(I0,M1);       {get new x-coordinate}
                DM(dax)=AR;         {write to dac with beam off}
                DM(oldx)=AR;        {update old x-coordinate}
                AR=DM(I0,M1);       {get new y-coordinate}
                DM(day)=AR;         {write to dac with beam off}
                DM(oldy)=AR;        {update old y-coordinate}
                JUMP repeat;

lineto:                             {get 1st point and}
                SI=AX1;             {do a left shift by one to account}
                SR=LSHIFT SI BY 1 (HI);   {for xy interleave of ecs array}
                AY0=SR1;            {with respect to line list}
                AR=AX0+AY0;         {add in base address of ecs-2}
                I0=AR;              {transfer x-addr of ecs to ag1}
                AR=DM(I0,M1);
                DM(newx)=AR;        {update new x-coordinate}
                AR=DM(I0,M1);
                DM(newy)=AR;        {update new y-coordinate}
                AF=PASS 0;          {init for indirect jump offset}
                AX1=DM(newx);       {do delta x}
                AY1=DM(oldx);
                AR=AX1-AY1;
                DM(dx)=AR;
                AX1=4;
                IF GT AF=AX1 OR AF;
                AX1=DM(newy);       {do delta y}
                AY1=DM(oldy);
                AR=AX1-AY1;
                DM(dy)=AR;
                AX1=2;
                IF GT AF=AX1 OR AF;
                AX1=DM(dy);         {do |dx|-|dy|}
                AR=ABS AX1;
                AY1=AR;
                AX1=DM(dx);
                AR=ABS AX1;
                AX1=AR;
                AR=AX1-AY1;
                AX1=1;
                IF GT AF=AX1 OR AF;
                AX1=^jump_table;    {add in jump table base address}
                AR=AX1+AF;
                I4=AR;
                JUMP (I4);          {do the indirect jump}

jump_table:     JUMP octant6;
                JUMP octant5;
                JUMP octant3;
                JUMP octant4;
                JUMP octant7;
                JUMP octant8;
                JUMP octant2;
                JUMP octant1;
{
In the following code segments:
     MY0' holds +2 (really +1, but shift makes it +2)
     MY1' holds -2 (really -1, but shift makes it -2)
     SR0' holds incr1
     SR1' holds incr2
     AY0' holds current x pixel
     AY1' holds current y pixel
     AX0' holds h#00FF to turn off the beam with
     AX1' holds h#0000 to turn on the beam
     AF   tracks the error term, d
     MX0' holds intermediate stuff
     MR gets hosed
     AR' holds intermediate stuff
}
octant1:        ENA SEC_REG;
                MX0=DM(dy);
                MR=MX0*MY0(SS);
                SR0=MR0;                {incr1 = 2dy}
                SR1=DM(dx);
                AY1=DM(dy);
                AR=SR1-AY1;
                MR=AR*MY1(SS);
                SR1=MR0;                {incr2 = -2(dx-dy)}
                AR=DM(dx);
                AF=ABS AR;
                AR=AF+1;
                CNTR=AR;                {draw line with |dx|+1 pixels}
                AY0=DM(dx);             {init AF with d = incr1-dx}
                AF=PASS AY0;
                AF=SR0-AF;
                AY0=DM(oldx);           {start at last point}
                AY1=DM(oldy);
                DO octant1loop UNTIL CE; {ponder the powerful pixel pokeing}
                   DM(dax)=AY0;         {move beam to new x along line segment}
                   DM(day)=AY1;         {move beam to new y along line segment}
                   beam_me_up_scotty;
                   AR=AY0+1;            {increment x}
                   AY0=AR;
                   AR=PASS AF;          {check sign of error term, d}
                   PUSH STS;            {save status for 'else' test}
                   IF LT AF=SR0+AF;     {'if then' clause: d = d+incr1}
                   POP STS;
                   IF LT JUMP octant1loop;
                   AR=AY1+1;            {'else' clause...}
                   AY1=AR;              {inc y}
                   AF=SR1+AF;           {d = d+incr2}
octant1loop:       NOP;
                JUMP update;

octant2:        ENA SEC_REG;
                MX0=DM(dx);
                MR=MX0*MY0(SS);
                SR0=MR0;                {incr1 = 2dx}
                SR1=DM(dx);
                AY1=DM(dy);
                AR=SR1-AY1;
                MR=AR*MY0(SS);
                SR1=MR0;                {incr2 = 2(dx-dy)}
                AR=DM(dy);
                AF=ABS AR;
                AR=AF+1;
                CNTR=AR;                {draw line with |dy|+1 pixels}
                AY0=DM(dy);             {init AF with d = incr1-dy}
                AF=PASS AY0;
                AF=SR0-AF;
                AY0=DM(oldx);           {start at last point}
                AY1=DM(oldy);
                DO octant2loop UNTIL CE;  {ponder the powerful pixel pokeing}
                   DM(dax)=AY0;         {move beam to new x along line segment}
                   DM(day)=AY1;         {move beam to new y along line segment}
                   beam_me_up_scotty;
                   AR=AY1+1;            {increment y}
                   AY1=AR;
                   AR=PASS AF;          {check sign of error term, d}
                   PUSH STS;            {save status for 'else' test}
                   IF LT AF=SR0+AF;     {'if then' clause: d = d+incr1}
                   POP STS;
                   IF LT JUMP octant2loop;
                   AR=AY0+1;            {'else' clause...}
                   AY0=AR;              {inc x}
                   AF=SR1+AF;           {d = d+incr2}
octant2loop:       NOP;
                JUMP update;

octant3:        ENA SEC_REG;
                MX0=DM(dx);
                MR=MX0*MY1(SS);
                SR0=MR0;                {incr1 = -2dx}
                SR1=DM(dx);
                AY1=DM(dy);
                AR=SR1+AY1;
                MR=AR*MY1(SS);
                SR1=MR0;                {incr2 = -2(dx+dy)}
                AR=DM(dy);
                AF=ABS AR;
                AR=AF+1;
                CNTR=AR;                {draw line with |dy|+1 pixels}
                AY0=DM(dy);             {init AF with d = incr1-dy}
                AF=PASS AY0;
                AF=SR0-AF;
                AY0=DM(oldx);           {start at last point}
                AY1=DM(oldy);
                DO octant3loop UNTIL CE;  {ponder the powerful pixel pokeing}
                   DM(dax)=AY0;         {move beam to new x along line segment}
                   DM(day)=AY1;         {move beam to new y along line segment}
                   beam_me_up_scotty;
                   AR=AY1+1;            {increment y}
                   AY1=AR;
                   AR=PASS AF;          {check sign of error term, d}
                   PUSH STS;            {save status for 'else' test}
                   IF LT AF=SR0+AF;     {'if then' clause: d = d+incr1}
                   POP STS;
                   IF LT JUMP octant3loop;
                   AR=AY0-1;            {'else' clause...}
                   AY0=AR;              {dec x}
                   AF=SR1+AF;           {d = d+incr2}
octant3loop:       NOP;
                JUMP update;

octant4:        ENA SEC_REG;
                MX0=DM(dy);
                MR=MX0*MY0(SS);
                SR0=MR0;                {incr1 = 2dy}
                SR1=DM(dx);
                AY1=DM(dy);
                AR=SR1+AY1;
                MR=AR*MY0(SS);
                SR1=MR0;                {incr2 = 2(dx+dy)}
                AR=DM(dx);
                AF=ABS AR;
                AR=AF+1;
                CNTR=AR;                {draw line with |dx|+1 pixels}
                AY0=DM(dx);             {init AF with d = incr1+dx}
                AF=PASS AY0;
                AF=SR0+AF;
                AY0=DM(oldx);           {start at last point}
                AY1=DM(oldy);
                DO octant4loop UNTIL CE;  {ponder the powerful pixel pokeing}
                   DM(dax)=AY0;         {move beam to new x along line segment}
                   DM(day)=AY1;         {move beam to new y along line segment}
                   beam_me_up_scotty;
                   AR=AY0-1;            {decrement x}
                   AY0=AR;
                   AR=PASS AF;          {check sign of error term, d}
                   PUSH STS;            {save status for 'else' test}
                   IF LT AF=SR0+AF;     {'if then' clause: d = d+incr1}
                   POP STS;
                   IF LT JUMP octant4loop;
                   AR=AY1+1;            {'else' clause...}
                   AY1=AR;              {inc y}
                   AF=SR1+AF;           {d = d+incr2}
octant4loop:       NOP;
                JUMP update;

octant5:          ENA SEC_REG;
                MX0=DM(dy);
                MR=MX0*MY1(SS);
                SR0=MR0;                {incr1 = -2dy}
                SR1=DM(dx);
                AY1=DM(dy);
                AR=SR1-AY1;
                MR=AR*MY0(SS);
                SR1=MR0;                {incr2 = 2(dx-dy)}
                AR=DM(dx);
                AF=ABS AR;
                AR=AF+1;
                CNTR=AR;                {draw line with |dx|+1 pixels}
                AY0=DM(dx);             {init AF with d = incr1+dx}
                AF=PASS AY0;
                AF=SR0+AF;
                AY0=DM(oldx);           {start at last point}
                AY1=DM(oldy);
                DO octant5loop UNTIL CE;  {ponder the powerful pixel pokeing}
                   DM(dax)=AY0;         {move beam to new x along line segment}
                   DM(day)=AY1;         {move beam to new y along line segment}
                   beam_me_up_scotty;
                   AR=AY0-1;            {decrement x}
                   AY0=AR;
                   AR=PASS AF;          {check sign of error term, d}
                   PUSH STS;            {save status for 'else' test}
                   IF LT AF=SR0+AF;     {'if then' clause: d = d+incr1}
                   POP STS;
                   IF LT JUMP octant5loop;
                   AR=AY1-1;            {'else' clause...}
                   AY1=AR;              {dec y}
                   AF=SR1+AF;           {d = d+incr2}
octant5loop:       NOP;
                JUMP update;

octant6:        ENA SEC_REG;
                MX0=DM(dx);
                MR=MX0*MY1(SS);
                SR0=MR0;                {incr1 = -2dx}
                SR1=DM(dx);
                AY1=DM(dy);
                AR=SR1-AY1;
                MR=AR*MY1(SS);
                SR1=MR0;                {incr2 = -2(dx-dy)}
                AR=DM(dy);
                AF=ABS AR;
                AR=AF+1;
                CNTR=AR;                {draw line with |dy|+1 pixels}
                AY0=DM(dy);             {init AF with d = incr1+dy}
                AF=PASS AY0;
                AF=SR0+AF;
                AY0=DM(oldx);           {start at last point}
                AY1=DM(oldy);
                DO octant6loop UNTIL CE;  {ponder the powerful pixel pokeing}
                   DM(dax)=AY0;         {move beam to new x along line segment}
                   DM(day)=AY1;         {move beam to new y along line segment}
                   beam_me_up_scotty;
                   AR=AY1-1;            {decrement y}
                   AY1=AR;
                   AR=PASS AF;          {check sign of error term, d}
                   PUSH STS;            {save status for 'else' test}
                   IF LT AF=SR0+AF;     {'if then' clause: d = d+incr1}
                   POP STS;
                   IF LT JUMP octant6loop;
                   AR=AY0-1;            {'else' clause...}
                   AY0=AR;              {dec x}
                   AF=SR1+AF;           {d = d+incr2}
octant6loop:       NOP;
                JUMP update;

octant7:        ENA SEC_REG;
                MX0=DM(dx);
                MR=MX0*MY0(SS);
                SR0=MR0;                {incr1 = 2dx}
                SR1=DM(dx);
                AY1=DM(dy);
                AR=SR1+AY1;
                MR=AR*MY0(SS);
                SR1=MR0;                {incr2 = 2(dx+dy)}
                AR=DM(dy);
                AF=ABS AR;
                AR=AF+1;
                CNTR=AR;                {draw line with |dy|+1 pixels}
                AY0=DM(dy);             {init AF with d = incr1+dy}
                AF=PASS AY0;
                AF=SR0+AF;
                AY0=DM(oldx);           {start at last point}
                AY1=DM(oldy);
                DO octant7loop UNTIL CE;  {ponder the powerful pixel pokeing}
                   DM(dax)=AY0;         {move beam to new x along line segment}
                   DM(day)=AY1;         {move beam to new y along line segment}
                   beam_me_up_scotty;
                   AR=AY1-1;            {decrement y}
                   AY1=AR;
                   AR=PASS AF;          {check sign of error term, d}
                   PUSH STS;            {save status for 'else' test}
                   IF LT AF=SR0+AF;     {'if then' clause: d = d+incr1}
                   POP STS;
                   IF LT JUMP octant7loop;
                   AR=AY0+1;            {'else' clause...}
                   AY0=AR;              {inc x}
                   AF=SR1+AF;           {d = d+incr2}
octant7loop:       NOP;
                JUMP update;

octant8:        ENA SEC_REG;
                MX0=DM(dy);
                MR=MX0*MY1(SS);
                SR0=MR0;                {incr1 = -2dy}
                SR1=DM(dx);
                AY1=DM(dy);
                AR=SR1+AY1;
                MR=AR*MY1(SS);
                SR1=MR0;                {incr2 = -2(dx+dy)}
                AR=DM(dx);
                AF=ABS AR;
                AR=AF+1;
                CNTR=AR;                {draw line with |dx|+1 pixels}
                AY0=DM(dx);             {init AF with d = incr1-dx}
                AF=PASS AY0;
                AF=SR0-AF;
                AY0=DM(oldx);           {start at last point}
                AY1=DM(oldy);
                DO octant8loop UNTIL CE;  {ponder the powerful pixel pokeing}
                   DM(dax)=AY0;         {move beam to new x along line segment}
                   DM(day)=AY1;         {move beam to new y along line segment}
                   beam_me_up_scotty;
                   AR=AY0+1;            {increment x}
                   AY0=AR;
                   AR=PASS AF;          {check sign of error term, d}
                   PUSH STS;            {save status for 'else' test}
                   IF LT AF=SR0+AF;     {'if then' clause: d = d+incr1}
                   POP STS;
                   IF LT JUMP octant8loop;
                   AR=AY1-1;            {'else' clause...}
                   AY1=AR;              {dec y}
                   AF=SR1+AF;           {d = d+incr2}
octant8loop:         NOP;
                JUMP update;

update:         AY0=DM(newx);           {update old with last new point...}
                DM(oldx)=AY0;
                AY1=DM(newy);
                DM(oldy)=AY1;
                DIS SEC_REG;
                JUMP repeat;

done:           IMASK=h#0004;           {re-enable irq2}
                RTS;

.ENDMOD;

