{   CPFSK - Continuous Phase Frequency Shift Keying modulator

    ADSP-2101 EZ-LAB demonstration

    Analog Devices, Inc.
    DSP Division
    P.O.Box 9106
    Norwood, MA 02062
    
    28-Sept-1989 (orig. 2100), rewritten for ADSP-2101 2-May-1990
    modified 15-June-1990, 21-Nov-1990
    Christoph Cavigioli (DSP Applications)

    input:  data stream stored in DM circ buffer (for demo)
            in real life, data would probably be state of FLAG_IN pin
    output: dac0 - CPFSK output waveform
            dac1 - input data stream (echoed for demo display)
            spkr - CPFSK "sound"
}
.module/boot=0/abs=0   CPFSK_modulator;
.external boot_sin;
.external init2101;
.port       write_dac0;
.port       write_dac1;
.port       load_dac;

.const LO_TONE      =   220;            { Hertz }
.const HI_TONE      =   880;            { Hertz }
.const LOGIC_ONE    =   h#7f00;
.const LOGIC_ZERO   =   0;

.const  TSCALE=                 0x3ffb;
.const  TCOUNT=                 0x3ffc;
.const  TPERIOD=                0x3ffd;

.var/circ demo_input_data[7];
.var hertz0, phase_incr_0;
.var hertz1, phase_incr_1;
.var phase_accumulator, phase_increment;

{------------------------------------------------------------------------------}
        jump start; rti; rti; rti;      {Reset Vector}    
        rti; rti; rti; rti;             {irq2}
        rti; rti; rti; rti;             {sport0 TX}
        jump sample; rti; rti; rti;     {sport0 RX} { at 8.000 kHz rate }
        rti; rti; rti; rti;             {irq0}
        rti; rti; rti; rti;             {irq1}
        call symbol; rti; rti; rti;     {timer}     { at 2400 baud rate }

{--------------------}
start:  call init2101;             { set up SPORTS, TIMER, etc }
        m7=1; l7=0;                { used by bootsin routine }

{--------------------}
baud_clock:     { 2400 baud = 5120 cycles @ 12.288 MHz }
    ax0=0;      dm(TSCALE)=ax0;     {TIMER ... TSCALE}
    ax0=5119;   dm(TPERIOD)=ax0;    {TIMER ... TPERIOD} 
    ax0=5119;   dm(TCOUNT)=ax0;     {TIMER ... TCOUNT}

{--------------------}
make_demo_data:
        si=LO_TONE;     dm(hertz0)=si;
        si=HI_TONE;     dm(hertz1)=si;
        si=LOGIC_ONE;   dm(demo_input_data)=si;
        si=LOGIC_ZERO;  dm(demo_input_data+1)=si;
                        dm(demo_input_data+2)=si;
                        dm(demo_input_data+3)=si;
                        dm(demo_input_data+4)=si;
                        dm(demo_input_data+5)=si;
                        dm(demo_input_data+6)=si;
        i0=^demo_input_data;
        l0=%demo_input_data;

{--------------------}  
{ These segments convert "Hertz" to 8 kHz Phase_Increment }

load_tone1:  
        si=dm(hertz1);
        sr=ashift si by 3 (hi);
        my0=h#4189;            { mult Hz by .512 * 2 }
        mr=sr1*my0(rnd);        { i.e. mult by 1.024 }
        sr=ashift mr1 by 1 (hi);
        dm(phase_incr_1)=sr1;

load_tone0:  
        si=dm(hertz0);
        sr=ashift si by 3 (hi);
        my0=h#4189;            { mult Hz by .512 * 2 }
        mr=sr1*my0(rnd);        { i.e. mult by 1.024 }
        sr=ashift mr1 by 1 (hi);
        dm(phase_incr_0)=sr1;

{--------------------}
        si=0;
        dm(phase_accumulator)=si;   { clear phase accumulator upon start-up }
        call symbol;                { start with first symbol }
        icntl=b#01111;
        imask=b#001001;             { enable SPORT0_RX, TIMER now }
        ena timer;                  { start baud_clock now }

{--------------------}
here:   jump here;                  { wait for symbol and sample interrupts }





{=============================================================}
{========== P R O C E S S   A   N E W   S A M P L E ==========}
{=============================================================}
sample: ax0=dm(phase_accumulator);
        ay0=dm(phase_increment);
        ar=ax0+ay0;
        dm(phase_accumulator)=ar;
        ax0=ar;
        call boot_sin;
sound:  dm(write_dac0)=ar;      { "display" CPFSK on oscilloscope }
        dm(load_dac)=ar;
        sr=ashift ar by -2 (hi);
        tx0=sr1;                 { "hear" CPFSK from speaker (PCM out) }
        rti;


{=============================================================}
{========== P R O C E S S   A   N E W   S Y M B O L ==========}
{=============================================================}
symbol: ax1=dm(i0,m0);          { get input data (or could be FLAG_IN pin) }
        dm(write_dac1)=ax1;     { echo back input data stream for demo }
        dm(load_dac)=ar;
        af=pass ax1;
        if eq jump zero;
one:    si=dm(phase_incr_1);    dm(phase_increment)=si;     rts;
zero:   si=dm(phase_incr_0);    dm(phase_increment)=si;     rts;

.endmod;
