{   Switch Between Four Different Bandpass FIR Filters

    ADSP-2101 EZ-LAB demonstration

    Analog Devices, Inc.
    DSP Division
    P.O.Box 9106
    Norwood, MA 02062
    
    23-Feb-1990, modified 26-Feb-1990, 21-Nov-1990
    Christoph Cavigioli (DSP Applications)

    input signal from microphone
    output to speaker for audio, dac for observation

    Move through state machine by pushing IRQ2 button:
    -------------------------------------------------
    state_0:    voice input, pass through unfiltered
    state_1:    voice input, bandpass filter 1 (low freq)
    state_2:    voice input, bandpass filter 2 (higher freq)
    state_3:    voice input, bandpass filter 3 (even higher freq)
    state_4:    voice input, bandpass filter 4 (highest freq)

    state_5:    noise input, pass through unfiltered
    state_6:    noise input, bandpass filter 1 (low freq)
    state_7:    noise input, bandpass filter 2 (higher freq)
    state_8:    noise input, bandpass filter 3 (even higher freq)
    state_9:    noise input, bandpass filter 4 (highest freq)
    
    <next push of IRQ2 brings you back to state_0>
}
.module/boot=0/abs=0        Mic_Many_FIRs;
.external   init2101;
.external   newrnd;
.port       write_dac0;
.port       load_dac;
.const      taps=256;

.var/circ       data[taps];
.var            which_fir;
.var            voice_or_noise;
.var            saveI4;
.var            seed_msw, seed_lsw;
.global         seed_msw, seed_lsw;
.var            rndnum;

.var/pm/circ    fir1_coefs[taps];
.var/pm/circ    fir2_coefs[taps];
.var/pm/circ    fir3_coefs[taps];
.var/pm/circ    fir4_coefs[taps];
.init           fir1_coefs: <fir1.dat>;
.init           fir2_coefs: <fir2.dat>;
.init           fir3_coefs: <fir3.dat>;
.init           fir4_coefs: <fir4.dat>;

{------------------------------------------------------------------------------}
chip_reset:     jump start; rti; rti; rti;      {Reset Vector}    
ext_irq2:       jump newfir; rti; rti; rti;     {external irq2}
s0_tx:          rti; rti; rti; rti;             {sport0 TX}
s0_rx:          jump sample; rti; rti; rti;     {sport0 RX}
s1_tx_irq0:     rti; rti; rti; rti;             {sport1 TX or irq0}
s1_rx_irq1:     rti; rti; rti; rti;             {sport1 RX or irq1}
timer_done:     rti; rti; rti; rti;             {timer expired}

start:  call init2101;                          { set up SPORTS, TIMER, etc }
        i0=^data;       m0=1;   l0=taps;
        i4=^fir1_coefs; m4=1;   l4=taps;
        i5=^fir2_coefs;         l5=taps;
        i6=^fir3_coefs;         l6=taps;
        i7=^fir4_coefs;         l7=taps;
        si=0;
        dm(which_fir)=si;       { start with no filtering of input signal }
        dm(voice_or_noise)=si;  { start with voice input, not noise input }
        si=h#1234;
        dm(seed_lsw)=si;        { arbitrary seed for random noise generator }
        dm(seed_msw)=si;
        i0=^data;
        cntr=taps;
        {do zero until ce;}
zero:       dm(i0,m0)=0;        { clear out the filter delay line buffer }
            if not ce jump zero;
        icntl=b#00111;          {disable IRQ nesting, all IRQs edge-sensitive}
        imask=b#101000;         { enable IRQ2 and SPORT0_RX interrupt }
here:   jump here;

sample: ar=dm(voice_or_noise);
        ar=pass ar;
        if eq jump voice_input;

noise_input:
        call newrnd;            { generate a 16-bit random number in sr1 }
        jump process_sample;

voice_input:
        si=rx0;                 { get new sample from SPORT0 (from microphone) }
        sr=ashift si by 2 (hi); { shift 14 LSBs into MSBs }

process_sample:
        dm(i0,m0)=sr1;          { store sample in data buffer (delay line) }
        ax1=dm(which_fir);      { decide which filter to do }
        ay1=^jump_table;
        ar=ax1+ay1;
        dm(saveI4)=i4;          { restore I4 later in filter routines }
        i4=ar;
        jump (i4);

jump_table:
        jump nofilt;            { which_fir = 0 }
        jump fir1;              { which_fir = 1 }
        jump fir2;              { which_fir = 2 }
        jump fir3;              { which_fir = 3 }
        jump fir4;              { which_fir = 4 }

output: tx0=mr1;                { filtered output to SPORT (to spkr) }
        dm(write_dac0)=mr1;     { latch sample for D/A }
        dm(load_dac)=mr1;       { display sample on oscilloscope w/ D/A }
        rti;

nofilt: i4=dm(saveI4);
        sr=ashift sr1 by -1 (hi);   { save the audience's ears from damage }
        mr1=sr1;
        jump output;

fir1:   i4=dm(saveI4);
        cntr=taps-1;
        mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
        {do fir1loop until ce;}
fir1loop:   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
            if not ce jump fir1loop;
        mr=mr+mx0*my0(rnd);
        if mv sat mr;
        jump output;

fir2:   i4=dm(saveI4);
        cntr=taps-1;
        mr=0, mx0=dm(i0,m0), my0=pm(i5,m4);
        {do fir2loop until ce;}
fir2loop:   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i5,m4);
            if not ce jump fir2loop;
        mr=mr+mx0*my0(rnd);
        if mv sat mr;
        jump output;

fir3:   i4=dm(saveI4);
        cntr=taps-1;
        mr=0, mx0=dm(i0,m0), my0=pm(i6,m4);
        {do fir3loop until ce;}
fir3loop:   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i6,m4);
            if not ce jump fir3loop;
        mr=mr+mx0*my0(rnd);
        if mv sat mr;
        jump output;

fir4:   i4=dm(saveI4);
        cntr=taps-1;
        mr=0, mx0=dm(i0,m0), my0=pm(i7,m4);
        {do fir4loop until ce;}
fir4loop:   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i7,m4);
            if not ce jump fir4loop;
        mr=mr+mx0*my0(rnd);
        if mv sat mr;
        jump output;

newfir: ay0=dm(which_fir);  { push into next state of state machine }
        ar=ay0+1;
        dm(which_fir)=ar;
        ay0=5;              { if in state4, go to state0, not state5 }
        ar=ar-ay0;
        if ne rti;
        ar=pass 0;
        dm(which_fir)=ar;
        ar=dm(voice_or_noise);
        ar=not ar;          { at the same time, also toggle voice/noise input }
        dm(voice_or_noise)=ar;
        rti;

.endmod;
