/* system includes*/
#include	<ADSP2101.h>
#include	<math.h>
#include	<signal.h>
#include	<misc.h>

/*user includes*/
#include	"types.h"

/*user function prototypes*/
extern void	flagin(void);		/* used to force a s/w boot */
extern void	interrupt2_routine();	/* used to randomize parameters */
extern void	timer_interrupt();


/*make some global variables so the interrupt handlers can get at them*/
int		inc1, inc2, inc3, timer_count, modifier;
volatile int	rad1, rad2, rad3;
unsigned int	seed;

main()
{
    float	arg, pi_over_180 = 3.14159/180.0;
    point	point1, point2, point3;
    int		ang1 = 0, ang2 = 0, ang3 = 0;
    int		intensity = 1;

/* setup the timer... */
/* using a tscale value of 255, the time between decrements is 20.40us  */
/* therefore, divide the # of seconds you want between interrupts by... */
    unsigned int	timeout_val = 0.07 / 0.00002040;
    timer_set(timeout_val, timeout_val, 255);
    timer_on();

/* initialize some stuff... */
    modifier = 1; seed = 26;

/* configure SPORT1 for flag I/O */
#pragma inline
	dm(0x3fff) = m0;
#pragma inline

/* enable interrupts */
    interrupt(SIGINT2, interrupt2_routine);
    interrupt(SIGTIMER, timer_interrupt);

/* force 1st user interrupt to initialize stuff */
    raise(SIGINT2);

    srand(seed);			/* seed the random number generator */

    for (;;)				/* infinate loop */
    {
	flagin();			/* poll for a flagin button depressed */

	/* increment rotation of each radius and wrap around 360 degrees */
	ang1 = ang1 + inc1; if (ang1 >= 360) ang1 = ang1 - 360;
	ang2 = ang2 + inc2; if (ang2 >= 360) ang2 = ang2 - 360;
	ang3 = ang3 + inc3; if (ang3 >= 360) ang3 = ang3 - 360;

	/* calculate first endpoint */
	arg = pi_over_180*(float)ang1;
	point1.x = 127 + (int)rad1 * cos(arg);
	point1.y = 127 + (int)rad1 * sin(arg);

	/* calculate second endpoint */
	arg = pi_over_180*(float)ang2;
	point2.x = point1.x + (int)rad2 * cos(arg);
	point2.y = point1.y + (int)rad2 * sin(arg);

	/* calculate third endpoint */
	arg = pi_over_180*(float)ang3;
	point3.x = point2.x + (int)rad3 * cos(arg);
	point3.y = point2.y + (int)rad3 * sin(arg);

	/* plot the endpoint of the third radius */
        plot(point3.x, point3.y, intensity);
    }
}
