{Program to interface the AD1848 to ADSP-2105


Polling is used to detect when the A/Ds and D/As are ready. This program reads
data from the A/Ds and sends it to the D/As.It should be noted that the A/Ds
and D/As do not become ready at the same time. A typical program flow for real
time signal processing would be as follows:


When the the D/As become ready the program jumps to a routine that
a) sends data that has already been processed to the D/As
b) processes the last data received from the A/Ds
c) returns to polling


When the A/Ds become ready the progam jumps to a routine that reads the new 
data into a buffer (to be processed when the D/As become ready)and then 
returns to polling.}
.MODULE/ABS=0/BOOT=0  AD1848;
.VAR/CIRC SERINDATA[4];
.VAR          FIRST;
.VAR          MODE;
.VAR          REG[19];
.VAR          DATA[4];


{MEMORY MAP: THE FOLLOWING DESCRIBES WHERE IN THE DSP'S MEMORY SPACE
THE AD1848 RESIDES}
.CONST        IAR=B#00000000;   {INDEX ADDRESS REGISTER ADDRESS}
.CONST        IDR=B#00000001;   {      DATA                    }
.CONST        STR=B#00000010;   {      STATUS  REGISTER        }
.CONST        PIO=B#00000011;   {      PIO     REGISTER        }                   

                   JUMP START;nop;nop;nop;      {restart interrupt}
                   RTI;nop;nop;nop;             {IRQ2 int}
                   RTI;nop;nop;nop; 
                   RTI;nop;nop;nop;
                   RTI;nop;nop;nop;              {SPORT0 transmit int}
                   RTI;nop;nop;nop;              {SPORT0 receive int}
                   RTI;nop;nop;nop;              {timer int}
                   
START:                 

                      CALL INITDSP;            {INITIALIZE THE DSP}
                      CALL INIT1848;           {INITIALIZE THE AD1848}
                      
WAIT_DATA:                 

                    AX1=DM(STR);             {WAIT FOR D/A TO BE READY}
                    AY1=B#0000010;
                    AF=AX1 AND AY1;
                    IF NE CALL DARDY;
                    NOP;NOP;                 
                    

                    AY0=B#00100000;         {HAVE TO WAIT FOR A/D TO BE READY}
                    AX0=DM(STR);NOP;
                    AF=AX0 AND AY0;
                    IF NE CALL ADDATA;
                    NOP;NOP;
                    JUMP WAIT_DATA;
                    
DARDY:           {DATA THAT WAS PREVIOUSLY READ FROM A/Ds WILL BE
                  WRITTEN TO D/As}
                  I1=^DATA;                 
                  

                  AX0=DM(I1,M1);
                  DM(PIO)=AX0;              {WRITE LOWER BYTE TO AD1848}
                  NOP;NOP;                  {DELAY TO MEET AD1848 TIMING}
                  AX0=DM(I1,M1);
                  DM(PIO)=AX0;              {SEND UPPER BYTE TO AD1848}
                  NOP;NOP;                 
                  

                  AX0=DM(I1,M1);
                  DM(PIO)=AX0;              {WRITE LOWER BYTE TO AD1848}
                  NOP;NOP;                  {DELAY TO MEET AD1848 TIMING} 
                  AX0=DM(I1,M1);
                  DM(PIO)=AX0;              {SEND UPPER BYTE TO AD1848}
                  NOP;NOP;
                  RTS;


ADDATA:          {A/D DATA IS TO BE READ INTO A BUFFER}                 
                  I1=^DATA;                 
                  AX0=DM(PIO);                 
                  DM(I1,M1)=AX0;                 
                  NOP;NOP;                 
                  AX0=DM(PIO);                 
                  DM(I1,M1)=AX0;                 
                  NOP;NOP;                 
                  AX0=DM(PIO);                 
                  DM(I1,M1)=AX0;                 
                  NOP;NOP;                 
                  AX0=DM(PIO);                 
                  DM(I1,M1)=AX0;                 
                  NOP;NOP;                 
                  RTS;

{___________________________________________________________}

INITDSP:                   
                  L1 = 0;             {linear addressing for register }
                  I1 = 0x3fef;             {pt. to last DM cntrl reg}
                  M1=1;
{the numbers on the left hand side is the address of the control 
register being initialised}
{3FEF}             DM(I1,M1)= 0;
{3FF0}             DM(I1,M1) = 0;
{3FF1}             DM(I1,M1) = 0;
{3FF2}             DM(I1,M1) = 0;
{3FF3}             DM(I1,M1) = 0;
{3FF4}             DM(I1,M1) = 0;
{3FF5}             DM(I1,M1) = 0;
{3FF6}             DM(I1,M1) = 0;
{3FF7}             DM(I1,M1) = 0;
{3FF8}             DM(I1,M1) = 0;
{3FF9}             DM(I1,M1) = 0;
{3FFA}             DM(I1,M1) = 0;
{3FFB}             DM(I1,M1) = 0;           {TSCALE register}
{3FFC}             DM(I1,M1) = 0xFFFF;      {TCOUNT register}
{3FFD}             DM(I1,M1) = 0xFFFF;      {TPERIOD register}
{3FFE}             DM(I1,M1) = B#0010010010010010; {external data memory 
                                                    waits}
{3FFF}             DM(I1,M1) = B#0000010000011000;                   
                   RTS;

{___________________________________________________________}

INIT1848:                 
                   {WAIT FOR THE AD1848 TO "WAKE UP"}                 
                   
                   AY0=0X0080;               {AY0 GETS A MASK}
INITIAL:           AX0=DM(IAR);NOP;          {READ INIT BIT OF INDEX 
                                              ADDRESS REGISTER}
                   AR=AX0 AND AY0;           {CHECK WHETHER IT IS 0}
                   IF NE JUMP INITIAL;       {IF NOT THEN PART IS NOT 
                                              READY}                 
                   

                          {INITIALIZE AD1848 REGISTERS}                 
                          
                   AX0=B#01001001;                 
                   DM(IAR)=AX0;NOP;                 
                   AX0=B#11001011;       {ACAL=1}                 
                   DM(IDR)=AX0;                 
                   
                   
                   AX0=B#01001000;                 
                   DM(IAR)=AX0;NOP;                 
                   AX0=B#01011011;       {FMT=1,SM=1,CFS=5,L/C=0,
                                          Fs=44.1kHZ}                 
                   DM(IDR)=AX0;


{SINCE THIS INVOLVED CHANGING SAMPLE RATES WE NEED TO POLL THE INIT BIT
AND THEN THE ACI BIT TO WAIT FOR AUTOCALIBRATION TO FINISH}                

                  CALL GETREADY;                 
                      
                   AX0=B#00000110;       {NEXT WE UNMUTE THE LEFT DAC 
                                          OUTPUT}                 
                   DM(IAR)=AX0;NOP;                 
                   AX0=B#00000000;       {LDM=0}                 
                   DM(IDR)=AX0;                 
                   
                   AX0=B#00000111;       {THEN WE UNMUTE THE RIGHT DAC 
                                          OUTPUT}                 
                   DM(IAR)=AX0;NOP;                 
                   AX0=B#00000000;       {RDM=0}                 
                   DM(IDR)=AX0;                
                   
                  RTS;                   {AD1848 INITIALIZATION FINISHED}



GETREADY:          {WHEN ENTERING THIS ROUTINE THE MCE BIT IS HIGH. THIS
                    ROUTINE WAITS FOR THE PART TO BE ABLE TO RESPOND AND
                    THEN BRINGS THE MCE BIT TO 0 AND WAITS FOR THE AD1848
                    TO BECOME READY}                 
                    

                   AY0=0X0080;               {AY0 GETS A MASK}
RDY:               AX0=DM(IAR);NOP;          {READ INIT BIT OF INDEX
                                              ADDRESS REGISTER}
                   AR=AX0 AND AY0;           {CHECK WHETHER IT IS 0}
                   IF NE JUMP RDY;           {IF NOT THEN PART IS NOT 
                                              READY}                 
                   AX0=B#00101011;           {PART IS NOW READY FOR 
                                              COMMANDS}                 
                   AY0=B#00100000;           {AY0 GETS A MASK}                 
                   DM(IAR)=AX0;NOP;          {PUT MCE TO 0 WHICH STARTS 
                                              AUTOCAL}


 {NEXT WE WAIT FOR AUTOCALIBRATION TO START}

CALST:            AX0=DM(IDR);              {READ ACI BIT}                 
                  AR=AX0 AND AY0;           {CHECK WHETER IT IS 1:START 
                                             OF AUTOCAL}                 
                  IF EQ JUMP CALST;         {IF NOT CHECK AGAIN}
                  
                  {THEN WE WAIT FOR AUTOCALIBRATION TO FINISH}

CALFIN:          AX0=DM(IDR);              {READ ACI BIT}                 
                 AR=AX0 AND AY0;           {CHECK WHETER IT IS 0:END 
                                            OF AUTOCAL}                 
                 IF NE JUMP CALFIN;        {IF NOT THEN PART IS NOT READY}                 
                 

                 RTS;                      {AUTOCAL IS FINISHED, AD1848
                                            IS READY}


.ENDMOD;


