/*****************************************************************************
 ADSP-2101 -> AD1847: Talkthru Interface                      
                                                              
 This code provides the necessary initializations and setup to allow for 
 communication between the AD1847 SoundPort codec and the ADSP-2100 family 
 Serial Port 0.                
                                                              
 This interface code was written around an AD1847 connected to an ADSP-2101 
 EZ-Lab board through the J2 SPORT Connector.
 
When the board is RESET, the codec is initialized for an    
8kHz sampling rate of stereo PCM data.                    
                                                              

 This module can be used without modification for the ADSP-2101, ADSP-2103, 
 and ADSP-2115. Use with the ADSP-2111, ADSP-2171, or ADSP-21msp5x processors
 would require modification of the interrupt vector table and any instructions
 relating to IFC, IMASK, or any other interrupt-related structure.
                                                              
 This interface is not recommended for the ADSP-2105.       

**********
 
 8/3/94
 Modified to initialize data memory out of program memory by
 Charles Steinkuehler.

*****************************************************************************/

.MODULE/ABS=0/BOOT=0  AD1847;

.var/dm/ram         	stat_flag;
.var/dm/ram             cnt;
.var/dm/ram/circ        rx_buf[3];      /* Status + L data + R data */
.var/dm/ram/circ        tx_buf[3];      /* Cmd + L data + R data    */
.var/dm/ram/circ        init_cmds[13];

.PORT		wrdcnt;    { R/W }
.PORT		blkcnt;    { R/W }
.PORT		endblk;	   { R/W }
.PORT		dmactrl;   { R/W }
{
.PORT		spare4;    
.PORT		spare5;	   
.PORT		spare6;	   
.PORT		spare7;    
}     	
.PORT		Xreg0;     { R=Cpu0,W=Dma0 }
.PORT		Xreg1;     { R=Cpu1,W=Dma1 }
.PORT		Xreg2;     { R=Cpu2,W=Dma2 }
.PORT		page_X12;  { R/W }
{
.PORT		spareC;    
.PORT		spareD;	   
.PORT		spareE;	   
.PORT		spareF;    
}
       			
       		
/*********************************************************/
/*      Initial codec setup:                             */
/* 0:  Left Input Control: 0 gain, Line 1 input          */
/* 1:  Right Input Control: 0 gain, Line 1 input         */
/* 2:  Left Aux #1 Input Control: Muted                  */
/* 3:  Right Aux #1 Input Control: Muted                 */
/* 4:  Left Aux #2 Input Control: Muted                  */
/* 5:  Right Aux #2 Input Control: Muted                 */
/* 6:  Left DAC Control: 0 attenuation, muted            */
/* 7:  Right DAC Control: 0 attenuation, muted           */
/* 8:  Data Format: XTAL2, 44.1kHz sampling, stereo,     */
/*                 16-bit linear PCM                     */
/* 9:  Interface Config: Playback enabled, ACAL allowed  */
/* 10: Pin Control: CLKOUT active, XCTL1/0 LO            */
/* 12: Misc. Info: Transmit on 0,1,2, 32-word frame      */
/* 13: Digital Mix Control: DME Disabled, 0 attenuation  */
/*                                                       */
/* To start-up the codec with any other properties,      */
/* change the appropriate initialization value.          */
/* Refer to the AD1847 data sheet for detailed register  */
/* descriptions.                                         */
/*********************************************************/


reset_vect:     jump start; nop; nop; nop;

irq2_svc:       rti; nop; nop; nop;

tx0_irq:        ar = dm(stat_flag);
		ar = pass ar;
		if eq rti; 
		jump next_cmd;

rx0_irq:        jump input_samples;
		rti; nop; nop;

		rti; nop; nop; nop;
		rti; nop; nop; nop;
		rti; nop; nop; nop;

/***************
   Variable Init:
****************/
start:
		{ Initialize TX buffer }
		i1 = ^tx_buf;
		l1 = %tx_buf;
		m1 = 1;

		ax0=0xc000;		{ Initially set MCE }
		dm(i1,m1)=ax0;
		ax0=0x0000;
		dm(i1,m1)=ax0;
		ax0=0x0000;
		dm(i1,m1)=ax0;

		{ Initialize Command buffer }
		i3 = ^init_cmds;
		l3 = %init_cmds;
		m1 = 1;

		ax0=0xc000;  
		dm(i3,m1)=ax0;
 		ax0=0xc100;  
		dm(i3,m1)=ax0;
		ax0=0xc280;  
		dm(i3,m1)=ax0;
		ax0=0xc380; 
		dm(i3,m1)=ax0;
		ax0=0xc480;  
		dm(i3,m1)=ax0;
		ax0=0xc580;  
		dm(i3,m1)=ax0;
		ax0=0xc680;  
		dm(i3,m1)=ax0;
		ax0=0xc780; 
		dm(i3,m1)=ax0;
		ax0=0xc85B;  
		dm(i3,m1)=ax0;
		ax0=0xc909;  
		dm(i3,m1)=ax0;
		ax0=0xca00;  
		dm(i3,m1)=ax0;
		ax0=0xcc40; 
		dm(i3,m1)=ax0;
		ax0=0xcd00; 
		dm(i3,m1)=ax0;

/***************
   Previous Code Start:
****************/
sys_init:
		i0 = ^rx_buf;
		l0 = %rx_buf;
		i1 = ^tx_buf;
		l1 = %tx_buf;
		i3 = ^init_cmds;
		l3 = %init_cmds;

		m1 = 1;

/**********************
	sport0 setup: multichannel enable, ext. sclk, MFD=1,
			32 words, ext. rfs, DTYPE 00, 16 bits
**********************/ 
		{ bit # 1111110000000000 }
		{     # 5432109876543210 }
		{------------------------}
		ax0 = b#1000011000001111;
		dm(0x3ff6) = ax0;

/*********************
	Multichannel enable setup:
		Rx: 3, 4, 5, 19, 20, 21
		Tx: 0, 1, 2, 16, 17, 18
*********************/
		{ bit # 1111110000000000 }
		{     # 5432109876543210 }
		{------------------------}
		ax0 = b#0000000000000111;
		dm(0x3ff9) = ax0;
		dm(0x3ffa) = ax0;

		{ bit # 1111110000000000 }
		{     # 5432109876543210 }
		{------------------------}
		ax0 = b#0000000000000111;
		dm(0x3ff7) = ax0;
		dm(0x3ff8) = ax0;

/*********************
	SPORT Autobuffer Setup
		Rx: I0, M1
		Tx: I1, M1
**********************/
		{ bit # 1111110000000000 }
		{     # 5432109876543210 }
		{------------------------}
		ax0 = b#0000001010000111;
		dm(0x3ff3) = ax0;

		{ bit # 1111110000000000 }
		{     # 5432109876543210 }
		{------------------------}
		ax0 = b#0001000000000000;
		dm(0x3fff) = ax0;               /* SPORT0 Enabled,
						   SPORT1 = FI, FO, IRQs */
start_setup:    ifc = b#000000111111;
		nop;
		
                ax0 = 1;
                dm(stat_flag) = ax0;

		imask = b#010000;               /* enable tx0 interrupt */

		{ why sending word to sport? }
		ax0 = dm(i1,m1);
		tx0 = ax0;
		
check_init:     ax0 = dm(stat_flag);            /* wait for entire init */
		af = pass ax0;                  /* buffer to be sent to */
		if ne jump check_init;          /* the codec            */

check_aci:      ax0 = dm(rx_buf);               /* once initialized, wait */
		ay0 = b#0000000000000010;       /* for codec to come out  */
		ar = ax0 and ay0;               /* of autocalibration     */
		if ne jump check_aci;

		idle;

		ay0 = 0xbf3f;
		ax0 = dm(init_cmds+6);
		ar = ax0 AND ay0;
		dm(tx_buf) = ar;               /* unmute left DAC */
		idle;

		ax0 = dm(init_cmds+7);
		ar = ax0 AND ay0;
		dm(tx_buf) = ar;               /* unmute right DAC */
		idle;

		ifc = b#000000111111;
		nop;
		imask = b#001000;       /* enable rx0 interrupt */

/************************************************************************/
/* Main Loop: talkthru                                                  */
/************************************************************************/

talkthru:       
		jump talkthru;
		

/************************************************************************/
input_samples:
		ena sec_reg;
		ax1 = dm(rx_buf+1);     /* L channel input */
		mx1 = dm(rx_buf+2);     /* R channel input */
		dm(tx_buf+2) = mx1;
		dm(tx_buf+1) = ax1;
		dis sec_reg;
		rti;

next_cmd:
		ena sec_reg;
		ax0 = dm(i3,m1);	{ Read next command }
		dm(tx_buf) = ax0;	{ Place in Tx buffer }
		ax0 = i3;		{ Read current pos in cmd buffer }
		ay0 = ^init_cmds;	{ Start location of cmd buffer }
		ar = ax0 - ay0;
		if eq jump done;	{ Return if not wrapped around }
		dis sec_reg;
		rti;
done:
		ax0 = 0x8000;		{ Otherwise done with init }
		dm(tx_buf) = ax0;
		ax0 = 0;               /* remove MCE if done initialization */
		dm(stat_flag) = ax0;   /* reset status flag */   
		dis sec_reg;
		rti;

.endmod;
