/*****************************************************************************
 ADSP-2101 -> AD1847: Talkthru Interface                      
                                                              
 This code provides the necessary initializations and setup to allow for 
 communication between the AD1847 SoundPort codec and the ADSP-2100 family 
 Serial Port 0.                
                                                              
 This interface code was written around an AD1847 connected to an ADSP-2101 
 EZ-Lab board through the J2 SPORT Connector.
 
When the board is RESET, the codec is initialized for an    
8kHz sampling rate of stereo PCM data.                    
                                                              

 This module can be used without modification for the ADSP-2101, ADSP-2103, 
 and ADSP-2115. Use with the ADSP-2111, ADSP-2171, or ADSP-21msp5x processors
 would require modification of the interrupt vector table and any instructions
 relating to IFC, IMASK, or any other interrupt-related structure.                                 
                                                              
 This interface is not recommended for the ADSP-2105.       

**********
 
 8/3/94
 Modified to initialize data memory out of program memory by
 Charles Steinkuehler.

*****************************************************************************/

.MODULE/ABS=0/BOOT=0  AD1847;

.CONST READMASK	= B#00000001;
.CONST WRITEMASK= B#00000010;
.CONST REGBASE	= 0x37F0;

.var/dm/ram		dmacount;

.PORT		wrdcnt;    { R/W }
.PORT		blkcnt;    { R/W }
.PORT		endblk;	   { R/W }
.PORT		dmactrl;   { R/W }
{
.PORT		spare4;
.PORT		spare5;
.PORT		spare6;
.PORT		spare7;
}
     	
.PORT		Xreg0;     { R=Cpu0,W=Dma0 }
.PORT		Xreg1;     { R=Cpu1,W=Dma1 }
.PORT		Xreg2;     { R=Cpu2,W=Dma2 }
.PORT		page_X12;  { R/W }
{
.PORT		spareC;
.PORT		spareD;
.PORT		spareE;
.PORT		spareF;
}
       			
.var/dm         stat_flag;

reset_vect:     jump start; nop; nop; nop;

irq2_svc:       rti; nop; nop; nop;

tx0_irq:        rti; nop; nop; nop;

rx0_irq:        rti; nop; nop; nop;

		jump dma_irq;			/* IRQ1 - DMA done */
		nop; nop; nop;

		jump cmd_irq;			/* IRQ0 - New Cmd */
		nop; nop; nop;
		rti; nop; nop; nop;

/***************
   Variable Init:
****************/
start:		
		ax0=0x2000;		{ Set 2 wait states for orca }
		dm(0x3ffe)=ax0;		{ Set 0 wait states for mem  }
		i0 = 0;
		m0 = 0;
		l0 = 0;

		ifc = b#000000111111;		/* Clear all interrupts */
		nop;
		imask = b#000110;	{ Enable Dma IRQ }
		icntl = b#00111;	/* No nesting, edge sensitive */

		ax0 = 0x77;
		dm(Xreg1) = ax0;
		dm(dmacount) = ax0;

		ax0 = dm(dmacount);
		ay0 = 1;
		ar = ax0 + ay0;
		dm(dmacount) = ar;
		dm(Xreg1) = ar;

CmdLoop:
		ax0=dm(Xreg0);
		ar=pass ax0;
		if ne jump CmdLoop;
		dm(Xreg0)=ax0;		{ Mirror cmd in xfer reg }
WaitCmd:
		ax0=dm(Xreg0);
		ar=pass ax0;
		if eq jump WaitCmd;
		dm(Xreg0)=ax0;		{ Mirror cmd in xfer reg }
Read:
		ay0=READMASK;		{ Test for read command }
		ar=ax0 - ay0;
		if ne jump Write;
		ax0=dm(Xreg1);		{ Read address }
{		dm(Xreg1)=ax0; }		{ Mirror adr in xfer reg }
		ay0=REGBASE;		{ Calculate physical address }
		ar=ax0 + ay0;
		i0=ar;
		ax0=dm(i0,m0);		{ Read from physical address }
		dm(Xreg2)=ax0;		{ Place result in xfer reg }
		jump CmdLoop;
Write:
		ay0=WRITEMASK;
		ar=ax0 - ay0;
		if ne jump CmdLoop;
		ax0=dm(Xreg1);		{ Read address }
{		dm(Xreg1)=ax0; }		{ Mirror adr in xfer reg }
		ay0=REGBASE;		{ Calculate physical address }
		ar=ax0 + ay0;
		i0=ar;
		ax0=dm(Xreg2);		{ Read value to write }
		dm(i0,m0)=ax0;		{ Write to physical address }
		dm(Xreg2)=ax0;		{ Mirror data in xfer reg }
		jump CmdLoop;


cmd_irq:
dma_irq:	ENA sec_reg;

		ax0 = dm(dmacount);
		ay0 = 1;
		ar = ax0 + ay0;
		dm(dmacount) = ar;
		dm(Xreg1) = ar;

		DIS sec_reg;
		rti;
		
.endmod;
