{*****************************************************************}
{* Flyer Audio DSP Code (For rev 4)                              *}
{*                                                               *}
{* Written for ADSP-2115 and AD1847.                             *}
{*                                                               *}
{* Performs Audio playback and record at 16-bit linear, 44.1 KHz *}
{*                                                               *}
{*                                                               *}
{* 08-26-94       Marty Flickinger                               *}
{*                                                               *}
{*****************************************************************}

.MODULE/ABS=0/BOOT=0  AD1847;

.INCLUDE <..\include\DEF2101.H>;

{**************** Constants ****************}

.CONST NUMCHANS	= 8;		{ 8 mono channels }
.CONST SAMPLES	= 735;		{ Samples per field }
.CONST BANKBIT	= 1;		{ Bit value for RAM bank select bit }

.CONST CHANBLKSIZE	= 3;
.CONST CHANBUFSIZE	= CHANBLKSIZE * 256;

.CONST HALFNUMCHANS = NUMCHANS/2;

{**************** Variables ****************}

{ Codec initialization command list array }
.VAR/PM/RAM/CIRC	InitCmdList[14];

{ Left/Right coefficients for each channel (left group, right group) }
.VAR/PM/RAM/CIRC	LRcoefs[NUMCHANS * 2];

{ Channel data buffers -- on each RAM page }
.VAR/DM/RAM/CIRC/ABS=0x0000	ChansBuf[NUMCHANS * CHANBUFSIZE];

{ Mixed-down stereo data buffer -- on each RAM page }
.VAR/DM/RAM/ABS=0x1800		MixBuf[2 * CHANBUFSIZE];

{ CPU control buffer -- page 0 only??? }
.VAR/DM/RAM/ABS=0x1E00		CtrlBuf[256];

{ Codec Rx autobuffer: Status + L data + R data }
.VAR/DM/RAM/CIRC	RxBuf[3];

{ Codec Tx autobuffer: Cmd + L data + R data }
.VAR/DM/RAM/CIRC	TxBuf[3];

{*** Global internal registers ***}
.VAR/DM/RAM	GlobFlags,GlobSpare;
.CONST			GFLG_REC = B#00000001;

{*** Channel-specific registers ***}
.CONST NUMREGS	= 2;
.CONST REG_Vol	= 0;
.CONST REG_Pan	= 1;
.VAR/DM/RAM	ChanVols[NUMCHANS];
.VAR/DM/RAM	ChanPans[NUMCHANS];

.VAR/DM/RAM		InitFlag;	{ True while initializing codec }
.VAR/DM/RAM		ProcessBank;	{ RAM bank being processed }
.VAR/DM/RAM		CodecBank;	{ RAM bank connected to codec }
.VAR/DM/RAM		RecalcVols;	{ Need to recalc volume coefficients }
.VAR/DM/RAM		LastStatus;	{ Codec status store }
.VAR/DM/RAM		CodecCmd;	{ Codec command addr/data }
.VAR/DM/RAM		CodecCmdFlag;	{ Codec command waiting }

.VAR/DM/RAM		statbits;	{ Mirror of status bits to CPU }
.VAR/DM/RAM		i4_store;	{ Preserves i4 during interrupts }

.CONST STAT_ACK	= B#00000001;

{*** Orca DMA controller registers ***}
.PORT		wrdcnt;		{ R/W }
.PORT		blkcnt;		{ R/W }
.PORT		endblk;		{ R/W }
.PORT		dmactrl;	{ R/W }
{
.PORT		spare4;
.PORT		spare5;
.PORT		spare6;
.PORT		spare7;
}

{*** CPU Serial registers ***}
.PORT		Xreg0;		{ R=Cpu0,W=Dma0 }
.PORT		Xreg1;		{ R=Cpu1,W=Dma1 }
.PORT		Xreg2;		{ R=Cpu2,W=Dma2 }
.PORT		page_X12;	{ R=Cpu1/2,W=RAM bank select bit }
{
.PORT		spareC;
.PORT		spareD;
.PORT		spareE;
.PORT		spareF;
}


{********************************}
{* Initial codec setup commands *}
{********************************}

.INIT InitCmdList:
 0xC00A00,	{ Left Input Control: 0 gain, Line 1 input          }
 0xC10A00,	{ Right Input Control: 0 gain, Line 1 input         }
 0xC28000,	{ Left Aux #1 Input Control: Muted                  }
 0xC38000,	{ Right Aux #1 Input Control: Muted                 }
 0xC48000,	{ Left Aux #2 Input Control: Muted                  }
 0xC58000,	{ Right Aux #2 Input Control: Muted                 }
 0xC68000,	{ Left DAC Control: 0 attenuation, muted            }
 0xC78000,	{ Right DAC Control: 0 attenuation, muted           }
 0xC85B00,	{ Data Fmt: XTAL2, 44.1kHz, stereo, 16-bit lin PCM  }
 0xC90900,	{ Interface Config: Playback enabled, ACAL allowed  }
 0xCA0000,	{ Pin Control: CLKOUT active, XCTL1/0 LO            }
 0xCC4000,	{ Misc. Info: Transmit on 0,1,2, 32-word frame      }
 0xCD0000,	{ Digital Mix Control: DME Disabled, 0 attenuation  }
 0x8FFF00;	{ End }

{**********************}
{*  Interrupt Vectors *}
{**********************}

		jump Start;			{ Restart }
		nop;
		nop;
		nop;

		jump field_irq;			{ IRQ2 - new video field }
		nop;
		nop;
		nop;

		ar = dm(InitFlag);		{ SPORT0 Tx - codec }
		ar = pass ar;
		if eq rti; 
		jump NextCmd;

		jump codec_irq;			{ SPORT0 Rx - codec }
		nop;
		nop;
		nop;

		jump dma_irq;			{ IRQ1 - DMA done }
		nop;
		nop;
		nop;

		jump cmd_irq;			{ IRQ0 - New Command }
		nop;
		nop;
		nop;

		rti;				{ Timer irq }
		nop;
		nop;
		nop;


{*****************}
{* Program start *}
{*****************}

Start:
		ENA M_MODE;		{ Integer multiplies }

		ax0 = 0x2000;
		dm(Dm_Wait_Reg) = ax0;	{ 2 wait states for Orca DMA }

		i0 = ^RxBuf;
		l0 = %RxBuf;
		i1 = ^TxBuf;
		l1 = %TxBuf;
		m1 = 1;

		i4 = ^InitCmdList;
		l4 = %InitCmdList;
		m4 = 1;

{*** Initialize Tx buffer ***}

		ax0=0xCFFF;		{ Initially set MCE }
		dm(i1,m1)=ax0;
		ax0=0x0000;
		dm(i1,m1)=ax0;
		dm(i1,m1)=ax0;

{*******************************************************}
{* SPORT0 setup: multichannel enable, ext sclk, MFD=1, *}
{* 32 words, ext. rfs, DTYPE 00, 16 bits               *}
{*******************************************************} 

		ax0 = b#1000011000001111;
		dm(Sport0_Ctrl_Reg) = ax0;

{******************************}
{* Multichannel enable setup: *}
{*    Rx: 0, 1, 2, 16, 17, 18 *}
{*    Tx: 0, 1, 2, 16, 17, 18 *}
{******************************}

		ax0 = b#0000000000000111;
		dm(Sport0_Rx_Words0) = ax0;
		dm(Sport0_Rx_Words1) = ax0;

		ax0 = b#0000000000000111;
		dm(Sport0_Tx_Words0) = ax0;
		dm(Sport0_Tx_Words1) = ax0;

{**************************}
{* SPORT Autobuffer Setup *}
{*    Rx: I0, M1          *}
{*    Tx: I1, M1          *}
{**************************}

		ax0 = b#0000001010000111;
		dm(Sport0_Autobuf_Ctrl) = ax0;

		ax0 = b#0001000000000000;
		dm(Sys_Ctrl_Reg) = ax0;		{ SPORT0 Enabled,
						   SPORT1 = FI, FO, IRQs }

		ifc = b#000000111111;		{ Clear all interrupts }
		nop;
		
		ax0 = 1;
		dm(InitFlag) = ax0;		{ Show we're init'ing codec }

		imask = b#010000;		{ enable Tx0 interrupt }

		{ Why sending word to sport??? }
		ax0 = dm(i1,m1);
		tx0 = ax0;

{*** Wait for codec initialization to be completed ***}
		
wait_init:	ax0 = dm(InitFlag);		{ wait for entire init }
		af = pass ax0;			{ buffer to be sent to }
		if ne jump wait_init;		{ the codec            }


#ifdef NEVER
{*** Wait for codec to start and complete its autocalibration ***}

		ay0 = b#0000000000000010;	{ Mask for ACI bit }

wait_cal_start:	ax0 = dm(RxBuf);
		ar = ax0 and ay0;		{ Started yet? }
		if eq jump wait_cal_start;

wait_cal_done:	ax0 = dm(RxBuf);
		ar = ax0 and ay0;		{ Finished? }
		if ne jump wait_cal_done;
#endif

{*** Do more system initialization ***}

		ax0 = 0;
		dm(ProcessBank) = ax0;		{ Process bank 0 first }
		ax1 = 1;
		dm(CodecBank) = ax1;		{ Codec on bank 1 }
		ax1 = GFLG_REC;
		dm(GlobFlags) = ax1;		{ In record mode }

		ax0 = 0;
		dm(statbits) = ax0;
		dm(Xreg0) = ax0;		{ Clear status bits }

		dm(CodecCmdFlag) = ax0;		{ No command waiting }

		l4 = %LRcoefs;
		m4 = 1;

		l6 = 0;
		m6 = 1;
		l7 = 0;
		m7 = 1;

{*** Clear data buffers ***}
		i2 = ^ChansBuf;		{ Group of channel buffers }
		l2 = %ChansBuf;
		m2 = CHANBUFSIZE;

clearit:	dm(i2,m1) = 0;
		ax0 = i2;
		ay0 = ^ChansBuf;
		ar  = ax0 - ay0;
		if ne jump clearit;


{*** Wait a bit, so as not to sound nasty (why???) ***}

		cntr = 5000;
waiter:		idle;
		if not ce jump waiter;


{*** Set default coef array: panned for 4 pairs of stereo channels ***}

		i7 = ^ChanVols;
		ax0 = 0xFFFF;
		cntr = NUMCHANS;
		do dflt_vols until ce;
dflt_vols:	dm(i7,m7) = ax0;

		i7 = ^ChanPans;
		ax0 = 0x8000;
		cntr = NUMCHANS;
		do dflt_pans until ce;
dflt_pans:	dm(i7,m7) = ax0;

		call CalcVolCoefs;	{ Please figure initial coefficients }


{*** Remove mute from output DACs ***}

		idle;

		ax0 = 0x8600;
		dm(TxBuf) = ax0;		{ Unmute left DAC }
		idle;

		ax0 = 0x8700;
		dm(TxBuf) = ax0;		{ Unmute right DAC }
		idle;

		ifc = b#000000111111;		{ Clear all interrupts }
		nop;
		imask = b#101110;		{ Enable Rx0,Field,Cmd,Dma IRQs }

{*****************************}
{*         Main loop         *}
{*****************************}

main:
	{*** Re-calculate volume coefficients, if necessary ***}
		ax0 = dm(RecalcVols);
		ar  = PASS ax0;
		if ne call CalcVolCoefs;

		jump	main;
		

{***************}
{* Subroutines *}
{***************}

CalcVolCoefs:
		i4  = ^LRcoefs;
		i6  = ^ChanVols;
		l6  = %ChanVols;
		i7  = ^ChanPans;
		l7  = %ChanPans;

		ay1 = 0x8000;
		my0 = 2;

{*** Convert vol,pan to left volume for each channel ***}
		cntr = NUMCHANS;
		do left_coefs until ce;
		ay0 = dm(i7,m7);	{ Channel pan }
		ar  = -ay0;
		mr  = ar * my0 (UU);
		if mv sat mr;
		ar  = mr1;
		if mv ar = ar + ay1;
		my1 = dm(i6,m6);	{ Channel volume }
		mr  = ar * my1 (UU);
left_coefs:	pm(i4,m4) = mr1;

{*** Convert vol,pan to right volume for each channel ***}
		cntr = NUMCHANS;
		do right_coefs until ce;
		mx0 = dm(i7,m7);	{ Channel pan }
		mr  = mx0 * my0 (UU);
		if mv sat mr;
		ar  = mr1;
		if mv ar = ar + ay1;
		my1 = dm(i6,m6);	{ Channel volume }
		mr  = ar * my1 (UU);
right_coefs:	pm(i4,m4) = mr1;

		ax0 = 0;
		dm(RecalcVols) = ax0;

		rts;


{*********************************************************************}
{* Command interrupt handler -- called when command register written *}
{*********************************************************************}
cmd_irq:	ENA sec_reg;		{ Use secondary registers }

		ax0 = dm(Xreg0);
		ay0 = B#11110000;
		ar  = ax0 AND ay0;	{ Get just command type }
		sr  = LSHIFT ar BY -4 (LO);
		ay1 = ^jump_table;
		ar  = sr0 + ay1;
		i7  = ar;

		ay0 = B#00001111;
		ar  = ax0 AND ay0;
		ax0 = ar;		{ ax0 = arg }

		jump (i7);

jump_table:	jump do_nop;
		jump do_nop;
		jump write_chanreg;
		jump write_globreg;
		jump do_nop;
		jump write_codec;
		jump read_codec;
		jump do_nop;
		jump dma_read;
		jump dma_write;
		jump do_nop;
		jump do_nop;
		jump do_nop;
		jump do_nop;
		jump do_nop;
		jump do_nop;

{*** Write internal global register ***}
write_globreg:	ay0 = ^GlobFlags;
		ar  = ax0 + ay0;
		i7  = ar;
		ax0 = dm(page_X12);		{ Read word }
		dm(i7,m7) = ax0;
		jump do_nop;

{*** Write internal channel register ***}
write_chanreg:	ay1 = ^ChanVols;
		ay0 = B#00001000;
		ar  = ax0 AND ay0;	{ Which of 2 registers? }
		if eq jump intovols;
		ay1 = ^ChanPans;
intovols:	ay0 = B#00000111;
		ar  = ax0 AND ay0;	{ Get channel # }
		ar  = ar + ay1;
		i7  = ar;
		ax0 = dm(page_X12);	{ Read word }
		dm(i7,m7) = ax0;
		ax0 = 1;
		dm(RecalcVols) = ax0;	{ Need to update coefficients now }
		jump do_nop;

{*** Write codec register ***}
write_codec:	ax0 = dm(Xreg1);
		dm(CodecCmd) = ax0;	{ Pipe to codec next time }
		ax0 = 1;
		dm(CodecCmdFlag) = ax0;
		jump do_nop;

{*** Read most recent codec status register ***}
read_codec:	ax0 = dm(LastStatus);
		dm(Xreg1) = ax0;	{ Write to CPU register }
		jump do_nop;

{*** Initiate DMA from DSP RAM to DMA RAM ***}
dma_read:	ay1 = B#00010011;
		jump dma_join;

{*** Initiate DMA from DMA RAM to DSP RAM ***}
dma_write:	ay1 = B#00010001;
dma_join:
		{*** Reset Audio DMA ***}
		ax1 = B#00000000;
		dm(dmactrl) = ax1;
		ax1 = B#00010000;
		dm(dmactrl) = ax1;

		{*** Turn channel # into a block address ***}
		mx0 = ax0;
		my0 = CHANBLKSIZE;
		mr  = mx0 * my0 (UU);
		ar  = mr0;

		{*** If codec is talking to 1st page, do DMA to 2nd page ***}
		ax0 = dm(CodecBank);
		af  = PASS ax0;
		ay0 = 0x40;
		if eq ar = ar + ay0;

		dm(blkcnt) = ar;		{ Set DMA start blk }

		{*** Compute DMA end block ***}
		ay0 = dm(Xreg1);
		af  = ar + ay0;
		ar  = af-1;
		dm(endblk) = ar;		{ Write to DMA }

		{*** Enable DMA ***}
		dm(dmactrl) = ay1;	{ Use direction bit set earlier }

		jump do_nop;

do_nop:
		{*** Send acknowledge to CPU ***}
		ax0 = dm(statbits);
		ay0 = STAT_ACK;
		ar  = ax0 XOR ay0;
		dm(statbits) = ar;
		dm(Xreg0) = ar;

		DIS sec_reg;		{ Restore to primary registers }
		rti;

{******************************************************}
{* DMA interrupt handler -- called when DMA completes *}
{******************************************************}
dma_irq:	ENA sec_reg;		{ Use secondary registers }


		DIS sec_reg;		{ Restore to primary registers }
		rti;

{********************************************************************}
{* Field interrupt handler -- called at the top of each video field *}
{********************************************************************}

field_irq:	ENA sec_reg;		{ Use secondary registers }

{*** Toggle buffers ***}
		ax0 = dm(CodecBank);
		ay0 = BANKBIT;
		ar  = ax0 XOR ay0;
		dm(CodecBank) = ar;

		i2 = ^ChansBuf;		{ At start of each channel buffer }

		DIS sec_reg;		{ Restore to primary registers }
		rti;


{******************************************************************}
{* Codec interrupt handler -- called after each sample transacted *}
{******************************************************************}

codec_irq:	ENA sec_reg;		{ Use secondary registers }
		dm(i4_store) = i4;	{ Preserve this }

		ax0 = dm(CodecBank);
		dm(page_X12) = ax0;	{*** Select RAM bank for codec ***}

		ax0 = dm(RxBuf+0);	{ Keep codec status }
		dm(LastStatus) = ax0;

	{*** If a user codec command is waiting ,send it now ***}
		ax0 = dm(CodecCmdFlag);
		ar  = PASS ax0;
		if eq jump no_codec_cmd;
		ax0 = dm(CodecCmdFlag);
		dm(TxBuf+0) = ax0;
		ax0 = 0;
		dm(CodecCmdFlag) = ax0;
no_codec_cmd:

		ax0 = dm(GlobFlags);
		ay0 = GFLG_REC;		{ Determine if playing/recording }
		ar  = ax0 AND ay0;
		if eq jump data_play;

{*** Recording - copy samples to mix buffer as well as looping to DACs ***}
data_rec:	ax0 = dm(RxBuf+1);	{ L channel input }
		dm(i2,m2) = ax0;	{ Store in channel 0 }
		dm(TxBuf+1) = ax0;	{ Pipe thru to output DAC }

		ax0 = dm(RxBuf+2);	{ R channel input }
		dm(i2,m2) = ax0;	{ Store in channel 1 }
		dm(TxBuf+2) = ax0;	{ Pipe thru to output DAC }

		cntr = NUMCHANS-2;
		do wrapback until ce;
wrapback:	MODIFY(i2,m2);

		jump data_end;

{*** Playing - copy samples from mix buffer ***}
data_play:
		i4 = ^LRcoefs;			{ Ensure start of coefs }

		{ Compute the "left" value }
		cntr = NUMCHANS-1;
		mr = 0, my0 = pm(i4,m4), mx0 = dm(i2,m2);
		do MixLeft until ce;
MixLeft:	mr = mr + mx0 * my0 (SU), my0 = pm(i4,m4), mx0 = dm(i2,m2);
		mr = mr + mx0 * my0 (SU);
		mr = mr (RND);
		if mv sat mr;
		dm(TxBuf+1) = mr1;	{ Send to output DAC }

		{ Compute the "right" value }
		cntr = NUMCHANS-1;
		mr = 0, my0 = pm(i4,m4), mx0 = dm(i2,m2);
		do MixRight until ce;
MixRight:	mr = mr + mx0 * my0 (SU), my0 = pm(i4,m4), mx0 = dm(i2,m2);
		mr = mr + mx0 * my0 (SU);
		mr = mr (RND);
		if mv sat mr;
		dm(TxBuf+2) = mr1;	{ Send to output DAC }

data_end:
		MODIFY(i2,m1);		{ Bump to next sample }

		ax0 = dm(ProcessBank);
		dm(page_X12) = ax0;	{ Restore RAM bank }

		i4 = dm(i4_store);	{ Restore this }

		DIS sec_reg;		{ Restore to primary registers }
		rti;

{**********************************}
{* Codec initialization interrupt *}
{**********************************}

NextCmd:	ENA sec_reg;
		ax0 = pm(i4,m4);	{ Read next command }
		dm(TxBuf) = ax0;	{ Place in Tx buffer }
		ax0 = i4;		{ Read current pos in cmd buffer }
		ay0 = ^InitCmdList;	{ Start location of cmd buffer }
		ar = ax0 - ay0;
		if ne jump still_init;	{ Wrapped around? }
		ax0 = 0;
		dm(InitFlag) = ax0;	{ Clear status flag }
still_init:	DIS sec_reg;
		rti;

.ENDMOD;
