{*****************************************************************}
{* Flyer Audio DSP Code (For rev 4)                              *}
{*                                                               *}
{* Written for ADSP-2115 and AD1847.                             *}
{*                                                               *}
{* Performs Audio playback and record at 16-bit linear, 44.1 KHz *}
{*                                                               *}
{*                                                               *}
{* 08-26-94       Marty Flickinger                               *}
{*                                                               *}
{*****************************************************************}
{* Test version -- CMD IRQ generates handshake IRQ *}

.MODULE/ABS=0/BOOT=0  AD1847;

.INCLUDE <..\include\DEF2101.H>;

{**************** Constants ****************}

.CONST NUMCHANS	= 8;		{ 8 mono channels }
.CONST SAMPLES	= 735;		{ Samples per field }

.CONST CHANBLKSIZE	= 3;
.CONST CHANBUFSIZE	= CHANBLKSIZE * 256;

{**************** Variables ****************}

{ Codec initialization command list array }
.VAR/PM/RAM/CIRC	InitCmdList[14];

{ Left/Right coefficients for each channel (left group, right group) }
.VAR/PM/RAM/CIRC	LRtargets[NUMCHANS * 2];

{ Left/Right volumes (32 bit -- F/I) }
.VAR/PM/RAM/CIRC	LRvolumes[NUMCHANS * 4];

{ Left/Right volume ramp counters (32 bit -- F/I) }
.VAR/PM/RAM/CIRC	LRstepcounts[NUMCHANS * 4];

{ Channel data buffers -- on each RAM page }
.VAR/DM/RAM/CIRC/ABS=0x0000	ChansBuf[NUMCHANS * CHANBUFSIZE];

{ Left/Right volume ramp steps (32 bit -- Lo/Hi negative) }
.VAR/DM/RAM/CIRC	LRsteps[NUMCHANS * 4];

{ Mixed-down stereo data buffer -- on each RAM page }
.VAR/DM/RAM/ABS=0x1800		MixBuf[2 * CHANBUFSIZE];

{ CPU control buffer -- page 0 only??? }
.VAR/DM/RAM/ABS=0x1E00		CtrlBuf[256];

{ Codec Rx autobuffer: Status + L data + R data }
.VAR/DM/RAM/CIRC	RxBuf[3];

{ Codec Tx autobuffer: Cmd + L data + R data }
.VAR/DM/RAM/CIRC	TxBuf[3];

{*** Global internal registers ***}
.VAR/DM/RAM	GlobFlags,GlobSpare;
.CONST			GFLG_REC = B#00000001;

{*** Channel-specific registers ***}
.CONST NUMREGS	= 2;
.CONST REG_Vol	= 0;
.CONST REG_Pan	= 1;	{ 0000 = Left, 8000 = Ctr, FFFF = Right }
.CONST REG_Rate	= 2;
.VAR/DM/RAM	ChanVols[NUMCHANS],ChanPans[NUMCHANS],ChanRates[NUMCHANS];


.VAR/DM/RAM		InitFlag;	{ True while initializing codec }
.VAR/DM/RAM		DMABank;	{ RAM bank available to DMA }
.VAR/DM/RAM		CodecBank;	{ RAM bank connected to codec }
.VAR/DM/RAM		SampleCount;	{ Counts Codec samples }
.VAR/DM/RAM		CodecSyncUp;	{ Codec sync-up flag/counter }
.VAR/DM/RAM		LastStatus;	{ Codec status store }
.VAR/DM/RAM		CodecCmd;	{ Codec command addr/data }
.VAR/DM/RAM		CodecCmdFlag;	{ Codec command waiting }
.VAR/DM/RAM		WhichChan;	{ Channel to update/work on }
.VAR/DM/RAM		NewRampChans;	{ Flags for each chan: do new ramp }

.VAR/DM/RAM		DebugAddr;	{ FOR DEBUGGING }
.VAR/DM/RAM		Result;
.VAR/DM/RAM		Pwr2,NegHi,NegLo;

.VAR/DM/RAM		statbits;	{ Mirror of status bits to CPU }
.VAR/DM/RAM		i3_store;	{ Preserves i3 during interrupts }
.VAR/DM/RAM		i4_store;	{ Preserves i4 during interrupts }
.VAR/DM/RAM		i6_store;	{ Preserves i6 during interrupts }
.VAR/DM/RAM		l4_store;	{ Preserves l4 during interrupts }
.VAR/DM/RAM		l6_store;	{ Preserves l6 during interrupts }

.CONST STAT_ACK	= B#00000001;

{*** Orca DMA controller registers ***}
.PORT	wrdcnt;		{ R/W }
.PORT	blkcnt;		{ R/W }
.PORT	endblk;		{ R/W }
.PORT	dmactrl;	{ R/W }
{
.PORT	spare4;
.PORT	spare5;
.PORT	spare6;
.PORT	spare7;
}

{*** CPU Serial registers ***}
.PORT	Xreg0;		{ R=Cpu0,W=Dma0 }
.PORT	Xreg1;		{ R=Cpu1,W=Dma1 }
.PORT	Xreg2;		{ R=Cpu2,W=Dma2 }
.PORT	page_X12;	{ R=Cpu1/2,W=RAM bank select bit }
.CONST		BANKBIT	= 1;		{ Bit value for RAM bank select bit }
{
.PORT	spareC;
.PORT	spareD;
.PORT	spareE;
.PORT	spareF;
}


{********************************}
{* Initial codec setup commands *}
{********************************}

.INIT InitCmdList:
 0xC00600,	{ Left Input Control: 0 gain, Line 1 input          }
 0xC10600,	{ Right Input Control: 0 gain, Line 1 input         }
 0xC28000,	{ Left Aux #1 Input Control: Muted                  }
 0xC38000,	{ Right Aux #1 Input Control: Muted                 }
 0xC48000,	{ Left Aux #2 Input Control: Muted                  }
 0xC58000,	{ Right Aux #2 Input Control: Muted                 }
 0xC68000,	{ Left DAC Control: 0 attenuation, muted            }
 0xC78000,	{ Right DAC Control: 0 attenuation, muted           }
 0xC85B00,	{ Data Fmt: XTAL2, 44.1kHz, stereo, 16-bit lin PCM  }
 0xC90900,	{ Interface Config: Playback enabled, ACAL allowed  }
 0xCA0000,	{ Pin Control: CLKOUT active, XCTL1/0 LO            }
 0xCC4000,	{ Misc. Info: Transmit on 0,1,2, 32-word frame      }
 0xCD0000,	{ Digital Mix Control: DME Disabled, 0 attenuation  }
 0x8FFF00;	{ End }

{**********************}
{*  Interrupt Vectors *}
{**********************}

		jump Start;			{ Restart }
		nop;
		nop;
		nop;

		jump field_irq;			{ IRQ2 - new video field }
		nop;
		nop;
		nop;

		jump init_cmd;			{ SPORT0 Tx - codec }
		nop;
		nop;
		nop;

		jump codec_irq;			{ SPORT0 Rx - codec }
		nop;
		nop;
		nop;

		jump dma_irq;			{ IRQ1 - DMA done }
		nop;
		nop;
		nop;

		jump cmd_irq;			{ IRQ0 - New Command }
		nop;
		nop;
		nop;

		rti;				{ Timer irq }
		nop;
		nop;
		nop;


{*****************}
{* Program start *}
{*****************}

Start:
		ENA M_MODE;		{ Integer multiplies }

		ax0 = 0x2000;
		dm(Dm_Wait_Reg) = ax0;	{ 2 wait states for Orca DMA }

		i0 = ^RxBuf;
		l0 = %RxBuf;
		i1 = ^TxBuf;
		l1 = %TxBuf;
		m1 = 1;

		i4 = ^InitCmdList;
		l4 = %InitCmdList;
		m4 = 1;

{*** Initialize Tx buffer ***}

		ax0=0xCFFF;		{ Initially set MCE }
		dm(i1,m1)=ax0;
		ax0=0x0000;
		dm(i1,m1)=ax0;
		dm(i1,m1)=ax0;

{*******************************************************}
{* SPORT0 setup: multichannel enable, ext sclk, MFD=1, *}
{* 32 words, ext. rfs, DTYPE 00, 16 bits               *}
{*******************************************************} 

		ax0 = b#1000011000001111;
		dm(Sport0_Ctrl_Reg) = ax0;

{******************************}
{* Multichannel enable setup: *}
{*    Rx: 0, 1, 2, 16, 17, 18 *}
{*    Tx: 0, 1, 2, 16, 17, 18 *}
{******************************}

		ax0 = b#0000000000000111;
		dm(Sport0_Rx_Words0) = ax0;
		dm(Sport0_Rx_Words1) = ax0;

		ax0 = b#0000000000000111;
		dm(Sport0_Tx_Words0) = ax0;
		dm(Sport0_Tx_Words1) = ax0;

{**************************}
{* SPORT Autobuffer Setup *}
{*    Rx: I0, M1          *}
{*    Tx: I1, M1          *}
{**************************}

		ax0 = b#0000001010000111;
		dm(Sport0_Autobuf_Ctrl) = ax0;

		ax0 = b#0001000000000000;
		dm(Sys_Ctrl_Reg) = ax0;		{ SPORT0 Enabled,
						   SPORT1 = FI, FO, IRQs }

		ifc = b#000000111111;		{ Clear all interrupts }
		nop;
		
		ax0 = 1;
		dm(InitFlag) = ax0;		{ Show we're init'ing codec }

		imask = b#010000;		{ enable Tx0 interrupt }

		{ Why sending word to sport??? }
		ax0 = dm(i1,m1);
		tx0 = ax0;

{*** Wait for codec initialization to be completed ***}
		
wait_init:	ax0 = dm(InitFlag);		{ wait for entire init }
		af = pass ax0;			{ buffer to be sent to }
		if ne jump wait_init;		{ the codec            }


#ifdef NEVER
{*** Wait for codec to start and complete its autocalibration ***}

		ay0 = b#0000000000000010;	{ Mask for ACI bit }

wait_cal_start:	ax0 = dm(RxBuf);
		ar = ax0 and ay0;		{ Started yet? }
		if eq jump wait_cal_start;

wait_cal_done:	ax0 = dm(RxBuf);
		ar = ax0 and ay0;		{ Finished? }
		if ne jump wait_cal_done;
#endif

{*** Do more system initialization ***}

		ax0 = 0;
{		dm(DMABank) = ax0;    }		{ DMA bank (same for a bit) }
		dm(CodecBank) = ax0;		{ Codec on bank 0 }
		ax1 = GFLG_REC;
		dm(GlobFlags) = ax1;		{ In record mode }

		ax0 = 0;
		dm(statbits) = ax0;
		dm(Xreg0) = ax0;		{ Clear status bits }
		dm(CodecCmdFlag) = ax0;		{ No command waiting }
		dm(NewRampChans) = ax0;		{ No new ramps??? }

		ax0 = 3;
		dm(CodecSyncUp) = ax0;		{ Need to sync sample counter }

		m0 = 0;
		m3 = -2;
		m4 = 1;
		m6 = -1;
		m7 = 0;

		l7 = 0;

{*** Clear data buffers (both banks) ***}
		CNTR = 2;
		do clearboth until ce;

		ay0 = cntr;
		ar  = ay0-1;
		dm(page_X12) = ay0;	{ Select bank }

		i2 = ^ChansBuf;		{ Group of channel buffers }
		l2 = %ChansBuf;
		m2 = CHANBUFSIZE;

clearit:	dm(i2,m1) = 0;
		ax0 = i2;
		ay0 = ^ChansBuf;
		ar  = ax0 - ay0;
		if ne jump clearit;
clearboth:	nop;

{*** Wait a bit, so as not to sound nasty (why???) ***}

		cntr = 5000;
waiter:		idle;
		if not ce jump waiter;


{*** Set default coef array: all channels full off, panned to center ***}

		i7 = ^ChanVols;
		cntr = NUMCHANS;
		do dflt_vols until ce;
dflt_vols:	dm(i7,m4) = 0x0000;

		i7 = ^ChanPans;
		cntr = NUMCHANS;
		do dflt_pans until ce;
dflt_pans:	dm(i7,m4) = 0x8000;

		i7 = ^ChanRates;
		cntr = NUMCHANS;
		do dflt_rates until ce;
dflt_rates:	dm(i7,m4) = 0xFFFF;

		i5 = ^LRvolumes;
		l5 = %LRvolumes;
		ax0 = 0x0000;
		cntr = NUMCHANS*4;
		do dflt_lrv until ce;
dflt_lrv:	pm(i5,m4) = ax0;

		i3 = ^LRsteps;
		l3 = %LRsteps;
		cntr = NUMCHANS*4;
		do dflt_steps until ce;
dflt_steps:	dm(i3,m1) = 0;

		cntr = NUMCHANS;
		do calc_all until ce;
		ay0 = CNTR;
		ar  = ay0-1;
		dm(WhichChan) = ar;
		call CalcVolCoefs;	{ Calculate volume coefs,steps,etc }
calc_all:	nop;

{*** Remove mute from output DACs ***}

		idle;

		ax0 = 0x8600;
		dm(TxBuf) = ax0;		{ Unmute left DAC }
		idle;

		ax0 = 0x8700;
		dm(TxBuf) = ax0;		{ Unmute right DAC }
		idle;

		{*** Reset Audio DMA ***}
		ax1 = B#00000000;
		dm(dmactrl) = ax1;

		icntl = b#00111;		{ No nesting, edge sensitive }

		ifc = b#000000111111;		{ Clear all interrupts }
		nop;
		imask = b#101110;		{ Enable Rx0,Field,Cmd,Dma IRQs }


{*****************************}
{*         Main loop         *}
{*****************************}

main:
		jump	main;
		

{***************}
{* Subroutines *}
{***************}

{**********************************************************************}
{* Converts a channel's volume and pan registers into separate volume *}
{* coefficients for left and right channels.  These "target" values   *}
{* are picked up at the start of the next field, at which time the    *}
{* DSP will begin to ramp toward the target volumes.                  *}
{* "WhichChan" specifies register to process.                         *}
{**********************************************************************}
CalcVolCoefs:
#ifdef NEVER
		{ Interrupt 68000  -- CPU will clear when serviced }
		ax0 = dm(CodecBank);
		ay0 = 0x80;
		ar  = ax0 OR ay0;
		dm(page_X12) = ar;	{ Set CPU IRQ flag }
#endif

		m5  = dm(WhichChan);

		i4  = ^LRtargets;
		l4  = %LRtargets;
		modify(i4,m5);		{ Work on requested channel }
		i6  = ^ChanVols;
		l6  = 0;
		modify(i6,m5);		{ Work on requested channel }
		i7  = ^ChanPans;
		l7  = 0;
		modify(i7,m5);		{ Work on requested channel }

		m5  = NUMCHANS;		{ Between left and right for channel }

		my1 = dm(i6,m7);	{ Channel volume }
		ax0 = dm(i7,m7);	{ Channel pan }

{*** Convert vol,pan to left volume for channel ***}
		ar  = NOT ax0;		{ Flip ramp for left sense }
		ay0 = ar;
		ar  = ar + ay0;		{ Double the slope }
		if NOT ac jump nosatpan1;
		ar = 0xFFFF;		{ Saturate at FFFF (left of center) }
nosatpan1:	mr  = ar * my1 (UU);	{ Left coefficient * channel volume }
		pm(i4,m5) = mr1;	{ Save off }

{*** Convert vol,pan to right volume for channel ***}
		ay0 = ax0;
		ar  = ax0 + ay0;	{ Double the slope }
		if NOT ac jump nosatpan2;
		ar = 0xFFFF;		{ Saturate at FFFF (right of center) }
nosatpan2:	mr  = ar * my1 (UU);	{ Right coefficient * channel volume }
		pm(i4,m5) = mr1;	{ Save off }

		jump SimpleNewRamp;
		rts;

{**********************************************************************}
{* Uses channel's target volumes (L and R) and rate register to calc- *}
{* ulate the necessary volume step pair and step count pair to effect *}
{* the specified volume ramp.  "WhichChan" specifies register to process.     *}
{**********************************************************************}
SimpleNewRamp:
#ifdef NEVER
		{ Interrupt 68000  -- CPU will clear when serviced }
		ax0 = dm(CodecBank);
		ay0 = 0x80;
		ar  = ax0 OR ay0;
		dm(page_X12) = ar;	{ Set CPU IRQ flag }
#endif
		imask = b#000000;	{ Enable Rx0,Field,Cmd,Dma IRQs }

		m5 = dm(WhichChan);

		i7  = ^ChanRates;
		l7  = 0;
		modify(i7,m5);		{ Work on requested channel }
		ay1 = dm(i7,m7);	{ Get rate value }
		ay0 = 0;
		ax0 = 16;
		si  = 1;

trypwr2:	se  = ay0;
		sr  = LSHIFT si (LO);
		ar  = sr0 - ay1;	{ This value >= requested value? }
		if ac jump usepwr2;	{ If yes, use this power of 2 }
		ar  = ay0 - ax0;	{ Hit 16? -- if so, break out }
		if ac jump usepwr2;
		ar  = ay0+1;		{ ay0++ }
		ay0 = ar;
		jump trypwr2;
usepwr2:	ar  = ax0 - ay0;	{ se = 16-n }
		se  = ar;
		dm(Pwr2) = ar;

		{ Make negative step counts -- do a 32-bit negate operation }
		af  = NOT sr1;		{ High }
		ar  = -sr0;
		dm(NegLo) = ar;		{ Low  }
		ax0 = 0;
		ar  = ax0+af+C;		{ Bump 1 if carry from low 16 negate }
		dm(NegHi) = ar;


		i5  = ^LRtargets;
		l5  = %LRtargets;
		modify(i5,m5);		{ Work on requested channel }

		i4  = ^LRvolumes;
		l4  = %LRvolumes;
		modify(i4,m5);		{ Work on requested channel }
		modify(i4,m5);		{ Work on requested channel }

		i7  = ^LRsteps;
		l7  = %LRsteps;
		modify(i7,m5);		{ Work on requested channel }
		modify(i7,m5);		{ Work on requested channel }

		i6  = ^LRstepcounts;
		l6  = %LRstepcounts;
		modify(i6,m5);		{ Work on requested channel }
		modify(i6,m5);		{ Work on requested channel }

		m5 = NUMCHANS-1;
		CNTR = 2;
		do simple_ramp until ce;
{		se = 16-15; }
		ax1 = 0;
		pm(i4,m4) = ax1;	{ Throw away past fractional }
		ay0 = pm(i4,m4);
		ax0 = pm(i5,m4);
		ax1 = 0xFFFF;		{ Upper 16 FFFF if negative delta }
		ar = ax0 - ay0;		{ Volume difference }
		si = ar;
		ar = ax1+C;		{ 0000 if positive delta }
		sr = ASHIFT ar (HI);		{ High half }
		sr = sr OR LSHIFT si (LO);	{ Low half }

		dm(i7,m4) = sr0;	{ Fractional steps }
		dm(i7,m4) = sr1;	{ Whole steps }
#ifdef NEVER
		ax0 = 0x8000;
		pm(i6,m4) = ax0;
		ax0 = 0xFFFF;
		pm(i6,m4) = ax0;	{ 32768 counts then stop }
#endif
		ar  = dm(NegLo);	{ Use negative step counts }
		pm(i6,m4) = ar;		{ Low  }
		ar  = dm(NegHi);	{ High }
		pm(i6,m4) = ar;		{ (-)n counts then stop }

		modify(i5,m5);
		modify(i4,m5);
		modify(i4,m5);
		modify(i6,m5);
		modify(i6,m5);
		modify(i7,m5);
		modify(i7,m5);
simple_ramp:	nop;

		imask = b#101110;		{ Enable Rx0,Field,Cmd,Dma IRQs }
		rts;


NewRamps:
		i7  = ^LRtargets;
		l7  = %LRtargets;
		i5  = ^ChanRates;
		l5  = 0;
		i4  = ^LRvolumes;
		l4  = %LRvolumes;
		i3  = ^LRsteps;
		l3  = %LRsteps;
		i6  = ^LRstepcounts;
		l6  = %LRstepcounts;

		ay1 = 0;		{ ???!!!!!!! }
		my0 = 2;

		CNTR = NUMCHANS * 2;
		do calc_ramps until ce;
	{ This channel needs new ramp values? }
		se  = cntr;
		si  = 1;
		sr  = LSHIFT si (LO);
		ay1 = sr0;
		ax0 = dm(NewRampChans);
		ar  = ax0 AND ay1;	{ This channel needs new ramp? }
		if eq jump no_ramp;
		ar  = ax0 XOR ay1;	{ Clear bit now }
		dm(NewRampChans) = ar;

		ax0 = pm(i7,m4);	{ Target volume }
		modify(i4,m4);
		ay0 = pm(i4,m7);	{ Current volume m???!!!!!!!! }
		ar  = ax0 - ay0;	{ Difference }
		ay1 = ar;
		ay0 = 0;
		ax0 = dm(i5,m4);	{ Get ramp length (in samples) }
		divs ay1,ax0;
		divq ax0; divq ax0; divq ax0;
		divq ax0; divq ax0; divq ax0;
		divq ax0; divq ax0; divq ax0;
		divq ax0; divq ax0; divq ax0;
		divq ax0; divq ax0; divq ax0;

{ WORKING HERE... }
{		mr  = quotient * rate;
		ar  = difference - mr;
		(ar << 16) DIV RATE ---> Steps frac
}
no_ramp:	modify(i7,m4);
		modify(i5,m4);
		modify(i3,m1);
		modify(i3,m1);
		modify(i4,m4);
		modify(i4,m4);
		modify(i6,m4);
		modify(i6,m4);
calc_ramps:	nop;


{*********************************************************************}
{* Command interrupt handler -- called when command register written *}
{*********************************************************************}
cmd_irq:	ENA sec_reg;		{ Use secondary registers }

		ax1 = dm(Xreg0);
		ay0 = B#11110000;
		ar  = ax1 AND ay0;	{ Get just command type }
		sr  = LSHIFT ar BY -4 (LO);
		ay1 = ^jump_table;
		ar  = sr0 + ay1;
		i7  = ar;

		ay0 = B#00001111;
		ar  = ax1 AND ay0;
		ax0 = ar;		{ ax0 = arg }

		jump (i7);

jump_table:	jump do_nop;
		jump resync;
		jump write_chanreg;
		jump write_chanreg;
		jump write_globreg;
		jump write_codec;
		jump read_codec;
		jump do_nop;
		jump dma_read;
		jump dma_write;
		jump do_nop;
		jump dbg_readdm;
		jump dbg_readpm;
		jump dbg_writedm;
		jump dbg_writepm;
		jump dbg_addr;

{*** Debugging - store address ***}
dbg_addr:	ax0 = dm(page_X12);		{ Read word }
		dm(DebugAddr) = ax0;
		jump do_nop;

{*** Debugging - read address ***}
dbg_readdm:	i7 = dm(DebugAddr);
		si = dm(i7,m7);
		jump r_merge;
dbg_readpm:	i7 = dm(DebugAddr);
		si = pm(i7,m7);
r_merge:	dm(Xreg2) = si;		{ Lo 8 }
		sr = LSHIFT si BY -8 (LO);
		dm(Xreg1) = sr0;	{ Hi 8 }
		jump do_nop;

{*** Debugging - write address ***}
dbg_writedm:	i7 = dm(DebugAddr);
		ax0 = dm(page_X12);		{ Read word }
		dm(i7,m7) = ax0;
		jump do_nop;
dbg_writepm:	i7 = dm(DebugAddr);
		ax0 = dm(page_X12);		{ Read word }
		pm(i7,m7) = ax0;
		jump do_nop;

resync:		ax0 = 1;
		dm(CodecSyncUp) = ax0;		{ Force re-sync }
		jump do_nop;

{*** Write internal global register ***}
write_globreg:	ay0 = ^GlobFlags;
		ar  = ax0 + ay0;
		i7  = ar;
		ax0 = dm(page_X12);		{ Read word }
		dm(i7,m7) = ax0;
		jump do_nop;

{*** Write internal channel register ***}
write_chanreg:
		ay0 = B#00011000;
		ar  = ax1 AND ay0;	{ Which of 4 registers? }
		si  = ar;
		sr  = LSHIFT si BY -3 (LO);	{ Get register}
		mx0 = sr0;
		my0 = NUMCHANS;
		mr  = mx0 * my0 (UU);
		ax0 = mr0;

		ay1 = ^ChanVols;	{ Start of arrays }
		af  = ax0 + ay1;
		ay0 = B#00000111;
		ar  = ax1 AND ay0;	{ Get channel # }
		dm(WhichChan) = ar;	{ Keep for later }

		ar  = ar + af;
		i7  = ar;
		ax0 = dm(page_X12);	{ Read word }
		dm(i7,m7) = ax0;	{ Update specified register }

		ax0 = sr0;
		ay0 = REG_Rate;
		ar  = ax0 - ay0;	{ Wrote RATE register? }

		{ If so, re-calculate L/R target volumes based on pan }
		{ Generate volume steps and counts to effect ramping }
		if eq call CalcVolCoefs;

		jump do_nop;

{*** Write codec register ***}
write_codec:	ax0 = dm(page_X12);
		dm(CodecCmd) = ax0;	{ Pipe to codec next time }
		ax0 = 1;
		dm(CodecCmdFlag) = ax0;
		jump do_nop;

{*** Read most recent codec status register ***}
read_codec:	ax0 = dm(LastStatus);
		dm(Xreg1) = ax0;	{ Write to CPU register }
		jump do_nop;

{*** Initiate DMA from DSP RAM to DMA RAM ***}
dma_read:	ay1 = B#00010011;
		jump dma_join;

{*** Initiate DMA from DMA RAM to DSP RAM ***}
dma_write:	ay1 = B#00010001;
dma_join:
		{*** Un-Reset Audio DMA ***}
		ax1 = B#00010000;
		dm(dmactrl) = ax1;

		{*** Turn channel # into a block address ***}
		mx0 = ax0;
		my0 = CHANBLKSIZE;
		mr  = mx0 * my0 (UU);
		ar  = mr0;

		{*** If DMA gets 2nd page, add offset to it ***}
		{*** If Codec gets 1st page, add offset to DMA 2nd page ***}
{		ax0 = dm(DMABank); }
		ax0 = dm(CodecBank);
		af  = PASS ax0;
		ay0 = 0x40;
{		if NE ar = ar + ay0; }
		if EQ ar = ar + ay0;

		dm(blkcnt) = ar;		{ Set DMA start blk }

		{*** Compute DMA end block ***}
		ay0 = dm(Xreg2);
		af  = ar + ay0;
		ar  = af-1;
		dm(endblk) = ar;		{ Write to DMA }

		{*** Enable DMA ***}
		dm(dmactrl) = ay1;	{ Use direction bit set earlier }

		jump do_nop;

do_nop:
		{*** Send acknowledge to CPU ***}
		ax0 = dm(statbits);
		ay0 = STAT_ACK;
		ar  = ax0 XOR ay0;
		dm(statbits) = ar;
		dm(Xreg0) = ar;

		{ Interrupt 68000  -- CPU will clear when serviced }
		ax0 = dm(CodecBank);
		ay0 = 0x80;
		ar  = ax0 OR ay0;
		dm(page_X12) = ar;	{ Set CPU IRQ flag }

		DIS sec_reg;		{ Restore to primary registers }
		rti;

{******************************************************}
{* DMA interrupt handler -- called when DMA completes *}
{******************************************************}
dma_irq:	ENA sec_reg;		{ Use secondary registers }

		{*** Clr IRQ, Reset Audio DMA ***}
		ax1 = B#00000000;
		dm(dmactrl) = ax1;

		{ Interrupt 68000  -- CPU will clear when serviced }
		ax0 = dm(CodecBank);
		ay0 = 0x80;
		ar  = ax0 OR ay0;
		dm(page_X12) = ar;	{ Set CPU IRQ flag }

		DIS sec_reg;		{ Restore to primary registers }
		rti;

{********************************************************************}
{* Field interrupt handler -- called at the top of each video field *}
{********************************************************************}

field_irq:	ENA sec_reg;		{ Use secondary registers }

{*** Toggle buffers for DMA ***}
{		ax0 = dm(DMABank);
		ay0 = BANKBIT;
		ar  = ax0 XOR ay0;
		dm(DMABank) = ar; }

{*** Synchronize Codec sample counter to the top of field (once only) ***}
		ay0 = dm(CodecSyncUp);
		ar  = PASS ay0;		{ Need to synchronize? }
		if eq jump insync;
		ar  = ay0-1;
		dm(CodecSyncUp) = ar;	{ Did one more }
		ax0 = 0;
		dm(SampleCount) = ax0;	{ Zero sample counter }
		i2 = ^ChansBuf;		{ At start of each channel buffer }
insync:
		ax0 = dm(SampleCount);	{ Snapshot sample counter }
		dm(Result) = ax0;

		DIS sec_reg;		{ Restore to primary registers }
		rti;


{******************************************************************}
{* Codec interrupt handler -- called after each sample transacted *}
{******************************************************************}

codec_irq:	ENA sec_reg;		{ Use secondary registers }
		dm(i3_store) = i3;	{ Preserve this }
		dm(i4_store) = i4;	{ Preserve this }
		dm(i6_store) = i6;	{ Preserve this }
		dm(l4_store) = l4;	{ Preserve this }
		dm(l6_store) = l6;	{ Preserve this }

		ax0 = dm(CodecSyncUp);
		ar  = PASS ax0;		{ Can we process Codec data yet? }
		if NE jump codec_abort;

		ax0 = dm(CodecBank);
		dm(page_X12) = ax0;	{*** Select RAM bank for codec ***}

		ax0 = dm(RxBuf+0);	{ Keep codec status }
		dm(LastStatus) = ax0;

	{*** If a user codec command is waiting ,send it now ***}
		ax1 = 0x0FFF;		{ NOP command }
		ax0 = dm(CodecCmdFlag);
		ar  = PASS ax0;
		if eq jump no_codec_cmd;
		ax1 = dm(CodecCmd);
		ax0 = 0;
		dm(CodecCmdFlag) = ax0;
no_codec_cmd:	dm(TxBuf+0) = ax1;	{ New cmd or NOP }

		ax0 = dm(GlobFlags);
		ay0 = GFLG_REC;		{ Determine if playing/recording }
		ar  = ax0 AND ay0;
		if eq jump data_play;

{*** Recording - copy samples to mix buffer as well as looping to DACs ***}
data_rec:
		ax0 = dm(RxBuf+1);	{ L channel input }
		dm(i2,m2) = ax0;	{ Store in channel 0 }
		dm(TxBuf+1) = ax0;	{ Pipe thru to output DAC }

		ax0 = dm(RxBuf+2);	{ R channel input }
		dm(i2,m2) = ax0;	{ Store in channel 1 }
		dm(TxBuf+2) = ax0;	{ Pipe thru to output DAC }

#ifdef NEVER
		i4 = ^LRtargets;
		l4 = %LRtargets;
		m5 = NUMCHANS;

		mx0 = dm(RxBuf+1);	{ L channel input }
		my0 = pm(i4,m5);
		mr  = mx0 * my0 (SU);
		dm(i2,m2) = mr1;	{ Store in channel 0 }
		dm(TxBuf+1) = mr1;	{ Pipe thru to output DAC }

		mx0 = dm(RxBuf+2);	{ R channel input }
		my0 = pm(i4,m5);
		mr  = mx0 * my0 (SU);
		dm(i2,m2) = mr1;	{ Store in channel 1 }
		dm(TxBuf+2) = mr1;	{ Pipe thru to output DAC }
#endif

		cntr = NUMCHANS-2;
		do wrapback until ce;
wrapback:	MODIFY(i2,m2);

		jump codec_end;


{*** Playing - copy samples from mix buffer ***}
data_play:
		i4 = ^LRvolumes;
		l4 = %LRvolumes;
		i3 = ^LRsteps;
		l3 = %LRsteps;
		i6 = ^LRstepcounts;
		l6 = %LRstepcounts;

		ax1 = 0;		{ Zero constant }

{*********** Compute the "left" value ***********}
		cntr = NUMCHANS;
		mr = 0;			{ Clear total }
		do MixLeft until ce;
{*** Update volume envelope for this channel, get new volume ***}
	{ Get fractional parts: volume step & current volume }
		ax0 = dm(i3,m1), ay0 = pm(i4,m7);
	{ Add fractional parts / get low count }
		ar  = ax0 + ay0, ay1 = pm(i6,m4);
	{ Store new fractional volume }
		pm(i4,m4) = ar;
	{ Get integral parts: volume step & current volume }
		ax0 = dm(i3,m1), ay0 = pm(i4,m7);
	{ Add integral parts (+C) / get next sample / get high count }
		ar  = ax0 + ay0 + C, mx0 = dm(i2,m2), ay0 = pm(i6,m6);
	{ Keep current volume }
		my0 = ar;
	{ Store current volume / inc locount }
		pm(i4,m4) = ar, ar = ay1+1;
	{ Keep locount / add carry into hicount }
		pm(i6,m4) = ar, ar = ax1+ay0+C;
	{ Hit zero?, then clear step pair so we plateau }
		if not ac jump noclearL;
		modify(i3,m3);
		dm(i3,m1) = 0;
		dm(i3,m1) = 0;
noclearL:
	{ Keep hicount, apply new volume to sample, add to mix }
MixLeft:	pm(i6,m4) = ar, mr = mr + mx0 * my0 (SU);
		mr = mr (RND);
		if mv sat mr;
		dm(TxBuf+1) = mr1;	{ Send to output DAC }

{*********** Compute the "right" value ***********}
		cntr = NUMCHANS;
		mr = 0;			{ Clear total }
		do MixRight until ce;
{*** Update volume envelope for this channel, get new volume ***}
	{ Get fractional parts: volume step & current volume }
		ax0 = dm(i3,m1), ay0 = pm(i4,m7);
	{ Add fractional parts / get low count }
		ar  = ax0 + ay0, ay1 = pm(i6,m4);
	{ Store new fractional volume }
		pm(i4,m4) = ar;
	{ Get integral parts: volume step & current volume }
		ax0 = dm(i3,m1), ay0 = pm(i4,m7);
	{ Add integral parts (+C) / get next sample / get high count }
		ar  = ax0 + ay0 + C, mx0 = dm(i2,m2), ay0 = pm(i6,m6);
	{ Keep current volume }
		my0 = ar;
	{ Store current volume / inc locount }
		pm(i4,m4) = ar, ar = ay1+1;
	{ Keep locount / add carry into hicount }
		pm(i6,m4) = ar, ar = ax1+ay0+C;
	{ Hit zero?, then clear step pair so we plateau }
		if not ac jump noclearR;
		modify(i3,m3);
		dm(i3,m1) = 0;
		dm(i3,m1) = 0;
noclearR:
	{ Keep hicount, apply new volume to sample, add to mix }
MixRight:	pm(i6,m4) = ar, mr = mr + mx0 * my0 (SU);
		mr = mr (RND);
		if mv sat mr;
		dm(TxBuf+2) = mr1;	{ Send to output DAC }


codec_end:
		MODIFY(i2,m1);		{ Bump to next sample }

		ay0 = dm(SampleCount);
		ar  = ay0+1;
		dm(SampleCount) = ar;

		ay0 = SAMPLES;
		ar  = ar - ay0;		{ Time to switch buffers? }
		if NOT AC jump noswitch;

		ax0 = 0;
		dm(SampleCount) = ax0;	{ Reset counter }
		i2 = ^ChansBuf;		{ At start of each channel buffer }

{*** Toggle buffers for Codec ***}
		ax0 = dm(CodecBank);
		ay0 = BANKBIT;
		ar  = ax0 XOR ay0;
		dm(CodecBank) = ar;
noswitch:

		ax0 = dm(CodecBank);
		dm(page_X12) = ax0;	{ Restore RAM bank }

codec_abort:
		l4 = dm(l4_store);	{ Restore this }
		l6 = dm(l6_store);	{ Restore this }
		i3 = dm(i3_store);	{ Restore this }
		i4 = dm(i4_store);	{ Restore this }
		i6 = dm(i6_store);	{ Restore this }

		DIS sec_reg;		{ Restore to primary registers }
		rti;

{**********************************}
{* Codec initialization interrupt *}
{**********************************}

init_cmd:	ENA sec_reg;
		ar = dm(InitFlag);
		ar = pass ar;
		if eq jump no_init;

		ax0 = pm(i4,m4);	{ Read next command }
		dm(TxBuf) = ax0;	{ Place in Tx buffer }
		ax0 = i4;		{ Read current pos in cmd buffer }
		ay0 = ^InitCmdList;	{ Start location of cmd buffer }
		ar = ax0 - ay0;
		if ne jump still_init;	{ Wrapped around? }
		ax0 = 0;
		dm(InitFlag) = ax0;	{ Clear status flag }

still_init:
no_init:	DIS sec_reg;
		rti;

.ENDMOD;
