/*****************************************************************/
/* Flyer Audio DSP Code (For rev 4)                              */
/*                                                               */
/* Written for ADSP-2115 and AD1847.                             */
/*                                                               */
/* Performs Audio playback and record at 16-bit linear, 44.1 KHz */
/*                                                               */
/*                                                               */
/* 08-26-94       Marty Flickinger                               */
/*                                                               */
/*****************************************************************/
/* Program to test DMA operations and IRQs                       */

.MODULE/ABS=0/BOOT=0  AD1847;

.CONST READMASK	= B#00000001;
.CONST WRITEMASK= B#00000010;
.CONST REGBASE	= 0x37F0;

.var/pm/ram/circ	InitCmdList[14];	/* codec init command list */

.var/dm/ram/circ	rx_buf[3];	/* Status + L data + R data */
.var/dm/ram/circ	tx_buf[3];	/* Cmd + L data + R data */

.var/dm/ram		init_flag;	/* True while initializing codec */
.var/dm/ram		dmacount;
.var/dm/ram		cnt;
.var/dm/ram		Bank;

/*** Orca DMA controller registers ***/
.PORT		wrdcnt;		/* R/W */
.PORT		blkcnt;		/* R/W */
.PORT		endblk;		/* R/W */
.PORT		dmactrl;	/* R/W */
{
.PORT		spare4;
.PORT		spare5;
.PORT		spare6;
.PORT		spare7;
}

/*** CPU Serial registers ***/
.PORT		Xreg0;		/* R=Cpu0,W=Dma0 */
.PORT		Xreg1;		/* R=Cpu1,W=Dma1 */
.PORT		Xreg2;		/* R=Cpu2,W=Dma2 */
.PORT		page_X12;	/* R/W */
{
.PORT		spareC;
.PORT		spareD;
.PORT		spareE;
.PORT		spareF;
}

/********************************/
/* Initial codec setup commands */
/********************************/

.init InitCmdList:
{ Data must be padded with 8 '0' bits to account for PM<-->DM transfer }
 0x400100,	/* Left Input Control: 0 gain, Line 1 input          */
 0x410100,	/* Right Input Control: 0 gain, Line 1 input         */
 0x428000,	/* Left Aux #1 Input Control: Muted                  */
 0x438000,	/* Right Aux #1 Input Control: Muted                 */
 0x448000,	/* Left Aux #2 Input Control: Muted                  */
 0x458000,	/* Right Aux #2 Input Control: Muted                 */
 0x468000,	/* Left DAC Control: 0 attenuation, muted            */
 0x478000,	/* Right DAC Control: 0 attenuation, muted           */
 0x485B00,	/* Data Fmt: XTAL2, 44.1kHz, stereo, 16-bit lin PCM  */
 0x490900,	/* Interface Config: Playback enabled, ACAL allowed  */
 0x4A0000,	/* Pin Control: CLKOUT active, XCTL1/0 LO            */
 0x4C4000,	/* Misc. Info: Transmit on 0,1,2, 32-word frame      */
 0x4D0000,	/* Digital Mix Control: DME Disabled, 0 attenuation  */
 0x0FFF00;	/* End */

/**********************/
/*  Interrupt Vectors */
/**********************/

		jump Start;			/* Restart */
		nop;
		nop;
		nop;

		jump field_irq;			/* IRQ2 - new video field */
		nop;
		nop;
		nop;

		ar = dm(init_flag);		/* SPORT0 Tx - codec */
		ar = pass ar;
		if eq rti; 
		jump NextCmd;

		jump codec_irq;			/* SPORT0 Rx - codec */
		nop;
		nop;
		nop;

		jump dma_irq;			/* IRQ1 - DMA done */
		nop;
		nop;
		nop;

		jump cmd_irq;			/* IRQ0 - New Command */
		nop;
		nop;
		nop;

		rti;				/* Timer irq */
		nop;
		nop;
		nop;


/*****************/
/* Program start */
/*****************/

Start:
		ENA M_MODE;		/* Integer multiplies */

		ax0 = 0x2000;
		dm(0x3ffe) = ax0;	/* 2 wait states for Orca DMA */

		i0 = ^rx_buf;
		l0 = %rx_buf;
		i1 = ^tx_buf;
		l1 = %tx_buf;
		m1 = 1;

		i4 = ^InitCmdList;
		l4 = %InitCmdList;
		m4 = 1;

/*** Initialize Tx buffer ***/

		ax0=0xCFFF;		/* Initially set MCE */
		dm(i1,m1)=ax0;
		ax0=0x0000;
		dm(i1,m1)=ax0;
		dm(i1,m1)=ax0;

/*******************************************************/
/* SPORT0 setup: multichannel enable, ext sclk, MFD=1, */
/* 32 words, ext. rfs, DTYPE 00, 16 bits               */
/*******************************************************/ 

		ax0 = b#1000011000001111;
		dm(0x3ff6) = ax0;

/******************************/
/* Multichannel enable setup: */
/*    Rx: 0, 1, 2, 16, 17, 18 */
/*    Tx: 0, 1, 2, 16, 17, 18 */
/******************************/

		ax0 = b#0000000000000111;
		dm(0x3ff9) = ax0;
		dm(0x3ffa) = ax0;

		ax0 = b#0000000000000111;
		dm(0x3ff7) = ax0;
		dm(0x3ff8) = ax0;

/**************************/
/* SPORT Autobuffer Setup */
/*    Rx: I0, M1          */
/*    Tx: I1, M1          */
/**************************/

		ax0 = b#0000001010000111;
		dm(0x3ff3) = ax0;

		ax0 = b#0001000000000000;
		dm(0x3fff) = ax0;		/* SPORT0 Enabled,
						   SPORT1 = FI, FO, IRQs */

		ifc = b#000000111111;		/* Clear all interrupts */
		nop;
		
		ax0 = 0;
		dm(Bank) = ax0;
		dm(page_X12) = ax0;

		ax0 = 1;
		dm(init_flag) = ax0;		/* Show we're init'ing codec */

		imask = b#010000;		/* enable Tx0 interrupt */

		/* Why sending word to sport??? */
		ax0 = dm(i1,m1);
		tx0 = ax0;


/*** Wait for codec initialization to be completed ***/
		
wait_init:	ax0 = dm(init_flag);		/* wait for entire init */
		af = pass ax0;			/* buffer to be sent to */
		if ne jump wait_init;		/* the codec            */

#ifdef NEVER
/*** Wait for codec to start and complete its autocalibration ***/

		ay0 = b#0000000000000010;	/* Mask for ACI bit */

		idle;
		idle;

wait_cal_start:	idle;
		ax0 = dm(rx_buf);
		ar = ax0 AND ay0;	    	/* Started yet? */
		if eq jump wait_cal_start;

wait_cal_done:	idle;
		ax0 = dm(rx_buf);
		ar = ax0 and ay0;	    	/* Finished? */
		if ne jump wait_cal_done;
#endif

		ax0 = 0;
		dm(Bank) = ax0;
		dm(page_X12) = ax0;

/*** Wait a bit, so as not to sound nasty (why???) ***/

		cntr = 5000;
waiter:		idle;
		if not ce jump waiter;


/*** Remove mute from output DACs ***/

		idle;
		ax0 = 0x0600;
		dm(tx_buf) = ax0;		/* Unmute left DAC */

		idle;
		ax0 = 0x0700;
		dm(tx_buf) = ax0;		/* Unmute right DAC */

		idle;

		ifc = b#000000111111;		/* Clear all interrupts */
		nop;

		imask = b#101110;		/* Enable Rx0,Field,Cmd,Dma IRQs */

		icntl = b#00111;	/* No nesting, edge sensitive */

/***********************/
/* Mixer here...       */
/***********************/

main:
		ax0 = 0;
		dm(Xreg1) = ax0;
		dm(dmacount) = ax0;

CmdLoop:
		ax0 = dm(rx_buf);	/* Status word */
		dm(Xreg0) = ax0;	/* Send to cpu */

		i2 = 0;
		m2 = 0;
		l2 = 0;

		ax0=dm(Xreg0);
		ar=pass ax0;
		if ne jump CmdLoop;
{		dm(Xreg0)=ax0; }		{ Mirror cmd in xfer reg }
WaitCmd:
		ax0=dm(Xreg0);
		ar=pass ax0;
		if eq jump WaitCmd;
{		dm(Xreg0)=ax0; }		{ Mirror cmd in xfer reg }
Read:
		ay0=READMASK;		{ Test for read command }
		ar=ax0 - ay0;
		if ne jump Write;
		ax0=dm(Xreg1);		{ Read address }
{		dm(Xreg1)=ax0; }		{ Mirror adr in xfer reg }
		ay0=REGBASE;		{ Calculate physical address }
		ar=ax0 + ay0;
		i2=ar;
		ax0=dm(i2,m2);		{ Read from physical address }
		dm(Xreg2)=ax0;		{ Place result in xfer reg }
		jump CmdLoop;
Write:
		ay0=WRITEMASK;
		ar=ax0 - ay0;
		if ne jump CmdLoop;
		ax0=dm(Xreg1);		{ Read address }
{		dm(Xreg1)=ax0; }		{ Mirror adr in xfer reg }
		ay0=REGBASE;		{ Calculate physical address }
		ar=ax0 + ay0;
		i2=ar;
		ax0=dm(Xreg2);		{ Read value to write }
		dm(i2,m2)=ax0;		{ Write to physical address }
		dm(Xreg2)=ax0;		{ Mirror data in xfer reg }
		jump CmdLoop;
		

/******************************************************************/
/* Field interrupt handler -- called at the top of each field     */
/******************************************************************/

field_irq:
		ENA sec_reg;

		DIS sec_reg;
		rti;

/******************************************************************/
/* Codec interrupt handler -- called after each sample transacted */
/******************************************************************/

codec_irq:
		ENA sec_reg;

		ax0 = dm(rx_buf+1);	/* L channel input */
		dm(tx_buf+1) = ax0;

		ax0 = dm(rx_buf+2);	/* R channel input */
		dm(tx_buf+2) = ax0;

		DIS sec_reg;
		rti;

/**********************************/
/* Codec initialization interrupt */
/**********************************/


NextCmd:	ENA sec_reg;

		ax0 = pm(i4,m4);	/* Read next command */
		dm(tx_buf) = ax0;	/* Place in Tx buffer */
		ax0 = i4;		/* Read current pos in cmd buffer */
		ay0 = ^InitCmdList;	/* Start location of cmd buffer */
		ar = ax0 - ay0;
		if ne jump still_init;	/* Wrapped around? */
		ax0 = 0;
		dm(init_flag) = ax0;	/* Done with init */

still_init:	DIS sec_reg;
		rti;


/**********************************/
/* "Xreg0 Written" Interrupt      */
/**********************************/

cmd_irq:	ENA sec_reg;		{ Use secondary registers }

		ax0 = dm(Xreg0);

		DIS sec_reg;
		rti;

dma_irq:
		ENA sec_reg;

		{ Interrupt 68000 }
		ax0 = 0x80;
		dm(page_X12) = ax0;


		ay0 = dm(dmacount);
		ar = ay0+1;
		dm(dmacount) = ar;
		dm(Xreg1) = ar;

		DIS sec_reg;
		rti;

.endmod;
