{*****************************************************************}
{* Flyer Audio DSP Test Code (For rev 4)                         *}
{*                                                               *}
{* Written for ADSP-2115 and AD1847.                             *}
{*                                                               *}
{*                                                               *}
{* 11-15-94       Marty Flickinger                               *}
{*                                                               *}
{*****************************************************************}

.MODULE/ABS=0/BOOT=0  AD1847;

.INCLUDE <..\include\DEF2101.H>;

{**************** Constants ****************}

.CONST CHANBLKSIZE	= 3;
.CONST CHANBUFSIZE	= CHANBLKSIZE * 256;

{**************** Variables ****************}

{ Codec initialization command list array }
.VAR/PM/RAM/CIRC	InitCmdList[14];

{ Codec Rx autobuffer: Status + L data + R data }
.VAR/DM/RAM/CIRC	RxBuf[3];

{ Codec Tx autobuffer: Cmd + L data + R data }
.VAR/DM/RAM/CIRC	TxBuf[3];


.VAR/DM/RAM		InitFlag;	{ True while initializing codec }
.VAR/DM/RAM		LastStatus;	{ Codec status store }
.VAR/DM/RAM		CodecCmd;	{ Codec command addr/data }
.VAR/DM/RAM		CodecCmdFlag;	{ Codec command waiting }
.VAR/DM/RAM		DebugAddr;	{ FOR DEBUGGING }
.VAR/DM/RAM		BankPicked;	{ User's choice of bank }
.VAR/DM/RAM		statbits;	{ Mirror of status bits to CPU }
.VAR/DM/RAM		TestSRAM;	{ Flag: do SRAM test }
.VAR/DM/RAM		UpperAddr;	{ Upper addr for test }

.CONST STAT_ACK	= B#00000001;

{*** Orca DMA controller registers ***}
.PORT	wrdcnt;		{ R/W }
.PORT	blkcnt;		{ R/W }
.PORT	endblk;		{ R/W }
.PORT	dmactrl;	{ R/W }
{
.PORT	spare4;
.PORT	spare5;
.PORT	spare6;
.PORT	spare7;
}

{*** CPU Serial registers ***}
.PORT	Xreg0;		{ R=Cpu0,W=Dma0 }
.PORT	Xreg1;		{ R=Cpu1,W=Dma1 }
.PORT	Xreg2;		{ R=Cpu2,W=Dma2 }
.PORT	page_X12;	{ R=Cpu1/2,W=RAM bank select bit }
.CONST		BANKBIT	= 1;		{ Bit value for RAM bank select bit }
{
.PORT	spareC;
.PORT	spareD;
.PORT	spareE;
.PORT	spareF;
}


{********************************}
{* Initial codec setup commands *}
{********************************}

.INIT InitCmdList:
 0xC00100,	{ Left Input Control: 0 gain, Line 1 input          }
 0xC10100,	{ Right Input Control: 0 gain, Line 1 input         }
 0xC28000,	{ Left Aux #1 Input Control: Muted                  }
 0xC38000,	{ Right Aux #1 Input Control: Muted                 }
 0xC48000,	{ Left Aux #2 Input Control: Muted                  }
 0xC58000,	{ Right Aux #2 Input Control: Muted                 }
 0xC68000,	{ Left DAC Control: 0 attenuation, muted            }
 0xC78000,	{ Right DAC Control: 0 attenuation, muted           }
 0xC85B00,	{ Data Fmt: XTAL2, 44.1kHz, stereo, 16-bit lin PCM  }
 0xC90900,	{ Interface Config: Playback enabled, ACAL allowed  }
 0xCA0000,	{ Pin Control: CLKOUT active, XCTL1/0 LO            }
 0xCC4000,	{ Misc. Info: Transmit on 0,1,2, 32-word frame      }
 0xCD0000,	{ Digital Mix Control: DME Disabled, 0 attenuation  }
 0x8FFF00;	{ End }

{**********************}
{*  Interrupt Vectors *}
{**********************}

		jump Start;			{ Restart }
		nop;
		nop;
		nop;

		jump field_irq;			{ IRQ2 - new video field }
		nop;
		nop;
		nop;

		jump init_cmd;			{ SPORT0 Tx - codec }
		nop;
		nop;
		nop;

		jump codec_irq;			{ SPORT0 Rx - codec }
		nop;
		nop;
		nop;

		jump dma_irq;			{ IRQ1 - DMA done }
		nop;
		nop;
		nop;

		jump cmd_irq;			{ IRQ0 - New Command }
		nop;
		nop;
		nop;

		rti;				{ Timer irq }
		nop;
		nop;
		nop;


{*****************}
{* Program start *}
{*****************}

Start:
		ENA M_MODE;		{ Integer multiplies }

		ax0 = 0x2249;
		ax0 = 0x2000;
		dm(Dm_Wait_Reg) = ax0;	{ 2 wait states for Orca DMA }

		i0 = ^RxBuf;
		l0 = %RxBuf;
		i1 = ^TxBuf;
		l1 = %TxBuf;
		m1 = 1;

		i4 = ^InitCmdList;
		l4 = %InitCmdList;
		m4 = 1;

{*** Initialize Tx buffer ***}

		ax0=0xCFFF;		{ Initially set MCE }
		dm(i1,m1)=ax0;
		ax0=0x0000;
		dm(i1,m1)=ax0;
		dm(i1,m1)=ax0;

{*******************************************************}
{* SPORT0 setup: multichannel enable, ext sclk, MFD=1, *}
{* 32 words, ext. rfs, DTYPE 00, 16 bits               *}
{*******************************************************} 

		ax0 = b#1000011000001111;
		dm(Sport0_Ctrl_Reg) = ax0;

{******************************}
{* Multichannel enable setup: *}
{*    Rx: 0, 1, 2, 16, 17, 18 *}
{*    Tx: 0, 1, 2, 16, 17, 18 *}
{******************************}

		ax0 = b#0000000000000111;
		dm(Sport0_Rx_Words0) = ax0;
		dm(Sport0_Rx_Words1) = ax0;

		ax0 = b#0000000000000111;
		dm(Sport0_Tx_Words0) = ax0;
		dm(Sport0_Tx_Words1) = ax0;

{**************************}
{* SPORT Autobuffer Setup *}
{*    Rx: I0, M1          *}
{*    Tx: I1, M1          *}
{**************************}

		ax0 = b#0000001010000111;
		dm(Sport0_Autobuf_Ctrl) = ax0;

		ax0 = b#0001000000000000;
		dm(Sys_Ctrl_Reg) = ax0;		{ SPORT0 Enabled,
						   SPORT1 = FI, FO, IRQs }

		ifc = b#000000111111;		{ Clear all interrupts }
		nop;
		
		ax0 = 1;
		dm(InitFlag) = ax0;		{ Show we're init'ing codec }

		imask = b#010000;		{ enable Tx0 interrupt }

		{ Why sending word to sport??? }
		ax0 = dm(i1,m1);
		tx0 = ax0;

{*** Wait for codec initialization to be completed ***}
		
wait_init:	ax0 = dm(InitFlag);		{ wait for entire init }
		af = pass ax0;			{ buffer to be sent to }
		if ne jump wait_init;		{ the codec            }


{*** Do more system initialization ***}

		ax0 = 0;
		dm(statbits) = ax0;
		dm(Xreg0) = ax0;		{ Clear status bits }
		dm(CodecCmdFlag) = ax0;		{ No command waiting }
		dm(TestSRAM) = ax0;		{ No test yet }

		m4 = 1;
		m7 = 1;
		l7 = 0;

		ax0 = 0;
		dm(BankPicked) = ax0;
		dm(page_X12) = ax0;	{ Select bank }


{*** Wait a bit, so as not to sound nasty (why???) ***}

		cntr = 5000;
waiter:		idle;
		if not ce jump waiter;


{*** Remove mute from output DACs ***}

		idle;

		ax0 = 0x8600;
		dm(TxBuf) = ax0;		{ Unmute left DAC }
		idle;

		ax0 = 0x8700;
		dm(TxBuf) = ax0;		{ Unmute right DAC }
		idle;

		{*** Reset Audio DMA ***}
		ax1 = B#00000000;
		dm(dmactrl) = ax1;

		icntl = b#00111;		{ No nesting, edge sensitive }

		ifc = b#000000111111;		{ Clear all interrupts }
		nop;
		imask = b#101110;		{ Enable Rx0,Field,Cmd,Dma IRQs }


{*****************************}
{*         Main loop         *}
{*****************************}

main:		ax0 = dm(TestSRAM);
		ar  = PASS ax0;
		if eq jump main;
		call	DoSRAMtest;
		ax0 = 0;
		dm(TestSRAM) = ax0;
		jump	main;
		

{***************}
{* Subroutines *}
{***************}


{*********************************************************************}
{* Command interrupt handler -- called when command register written *}
{*********************************************************************}
cmd_irq:	ENA sec_reg;		{ Use secondary registers }

		ax1 = dm(Xreg0);
		ay0 = B#11110000;
		ar  = ax1 AND ay0;	{ Get just command type }
		sr  = LSHIFT ar BY -4 (LO);
		ay1 = ^jump_table;
		ar  = sr0 + ay1;
		i7  = ar;

		ay0 = B#00001111;
		ar  = ax1 AND ay0;
		ax0 = ar;		{ ax0 = arg }

		jump (i7);

jump_table:	jump no_cmd;
		jump selftest;
		jump do_nop;
		jump do_nop;
		jump do_nop;
		jump write_codec;
		jump read_codec;
		jump do_nop;
		jump dma_read;
		jump dma_write;
		jump dbg_bank;
		jump dbg_readdm;
		jump dbg_readpm;
		jump dbg_writedm;
		jump dbg_writepm;
		jump dbg_addr;

{*** Debugging - do various tests ***}
selftest:	ax1 = dm(page_X12);		{ Read test type }
		dm(UpperAddr) = ax1;
		ax1 = 1;
		dm(TestSRAM) = ax1;		{ Start test soon }
		jump no_cmd;		{ No ACK yet! }

{*** Debugging - store address ***}
dbg_addr:	ax0 = dm(page_X12);		{ Read word }
		dm(DebugAddr) = ax0;
		jump handshake;

dbg_bank:	ax0 = dm(page_X12);		{ Read word }
		dm(BankPicked) = ax0;
		dm(page_X12) = ax0;		{ Select bank }
		jump handshake;

{*** Debugging - read address ***}
dbg_readdm:	i7 = dm(DebugAddr);
		si = dm(i7,m7);
		jump r_merge;
dbg_readpm:	i7 = dm(DebugAddr);
		si = pm(i7,m7);
r_merge:
		dm(DebugAddr) = i7;
		dm(Xreg2) = si;		{ Lo 8 }
		sr = LSHIFT si BY -8 (LO);
		dm(Xreg1) = sr0;	{ Hi 8 }
		jump handshake;

{*** Debugging - write address ***}
dbg_writedm:	i7 = dm(DebugAddr);
		ax0 = dm(page_X12);		{ Read word }
		dm(i7,m7) = ax0;
		dm(DebugAddr) = i7;
		jump handshake;
dbg_writepm:	i7 = dm(DebugAddr);
		ax0 = dm(page_X12);		{ Read word }
		pm(i7,m7) = ax0;
		dm(DebugAddr) = i7;
		jump handshake;


{*** Write codec register ***}
write_codec:	ax0 = dm(page_X12);
		dm(CodecCmd) = ax0;	{ Pipe to codec next time }
		ax0 = 1;
		dm(CodecCmdFlag) = ax0;
		jump handshake;

{*** Read most recent codec status register ***}
read_codec:	ax0 = dm(LastStatus);
		dm(Xreg1) = ax0;	{ Write to CPU register }
		jump handshake;

{*** Initiate DMA from DSP RAM to DMA RAM ***}
dma_read:	ay1 = B#00010011;
		jump dma_join;

{*** Initiate DMA from DMA RAM to DSP RAM ***}
dma_write:	ay1 = B#00010001;
dma_join:
		{*** Un-Reset Audio DMA ***}
		ax1 = B#00010000;
		dm(dmactrl) = ax1;

		{*** Turn channel # into a block address ***}
		mx0 = ax0;
		my0 = CHANBLKSIZE;
		mr  = mx0 * my0 (UU);
		ar  = mr0;

		{*** If user picked 2nd page, add offset to it ***}
		ax0 = dm(BankPicked);
		af  = PASS ax0;
		ay0 = 0x40;
		if NE ar = ar + ay0;

		dm(blkcnt) = ar;		{ Set DMA start blk }

		{*** Compute DMA end block ***}
		ay0 = dm(Xreg2);
		af  = ar + ay0;
		ar  = af-1;
		dm(endblk) = ar;		{ Write to DMA }

		{*** Enable DMA ***}
		dm(dmactrl) = ay1;	{ Use direction bit set earlier }

		jump handshake;

handshake:
do_nop:
		{*** Send acknowledge to CPU ***}
		ax0 = dm(statbits);
		ay0 = STAT_ACK;
		ar  = ax0 XOR ay0;
		dm(statbits) = ar;
		dm(Xreg0) = ar;

no_cmd:		{ No handshake for this?!!! }
		DIS sec_reg;		{ Restore to primary registers }
		rti;

{******************************************************}
{* DMA interrupt handler -- called when DMA completes *}
{******************************************************}
dma_irq:	ENA sec_reg;		{ Use secondary registers }

		{*** Clr IRQ, Reset Audio DMA ***}
		ax1 = B#00000000;
		dm(dmactrl) = ax1;

		{ Interrupt 68000  -- CPU will clear when serviced }
		ax0 = dm(BankPicked);
		ay0 = 0x80;
		ar  = ax0 OR ay0;
		dm(page_X12) = ar;	{ Set CPU IRQ flag }

		DIS sec_reg;		{ Restore to primary registers }
		rti;

{********************************************************************}
{* Field interrupt handler -- called at the top of each video field *}
{********************************************************************}

field_irq:	ENA sec_reg;		{ Use secondary registers }


		DIS sec_reg;		{ Restore to primary registers }
		rti;


{******************************************************************}
{* Codec interrupt handler -- called after each sample transacted *}
{******************************************************************}

codec_irq:	ENA sec_reg;		{ Use secondary registers }

		ax0 = dm(RxBuf+0);	{ Keep codec status }
		dm(LastStatus) = ax0;

	{*** If a user codec command is waiting ,send it now ***}
		ax1 = 0x0FFF;		{ NOP command }
		ax0 = dm(CodecCmdFlag);
		ar  = PASS ax0;
		if eq jump no_codec_cmd;
		ax1 = dm(CodecCmd);
		ax0 = 0;
		dm(CodecCmdFlag) = ax0;
no_codec_cmd:	dm(TxBuf+0) = ax1;	{ New cmd or NOP }


{*** Recording - loop data back to DACs ***}
data_rec:
		ax0 = dm(RxBuf+1);	{ L channel input }
		dm(TxBuf+1) = ax0;	{ Pipe thru to output DAC }

		ax0 = dm(RxBuf+2);	{ R channel input }
		dm(TxBuf+2) = ax0;	{ Pipe thru to output DAC }

		DIS sec_reg;		{ Restore to primary registers }
		rti;

{**********************************}
{* Codec initialization interrupt *}
{**********************************}

init_cmd:	ENA sec_reg;
		ar = dm(InitFlag);
		ar = pass ar;
		if eq jump no_init;

		ax0 = pm(i4,m4);	{ Read next command }
		dm(TxBuf) = ax0;	{ Place in Tx buffer }
		ax0 = i4;		{ Read current pos in cmd buffer }
		ay0 = ^InitCmdList;	{ Start location of cmd buffer }
		ar = ax0 - ay0;
		if ne jump still_init;	{ Wrapped around? }
		ax0 = 0;
		dm(InitFlag) = ax0;	{ Clear status flag }

still_init:
no_init:	DIS sec_reg;
		rti;


{******************* MEMORY TEST CODE ***************}
DoSRAMtest:	ay1 = 0;		{ Error count }
		ax1 = dm(UpperAddr);

		CNTR = 2;
		do testboth until ce;

		ay0 = cntr;
		ar  = ay0-1;
		dm(page_X12) = ar;	{ Select bank }

		l7 = 0;

		i7 = dm(DebugAddr);	{ Start address for test }
writepat:	ax0 = i7;
		dm(i7,m1) = ax0;
		ay0 = i7;
		ar  = ax1 - ay0;
		if ne jump writepat;

		i7 = dm(DebugAddr);	{ Start address for test }
readpat:	ax0 = i7;
		ay0 = dm(i7,m1);
		ar  = ax0 - ay0;	{ Same? }
		if eq jump verfok;
		ar  = ay1+1;
		ay1 = ar;
verfok:		ay0 = i7;
		ar  = ax1 - ay0;
		if ne jump readpat;

testboth:	nop;
		si  = ay1;		{ Return error count }
		dm(Xreg2) = si;		{ Lo 8 }
		sr = LSHIFT si BY -8 (LO);
		dm(Xreg1) = sr0;	{ Hi 8 }

		ax0 = dm(BankPicked);
		dm(page_X12) = ax0;	{ Restore previous bank }

		{*** Send acknowledge to CPU ***}
		ax0 = dm(statbits);
		ay0 = STAT_ACK;
		ar  = ax0 XOR ay0;
		dm(statbits) = ar;
		dm(Xreg0) = ar;
		rts;

.ENDMOD;
