{*****************************************************************}
{* Flyer Audio DSP Test Code (For rev 4)                         *}
{*                                                               *}
{* Written for ADSP-2115 and AD1847.                             *}
{*                                                               *}
{*                                                               *}
{* 11-15-94       Marty Flickinger                               *}
{*                                                               *}
{*****************************************************************}

.MODULE/ABS=0/BOOT=0  AD1847;

.INCLUDE <..\include\DEF2101.H>;

{**************** Constants ****************}

{**************** Variables ****************}

{ Codec Rx autobuffer: Status + L data + R data }
.VAR/DM/RAM/CIRC	RxBuf[3];

{ Codec Tx autobuffer: Cmd + L data + R data }
.VAR/DM/RAM/CIRC	TxBuf[3];


{*** Orca DMA controller registers ***}
.PORT	wrdcnt;		{ R/W }
.PORT	blkcnt;		{ R/W }
.PORT	endblk;		{ R/W }
.PORT	dmactrl;	{ R/W }
{
.PORT	spare4;
.PORT	spare5;
.PORT	spare6;
.PORT	spare7;
}

{*** CPU Serial registers ***}
.PORT	Xreg0;		{ R=Cpu0,W=Dma0 }
.PORT	Xreg1;		{ R=Cpu1,W=Dma1 }
.PORT	Xreg2;		{ R=Cpu2,W=Dma2 }
.PORT	page_X12;	{ R=Cpu1/2,W=RAM bank select bit }
.CONST		BANKBIT	= 1;		{ Bit value for RAM bank select bit }
{
.PORT	spareC;
.PORT	spareD;
.PORT	spareE;
.PORT	spareF;
}

{**********************}
{*  Interrupt Vectors *}
{**********************}

		jump Start;			{ Restart }
		nop;
		nop;
		nop;

		rti;			{ IRQ2 - new video field }
		nop;
		nop;
		nop;

		rti;			{ SPORT0 Tx - codec }
		nop;
		nop;
		nop;

		rti;			{ SPORT0 Rx - codec }
		nop;
		nop;
		nop;

		rti;			{ IRQ1 - DMA done }
		nop;
		nop;
		nop;

		rti;			{ IRQ0 - New Command }
		nop;
		nop;
		nop;

		rti;				{ Timer irq }
		nop;
		nop;
		nop;


{*****************}
{* Program start *}
{*****************}

Start:
		ENA M_MODE;		{ Integer multiplies }

		ax0 = 0x2249;
		ax0 = 0x2000;
		dm(Dm_Wait_Reg) = ax0;	{ 2 wait states for Orca DMA }

		i0 = ^RxBuf;
		l0 = %RxBuf;
		i1 = ^TxBuf;
		l1 = %TxBuf;
		m1 = 1;

{*** Initialize Tx buffer ***}

		m2 = 1;

		ax0=0xCFFF;		{ Initially set MCE }
		dm(i1,m2)=ax0;
		ax0=0x0000;
		dm(i1,m2)=ax0;
		dm(i1,m2)=ax0;

{*******************************************************}
{* SPORT0 setup: multichannel enable, ext sclk, MFD=1, *}
{* 32 words, ext. rfs, DTYPE 00, 16 bits               *}
{*******************************************************} 

		ax0 = b#1000011000001111;
		dm(Sport0_Ctrl_Reg) = ax0;
		dm(Sport1_Ctrl_Reg) = ax0;

{******************************}
{* Multichannel enable setup: *}
{*    Rx: 0, 1, 2, 16, 17, 18 *}
{*    Tx: 0, 1, 2, 16, 17, 18 *}
{******************************}

		ax0 = b#0000000000000111;
		dm(Sport0_Rx_Words0) = ax0;
		dm(Sport0_Rx_Words1) = ax0;

		ax0 = b#0000000000000111;
		dm(Sport0_Tx_Words0) = ax0;
		dm(Sport0_Tx_Words1) = ax0;

{**************************}
{* SPORT Autobuffer Setup *}
{*    Rx: I0, M1          *}
{*    Tx: I1, M1          *}
{**************************}

		ax0 = b#0000001010000111;
		dm(Sport0_Autobuf_Ctrl) = ax0;

		ax0 = b#0000001010000100;
		dm(Sport1_Autobuf_Ctrl) = ax0;

{		ax0 = b#0000000000000000; }
		ax0 = b#0001000000000000;
		dm(Sys_Ctrl_Reg) = ax0;		{ SPORT0 Enabled,
						   SPORT1 = FI, FO, IRQs }

		imask = b#000000;		{ Enable Rx0,Field,Cmd,Dma IRQs }
		ifc = b#000000111111;		{ Clear all interrupts }
		nop;
		

{*** Do more system initialization ***}

		ax0 = 0;
		dm(Xreg0) = ax0;		{ Clear status bits }

		ax0 = 0;
		dm(page_X12) = ax0;	{ Select bank }

		{*** Reset Audio DMA ***}
		ax1 = B#00000000;
		dm(dmactrl) = ax1;

		icntl = b#00111;		{ No nesting, edge sensitive }

		imask = b#000000;		{ Enable Rx0,Field,Cmd,Dma IRQs }
		ifc = b#000000111111;		{ Clear all interrupts }
		nop;



{******************* MEMORY TEST CODE ***************}
DoSRAMtest:	ay1 = 0;		{ Error count }
		ax1 = 0x37C0;

		CNTR = 2;
		do testboth until ce;

		ay0 = cntr;
		ar  = ay0-1;
		dm(page_X12) = ar;	{ Select bank }

		l7 = 0;
		m4 = 0;

		i7 = 0x3400;
writepat:	ax0 = i7;
		ar  = dm(Xreg1);		{ Reg read }
		dm(i7,m4) = ax0;
		ar  = dm(Xreg1);		{ Reg read }
		ay0 = i7;
		ar  = ax1 - ay0;
		if ne jump writepat;

		i7 = 0x3400;
readpat:	ax0 = i7;
		ar  = dm(Xreg1);		{ Reg read }
		ay0 = dm(i7,m4);
		ar  = dm(Xreg1);		{ Reg read }
		ar  = ax0 - ay0;	{ Same? }
		if eq jump verfok;
		ar  = ay1+1;
		ay1 = ar;
verfok:		ay0 = i7;
		ar  = ax1 - ay0;
		if ne jump readpat;

testboth:	nop;

		jump	DoSRAMtest;

.ENDMOD;
