/*********************************************************************\
*
* SetTimeCode - Set a clip's starting TimeCode
*
* $Id: SetTimeCode.c,v 1.0 1995/10/16 17:16:54 Flick Exp $
*
* $Log: SetTimeCode.c,v $
*Revision 1.0  1995/10/16  17:16:54  Flick
*FirstCheckIn
*
*
* Copyright (c) 1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* All information contained herein is property of
* NewTek, Inc. and is subject to change without
* notice.
*
*	10/16/95		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <string.h>
#include <stdio.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include "flyerlib.h"
#include "flyer.h"
#include "flyerClipFmt.h"


struct Library *FlyerBase;
struct FrameHeader	HdrBuff;



/* Prototypes */
ULONG SetTimeCode(char *pathname, ULONG field, ULONG timecode);
UBYTE Binary2BCD(UBYTE bin);


main(int argc,char **argv)
{
	char	*ptr;
	ULONG	err,timecode;
	int	hh,mm,ss,ff;

	if ((argc!=3)||(argv[1][0]=='?'))
	{
		printf("Usage: %ls <pathname> <xx:xx:xx:xx>\n",argv[0]);
		return(20);
	}

	ptr = argv[2];

	stcd_i(&ptr[0],&hh);
	stcd_i(&ptr[3],&mm);
	stcd_i(&ptr[6],&ss);
	stcd_i(&ptr[9],&ff);

	/******************************************************************/
	/** NOTE: the formula below does not do drop-frame conversion    **/
	/** Neither does the code herein that rolls our own Horita       **/
	/** SMPTE codes.  This example assumes that the time code        **/
	/** supplied is already drop-frame.  Converting drop-frame       **/
	/** to/from non-drop frame is an excercise left to the developer **/
	/******************************************************************/
	timecode = ((hh*60+mm)*60+ss)*30+ff;

	if (FlyerBase = OpenLibrary(FLYERLIBNAME,0))
	{
		err = SetTimeCode(argv[1],0,timecode);	// Set timecode for first field of clip

		CloseLibrary(FlyerBase);
	}
	else
		printf("Open flyer.library failed\n");

	return(0);
}


ULONG SetTimeCode(char *pathname, ULONG field, ULONG timecode)
{
	struct FlyerVolume FlyVol,*volptr;
	struct ClipAction	 Action,*actptr;
	ULONG	res;
	int	i;
	struct Field			*fld;
	UBYTE	*buf,hours,minutes,seconds,frames;

	actptr = &Action;
	volptr = &FlyVol;

	volptr->Board = 0;
	volptr->Path = pathname;		// Full path and filename (on Flyer drive)
	volptr->Flags = 0;							// None
	actptr->Volume = volptr;					// Link together
	actptr->ReturnTime = RT_STOPPED;			// Don't return until done
	actptr->VidStartField = field;			// Get header for this field

	res = GetFrameHeader(actptr,&HdrBuff);		// Get it!
	if (res)
	{
		printf("Error %ld - %ls\n",res,Error2String(res));
		return(res);
	}

	fld = &HdrBuff.fields[field % 4];		// Pick out which of the 4 fields we wanted
	buf = &fld->SerData[0];

	printf("This came from %lx\n",actptr->StartBlk);
	printf("Previous serial data (len=%ld): ",fld->SerialDataLen);
	for (i=0; i<36; i++)
		printf("%02lx ",buf[i]);
	printf("\n");


	hours = timecode / (60*60*30);
	minutes = (timecode / (60*30)) % 60;
	seconds = (timecode / 30) % 60;
	frames = timecode % 30;

	buf[0] = 0xF2;				// Use this code to signify drop-frame timecode
//	buf[0] = 0xF0;				// Use this code to signify non-drop timecode
	buf[1] = Binary2BCD(frames);
	buf[2] = Binary2BCD(seconds);
	buf[3] = Binary2BCD(minutes);
	buf[4] = Binary2BCD(hours);

	fld->SerialDataLen = 5;				// Amount of data we placed in buffer
	fld->SerType = SERDEVTYPE_SMPTE;	// This is SMPTE info
	fld->SerMake = SERDEV_HORITA;		//	Horita format

	printf("New serial data (len=%ld): ",fld->SerialDataLen);
	for (i=0; i<36; i++)
		printf("%02lx ",buf[i]);
	printf("\n");


	volptr->Board = 0;
	volptr->Path = pathname;		// Full path and filename (on Flyer drive)
	volptr->Flags = 0;							// None
	actptr->Volume = volptr;					// Link together
	actptr->ReturnTime = RT_STOPPED;			// Don't return until done
	actptr->VidStartField = field;			// Put header for this field

	res = PutFrameHeader(actptr,&HdrBuff);		// Put it back!
	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));

	return(res);
}


UBYTE Binary2BCD(UBYTE bin)
{
	UBYTE	lo,hi;

	hi = bin/10;
	lo = bin%10;

	return((UBYTE)((hi<<4)+lo));
}
