/*********************************************************************\
*
* DumpProject - Dump and document a 4.0 Toaster/Flyer project file
*
* Copyright (c) 1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	08/24/95		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "Tags.h"

/*** Prototypes ***/
void EndIt(void);
void DumpProject(UBYTE *ptr, char *name);
UBYTE *DumpTagList(UBYTE *ptr);
ULONG ReadFile(char *name, UBYTE **bufferptr);

/*** Globals ***/
UBYTE	*ProjBuffer;

FILE *outfile;

main(int argc,char **argv)
{
	if (argc!=3)
	{
		printf("Usage: %ls <ProjFIle> <DumpFile> \n",argv[0]);
		exit(100);
	}

outfile = fopen(argv[2],"w");


	if (ReadFile(argv[1],&ProjBuffer))
	{
		DumpProject(ProjBuffer,argv[1]);
	}

	EndIt();

fclose(outfile);
}

struct ProjHeader	{
	ULONG	ID1,ID2;
	ULONG	Version;
	ULONG	pad[4];
	ULONG	SysTagsSize;
};

struct ProjEntry {
	ULONG	Length;
	ULONG	ObjType;
	ULONG	ObjVersion;
	ULONG	future[4];
	UWORD	NameLength;
	char	Name[0];
};

void DumpProject(UBYTE *ptr, char *name)
{
	struct ProjHeader	*hdr;
	struct ProjEntry	*entry;
	char	typestring[5] = "xxxx";

	fprintf(outfile,"******* Dump of project file \"%ls\" *******\n",name);

	hdr = (struct ProjHeader *)ptr;
	if ((hdr->ID1 != 0x50726f6a) || (hdr->ID2 != 0x65637400))		// 'PROJ' & 'ECT '
	{
		printf("This is not a Toaster/Flyer project file!\n");
		return;
	}

	if (hdr->Version != 0)
	{
		printf("Unknown project file version\n");
		return;
	}

	ptr += sizeof(struct ProjHeader);

	fprintf(outfile,"System Tags\n");
	ptr = DumpTagList(ptr);

	while (entry=(struct ProjEntry *)ptr, entry->Length)
	{
	CopyMem(&entry->ObjType,typestring,sizeof(ULONG));
	fprintf(outfile,"\n SIZE  %ld \n",entry->Length);
	fprintf(outfile,"%5ls  %ls\n",typestring,entry->Name);
	ptr = DumpTagList(ptr+sizeof(struct ProjEntry)+entry->NameLength);
	}

	fprintf(outfile,">>>>>>>>>> End <<<<<<<<<<\n");
}



char *TagNames[] = {
	"???",
	"Version",
	"Revision",
	"AAeffect",
	"NonAAeffect",
	"KillInterfaceNonAA",
	"KillInterfaceAA",
	"ButtonELHlogic",
	"CustomButtonELHlogic",
	"NumberOfAnims",
	"RequestFileName",
	"AnimFiles",
	"Frames",
	"FieldSync",
	"VariableSpeeds",
	"ForcePlayForward",
	"ForcePlayReverse",
	"LoopAnims",
	"AnimControl",
	"AudioFastSamples",
	"AudioMediumSamples",
	"AudioSlowSamples",
	"AudioFiles",
	"AudioControl",
	"PauseTimes",
	"LatchColors",
	"TransparentColors",
	"PaletteColors",
	"Equations",
	"Encoder",
	"VerticalScroll",
	"ReverseTime",
	"ReverseButtLog",
	"ReverseCustomButtLog",
	"KeyMode",
	"MatteColor",
	"CustomMatteColor ",
	"BorderColor",
	"CustomBorderColor",
	"LineNumbers",
	"LineNumberPlane",
	"ForceFreeze4",
	"ForceFreeze8",
	"ForceLive",
	"ForceLumKeyOn",
	"ForceLumKeyOff",
	"ForceLumKeyOnBlack",
	"ForceLumKeyOnWhite",
	"BadDefaultFX",
	"LoadPictures",
	"FadeInDuration",
	"FadeOutDuration",
	"DigitalFX",
	"TimeMode",
	"LUT",
	"DigitalPairs",
	"LatchAM",
	"LatchBM",
	"LatchIS",
	"HonorPreviewOverLay",
	"ForceDefaultMatte",
	"TurnAudioFilterOff",
	"AudioStartField",
	"NumAudioFields",
	"ISandClipPause",
	"Interlaced",
	"FirstFieldNTSCII",
	"BounceILBM",
	"LatchRanges",
	"LatchList",
	"TransparentRanges",
	"TransparentList",
	"Color0Transparent",
	"AbortIfSlow",
	"NumSkipFieldsAtEnd",
	"TBarDoesAuto",
	"DoNotStompSprite",
	"AbortLoopAtEnd",
	"NonAAremap",
	"100PercentWhiteMatte",
	"UseEffectColor",
	"LoopCount",
	"CroutonType",
	"TimeLine",
	"IndexID",
	"FCountMode",
	"VariableFCount",
	"SlowFCount",
	"MedFCount",
	"FastFCount",
	"VariableFCount68000",
	"SlowFCount68000",
	"MedFCount68000",
	"FastFCount68000",
	"NumFields",
	"StartTime",
	"DescriptorList",
	"AboutList",
	"CommentList",
	"AlgoFXtype",
	"AlgoFXborder",
	"NumFramesSlow",
	"NumFramesMedium",
	"NumFramesFast",
	"NumFramesVariable",
	"Page",
	"Speed",
	"Delay",
	"Duration",
	"AudioAttack",
	"AudioDecay",
	"RecFields",
	"AudioOn",
	"AudioStart",
	"AudioDuration",
	"ClipStartField",
	"FadeInVideo",
	"MaxDuration",
	"VideoSource",
	"LoadedSlices",
	"OriginalLocation",
	"AudioVolume1",
	"AudioVolume2",
	"AudioPan1",
	"AudioPan2",
	"PanelMode",
	"ColorMode",
	"CycleMode",
	"DataMode",
	"AdjustedVideoStart",
	"AdjustedVideoDuration",
	"Asynchronous",
	"CommandLine",
	"SMPTEtime",
	"TBarPosition",
	"HoldFields",
	"TakeOffset",
	"ASourceLen",
	"BSourceLen",
	"AudioFadeFlags",
};
#define	TAGTABLEENTRIES	(sizeof(TagNames)/sizeof(char *))


UBYTE *DumpTagList(UBYTE *ptr)
{
	ULONG	*tptr,tagid,tagflags,len;
	char	*tagname;
	UBYTE	*bptr;
	int	i,j;

	tptr = (ULONG *)ptr;

	while (tagflags = *tptr++)
	{
		tagid = tagflags & 0x00FFFFFF;
		if (tagid < TAGTABLEENTRIES)
			tagname = TagNames[tagid];
		else
			tagname = TagNames[0];

		if (TAGCTRL_LONG & tagflags)
		{
			if (*tptr <= 999999)		// Show in hex on really big numbers, else decimal
				fprintf(outfile,"   %ls = %ld",tagname,*tptr);
			else
				fprintf(outfile,"   %ls = $%lx",tagname,*tptr);

			switch (tagid | 0x80000000)			// Must have TAGCTRL_LONG in it to match!
			{
			case TAG_ClipStartField:
			case TAG_Duration:
			case TAG_AudioStart:
			case TAG_AudioDuration:
			case TAG_AdjustedVideoStart:
			case TAG_AdjustedVideoDuration:
				if (*tptr & 3)
					fprintf(outfile,"  (WARNING! Not color-frame aligned!)\n");
				else
					fprintf(outfile,"  (okay)\n");
				break;

			default:
				fprintf(outfile,"\n");
			}

			tptr++;
		}
		else
		{
			fprintf(outfile,"   %ls = ",tagname);

			len = *tptr;

			switch (tagid)
			{
			case TAG_OriginalLocation:
			case TAG_CommentList:
				fprintf(outfile,"%ls\n",tptr+1);
				break;
			default:
				fprintf(outfile,"(%ld bytes)\n",len);

				bptr = ((UBYTE *)tptr)+4;
				for (i=0; i<len; i+=16)
				{
					fprintf(outfile,"      ");
					for (j=0; j<16; j++)
					{
						if ((i+j) < len)
							fprintf(outfile,"%02lx ",*bptr++);
					}
					fprintf(outfile,"\n");
				}
			}

			tptr = (ULONG *)  ( ((UBYTE *)tptr) + 4 + len);
		}
	}

	return((UBYTE *)tptr);
}

void EndIt(void)
{
	if (ProjBuffer)		FreeVec(ProjBuffer);

	exit(10);
}


ULONG ReadFile(char *name, UBYTE **bufferptr)
{
	FILE	*fp;
	int	read;
	ULONG	length;

	read = 0;
	fp = fopen(name,"r");
	if (fp)
	{
		fseek(fp,0,SEEK_END);
		length = ftell(fp);
		fseek(fp,0,SEEK_SET);

		if (*bufferptr = AllocVec(length,MEMF_PUBLIC))
		{
			read = fread(*bufferptr,length,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",name);

	if (read==1)
		return(length);
	else
		return(0);
}

