/*********************************************************************\
*
* FlyerProc - Processes D2 data out of source clip into new clip
*
* $Id: FlyerProc.c,v 1.0 1995/08/31 15:48:35 Flick Exp $
*
* $Log: FlyerProc.c,v $
*Revision 1.0  1995/08/31  15:48:35  Flick
*FirstCheckIn
*
*
* Copyright (c) 1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	08/23/95		Marty		Created
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos/dos.h>
#include <dos.h>

#include "flyerlib.h"
#include "flyer.h"
#include "FlyerPrivate.h"

#define	DBUG(x)		/* x */

void Cleanup(void);
ULONG ReadField(char *name, ULONG field, BOOL IlaceRvs);
ULONG WriteField(char *name, ULONG field, UBYTE *buff);

struct Library *FlyerBase;

__far UBYTE Buffer2[242][752];

struct FlyerVolume	FlyVol;
struct ClipAction		Action;
struct VidCompInfo	CompInfo;
struct ClipInfo		Cinfo;

BOOL	Abort = FALSE, Corrected = FALSE;
BOOL	FIR = FALSE;

main(int argc,char **argv)
{
ULONG	res;
LONG	lofield,hifield,destfield,srcfield,startfield,stopfield,fields,srcmod,destdupe;
char	*srcname,*dstname,*parse,chr;
int	i;
BOOL	reverse,IlaceRvs,strobe=FALSE;
UBYTE	*buff;


srcname = argv[1];
stcd_l(argv[2],&lofield);
stcd_l(argv[3],&hifield);
dstname = argv[4];

reverse = FALSE;	srcmod = 1;	 destdupe = 1;		// Defaults

if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0)
   {
   printf("Open flyer.library failed\n");
   Cleanup();
   return(10);
   }

	FlyVol.Board = 0;
	FlyVol.Flags = 0;
	FlyVol.SCSIdrive = 0;
	FlyVol.Path = srcname;
	Cinfo.len = sizeof(struct ClipInfo);
	res = GetClipInfo(&FlyVol,&Cinfo);
	if (res)
		printf("ClipInfo Error %ld - %ls\n",res,Error2String(res));
	else
	{
		// Clip in/out points to legal values
		if (lofield >= Cinfo.Fields)
			lofield = Cinfo.Fields-1;
		if (hifield >= Cinfo.Fields)
			hifield = Cinfo.Fields-1;

		fields = hifield-lofield+1;

		//****************************//
		// Effective new clip speed = srcmod/destdupe
		// Sign of srcmod determines direction of motion
		printf("Generating %ld fields ",((fields+srcmod-1)/srcmod)*destdupe);
		if (destdupe>1)
			printf("at %ld/%ld speed",srcmod,destdupe);
		else
			printf("at %ldx speed",srcmod);
		printf("\n");
		//****************************//

		startfield = lofield;
		stopfield = startfield+fields;

		//*** Here we go! ***//

		srcfield = startfield;
		destfield=0;
		while (!Abort)
		{
			if (reverse)
			{
				if (srcfield <= stopfield)
					break;
			}
			else
			{
				if (srcfield >= stopfield)
					break;
			}


			res = ReadField(srcname,srcfield,IlaceRvs);			// Read source field
			if (res != FERR_OKAY)	break;

			for (i=0; (i<destdupe) && (!Abort); i++)
			{
				buff = Buffer2;

				printf("Writing field %ld...\n",destfield+1);

				res = WriteField(dstname,destfield,buff);	// Write new output field
				DBUG(printf("WF result %ld\n",res);)
				if (res != FERR_OKAY)	break;
				destfield++;
			}

			srcfield += srcmod;

			if (res != FERR_OKAY)
			{
				printf("Stopped on error %ld - %ls\n",res,Error2String(res));
				break;
			}
			chkabort();
		}
	}

	if (Abort)
		printf("Wrote %ld fields before aborting\n",destfield);

	Cleanup();

	return(0);
}


void Cleanup(void)
{
	if (FlyerBase)		CloseLibrary(FlyerBase);
}


ULONG ReadField(char *name, ULONG field, BOOL IlaceRvs)
{
	int	line;
	UBYTE	*buffer,modes;
	ULONG	res = FERR_OKAY;

	Corrected = FALSE;		// Invalidate inverted field data

	DBUG(printf("read ");)

	FlyVol.Board = 0;
	FlyVol.Flags = 0;
	FlyVol.SCSIdrive = 0;
	FlyVol.Path = name;
	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_STOPPED;

	modes = 0;
	if (!FIR)
		modes |= FWF_NOFIR;								// No FIR filtering required

	res = OpenReadField(&Action,field,modes);
	if (res)
		printf("\nRead open error %ld - %ls\n",res,Error2String(res));
	else
	{
		buffer = &Buffer2;


		for (line=0; line<TOASTERLINES; line++)
		{
			Action.ReturnTime = RT_STOPPED;
			res = FlyerReadLine(&Action,buffer);
			if (res)
			{
				printf("\nRead error %ld on line %ld - %ls\n",res,line,Error2String(res));
				CloseField(&Action);
				return(res);
			}
			buffer += SAMPLESPERLINE;

			chkabort();
		}

		res = CloseField(&Action);
		if (res)
			printf("\nRead close error %ld - %ls\n",res,Error2String(res));
	}

	return(res);
}


ULONG WriteField(char *name, ULONG field, UBYTE *buffer)
{
int	toler,line,k;
UBYTE	*buff,modes;
ULONG	res = FERR_OKAY,res2 = FERR_OKAY;
UBYTE temp,temp1, temp2;
       
DBUG(printf("write\n");)

FlyVol.Board = 0;
FlyVol.Flags = 0;
FlyVol.SCSIdrive = 0;
FlyVol.Path = name;
Action.Volume = &FlyVol;
Action.ReturnTime = RT_STOPPED;

for (toler=0; toler<4; toler++)
   {
   DBUG(printf("Tolerance %ld\n",toler);)

   CompInfo.Algo = (toler<2) ? ALGO_D2 : ALGO_SN;
   CompInfo.Tolerance = toler & 1;
   CompInfo.RndFreq = 122;
   CompInfo.FIRcomp = 2;
   CompInfo.DataSize = 0;		//Let Flyer pick

   modes = FWF_APPEND;
   if (field==0)
      modes |= FWF_NEW;
   if (toler > 0)
      modes = FWF_REWRITE | FWF_APPEND;		// NEW not needed here???
   if (!FIR)
      modes |= FWF_NOFIR;								// No FIR filtering required

   res = OpenWriteField(&Action,field,modes,&CompInfo);
   if (res)
      {
      printf("\nWrite open error %ld - %ls\n",res,Error2String(res));
      toler = 99;			// Break out, don't try any other tolerances!
      }
   else
      {
      buff = Buffer2;
      for (line=0; line<TOASTERLINES; line++)
         {
	 Action.ReturnTime = RT_STOPPED;
         if ((field%4 == 0) || (field%4 == 3))
            {
            for (k=0;k<188;k++)
               {
               temp1 = (Buffer2[line][k*4] - Buffer2[line][k*4+2])/2;
               temp2 = (Buffer2[line][k*4+1] - Buffer2[line][k*4+3])/2;
               Buffer2[line][k*4] = Buffer2[line][k*4]-temp1;
               Buffer2[line][k*4+1] = Buffer2[line][k*4+1]-temp2;
               Buffer2[line][k*4+2] = Buffer2[line][k*4+2]+temp1;
               Buffer2[line][k*4+3] = Buffer2[line][k*4+3]+temp2;
               }   
            }
         else
            {
            for (k=0;k<188;k++)
               {
               temp1 = (Buffer2[line][k*4+2] - Buffer2[line][k*4])/2;
               temp2 = (Buffer2[line][k*4+3] - Buffer2[line][k*4+1])/2;
               Buffer2[line][k*4] = Buffer2[line][k*4]+temp1;
               Buffer2[line][k*4+1] = Buffer2[line][k*4+1]+temp2;
               Buffer2[line][k*4+2] = Buffer2[line][k*4+2]-temp1;
               Buffer2[line][k*4+3] = Buffer2[line][k*4+3]-temp2;
               }   
            }                       
         res = FlyerWriteLine(&Action,buff);
	 if (res)
	    {
	    if (res != FERR_FULL)
	       printf("\nWrite error %ld on line %ld - %ls\n",res,line,Error2String(res));
	    else if (res == FERR_FULL)
	       {
	       DBUG(printf("FULL on line %ld\n",line);)
	       }
	    break;
	    }
	 buff += SAMPLESPERLINE;
         chkabort();
	 }
      
      res2 = CloseField(&Action);
      if (res2 == FERR_FULL)
	 {
	 DBUG(printf("FULL on close\n");)
	 }
 
      if ((res2) && (res2 != FERR_FULL))
	 printf("\nWrite close error %ld - %ls\n",res2,Error2String(res2));

      if (res == FERR_OKAY)		// If WriteLine didn't fail, let "close" error show thru
	 res = res2;
      }

   DBUG(printf("Result %ld\n",res);)

   if (res == FERR_OKAY)			// Succeeded
      break;
   else if (res != FERR_FULL)		// Something other than "Full"
      break;
   if (Abort)
      break;
   }

DBUG(printf("Result2 %ld\n",res);)
return(res);
}
