/*********************************************************************\
*
* Flyer.h - Flyer include file
*
* $Id: Flyer.h,v 1.4 1995/10/16 17:28:47 Flick Exp $
*
* $Log: Flyer.h,v $
*Revision 1.4  1995/10/16  17:28:47  Flick
*Added sequence errors
*Added definitions for VideoGrade
*Added ClipAction.Status
*Added FACOF_SENSE8 for FlyerAudioCtrl function
*
*Revision 1.3  1995/08/25  15:29:52  Flick
*Added NOFIR option for field reading/writing
*
*Revision 1.2  1995/08/11  12:38:45  Flick
*Added UserID to ClipAction structure
*
*Revision 1.1  1995/08/07  11:51:38  Flick
*Added FlyerOptions flag defines
*
*Revision 1.0  1995/05/05  15:43:38  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/23/94		Marty		Created
\*********************************************************************/

#ifndef	INC_FLYER_H
#define	INC_FLYER_H

/*
**	Include file for use by Apps calling flyer.library
**
**	(c) Copyright 1994 NewTek, Inc.
**	    All Rights Reserved
*/

#ifndef EXEC_NODES_H
#include "exec/nodes.h"
#endif

#ifndef DOS_DOS_H
#include "dos/dos.h"
#endif


#define	FLYERLIBNAME	"flyer.library"

/*******************************************\
**** Public Return Codes for Flyer Calls ****
\*******************************************/

/*** General Flyer Errors ***/
#define	FERR_OKAY			0x00	/* All went well */
#define	FERR_CMDFAILED		0x01	/* Command failed for some reason */
#define	FERR_BUSY			0x02	/* Still in progress */
#define	FERR_ABORTED		0x03	/* User abort */
#define	FERR_BADPARAM		0x04	/* Bad command parameter */
#define	FERR_BADCOMMAND	0x05	/* Command not defined/supported */
#define	FERR_BADVIDHDR		0x06	/* Ran out of video - no header detected */
#define	FERR_WRONGMODE		0x07	/* Wrong play/rec mode for action */
#define	FERR_OLDDATA		0x08	/* Incompatible data */
#define	FERR_NOAUDIOCHAN	0x09	/* No free audio channel(s) */
#define	FERR_CHANINUSE		0x0A	/* Video/SCSI channel not available */
#define	FERR_BADFLDHAND	0x0B	/* Bad field handle */
#define	FERR_CLIPLATE		0x0C	/* A/V clip started late */
#define	FERR_DROPPEDFLDS	0x0D	/* Dropped 1 or more fields */

/*** Flyer Internal Errors ***/
#define	FERR_NOTASKS		0x10	/* No SCSI tasks available for use */
#define	FERR_LISTCORRUPT	0x11	/* Internal list corrupt */
#define	FERR_NOTINRANGE	0x12	/* Internal list error */
#define	FERR_EEFAILURE		0x13	/* EEPROM failure */
#define	FERR_NOFINDERS		0x14	/* No FrameFinders available for use */
#define	FERR_BADMODULE		0x1F	/* Incompatible module provided */

/*** FileSystem Errors ***/
#define		FIRSTFSERR			0x20
#define	FERR_OBJNOTFOUND	0x20	/* Could not find file/dir */
#define	FERR_FULL			0x21	/* Drive full */
#define	FERR_DIRFULL		0x22	/* Directory full */
#define	FERR_EXHAUSTED		0x23	/* Directory list exhausted */
#define	FERR_FSFAIL			0x24	/* FileSystem failure */
#define	FERR_WRONGTYPE		0x25	/* Wrong type of object */
#define	FERR_UNFORMATTED	0x26	/* Drive not high-level formatted */
#define	FERR_EXCLUDED		0x27	/* Exclusive lock prevented action */
#define	FERR_OUTOFRANGE	0x28	/* Seek beyond bounds */
#define	FERR_CANTEXTEND	0x29	/* End of file, and cannot extend file */
#define	FERR_PROTECTED		0x2A	/* Drive write-protected */
#define	FERR_DIFFERENT		0x2B	/* Grips are different objects */
#define	FERR_EXISTS			0x2C	/* File already exists */
#define	FERR_NOMEM			0x2D	/* Out of storage */
#define	FERR_DELPROT		0x2E	/* Delete-protected file */
#define	FERR_READPROT		0x2F	/* Read-protected file */
#define	FERR_WRITEPROT		0x30	/* Write-protected file */
#define	FERR_INUSE			0x31	/* Disk/object in use */
#define	FERR_DIRNOTEMPTY	0x32	/* Directory was not empty */
#define		LASTFSERR			0x32

/*** SCSI Errors ***/
#define	FERR_SELTIMEOUT	        0x40	/* SCSI Time-out -- no drive */
#define	FERR_BADSTATUS		0x41	/* Bad status after executing command */

/*** Sequencing Errors ***/
#define	FERR_WRONGDATATYPE	0x60	/* Asked for improper type of data from clip */
#define	FERR_DRIVEINCAPABLE	0x61	/* Using video clip from a non-video drive */
#define	FERR_NO_BROLLDRIVE	0x62	/* No video B-roll drive found */
#define	FERR_HEADFAILED		0x63	/* A/B head missing/problem */


/*** Amiga Library Errors ***/
#define	FERR_NOCARD		0x70	/* Flyer card specified does not exist */
#define	FERR_LIBFAIL		0x71	/* Library failed to pass command to Flyer */
#define	FERR_ASYNCFAIL		0x72	/* An asynchronous command failed */
#define	FERR_VOLNOTFOUND	0x73	/* Volume name not found */
#define	FERR_NOFREECMD		0x74	/* Library<->Flyer RAM clogged */
#define	FERR_BADID		0x75	/* Illegal async ID */

#define	FERR_LIMIT		0x7F

/*** Mode - for use with StillMode command ***/
#define	MODE_FIELD	1
#define	MODE_PAIR	2
#define	MODE_FRAME	4

/*** Structure returned from FlyerDriveInfo call ***/

struct FlyerVolInfo {
	UWORD		len;					/* Length of this structure */
	ULONG		Ident;				/* 'ROOT' for good volumes */
	UBYTE		Version;				/* Version of FileSystem that wrote drive */
	UBYTE		LTitle;				/* Length for L-String */
	char		Title[80];			/* String (null-terminated) */
	ULONG		Blocks;				/* Total user blocks */
	ULONG		BlksFree;			/* User blocks free */
	UBYTE		Flags;				/* FVIF_xxx - see below */
	UBYTE		DiskOkay;			/* FileSys in good shape? */
	UWORD		BlkSize;				/* Block size */
	struct	DateStamp	DiskDate;
	ULONG		FragBlks;			/* Fragmented blocks */
	ULONG		Largest;				/* Largest free chunk (in blocks) */
	ULONG		Optimized;			/* Largest free chunk if optimized (in blocks) */
	ULONG		reserved[4];
};

/*** FlyerVolInfo Flags ***/
#define	FVIF_VIDEOREADY	1		/* Drive can handle video */
#define	FVIF_AUDIOREADY	2		/* Drive can handle audio */
#define	FVIF_WRITEPROT	4		/* Drive is not writable */


/*** Volume node structure ***/

struct FlyerVolNode {
	struct	Node	Node;
	char		Name[80];				/* Name of volume */
	UBYTE		Board;					/* Flyer board number */
	UBYTE		SCSIdrive;				/* SCSI channel/unit */
	UBYTE		Flags;					/* From FlyerVolInfo (FVIF_xxx) */
	UBYTE		pad;
};


/*** Structure returned from GetClipInfo call ***/

struct ClipInfo {
	UWORD		len;					/* Length of this structure */
	char		Name[80];
	char		Comment[80];
	UBYTE		Flags;				/* OIF_xxxx -- see below */
	UBYTE		Type;					/* FLYER_TYPE_xxx */
	struct	DateStamp	Date;
	ULONG		Bits;					/* User defined */
	ULONG		Fields;				/* Length in fields */
	ULONG		Start;				/* Start block on drive */
	ULONG		Length;				/* Byte length */
	ULONG		IndexBlk;			/* Location of clip's index */
	UBYTE		NumAudChans;		/* Number of audio channels contained */
	UBYTE		VideoGrade;			/* VG_xxx below */
	ULONG		EndBlk;				/* Last blk used +1 */
	ULONG		LengthExt;			/* Extended Byte length: high 32 bits */
	UBYTE		reserved[22];
};

/*** Flyer object types ***/
#define	FLYER_TYPE_FILE	1
#define	FLYER_TYPE_DIR	2
#define	FLYER_TYPE_ROOT	3

/*** ClipInfo Flags ***/
#define	CIF_HASVIDEO	1		/* Clip contains video */
#define	CIF_HASAUDIO	2		/* Clip contains audio */

/*** Video Grade values ***/
#define	VG_STD			0		/* Standard grade video */
#define	VG_HQ5			1		/* HQ5 grade video */

/*** Structure returned from GetSMPTE call ***/

struct SMPTEinfo {
	UBYTE	SMPTEvalid;					/* SMPTE info valid? */
	UBYTE	SMPTEhours;					/* Hours */
	UBYTE	SMPTEminutes;				/* Minutes */
	UBYTE	SMPTEseconds;				/* Seconds */
	UBYTE	SMPTEframes;				/* Frames (1/30th) */
	UBYTE	SMPTEuser1;					/* BW1 and 2 */
	UBYTE	SMPTEuser2;					/* BW3 and 4 */
	UBYTE	SMPTEuser3;					/* BW5 and 6 */
	UBYTE	SMPTEuser4;					/* BW7 and 8 */
	UBYTE	SMPTEflags;					/* See SIF_xxxx below */
};

/*** SMPTE flags ***/
#define	SIF_DropFrame		1	/* Position is in drop-frame format */
#define	SIF_ColorFrame		2	/* Color Frame identification applied */
#define	SIF_Reverse			4	/* Time code is reverse-direction */


/*** Defines for the SetSerDevice command ***/

#define	SERDEVTYPE_NONE		0
#define	SERDEVTYPE_SMPTE		1
#define	SERDEVTYPE_CTRL		2
#define	SERDEVTYPE_MIDI		3

#define	SERDEV_GEN_9600		1
#define	SERDEV_HORITA			10
#define	SERDEV_TELCOM			11


/*** Structure used with OpenWriteField call ***/

struct VidCompInfo {
	UBYTE		Algo;					/* Algorithm type */
	UBYTE		Tolerance;			/* Error tolerance */
	UBYTE		FIRcomp;				/* FIR compensation level */
	UBYTE		RndFreq;				/* Noise frequency */
	UBYTE		RndSeed;				/* Noise seed */
	UBYTE		Flags;				/* VCIF_xxx Flags */
	UWORD		DataSize;			/* Max field size in blocks */
	ULONG		Private1;
	ULONG		Private2;
	UBYTE		reserved[6];
};

#define	ALGO_D2			1			/* D2 */
#define	ALGO_SN			2			/* Sub-nyquist */

#define	VCIF_AUTOMODE	1			/* Optimize compression on the fly */

#define	SAMPLESPERLINE	752		/* Samples per active video line */
#define	TOASTERLINES	242		/* Lines per active video field (Toaster) */
#define	LINESPERFIELD	243		/* Lines per active video field (Full NTSC) */

/*** FieldModes - for use with OpenWriteField command ***/
#define	FWF_NEW			1			/* Create new clip */
#define	FWF_APPEND		2			/* Append field to clip */
#define	FWF_REWRITE		4			/* Rewrite field */
#define	FWF_FRAME		8			/* Redo entire frame */
#define	FWF_HALFLINES	16			/* Support full NTSC fields (+half lines) */
#define	FWF_NOFIR		32			/* Omit FIR filtering */

/*** FieldModes - for use with OpenReadField command ***/
#define	FRF_HALFLINES	16			/* Support full NTSC fields (+half lines) */

struct FlyerVolume {
	char		*Path;				/* Pointer to volume:clipname string */
	UBYTE		Board;				/* Board number (0...n) */
	UBYTE		SCSIdrive;			/* SCSI channel/unit */
	UBYTE		Flags;				/* See below */
	UBYTE		pad;
	ULONG		reserved[4];
};

#define	FVF_USENUMS		0x01	/* Ignore volume in string, use numbers */


struct ClipAction {
	struct FlyerVolume *Volume;	/* Volume and clip name info */
	ULONG	ID;			/* For asynchronous operation */
	UBYTE	ReturnTime;		/* See RT_xxx below */
	UBYTE	Channel;		/* Video channel to use: 0, 1, or FF for choice */
	UBYTE	Flags;			/* CAF_xxxxx -- see below */
	UBYTE	PermissFlags;		/* CAPF_xxxxx -- see below */
	ULONG	VidStartField;
	ULONG	AudStartField;
	ULONG	VidFieldCount;
	ULONG	AudFieldCount;
	ULONG	GoClock;
        UWORD	MatteY;			/* For use with CAF_USEMATTE */
	BYTE	MatteI;
	BYTE	MatteQ;
	UWORD	VolAttack;		/* Attack time/ramp time */
	UWORD	VolSust1;		/* Channel 1 sustain volume */
	UWORD	VolSust2;		/* Channel 2 sustain volume */
	UWORD	VolDecay;		/* Decay time */
	WORD	AudioPan1;		/* Channel 1 pan (-left, 0 ctr, +right) */
	WORD	AudioPan2;		/* Channel 2 pan (-left, 0 ctr, +right) */
	ULONG	TotalAudStart;		/* Combined audio start field */
	ULONG	TotalAudLength;		/* Combined audio field count */
	ULONG	reserved0[4];
	/* These block values provide a "raw" access to clip points */
	ULONG	StartBlk;
	ULONG	EndBlk;
	ULONG	UserID;			/* Caller-private ID used for sequencing errors */
	ULONG	reserved1[3];
	/* These things used by the FileSystem */
	ULONG	Grip;
	ULONG	FileID;
	UBYTE	Access;
	UBYTE	pad2;
	ULONG	FldHandle;		/* Handle to open field for direct R/W */
	ULONG	reserved2[3];
	/* Status communication data */
	ULONG	Status;
	/* Special return values -- Valid only when command complete */
	ULONG	LastFieldDone;
};

/*** ReturnTime values ***/
#define	RT_FREE		0	/* Return immediately, never follow-up */
#define	RT_IMMED	1	/* Return immediately */
#define	RT_STARTED	2	/* Return when actually started */
#define	RT_STOPPED	3	/* Return when action stops */

/*** ClipAction flags ***/
#define	CAF_VIDEO		0x01	/* Include clip video */
#define	CAF_AUDIOL		0x02	/* Include left audio channel */
#define	CAF_AUDIOR		0x04	/* Include right audio channel */
#define	CAF_USEMATTE		0x08	/* Display matte color when clip done */
#define	CAF_NOPREROLL		0x10	/* Skip pre-roll */
#define	CAF_APPEND		0x20	/* For single-frame appending -- NOT SUPPORTED HERE */
#define	CAF_REPROCESS		0x40	/* Force clip re-processing */

/*** ClipAction permission flags ***/
#define	CAPF_STEALOURVIDEO	0x01	/* Can steal requested video channel */
#define	CAPF_KILLOTHERVIDEO	0x02	/* Can kill other video channel(s) */
#define	CAPF_ERRIFBUSY		0x04	/* Return error rather than wait for it */
#define	CAPF_AUTOMUTE		0x08	/* Automatically mute audio when looping */
#define	CAPF_USEHEADS		0x10	/* Use A/V heads when present and needed */

/*** FlyerInputSel values (video source) ***/
#define	FI_Camcorder	0	/* Needs TBC */
#define	FI_SVHS		1	/* Needs TBC */
#define	FI_Toaster1	2	/* Toaster input 1 */
#define	FI_Toaster2	3	/* Toaster input 2 */
#define	FI_ToasterMain	4	/* Toaster Main bus output */
#define	FI_ToasterPV	5	/* Toaster Preview bus output */

/*** FlyerInputSel values (sync source) ***/
#define	FS_ToasterMain	0	/* Toaster Main bus output */
#define	FS_Toaster1	1	/* Toaster input 1 */

/*** Flyer Calibration values ***/
#define	CALIB_DACA_PHASE_EDGE		0
#define	CALIB_DACA_PHASE_COURSE		1
#define	CALIB_DACA_PHASE_FINE		2
#define	CALIB_DACB_PHASE_EDGE		3
#define	CALIB_DACB_PHASE_COURSE		4
#define	CALIB_DACB_PHASE_FINE		5
#define	CALIB_ADC_PHASE_EDGE		6
#define	CALIB_ADC_PHASE_COURSE		7
#define	CALIB_ADC_PHASE_FINE		8
#define	CALIB_HPLAYOFFSETA		9
#define	CALIB_HPLAYOFFSETB		10
#define	CALIB_HRECOFFSETA		11
#define	CALIB_HRECOFFSETB		12
#define	CALIB_PEDESTALA			13
#define	CALIB_PEDESTALB			14


/*** TBC control structure and defines ***/

struct TBCctrl {
	UBYTE	Status;				/* Status flags */
	UBYTE	Flags;				/* General flags */
	UBYTE	DecFlags;			/* Decoder flags */
	UBYTE	EncFlags;			/* Encoder flags */
	UBYTE	InputSel;			/* Input mux control */
	UBYTE	Term;					/* Termination control */
	BYTE	Bright;				/* Brightness value (-64 to 63) */
	UBYTE	Contrast;			/* Contrast value (0 to 127) */
	UBYTE	Sat;					/* Saturation value (0 to 127) */
	BYTE	Hue;					/* Hue value (-64 to 63) */
	UWORD	Phase;				/* Phase adjust ($000 to $7FF) */
	UWORD	HorAdj;				/* Horizontal adjust ($000 to $FFF) */
	BYTE	Fader;				/* Fader value (0 to 255) */
	UBYTE	KeyerFlags;			/* Keyer flags */
	UBYTE	reserved[38];
};

/*** TBC Operations ***/
#define	TBCOF_STATUS		0x01	/* Fill in status field */
#define	TBCOF_MODES			0x02	/* Set modes/flags/muxes */
#define	TBCOF_ADJUST		0x04	/* Set adjustment values */

/*** TBC Status flags ***/
#define	TBCSF_MODULE		0x01	/* TBC module present */
#define	TBCSF_VIDEO			0x02	/* Video present at input */
#define	TBCSF_STABLE		0x04	/* Video input stable */

/*** TBC General flags ***/
#define	TBCGF_BYPASS		0x01	/* Bypass TBC */
#define	TBCGF_FREEZE		0x02	/* Freeze video */

/*** TBC Decoder flags ***/
#define	TBCDF_AGC			0x01	/* Enable AGC */
#define	TBCDF_CHROMAAGC	0x02	/* Enable chroma AGC */
#define	TBCDF_MONOCHROME	0x04	/* Monochrome input */

/*** TBC Encoder flags ***/
#define	TBCEF_BARS			0x01	/* Output 100% sat, 75% ampl bars */
#define	TBCEF_KILLCOLOR	0x02	/* Disable color on output */

/*** TBC InputSel values ***/
#define	TBCIN_YC				0		/* Flyer SVHS input */
#define	TBCIN_COMP			1		/* Flyer Composite input */
#define	TBCIN_TMAIN			2		/* Toaster Main output */
#define	TBCIN_FADER			3		/* TBC Fader output */

/*** TBC Termination ***/
#define	TBCTF_FADERA		0x04	/* Flyer A channel input */
#define	TBCTF_FADERB		0x08	/* Flyer B channel input */
#define	TBCTF_OUT			0x10	/* Video output */
#define	TBCTF_GENIN			0x20	/* Genlock input */
#define	TBCTF_COMPIN		0x80	/* Composite video input */

/*** TBC Keyer flags ***/
#define	TBCKF_KEYONB		0x01	/* Src (A or B) */
#define	TBCKF_MODE0			0x02	/* Keyer mode 0 */
#define	TBCKF_MODE1			0x04	/* Keyer mode 1 */
#define	TBCKF_FADEROUT		0x08	/* TBC/Fader mux (1 = Fader) */

/*** StartClipCutList flags ***/
#define	CCLF_DESTRUCTIVE	0x01	/* Delete unused portions of original clip */


/*** Structure used with FlyerAudioCtrl call ***/

struct FlyAudCtrl {
	UBYTE	Flags;				/* General flags */
	UBYTE	pad;
	UBYTE	LeftSense;			/* Left channel overrange detector (0 - 3) */
	UBYTE	RightSense;			/* Right channel overrange detector (0 - 3) */
	UBYTE	LeftSrc;				/* Left input source mux (see below) */
	UBYTE	RightSrc;			/* Right input source mux (see below) */
	UBYTE	LeftGain;			/* Left channel gain (0 or 1 to 16) */
	UBYTE	RightGain;			/* Right channel gain (0 or 1 to 16) */
	UBYTE	Aux1Mix;				/* Aux1 mix-in amount (-16 to 15, -128 to mute) */
	UBYTE	reserved1;
	UBYTE	Aux2Mix;				/* Aux2 mix-in amount (-16 to 15, -128 to mute) */
	UBYTE	reserved2;
	UBYTE	reserved[8];
};

/*** Audio Ctrl Operations ***/
#define	FACOF_SENSE		0x01	/* Report overrange information */
#define	FACOF_SETGAIN		0x02	/* Set input gain as specified */
#define	FACOF_SETSRC		0x04	/* Set input sources as specified */
#define	FACOF_SETMIX		0x08	/* Set aux mixing as specified */
#define	FACOF_SENSE8		0x10	/* Return 8 bit L/R readings in record mode */

/*** Audio Ctrl Input Sources ***/
#define	FACS_LINE1			0		/* RCA line-in jacks on rear of card */
#define	FACS_AUX1			1		/* Aux1 connector (JP19) */
#define	FACS_LINE2			2		/* Line2 connector (JP1) */


/*** FlyerOptions Non-Volatile Flags ***/
#define	FLYOPTF_DropFramDet	0x01		/* Stop recording on dropped frame */
#define	FLYOPTF_NOT_HQ5		0x02		/* =0 to enable HQ5 */

#endif	/* INC_FLYER_H */
