/*********************************************************************\
*
* PlayClip - Play video clip
*
* $Id: PlayClip.c,v 1.3 1995/10/16 17:22:14 Flick Exp $
*
* $Log: PlayClip.c,v $
*Revision 1.3  1995/10/16  17:22:14  Flick
*Now uses specific AbortAction() again (not global abort)
*
*Revision 1.2  1995/07/17  14:24:38  Flick
*Added up/down volume option (on the fly)
*
*Revision 1.1  1995/07/11  15:38:34  Flick
*Changed abort method, due to single-aborts no longer implemented
*
*Revision 1.0  1995/05/05  15:52:12  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	02/24/94		Marty		Created
*	05/04/94		Marty		Added fast frame-indexing
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "flyerlib.h"
#include "flyer.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);

struct Library *FlyerBase;
struct FlyerVolume FlyVol;
struct ClipAction	 Action;

BPTR	handle;

main(int argc,char **argv)
{
	int	unit,vidchan,drive,audchans,vidflag;
	ULONG	res,clock,frame,frames,theclock;
	UWORD	Volume;
	char	key;

	if ((argc!=10)||(argv[1][0]=='?')) {
		printf("Usage: %ls <board> <vchan> <drive> <name> <frame> <frames> <clock> <vidflags> <audchans>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	GetArg(argv[2],&vidchan,0,3,"Video channel");
	GetArg(argv[3],&drive,0,23,"SCSI drive");
	GetLongArg(argv[5],&frame,0,0x7FFFFFFF,"Start Frame");
	GetLongArg(argv[6],&frames,0,0x7FFFFFFF,"Frame Count");
	GetLongArg(argv[7],&clock,0,0x7FFFFFFF,"Frame Clock");
	GetArg(argv[8],&vidflag,0,1,"Video Flag");
	GetArg(argv[9],&audchans,0,2,"Audio Channels");

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0) {
		printf("Open flyer.library failed\n");
		return;
	}

	printf("Press RETURN to abort\n");

	handle = Input();

	ToasterMux(unit,1,1,0);

	FlyVol.Path = argv[4];
	FlyVol.Board = unit;
	FlyVol.SCSIdrive = drive;

	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_STARTED;

	Action.Channel = vidchan;
	Action.Flags = 0;
	if (vidflag)
		Action.Flags |= CAF_VIDEO;
	if (audchans==1)
		Action.Flags |= CAF_AUDIOL;
	else if (audchans==2)
		Action.Flags |= CAF_AUDIOL | CAF_AUDIOR;
#if 0
	Action.Flags |= CAF_USEMATTE;
#endif

	Volume = 0xFFFF;

	Action.VidStartField = frame * 4;
	Action.VidFieldCount = frames * 4;
	Action.TotalAudStart = Action.AudStartField = frame * 4;
	Action.TotalAudLength = Action.AudFieldCount = frames * 4;
	Action.GoClock = clock;
	Action.VolSust1 = Volume;
	Action.VolSust2 = Volume;
	Action.AudioPan1 = -0x8000;
	Action.AudioPan2 = 0x7FFF;

	res = FlyerPlay(&Action);
	if (res==FERR_OKAY) {
		res = GetFieldClock(&theclock);
		printf("Started at clock %lx (asked for %lx)\n",theclock,clock);
		for (;;) {
			if (WaitForChar(handle,30)) {
				key = getchar();
				if (key=='a') {
					Action.Flags ^= (CAF_AUDIOL | CAF_AUDIOR);		/* Toggle audio */
					printf("Audio turned %ls\n",(Action.Flags & (CAF_AUDIOL | CAF_AUDIOR))?"on":"off");
					res = ChangeAudio(&Action);
					getchar();
				}
				else if (key=='u') {
					if (Volume < 0xC000)
					{
						Volume += 0x4000;
						printf("Audio vol= %ld\n",Volume);

						Action.VolAttack = 60;
						Action.VolSust1 = Action.VolSust2 = Volume;
						res = ChangeAudio(&Action);
					}
					getchar();
				}
				else if (key=='d') {
					if (Volume >= 0x4000)
					{
						Volume -= 0x4000;
						printf("Audio vol= %ld\n",Volume);

						Action.VolAttack = 60;
						Action.VolSust1 = Action.VolSust2 = Volume;
						res = ChangeAudio(&Action);
					}
					getchar();
				}
				else {
					res = AbortAction(&Action);		// This aborts just my action
//					res = AbortAction(NULL);			// Does a global (panic) abort

					printf("Aborted\n");
					break;
				}
//				printf(".\n");
			}
			res = CheckAction(&Action);
			if (res != FERR_BUSY) {
				printf("Complete\n");
				break;
			}
		}
	}

	printf("Last frame played = %ld\n",Action.LastFieldDone>>2);

	if (res)
		printf("Error %ld - %ls\n",res,Error2String(res));
	else
		printf("Okay\n");

	CloseLibrary(FlyerBase);

	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}


void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi)) {
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
