/**********************************i******************************************
**
**                            Graph Library
**
**
**
** Changelog:
**
**
**
**
**
****************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <exec/types.h>
#include <exec/libraries.h>
#include <dos/dos.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>


#include <graphics/text.h>
#include <graphics/rastport.h>
#include <graphics/displayinfo.h>
#include <libraries/dos.h>
#include <libraries/asl.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/asl_protos.h>
#include <clib/dos_protos.h>


/*-- User Include Files --*/

#include "rfx_common.h"
#include "pw_util_lib.h"
#include "graph_lib.h"
#include "err_lib.h"


/*-- global variables --*/

/* init graph structure */
struct graph_struct *init_graph( WORD xoffset, WORD yoffset, WORD width,
                                 WORD height, WORD xticheight, WORD
                                 yticwidth, unsigned long int end_frame,
                                 double max_percent, int max_num_frames,
                                 int max_pbuff, int num_t_steps)
{
    struct graph_struct *graph;
    int                 i;
    double              tval;



    /* allocate the structure */
    /* then allocate storage */
    if( (graph = calloc(1, sizeof (struct graph_struct)) ) == NULL)
    {
        return (NULL);
    }
    if( (graph->frame = calloc(max_num_frames, sizeof(unsigned long int)))
        == NULL)
    {
        free(graph);
        return (NULL);
    }

    if( (graph->percent = calloc(max_num_frames, sizeof(double)))
        == NULL)
    {
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    if( (graph->x = calloc(max_num_frames, sizeof(WORD)))
        == NULL)
    {
        free(graph->percent);
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    if( (graph->y = calloc(max_num_frames, sizeof(WORD)))
        == NULL)
    {
        free(graph->x);
        free(graph->percent);
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    if( (graph->spline = calloc(max_num_frames, sizeof
                                               (struct spline_params_struct)))
        == NULL)
    {
        free(graph->y);
        free(graph->x);
        free(graph->percent);
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    if( (graph->pbuffx = calloc(max_pbuff, sizeof(WORD)))
        == NULL)
    {
        free(graph->spline);
        free(graph->y);
        free(graph->x);
        free(graph->percent);
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    if( (graph->pbuffy = calloc(max_pbuff, sizeof(WORD)))
        == NULL)
    {
        free(graph->pbuffx);
        free(graph->spline);
        free(graph->y);
        free(graph->x);
        free(graph->percent);
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    if( (graph->t = calloc(num_t_steps, sizeof(double)))
        == NULL)
    {
        free(graph->pbuffy);
        free(graph->pbuffx);
        free(graph->spline);
        free(graph->y);
        free(graph->x);
        free(graph->percent);
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    if( (graph->t2 = calloc(num_t_steps, sizeof(double)))
        == NULL)
    {
        free(graph->t);
        free(graph->pbuffy);
        free(graph->pbuffx);
        free(graph->spline);
        free(graph->y);
        free(graph->x);
        free(graph->percent);
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    if( (graph->t3 = calloc(num_t_steps, sizeof(double)))
        == NULL)
    {
        free(graph->t2);
        free(graph->t);
        free(graph->pbuffy);
        free(graph->pbuffx);
        free(graph->spline);
        free(graph->y);
        free(graph->x);
        free(graph->percent);
        free(graph->frame);
        free(graph);
        return (NULL);
    }


    /* initialize the variables */

    graph->xoffset = xoffset;
    graph->yoffset = yoffset;
    graph->width   = width;
    graph->height  = height;
    graph->max_num_frames = max_num_frames;
    graph->end_frame = end_frame;
    graph->max_percent = max_percent;
    graph->deltax = (double)width / (double)(end_frame+1);
    graph->deltay = (double)height / (double) max_percent;

    graph->top = yoffset;
    graph->left = xoffset;
    graph->right = xoffset + width;
    graph->bottom = yoffset + height;
    graph->xtic_majorheight = xticheight;
    graph->xtic_minorheight = (WORD)(xticheight/2);
    graph->ytic_majorwidth  = yticwidth;
    graph->ytic_minorwidth  = (WORD)(yticwidth/2);
    graph->xtic_majorinterval = 10;
    graph->xtic_minorinterval = 5;
    graph->ytic_majorinterval = 20;
    graph->ytic_minorinterval = 10;
    graph->current_index = 0;

    graph->max_pbuff = max_pbuff;
    graph->num_pbuff = 0;

    graph->num_t_steps = num_t_steps;
    graph->t_step = (double)( 1.0 / (double)(num_t_steps) );

    tval = 0.0;
    for(i=0; i< num_t_steps; i++)
    {

        graph->t[i]   = tval;
        graph->t2[i]  = tval * tval;
        graph->t3[i]  = graph->t2[i] * tval;

        tval += graph->t_step;
    }

    /* make sure the last value is 1.0 */
    i = num_t_steps - 1;
    graph->t[i]   = 1.0;
    graph->t2[i]  = 1.0;
    graph->t3[i]  = 1.0;




    /* initialize the chart with 2 points as a linear transition */

    graph->num_points = 0;

    /* initialize drawing colors */
    graph->bg_pen = 5;
    graph->fg_pen = 2;
    graph->fill_pen = 4;
    graph->border_pen = 1;
    graph->hilite_pen = 7;


    graph->status = INITIALIZED;

    /* return the pointer */
    return(graph);
}




/*-- delete the graph structure --*/
void free_graph(struct graph_struct *graph)
{
    if (graph->status == INITIALIZED)
    {

        if (graph->pbuffx != NULL)
        {
            free (graph->pbuffx);
        }

        if (graph->pbuffy != NULL)
        {
            free (graph->pbuffy);
        }

        if (graph->spline != NULL)
        {
            free (graph->spline);
        }

        if (graph->frame != NULL)
        {
            free (graph->frame);
        }

        if (graph->percent != NULL)
        {
            free (graph->percent);
        }

        if (graph->x != NULL)
        {
            free (graph->x);
        }

        if (graph->y != NULL)
        {
            free (graph->y);
        }

        if (graph->t != NULL)
        {
            free (graph->t);
        }

        if (graph->t2 != NULL)
        {
            free (graph->t2);
        }

        if (graph->t3 != NULL)
        {
            free (graph->t3);
        }

        free (graph);
    }

    return;
}


/* highlite the current selected index */

int hilight_current_index( struct RastPort *rport, struct graph_struct *graph )
{

    /* set pen colors */
    SetAPen(rport, graph->hilite_pen);
    SetDrMd(rport, JAM1);

    draw_box(rport, graph, graph->x[graph->current_index],
                graph->y[graph->current_index], 2);

    return(OK);
}

/* unhighlight selected index */

int unhilight_current_index(struct RastPort *rport, struct graph_struct *graph)
{

    /* set pen colors */
    SetAPen(rport, graph->fg_pen);
    SetDrMd(rport, JAM1);

    draw_box(rport, graph, graph->x[graph->current_index],
                graph->y[graph->current_index], 2);

    return(OK);
}





/* Draw box around a point, forcing it into the graph */
/* pens and drawmode must have been setup before calling */

int draw_box( struct RastPort *rport, struct graph_struct *graph,
                    WORD x, WORD y, int half_width)
{

    WORD  top,left,bottom,right;
    int   width;

    width = 2 * half_width;

    /* calculate corners (fit inside graph) */
    left = x - half_width;
    if (left < graph->left)
    {
        left = graph->left;
    }
    else if ( left > graph->right)
    {
        left = (graph->right - width);
    }

    right = left + width;
    /* left already clipped to graph, no need to check under */
    if (right > graph->right)
    {
        right = graph->right;
        left = right - width;
    }

    top = y - half_width;
    if (top < graph->top)
    {
        top = graph->top;
    }
    else if ( top > graph->bottom)
    {
        top = graph->bottom - width;
    }

    bottom = top + width;
    if (bottom > graph->bottom)
    {
        bottom = graph->bottom;
        top = bottom - width;
    }

    RectFill(rport, left, top, right, bottom);



    return(OK);
}



/* append a point to the graph (use only when building a list for */
/* the first time), it recalculates the  */

int append_graph_point( struct graph_struct *graph, unsigned long int frame,
                        double percent, double tension, double bias,
                        double cont, int linear)
{

    int index;

    if (graph->status != INITIALIZED)
    {
        log_message(ERROR_MSG,"Graph not init in append_graph_point()\n");
        return(ERROR);
    }

    if (graph->num_points == graph->max_num_frames)
    {
        log_message(ERROR_MSG,"point overflow in append_graph_point()\n");
        return(ERROR);
    }

    index = graph->num_points;

    graph->frame[index] = frame;
    graph->percent[index] = percent;
    graph->spline[index].tension = tension;
    graph->spline[index].bias = bias;
    graph->spline[index].cont = cont;
    graph->spline[index].linear = linear;


    graph->num_points++;
    graph->end_frame = frame;

    /* change graph x scalefactor */

    graph->deltax = (double)graph->width / (double)(graph->end_frame+1);


    return(OK);
}


/* find screen coords for a point */
int map_graph_screen( struct graph_struct *graph, unsigned long int frame,
                        double percent, WORD *x, WORD *y)
{

    /* for speed, do not perform graph error checking */


    /* find coords that corresponds to the frame and percentage */
    if (frame < graph->end_frame)
    {
        *x = graph->xoffset + (WORD)( (double)frame * graph->deltax );
    }
    else
    {
        *x = graph->xoffset + graph->width;
    }
    /* note, frame is unsigned, never negative */


    if (percent < 0.0)
    {
        *y = graph->bottom;
    }
    else if (percent < graph->max_percent )
    {
        *y = graph->bottom - ((WORD)(percent * graph->deltay));
    }
    else
    {
        *y = graph->yoffset;   /* clip to maximum */
    }

    return(OK);

}



/* find frame and percent for a screen coord */

int map_screen_graph( struct graph_struct *graph, WORD x, WORD y,
                        unsigned long int *frame, double *percent)
{

    /* for speed, do not perform graph error checking */


    /* find point that corresponds to the screen coordinates */

    if (x <= graph->xoffset)
    {
        *frame = 0;
    }
    else if ( x < (graph->xoffset + graph->width) )
    {
        *frame = (unsigned long int)
                ((double)( x - graph->xoffset) / graph->deltax);
    }
    else
    {
        *frame = graph->end_frame;
    }


    if (y <= graph->yoffset)
    {
        *percent = 100.0;
    }
    else if (y < graph->bottom)
    {
        *percent =  ((double)(graph->bottom - y)) / graph->deltay ;
    }
    else
    {
        *percent = 0.0;
    }

    return(OK);

}


/* return graph values for an index */

int get_graph_index_values( struct graph_struct *graph,
                            unsigned long int *frame, double *percent,
                            double *tension, double *bias, double *cont,
                            int *linear)
{

    /* error checking */

    if (graph->current_index >= graph->num_points)
    {
        log_message(ERROR_MSG,"Invalid index in get_graph_index_values()\n");

        return(ERROR);
    }

    /* return values */

    *frame = graph->frame[graph->current_index];
    *percent = graph->percent[graph->current_index];
    *bias = graph->spline[graph->current_index].bias;
    *tension = graph->spline[graph->current_index].tension;
    *cont = graph->spline[graph->current_index].cont;
    *linear = graph->spline[graph->current_index].linear;



    return(OK);
}

/* set the spline parameters for the current index */

int set_graph_index_spline( struct graph_struct *graph,
                            double tension, double bias, double cont,
                            int linear)
{

    /* error checking */

    if (graph->current_index >= graph->num_points)
    {
        log_message(ERROR_MSG,"Invalid index in set_graph_index_spline()\n");
        return( ERROR);
    }

    graph->spline[graph->current_index].tension = tension;
    graph->spline[graph->current_index].bias = bias;
    graph->spline[graph->current_index].cont = cont;
    graph->spline[graph->current_index].linear = linear;

    return(OK);
}



/* set the graph current index point */

int set_graph_current_index( struct graph_struct *graph, int index)
{

    if ( (index >= graph->num_points) || (index < 0) )
    {
        log_message(ERROR_MSG,"Invalid index in set_graph_current_index()\n");
        return(ERROR);
    }

    graph->current_index = index;

    return(OK);
}


/* get the value of the current index for a graph */

int get_graph_current_index( struct graph_struct *graph, int *index)
{

    *index = graph->current_index;

    return(OK);
}


/* return the last index in the graph */

int last_graph_index( struct graph_struct *graph)
{

    return( (graph->num_points - 1) );
}




/* find nearest graph point index */


int find_nearest_graph_index( struct graph_struct *graph,
                                unsigned long int frame, int *index)
{

    int lower_index, higher_index;
    long int lower_delta, higher_delta;

    /* find the closest graph point (if equal, favor the lower) */

    /* skip error checking */

    find_graph_points( graph, frame, &lower_index, &lower_delta,
                                     &higher_index, &higher_delta);


    if (lower_delta > higher_delta)
    {
        *index = higher_index;
    }
    else
    {
        *index = lower_index;
    }

//    printf("lower_index = %d  lower_delta = %ld\n", lower_index, lower_delta);
//    printf("higher_index = %d  higher_delta = %ld\n", higher_index, higher_delta);
//    printf("index = %d\n", *index);

    return(OK);
}




/* find point above and below a specified point */
int find_graph_points( struct graph_struct *graph, unsigned long int frame,
                        int *lower_index, long int *lower_delta,
                        int *higher_index, long int *higher_delta)
{
    int     index, quitflag;


    /* error checking (graph, frame number) */
    /*!! skip for speed right now !!*/

    /* actions: */
    /*  Note: frame is unsigned and frame count must start with 0, so no */
    /*        underflow is possible */

    /*  (1.  frame is between first and last point frame (normal): */
    /*        will find index of lower and higher point */
    /*
    /*  (2.  frame is greater than last point frame (overflow): */
    /*        will give last point frame as both lower and higer index */



    /* start from bottom and find the last point smaller than the frame */
    /*!! a linear search should be OK for the small number of points we */
    /*!! will have.  May want to do a bisection search if it gets too long */

    index = 0;
    quitflag = FALSE;

    while (!quitflag)
    {
        if ((graph->frame[index] <= frame)&&(graph->frame[index+1] >= frame))
        {
            quitflag = TRUE;
            *lower_index = index;
            *higher_index = index+1;
            *lower_delta = frame - graph->frame[index];
            *higher_delta = graph->frame[(index+1)] - frame;
        }
        else
        {
            index++;
            if (index == graph->num_points)
            {
                quitflag = TRUE;

                /* we have overflowed, report last frame */
                index--;
                *lower_index = index;
                *higher_index = index;
                *lower_delta = frame - graph->frame[index];
                *higher_delta = *lower_delta;

            }
        }

    }

    return(OK);
}




/* insert point */
int insert_graph_point( struct graph_struct *graph, unsigned long int frame,
                        double percent, double tension, double bias,
                        double cont, int linear)
{

    int     index, quitflag, i, i1;

    /* error checking (graph, frame number, point overflow or precent bounds)*/
    /* note that we are not allowed to insert points at frame 0 or the last */
    /*  frame */

    if ( graph->status != INITIALIZED )
    {
        log_message(ERROR_MSG,"Graph not init in insert_graph_point()\n");
        return(ERROR);
    }

    if (graph->num_points == graph->max_num_frames)
    {
        log_message(ERROR_MSG,"Index overflow in insert_graph_point()\n");
        return(ERROR);
    }

    if( (frame >= graph->end_frame) || (percent > graph->max_percent)  ||
        (frame == 0) )
    {
        log_message(ERROR_MSG,
                    "Invalid frame or percent in insert_graph_point()\n");
        return(ERROR);
    }



    /* find location to insert in graph array */
    /* simple linear search */
    quitflag = FALSE;
    index = 0;

    while (!quitflag)
    {
        if (graph->frame[index] >= frame)
        {
            quitflag = TRUE;
        }
        else
        {
            index++;
            if (index == graph->num_points)
            {
                /* run out of points, quit now (we should never reach this */
                /*  this case */
                log_message(ERROR_MSG,
                            "Point overlfow in insert_graph_point()\n");
                return(ERROR);
            }
        }
    }

    /* if the new frame is already a key point, return */

    if (graph->frame[index] == frame)
    {
        log_message(INFO_MSG,"Invalid index in get_graph_index_values()\n");
        return(ERROR);
    }


    /* move the higer points up in the point and percent arrays */
    for(i = graph->num_points; i >= index; i--)
    {
    	i1 = i - 1;

        graph->frame[i] = graph->frame[i1];
        graph->percent[i] = graph->percent[i1];
        graph->spline[i].tension = graph->spline[i1].tension;
        graph->spline[i].bias = graph->spline[i1].bias;
        graph->spline[i].cont = graph->spline[i1].cont;
        graph->spline[i].linear = graph->spline[i1].linear;

        /* no need to update the x, y coords, will recalc anyway */

    }

    /* insert the new point, increment count */

        graph->frame[index] = frame;
        graph->percent[index] = percent;
        graph->spline[index].tension = tension;
        graph->spline[index].bias = bias;
        graph->spline[index].cont = cont;
        graph->spline[index].linear = linear;

        graph->num_points++;

//    /* adjust the current_index */
//
//        if (index <= graph->current_index)
//        {
//            graph->current_index++;
//
//        }

        /* make the new point the current index */
        graph->current_index = index;


    return(OK);
}




/* delete point */
int delete_graph_point( struct graph_struct *graph, int frame_index)
{
    int i, i1;

    /* error checking (graph, frame index number point underflow) */
    if (graph->status != INITIALIZED)
    {
        log_message(ERROR_MSG,"Uninitialize graph in delete_graph_point()\n");
        return(ERROR);
    }

    if( (frame_index >= (graph->num_points-1)) || ( graph->num_points == 2) ||
        (frame_index == 0) )
    {
        /* can't have less than 2 points in graph) */

        log_message(INFO_MSG,
                "Attempt to delete below 2 pts in delete_graph_point()\n");

        return(ERROR);
    }

    /* move the higher points down in the point and percent arrays */

    for( i=frame_index; i<(graph->num_points - 1); i++)
    {
        i1 = i+1;
        graph->frame[i] = graph->frame[i1];
        graph->percent[i] = graph->percent[i1];
        graph->spline[i].tension = graph->spline[i1].tension;
        graph->spline[i].bias = graph->spline[i1].bias;
        graph->spline[i].cont = graph->spline[i1].cont;
        graph->spline[i].linear = graph->spline[i1].linear;

        /* x and y will be re-calculated anyway */
    }

    graph->num_points--;

    if (frame_index <= graph->current_index)
    {
        graph->current_index--;
    }

    return(OK);
}


/* redraw graph */
/* note:  This assumes that the rastport has a properly initialized */
/*        set of graphics buffers */


int redraw_graph( struct RastPort *rport, struct graph_struct *graph)
{

    int             i, end_index;

    /* error checking */
    if ( (rport == NULL) || (graph->status != INITIALIZED) )
    {
        log_message(ERROR_MSG,
                "Bad RPORT or graph uninit in redraw_graph()\n");

        return (ERROR);
    }


    /* blank the graph area */

    SetAPen(rport,graph->bg_pen);
    SetDrMd(rport, JAM1);

    RectFill(rport, graph->xoffset, graph->yoffset,
                    (graph->xoffset + graph->width),
                    (graph->yoffset + graph->height) );


    /* Draw the graph and draw points */

    SetAPen(rport, graph->fg_pen);
    SetBPen(rport, graph->bg_pen);
    SetDrMd(rport, JAM1);

    end_index = graph->num_points - 1;

    for(i=0; i<end_index; i++)
    {

        draw_graph_segment(rport, graph, i, FALSE);

        draw_box(rport, graph, graph->pbuffx[0], graph->pbuffy[0], 3);

    }

    /* draw the box for the last point */

    i = graph->num_pbuff - 1;

    draw_box(rport, graph, graph->pbuffx[i], graph->pbuffy[i], 3);


    hilight_current_index(rport, graph );

    return(OK);

}


/* draw a segment of the graph, either as a linear segment or a spline */
/* draws from (index) to (index + 1).  It will save the points drawn */
/* in the pbuffx and pbuffy segments for use by the redraw_segment function */
/* the spline segments will use the pre-calculated parameter values stored */
/* in the graph structure.  No error checking is done to find out if these */
/* values are actually vaild.  We also assume that there are at least 2  */
/* points available in the pbuff buffers */


int draw_graph_segment( struct RastPort *rport, struct graph_struct *graph,
                        int index, int append)
{


    /* note: 0 is previous (i-1), i is index, 1 is next (i+1) */

    WORD                x1,y1, x2, y2, x0, y0, x3, y3;
    int                 i;
    double              r1x, r1y, r2x, r2y;
    int                 nextindex, previndex, nextindex1;
    double              tensioni, biasi, conti;
    double              tension1, bias1, cont1;
    double              temp_x, temp_y;
    double              temp1, temp2, temp1a, temp2a;
    double              cx0, cx1, cx2, cx3, cy0, cy1, cy2, cy3;
    double              p0x, p0y, pix, piy, p1x, p1y, p2x, p2y;



    if (index >= graph->num_points)
    {
        log_message(ERROR_MSG, "Invalid index in draw_graph_segment()\n");
        return(ERROR);
    }

    nextindex = index + 1;
    if (nextindex >= (graph->num_points - 1) )
    {
        nextindex = graph->num_points - 1;
    }

    nextindex1 = nextindex + 1;
    if (nextindex1 >= (graph->num_points - 1) )
    {
        nextindex1 = graph->num_points - 1;
    }
    previndex = index - 1;
    if (previndex < 0)
    {
        previndex = 0;
    }


    /* if we are not appending points, reset the pbuff counter, otherwise, keep */
    /* going! */

    if (append == FALSE)
    {
        graph->num_pbuff = 0;
    }


    map_graph_screen(graph, graph->frame[previndex], graph->percent[previndex],
                         &x0, &y0);

    map_graph_screen(graph, graph->frame[index], graph->percent[index],
                         &x1, &y1);


    map_graph_screen(graph, graph->frame[nextindex], graph->percent[nextindex],
                         &x2, &y2);


    map_graph_screen(graph,graph->frame[nextindex1],graph->percent[nextindex1],
                         &x3, &y3);


    /* go ahead and save the x and y values for the endpoints (used by other */
    /* functions) */

    graph->x[index] = x1;
    graph->y[index] = y1;
    graph->x[nextindex] = x2;
    graph->y[nextindex] = y2;


    /* first test for a linear segment (the easy one!) */

    if (graph->spline[index].linear == TRUE)
    {

        graph->pbuffx[graph->num_pbuff] = x1;
        graph->pbuffy[graph->num_pbuff] = y1;
        graph->num_pbuff++;
        graph->pbuffx[graph->num_pbuff] = x2;
        graph->pbuffy[graph->num_pbuff] = y2;
        graph->num_pbuff++;

        Move(rport, x1, y1);
        Draw(rport, x2, y2);
    }
    else if (graph->spline[index].linear == FALSE)
    {

        tensioni = graph->spline[index].tension;
        biasi = graph->spline[index].bias;
        conti = graph->spline[index].cont;

        tension1 = graph->spline[nextindex].tension;
        bias1 = graph->spline[nextindex].bias;
        cont1 = graph->spline[nextindex].cont;

        p0x = (double)x0;
        p0y = (double)y0;
        pix = (double)x1;
        piy = (double)y1;
        p1x = (double)x2;
        p1y = (double)y2;
        p2x = (double)x3;
        p2y = (double)y3;

//printf("%lf  %lf\n", p0x, p0y);
//printf("%lf  %lf\n", pix, piy);
//printf("%lf  %lf\n", p1x, p1y);


//printf("px %lf  %lf  %lf\n", p0x, pix, p1x);



        /* calculate R1 and R2 */

        temp1 = (1.0-tensioni) * (1.0+biasi) * (1.0+conti)/2.0;
        temp2 = (1.0-tensioni) * (1.0-biasi) * (1.0-conti)/2.0;

        r1x = ( (temp1 * (double)(pix - p0x)) +
                (temp2 * (double)(p1x - pix)) );

        r1y = ( (temp1 * (double)(piy - p0y)) +
                (temp2 * (double)(p1y - piy)) );


        temp1a = (1.0-tension1) * (1.0+bias1) * (1.0-cont1)/2.0;
        temp2a = (1.0-tension1) * (1.0-bias1) * (1.0+cont1)/2.0;

        r2x = ( (temp1a * (double)(p1x - pix)) +
                (temp2a * (double)(p2x - p1x)) );

        r2y = ( (temp1a * (double)(p1y - piy)) +
                (temp2a * (double)(p2y - p1y)) );

//printf("r1  %lf   %lf\n", r1x,  r1y);
//printf("r2  %lf   %lf\n", r2x,  r2y);

        /* G =  [P1] */
        /*      [P2] */
        /*      [r1] */
        /*      [r2] */

        /* M = (for Hermite Curves) [ 2 -2  1  1] */
        /*                          [-3  3 -2 -1] */
        /*                          [ 0  0  1  0] */
        /*                          [ 1  0  0  0] */


        /* precalculate the Cx = M.Gx and Cy = M.Gy) */

        cx0 = (2.0*pix) - (2.0*p1x) + (r1x) + (r2x);
        cx1 = (-3.0*pix) + (3.0*p1x) - (2.0*r1x) - (r2x);
        cx2 = r1x;
        cx3 = pix;

        cy0 = (2.0*piy) - (2.0*p1y) + (r1y) + (r2y);
        cy1 = (-3.0*piy) + (3.0*p1y) - (2.0*r1y) - (r2y);
        cy2 = r1y;
        cy3 = piy;


        for(i=0; i<graph->num_t_steps; i++)
        {
            temp_x =(cx0*graph->t3[i]) + (cx1*graph->t2[i]) +
                    (cx2*graph->t[i]) + cx3;

            temp_y =(cy0*graph->t3[i]) + (cy1 * graph->t2[i]) +
                             (cy2*graph->t[i]) + cy3;


//printf("t = %lf, t2= %lf, t3 = %lf\n", graph->t[i], graph->t2[i], graph->t3[i]);
//printf("x = %lf  = %ld, y = %lf\n", temp_x, (unsigned long int)(temp_x), temp_y);

//            map_graph_screen(graph, (unsigned long int)(temp_x),
//                                temp_y, &x1, &y1);

            x1 = (WORD)temp_x;
            if (x1 < graph->left)
            {
                x1 = graph->left;
            }
            else if (x1 > graph->right)
            {
                x1 = graph->right;
            }




            y1 = (WORD)temp_y;
            if (y1 < graph->top)
            {
                y1 = graph->top;
            }
            else if (y1 > graph->bottom)
            {
                y1 = graph->bottom;
            }



            if (i == 0)
            {
                Move(rport, x1, y1);
            }
            else
            {
                Draw(rport, x1, y1);
            }

            graph->pbuffx[graph->num_pbuff] = x1;
            graph->pbuffy[graph->num_pbuff] = y1;
            graph->num_pbuff++;


        }

    }


    return(OK);

}


/* function to generate the actual field/percent values for a graph segment */
/* if the segment is linear, it will just return the end points, otherwise it*/
/* will fill out the data array with the same number of data points as is */
/* used to generate the display curves.  */

/* It is the responsiblity of the calling program to discard points that */
/* are not useable (for example, the starting point of a segment is not */
/* usually needed since it is the same as the ending point of the previous */
/* segment.  Points may round to the same field as a previous point or even */
/* an earlier field number, these must be clipped */


int generate_segment_points( struct graph_struct *graph, int index,
                int max_points, unsigned long int *fields,
                double *percent, int *num_points)

{


    /* note: 0 is previous (i-1), i is index, 1 is next (i+1) */

    int                 i;
    double              r1x, r1y, r2x, r2y;
    int                 nextindex, previndex, nextindex1;
    double              tensioni, biasi, conti;
    double              tension1, bias1, cont1;
    double              temp_x, temp_y;
    double              temp1, temp2, temp1a, temp2a;
    double              cx0, cx1, cx2, cx3, cy0, cy1, cy2, cy3;
    double              p0x, p0y, pix, piy, p1x, p1y, p2x, p2y;


    /* REMEMBER! We are working FIELDS now, not FRAMES */


    *num_points = 0;

    if(  (index >= (graph->num_points - 1)) ||
         (max_points > graph->num_t_steps) )
    {
        log_message(ERROR_MSG,
            "Invalid index or num pts in generate_segment_points()\n");
        return(ERROR);
    }

    nextindex = index + 1;
    if (nextindex >= (graph->num_points - 1) )
    {
        nextindex = graph->num_points - 1;
    }

    nextindex1 = nextindex + 1;
    if (nextindex1 >= (graph->num_points - 1) )
    {
        nextindex1 = graph->num_points - 1;
    }
    previndex = index - 1;
    if (previndex < 0)
    {
        previndex = 0;
    }


    p0x = (double)graph->frame[previndex] * 2.0;  /* remember: fields */
    p0y = graph->percent[previndex];
    pix = (double)graph->frame[index] * 2.0;
    piy = graph->percent[index];
    p1x = (double)graph->frame[nextindex] * 2.0;
    p1y = graph->percent[nextindex];
    p2x = (double)graph->frame[nextindex1] * 2.0;
    p2y = graph->percent[nextindex1];



    /* first test for a linear segment (the easy one!) */

    if (graph->spline[index].linear == TRUE)
    {

        fields[0] = (unsigned long int)pix;
        percent[0] = piy;

        fields[1] = (unsigned long int)p1x;
        percent[1] = p1y;

        *num_points = 2;

    }
    else if (graph->spline[index].linear == FALSE)
    {

        tensioni = graph->spline[index].tension;
        biasi = graph->spline[index].bias;
        conti = graph->spline[index].cont;

        tension1 = graph->spline[nextindex].tension;
        bias1 = graph->spline[nextindex].bias;
        cont1 = graph->spline[nextindex].cont;



        /* calculate R1 and R2 */

        temp1 = (1.0-tensioni) * (1.0+biasi) * (1.0+conti)/2.0;
        temp2 = (1.0-tensioni) * (1.0-biasi) * (1.0-conti)/2.0;

        r1x = ( (temp1 * (double)(pix - p0x)) +
                (temp2 * (double)(p1x - pix)) );

        r1y = ( (temp1 * (double)(piy - p0y)) +
                (temp2 * (double)(p1y - piy)) );


        temp1a = (1.0-tension1) * (1.0+bias1) * (1.0-cont1)/2.0;
        temp2a = (1.0-tension1) * (1.0-bias1) * (1.0+cont1)/2.0;

        r2x = ( (temp1a * (double)(p1x - pix)) +
                (temp2a * (double)(p2x - p1x)) );

        r2y = ( (temp1a * (double)(p1y - piy)) +
                (temp2a * (double)(p2y - p1y)) );

        /* G =  [P1] */
        /*      [P2] */
        /*      [r1] */
        /*      [r2] */

        /* M = (for Hermite Curves) [ 2 -2  1  1] */
        /*                          [-3  3 -2 -1] */
        /*                          [ 0  0  1  0] */
        /*                          [ 1  0  0  0] */


        /* precalculate the Cx = M.Gx and Cy = M.Gy) */

        cx0 = (2.0*pix) - (2.0*p1x) + (r1x) + (r2x);
        cx1 = (-3.0*pix) + (3.0*p1x) - (2.0*r1x) - (r2x);
        cx2 = r1x;
        cx3 = pix;

        cy0 = (2.0*piy) - (2.0*p1y) + (r1y) + (r2y);
        cy1 = (-3.0*piy) + (3.0*p1y) - (2.0*r1y) - (r2y);
        cy2 = r1y;
        cy3 = piy;


        for(i=0; i<graph->num_t_steps; i++)
        {
            temp_x =(cx0*graph->t3[i]) + (cx1*graph->t2[i]) +
                    (cx2*graph->t[i]) + cx3;

            temp_y =(cy0*graph->t3[i]) + (cy1 * graph->t2[i]) +
                             (cy2*graph->t[i]) + cy3;



            /* clip field to positive values */
            if ( temp_x < 0.0 )
            {
                fields[i] = 0;
            }
            else
            {

                fields[i] = (unsigned long int)(temp_x + 0.5);
            }

            /* clip percent to 0 - 100 */

            if (temp_y > 100.0)
            {
                percent[i] = 100.0;
            }
            else if (temp_y < 0.0)
            {
                percent[i] = 0.0;
            }
            else
            {
                percent[i] = temp_y;
            }

            *num_points = *num_points + 1;

        }

    }


    return(OK);

}



/* a function to redraw the segment currently stored in the graph's */
/* point buffer.  Very little error checking is done for speed purposes */



int redraw_pbuff( struct RastPort *rport, struct graph_struct *graph)
{

    int     i;


    Move( rport, graph->pbuffx[0], graph->pbuffy[0]);

    for(i=1; i < graph->num_pbuff; i++)
    {

        Draw(rport, graph->pbuffx[i], graph->pbuffy[i]);

    }


    return(OK);
}




/* a function to change and re-draw the splines to and from an index point */
/* it first blanks the spline with the old parameters, then re-draws it with */
/* the new parameters */


int change_index_spline( struct RastPort *rport, struct graph_struct *graph,
                         double new_tension, double new_bias,
                         double new_cont, int new_linear)
{

    int         index, previndex, nextindex;


    index = graph->current_index;

    previndex = index - 1;
    if (previndex < 0)
    {
        previndex = 0;
    }

    nextindex = index + 1;
    if (nextindex > (graph->num_points - 1) )
    {
        nextindex--;
    }


    /* setup to erase the old lines */
    SetAPen(rport, graph->bg_pen);

    draw_graph_segment(rport, graph, previndex, FALSE);

    draw_graph_segment(rport, graph, index, FALSE);

    /* change the parameters and redraw */
    graph->spline[index].tension = new_tension;
    graph->spline[index].bias = new_bias;
    graph->spline[index].cont = new_cont;
    graph->spline[index].linear = new_linear;

    SetAPen(rport, graph->fg_pen);

    draw_graph_segment(rport, graph, previndex, FALSE);

    draw_graph_segment(rport, graph, index, FALSE);

    draw_box(rport, graph, graph->x[previndex], graph->y[previndex], 3);
    draw_box(rport, graph, graph->x[index], graph->y[index], 3);
    draw_box(rport, graph, graph->x[nextindex], graph->y[nextindex], 3);

    hilight_current_index(rport, graph);


    return(OK);
}




/* move a keyframe index */
/* note that this function blanks the lines and redraws the points */


int move_graph_point( struct RastPort *rport, struct graph_struct *graph,
                        int index, unsigned long int new_frame,
                        double new_percent)
{


    /* note: lower_index1 = index-2, lower_index = index-1, */
    /*       higher_index = index+1 */

    int                     lower_index, lower_index1, higher_index;
    unsigned long int       temp_frame;
    double                  temp_percent;
    WORD                    tempx, tempy;
    int                     drawlower = TRUE;
    int                     drawupper = TRUE;





    /* basic error checking */
    if (graph->status != INITIALIZED)
    {
        log_message(ERROR_MSG,"invalid graph in move_graph_point()\n");
        return(ERROR);
    }

    if ( (index < 0) || (index >= graph->num_points) )
    {
        log_message(ERROR_MSG,"invalid index in move_graph_point()\n");
        return(ERROR);
    }


    /* Note: we are not allowed to shift first or last keyframe, only vary */
    /* the percent level */

    /* the new keyframe value must be bounded by the keyframes above and below */
    /* it, if it is the first or last keyframe, it is not allowd to move at all */

    if (index == 0)
    {
        /* it is first keyframe, do not allow new frame */
        temp_frame = 0;
        lower_index = 0;
        lower_index1 = 0;
        higher_index = 1;
        drawlower = FALSE;

    }
    else if (index == (graph->num_points - 1) )
    {
        /* it is last frame, do not allow new frame */
        temp_frame = graph->frame[(graph->num_points-1)];
        lower_index = index-1;
        lower_index1 = index-2;
        higher_index = index;
        drawupper = FALSE;
    }
    else
    {
        temp_frame = new_frame;

        /* clip frame to lower and higher indices */
        lower_index = index - 1;
        lower_index1 = index - 2;
        higher_index = index + 1;

        if (temp_frame <= graph->frame[lower_index])
        {
            temp_frame = graph->frame[lower_index] + 1;
        }
        else if (temp_frame >= graph->frame[higher_index])
        {
            temp_frame = graph->frame[higher_index] - 1;
        }
    }

    /* clip the lower_index1 index value */
    if(lower_index1 < 0)
    {
        lower_index1 = 0;
    }


    /* now clip percentage */
    temp_percent = new_percent;

    if (temp_percent < 0.0 )
    {
        temp_percent = 0.0;
    }
    else if (temp_percent > graph->max_percent )
    {
        temp_percent = graph->max_percent;
    }


    /* now find the new coordinates */

    map_graph_screen(graph, temp_frame, temp_percent, &tempx, &tempy);

    /* erase the old lines and draw the new ones */

    SetAPen(rport, graph->bg_pen);
//    Move(rport, graph->x[lower_index], graph->y[lower_index]);
//    Draw(rport, graph->x[index], graph->y[index]);
//    Draw(rport, graph->x[higher_index], graph->y[higher_index]);


    draw_graph_segment(rport, graph, lower_index1, FALSE);
    draw_graph_segment(rport, graph, lower_index, FALSE);
    draw_graph_segment(rport, graph, index, FALSE);
    draw_graph_segment(rport, graph, higher_index, FALSE);

    draw_box(rport, graph, graph->x[lower_index1], graph->y[lower_index1], 3);
    draw_box(rport, graph, graph->x[lower_index], graph->y[lower_index], 3);
    draw_box(rport, graph, graph->x[index], graph->y[index], 3);
    draw_box(rport, graph, graph->x[higher_index], graph->y[higher_index], 3);


    /* save the new values */
    graph->frame[index] = temp_frame;
    graph->percent[index] = temp_percent;
    graph->x[index] = tempx;
    graph->y[index] = tempy;


//    draw_box(rport, graph, graph->x[index], graph->y[index], 3);

    SetAPen(rport, graph->fg_pen);

    draw_graph_segment(rport, graph, lower_index1, FALSE);
    draw_graph_segment(rport, graph, lower_index, FALSE);
    draw_graph_segment(rport, graph, index, FALSE);
    draw_graph_segment(rport, graph, higher_index, FALSE);


    draw_box(rport, graph, graph->x[lower_index1], graph->y[lower_index1], 3);
    draw_box(rport, graph, graph->x[lower_index], graph->y[lower_index], 3);
    draw_box(rport, graph, graph->x[index], graph->y[index], 3);
    draw_box(rport, graph, graph->x[higher_index], graph->y[higher_index], 3);



//    Move(rport, graph->x[lower_index], graph->y[lower_index]);

//    if (drawlower == TRUE)
//    {
//        Draw(rport, tempx, tempy);
//    }
//    else
//    {
//        Move(rport, tempx, tempy);
//    }
//
//    if (drawupper == TRUE)
//    {
//        Draw(rport, graph->x[higher_index], graph->y[higher_index]);
//    }
//
//    if (drawlower == TRUE)
//    {
//        draw_box(rport, graph, graph->x[lower_index], graph->y[lower_index],3);
//    }
//
//    draw_box(rport, graph, tempx, tempy, 3);
//
//   if (drawupper == TRUE)
//    {
//       draw_box(rport, graph,graph->x[higher_index],graph->y[higher_index],3);
//    }

    hilight_current_index(rport, graph );


    return(OK);
}



/* clear all graph points */

int clear_graph_points( struct graph_struct *graph)
{

    if (graph->status != INITIALIZED)
    {
        log_message(ERROR_MSG,"uninitialized graph in clear_graph_points()\n");
        return(ERROR);
    }

    graph->num_points = 0;

    return(OK);
}


/* change graph lenght and scale any graph points loaded in it */

int change_graph_length ( struct graph_struct *graph,
                            unsigned long int new_length)
{
    int                 i;
    double              scale, temp_dbl;
    unsigned long int   temp_frame;
    unsigned long int   old_length;


    if (graph->status != INITIALIZED)
    {
        log_message(ERROR_MSG,"uninit graph in change_graph_lenght()\n");
        return(ERROR);
    }

    old_length = graph->end_frame + 1;

    graph->end_frame = (new_length - 1);


    if (old_length == 0)
    {
        /* graph has no current length, no point scaling will be necessary */
        return(OK);
    }


    /* change graph x scalefactor */

    graph->deltax = (double)graph->width / (double)(graph->end_frame+1);



    /* setup to scale data points, don't bother if graph is empty */

    if (graph->num_points == 0)
    {
        return(OK);
    }


    scale = ((double)(new_length)) / ((double)(old_length));


// printf("scale = %lf\n", scale);


    /* process the points, don't process the first or last point, they
    /* will be explicitly set later */

    i = 1;
    while ( i < (graph->num_points - 1) )
    {

        temp_dbl = scale * (double)(graph->frame[i]);

        temp_frame = (unsigned long int)( (scale * (double)(graph->frame[i])) );

        /* round to closest frame frame */

        temp_frame = (unsigned long int)(temp_dbl);

        if ( (temp_dbl - (double)(temp_frame)) >= 0.5 )
        {
            temp_frame++;
        }


//        /* force to colorframe */
//        if( (temp_frame % 2) != 0 )
//        {
//            temp_frame--;
//        }

//printf("temp_frame = %ld\n", temp_frame);

//!!!! add checks to make sure that the frames to not overlap each
//!!!! other or clip to 0

        if (i != 0)
        {

            /* There are 3 cases where we will want to delete a point */
            /* (1.  The scaled frame value has rounded to 0 */
            /* (2.  The scaled frame value has rounded so that is the */
            /*        same as the previous index */
            /* (3.  The scaled value is equal or greater to the last frame */
            /*        (possible for large reduction)*/


            if( (temp_frame <= graph->frame[i-1]) ||
                (temp_frame == 0) ||
                (temp_frame >= graph->end_frame) )
            {
               delete_graph_point( graph, i);
                /* decrement the item counter, so we'll the new i'th point */
                i--;

            }
            else
            {
                graph->frame[i] = temp_frame;
            }

        }

        i++;
    }


    /* force last frame to end of graph, in case it got rounded down */
    graph->frame[(graph->num_points - 1)] = graph->end_frame;

    return(OK);
}





#ifdef UNCONVERTED






/* blank and redraw the axes for the graph */
/*  draw a tic mark every 10 frames and every 20 percent */

int redraw_axis( struct RastPort *rport, struct graph_struct *graph,
                    int x_flag, int y_flag)
{

    long int            frame;
    long int            increment;
    WORD                x, y, bottom, ticbottom;
    int                 zerominor_flag;



    bottom = graph->bottom + 1;
    ticbottom = bottom + graph->xticheight;

    /* clear and draw x axis */
    if (x_flag == TRUE)
    {
        /* clear the old x axis region */
        SetAPen(rport, graph->bg_pen);
        SetDrMd(rport, JAM1);

        RectFill(rport, graph->left, bottom,
                        (graph->left + graph->width),
                        ticbottom );

        SetAPen(rport, graph->fg_pen);

        /* draw the new tic points */
        /* find the smallest increment */
        if (graph->xminorincrement == 0)
        {
            zerominor_flag == TRUE;
            increment = graph->xmajorincrement;
        }
        else
        {
            /* simplifying assumption, major increment is a multiple */
            /* of minor increment */

            zerominor_flag == FALSE;
            increment = graph->xminorincrement;
        }

        Move(rport, graph->left, bottom);
        for( frame = 0; frame < graph->end_frame; frame += increment)
        {


!!!!!!!!!




    }



    return(OK);
}






int move_graph_point( struct RastPort *rport, struct graph_struct *graph,
                        int index, unsigned long int new_frame,
                        double new_percent)
{


    int                     lower_index, higher_index;
    unsigned long int       temp_frame;
    double                  temp_percent;
    WORD                    tempx, tempy;
    int                     drawlower = TRUE;
    int                     drawupper = TRUE;




    /* basic error checking */
    if (graph->status != INITIALIZED)
    {
        log_message(ERROR_MSG,"uninit graph in move_graph_point()\n");
        return(ERROR);
    }

    if ( (index < 0) || (index >= graph->num_points) )
    {
        log_message(ERROR_MSG,"invalid index in move_graph_point()\n");
        return(ERROR);
    }


    /* Note: we are not allowed to shift first or last keyframe, only vary */
    /* the percent level */

    /* the new keyframe value must be bounded by the keyframes above and below */
    /* it, if it is the first or last keyframe, it is not allowd to move at all */

    if (index == 0)
    {
        /* it is first keyframe, do not allow new frame */
        temp_frame = 0;
        lower_index = 0;
        higher_index = 1;
        drawlower = FALSE;

    }
    else if (index == (graph->num_points - 1) )
    {
        /* it is last frame, do not allow new frame */
        temp_frame = graph->frame[(graph->num_points-1)];
        lower_index = index-1;
        higher_index = index;
        drawupper = FALSE;
    }
    else
    {
        temp_frame = new_frame;

        /* clip frame to lower and higher indices */
        lower_index = index - 1;
        higher_index = index + 1;

        if (temp_frame <= graph->frame[lower_index])
        {
            temp_frame = graph->frame[lower_index] + 1;
        }
        else if (temp_frame >= graph->frame[higher_index])
        {
            temp_frame = graph->frame[higher_index] - 1;
        }
    }


    /* now clip percentage */
    temp_percent = new_percent;

    if (temp_percent < 0.0 )
    {
        temp_percent = 0.0;
    }
    else if (temp_percent > graph->max_percent )
    {
        temp_percent = graph->max_percent;
    }


    /* now find the new coordinates */

    map_graph_screen(graph, temp_frame, temp_percent, &tempx, &tempy);

    /* erase the old lines and draw the new ones */

    SetAPen(rport, graph->bg_pen);
    Move(rport, graph->x[lower_index], graph->y[lower_index]);
    Draw(rport, graph->x[index], graph->y[index]);
    Draw(rport, graph->x[higher_index], graph->y[higher_index]);


    draw_box(rport, graph, graph->x[index], graph->y[index], 3);

    SetAPen(rport, graph->fg_pen);
    Move(rport, graph->x[lower_index], graph->y[lower_index]);

    if (drawlower == TRUE)
    {
        Draw(rport, tempx, tempy);
    }
    else
    {
        Move(rport, tempx, tempy);
    }

    if (drawupper == TRUE)
    {
        Draw(rport, graph->x[higher_index], graph->y[higher_index]);
    }

    if (drawlower == TRUE)
    {
        draw_box(rport, graph, graph->x[lower_index], graph->y[lower_index],3);
    }

    draw_box(rport, graph, tempx, tempy, 3);

    if (drawupper == TRUE)
    {
        draw_box(rport, graph,graph->x[higher_index],graph->y[higher_index],3);
    }

    /* save the new values */
    graph->frame[index] = temp_frame;
    graph->percent[index] = temp_percent;
    graph->x[index] = tempx;
    graph->y[index] = tempy;


    hilight_current_index(rport, graph );


    return(OK);
}



#endif /* UNCONVERTED */


/*--- Spare Template ---*/


/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/

