/****************************************************************************
**
**                          graph_lib.h
**
**
**
****************************************************************************/


/* make sure file is only included once */

#ifndef GRAPH_LIB_H

#define GRAPH_LIB_H


struct spline_params_struct {
    double              bias, tension, cont;
    int                 linear;
};

struct graph_struct {
    int                 status;
    WORD                xoffset;
    WORD                yoffset;
    WORD                width;
    WORD                height;
    WORD                xtic_majorheight;
    WORD                xtic_minorheight;
    WORD                ytic_majorwidth;
    WORD                ytic_minorwidth;
    int                 xtic_majorinterval;
    int                 xtic_minorinterval;
    int                 ytic_majorinterval;
    int                 ytic_minorinterval;
    int                 max_num_frames;
    double              max_percent;
    unsigned long int   end_frame;
    double              deltax;
    double              deltay;
    int                 num_points;
    unsigned long int   *frame;
    double              *percent;
    WORD                *x;
    WORD                *y;
    struct spline_params_struct    *spline;
    WORD                *pbuffx;                /* bufer for curve redraws */
    WORD                *pbuffy;
    int                 max_pbuff;
    int                 num_pbuff;
    int                 current_index;

    /* curve parameter steps */
    int                 num_t_steps;            /* # of spline steps */
    double              t_step;                 /* = 1 / num_t_steps */
    double              *t;
    double              *t2;
    double              *t3;

    /* colors for drawing */
    UBYTE               bg_pen;
    UBYTE               fg_pen;
    UBYTE               fill_pen;
    UBYTE               border_pen;
    UBYTE               hilite_pen;

    /* borders for drawing (xoffset is same as left, yoffset is same as top) */
    WORD                left;
    WORD                top;
    WORD                right;
    WORD                bottom;

    };


/*-- prototypes --*/
struct graph_struct *init_graph( WORD xoffset, WORD yoffset, WORD width,
                                 WORD height, WORD xticheight,
                                 WORD yticwidth, unsigned long int end_frame,
                                 double max_percent, int max_num_frames,
                                 int max_pbuff, int num_t_steps);


void free_graph(struct graph_struct *graph);

int redraw_graph( struct RastPort *rport, struct graph_struct *graph);

int redraw_axis( struct RastPort *rport, struct graph_struct *graph,
                    int x_flag, int y_flag);

int insert_graph_point( struct graph_struct *graph, unsigned long int frame,
                        double percent, double tension, double bias,
                        double cont, int linear);

int delete_graph_point( struct graph_struct *graph, int frame_index);

int find_graph_points( struct graph_struct *graph, unsigned long int frame,
                        int *lower_index, long int *lower_delta,
                        int *higher_index, long int *higher_delta);

int find_nearest_graph_index( struct graph_struct *graph,
                                unsigned long int frame, int *index);

int map_graph_screen( struct graph_struct *graph, unsigned long int frame,
                        double percent, WORD *x, WORD *y);

int map_screen_graph( struct graph_struct *graph, WORD x, WORD y,
                        unsigned long int *frame, double *percent);


int draw_box( struct RastPort *rport, struct graph_struct *graph,
                    WORD x, WORD y, int half_width);

int move_graph_point( struct RastPort *rport, struct graph_struct *graph,
                        int index, unsigned long int new_frame,
                        double new_percent) ;

int get_graph_index_values( struct graph_struct *graph,
                            unsigned long int *frame, double *percent,
                            double *tension, double *bias, double *cont,
                            int *linear);

int set_graph_current_index( struct graph_struct *graph, int index);

int clear_graph_points( struct graph_struct *graph);

int get_graph_current_index( struct graph_struct *graph, int *index);

int hilight_current_index( struct RastPort *rport, struct graph_struct *graph);

int unhilight_current_index(struct RastPort *rport,struct graph_struct *graph);

int append_graph_point( struct graph_struct *graph, unsigned long int frame,
                        double percent, double tension, double bias,
                        double cont, int linear);

int change_graph_length ( struct graph_struct *graph,
                            unsigned long int new_length);

int last_graph_index( struct graph_struct *graph);

int set_graph_index_spline( struct graph_struct *graph,
                            double tension, double bias, double cont,
                            int linear);

int draw_graph_segment( struct RastPort *rport, struct graph_struct *graph,
                        int index, int append);

int redraw_pbuff( struct RastPort *rport, struct graph_struct *graph);

int change_index_spline( struct RastPort *rport, struct graph_struct *graph,
                         double new_tension, double new_bias,
                         double new_cont, int new_linear);

int generate_segment_points( struct graph_struct *graph, int index,
                int max_points, unsigned long int *fields,
                double *percent, int *num_points);



#endif /*GRAPH_LIB_H*/

