/****************************************************************************
**
**                            Graph Library
**
**
**
** Changelog:
**
**
**
**
**
****************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <exec/types.h>
#include <exec/libraries.h>
#include <dos/dos.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>


#include <graphics/text.h>
#include <graphics/rastport.h>
#include <graphics/displayinfo.h>
#include <libraries/dos.h>
#include <libraries/asl.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/asl_protos.h>
#include <clib/dos_protos.h>


/*-- User Include Files --*/

#include "rfx_common.h"
#include "pw_util_lib.h"
#include "graph_lib.h"


/* init graph structure */
struct graph_struct *init_graph( WORD xoffset, WORD yoffset, WORD width,
                                 WORD height, WORD xticheight, WORD
                                 yticwidth, unsigned long int end_frame,
                                 int max_percent, int max_num_frames)
{
    struct graph_struct *graph;

    /* allocate the structure */
    /* then allocate storage */
    if( (graph = calloc(1, sizeof (struct graph_struct)) ) == NULL)
    {
        return (NULL);
    }
    if( (graph->frame = calloc(max_num_frames, sizeof(unsigned long int)))
        == NULL)
    {
        free(graph);
        return (NULL);
    }

    if( (graph->percent = calloc(max_num_frames, sizeof(int)))
        == NULL)
    {
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    if( (graph->x = calloc(max_num_frames, sizeof(WORD)))
        == NULL)
    {
        free(graph->percent);
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    if( (graph->y = calloc(max_num_frames, sizeof(WORD)))
        == NULL)
    {
        free(graph->x);
        free(graph->percent);
        free(graph->frame);
        free(graph);
        return (NULL);
    }

    /* initialize the variables */

    graph->xoffset = xoffset;
    graph->yoffset = yoffset;
    graph->width   = width;
    graph->height  = height;
    graph->max_num_frames = max_num_frames;
    graph->end_frame = end_frame;
    graph->max_percent = max_percent;
    graph->deltax = (double)width / (double)(end_frame+1);
    graph->deltay = (double)height / (double) max_percent;

    graph->top = yoffset;
    graph->left = xoffset;
    graph->right = xoffset + width;
    graph->bottom = yoffset + height;
    graph->xticheight = xticheight;
    graph->yticwidth  = yticwidth;
    graph->xtic_majorinterval = 10;
    graph->xtic_minorinterval = 0;
    graph->ytic_majorinterval = 20;
    graph->ytic_minorinterval = 10;
    graph->current_index = 0;

    /* initialize the chart with 2 points as a linear transition */
//    graph->num_points = 2;
//    graph->frame[0] = 0;
//    graph->frame[1] = end_frame;
//    graph->percent[0] = 0;
//    graph->percent[1] = 100;

    graph->num_points = 0;

//    graph->num_points = 4;
//    graph->frame[0] = 0;
//    graph->percent[0] = 0;
//    graph->frame[1] = end_frame/4;
//    graph->percent[1] = 50;
//    graph->frame[2] = 3*end_frame/4;
//    graph->percent[2] = 50;
//    graph->frame[3] = end_frame;
//    graph->percent[3] = 100;

    /* initialize drawing colors */
    graph->bg_pen = 5;
    graph->fg_pen = 2;
    graph->fill_pen = 4;
    graph->border_pen = 1;
    graph->hilite_pen = 7;


    graph->status = INITIALIZED;

    /* return the pointer */
    return(graph);
}

/*-- delete the graph structure --*/
void free_graph(struct graph_struct *graph)
{
    if (graph != NULL)
    {

        if (graph->frame != NULL)
        {
            free (graph->frame);
        }

        if (graph->percent != NULL)
        {
            free (graph->percent);
        }

        if (graph->x != NULL)
        {
            free (graph->x);
        }

        if (graph->y != NULL)
        {
            free (graph->y);
        }

        free (graph);
    }

    return;
}


/* redraw graph */
/* note:  This assumes that the rastport has a properly initialized */
/*        set of graphics buffers */


int redraw_graph( struct RastPort *rport, struct graph_struct *graph)
{

    WORD            x,y;
    int             i;

    /* error checking */
    if ( (rport == NULL) || (graph == NULL) )
    {
        return (ERROR);
    }


    /* blank the graph area */

    SetAPen(rport,graph->bg_pen);
    SetDrMd(rport, JAM1);

    RectFill(rport, graph->xoffset, graph->yoffset,
                    (graph->xoffset + graph->width),
                    (graph->yoffset + graph->height) );


    /* Draw the graph and draw points */

    SetAPen(rport, graph->fg_pen);
    SetBPen(rport, graph->bg_pen);
    SetDrMd(rport, JAM1);

    map_graph_screen(graph, graph->frame[0], graph->percent[0],
                         &x, &y);

    graph->x[0] = x;
    graph->y[0] = y;
    Move(rport, x, y);

    draw_box(rport, graph, x, y, 3);

    for(i=1; i<graph->num_points; i++)
    {

        map_graph_screen(graph, graph->frame[i], graph->percent[i],
                         &x, &y);

        graph->x[i] = x;
        graph->y[i] = y;

        Draw(rport, x, y);

        draw_box(rport, graph, x, y, 3);

    }

    hilight_current_index(rport, graph );

    return(OK);

}


/* highlite the current selected index */

int hilight_current_index( struct RastPort *rport, struct graph_struct *graph )
{

    /* set pen colors */
    SetAPen(rport, graph->hilite_pen);
    SetDrMd(rport, JAM1);

    draw_box(rport, graph, graph->x[graph->current_index],
                graph->y[graph->current_index], 2);

    return(OK);
}

/* unhighlight selected index */

int unhilight_current_index(struct RastPort *rport, struct graph_struct *graph)
{

    /* set pen colors */
    SetAPen(rport, graph->fg_pen);
    SetDrMd(rport, JAM1);

    draw_box(rport, graph, graph->x[graph->current_index],
                graph->y[graph->current_index], 2);

    return(OK);
}






/* Draw box around a point, forcing it into the graph */
/* pens and drawmode must have been setup before calling */

int draw_box( struct RastPort *rport, struct graph_struct *graph,
                    WORD x, WORD y, int half_width)
{

    WORD  top,left,bottom,right;
    int   width;

    width = 2 * half_width;

    /* calculate corners (fit inside graph) */
    left = x - half_width;
    if (left < graph->left)
    {
        left = graph->left;
    }
    else if ( left > graph->right)
    {
        left = (graph->right - width);
    }

    right = left + width;
    /* left already clipped to graph, no need to check under */
    if (right > graph->right)
    {
        right = graph->right;
        left = right - width;
    }

    top = y - half_width;
    if (top < graph->top)
    {
        top = graph->top;
    }
    else if ( top > graph->bottom)
    {
        top = graph->bottom - width;
    }

    bottom = top + width;
    if (bottom > graph->bottom)
    {
        bottom = graph->bottom;
        top = bottom - width;
    }

    RectFill(rport, left, top, right, bottom);



    return(OK);
}



/* blank and redraw the axes for the graph */
/*  draw a tic mark every 10 frames and every 20 percent */

int redraw_axis( struct RastPort *rport, struct graph_struct *graph,
                    int x_flag, int y_flag)
{

    long int            frame;
    long int            increment;
    WORD                x, y, bottom, ticbottom;
    int                 zerominor_flag;



    bottom = graph->bottom + 1;
    ticbottom = bottom + graph->xticheight;

    /* clear and draw x axis */
    if (x_flag == TRUE)
    {
        /* clear the old x axis region */
        SetAPen(rport, graph->bg_pen);
        SetDrMd(rport, JAM1);

        RectFill(rport, graph->left, bottom,
                        (graph->left + graph->width),
                        ticbottom );

        SetAPen(rport, graph->fg_pen);

        /* draw the new tic points */
        /* find the smallest increment */
        if (graph->xminorincrement == 0)
        {
            zerominor_flag == TRUE;
            increment = graph->xmajorincrement;
        }
        else
        {
            /* simplifying assumption, major increment is a multiple */
            /* of minor increment */

            zerominor_flag == FALSE;
            increment = graph->xminorincrement;
        }

        Move(rport, graph->left, bottom);
        for( frame = 0; frame < graph->end_frame; frame += increment)
        {


!!!!!!!!!




    }



    return(OK);
}


/* insert point */
int insert_graph_point( struct graph_struct *graph, unsigned long int frame,
                        int percent)
{

    int     index, quitflag, i, i1;
    WORD    newx, newy;

    /* error checking (graph, frame number, point overflow or precent bounds)*/
    /* note that we are not allowed to insert points at frame 0 or the last */
    /*  frame */

    if ( graph == NULL )
    {
        return(ERROR);
    }

    if (graph->num_points == graph->max_num_frames)
    {
        return(ERROR);
    }

    if( (frame >= graph->end_frame) || (percent > graph->max_percent)  ||
        (frame == 0) )
    {
        return(ERROR);
    }



    /* find location to insert in graph array */
    /* simple linear search */
    quitflag = FALSE;
    index = 0;

    while (!quitflag)
    {
        if (graph->frame[index] >= frame)
        {
            quitflag = TRUE;
        }
        else
        {
            index++;
            if (index == graph->num_points)
            {
                /* run out of points, quit now (we should never reach this */
                /*  this case */
                return(ERROR);
            }
        }
    }

    /* if the new frame is already a key point, return */

    if (graph->frame[index] == frame)
    {
        return(ERROR);
    }


    /* move the higer points up in the point and percent arrays */
    for(i = graph->num_points; i >= index; i--)
    {
    	i1 = i - 1;

        graph->frame[i] = graph->frame[i1];
        graph->percent[i] = graph->percent[i1];
        /* no need to update the x, y coords, will recalc anyway */

    }

    /* insert the new point, increment count */

        graph->frame[index] = frame;
        graph->percent[index] = percent;
        graph->num_points++;

    /* adjust the current_index */

        if (index <= graph->current_index)
        {
            graph->current_index++;
        }


    return(OK);
}


/* delete point */
int delete_graph_point( struct graph_struct *graph, int frame_index)
{
    int i, i1;

    /* error checking (graph, frame index number point underflow) */
    if (graph == NULL)
    {
        return(ERROR);
    }

    if( (frame_index >= (graph->num_points-1)) || ( graph->num_points == 2) ||
        (frame_index == 0) )
    {
        /* can't have less than 2 points in graph) */

        return(ERROR);
    }

    /* move the higher points down in the point and percent arrays */

    for( i=frame_index; i<graph->num_points; i++)
    {
        i1 = i+1;
        graph->frame[i] = graph->frame[i1];
        graph->percent[i] = graph->percent[i1];
        /* x and y will be re-calculated anyway */
    }

    graph->num_points--;

    if (frame_index <= graph->current_index)
    {
        graph->current_index--;
    }

    return(OK);
}


/* move a keyframe index */
/* note that this function blanks the lines and redraws the points */


int move_graph_point( struct RastPort *rport, struct graph_struct *graph,
                        int index, unsigned long int new_frame,
                        int new_percent)
{


    int                     lower_index, higher_index;
    unsigned long int       temp_frame;
    int                     temp_percent;
    WORD                    tempx, tempy;
    int                     drawlower = TRUE;
    int                     drawupper = TRUE;




    /* basic error checking */
    if (graph == NULL)
    {
        return(ERROR);
    }

    if ( (index < 0) || (index >= graph->num_points) )
    {
        return(ERROR);
    }


    /* Note: we are not allowed to shift first or last keyframe, only vary */
    /* the percent level */

    /* the new keyframe value must be bounded by the keyframes above and below */
    /* it, if it is the first or last keyframe, it is not allowd to move at all */

    if (index == 0)
    {
        /* it is first keyframe, do not allow new frame */
        temp_frame = 0;
        lower_index = 0;
        higher_index = 1;
        drawlower = FALSE;

    }
    else if (index == (graph->num_points - 1) )
    {
        /* it is last frame, do not allow new frame */
        temp_frame = graph->frame[(graph->num_points-1)];
        lower_index = index-1;
        higher_index = index;
        drawupper = FALSE;
    }
    else
    {
        temp_frame = new_frame;

        /* clip frame to lower and higher indices */
        lower_index = index - 1;
        higher_index = index + 1;

        if (temp_frame <= graph->frame[lower_index])
        {
            temp_frame = graph->frame[lower_index] + 1;
        }
        else if (temp_frame >= graph->frame[higher_index])
        {
            temp_frame = graph->frame[higher_index] - 1;
        }
    }


    /* now clip percentage */
    temp_percent = new_percent;

    if (temp_percent < 0 )
    {
        temp_percent = 0;
    }
    else if (temp_percent > graph->max_percent )
    {
        temp_percent = graph->max_percent;
    }


    /* now find the new coordinates */

    map_graph_screen(graph, temp_frame, temp_percent, &tempx, &tempy);

    /* erase the old lines and draw the new ones */

    SetAPen(rport, graph->bg_pen);
    Move(rport, graph->x[lower_index], graph->y[lower_index]);
    Draw(rport, graph->x[index], graph->y[index]);
    Draw(rport, graph->x[higher_index], graph->y[higher_index]);


    draw_box(rport, graph, graph->x[index], graph->y[index], 3);

    SetAPen(rport, graph->fg_pen);
    Move(rport, graph->x[lower_index], graph->y[lower_index]);

    if (drawlower == TRUE)
    {
        Draw(rport, tempx, tempy);
    }
    else
    {
        Move(rport, tempx, tempy);
    }

    if (drawupper == TRUE)
    {
        Draw(rport, graph->x[higher_index], graph->y[higher_index]);
    }

    if (drawlower == TRUE)
    {
        draw_box(rport, graph, graph->x[lower_index], graph->y[lower_index],3);
    }

    draw_box(rport, graph, tempx, tempy, 3);

    if (drawupper == TRUE)
    {
        draw_box(rport, graph,graph->x[higher_index],graph->y[higher_index],3);
    }

    /* save the new values */
    graph->frame[index] = temp_frame;
    graph->percent[index] = temp_percent;
    graph->x[index] = tempx;
    graph->y[index] = tempy;


    hilight_current_index(rport, graph );


    return(OK);
}


/* find point above and below a specified point */
int find_graph_points( struct graph_struct *graph, unsigned long int frame,
                        int *lower_index, long int *lower_delta,
                        int *higher_index, long int *higher_delta)
{
    int     index, quitflag;


    /* error checking (graph, frame number) */
    /*!! skip for speed right now !!*/

    /* actions: */
    /*  Note: frame is unsigned and frame count must start with 0, so no */
    /*        underflow is possible */

    /*  (1.  frame is between first and last point frame (normal): */
    /*        will find index of lower and higher point */
    /*
    /*  (2.  frame is greater than last point frame (overflow): */
    /*        will give last point frame as both lower and higer index */



    /* start from bottom and find the last point smaller than the frame */
    /*!! a linear search should be OK for the small number of points we */
    /*!! will have.  May want to do a bisection search if it gets too long */

    index = 0;
    quitflag = FALSE;

    while (!quitflag)
    {
        if ((graph->frame[index] <= frame)&&(graph->frame[index+1] >= frame))
        {
            quitflag = TRUE;
            *lower_index = index;
            *higher_index = index+1;
            *lower_delta = frame - graph->frame[index];
            *higher_delta = graph->frame[(index+1)] - frame;
        }
        else
        {
            index++;
            if (index == graph->num_points)
            {
                quitflag = TRUE;

                /* we have overflowed, report last frame */
                index--;
                *lower_index = index;
                *higher_index = index;
                *lower_delta = frame - graph->frame[index];
                *higher_delta = *lower_delta;

            }
        }

    }

    return(OK);
}


/* find nearest graph point index */


int find_nearest_graph_index( struct graph_struct *graph,
                                unsigned long int frame, int *index)
{

    int lower_index, higher_index;
    long int lower_delta, higher_delta;

    /* find the closest graph point (if equal, favor the lower) */

    /* skip error checking */

    find_graph_points( graph, frame, &lower_index, &lower_delta,
                                     &higher_index, &higher_delta);


    if (lower_delta > higher_delta)
    {
        *index = higher_index;
    }
    else
    {
        *index = lower_index;
    }

//    printf("lower_index = %d  lower_delta = %ld\n", lower_index, lower_delta);
//    printf("higher_index = %d  higher_delta = %ld\n", higher_index, higher_delta);
//    printf("index = %d\n", *index);

    return(OK);
}




/* find screen coords for a point */
int map_graph_screen( struct graph_struct *graph, unsigned long int frame,
                        int percent, WORD *x, WORD *y)
{

    /* for speed, do not perform graph error checking */


    /* find coords that corresponds to the frame and percentage */
    if (frame < graph->end_frame)
    {
        *x = graph->xoffset + (WORD)( (double)frame * graph->deltax );
    }
    else
    {
        *x = graph->xoffset + graph->width;
    }
    /* note, frame is unsigned, never negative */


    if (percent < 0)
    {
        *y = graph->bottom;
    }
    else if (percent < graph->max_percent )
    {
        *y = graph->bottom - ((WORD)( (double)percent * graph->deltay));
    }
    else
    {
        *y = graph->yoffset;   /* clip to maximum */
    }

    return(OK);

}



/* find frame and percent for a screen coord */

int map_screen_graph( struct graph_struct *graph, WORD x, WORD y,
                        unsigned long int *frame, int *percent)
{

    /* for speed, do not perform graph error checking */


    /* find point that corresponds to the screen coordinates */

    if (x <= graph->xoffset)
    {
        *frame = 0;
    }
    else if ( x < (graph->xoffset + graph->width) )
    {
        *frame = (unsigned long int)
                ((double)( x - graph->xoffset) / graph->deltax);
    }
    else
    {
        *frame = graph->end_frame;
    }


    if (y <= graph->yoffset)
    {
        *percent = 100;
    }
    else if (y < graph->bottom)
    {
        *percent = (int)( (double)(graph->bottom - y) / graph->deltay );
    }
    else
    {
        *percent = 0;
    }

    return(OK);

}


/* return graph values for an index */

int get_graph_index_values( struct graph_struct *graph,
                            unsigned long int *frame, int *percent)
{

    /* error checking */

    if (graph->current_index >= graph->num_points)
    {
        return(ERROR);
    }

    /* return values */

    *frame = graph->frame[graph->current_index];
    *percent = graph->percent[graph->current_index];

    return(OK);
}


/* set the graph current index point */

int set_graph_current_index( struct graph_struct *graph, int index)
{

    if ( (index >= graph->num_points) || (index < 0) )
    {
        return(ERROR);
    }

    graph->current_index = index;

    return(OK);
}


/* get the value of the current index for a graph */

int get_graph_current_index( struct graph_struct *graph, int *index)
{

    *index = graph->current_index;

    return(OK);
}


/* append a point to the graph (use only when building the list for */
/* the first time) */

int append_graph_point( struct graph_struct *graph, unsigned long int frame,
                        int percent)
{

    if (graph->status != INITIALIZED)
    {
        return(ERROR);
    }

    if (graph->num_points == graph->max_num_frames)
    {
        return(ERROR);
    }

    graph->frame[graph->num_points = frame;
    graph->percent[graph->num_points = percent;

    graph->num_points++;

    return(OK);
}




/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/










/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/








/*--- Spare Template ---*/


/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/

