/****************************************************************************
**
**                         prj_lib.h
**
** header file for functions in prj_lib.c
**
**
**
****************************************************************************/


/* make sure file is only included once */

#ifndef PRJ_LIB_H

#define PRJ_LIB_H



/*-- CONSTANTS --*/
#define MODE_NORMAL     (0)
#define MODE_SELECT     (1)
#define MODE_UNSELECT   (2)


/*-- Data Structures --*/



struct prj_clip_list {
    int                         status;
    int                         num_elements;

    struct prj_clip_struct     *head;
    struct prj_clip_struct     *tail;
    struct prj_clip_struct     *cursor;

    /* display-related items */
    struct hitbox_struct        extents;
    struct prj_clip_struct     *disp_start;  //- 1st clip in edit window
    int                        num_visible;  // - number of visible clips

    };

struct audio_env_struct {
    unsigned long int       frame;       /* starting frame */
    unsinged long int       num_frames;  /* number of frames */
    double                  level;       /* 0.0 - 1.0 */
    unsigned short int      x_coord;     /* screen x coord */
    unsigned short int      y_coord;     /* screen y coord at point */
    };




struct prj_clip_struct {

    /* values from project list */
    unsigned long          startframe;  //- Startframe rel to clip 0
    unsigned long          framecount;  //- # of frames to play
    unsigned long          prj_start_frame; //- Clip start rel to project 0
    unsigned int           audio_on;    //- Audio flags
    unsigned int           vol1;        //- Left volume level
    unsigned int           vol2;        //- Right volume level
    int                    pan1;        //- Left pan
    int                    pan2;        //- Right pan
    unsigned int           attack;      //- Ramp-up frames
    unsigned int           decay;       //- Ramp-down frames

    /* derived items */
    unsigned long          length;      //- Total Clip Length
    unsigned lone          prj_end_frame; //- = prj_start_frame + framecount
                                                (stored to speed calculations)

    char                   *volume;     //- Volume name (no ':')
    char                   *path;       //- Path        (no trailing '/')
    char                   *name;       //- Name

    char                   *clip_name;  //- name buffer for clipping

    /* drawing related items */
    struct hitbox_struct    hitbox;     //- hitbox
    int                     leftedge;   //- left edge of TOTAL clip box
    int                     rightedge;  //- right edge if TOTAL clip box

    /* audio ramp items */
    /* note: at first, will only have 4 points: start, attack, decay, stop */
    /*       but may grow if audio envelopes are implemented */
    int                     num_env_pts;   /* = 4, for now */
    struct audio_env_struct aud_env[4];    /* may grow in future */

                                           /*! Add hitbox in future */


    /* linked list pointers */

    struct prj_clip_struct *prev;
    struct prj_clip_struct *next;

    /*!! also need some way to link clip elemetns that show up in */
    /* 2 places (like video and audio bus) */
    /* perhaps a link from the video clip rendition?? */

    };



/*-- Constants --*/
#define CLIP_ABSOLUTE (0)
#define CLIP_RELATIVE (1)



/* Global variables --*/

extern struct prj_clip_list    g1_list;
extern struct prj_clip_list    v1_list;
extern struct prj_clip_list    a1_list;
extern struct prj_clip_list    a2_list;
extern struct prj_clip_list    a3_list;
extern struct prj_clip_list    a4_list;





/*-- Function declarations --*/








#endif /* PRJ_LIB_H */


