/****************************************************************************
**
**                          graph_lib.h
**
**
**
****************************************************************************/


/* make sure file is only included once */

#ifndef GRAPH_LIB_H

#define GRAPH_LIB_H



struct graph_struct {
    int                 status;
    WORD                xoffset;
    WORD                yoffset;
    WORD                width;
    WORD                height;
    WORD                xticheight;
    WORD                yticwidth;
    int                 xtic_majorinterval;
    int                 xtic_minorinterval;
    int                 ytic_majorinterval;
    int                 ytic_minorinterval;
    int                 max_num_frames;
    int                 max_percent;
    unsigned long int   end_frame;
    double              deltax;
    double              deltay;
    int                 num_points;
    unsigned long int   *frame;
    int                 *percent;
    WORD                *x;
    WORD                *y;
    int                 current_index;

    /* colors for drawing */
    UBYTE               bg_pen;
    UBYTE               fg_pen;
    UBYTE               fill_pen;
    UBYTE               border_pen;
    UBYTE               hilite_pen;

    /* borders for drawing (xoffset is same as left, yoffset is same as top) */
    WORD                left;
    WORD                top;
    WORD                right;
    WORD                bottom;

    };


/*-- prototypes --*/
struct graph_struct *init_graph( WORD xoffset, WORD yoffset, WORD width,
                                 WORD height, WORD xticheight,
                                 WORD yticwidth, unsigned long int end_frame,
                                 int max_percent, int max_num_frames);

void free_graph(struct graph_struct *graph);

int redraw_graph( struct RastPort *rport, struct graph_struct *graph);

int redraw_axis( struct RastPort *rport, struct graph_struct *graph,
                    int x_flag, int y_flag);

int insert_graph_point( struct graph_struct *graph, unsigned long int frame,
                        int percent);

int delete_graph_point( struct graph_struct *graph, int frame_index);

int find_graph_points( struct graph_struct *graph, unsigned long int frame,
                        int *lower_index, long int *lower_delta,
                        int *higher_index, long int *higher_delta);

int find_nearest_graph_index( struct graph_struct *graph,
                                unsigned long int frame, int *index);

int map_graph_screen( struct graph_struct *graph, unsigned long int frame,
                        int percent, WORD *x, WORD *y);

int map_screen_graph( struct graph_struct *graph, WORD x, WORD y,
                        unsigned long int *frame, int *percent);


int draw_box( struct RastPort *rport, struct graph_struct *graph,
                    WORD x, WORD y, int half_width);

int move_graph_point( struct RastPort *rport, struct graph_struct *graph,
                        int index, unsigned long int new_frame,
                        int new_percent) ;

int get_graph_index_values( struct graph_struct *graph,
                            unsigned long int *frame, int *percent);

int set_graph_current_index( struct graph_struct *graph, int index);


int get_graph_current_index( struct graph_struct *graph, int *index);

int hilight_current_index( struct RastPort *rport, struct graph_struct *graph);

int unhilight_current_index(struct RastPort *rport,struct graph_struct *graph);

int append_graph_point( struct graph_struct *graph, unsigned long int frame,
                        int percent);




#endif /*GRAPH_LIB_H*/

