/****************************************************************************
**                            Render FX shell
**
**
**
**
**
****************************************************************************/


/*-- System Include files --*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <exec/types.h>
#include <devices/input.h>
#include <devices/inputevent.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>

#include <graphics/displayinfo.h>
#include <libraries/dos.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>



/*-- User Include Files --*/

#include "rfx_common.h"
#include "timecode_lib.h"
#include "pw_util_lib.h"
#include "rfx_init.h"
#include "err_lib.h"
#include "graph_lib.h"
#include "rfx_gadget_ids.h"


#define  VIDEO_TYPE         (1)
#define  AUDIO_TYPE         (2)
#define  VIDEO_AUDIO_TYPE   (3)
#define  STILL_TYPE         (4)

#define  A_CLIP             (1)
#define  B_CLIP             (2)



/*- invisible pointer sprite -*/
#define INV_PTR_WIDTH       (16)
#define INV_PTR_HEIGHT      (2)
#define INV_PTR_XOFFSET     (0)
#define INV_PTR_YOFFSET     (0)

UWORD __chip inv_ptr[] =
{
    0x0000, 0x0000,     /* reserved, must be NULL */

    0x0000, 0x0000,     /* data, all 0 values */
    0x0000, 0x0000,

    0x0000, 0x0000     /* reserved, must be NULL */
};





/*-- Global variables --*/
/*- the A clip -*/
char                    afilename[257];
int                     afilenamesize = 256;
unsigned long int       acliplen;
unsigned long int       aclipstart;
unsigned long int       aclipmax;
int                     acliptype;


/*- the B clip -*/
char                    bfilename[257];
int                     bfilenamesize = 256;
unsigned long int       bcliplen;
unsigned long int       bclipstart;
unisgned long int       bclipmin;
int                     bcliptype;

unsigned long int       trans_len;
char                    *comment;
char                    outfilename[257];
int                     outfilenamesize = 256;
int                     outquality;
int                     matte;

int                     max_points = 200;

/*- Graph Mode -*/
int                     graph_mode = 0;
int                     drag_flag  = FALSE;

unsigned long int       curr_index_frame = 0;
int                     curr_index_percent = 0;


/*- Graph constants -*/
#define GRAPH_TOP       (180)
#define GRAPH_LEFT      (60)
#define GRAPH_WIDTH     (550)
#define GRAPH_HEIGHT    (150)
#define GRAPH_XTICHEIGHT (5)
#define GRAPH_YTICWIDTH  (5)


/*- "slider button" globals -*/
WORD    bs_basex        = 0;
WORD    bs_basey        = 0;
WORD    bs_screen_basex = 0;
WORD    bs_screen_basey = 0;
int     bs_sensitivity  = 0;
int     bs_gadget_id    = 0;
int     bs_active       = FALSE;


/*- function prototypes -*/
void select_graph_mode( int mode_id );
void draw_clipstart( struct RastPort *rport, int a_b, unsigned long int frame);
int update_intsgad( struct Gadget *gad, long int value, int fieldwidth);
int read_intsgad( struct Gadget *gad, long int *value);
int setup_button_slider(WORD x, WORD y, int sensitivity, struct Gadget *gad);
int clear_button_slider(void);
int process_button_slider( WORD x);
void set_mouse( WORD x, WORD y);
void return_mouse( void);
void adjust_aclip_start( int delta_cframes );
void adjust_bclip_start( int delta_cframes );
void adjust_trans_len( int delta_cframes );
int display_graph_index( struct graph_struct *graph);
int adjust_graph_index( struct graph_struct *graph, int movement);
int read_control_file( char *filename);



/***************************************************************************/


int main(int argc, char *argv[])
{
    int        status;
    char       reply[50];

    struct graph_struct *graph;

    BOOL       quitflag;
    ULONG      signalmask, winsignal, signals;
    struct     IntuiMessage *message = NULL;
    ULONG      class;
    UWORD      code;
    UWORD      qualifier;
    WORD       mousex, mousey;
    struct     Gadget  *gad;

    unsigned long int       temp_frame, new_frame;
    int                     temp_percent, new_percent;
    int                     temp_index, new_index;

    WORD                    tempx, tempy;
    WORD                    drag_basex, drag_basey;


    int         i,j;
    int         lower_index, higher_index;
    long int    lower_delta, higher_delta;

    long int   tf1, tf2;

    long int mid_frame, new_startframe, new_endframe;
    int         movement;


    unsigned long int  delay_counter;

    struct hitbox_struct   graph_hitbox = {GRAPH_LEFT, GRAPH_TOP,
                                            (GRAPH_LEFT + GRAPH_WIDTH),
                                            (GRAPH_TOP + GRAPH_HEIGHT) };


    /*!!! dummy setup !!!!*/
    acliplen    = 300;
    aclipstart  = 200;
    acliptype   = VIDEO_AUDIO_TYPE;

    bcliplen    = 5000;
    bclipstart  = 100;
    bcliptype   = VIDEO_AUDIO_TYPE;

    trans_len   = 46;




    /* open libraries and setup resources */

    status = init_rfx();

    if (status != OK)
    {
        printf("Error from init_rfx status is %d\n", status);
        shutdown_rfx();
        exit(1);
    }

    select_graph_mode(_CREATE_ID);

    /*! parse the input command line */


    /*! open and parse the input file */


    /*! test for aclip and bclip, error if not found, find size if OK */
    /*! make sure start clip points are sane */


    /*! make sure transition length is sane */



    /* draw the start values */

    draw_clipstart(rf_win->RPort, A_CLIP, aclipstart);
    draw_clipstart(rf_win->RPort, B_CLIP, bclipstart);


    /* draw the graph */

    graph = init_graph( GRAPH_LEFT, GRAPH_TOP, GRAPH_WIDTH, GRAPH_HEIGHT,
                        GRAPH_XTICHEIGHT, GRAPH_YTICWIDTH,
                        trans_len, 100, max_points);

    redraw_graph(rf_win->RPort ,graph);



    /* setup is OK, enter event loop*/

    update_intsgad( &_len_str_gad, trans_len, LEN_STR_WIDTH);
    update_intsgad( &_frame_str_gad, curr_index_frame, FRAME_STR_WIDTH);
    update_intsgad( &_percent_str_gad, curr_index_percent, PERCENT_STR_WIDTH);



    /* Set up the signalswe want to hear about ... */
    winsignal = 1L << rf_win->UserPort->mp_SigBit;   /* window IDCMP */
    signalmask = winsignal;


    quitflag = FALSE;


    while( !quitflag )
    {
        signals = Wait(signalmask);

        /* An Event occured - now act on the signal(s) we received */
        /* We were only waiting for one signal (winsignal) in out */
        /* signalmask, so we actually knw we recieved winsignal. */

        if( signals & winsignal)
        {
            while( message = (struct IntuiMessage *)GetMsg(rf_win->UserPort))
            {
                class = message->Class;  /* get all the data needed from  msg*/
                code = message->Code;
                qualifier = message->Qualifier;
                mousex = message->MouseX;
                mousey = message->MouseY;


                /* gadget address is ONLY valid for gadget messages */
               if( (class==IDCMP_GADGETUP) || (class == IDCMP_GADGETDOWN) )
                {
                    gad = (struct Gadget *)(message->IAddress);
                }

                /* when we're through with the message, reply */
                ReplyMsg( (struct Message *)message);


                /* see what events occured */

                switch(class)
                {

                case IDCMP_GADGETUP:
                    /* caused by GACT_RELVERIFY */
                    printf("recieved an IDCMP_GADGETUP, gadget number %d\n",
                            gad->GadgetID);


                    if (gad->GadgetID == _CANCEL_ID)
                    {
                        quitflag = TRUE;
                    }

    /* this block handles the shutting down of the "button sliders" */
                    else if ( ( (gad->GadgetID == _A_LEFT_ID) ||
                                (gad->GadgetID == _A_RIGHT_ID) ) &&
                              (acliptype != STILL_TYPE) )
                    {
                        clear_button_slider();

                    }
                    else if ( ( (gad->GadgetID == _B_LEFT_ID) ||
                                (gad->GadgetID == _B_RIGHT_ID) ) &&
                              (acliptype != STILL_TYPE) )
                    {
                        clear_button_slider();

                    }
                    else if ( (gad->GadgetID == _LEN_LEFT_ID) ||
                                (gad->GadgetID == _LEN_RIGHT_ID) )
                    {
                        clear_button_slider();
                    }


    /* the keyframe select gadget is NOT a "button slider" */
                    else if ( (gad->GadgetID == _FRAME_LEFT_ID) )
                    {

                        status = adjust_graph_index(graph, -1);
                        if (status == OK)
                        {
                            display_graph_index(graph);
                        }


                    }
                    else if ( (gad->GadgetID == _FRAME_RIGHT_ID) )
                    {

                        status = adjust_graph_index(graph, 1);
                        if (status == OK)
                        {
                            display_graph_index(graph);
                        }
                    }
                    else if ( (gad->GadgetID == _FRAME_STRING_ID) )
                    {
                        /* attempt to set the current indexed point */

                        /* read the string */
                        read_intsgad( &_frame_str_gad, &new_frame);

                        get_graph_current_index( graph, &temp_index);

                        get_graph_index_values( graph, &temp_frame,
                                        &temp_percent);


                        /* try to set the point */
                        move_graph_point( rf_win->RPort, graph, temp_index,
                                            new_frame, temp_percent) ;


                        /* see what was actually done and refresh the gadget*/
                        display_graph_index( graph );

                    }

                    else if ( (gad->GadgetID == _PERCENT_STRING_ID) )
                    {
                        /* attempt to set the current indexed point */

                        /* read the string */
                        read_intsgad( &_percent_str_gad, &new_percent);

                        get_graph_current_index( graph, &temp_index);

                        get_graph_index_values( graph, &temp_frame,
                                        &temp_percent);


                        /* try to set the point */
                        move_graph_point( rf_win->RPort, graph, temp_index,
                                            temp_frame, new_percent) ;


                        /* see what was actually done and refresh the gadget*/
                        display_graph_index( graph );

                    }


    /* handle the string gadgets */

                    else if ( (gad->GadgetID == _LEN_STRING_ID) )
                    {
//  read the value from the gadget, force to colorframe
//  update the value in the string gadget
//  resize the graph
                    }



                    break;

                case IDCMP_GADGETDOWN:
                    /* caused by GACT_IMMEDIATE */
                    printf("receive an IDCMP_GADGETDOW, gadget number %d\n",
                            gad->GadgetID);


    /* test the graph mode radio button group */

                    if ( (gad->GadgetID == _CREATE_ID) )
                    {

                        select_graph_mode(_CREATE_ID);
                    }
                    else if ( (gad->GadgetID == _DELETE_ID) )
                    {

                        select_graph_mode(_DELETE_ID);
                    }
                    else if ( (gad->GadgetID == _DRAG_ID) )
                    {

                        select_graph_mode(_DRAG_ID);
                    }

    /* check for the "button slider" type gadets */

                    else if ( (gad->GadgetID == _A_LEFT_ID) &&
                              (acliptype != STILL_TYPE) )
                    {

                        setup_button_slider(mousex, mousey, 2,
                                            gad);
                    }

                    else if ( (gad->GadgetID == _A_RIGHT_ID) &&
                              (acliptype != STILL_TYPE) )
                    {
                        setup_button_slider(mousex, mousey, 2,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _B_LEFT_ID) &&
                              (acliptype != STILL_TYPE) )
                    {

                        setup_button_slider(mousex, mousey, 2,
                                            gad);
                    }

                    else if ( (gad->GadgetID == _B_RIGHT_ID) &&
                              (acliptype != STILL_TYPE) )
                    {
                        setup_button_slider(mousex, mousey, 2,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _LEN_LEFT_ID) )
                    {
                        setup_button_slider(mousex, mousey, 2,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _LEN_RIGHT_ID) )
                    {
                        setup_button_slider(mousex, mousey, 2,
                                            gad);
                    }



                    break;


                case IDCMP_RAWKEY:

//                    printf("rawkey\n");

                    break;

                case IDCMP_MOUSEBUTTONS:
                    printf("MouseButton\n");

                    /* note, these messages are only enabled when there is */
                    /* something loaded, so there WILL be data to operate on */

                    /* SELECTDOWN States */
                    if (code == SELECTDOWN)
                    {

                        if (in_hitbox(mousex, mousey, graph_hitbox))
                        {

                            /* find the position in the graph */
                            map_screen_graph( graph, mousex, mousey,
                                                &temp_frame,
                                                &temp_percent);



                            if (graph_mode == _CREATE_ID)
                            {
                                status = insert_graph_point(graph, temp_frame,
                                                    temp_percent);

                                if (status == OK)
                                {
                                    redraw_graph(rf_win->RPort ,graph);
                                    display_graph_index(graph);
                                }
                            }
                            else if (graph_mode == _DELETE_ID)
                            {

                                find_nearest_graph_index( graph, temp_frame,
                                                            &temp_index);

                                status = delete_graph_point(graph, temp_index);

                                if (status == OK)
                                {
                                    redraw_graph(rf_win->RPort, graph);
                                    display_graph_index(graph);
                                }
                            }
                            else if (graph_mode == _DRAG_ID)
                            {

                                find_nearest_graph_index( graph, temp_frame,
                                                            &temp_index);

                                status = set_graph_current_index(graph,
                                                                 temp_index);

                                if (status == OK)
                                {
                                    redraw_graph(rf_win->RPort, graph);
                                    display_graph_index(graph);
                                }

                                status = map_graph_screen( graph,
                                          curr_index_frame, curr_index_percent,
                                          &drag_basex, &drag_basey);

                                if (status == OK)
                                {
                                    /* warp mouse pointer to base posiiton and */
                                    /* prepare to drag */

                                    set_mouse(drag_basex, drag_basey);
                                    drag_flag = TRUE;
                                }


                            }



                        }



                    }

                    else if (code == SELECTUP)
                    {

                        /* if mouseup during a button_slider movement, stop it */
                        if (bs_active == TRUE)
                        {
                            clear_button_slider();

                            /* note that there may be other close-up funtions */
                            /* required here later */

                        }
                        else if (drag_flag == TRUE);
                        {
                            drag_flag = FALSE;
                        }
                    }



                    break;

                case IDCMP_MOUSEMOVE:

//                    printf("MouseMove\n");

                    if ( bs_active == TRUE )
                    {
                        movement = process_button_slider(mousex);
//                        printf("m = %d\n", movement);

                        if ( ( (gad->GadgetID == _A_LEFT_ID) ||
                                (gad->GadgetID == _A_RIGHT_ID) ) &&
                                  (acliptype != STILL_TYPE) )
                        {
                            adjust_aclip_start(movement);

                        }
                        else if ( ( (gad->GadgetID == _B_LEFT_ID) ||
                                (gad->GadgetID == _B_RIGHT_ID) ) &&
                                  (bcliptype != STILL_TYPE) )
                        {
                            adjust_bclip_start(movement);

                        }
                        else if ( ( (gad->GadgetID == _LEN_LEFT_ID) ||
                                (gad->GadgetID == _LEN_RIGHT_ID) ) )
                        {
                            adjust_trans_len(movement);

                        }

                    }
                    else if (drag_flag == TRUE)
                    {

                        /* find the new position in graph coordinates */

                        map_screen_graph( graph, mousex, mousey, &temp_frame,
                                          &temp_percent);



                        /* attempt to move the point */

                        move_graph_point( rf_win->RPort, graph, temp_index,
                                            temp_frame, temp_percent);


                        /* update what was actually done */

                        display_graph_index(graph);
                    }





                    break;

                default:
                    printf("recieved unknown IDCMP code %d\n", class);
                    break;
                }
            }
        }

    }

    /* Clean Up and exit */


    shutdown_rfx();



    return(0);

}



/*-- Toggle Graph Modes --*/

void select_graph_mode( int mode_id )
{
    struct Gadget   *old_gad, *new_gad;
    UWORD           position;

    /* check for valid mode, exit if not OK */
    if ( (mode_id != _CREATE_ID) && (mode_id != _DELETE_ID) &&
         (mode_id != _DRAG_ID) )
    {
        return;
    }

    /* set the global mode */

    if ( graph_mode != mode_id )
    {

        /* remove the old mode gadget and deactivate it */

        if (graph_mode != 0)
        {
            if (graph_mode == _CREATE_ID)
            {
                old_gad = &_create_gad;
            }
            else if (graph_mode == _DELETE_ID)
            {
                old_gad = &_delete_gad;
            }
            else if (graph_mode == _DRAG_ID)
            {
                old_gad = &_drag_gad;
            }

            if ( (position = RemoveGadget(rf_win, old_gad)) == -1)
            {
            printf("crap1\n");
                return;
            }

            old_gad->Flags = (old_gad->Flags & (~GFLG_SELECTED));

            AddGadget(rf_win, old_gad, position);
        }


        /* remove the new mode gadget and activate it */


        if (mode_id == _CREATE_ID)
        {
            new_gad = &_create_gad;
        }
        else if (mode_id == _DELETE_ID)
        {
            new_gad = &_delete_gad;
        }
        else if (mode_id == _DRAG_ID)
        {
            new_gad = &_drag_gad;
        }

        if ( (position = RemoveGadget(rf_win, new_gad)) == -1)
        {
            printf("crap2\n");
            return;
        }

        new_gad->Flags = (new_gad->Flags | GFLG_SELECTED);

        AddGadget(rf_win, new_gad, position);

        RefreshGadgets(rf_win->FirstGadget, rf_win, NULL);


        graph_mode = mode_id;
    }


    return;
}


/* function to draw a/b clip start points on the screen */

void draw_clipstart( struct RastPort *rport, int a_b, unsigned long int frame)
{
    struct timecode     tcode;
    UWORD               x,y;

    frame2tcode( frame, &tcode);

    if (a_b == A_CLIP)
    {
        x = A_TCODEX;
        y = A_TCODEY;
    }
    else if (a_b == B_CLIP)
    {
        x = B_TCODEX;
        y = B_TCODEY;
    }
    else
    {
        return;
    }

    SetAPen(rport, 0);
    SetBPen(rport, 1);
    SetDrMd(rport, JAM2);

    Move(rport, x, y);
    draw_timecode(rport, tcode, FALSE);

    return;
}



/* function to update the value in an integer stringgadget */

int update_intsgad( struct Gadget *gad, long int value, int fieldwidth)
{

    char        format_string[20], string[30];
    int         position;

    /* remove the gadget */
    if ( (position = RemoveGadget(rf_win, gad)) == -1)
    {
        printf("unable to remove gadget in update_intgad\n");
        return(ERROR);
    }


    /* set the LongInt value */

    ((struct StringInfo *)(gad->SpecialInfo))->LongInt = value;


    /* create format string of the correct width */
//    sprintf(format_string, "%%%dld", fieldwidth);

    /* format the value */
//    sprintf(string, format_string, value);
    sprintf(string, "%ld", value);

    strcpy( ((struct StringInfo *)(gad->SpecialInfo))->Buffer, string);

//    ((struct StringInfo *)(gad->SpecialInfo))->BufferPos = 0;
//    ((struct StringInfo *)(gad->SpecialInfo))->DispPos   = 0;


    AddGadget(rf_win, gad, position);

    RefreshGList(gad, rf_win, NULL, 1);

//    ActivateGadget(gad, rf_win, NULL);


    return(OK);
}



int read_intsgad( struct Gadget *gad, long int *value)
{

    int         position;

    /* remove the gadget */
    if ( (position = RemoveGadget(rf_win, gad)) == -1)
    {
        printf("unable to remove gadget in update_intgad\n");
        return(ERROR);
    }


    /* set the LongInt value */

    *value = ((struct StringInfo *)(gad->SpecialInfo))->LongInt;


    AddGadget(rf_win, gad, position);

    RefreshGList(gad, rf_win, NULL, 1);


    return(OK);

}





/* prepare "button slider" to track mousemovements */

int setup_button_slider(WORD x, WORD y, int sensitivity, struct Gadget *gad)
{
    ULONG   lock;

    /* this function sets up several global variables used for */
    /* tracking mouse movements on the button slider */

    if (bs_active == FALSE )
    {
        bs_sensitivity  = sensitivity;
        bs_active       = TRUE;
        bs_gadget_id    = gad->GadgetID;


        /* get the screen position of the center of the gadget*/

        /* center of gadget, in window coords */
        bs_basex        = (gad->LeftEdge + (gad->Width / 2));
        bs_basey        = (gad->TopEdge + (gad->Height / 2));

        /* center of gadget, in screen coords */
        bs_screen_basex =(rf_win->LeftEdge+bs_basex);
        bs_screen_basey =(rf_win->TopEdge+bs_basey);
        set_mouse( bs_screen_basex, bs_screen_basey);


        /* blank the pointer */
        SetPointer( rf_win, inv_ptr, INV_PTR_HEIGHT, INV_PTR_WIDTH,
                    INV_PTR_XOFFSET, INV_PTR_YOFFSET );



        /* provide immediate feedback on the slider movement */

        if( gad->GadgetID == _A_LEFT_ID )
        {
            adjust_aclip_start(-1);  /* this is Colorframes */
        }
        else if ( gad->GadgetID == _A_RIGHT_ID )
        {
            adjust_aclip_start(1);
        }
        else if( gad->GadgetID == _B_LEFT_ID )
        {
            adjust_bclip_start(-1);  /* this is Colorframes */
        }
        else if ( gad->GadgetID == _B_RIGHT_ID )
        {
            adjust_bclip_start(1);
        }
        else if ( gad->GadgetID == _LEN_LEFT_ID )
        {
            adjust_trans_len(-1);
        }
        else if ( gad->GadgetID == _LEN_RIGHT_ID )
        {
            adjust_trans_len(1);
        }


        return (OK);
    }

    return (ERROR);
}


/* end a button_slider event sequence */

int clear_button_slider(void)
{

    if (bs_active == TRUE)
    {

        bs_basex        = 0;
        bs_basey        = 0;
        bs_screen_basex = 0;
        bs_screen_basey = 0;
        bs_sensitivity  = 0;
        bs_active       = FALSE;
        bs_gadget_id    = 0;

        /* blank the pointer */
        ClearPointer( rf_win);

        return (OK);
    }

    return(ERROR);
}


/* process a mousemove registered from an active button_slider */

int process_button_slider( WORD x)
{


    /* check to see if the current mouse position is further than */
    /* the "sensitivity" distance in the x direction */
                              
    /* returns 1 if it is greater than the sensitvity  +x direction */
    /* returns -1 if it is greater than the sensitivity -x direction */
    /* returns 0 if it is less than the sensitivity */

    /* see if greater */
    if ( (x - bs_basex) > bs_sensitivity )
    {
        return_mouse();
        return (1);
    }

    if ( (bs_basex - x) > bs_sensitivity )
    {
        return_mouse();
        return (-1);
    }

    return (0);
}


/* insert event to move mouse back to a specific point */

void set_mouse( WORD x, WORD y)
{

    static struct InputEvent       FakeEvent;
    static struct IEPointerPixel   NewPixel;

    NewPixel.iepp_Screen = screen;
    NewPixel.iepp_Position.X = x;
    NewPixel.iepp_Position.Y = y;

    FakeEvent.ie_EventAddress = (APTR) &NewPixel;
    FakeEvent.ie_NextEvent = NULL;
    FakeEvent.ie_Class = IECLASS_NEWPOINTERPOS;
    FakeEvent.ie_SubClass = IESUBCLASS_PIXEL;
    FakeEvent.ie_Code = IECODE_NOBUTTON;
    FakeEvent.ie_Qualifier = NULL;

    InputIO->io_Data = (APTR) &FakeEvent;
    InputIO->io_Length = sizeof(struct InputEvent);
    InputIO->io_Command = IND_WRITEEVENT;

    DoIO((struct IORequest *)InputIO);


    return;
}



/* insert event to move mouse back to starting point */

void return_mouse( void)
{

    static struct InputEvent       FakeEvent;
    static struct IEPointerPixel   NewPixel;

    NewPixel.iepp_Screen = screen;
    NewPixel.iepp_Position.X = bs_screen_basex;
    NewPixel.iepp_Position.Y = bs_screen_basey;

    FakeEvent.ie_EventAddress = (APTR) &NewPixel;
    FakeEvent.ie_NextEvent = NULL;
    FakeEvent.ie_Class = IECLASS_NEWPOINTERPOS;
    FakeEvent.ie_SubClass = IESUBCLASS_PIXEL;
    FakeEvent.ie_Code = IECODE_NOBUTTON;
    FakeEvent.ie_Qualifier = NULL;

    InputIO->io_Data = (APTR) &FakeEvent;
    InputIO->io_Length = sizeof(struct InputEvent);
    InputIO->io_Command = IND_WRITEEVENT;

    DoIO((struct IORequest *)InputIO);


    return;
}


/*-- change the aclip start point and update display --*/
void adjust_aclip_start( int delta_cframes )
{

    unsigned long int   temp_frame;
    long int            movement;

    /* make sure new value is legal */
    /* we're working with unsigned integers, have to do some backflips */
    /* to check for underflow */

    movement = 2 * delta_cframes;
    temp_frame = aclipstart + movement;
    //printf("tf = %ld\n", temp_frame);

    if (movement < 0L)
    {
        if (temp_frame > aclipstart)
        {
            temp_frame = 0;
        }
    }


    if ( ((temp_frame + trans_len) >= acliplen) )
    {
        return;
    }

    /* is's cool, update the display */

    aclipstart = temp_frame;
    draw_clipstart(rf_win->RPort, A_CLIP, aclipstart);

    return;
}


/*-- change the bclip start point and update display --*/
void adjust_bclip_start( int delta_cframes )
{

    unsigned long int   temp_frame;
    long int            movement;

    /* make sure new value is legal */
    /* we're working with unsigned integers, have to do some backflips */
    /* to check for underflow */

    movement = 2 * delta_cframes;
    temp_frame = bclipstart + movement;
    //printf("tf = %ld\n", temp_frame);

    if (movement < 0L)
    {
        if (temp_frame > bclipstart)
        {
            temp_frame = 0;
        }
    }


    if ( (temp_frame < trans_len) || (temp_frame >= bcliplen) )
    {
        return;
    }

    /* is's cool, update the display */

    bclipstart = temp_frame;
    draw_clipstart(rf_win->RPort, B_CLIP, bclipstart);

    return;
}



/*-- change the clip lenght and update display --*/
void adjust_trans_len( int delta_cframes )
{

    long int value;


    /* read the value */

    value = ((struct StringInfo *)(_len_str_gad.SpecialInfo))->LongInt;

    /* increment or decrement value */

    value += ( delta_cframes * 2 );


    /* make sure it is a legal value */

    if (value < 2)
    {
        return;
    }

    if( (acliptype != STILL_TYPE) && (value > (acliplen - aclipstart)) )
    {
        return;
    }

    if( (bcliptype != STILL_TYPE) && (value > (bclipstart)) )
    {
        return;
    }

    /* update the gadget */


    trans_len = value;
    update_intsgad( &_len_str_gad, value, LEN_STR_WIDTH);


    return;
}


/*-- display graph data for the indicated index in the string gadgets */

int display_graph_index( struct graph_struct *graph)
{
    int     status;

    /* get the values */

    status = get_graph_index_values( graph, &curr_index_frame,
                                        &curr_index_percent);

    /* if successful, update the gadgets */
    if (status == OK)
    {
        update_intsgad( &_frame_str_gad, curr_index_frame, FRAME_STR_WIDTH);
        update_intsgad( &_percent_str_gad, curr_index_percent,
                         PERCENT_STR_WIDTH);

        return(OK);
    }

    return(ERROR);

}


int adjust_graph_index( struct graph_struct *graph, int movement)
{

    int new_index;

    get_graph_current_index( graph, &new_index);

    new_index += movement;


    unhilight_current_index(rf_win->RPort, graph );

    if ( (set_graph_current_index(graph, new_index)) == ERROR )
    {
        hilight_current_index(rf_win->RPort, graph );
        return(ERROR);
    }


    hilight_current_index(rf_win->RPort, graph );



    return(OK);

}




/* read the input file and initialize variables */
int read_control_file( char *filename, struct graph_struct *graph)
{

char                    afilename[257];
int                     afilenamesize = 256;
unsigned long int       acliplen;
unsigned long int       aclipstart;
int                     acliptype;


/*- the B clip -*/
char                    bfilename[257];
int                     bfilenamesize = 256;
unsigned long int       bcliplen;
unsigned long int       bclipstart;
int                     bcliptype;

unsigned long int       trans_len;
char                    *comment;
char                    *outfilename;
int                     outquality;


    FILE        *input_file;
    char        buf5[256], temp[50];
    char        *tokenpt;
    int         err;

    int         translen;
    int         half_translen;


    input_file = fopen(filename,"r");
    if (input_file == NULL)
    {
        printf("%s :No such file found\n",argv[1]);
        return(ERROR);
    }

    fgets(buff5,255,input_file);
    tokenpt = strtok(buff5,",");
    strncpy(outpath,tokenpt,49);

    tokenpt = strtok(NULL,",");
    outquality = atoi(tokenpt);
    tokenpt = strtok(NULL,",");
    matte = atoi(tokenpt);

    fgets(buff5,255,input_file);
    tokenpt = strtok(buff5,",");
    strncpy(afilename,tokenpt,49);
    tokenpt = strtok(NULL,",");
    acliptype = atoi(tokenpt);
    tokenpt = strtok(NULL,",");
    aclipstart = atoi(tokenpt)/2;
    tokenpt = strtok(NULL,",");
    acliplen = atoi(tokenpt)/2;

fgets(buff5,255,input_file);
tokenpt = strtok(buff5,",");
strncpy(pathname[1],tokenpt,49);
tokenpt = strtok(NULL,",");
Type2 = atoi(tokenpt);
tokenpt = strtok(NULL,",");
StartFrame2 = atoi(tokenpt)/2;
tokenpt = strtok(NULL,",");
Length2 = atoi(tokenpt)/2;

fgets(buff5,255,input_file);
tokenpt = strtok(buff5,",");
strncpy(pathname[2],tokenpt,49);

err = fscanf(input_file,"%s ",temp);
looplength = atoi(temp)/2;

err = fscanf(input_file,"%s ",temp);
pathcount = atoi(temp);

for (i=0;i<pathcount;i++)
   {
   fgets(buff5,255,input_file);
   tokenpt = strtok(buff5,",");
   Env[i].Frame = atoi(tokenpt)/2;
   tokenpt = strtok(NULL,",");
   Env[i].Count = atoi(tokenpt);
   }

Frame1 = StartFrame1;
Frame2 = StartFrame2;

fgets(buff5,255,input_file);
tokenpt = strtok(buff5,",");
strncpy(KEYWORD,tokenpt,49);
if (KEYWORD[0] == 'S')
   {
   SWIT = 1;
   tokenpt = strtok(NULL,",");
   STYPE = atoi(tokenpt);
   tokenpt = strtok(NULL,",");
   SBUS = atoi(tokenpt);
   tokenpt = strtok(NULL,",");
   SKEY = atoi(tokenpt);
   tokenpt = strtok(NULL,",");
   SLEVEL = atoi(tokenpt);
   tokenpt = strtok(NULL,",");
   SFADE = atoi(tokenpt);
   }





    return(OK);
}



/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/



#ifdef TRASHBIN
    /* draw a test graph */

//    graph = init_graph( 50, 100, 200, 200, 35, 100, 200);

//    redraw_graph(rf_win->RPort ,graph);

//    insert_graph_point(graph, 20, 100);

//    redraw_graph(rf_win->RPort, graph);

//    insert_graph_point(graph, 1,25);
//    redraw_graph(rf_win->RPort, graph);

//    delete_graph_point(graph, 1);
//    redraw_graph(rf_win->RPort, graph);

//    delete_graph_point(graph,2);
//    redraw_graph(rf_win->RPort, graph);


//    insert_graph_point(graph, 20, 100);
//    redraw_graph(rf_win->RPort, graph);

//    move_graph_point( rf_win->RPort, graph, 3, 25, 75);



//    find_graph_points( graph, 25, &lower_index, &lower_delta, &higher_index,
//                        &higher_delta);

//    printf("lower_index = %d  lower_delta = %ld\n", lower_index, lower_delta);
//    printf("higher_index = %d  higher_delta = %ld\n", higher_index, higher_delta);


#endif /* TRASHBIN */