/****************************************************************************
**
**                           rfx_init.c
**
**  This file contains functions used during the initalization and shutdown
** of the ProWave program.
**
**
**
**
****************************************************************************/


/*-- System Include files --*/

#include <stdio.h>
#include <stdlib.h>

#include <exec/types.h>
#include <devices/input.h>
#include <devices/inputevent.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>

#include <graphics/displayinfo.h>
#include <graphics/rastport.h>
#include <graphics/text.h>
#include <libraries/dos.h>
#include <intuition/sghooks.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/diskfont_protos.h>



/*-- User Include Files --*/

#include "rfx_common.h"
#include "rfx_init.h"
#include "button_images.h"
#include "rfx_gadget_ids.h"
#include "err_lib.h"
#include "pw_util_lib.h"
#include "timecode_lib.h"



/*-- Internal function prototypes --*/


/*--onstants --*/

#define INTUITION_REV 37



/*-- Global Variables:  Library Base Pointers --*/

struct Library *IntuitionBase = NULL;
struct Library *DiskfontBase = NULL;


/*-- Global Variables:  Others --*/

struct Screen       *screen      = NULL;
struct Window       *rf_win      = NULL;
struct TextFont     *topaz11     = NULL;



struct ViewPort         *wvp;
struct TmpRas           tras;
struct AreaInfo         ainfo;
UWORD   areabuf[250];    /* area buffer for 100 points (2.5 words/point)*/
PLANEPTR                tbuf;

/*-- Input device variables --*/
struct IOStdReq         *InputIO = NULL;       /* IO Request Block */
struct MsgPort          *InputMP = NULL;       /* Message Port */
int                     iodevice_open = FALSE;




/*-- borders --*/

struct Border           *win_border;
struct Border           *framesel_border;

/*-- Slider Data --*/
struct PropInfo slider1prop = { (AUTOKNOB | FREEHORIZ),
                            0, 0, MAXBODY, 0, 0, 0, 0, 0, 0, 0};

struct Image    slider1image;   /* image struct to provde to slider gadget */

/*-- Integer String Gadget Constans/Variables --*/
struct Border       *len_border;

UBYTE   len_buffer[LEN_STR_WIDTH];
UBYTE   len_undobuffer[LEN_STR_WIDTH];
struct StringExtend len_extend;

struct StringInfo   len_strinfo = {len_buffer, len_undobuffer, 0,
                                    LEN_STR_WIDTH, 0, 0,0,0,0, NULL, 0,
                                    NULL };
struct Border   *percent_border;
UBYTE   percent_buffer[PERCENT_STR_WIDTH];
UBYTE   percent_undobuffer[PERCENT_STR_WIDTH];
struct StringExtend percent_extend;

struct StringInfo   percent_strinfo = {percent_buffer, percent_undobuffer, 0,
                                    PERCENT_STR_WIDTH, 0, 0,0,0,0, NULL, 0,
                                    NULL };

struct Border   *frame_border;
UBYTE   frame_buffer[FRAME_STR_WIDTH];
UBYTE   frame_undobuffer[FRAME_STR_WIDTH];
struct StringExtend frame_extend;

struct StringInfo   frame_strinfo = {frame_buffer, frame_undobuffer, 0,
                                    FRAME_STR_WIDTH, 0, 0,0,0,0, NULL, 0,
                                    NULL };


/*-- Gadget Structures --*/


/*-- Gadget List for rf_win --*/


struct Gadget _a_left_gad = { NULL,
                            30, 130,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _A_LEFT_ID, NULL};

struct Gadget _a_right_gad = { &_a_left_gad,
                            47, 130,     /* X and Y position */
                            16, 17,      /* Width and Height */
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _A_RIGHT_ID, NULL};


struct Gadget _b_left_gad = { &_a_right_gad,
                            430, 130,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _B_LEFT_ID, NULL};

struct Gadget _b_right_gad = { &_b_left_gad,
                            447, 130,     /* X and Y position */
                            16, 17,      /* Width and Height */
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _B_RIGHT_ID, NULL};

struct Gadget _len_left_gad = { &_b_right_gad,
                            315, 130,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _LEN_LEFT_ID, NULL};

struct Gadget _len_right_gad = { &_len_left_gad,
                            332, 130,     /* X and Y position */
                            16, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _LEN_RIGHT_ID, NULL};

struct Gadget _preview_gad = { &_len_right_gad,
                            362, 370,     /* X and Y position */
                            80, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_preview_image, &_preview_sel_image,
                            NULL, NULL, NULL,
                            _PREVIEW_ID, NULL};

struct Gadget _render_gad = { &_preview_gad,
                            444, 370,     /* X and Y position */
                            80, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_render_image, &_render_sel_image,
                            NULL, NULL, NULL,
                            _RENDER_ID, NULL};

struct Gadget _cancel_gad = { &_render_gad,
                            526, 370,     /* X and Y position */
                            80, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_cancel_image, &_cancel_sel_image,
                            NULL, NULL, NULL,
                            _CANCEL_ID, NULL};

struct Gadget _drag_gad = { &_cancel_gad,
                            438, 155,     /* X and Y position */
                            56, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_IMMEDIATE),
                            GTYP_BOOLGADGET,
                            &_drag_image, &_drag_sel_image,
                            NULL, NULL, NULL,
                            _DRAG_ID, NULL};

struct Gadget _create_gad = { &_drag_gad,
                            326, 155,     /* X and Y position */
                            56, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_IMMEDIATE),
                            GTYP_BOOLGADGET,
                            &_create_image, &_create_sel_image,
                            NULL, NULL, NULL,
                            _CREATE_ID, NULL};

struct Gadget _delete_gad = { &_create_gad,
                            382, 155,     /* X and Y position */
                            56, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_IMMEDIATE),
                            GTYP_BOOLGADGET,
                            &_delete_image, &_delete_sel_image,
                            NULL, NULL, NULL,
                            _DELETE_ID, NULL};


struct Gadget _percent_str_gad = { &_delete_gad,
                            200, 372,     /* X and Y position */
                            0, 0,    /* Width and Height !!will be set later*/
                            (GFLG_GADGHCOMP | GFLG_STRINGEXTEND),
                            (GACT_LONGINT |GACT_RELVERIFY | GACT_STRINGRIGHT),
                            GTYP_STRGADGET,
                            NULL, /* will be defined later*/
                            NULL,
                            NULL, 0, &percent_strinfo,
                            _PERCENT_STRING_ID, NULL};

struct Gadget _frame_str_gad = { &_percent_str_gad,
                            200, 355,     /* X and Y position */
                            0, 0,    /* Width and Height !!will be set later*/
                            (GFLG_GADGHCOMP | GFLG_STRINGEXTEND),
                            (GACT_LONGINT |GACT_RELVERIFY | GACT_STRINGRIGHT),
                            GTYP_STRGADGET,
                            NULL, /* will be defined later*/
                            NULL,
                            NULL, 0, &frame_strinfo,
                            _FRAME_STRING_ID, NULL};

struct Gadget _frame_left_gad = { &_frame_str_gad,
                            253, 352,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _FRAME_LEFT_ID, NULL};

struct Gadget _frame_right_gad = { &_frame_left_gad,
                            270, 352,     /* X and Y position */
                            16, 17,      /* Width and Height */
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _FRAME_RIGHT_ID, NULL};



struct Gadget _len_str_gad = { &_frame_right_gad,
                            260, 133,     /* X and Y position */
                            0, 0,    /* Width and Height !!will be set later*/
                            (GFLG_GADGHCOMP | GFLG_STRINGEXTEND),
                            (GACT_LONGINT |GACT_RELVERIFY | GACT_STRINGRIGHT),
                            GTYP_STRGADGET,
                            NULL, /* will be defined later*/
                            NULL,
                            NULL, 0, &len_strinfo,
                            _LEN_STRING_ID, NULL};





#define RFX_WIN_W        (640)
#define RFX_WIN_H        (399)



/****************************************************************************
**
** int  init_rfx(void)
**
** DESC:
**
**  init_rfx() performs the initialization functions for the render FX window.
** It initalizes the gadgets and global variables, opens the display windows
** and initializes the display buffers and data structures
**
**
** INPUTS:  void
**
** OUTPUTS:
**
** RETURNS:
**     OK        success
**     ERROR     error allocating some resource
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/

int init_rfx(void)
{

    struct  ColorSpec  cmap[17];
    UWORD   pens[] = {(UWORD)~0};

    struct TextExtent   t_ext;
    struct timecode     tcode;


    struct  TextAttr topazta = {"topaz.font", 11, FS_NORMAL, FPF_DISKFONT};

    char        format_string[20], string[30];



    /*-- Open the message port and setup input devicde --*/

    if ( (InputMP = CreateMsgPort()) == NULL)
    {
        return (ERROR);
    }

    if ( (InputIO = CreateIORequest(InputMP,sizeof(struct IOStdReq))) == NULL)
    {
        return (ERROR);
    }

    if ( (OpenDevice("input.device",NULL,(struct IORequest *)InputIO,NULL))
         != NULL)
    {
        return(ERROR);
    }
    iodevice_open = TRUE;



    /*-- Open Libraries --*/

    /*-- open the Intuition Library --*/

    if ( (IntuitionBase = OpenLibrary( "intuition.library", INTUITION_REV))
         == NULL)
    {
        return(ERROR);
    }

    /*-- open the diskfont library --*/
    if ( (DiskfontBase = OpenLibrary("diskfont.library", 37L)) == NULL)
    {
        return(ERROR);
    }




    /*-- Setup Data Structures --*/

    /*-- specify the color map --*/
    cmap[0].ColorIndex =  0;  /* BLACKPEN */
    cmap[0].Red        =  1;
    cmap[0].Green      =  1;
    cmap[0].Blue       =  1;
    cmap[1].ColorIndex =  1;  /* MDGREY1PEN */
    cmap[1].Red        =  5;
    cmap[1].Green      =  5;
    cmap[1].Blue       =  5;
    cmap[2].ColorIndex =  2;  /* LTGREY1PEN */
    cmap[2].Red        =  9;
    cmap[2].Green      =  9;
    cmap[2].Blue       =  9;
    cmap[3].ColorIndex =  3;  /* LTGREY2PEN */
    cmap[3].Red        = 13;
    cmap[3].Green      = 13;
    cmap[3].Blue       = 13;
    cmap[4].ColorIndex =  4;  /* DKGREY1PEN */
    cmap[4].Red        =  2;
    cmap[4].Green      =  2;
    cmap[4].Blue       =  2;
    cmap[5].ColorIndex =  5;  /* DKGREY2PEN */
    cmap[5].Red        =  4;
    cmap[5].Green      =  4;
    cmap[5].Blue       =  4;
    cmap[6].ColorIndex =  6;  /* DKYELLOWPEN */
    cmap[6].Red        = 11;
    cmap[6].Green      = 10;
    cmap[6].Blue       =  6;
    cmap[7].ColorIndex =  7;  /* LTYELLOWPEN */
    cmap[7].Red        = 15;
    cmap[7].Green      = 14;
    cmap[7].Blue       =  9;
    cmap[8].ColorIndex =  8;  /* LTBLUE1PEN */
    cmap[8].Red        =  6;
    cmap[8].Green      =  9;
    cmap[8].Blue       = 12;
    cmap[9].ColorIndex =  9;  /* LTPURPLE1PEN */
    cmap[9].Red        =  9;
    cmap[9].Green      =  6;
    cmap[9].Blue       =  6;
    cmap[10].ColorIndex = 10;  /* DKGREEN1PEN */
    cmap[10].Red        =  3;
    cmap[10].Green      =  3;
    cmap[10].Blue       =  2;
    cmap[11].ColorIndex = -1;
    cmap[11].Red        =  0;
    cmap[11].Green      =  0;
    cmap[11].Blue       =  0;


    /*-- Allocate the border lists --*/


    if( (win_border = create_win_border ( 640,  399, 2,
                                                    LOWERED, 3, 4,
                                                    TRUE, 0)) == NULL)
    {
        return (NULL);
    }

    if( (framesel_border = create_win_border ( 620,  40, 1,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {
        return (NULL);
    }



    /*-- Open Screen --*/

    screen = OpenScreenTags(NULL,
                             SA_Width,       640,
                             SA_Height,      400,
                             SA_Overscan,    OSCAN_VIDEO,
                             SA_Pens,        (ULONG)pens,
                             SA_DisplayID,   HIRESLACE_KEY,
                             SA_Depth,       4,
                             SA_Colors,      cmap,
                             SA_ShowTitle,   FALSE,
                             SA_Quiet,       TRUE,
                             TAG_DONE);


    if (screen == NULL)
    {
        return (ERROR);
    }


    /*-- Open the topaz font --*/
    if ( (topaz11 = OpenDiskFont(&topazta)) == NULL)
    {
        return(ERROR);
    }



    /*-- Open Backdrop Window --*/

    printf("%d  %d\n", screen->Width, screen->Height);

    rf_win = OpenWindowTags (  NULL,
                                WA_Left,                0,
                                WA_Top,                 1,
                                WA_Width,               RFX_WIN_W,
                                WA_Height,              RFX_WIN_H,
                                WA_SizeGadget,          FALSE,
                                WA_DragBar,             FALSE,
                                WA_DepthGadget,         FALSE,
                                WA_CloseGadget,         FALSE,
                                WA_Backdrop,            FALSE,
                                WA_Borderless,          TRUE,
                                WA_SmartRefresh,        TRUE,
                                WA_ReportMouse,         TRUE,
                                WA_DetailPen,           1,
                                WA_BlockPen,            4,
                                WA_IDCMP,         ( IDCMP_GADGETDOWN
                                                   |IDCMP_GADGETUP
                                                   /* | IDCMP_RAWKEY */
                                                   | IDCMP_MOUSEBUTTONS
                                                   | IDCMP_MOUSEMOVE ),
                                WA_CustomScreen,        screen,
                               TAG_DONE);

    if (rf_win == NULL )
    {
        return (ERROR);
    }



    /* create area for area fills */
    wvp = (struct ViewPort *)ViewPortAddress(rf_win);

    InitArea(&ainfo, areabuf, 100);        /* init area buffer */
    rf_win->RPort->AreaInfo = &ainfo;

    if ( (tbuf = (PLANEPTR)AllocRaster(700,100)) == NULL )
    {
        return(NULL);
    }

    rf_win->RPort->TmpRas = (struct TmpRas *)InitTmpRas
            (&tras, tbuf, RASSIZE(700,100) );

    /*-- color the background --*/
    SetRast(rf_win->RPort, 1);

    /*-- set the default font --*/
    SetFont(rf_win->RPort, topaz11);

    /* now that we have a font, fill out extended string structure */

    /*- setup transition length gadget -*/

    /* create format string of the correct width */
    sprintf(format_string, "%%0%dld", LEN_STR_WIDTH);

    /* format the value */
    sprintf(string, format_string, 0);


    TextExtent( rf_win->RPort, (STRPTR)string, LEN_STR_WIDTH, &t_ext);

    _len_str_gad.Width = t_ext.te_Width;
    _len_str_gad.Height = t_ext.te_Height;


    if( (len_border = create_win_border ( _len_str_gad.Width+4,
                                          _len_str_gad.Height+4, 2,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {
        return (NULL);
    }

    shift_border( len_border, -2, -2);  /* move border to surround gadget */

    _len_str_gad.GadgetRender = len_border;

    len_extend.Font = topaz11;
    len_extend.Pens[0] = 2;
    len_extend.Pens[1] = 0;
    len_extend.ActivePens[0] = 6;
    len_extend.ActivePens[1] = 0;
    len_extend.InitialModes   = (SGM_REPLACE);
    len_strinfo.Extension = &len_extend;


    /*- Setup percentage string gadget -*/
    /* create format string of the correct width */
    sprintf(format_string, "%%0%dld", PERCENT_STR_WIDTH);

    /* format the value */
    sprintf(string, format_string, 0);


    TextExtent( rf_win->RPort, (STRPTR)string, PERCENT_STR_WIDTH, &t_ext);

    _percent_str_gad.Width = t_ext.te_Width;
    _percent_str_gad.Height = t_ext.te_Height;


    if( (percent_border = create_win_border ( _percent_str_gad.Width+4,
                                          _percent_str_gad.Height+4, 2,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {
        return (NULL);
    }

    shift_border( percent_border, -2, -2);  /*move border to surround gadget*/

    _percent_str_gad.GadgetRender = percent_border;

    percent_extend.Font = topaz11;
    percent_extend.Pens[0] = 2;
    percent_extend.Pens[1] = 0;
    percent_extend.ActivePens[0] = 6;
    percent_extend.ActivePens[1] = 0;
    percent_extend.InitialModes   = (SGM_REPLACE);
    percent_strinfo.Extension = &len_extend;


    /*- setup frame gadget -*/
    /* create format string of the correct width */
    sprintf(format_string, "%%0%dld", FRAME_STR_WIDTH);

    /* format the value */
    sprintf(string, format_string, 0);


    TextExtent( rf_win->RPort, (STRPTR)string, FRAME_STR_WIDTH, &t_ext);

    _frame_str_gad.Width = t_ext.te_Width;
    _frame_str_gad.Height = t_ext.te_Height;


    if( (frame_border = create_win_border ( _frame_str_gad.Width+4,
                                          _frame_str_gad.Height+4, 2,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {
        return (NULL);
    }

    shift_border( frame_border, -2, -2);  /* move border to surround gadget */

    _frame_str_gad.GadgetRender = frame_border;

    frame_extend.Font = topaz11;
    frame_extend.Pens[0] = 2;
    frame_extend.Pens[1] = 0;
    frame_extend.ActivePens[0] = 6;
    frame_extend.ActivePens[1] = 0;
    frame_extend.InitialModes   = (SGM_REPLACE);
    frame_strinfo.Extension = &frame_extend;








    /*-- Draw Borders --*/
    DrawBorder( rf_win->RPort, win_border, 0, 0);
    DrawBorder( rf_win->RPort, framesel_border, 9, 110);

    /*-- add the text --*/

    frame2tcode( 0L, &tcode);

    SetAPen(rf_win->RPort, 0);
    SetBPen(rf_win->RPort, 1);
    SetDrMd(rf_win->RPort, JAM2);

    Move(rf_win->RPort, 66, 127);
    Text(rf_win->RPort, "A Start Point", 13);

    Move(rf_win->RPort, A_TCODEX, A_TCODEY);
    draw_timecode(rf_win->RPort, tcode, FALSE);


    Move(rf_win->RPort, 466, 127);
    Text(rf_win->RPort, "B Start Point", 13);

    Move(rf_win->RPort, B_TCODEX, B_TCODEY);
    draw_timecode(rf_win->RPort, tcode, FALSE);


    /*- Process Bar -*/

    /*-- Add the gadgets --*/

    AddGList(rf_win, &_len_str_gad, 0, -1, NULL);

    RefreshGList(rf_win->FirstGadget, rf_win, NULL, -1);






    return(OK);
}






/****************************************************************************
**
** int shutdown_rfx(void)
**
**
** DESC:
**
**  shutdown_rfx() frees all system resources associated with the
** RenderFX program.  It frees all allocated memory blocks and system
** resources, closes the windows & screen, and closes the libraries.
**
**
** INPUTS: void
**
** OUTPUTS: none
**
** RETURNS:
**    OK      success
**    ERROR   error
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/

int shutdown_rfx(void)
{


    /*-- Close the file --*/
    if ( topaz11 == NULL)
    {
        CloseFont(topaz11);
    }


    /*-- Close the backdrop window --*/

    if( rf_win != NULL )
    {
        CloseWindow(rf_win);
    }


    /*-- Close the Screen --*/
    if (screen != NULL)
    {
        CloseScreen(screen);
    }

    /* free the flood raster */
    if (tbuf != NULL)
    {
        FreeRaster(tbuf, 700, 100);
    }

    if (win_border != NULL)
    {
        free_win_border(win_border);
    }

    if (framesel_border != NULL)
    {
        free_win_border(framesel_border);
    }


    /* Close libraries and exit */

    if (DiskfontBase != NULL)
    {
        CloseLibrary(DiskfontBase);
    }

    if (IntuitionBase != NULL)
    {
        CloseLibrary(IntuitionBase);
    }

    if (iodevice_open == TRUE)
    {
        CloseDevice( (struct IORequest *)InputIO);
    }

    if (InputIO != NULL)
    {
        DeleteIORequest(InputIO);
    }

    if (InputMP != NULL)
    {
        DeleteMsgPort(InputMP);
    }




    return(OK);

}




/****************************************************************************
**
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/




