/****************************************************************************
**
**                          timecode_lib
**
**
**
**
**
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <exec/types.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>

#include <graphics/displayinfo.h>
#include <libraries/dos.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>


#include "rfx_common.h"
#include "timecode_lib.h"



/* load timecode */

/****************************************************************************
**
** void frame2tcode( long int frame, struct timecode *tcode);
**
** DESC:
**   frame2tcode() transforms a zero-baed frame count to a timecode
** structure.
**
**
** INPUTS:
**  long int frame  -- zero-based framecount
**
** OUTPUTS:
**  struct timecode *tcode  -- ptr to timecode struct;
**
** RETURNS: void
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
void frame2tcode( long int frame, struct timecode *tcode)
{
    long int temp;

    if (frame < 0)
    {
        tcode->negflag = TRUE;
        frame *= -1;
    }
    else
    {
        tcode->negflag = FALSE;
    }

    tcode->hh = (int)(frame / 10800L);
    temp = (long int)(frame % 10800L);

    tcode->mm = (int)(temp / 1800L);
    temp = (long int)(frame % 1800L);

    tcode->ss = (int)(temp / 30L);

    tcode->ff = (int)(temp % 30);


    return;
}


/****************************************************************************
**
** void tcode2frame( struct timecode tcode, long int frame);
**
** DESC:
**   tcode2frame() transforms a zero-based timecode structure to a raw
** framecount;
**
**
** INPUTS:
**  struct timecode *tcode  -- zero-based timecode struct;
**
** OUTPUTS:
**  long int *frame  -- ptr to framecount
**
** RETURNS: void
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
void tcode2frame( struct timecode tcode, long int *frame)
{


    /* load timecode */
    *frame = tcode.ff;
    *frame += (tcode.ss * 30);
    *frame += (tcode.mm * 1800);
    *frame += (tcode.hh * 108000);

    if (tcode.negflag == TRUE)
    {
        *frame *= -1;
    }


    return;
}




/****************************************************************************
**
** void draw_timecode( struct RastPort *rport, struct timecode tcode,
**                     int parenflag)
**
** DESC:
**   draw_timecode() draws the timecode value in the structure using the
** current pen and fonts at the current location.
**
**   If parenflag is set, the timecode will be drawn with parenthesis ()
** surrounding the timecode.
**
** NOTE:  if you are overwriting a value, make sure you have the background
** pen set and use JAM2 mode!
**
** INPUTS:
**   struct RastPort *rport  -- rastport to draw on
**   struct timecode  tcode  -- timecode to draw
**   int              parenflag -- flag to draw parenthesis around timecode
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
void draw_timecode( struct RastPort *rport, struct timecode tcode,
                    int parenflag)
{

    char   buffer[15];


    if (parenflag == TRUE)
    {
        sprintf(buffer, "( %02d:%02d:%02d:%02d)", tcode.hh,
                        tcode.mm, tcode.ss, tcode.ff);

        if (tcode.negflag == TRUE)
        {
            buffer[1] = '-';
        }

        /* draw the text */
        Text( rport, (STRPTR) buffer, (ULONG)14);


    }
    else
    {
        sprintf(buffer, " %02d:%02d:%02d:%02d", tcode.hh,
                        tcode.mm, tcode.ss, tcode.ff);

        if (tcode.negflag == TRUE)
        {
            buffer[0] = '-';
        }

        /* draw the text */
        Text( rport, (STRPTR) buffer, (ULONG)12);
    }


    return;
}




/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/






/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/









/*--- Spare Template ---*/


/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/





#ifdef TRASCAN_CODE


/****************************************************************************
**
** void add_tcodes( struct timecode tcode1, struct timecode tcode2,
**                 struct timecode *tcodeout);
**
**
** DESC:
**
**  add_tcodes() adds two timecodes and stores the result in an
**  output structure.
**
**
** INPUTS:
**  struct timecode tcode1  -- timecode struct
**  struct timecode tcode2  -- timecode struct
**
** OUTPUTS:
**  struct timecode *tcodeout  -- ptr to output timecode
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
void add_tcodes( struct timecode tcode1, struct timecode tcode2,
                 struct timecode *tcodeout)
{

    int mult1, mult2;

    mult1 = (tcode1.netflag == TRUE) ? -1 : 0;
    mult2 = (tcode2.netflag == TRUE) ? -1 : 0;


    /* first do the addition, the check for carrys */

    tcodeout->ff = (tcode1.ff*mult1) + (tcode2.ff*mult2);
    tcodeout->ss = (tcode1.ff*mult1) + (tcode2.ff*mult2);
    tcodeout->mm = (tcode1.ff*mult1) + (tcode2.ff*mult2);
    tcodeout->hh = (tcode1.ff*mult1) + (tcode2.ff*mult2);

    /* carry */

    if (tcodeout->ff > 29)
    {
        tcodeout->ss++;
        tcodeout->ff -= 30;
    }

    if (tcodeout->ss > 59)
    {
        tcodeout->mm++;
        tcodeout->ss -= 60;
    }

    if (tcodeout->mm > 59)
    {
        tcodeout->hh++;
        tcodeout->mm -= 60;
    }

    return;
}





/****************************************************************************
**
** void incr_tcode_frames( struct timecode *tcode, long int incr_frames)
**
** DESC:
**  incr_tcode_frames() increments the timecode struct by the desired number
** of frames.
**
**
** INPUTS:
**  struct timecode *tcode  -- ptr to timecode struct (also used for output)
**  long int incr_frames    -- number of frames to increment (may be negative)
**
** OUTPUTS: (results are returned in *tcode)
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
void incr_tcode_frames( struct timecode *tcode, long int incr_frames)
{

    /* increment the frames, taking sign of the timecode into account */

    tcode->ff = (tcode->negflag == TRUE) ? (-1 * tcode->ff + incr_frames)







#endif /* TRASCAN_CODE */


