/****************************************************************************
**
**                           rfx_init.c
**
**  This file contains functions used during the initalization and shutdown
** of the ProWave program.
**
**
**
**
****************************************************************************/


/*-- System Include files --*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <exec/types.h>
#include <exec/memory.h>
#include <utility/hooks.h>
#include <devices/input.h>
#include <devices/inputevent.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>
#include <intuition/sghooks.h>

#include <graphics/displayinfo.h>
#include <graphics/rastport.h>
#include <graphics/text.h>
#include <libraries/dos.h>
#include <intuition/sghooks.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/diskfont_protos.h>
#include <clib/utility_protos.h>
#include <clib/dos_protos.h>



/*-- User Include Files --*/

#include "rfx_common.h"
#include "rfx_init.h"
#include "button_images.h"
#include "rfx_gadget_ids.h"
#include "err_lib.h"
#include "pw_util_lib.h"
#include "timecode_lib.h"



/*-- Internal function prototypes --*/
ULONG  str_hookRoutine(struct Hook *hook, struct SGWork *sgw, ULONG *msg);
void   initHook(struct Hook *hook, ULONG (*ccode)());
int FindFlyer(void);


/*--onstants --*/

#define INTUITION_REV 37



/*-- Global Variables:  Library Base Pointers --*/

struct IntuitionBase *IntuitionBase = NULL;
struct Library *DiskfontBase = NULL;


/*-- Global Variables:  Others --*/

struct Screen       *screen      = NULL;
struct Window       *rf_win      = NULL;
struct TextFont     *topaz11     = NULL;
struct Window       *startupwin  = NULL;
struct Border       *startupborder = NULL;


struct Screen *Scr;
struct Window *Win;

struct ViewPort         *wvp;
struct TmpRas           tras;
struct AreaInfo         ainfo;
UWORD   areabuf[250];    /* area buffer for 100 points (2.5 words/point)*/
PLANEPTR                tbuf;

/*-- Input device variables --*/
struct IOStdReq         *InputIO = NULL;       /* IO Request Block */
struct MsgPort          *InputMP = NULL;       /* Message Port */
int                     iodevice_open = FALSE;




/*-- borders --*/

struct Border           *win_border;
struct Border           *framesel_border;

/*-- Slider Data --*/
struct PropInfo slider1prop = { (AUTOKNOB | FREEHORIZ),
                            0, 0, MAXBODY, 0, 0, 0, 0, 0, 0, 0};

struct Image    slider1image;   /* image struct to provde to slider gadget */

/*-- Integer String Gadget Constans/Variables --*/
struct Border       *len_border;

UBYTE   len_buffer[LEN_STR_WIDTH];
UBYTE   len_undobuffer[LEN_STR_WIDTH];
struct StringExtend len_extend;

struct StringInfo   len_strinfo = {len_buffer, len_undobuffer, 0,
                                    LEN_STR_WIDTH, 0, 0,0,0,0, NULL, 0,
                                    NULL };
struct Border   *percent_border;
UBYTE   percent_buffer[PERCENT_STR_WIDTH];
UBYTE   percent_undobuffer[PERCENT_STR_WIDTH];
struct StringExtend percent_extend;
struct Hook    percent_hook;
UBYTE          percent_workbuffer[PERCENT_STR_WIDTH];

struct StringInfo   percent_strinfo = {percent_buffer, percent_undobuffer, 0,
                                    PERCENT_STR_WIDTH, 0, 0,0,0,0, NULL, 0,
                                    NULL };

struct Border   *tension_border;
UBYTE   tension_buffer[TENSION_STR_WIDTH];
UBYTE   tension_undobuffer[TENSION_STR_WIDTH];
struct StringExtend tension_extend;
struct Hook    tension_hook;
UBYTE          tension_workbuffer[TENSION_STR_WIDTH];

struct StringInfo   tension_strinfo = {tension_buffer, tension_undobuffer, 0,
                                    TENSION_STR_WIDTH, 0, 0,0,0,0, NULL, 0,
                                    NULL };


struct Border   *cont_border;
UBYTE   cont_buffer[CONT_STR_WIDTH];
UBYTE   cont_undobuffer[CONT_STR_WIDTH];
struct StringExtend cont_extend;
struct Hook    cont_hook;
UBYTE          cont_workbuffer[CONT_STR_WIDTH];

struct StringInfo   cont_strinfo = {cont_buffer, cont_undobuffer, 0,
                                    CONT_STR_WIDTH, 0, 0,0,0,0, NULL, 0,
                                    NULL };


struct Border   *bias_border;
UBYTE   bias_buffer[BIAS_STR_WIDTH];
UBYTE   bias_undobuffer[BIAS_STR_WIDTH];
struct StringExtend bias_extend;
struct Hook    bias_hook;
UBYTE          bias_workbuffer[BIAS_STR_WIDTH];

struct StringInfo   bias_strinfo = {bias_buffer, bias_undobuffer, 0,
                                    BIAS_STR_WIDTH, 0, 0,0,0,0, NULL, 0,
                                    NULL };



struct Border   *frame_border;
UBYTE   frame_buffer[FRAME_STR_WIDTH];
UBYTE   frame_undobuffer[FRAME_STR_WIDTH];
struct StringExtend frame_extend;

struct StringInfo   frame_strinfo = {frame_buffer, frame_undobuffer, 0,
                                    FRAME_STR_WIDTH, 0, 0,0,0,0, NULL, 0,
                                    NULL };


/*-- Gadget Structures --*/


/*-- Gadget List for rf_win --*/


struct Gadget _a_left_gad = { NULL,
                            30, 110,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _A_LEFT_ID, NULL};

struct Gadget _a_right_gad = { &_a_left_gad,
                            47, 110,     /* X and Y position */
                            16, 17,      /* Width and Height */
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _A_RIGHT_ID, NULL};


struct Gadget _b_left_gad = { &_a_right_gad,
                            430, 110,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _B_LEFT_ID, NULL};

struct Gadget _b_right_gad = { &_b_left_gad,
                            447, 110,     /* X and Y position */
                            16, 17,      /* Width and Height */
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _B_RIGHT_ID, NULL};

struct Gadget _len_left_gad = { &_b_right_gad,
                            315, 110,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _LEN_LEFT_ID, NULL};

struct Gadget _len_right_gad = { &_len_left_gad,
                            332, 110,     /* X and Y position */
                            16, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _LEN_RIGHT_ID, NULL};

struct Gadget _save_gad = { &_len_right_gad,
                            10, 375,     /* X and Y position */
                            56, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_save_image, &_save_sel_image,
                            NULL, NULL, NULL,
                            _SAVE_ID, NULL};

struct Gadget _load_gad = { &_save_gad,
                            10, 350,     /* X and Y position */
                            56, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_load_image, &_load_sel_image,
                            NULL, NULL, NULL,
                            _LOAD_ID, NULL};

struct Gadget _realtime_gad = { &_load_gad,
                            300, 370,     /* X and Y position */
                            80, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_realtime_image, &_realtime_sel_image,
                            NULL, NULL, NULL,
                            _REALTIME_ID, NULL};


struct Gadget _preview_gad = { &_realtime_gad,
                            382, 370,     /* X and Y position */
                            80, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_preview_image, &_preview_sel_image,
                            NULL, NULL, NULL,
                            _PREVIEW_ID, NULL};

struct Gadget _render_gad = { &_preview_gad,
                            464, 370,     /* X and Y position */
                            80, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_render_image, &_render_sel_image,
                            NULL, NULL, NULL,
                            _RENDER_ID, NULL};

struct Gadget _cancel_gad = { &_render_gad,
                            546, 370,     /* X and Y position */
                            80, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_cancel_image, &_cancel_sel_image,
                            NULL, NULL, NULL,
                            _CANCEL_ID, NULL};

struct Gadget _view_gad = { &_cancel_gad,
                            494, 135,     /* X and Y position */
                            56, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_view_image, &_view_sel_image,
                            NULL, NULL, NULL,
                            _VIEW_ID, NULL};


struct Gadget _drag_gad = { &_view_gad,
                            438, 135,     /* X and Y position */
                            56, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_IMMEDIATE),
                            GTYP_BOOLGADGET,
                            &_drag_image, &_drag_sel_image,
                            NULL, NULL, NULL,
                            _DRAG_ID, NULL};

struct Gadget _create_gad = { &_drag_gad,
                            326, 135,     /* X and Y position */
                            56, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_IMMEDIATE),
                            GTYP_BOOLGADGET,
                            &_create_image, &_create_sel_image,
                            NULL, NULL, NULL,
                            _CREATE_ID, NULL};

struct Gadget _delete_gad = { &_create_gad,
                            382, 135,     /* X and Y position */
                            56, 20,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_IMMEDIATE),
                            GTYP_BOOLGADGET,
                            &_delete_image, &_delete_sel_image,
                            NULL, NULL, NULL,
                            _DELETE_ID, NULL};

struct Gadget _linear_gad = { &_delete_gad,
                            250, 322,     /* X and Y position */
                            20, 19,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_IMMEDIATE),
                            GTYP_BOOLGADGET,
                            &_toggle_image, &_toggle_sel_image,
                            NULL, NULL, NULL,
                            _LINEAR_ID, NULL};

struct Gadget _bias_left_gad = { &_linear_gad,
                            452, 322,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _BIAS_LEFT_ID, NULL};

struct Gadget _bias_right_gad = { &_bias_left_gad,
                            469, 322,     /* X and Y position */
                            16, 17,      /* Width and Height */
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _BIAS_RIGHT_ID, NULL};


struct Gadget _bias_str_gad = { &_bias_right_gad,
                            400, 325,     /* X and Y position */
                            0, 0,    /* Width and Height !!will be set later*/
                            (GFLG_GADGHCOMP | GFLG_STRINGEXTEND),
                            (GACT_RELVERIFY | GACT_STRINGRIGHT),
                            GTYP_STRGADGET,
                            NULL, /* will be defined later*/
                            NULL,
                            NULL, 0, &bias_strinfo,
                            _BIAS_STRING_ID, NULL};

struct Gadget _cont_left_gad = { &_bias_str_gad,
                            552, 322,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _CONT_LEFT_ID, NULL};

struct Gadget _cont_right_gad = { &_cont_left_gad,
                            569, 322,     /* X and Y position */
                            16, 17,      /* Width and Height */
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _CONT_RIGHT_ID, NULL};


struct Gadget _cont_str_gad = { &_cont_right_gad,
                            500, 325,     /* X and Y position */
                            0, 0,    /* Width and Height !!will be set later*/
                            (GFLG_GADGHCOMP | GFLG_STRINGEXTEND),
                            (GACT_RELVERIFY | GACT_STRINGRIGHT),
                            GTYP_STRGADGET,
                            NULL, /* will be defined later*/
                            NULL,
                            NULL, 0, &cont_strinfo,
                            _CONT_STRING_ID, NULL};


struct Gadget _tension_str_gad = { &_cont_str_gad,
                            300, 325,     /* X and Y position */
                            0, 0,    /* Width and Height !!will be set later*/
                            (GFLG_GADGHCOMP | GFLG_STRINGEXTEND),
                            (GACT_RELVERIFY | GACT_STRINGRIGHT),
                            GTYP_STRGADGET,
                            NULL, /* will be defined later*/
                            NULL,
                            NULL, 0, &tension_strinfo,
                            _TENSION_STRING_ID, NULL};

struct Gadget _tension_left_gad = { &_tension_str_gad,
                            352, 322,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _TENSION_LEFT_ID, NULL};

struct Gadget _tension_right_gad = { &_tension_left_gad,
                            369, 322,     /* X and Y position */
                            16, 17,      /* Width and Height */
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_FOLLOWMOUSE | GACT_IMMEDIATE | GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _TENSION_RIGHT_ID, NULL};




struct Gadget _percent_str_gad = { &_tension_right_gad,
                            200, 372,     /* X and Y position */
                            0, 0,    /* Width and Height !!will be set later*/
                            (GFLG_GADGHCOMP | GFLG_STRINGEXTEND),
                            (GACT_RELVERIFY | GACT_STRINGRIGHT),
                            GTYP_STRGADGET,
                            NULL, /* will be defined later*/
                            NULL,
                            NULL, 0, &percent_strinfo,
                            _PERCENT_STRING_ID, NULL};


struct Gadget _frame_str_gad = { &_percent_str_gad,
                            200, 355,     /* X and Y position */
                            0, 0,    /* Width and Height !!will be set later*/
                            (GFLG_GADGHCOMP | GFLG_STRINGEXTEND),
                            (GACT_LONGINT |GACT_RELVERIFY | GACT_STRINGRIGHT),
                            GTYP_STRGADGET,
                            NULL, /* will be defined later*/
                            NULL,
                            NULL, 0, &frame_strinfo,
                            _FRAME_STRING_ID, NULL};

struct Gadget _frame_left_gad = { &_frame_str_gad,
                            253, 352,     /* X and Y position */
                            17, 17,      /* Width and Height *///
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_left_arrow_image, &_left_arrow_sel_image,
                            NULL, NULL, NULL,
                            _FRAME_LEFT_ID, NULL};

struct Gadget _frame_right_gad = { &_frame_left_gad,
                            270, 352,     /* X and Y position */
                            16, 17,      /* Width and Height */
                            (GFLG_GADGIMAGE | GFLG_GADGHIMAGE),
                            (GACT_RELVERIFY),
                            GTYP_BOOLGADGET,
                            &_right_arrow_image, &_right_arrow_sel_image,
                            NULL, NULL, NULL,
                            _FRAME_RIGHT_ID, NULL};



struct Gadget _len_str_gad = { &_frame_right_gad,
                            260, 113,     /* X and Y position */
                            0, 0,    /* Width and Height !!will be set later*/
                            (GFLG_GADGHCOMP | GFLG_STRINGEXTEND),
                            (GACT_LONGINT |GACT_RELVERIFY | GACT_STRINGRIGHT),
                            GTYP_STRGADGET,
                            NULL, /* will be defined later*/
                            NULL,
                            NULL, 0, &len_strinfo,
                            _LEN_STRING_ID, NULL};





#define RFX_WIN_W        (640)
#define RFX_WIN_H        (399)



/****************************************************************************
**
** int  init_rfx(void)
**
** DESC:
**
**  init_rfx() performs the initialization functions for the render FX window.
** It initalizes the gadgets and global variables, opens the display windows
** and initializes the display buffers and data structures
**
**
** INPUTS:  void
**
** OUTPUTS:
**
** RETURNS:
**     OK        success
**     ERROR     error allocating some resource
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/

int init_rfx(void)
{

    struct  ColorSpec  cmap[17];
    UWORD pens[10];

    struct TextExtent   t_ext;
    struct timecode     tcode;


    struct  TextAttr topazta = {"topaz.font", 11, FS_NORMAL, FPF_DISKFONT};

    char        format_string[20], string[30];



    /*-- Open the message port and setup input devicde --*/

    if ( (InputMP = CreateMsgPort()) == NULL)
    {
        log_message(ERROR_MSG,"Unable to create message port in init_rfx()\n");
        return (ERROR);
    }

    if ( (InputIO = CreateIORequest(InputMP,sizeof(struct IOStdReq))) == NULL)
    {
        log_message(ERROR_MSG,"Unable to create IO Req in init_rfx()\n");
        return (ERROR);
    }

    if ( (OpenDevice("input.device",NULL,(struct IORequest *)InputIO,NULL))
         != NULL)
    {
        log_message(ERROR_MSG,"Unable to open input.device in init_rfx()\n");
        return(ERROR);
    }
    iodevice_open = TRUE;



    /*-- Open Libraries --*/

    /*-- open the Intuition Library --*/

    if ( (IntuitionBase = (struct IntuitionBase *)OpenLibrary( "intuition.library", INTUITION_REV))
         == NULL)
    {
        log_message(ERROR_MSG,"Unable to open intuition in init_rfx()\n");
        return(ERROR);
    }

    /*-- open the diskfont library --*/
    if ( (DiskfontBase = OpenLibrary("diskfont.library", 37L)) == NULL)
    {
        log_message(ERROR_MSG,"Unable to open diskfont in init_rfx()\n");
        return(ERROR);
    }




    /*-- Setup Data Structures --*/

    /*-- specify the color map --*/
    cmap[0].ColorIndex =  0;  /* BLACKPEN */
    cmap[0].Red        =  1;
    cmap[0].Green      =  1;
    cmap[0].Blue       =  1;
    cmap[1].ColorIndex =  1;  /* MDGREY1PEN */
    cmap[1].Red        =  5;
    cmap[1].Green      =  5;
    cmap[1].Blue       =  5;
    cmap[2].ColorIndex =  2;  /* LTGREY1PEN */
    cmap[2].Red        =  9;
    cmap[2].Green      =  9;
    cmap[2].Blue       =  9;
    cmap[3].ColorIndex =  3;  /* LTGREY2PEN */
    cmap[3].Red        = 13;
    cmap[3].Green      = 13;
    cmap[3].Blue       = 13;
    cmap[4].ColorIndex =  4;  /* DKGREY1PEN */
    cmap[4].Red        =  2;
    cmap[4].Green      =  2;
    cmap[4].Blue       =  2;
    cmap[5].ColorIndex =  5;  /* DKGREY2PEN */
    cmap[5].Red        =  4;
    cmap[5].Green      =  4;
    cmap[5].Blue       =  4;
    cmap[6].ColorIndex =  6;  /* DKYELLOWPEN */
    cmap[6].Red        = 11;
    cmap[6].Green      = 10;
    cmap[6].Blue       =  6;
    cmap[7].ColorIndex =  7;  /* LTYELLOWPEN */
    cmap[7].Red        = 15;
    cmap[7].Green      = 14;
    cmap[7].Blue       =  9;
    cmap[8].ColorIndex =  8;  /* LTBLUE1PEN */
    cmap[8].Red        =  6;
    cmap[8].Green      =  9;
    cmap[8].Blue       = 12;
    cmap[9].ColorIndex =  9;  /* LTPURPLE1PEN */
    cmap[9].Red        =  9;
    cmap[9].Green      =  6;
    cmap[9].Blue       =  6;
    cmap[10].ColorIndex = 10;  /* DKGREEN1PEN */
    cmap[10].Red        =  3;
    cmap[10].Green      =  3;
    cmap[10].Blue       =  2;
    cmap[11].ColorIndex = -1;
    cmap[11].Red        =  0;
    cmap[11].Green      =  0;
    cmap[11].Blue       =  0;


    pens[DETAILPEN] = 2;
    pens[BLOCKPEN]  = 1;
    pens[TEXTPEN]   = 2;
    pens[SHINEPEN]  = 7;
    pens[SHADOWPEN] = 4;
    pens[FILLPEN]   = 6;
    pens[FILLTEXTPEN] =  1;
    pens[BACKGROUNDPEN] = 0;
    pens[HIGHLIGHTTEXTPEN] = 2;
    pens[9] = (UWORD)~0;

    /*-- Allocate the border lists --*/


    if( (win_border = create_win_border ( 640,  399, 2,
                                                    LOWERED, 3, 4,
                                                    TRUE, 0)) == NULL)
    {
        log_message(ERROR_MSG,"Unable to create win border in init_rfx()\n");

        return (ERROR);
    }

    if( (framesel_border = create_win_border ( 620,  42, 2,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {
        log_message(ERROR_MSG,"Unable to create frame border in init_rfx()\n");
        return (ERROR);
    }



    /*-- Open Screen --*/

   if (FindFlyer() == 0)
   {


        screen = OpenScreenTags(NULL,
                             SA_Width,       640,
                             SA_Height,      400,
                             SA_Overscan,    OSCAN_VIDEO,
                             SA_Pens,        (ULONG)pens,
                             SA_DisplayID,   HIRESLACE_KEY,
                             SA_Depth,       4,
                             SA_Colors,      cmap,
                             SA_ShowTitle,   FALSE,
                             SA_Quiet,       TRUE,
                             TAG_DONE);

 
        if (screen == NULL)
        {
            log_message(ERROR_MSG,"Unable to open screen in init_rfx()\n");

            return (ERROR);
        }

    }
    else
    {
        screen = Scr;
    }




    /*-- Open the topaz font --*/
    if ( (topaz11 = OpenDiskFont(&topazta)) == NULL)
    {
        log_message(ERROR_MSG,"Unable to open Topaz11 in init_rfx()\n");

        return(ERROR);
    }



    /*-- Open Backdrop Window --*/

//    printf("%d  %d\n", screen->Width, screen->Height);

    rf_win = OpenWindowTags (  NULL,
                                WA_Left,                64,
                                WA_Top, 		40,
                                WA_Width,               RFX_WIN_W,
                                WA_Height,              RFX_WIN_H,
                                WA_SizeGadget,          FALSE,
                                WA_DragBar,             FALSE,
                                WA_DepthGadget,         FALSE,
                                WA_CloseGadget,         FALSE,
                                WA_Backdrop,            FALSE,
                                WA_Borderless,          TRUE,
                                WA_SmartRefresh,        TRUE,
                                WA_ReportMouse,         TRUE,
                                WA_DetailPen,           4, /*1 */
                                WA_BlockPen,            2, /* 4*/
                                WA_IDCMP,         ( IDCMP_GADGETDOWN
                                                   |IDCMP_GADGETUP
                                                   /* | IDCMP_RAWKEY */
                                                   | IDCMP_MOUSEBUTTONS
                                                   | IDCMP_MOUSEMOVE ),
                                WA_CustomScreen,        screen,
                               TAG_DONE);

    if (rf_win == NULL )
    {
        log_message(ERROR_MSG,"Unable to create main window in init_rfx()\n");
        return (ERROR);
    }



    /* create area for area fills */
    wvp = (struct ViewPort *)ViewPortAddress(rf_win);

    InitArea(&ainfo, areabuf, 100);        /* init area buffer */
    rf_win->RPort->AreaInfo = &ainfo;

    if ( (tbuf = (PLANEPTR)AllocRaster(700,100)) == NULL )
    {
        log_message(ERROR_MSG,"Unable to allocate rast buff in init_rfx()\n");
        return(ERROR);
    }

    rf_win->RPort->TmpRas = (struct TmpRas *)InitTmpRas
            (&tras, tbuf, RASSIZE(700,100) );

    /*-- color the background --*/
    SetRast(rf_win->RPort, 1);

    /*-- set the default font --*/
    SetFont(rf_win->RPort, topaz11);

    /* now that we have a font, fill out extended string structure */

    /*- setup transition length gadget -*/

    /* create format string of the correct width */
    sprintf(format_string, "%%0%dld", LEN_STR_WIDTH);

    /* format the value */
    sprintf(string, format_string, 0);


    TextExtent( rf_win->RPort, (STRPTR)string, LEN_STR_WIDTH, &t_ext);

    _len_str_gad.Width = t_ext.te_Width;
    _len_str_gad.Height = t_ext.te_Height;


    if( (len_border = create_win_border ( _len_str_gad.Width+4,
                                          _len_str_gad.Height+4, 2,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {
        log_message(ERROR_MSG,"Unable to create len border in init_rfx()\n");
        return (ERROR);
    }

    shift_border( len_border, -2, -2);  /* move border to surround gadget */

    _len_str_gad.GadgetRender = len_border;

    len_extend.Font = topaz11;
    len_extend.Pens[0] = 2;
    len_extend.Pens[1] = 0;
    len_extend.ActivePens[0] = 6;
    len_extend.ActivePens[1] = 0;
    len_extend.InitialModes   = (SGM_REPLACE);
    len_strinfo.Extension = &len_extend;


    /*- Setup percentage string gadget -*/
    /* create format string of the correct width */
    sprintf(format_string, "%%0%dld", PERCENT_STR_WIDTH);

    /* format the value */
    sprintf(string, format_string, 0);


    TextExtent( rf_win->RPort, (STRPTR)string, PERCENT_STR_WIDTH, &t_ext);

    _percent_str_gad.Width = t_ext.te_Width;
    _percent_str_gad.Height = t_ext.te_Height;


    if( (percent_border = create_win_border ( _percent_str_gad.Width+4,
                                          _percent_str_gad.Height+4, 2,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {

        log_message(ERROR_MSG,"Unable to create % border in init_rfx()\n");
        return (ERROR);
    }

    shift_border( percent_border, -2, -2);  /*move border to surround gadget*/

    _percent_str_gad.GadgetRender = percent_border;

    percent_extend.Font = topaz11;
    percent_extend.Pens[0] = 2;
    percent_extend.Pens[1] = 0;
    percent_extend.ActivePens[0] = 6;
    percent_extend.ActivePens[1] = 0;
    percent_extend.InitialModes   = (SGM_REPLACE);
    percent_extend.EditHook = &percent_hook;
    percent_extend.WorkBuffer = percent_workbuffer;

    percent_strinfo.Extension = &percent_extend;

    initHook(&percent_hook, str_hookRoutine);


    /*- setup frame gadget -*/
    /* create format string of the correct width */
    sprintf(format_string, "%%0%dld", FRAME_STR_WIDTH);

    /* format the value */
    sprintf(string, format_string, 0);


    TextExtent( rf_win->RPort, (STRPTR)string, FRAME_STR_WIDTH, &t_ext);

    _frame_str_gad.Width = t_ext.te_Width;
    _frame_str_gad.Height = t_ext.te_Height;


    if( (frame_border = create_win_border ( _frame_str_gad.Width+4,
                                          _frame_str_gad.Height+4, 2,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {
        log_message(ERROR_MSG,"Unable to frame border in init_rfx()\n");
        return (ERROR);
    }

    shift_border( frame_border, -2, -2);  /* move border to surround gadget */

    _frame_str_gad.GadgetRender = frame_border;

    frame_extend.Font = topaz11;
    frame_extend.Pens[0] = 2;
    frame_extend.Pens[1] = 0;
    frame_extend.ActivePens[0] = 6;
    frame_extend.ActivePens[1] = 0;
    frame_extend.InitialModes   = (SGM_REPLACE);
    frame_strinfo.Extension = &frame_extend;



    /*- Setup Spline Tension string gadget -*/
    /* create format string of the correct width */
    sprintf(format_string, "%%0%dld", TENSION_STR_WIDTH);

    /* format the value */
    sprintf(string, format_string, 0);


    TextExtent( rf_win->RPort, (STRPTR)string, TENSION_STR_WIDTH, &t_ext);

    _tension_str_gad.Width = t_ext.te_Width;
    _tension_str_gad.Height = t_ext.te_Height;


    if( (tension_border = create_win_border ( _tension_str_gad.Width+4,
                                          _tension_str_gad.Height+4, 2,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {
        log_message(ERROR_MSG,"Unable to create tens brdr in init_rfx()\n");
        return (ERROR);
    }

    shift_border( tension_border, -2, -2);  /*move border to surround gadget*/

    _tension_str_gad.GadgetRender = tension_border;

    tension_extend.Font = topaz11;
    tension_extend.Pens[0] = 2;
    tension_extend.Pens[1] = 0;
    tension_extend.ActivePens[0] = 6;
    tension_extend.ActivePens[1] = 0;
    tension_extend.InitialModes   = (SGM_REPLACE);
    tension_extend.EditHook = &tension_hook;
    tension_extend.WorkBuffer = tension_workbuffer;

    tension_strinfo.Extension = &tension_extend;

    initHook(&tension_hook, str_hookRoutine);




    /*- Setup Spline Continuity string gadget -*/
    /* create format string of the correct width */
    sprintf(format_string, "%%0%dld", CONT_STR_WIDTH);

    /* format the value */
    sprintf(string, format_string, 0);


    TextExtent( rf_win->RPort, (STRPTR)string, CONT_STR_WIDTH, &t_ext);

    _cont_str_gad.Width = t_ext.te_Width;
    _cont_str_gad.Height = t_ext.te_Height;


    if( (cont_border = create_win_border ( _cont_str_gad.Width+4,
                                          _cont_str_gad.Height+4, 2,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {
        log_message(ERROR_MSG,"Unable to create cont brdr in init_rfx()\n");
        return (ERROR);
    }

    shift_border( cont_border, -2, -2);  /*move border to surround gadget*/

    _cont_str_gad.GadgetRender = cont_border;

    cont_extend.Font = topaz11;
    cont_extend.Pens[0] = 2;
    cont_extend.Pens[1] = 0;
    cont_extend.ActivePens[0] = 6;
    cont_extend.ActivePens[1] = 0;
    cont_extend.InitialModes   = (SGM_REPLACE);
    cont_extend.EditHook = &cont_hook;
    cont_extend.WorkBuffer = cont_workbuffer;

    cont_strinfo.Extension = &cont_extend;

    initHook(&cont_hook, str_hookRoutine);



    /*- Setup Spline Bias string gadget -*/
    /* create format string of the correct width */
    sprintf(format_string, "%%0%dld", BIAS_STR_WIDTH);

    /* format the value */
    sprintf(string, format_string, 0);


    TextExtent( rf_win->RPort, (STRPTR)string, BIAS_STR_WIDTH, &t_ext);

    _bias_str_gad.Width = t_ext.te_Width;
    _bias_str_gad.Height = t_ext.te_Height;


    if( (bias_border = create_win_border ( _bias_str_gad.Width+4,
                                          _bias_str_gad.Height+4, 2,
                                                    LOWERED, 3, 4,
                                                    FALSE, 0)) == NULL)
    {
        log_message(ERROR_MSG,"Unable to create bias brdr in init_rfx()\n");
        return (ERROR);
    }

    shift_border( bias_border, -2, -2);  /*move border to surround gadget*/

    _bias_str_gad.GadgetRender = bias_border;

    bias_extend.Font = topaz11;
    bias_extend.Pens[0] = 2;
    bias_extend.Pens[1] = 0;
    bias_extend.ActivePens[0] = 6;
    bias_extend.ActivePens[1] = 0;
    bias_extend.InitialModes   = (SGM_REPLACE);
    bias_extend.EditHook = &bias_hook;
    bias_extend.WorkBuffer = bias_workbuffer;

    bias_strinfo.Extension = &bias_extend;

    initHook(&bias_hook, str_hookRoutine);




    DrawImage(rf_win->RPort, &_rflogolg_image, 210, 50);



    /*-- Draw Borders --*/
    DrawBorder( rf_win->RPort, win_border, 0, 0);
    DrawBorder( rf_win->RPort, framesel_border, 9, 90);

    /*-- add the text --*/

    frame2tcode( 0L, &tcode);

    SetAPen(rf_win->RPort, 0);
    SetBPen(rf_win->RPort, 1);
    SetDrMd(rf_win->RPort, JAM2);

    Move(rf_win->RPort, 66, 107);
    Text(rf_win->RPort, "A Start Point", 13);

    Move(rf_win->RPort, 240, 315);
    Text(rf_win->RPort, "Linear", 6);

    Move(rf_win->RPort, 308, 315);
    Text(rf_win->RPort, "Tension", 7);

    Move(rf_win->RPort, 414, 315);
    Text(rf_win->RPort, "Bias", 4);

    Move(rf_win->RPort, 494, 315);
    Text(rf_win->RPort, "Continuity", 10);




    Move(rf_win->RPort, A_TCODEX, A_TCODEY);
    draw_timecode(rf_win->RPort, tcode, FALSE);


    Move(rf_win->RPort, 466, 107);
    Text(rf_win->RPort, "B Start Point", 13);

    Move(rf_win->RPort, 76, 62);
    Text(rf_win->RPort, "Video A Out", 11);

    Move(rf_win->RPort, 478, 62);
    Text(rf_win->RPort, "Video B In", 10);


    Move(rf_win->RPort, 234, 105);
    Text(rf_win->RPort, "Transition Length", 17);

    Move(rf_win->RPort, 190, 148);
    Text(rf_win->RPort, "Graph Operations", 16);

    Move(rf_win->RPort, 18, 338);
    Text(rf_win->RPort, "Graph", 5);

    Move(rf_win->RPort, 154, 365);
    Text(rf_win->RPort, "Frame", 5);

    Move(rf_win->RPort, 120, 381);
    Text(rf_win->RPort, "Percent B", 9);

    Move(rf_win->RPort, 262, 381);
    Text(rf_win->RPort, "%", 1);

    Move(rf_win->RPort, B_TCODEX, B_TCODEY);
    draw_timecode(rf_win->RPort, tcode, FALSE);


    /*- Process Bar -*/

    /*-- Add the gadgets --*/

    AddGList(rf_win, &_len_str_gad, 0, -1, NULL);

    RefreshGList(rf_win->FirstGadget, rf_win, NULL, -1);


    return(OK);
}






/****************************************************************************
**
** int shutdown_rfx(void)
**
**
** DESC:
**
**  shutdown_rfx() frees all system resources associated with the
** RenderFX program.  It frees all allocated memory blocks and system
** resources, closes the windows & screen, and closes the libraries.
**
**
** INPUTS: void
**
** OUTPUTS: none
**
** RETURNS:
**    OK      success
**    ERROR   error
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/

int shutdown_rfx(void)
{


    /*-- Close the file --*/
    if ( topaz11 == NULL)
    {
        CloseFont(topaz11);
    }


    /*-- Close the backdrop window --*/

    if( rf_win != NULL )
    {
        CloseWindow(rf_win);
    }


    /*-- Close the Screen --*/
    if (screen != NULL)
    {
        CloseScreen(screen);
    }

    /* free the flood raster */
    if (tbuf != NULL)
    {
        FreeRaster(tbuf, 700, 100);
    }

    if (win_border != NULL)
    {
        free_win_border(win_border);
    }

    if (framesel_border != NULL)
    {
        free_win_border(framesel_border);
    }

    if (percent_border != NULL)
    {
        free_win_border(percent_border);
    }

    if (len_border != NULL)
    {
        free_win_border(len_border);
    }

    if (bias_border != NULL)
    {
        free_win_border(bias_border);
    }

    if (tension_border != NULL)
    {
        free_win_border(tension_border);
    }

    if (cont_border != NULL)
    {
        free_win_border(cont_border);
    }


    if (frame_border != NULL)
    {
        free_win_border(frame_border);
    }


    /* Close libraries and exit */

    if (DiskfontBase != NULL)
    {
        CloseLibrary(DiskfontBase);
    }

    if (IntuitionBase != NULL)
    {
        CloseLibrary((struct Library *)IntuitionBase);
    }

    if (iodevice_open == TRUE)
    {
        CloseDevice( (struct IORequest *)InputIO);
    }

    if (InputIO != NULL)
    {
        DeleteIORequest(InputIO);
    }

    if (InputMP != NULL)
    {
        DeleteMsgPort(InputMP);
    }




    return(OK);

}


/*
** this is a frunction which converts register-parameter
** hook calling convention into standard C conventions.
** It only works with SAS C 5.0+
**
** Without the fancy __asm stuff, you'd probably need to
** write this in assembler.
**
** You could conceivably declare all your C hook functions
** this way, and eliminate the middleman (you'd initialize
** the h_Entry field to your C function's address, and not
** bother with the h_SubEntry field).
**
** This is nice and easy, though...
**
*/

ULONG __saveds __asm hookEntry(register __a0 struct Hook *hookptr,
     register __a2 void *object,
     register __a1 void *message)
{

    return((*hookptr->h_SubEntry)(hookptr, object, message));
}


/*
** Initialize the hook to use the hookEntry() routine above.
*/

void initHook(struct Hook *hook, ULONG (*ccode)())
{
    hook->h_Entry = hookEntry;
    hook->h_SubEntry = ccode;
    hook->h_Data = 0;  /* this program does not use this */
}


/*
** This is the floating point string editing hook.  It only
** allows digits and decimal points.
**
**
*/

ULONG str_hookRoutine(struct Hook *hook, struct SGWork *sgw, ULONG *msg)
{
    UBYTE *work_ptr;
    ULONG return_code;

    /*
    ** hook must return non-zero if command is supported.
    ** this will be changed to zero if command is unsupported.
    */

    return_code = ~0L;

    if (*msg == SGH_KEY)
    {
        /* key hit -- could be any key (shift, repeat, character, etc.) */

        /*
        ** allow only upper case charactersto be entered.
        ** act only on modes that ad or update characters in the buffer.
        */

        if ( (sgw->EditOp == EO_REPLACECHAR) ||
             (sgw->EditOp == EO_INSERTCHAR) )
        {

            /*
            ** Code contains teh ASCII representation of the character
            ** entered, if it maps to a singly buyte.  We could also look
            ** into the owrk buffer to find the new character.
            **
            ** sgw->Code == sgw->WorkBuffer[sgw->BufferPos - 1]
            **
            ** If the character is not either a digit or a period, it is
            ** rejected
            **
            */

            if ( !(isdigit(sgw->Code)) && !(sgw->Code == '.') &&
                 (sgw->Code != '+') && (sgw->Code != '-') &&
                 (sgw->Code != 'E') && (sgw->Code != 'e') )
            {
                sgw->Actions |= SGA_BEEP;
                sgw->Actions &= ~SGA_USE;
            }
        }
        else
        {
            /* unknown command
            ** hook should return zero if the command is not supported.
            */

            return_code = 0;
        }

    }
    return(return_code);
}




/****************************************************************************
**
**  int create_startupwin(void);
**
**
** DESC:
**
**      create_startupwin() creates the RenderFX startup screen.  This screen
** contains a renderFX logo and copyright informaiton.
**
**
** INPUTS:
**  void;
**
** OUTPUTS:
**
**  OK -- success;
**  ERROR -- couldn't open the window;
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int create_startupwin(void)
{
    WORD    xoffset, yoffset, width, height;
    char    buffer[128];
    WORD    centx, centy;

    if (screen == NULL)
    {
        return(ERROR);
    }

    width = 320;
    height = 200;

    xoffset = 160;
    yoffset = 100;


    if( (startupborder = create_win_border ( width,  height, 4,
                                                    RAISED, 3, 4,
                                                    TRUE, 0)) == NULL)
    {
        log_message(ERROR_MSG,
           "Unable to create startup window border in create_startupwin()\n");

        return (ERROR);
    }



    startupwin = OpenWindowTags (  NULL,
                                WA_Left,          xoffset,
                                WA_Top,           yoffset,
                                WA_Width,         width,
                                WA_Height,        height,
                                WA_SizeGadget,    FALSE,
                                WA_DragBar,       FALSE,
                                WA_DepthGadget,   FALSE,
                                WA_CloseGadget,   FALSE,
                                WA_Backdrop,      FALSE,
                                WA_Borderless,    TRUE,
                                WA_SmartRefresh,  TRUE,
                                WA_ReportMouse,   FALSE,
                                WA_DetailPen,     4, /*1 */
                                WA_BlockPen,      2, /* 4*/
                                WA_IDCMP,         NULL,
                                WA_CustomScreen,  screen,
                               TAG_DONE);

    if (startupwin == NULL )
    {
        log_message(ERROR_MSG,
           "Unable to create startup window in create_startupwin()\n");
        return (ERROR);
    }

    /*-- color the background --*/
    SetRast(startupwin->RPort, 1);

    SetAPen(startupwin->RPort, 0);
    SetBPen(startupwin->RPort, 1);
    SetDrMd(startupwin->RPort, JAM2);


    DrawBorder( startupwin->RPort, startupborder, 0, 0);


    DrawImage(startupwin->RPort, &_rflogolg_image, 50, 20);


    sprintf(buffer,"Envelope Editor %s", VERSION_STRING);

    center_text(startupwin->RPort, width, height, buffer, &centx, &centy);

    Move(startupwin->RPort, centx, 60);
    Text(startupwin->RPort, buffer, strlen(buffer));

    sprintf(buffer,"%s","Copyright 1997, Prowave Inc.");

    center_text(startupwin->RPort, width, height, buffer, &centx, &centy);

    Move(startupwin->RPort, centx, 90);
    Text(startupwin->RPort, buffer, strlen(buffer));


    sprintf(buffer,"%s","Prowave is your Friend, Trust Prowave");

    center_text(startupwin->RPort, width, height, buffer, &centx, &centy);

    Move(startupwin->RPort, centx, 110);
    Text(startupwin->RPort, buffer, strlen(buffer));

    Delay(2);

    sprintf(buffer,"%s","                                     ");

    center_text(startupwin->RPort, width, height, buffer, &centx, &centy);

    Move(startupwin->RPort, centx, 110);
    Text(startupwin->RPort, buffer, strlen(buffer));

    sprintf(buffer,"%s","Technical Support:");

    center_text(startupwin->RPort, width, height, buffer, &centx, &centy);

    Move(startupwin->RPort, centx, 110);
    Text(startupwin->RPort, buffer, strlen(buffer));

    sprintf(buffer,"%s","prowave@amiga.org");

    center_text(startupwin->RPort, width, height, buffer, &centx, &centy);

    Move(startupwin->RPort, centx, 125);
    Text(startupwin->RPort, buffer, strlen(buffer));


    return(OK);
}



/****************************************************************************
**
**  void delete_startupwin(void);
**
**
** DESC:
**
**      delete_startupwin() deletes the RenderFX startup screen.
**
**
** INPUTS:
**  void;
**
** OUTPUTS:
**  void
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
void delete_startupwin(void)
{



    if (startupborder != NULL)
    {
        free_win_border(startupborder);
        startupborder = NULL;
    }


    if (startupwin != NULL)
    {
        CloseWindow(startupwin);
        startupwin = NULL;
    }



    return;
}



/*-- function to find the flyer screen --*/


int FindFlyer(void)
{
	int FoundScreen=0,add=-1;
	ULONG ilock;
    	ilock=LockIBase(1L);
    	if (!(Scr=(struct Screen *)(IntuitionBase->FirstScreen))) goto xx1;
    	do  {
		add = add +1;
		if ((Scr->Width == 768) && (Scr->Height == 481))
		{
			FoundScreen = 1;
			goto xx1;
		}
    	}while(Scr=(struct Screen *)Scr->NextScreen);
    	xx1:
    	UnlockIBase(ilock);
	Win = (struct Window *)Scr->FirstWindow;

//printf("%ld\n", (long int)Win);

	return(FoundScreen);
}









/****************************************************************************
**
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/




