/****************************************************************************
**                            Render FX shell
**
**
**
**
**
****************************************************************************/


/*-- System Include files --*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <exec/types.h>
#include <devices/input.h>
#include <devices/inputevent.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>

#include <graphics/displayinfo.h>
#include <libraries/dos.h>
#include <libraries/asl.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/asl_protos.h>
#include <clib/dos_protos.h>



/*-- User Include Files --*/

#include "rfx_common.h"
#include "timecode_lib.h"
#include "pw_util_lib.h"
#include "rfx_init.h"
#include "err_lib.h"
#include "graph_lib.h"
#include "rfx_gadget_ids.h"
#include "interface.h"


#define  VIDEO_TYPE         (1)
#define  STILL_TYPE         (2)
#define  FRAMESTORT_TYPE    (3)
#define  BKG_TYPE           (4)
#define  KEYPAGE_TYPE       (5)

#define  A_CLIP             (1)
#define  B_CLIP             (2)


/* number of segments to draw a spline */

#define NUM_T_STEPS         (15)


/*- invisible pointer sprite -*/
#define INV_PTR_WIDTH       (16)
#define INV_PTR_HEIGHT       (2)
#define INV_PTR_XOFFSET     (0)
#define INV_PTR_YOFFSET     (0)


UWORD __chip inv_ptr[] =
{
    0x0000, 0x0000,     /* reserved, must be NULL */

    0x0000, 0x0000,     /* data, all 0 values */
    0x0000, 0x0000,

    0x0000, 0x0000     /* reserved, must be NULL */
};





/*-- Global variables --*/
/*- the A clip -*/
char                    afilename[257];
int                     afilenamesize = 256;
unsigned long int       acliplen;
unsigned long int       acliptranslen;
unsigned long int       aclipstart;
unsigned long int       aclipout;
unsigned long int       aclipmax;
int                     acliptype;


/*- the B clip -*/
char                    bfilename[257];
int                     bfilenamesize = 256;
unsigned long int       bcliplen;
unsigned long int       bcliptranslen;
unsigned long int       bclipstart;
unsigned long int       bclipin;
unsigned long int       bclipmin;
int                     bcliptype;

unsigned long int       trans_len, temp_ulong;
char                    comment[512];
char                    outfilename[257];
int                     outfilenamesize = 256;
int                     outquality=3;
int                     matte = 0;

int                     max_points = 200;

/*- Graph Mode -*/
int                     graph_mode = 0;
int                     drag_flag  = FALSE;


/*- the data graph */
struct graph_struct     *graph;

/*- flag to indicate view in progress -*/
int			viewflag = FALSE;


/* values for current index point, initialized to bad values to force */
/* gadget update in first call to display_graph_index */

unsigned long int       curr_index_frame = 1000;
double                  curr_index_percent = 10000.0;
double                  curr_index_bias = 10000.0;
double                  curr_index_cont = 10000.0;
double                  curr_index_tension = 10000.0;
int                     curr_index_linear = 1000;

double                  new_tension, new_bias, new_cont;


/*- Graph constants -*/
#define GRAPH_TOP       (160)
#define GRAPH_LEFT      (50)
#define GRAPH_WIDTH     (520)
#define GRAPH_HEIGHT    (135)
#define GRAPH_XTICHEIGHT (5)
#define GRAPH_YTICWIDTH  (5)



/*- "slider button" globals -*/
WORD    bs_basex        = 0;
WORD    bs_basey        = 0;
WORD    bs_screen_basex = 0;
WORD    bs_screen_basey = 0;
int     bs_sensitivity  = 0;
int     bs_gadget_id    = 0;
int     bs_active       = FALSE;
int     bs_lastdirection = 0;
double  bs_scalefactor  = 1.0;
double  bs_scaleincrement = 1.0;


/* General use message buffer (for error logs, requestors, etc...*/
char    msg_buf[512];

/* flags to control preview functions and track Flyer status */
int     flyer_avail = FALSE;
int     preview_enabled = FALSE;





/*- function prototypes -*/

void select_graph_mode( int mode_id );
void draw_clipstart( struct RastPort *rport, int a_b);
int update_intsgad( struct Gadget *gad, long int value, int fieldwidth);
int read_intsgad( struct Gadget *gad, long int *value);
int setup_button_slider(WORD x, WORD y, int sensitivity,  double scale_increment,
                        struct Gadget *gad);
int clear_button_slider(void);
int process_button_slider( WORD x);
void set_mouse( WORD x, WORD y);
void return_mouse( void);
void adjust_aclip_start( int delta_cframes );
void adjust_bclip_start( int delta_cframes );
void adjust_trans_len( int delta_cframes );
int display_graph_index( struct graph_struct *graph);
int adjust_graph_index( struct graph_struct *graph, int movement);
int read_control_file( char *filename, struct graph_struct **graph);
int update_dblsgad(struct Gadget *gad,double value,char *fmt,int fieldwidth);
int read_dblsgad( struct Gadget *gad, double *value);
int  set_trans_len( unsigned long int new_len );
int write_control_file( char *filename, struct graph_struct *graph);
void draw_cliptxt(void);
int set_linear( int linear );
int set_gadget( struct Gadget *gad, int flag);
void adjust_tension( struct graph_struct *graph, int delta_units );
void adjust_bias( struct graph_struct *graph, int delta_units );
void adjust_cont( struct graph_struct *graph, int delta_units );
int load_graph(struct graph_struct *graph);
int save_graph(struct graph_struct *graph);
void update_in_out(void);







/***************************************************************************/


int main(int argc, char *argv[])
{
    int        status;

    BOOL       quitflag;
    ULONG      signalmask, winsignal, signals;
    struct     IntuiMessage *message = NULL;
    ULONG      class;
    UWORD      code;
    UWORD      qualifier;
    WORD       mousex, mousey;
    struct     Gadget  *gad;

    unsigned long int       temp_frame, new_frame;
    double                  temp_percent, new_percent;
    double                  temp_tension;
    double                  temp_bias;
    double                  temp_cont;
    int                     temp_linear;
    int                     temp_index;

    int                     tbar_level;
    WORD                    drag_basex, drag_basey;



    int         movement;


    struct hitbox_struct   graph_hitbox = {GRAPH_LEFT, GRAPH_TOP,
                                            (GRAPH_LEFT + GRAPH_WIDTH),
                                            (GRAPH_TOP + GRAPH_HEIGHT) };




    /*!! parse the command line */

    /* open the debug log */
    status = init_errlog(ERROR_MSG, "s:rfxenv_log");
    if (status == ERROR)
    {
        log_message(ERROR_MSG, "Error opening log file\n");
    }


    /* open libraries and setup resources */

    status = init_rfx();

    if (status != OK)
    {
        sprintf(msg_buf, "Error from init_rfx status is %d\n", status);
        log_message(ERROR_MSG, msg_buf);
        printf("%s\n", msg_buf);
        shutdown_rfx();
        close_errlog();
        return(1);
    }

    select_graph_mode(_CREATE_ID);


    /* open the startup screen */

restore_switcher();

    status = create_startupwin();

    if (status != OK)
    {
        log_message(ERROR_MSG, "create_startupwin failed!");
        delete_startupwin();
        shutdown_rfx();
        close_errlog();
        return(1);
    }


    /*! parse the input command line */
    if( (status = read_control_file( "env:envelope.txt", &graph)) == ERROR)
    {
        sprintf(msg_buf, "Error reading input file\n");
        log_message(ERROR_MSG, msg_buf);
        status = post_easyrequest(rf_win, "RFX INIT ERROR", msg_buf, "OK");
        delete_startupwin();
        shutdown_rfx();
        close_errlog();
        return(1);
    }

    /* attempt to open the Flyer Library */
    if (init_flyer() != OK)
    {
        status = post_easyrequest(rf_win, "FLYER ERROR",
"\nUnable to Open\nFlyer.library\n\nPreview and Render Functions\ndisabled.\n",
                                        "OK");
    }
    else
    {
        flyer_avail = TRUE;
        preview_enabled = TRUE;
    }

    /* attempt to initialize the AREXX */
    if (flyer_arexx_init() != OK)
    {
        status = post_easyrequest(rf_win, "FLYER ERROR",
"\nUnable to Open\nFlyer AREXX Port\n\nPreview and Render Functions\ndisabled.\n",
                                        "OK");
        flyer_avail = FALSE;
        preview_enabled = FALSE;
    }


    /* delete the startup window */

    Delay(50);

    delete_startupwin();


    /* draw the clip names */
    draw_cliptxt();

    /* draw the start values */

    draw_clipstart(rf_win->RPort, A_CLIP);
    draw_clipstart(rf_win->RPort, B_CLIP);

    redraw_graph(rf_win->RPort ,graph);

    SetAPen(rf_win->RPort, 0);
    SetBPen(rf_win->RPort, 1);
    SetDrMd(rf_win->RPort, JAM2);

    Move(rf_win->RPort, (GRAPH_LEFT+GRAPH_WIDTH+5), (GRAPH_TOP+5));
    Text(rf_win->RPort, "100 %", 5);

    Move(rf_win->RPort, (GRAPH_LEFT+GRAPH_WIDTH+5), (GRAPH_TOP+GRAPH_HEIGHT));
    Text(rf_win->RPort, "  0 %", 5);



    display_graph_index(graph);


    /* setup is OK, enter event loop*/

    update_intsgad( &_len_str_gad, trans_len, LEN_STR_WIDTH);
    update_intsgad( &_frame_str_gad, curr_index_frame, FRAME_STR_WIDTH);
    update_dblsgad( &_percent_str_gad, curr_index_percent, "%5.2lf",
                    PERCENT_STR_WIDTH);


//    /* post the beta flag */
//
//     post_easyrequest(rf_win, "RENDERFX",
//                            "RenderFX Envelope Editor, Beta 29 July",  "OK");


    /* Set up the signalswe want to hear about ... */
    winsignal = 1L << rf_win->UserPort->mp_SigBit;   /* window IDCMP */
    signalmask = winsignal;


    quitflag = FALSE;


    while( !quitflag )
    {
        signals = Wait(signalmask);

        /* An Event occured - now act on the signal(s) we received */
        /* We were only waiting for one signal (winsignal) in out */
        /* signalmask, so we actually knw we recieved winsignal. */

        if( signals & winsignal)
        {
            while( message = (struct IntuiMessage *)GetMsg(rf_win->UserPort))
            {
                class = message->Class;  /* get all the data needed from  msg*/
                code = message->Code;
                qualifier = message->Qualifier;
                mousex = message->MouseX;
                mousey = message->MouseY;


                /* gadget address is ONLY valid for gadget messages */
               if( (class==IDCMP_GADGETUP) || (class == IDCMP_GADGETDOWN) )
                {
                    gad = (struct Gadget *)(message->IAddress);
                }

                /* when we're through with the message, reply */
                ReplyMsg( (struct Message *)message);


                /* see what events occured */

                switch(class)
                {

                case IDCMP_GADGETUP:
                    /* caused by GACT_RELVERIFY */
//                    printf("recieved an IDCMP_GADGETUP, gadget number %d\n",
//                            gad->GadgetID);


                    if (gad->GadgetID == _CANCEL_ID)
                    {
                        quitflag = TRUE;
                    }
                    else if (gad->GadgetID == _RENDER_ID)
                    {
                        if (flyer_avail == TRUE)
                        {
                            status = write_control_file( "env:rfx_out.txt",graph);
                            if (status == ERROR)
                            {
                                status = post_easyrequest(rf_win,
                                            "RenderFX ERROR",
                                            "Error Writing output file\n",
                                            "OK");

                                log_message(ERROR_MSG,"Error writing output file\n");
                            }
                            else
                            {
                                status = system("prowave:renderfx env:rfx_out.txt");
                                if (status != 0)
                                {
                                        status = post_easyrequest(rf_win,
                                                 "RenderFX ERROR",
                                                 "Error executing RenderFX\n",
                                                 "OK");

                                        log_message(ERROR_MSG,
                                            "Error executing RenderFX\n");
                                }
                                else
                                {
                                    /* everything OK quit */
                                    quitflag = TRUE;
                                }
                            }
                        }
                        else
                        {
                            status = post_easyrequest(rf_win,
                                     "FLYER NOT AVAILABLE",
                                        "VideoFlyer Not Available, Abort",
                                        "OK");
                        }

                    }
                    else if (gad->GadgetID == _PREVIEW_ID)
                    {
                        if (flyer_avail == TRUE)
                        {
                            status = write_control_file( "env:rfx_out.txt",graph);
                            if (status == ERROR)
                            {
                                status = post_easyrequest(rf_win,
                                            "RenderFX ERROR",
                                            "Error Writing output file\n",
                                            "OK");

                                log_message(ERROR_MSG,"Error writing output file\n");
                            }
                            else
                            {
                                status = system("prowave:previewfx env:rfx_out.txt");
                                if (status != 0)
                                {
                                        status = post_easyrequest(rf_win,
                                                 "PreviewFX ERROR",
                                                 "Error executing PreviewFX\n",
                                                 "That Bites!");

                                        log_message(ERROR_MSG,
                                            "Error executing PreviewFX\n");
                                 }
                                restore_rfx_screen();
                                Delay(10);
//                                ScreenToFront(screen);
WindowToFront(rf_win); 
                           }
                        }
                        else
                        {
                            status = post_easyrequest(rf_win,
                                     "FLYER NOT AVAILABLE",
                                        "VideoFlyer Not Available, Abort",
                                        "OK");
                        }

                    }
                    else if (gad->GadgetID == _REALTIME_ID)
                    {


//!!! Patch code here
                        if (flyer_avail == TRUE)
                        {
                            status = write_control_file( "env:rfx_out.txt",graph);
                            if (status == ERROR)
                            {
                                status = post_easyrequest(rf_win,
                                            "RenderFX ERROR",
                                            "Error Writing output file\n",
                                            "OK");

                                log_message(ERROR_MSG,"Error writing output file\n");
                            }
                            else
                            {
                        get_graph_index_values( graph, &temp_frame,
                                    &temp_percent, &temp_tension,
                                    &temp_bias, &temp_cont, &temp_linear);
                                tbar_level = (int)(temp_percent * 5.11);  /* 511 / 100.0 */
                                sprintf(msg_buf,"prowave:previewfx env:rfx_out.txt %d",tbar_level);
                               
                                printf("%s \n",msg_buf);

                                status = system(msg_buf);
                                if (status != 0)
                                {
                                        status = post_easyrequest(rf_win,
                                                 "PreviewFX ERROR",
                                                 "Error executing PreviewFX\n",
                                                 "That Bites!");

                                        log_message(ERROR_MSG,
                                            "Error executing PreviewFX\n");
                                 }
                                restore_rfx_screen();
                                Delay(10);
                                ScreenToFront(screen);
                            }
                        }
                        else
                        {
                            status = post_easyrequest(rf_win,
                                     "FLYER NOT AVAILABLE",
                                        "VideoFlyer Not Available, Abort",
                                        "OK");
                        }

                    }
                    else if (gad->GadgetID == _LOAD_ID)
                    {
                        status = load_graph(graph);
                        set_graph_current_index(graph,0);
                        set_trans_len( (graph->end_frame + 1));


                        redraw_graph(rf_win->RPort ,graph);
                        display_graph_index(graph);
                    }
                    else if (gad->GadgetID == _SAVE_ID)
                    {
                        status = save_graph(graph);
                    }
                    else if (gad->GadgetID == _VIEW_ID)
                    {
                        viewflag = FALSE;
                        restore_rfx_screen();
                        Delay(10);
//                        ScreenToFront(screen);
WindowToFront(rf_win);

                        get_graph_current_index( graph, &temp_index);
                        get_graph_index_values( graph, &temp_frame,
                                    &temp_percent, &temp_tension,
                                    &temp_bias, &temp_cont, &temp_linear);
                        if( (temp_percent != view_percent) &&
                            (view_percent >= 0) )
                        {
                            temp_percent = view_percent;

                            move_graph_point( rf_win->RPort, graph, temp_index,
                                                temp_frame, temp_percent) ;

                            /* see what was actually done and refresh the gadget*/
                            display_graph_index( graph );
                        }


                    }


    /* this block handles the shutting down of the "button sliders" */
                    else if ( ( (gad->GadgetID == _A_LEFT_ID) ||
                                (gad->GadgetID == _A_RIGHT_ID) ) &&
                              (acliptype == VIDEO_TYPE) )
                    {
                        clear_button_slider();
                        if (preview_enabled == TRUE)
                        {
                            show_field(0, afilename, (2 * aclipstart));
                        }

                    }
                    else if ( ( (gad->GadgetID == _B_LEFT_ID) ||
                                (gad->GadgetID == _B_RIGHT_ID) ) &&
                              (bcliptype == VIDEO_TYPE) )
                    {
                        clear_button_slider();
                        if (preview_enabled == TRUE)
                        {
                            show_field(0, bfilename, (2 * bclipstart));
                        }


                    }
                    else if ( (gad->GadgetID == _LEN_LEFT_ID) ||
                                (gad->GadgetID == _LEN_RIGHT_ID) )
                    {
                        clear_button_slider();
                        status = change_graph_length ( graph, trans_len);
                        if (status == OK)
                        {
                            redraw_graph(rf_win->RPort ,graph);
                            display_graph_index(graph);
                        }

                    }
                    else if ( (gad->GadgetID == _TENSION_LEFT_ID) ||
                                (gad->GadgetID == _TENSION_RIGHT_ID) )
                    {
                        clear_button_slider();

                    }
                    else if ( (gad->GadgetID == _BIAS_LEFT_ID) ||
                                (gad->GadgetID == _BIAS_RIGHT_ID) )
                    {
                        clear_button_slider();

                    }
                    else if ( (gad->GadgetID == _CONT_LEFT_ID) ||
                                (gad->GadgetID == _CONT_RIGHT_ID) )
                    {
                        clear_button_slider();

                    }


    /* the keyframe select gadget is NOT a "button slider" */
                    else if ( (gad->GadgetID == _FRAME_LEFT_ID) )
                    {

                        status = adjust_graph_index(graph, -1);
                        if (status == OK)
                        {
                            display_graph_index(graph);
                        }


                    }
                    else if ( (gad->GadgetID == _FRAME_RIGHT_ID) )
                    {

                        status = adjust_graph_index(graph, 1);
                        if (status == OK)
                        {
                            display_graph_index(graph);
                        }
                    }


                    /* handle the string gadgets */

                    else if ( (gad->GadgetID == _FRAME_STRING_ID) )
                    {
                        /* attempt to set the current indexed point */

                        /* read the string */
                        read_intsgad( &_frame_str_gad, (long int *)&new_frame);

                        get_graph_current_index( graph, &temp_index);

                        get_graph_index_values( graph, &temp_frame,
                                        &temp_percent, &temp_tension,
                                        &temp_bias, &temp_cont, &temp_linear);


                        /* try to set the point */
                        move_graph_point( rf_win->RPort, graph, temp_index,
                                            new_frame, temp_percent) ;


                        /* see what was actually done and refresh the gadget*/
                        display_graph_index( graph );

                    }

                    else if ( (gad->GadgetID == _PERCENT_STRING_ID) )
                    {
                        /* attempt to set the current indexed point */

                        /* read the string */
                        read_dblsgad( &_percent_str_gad, &new_percent);

                        get_graph_current_index( graph, &temp_index);


                        get_graph_index_values( graph, &temp_frame,
                                        &temp_percent, &temp_tension,
                                        &temp_bias, &temp_cont, &temp_linear);

                        /* try to set the point */
                        move_graph_point( rf_win->RPort, graph, temp_index,
                                            temp_frame, new_percent) ;


                        /* see what was actually done and refresh the gadget*/
                        display_graph_index( graph );

                    }


                    else if ( (gad->GadgetID == _LEN_STRING_ID) )
                    {

                        read_intsgad( &_len_str_gad, (long int *)&temp_ulong);

                        if( (temp_ulong % 2) != 0 )
                        {
                            temp_ulong--;
                        }

                        status = set_trans_len(temp_ulong);


                        status = change_graph_length ( graph, trans_len);
                        if (status == OK)
                        {
                            redraw_graph(rf_win->RPort ,graph);
                            display_graph_index(graph);
                        }

                    }

                    else if ( (gad->GadgetID == _TENSION_STRING_ID) )
                    {

                        read_dblsgad( &_tension_str_gad, &new_tension);


                        if (new_tension > 1.0)
                        {
                            new_tension = 1.0;
                        }
                        else if (new_tension < -1.0)
                        {
                            new_tension = -1.0;
                        }



                        get_graph_index_values( graph, &temp_frame,
                                        &temp_percent, &temp_tension,
                                        &temp_bias, &temp_cont, &temp_linear);


                        change_index_spline( rf_win->RPort, graph,
                                             new_tension, temp_bias,
                                          temp_cont, temp_linear);

                        update_dblsgad( &_tension_str_gad, new_tension,
                                         "%2.2lf", TENSION_STR_WIDTH);

                    }
                    else if ( (gad->GadgetID == _BIAS_STRING_ID) )
                    {

                        read_dblsgad( &_bias_str_gad, &new_bias);

                        if (new_bias > 1.0)
                        {
                            new_bias = 1.0;
                        }
                        else if (new_bias < -1.0)
                        {
                            new_bias = -1.0;
                        }

                        get_graph_index_values( graph, &temp_frame,
                                        &temp_percent, &temp_tension,
                                        &temp_bias, &temp_cont, &temp_linear);

                        change_index_spline( rf_win->RPort, graph,
                                             temp_tension, new_bias,
                                          temp_cont, temp_linear);


                        update_dblsgad( &_bias_str_gad, new_bias,
                                         "%2.2lf", BIAS_STR_WIDTH);

                    }
                    else if ( (gad->GadgetID == _CONT_STRING_ID) )
                    {

                        read_dblsgad( &_cont_str_gad, &new_cont);

                        if (new_cont > 1.0)
                        {
                            new_cont = 1.0;
                        }
                        else if (new_cont < -1.0)
                        {
                            new_cont = -1.0;
                        }


                        get_graph_index_values( graph, &temp_frame,
                                        &temp_percent, &temp_tension,
                                        &temp_bias, &temp_cont, &temp_linear);

                        change_index_spline( rf_win->RPort, graph,
                                             temp_tension, temp_bias,
                                          new_cont, temp_linear);


                        update_dblsgad( &_cont_str_gad, new_cont,
                                         "%2.2lf", CONT_STR_WIDTH);

                    }



                    break;

                case IDCMP_GADGETDOWN:
                    /* caused by GACT_IMMEDIATE */
//                    printf("receive an IDCMP_GADGETDOW, gadget number %d\n",
//                            gad->GadgetID);


    /* test the graph mode radio button group */

                    if ( (gad->GadgetID == _CREATE_ID) )
                    {

                        select_graph_mode(_CREATE_ID);
                    }
                    else if ( (gad->GadgetID == _DELETE_ID) )
                    {

                        select_graph_mode(_DELETE_ID);
                    }
                    else if ( (gad->GadgetID == _DRAG_ID) )
                    {

                        select_graph_mode(_DRAG_ID);
                    }

                    else if ( (gad->GadgetID == _VIEW_ID) )
                    {

                        if (preview_enabled == TRUE)
                        {
                            get_graph_index_values( graph, &temp_frame,
                                        &temp_percent, &temp_tension,
                                        &temp_bias, &temp_cont, &temp_linear);

                            viewflag = TRUE;
                            if (bcliptype == KEYPAGE_TYPE)
                            {
                                show_transition(afilename,
                                   (2 * (aclipstart + temp_frame)),acliptype,
                                   bfilename,  bclipstart,bcliptype,
                                    temp_percent);
                            }
                            else
                            {

                                show_transition(afilename,
                                   (2 * (aclipstart + temp_frame)),acliptype,
                                   bfilename,
                                   (2 * (bclipstart + temp_frame)),bcliptype,
                                    temp_percent);
                            }

                        }
                    }




    /* check the linear button */
                    else if ( (gad->GadgetID == _LINEAR_ID) )
                    {
                        if (curr_index_linear == TRUE)
                        {
                            curr_index_linear = FALSE;
                        }
                        else if (curr_index_linear == FALSE)
                        {
                            curr_index_linear = TRUE;
                        }

                        set_graph_index_spline( graph, curr_index_tension,
                            curr_index_bias, curr_index_cont, curr_index_linear);

                        set_linear(curr_index_linear);

                        /* redraw the graph */
                        redraw_graph( rf_win->RPort, graph);

                    }


    /* check for the "button slider" type gadets */

                    else if ( (gad->GadgetID == _A_LEFT_ID) &&
                              (acliptype == VIDEO_TYPE) )
                    {

                        setup_button_slider(mousex, mousey, 2, 1.04,
                                            gad);
                    }

                    else if ( (gad->GadgetID == _A_RIGHT_ID) &&
                              (acliptype == VIDEO_TYPE) )
                    {
                        setup_button_slider(mousex, mousey, 2, 1.04,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _B_LEFT_ID) &&
                              (bcliptype == VIDEO_TYPE) )
                    {

                        setup_button_slider(mousex, mousey, 2, 1.04,
                                            gad);
                    }

                    else if ( (gad->GadgetID == _B_RIGHT_ID) &&
                              (bcliptype == VIDEO_TYPE) )
                    {
                        setup_button_slider(mousex, mousey, 2, 1.04,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _LEN_LEFT_ID) )
                    {
                        setup_button_slider(mousex, mousey, 2, 1.04,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _LEN_RIGHT_ID) )
                    {
                        setup_button_slider(mousex, mousey, 2, 1.04,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _TENSION_RIGHT_ID) )
                    {
                        setup_button_slider(mousex, mousey, 1, 1.0,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _TENSION_LEFT_ID) )
                    {
                        setup_button_slider(mousex, mousey, 1, 1.0,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _BIAS_RIGHT_ID) )
                    {
                        setup_button_slider(mousex, mousey, 1, 1.0,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _BIAS_LEFT_ID) )
                    {
                        setup_button_slider(mousex, mousey, 1, 1.0,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _CONT_RIGHT_ID) )
                    {
                        setup_button_slider(mousex, mousey, 1, 1.0,
                                            gad);
                    }
                    else if ( (gad->GadgetID == _CONT_LEFT_ID) )
                    {
                        setup_button_slider(mousex, mousey, 1, 1.0,
                                            gad);
                    }



                    break;


                case IDCMP_RAWKEY:

//                    printf("rawkey\n");

                    break;

                case IDCMP_MOUSEBUTTONS:
//                    printf("MouseButton\n");

                    /* note, these messages are only enabled when there is */
                    /* something loaded, so there WILL be data to operate on */

                    /* SELECTDOWN States */
                    if (code == SELECTDOWN)
                    {

                        if (in_hitbox(mousex, mousey, graph_hitbox))
                        {

                            /* find the position in the graph */
                            map_screen_graph( graph, mousex, mousey,
                                                &temp_frame,
                                                &temp_percent);



                            if (graph_mode == _CREATE_ID)
                            {
                                status = insert_graph_point(graph, temp_frame,
                                                    temp_percent, 0.0, 0.0,
                                                    0.0, TRUE );

                                if (status == OK)
                                {
                                    redraw_graph(rf_win->RPort ,graph);
                                    display_graph_index(graph);
                                }
                            }
                            else if (graph_mode == _DELETE_ID)
                            {

                                find_nearest_graph_index( graph, temp_frame,
                                                            &temp_index);

                                status = delete_graph_point(graph, temp_index);

                                if (status == OK)
                                {
                                    redraw_graph(rf_win->RPort, graph);
                                    display_graph_index(graph);
                                }
                            }
                            else if (graph_mode == _DRAG_ID)
                            {

                                find_nearest_graph_index( graph, temp_frame,
                                                            &temp_index);

                                status = set_graph_current_index(graph,
                                                                 temp_index);

                                if (status == OK)
                                {
                                    redraw_graph(rf_win->RPort, graph);
                                    display_graph_index(graph);
                                }

                                status = map_graph_screen( graph,
                                          curr_index_frame, curr_index_percent,
                                          &drag_basex, &drag_basey);

                                if (status == OK)
                                {
                                    /* warp mouse pointer to base posiiton and */
                                    /* prepare to drag */

                                    set_mouse(drag_basex, drag_basey);
                                    drag_flag = TRUE;
                                }


                            }



                        }



                    }

                    else if (code == SELECTUP)
                    {

                        /* if mouseup during a button_slider movement, stop it */
                        if (bs_active == TRUE)
                        {
                            clear_button_slider();

                            /* note that there may be other close-up funtions */
                            /* required here later */

                        }
                        else if (drag_flag == TRUE)
                        {
                            drag_flag = FALSE;
                        }
                        else if (viewflag == TRUE)
                        {
                            viewflag = FALSE;
                            restore_rfx_screen();
                            Delay(10);
//                            ScreenToFront(screen);
WindowToFront(rf_win);

                            get_graph_current_index( graph, &temp_index);
                            get_graph_index_values( graph, &temp_frame,
                                        &temp_percent, &temp_tension,
                                        &temp_bias, &temp_cont, &temp_linear);
                            if( (temp_percent != view_percent) &&
                                (view_percent >= 0) )
                            {
                                temp_percent = view_percent;

                                move_graph_point( rf_win->RPort, graph, temp_index,
                                                    temp_frame, temp_percent) ;

                                /* see what was actually done and refresh the gadget*/
                                display_graph_index( graph );
                            }
                        }
                    }



                    break;

                case IDCMP_MOUSEMOVE:

//                    printf("MouseMove\n");

                    if ( bs_active == TRUE )
                    {
                        movement = process_button_slider(mousex);
//                        printf("m = %d\n", movement);

                        if ( ( (gad->GadgetID == _A_LEFT_ID) ||
                                (gad->GadgetID == _A_RIGHT_ID) ) &&
                                  (acliptype != STILL_TYPE) )
                        {
                            adjust_aclip_start(movement);

                        }
                        else if ( ( (gad->GadgetID == _B_LEFT_ID) ||
                                (gad->GadgetID == _B_RIGHT_ID) ) &&
                                  (bcliptype != STILL_TYPE) )
                        {
                            adjust_bclip_start(movement);

                        }
                        else if ( ( (gad->GadgetID == _LEN_LEFT_ID) ||
                                (gad->GadgetID == _LEN_RIGHT_ID) ) )
                        {
                            adjust_trans_len(movement);

                        }
                        else if ( ( (gad->GadgetID == _TENSION_LEFT_ID) ||
                                (gad->GadgetID == _TENSION_RIGHT_ID) ) )
                        {
                            adjust_tension(graph, movement);

                        }
                        else if ( ( (gad->GadgetID == _BIAS_LEFT_ID) ||
                                (gad->GadgetID == _BIAS_RIGHT_ID) ) )
                        {
                            adjust_bias(graph, movement);

                        }
                        else if ( ( (gad->GadgetID == _CONT_LEFT_ID) ||
                                (gad->GadgetID == _CONT_RIGHT_ID) ) )
                        {
                            adjust_cont(graph, movement);

                        }

                    }
                    else if (drag_flag == TRUE)
                    {

                        /* find the new position in graph coordinates */

                        map_screen_graph( graph, mousex, mousey, &temp_frame,
                                          &temp_percent);



                        /* attempt to move the point */

                        move_graph_point( rf_win->RPort, graph, temp_index,
                                            temp_frame, temp_percent);


                        /* update what was actually done */

                        display_graph_index(graph);
                    }





                    break;

                default:
//                    printf("recieved unknown IDCMP code %d\n", class);
                    break;
                }
            }
        }

    }

    /* Clean Up and exit */


    shutdown_rfx();

    if (flyer_avail == TRUE)
    {
        restore_switcher();
    }
    flyer_arexx_shutdown();
    close_flyer();



    /* close the log file */
    close_errlog();


    return(0);

}



/*-- Toggle Graph Modes --*/

void select_graph_mode( int mode_id )
{
    struct Gadget   *old_gad, *new_gad;
    UWORD           position;

    /* check for valid mode, exit if not OK */
    if ( (mode_id != _CREATE_ID) && (mode_id != _DELETE_ID) &&
         (mode_id != _DRAG_ID) )
    {
        return;
    }

    /* set the global mode */

    if ( graph_mode != mode_id )
    {

        /* remove the old mode gadget and deactivate it */

        if (graph_mode != 0)
        {
            if (graph_mode == _CREATE_ID)
            {
                old_gad = &_create_gad;
            }
            else if (graph_mode == _DELETE_ID)
            {
                old_gad = &_delete_gad;
            }
            else if (graph_mode == _DRAG_ID)
            {
                old_gad = &_drag_gad;
            }

            if ( (position = RemoveGadget(rf_win, old_gad)) == -1)
            {
                return;
            }

            old_gad->Flags = (old_gad->Flags & (~GFLG_SELECTED));

            AddGadget(rf_win, old_gad, position);
        }


        /* remove the new mode gadget and activate it */


        if (mode_id == _CREATE_ID)
        {
            new_gad = &_create_gad;
        }
        else if (mode_id == _DELETE_ID)
        {
            new_gad = &_delete_gad;
        }
        else if (mode_id == _DRAG_ID)
        {
            new_gad = &_drag_gad;
        }

        if ( (position = RemoveGadget(rf_win, new_gad)) == -1)
        {
            return;
        }

        new_gad->Flags = (new_gad->Flags | GFLG_SELECTED);

        AddGadget(rf_win, new_gad, position);

        RefreshGadgets(rf_win->FirstGadget, rf_win, NULL);


        graph_mode = mode_id;
    }


    return;
}


/* function to draw a/b clip start points on the screen */

void draw_clipstart( struct RastPort *rport, int a_b)
{
    struct timecode     starttcode, in_out_tcode;
    UWORD               x,y, x1, y1;

    if (a_b == A_CLIP)
    {
        x = A_TCODEX;
        y = A_TCODEY;
        x1 = A_OUTX;
        y1 = A_OUTY;
        frame2tcode( aclipstart, &starttcode);
        frame2tcode( aclipout, &in_out_tcode);
    }
    else if (a_b == B_CLIP)
    {
        x = B_TCODEX;
        y = B_TCODEY;
        x1 = B_INX;
        y1 = B_INY;
        frame2tcode( bclipstart, &starttcode);
        frame2tcode( bclipin, &in_out_tcode);
    }
    else
    {
        return;
    }

    SetAPen(rport, 0);
    SetBPen(rport, 1);
    SetDrMd(rport, JAM2);


    Move(rport, x, y);
    draw_timecode(rport, starttcode, FALSE);

    Move(rport, x1, y1);
    draw_timecode(rport, in_out_tcode, FALSE);


    return;
}



/* function to update the value in an integer stringgadget */

int update_intsgad( struct Gadget *gad, long int value, int fieldwidth)
{

    char        string[30];
    int         position;

    /* remove the gadget */
    if ( (position = RemoveGadget(rf_win, gad)) == -1)
    {
        log_message(ERROR_MSG,"unable to remove gadget in update_intsgad\n");
        return(ERROR);
    }


    /* set the LongInt value */

    ((struct StringInfo *)(gad->SpecialInfo))->LongInt = value;

    sprintf(string, "%ld", value);

    strcpy( ((struct StringInfo *)(gad->SpecialInfo))->Buffer, string);

//    ((struct StringInfo *)(gad->SpecialInfo))->BufferPos = 0;
//    ((struct StringInfo *)(gad->SpecialInfo))->DispPos   = 0;


    AddGadget(rf_win, gad, position);

    RefreshGList(gad, rf_win, NULL, 1);

//    ActivateGadget(gad, rf_win, NULL);


    return(OK);
}



int read_intsgad( struct Gadget *gad, long int *value)
{

    int         position;

    /* remove the gadget */
    if ( (position = RemoveGadget(rf_win, gad)) == -1)
    {
        log_message(ERROR_MSG,"unable to remove gadget in read_intsgad\n");
        return(ERROR);
    }


    /* read the LongInt value */

    *value = ((struct StringInfo *)(gad->SpecialInfo))->LongInt;


    AddGadget(rf_win, gad, position);

    RefreshGList(gad, rf_win, NULL, 1);


    return(OK);

}



/* function to update the value in a double precision stringgadget */

int update_dblsgad(struct Gadget *gad,double value,char *fmt,int fieldwidth)
{

    char        string[30];
    int         position;

    /* remove the gadget */
    if ( (position = RemoveGadget(rf_win, gad)) == -1)
    {
        log_message(ERROR_MSG,"unable to remove gadget in update_dblsgad\n");
        return(ERROR);
    }



    /* format the value and save in the gadget */
    sprintf(string, fmt, value);
    strcpy( ((struct StringInfo *)(gad->SpecialInfo))->Buffer, string);


    AddGadget(rf_win, gad, position);

    RefreshGList(gad, rf_win, NULL, 1);

    return(OK);
}



int read_dblsgad( struct Gadget *gad, double *value)
{

    int         position;

    /* remove the gadget */
    if ( (position = RemoveGadget(rf_win, gad)) == -1)
    {
        log_message(ERROR_MSG,"unable to remove gadget in read_dblsgad\n");
        return(ERROR);
    }


    /* read the string into the value */

    sscanf( (((struct StringInfo *)(gad->SpecialInfo))->Buffer), "%lf", value);


    AddGadget(rf_win, gad, position);

    RefreshGList(gad, rf_win, NULL, 1);


    return(OK);

}






/* prepare "button slider" to track mousemovements */

int setup_button_slider(WORD x, WORD y, int sensitivity, double scale_increment,
                        struct Gadget *gad)
{

    /* this function sets up several global variables used for */
    /* tracking mouse movements on the button slider */

    if (bs_active == FALSE )
    {
        bs_sensitivity  = sensitivity;
        bs_active       = TRUE;
        bs_gadget_id    = gad->GadgetID;


        /* get the screen position of the center of the gadget*/

        /* center of gadget, in window coords */
        bs_basex        = (gad->LeftEdge + (gad->Width / 2));
        bs_basey        = (gad->TopEdge + (gad->Height / 2));

        /* center of gadget, in screen coords */
        bs_screen_basex =(rf_win->LeftEdge+bs_basex);
        bs_screen_basey =(rf_win->TopEdge+bs_basey);
        set_mouse( bs_screen_basex, bs_screen_basey);

        /* reset the scalefactor */
        bs_scalefactor = 1.0;

        bs_scaleincrement = scale_increment;



        /* blank the pointer */
        SetPointer( rf_win, inv_ptr, INV_PTR_HEIGHT, INV_PTR_WIDTH,
                    INV_PTR_XOFFSET, INV_PTR_YOFFSET );



        /* provide immediate feedback on the slider movement */

        if( gad->GadgetID == _A_LEFT_ID )
        {
            adjust_aclip_start(-1);  /* this is Colorframes */
        }
        else if ( gad->GadgetID == _A_RIGHT_ID )
        {
            adjust_aclip_start(1);
        }
        else if( gad->GadgetID == _B_LEFT_ID )
        {
            adjust_bclip_start(-1);  /* this is Colorframes */
        }
        else if ( gad->GadgetID == _B_RIGHT_ID )
        {
            adjust_bclip_start(1);
        }
        else if ( gad->GadgetID == _LEN_LEFT_ID )
        {
            adjust_trans_len(-1);
        }
        else if ( gad->GadgetID == _LEN_RIGHT_ID )
        {
            adjust_trans_len(1);
        }
        else if ( gad->GadgetID == _TENSION_LEFT_ID )
        {
            adjust_tension(graph, -1);
        }
        else if ( gad->GadgetID == _TENSION_RIGHT_ID )
        {
            adjust_tension(graph, 1);
        }
        else if ( gad->GadgetID == _BIAS_LEFT_ID )
        {
            adjust_bias(graph, -1);
        }
        else if ( gad->GadgetID == _BIAS_RIGHT_ID )
        {
            adjust_bias(graph, 1);
        }
        else if ( gad->GadgetID == _CONT_LEFT_ID )
        {
            adjust_cont(graph, -1);
        }
        else if ( gad->GadgetID == _CONT_RIGHT_ID )
        {
            adjust_cont(graph, 1);
        }


        return (OK);
    }

    return (ERROR);
}


/* end a button_slider event sequence */

int clear_button_slider(void)
{

    if (bs_active == TRUE)
    {

        bs_basex        = 0;
        bs_basey        = 0;
        bs_screen_basex = 0;
        bs_screen_basey = 0;
        bs_sensitivity  = 0;
        bs_active       = FALSE;
        bs_gadget_id    = 0;

        /* blank the pointer */
        ClearPointer( rf_win);

        return (OK);
    }

    return(ERROR);
}



/* process a mousemove registered from an active button_slider */

int process_button_slider( WORD x)
{

    int     outval;

    /* check to see if the current mouse position is further than */
    /* the "sensitivity" distance in the x direction */

    /* the return value is scaled by the scaling factor and the scaling */
    /* factor is incremented by the scaling incremnt factor */

    /* returns 0 if it is less than the sensitivity */

    /* if the mouse has reversed direction, the scalefactor is reset to the */
    /* the original scale factor */

    /* see if greater */
    if ( (x - bs_basex) > bs_sensitivity )
    {
        /* positive movement, reset scalefactor in a reversal */
        if (bs_lastdirection <= 0)
        {
            bs_scalefactor = 1.0;
            bs_lastdirection = 1;
        }

        outval = (int)( bs_scalefactor * 1.0);
        bs_scalefactor *= bs_scaleincrement;

//        printf("%d %lf\n", outval, bs_scalefactor);

        return_mouse();
        return (outval);
    }

    if ( (bs_basex - x) > bs_sensitivity )
    {

        /* negative movement, reset scalefactor in a reversal */
        if (bs_lastdirection >= 0)
        {
            bs_scalefactor = 1.0;
            bs_lastdirection = -1;
        }

        outval = (int)( bs_scalefactor * -1.0);
        bs_scalefactor *= bs_scaleincrement;

//        printf("%d %lf\n", outval, bs_scalefactor);

        return_mouse();
        return (outval);
    }

    return (0);
}




/* insert event to move mouse back to a specific point */

void set_mouse( WORD x, WORD y)
{

    static struct InputEvent       FakeEvent;
    static struct IEPointerPixel   NewPixel;

    NewPixel.iepp_Screen = screen;
    NewPixel.iepp_Position.X = x;
    NewPixel.iepp_Position.Y = y;

    FakeEvent.ie_EventAddress = (APTR) &NewPixel;
    FakeEvent.ie_NextEvent = NULL;
    FakeEvent.ie_Class = IECLASS_NEWPOINTERPOS;
    FakeEvent.ie_SubClass = IESUBCLASS_PIXEL;
    FakeEvent.ie_Code = IECODE_NOBUTTON;
    FakeEvent.ie_Qualifier = NULL;

    InputIO->io_Data = (APTR) &FakeEvent;
    InputIO->io_Length = sizeof(struct InputEvent);
    InputIO->io_Command = IND_WRITEEVENT;

    DoIO((struct IORequest *)InputIO);


    return;
}



/* insert event to move mouse back to starting point */

void return_mouse( void)
{

    static struct InputEvent       FakeEvent;
    static struct IEPointerPixel   NewPixel;

    NewPixel.iepp_Screen = screen;
    NewPixel.iepp_Position.X = bs_screen_basex;
    NewPixel.iepp_Position.Y = bs_screen_basey;

    FakeEvent.ie_EventAddress = (APTR) &NewPixel;
    FakeEvent.ie_NextEvent = NULL;
    FakeEvent.ie_Class = IECLASS_NEWPOINTERPOS;
    FakeEvent.ie_SubClass = IESUBCLASS_PIXEL;
    FakeEvent.ie_Code = IECODE_NOBUTTON;
    FakeEvent.ie_Qualifier = NULL;

    InputIO->io_Data = (APTR) &FakeEvent;
    InputIO->io_Length = sizeof(struct InputEvent);
    InputIO->io_Command = IND_WRITEEVENT;

    DoIO((struct IORequest *)InputIO);


    return;
}


/* change the tension value */
void adjust_tension( struct graph_struct *graph, int delta_units )
{

    unsigned long int       temp_frame;
    double                  temp_percent;
    double                  temp_tension;
    double                  temp_bias;
    double                  temp_cont;
    int                     temp_linear;


    if (delta_units == 0)
    {
        return;
    }


    get_graph_index_values( graph, &temp_frame,  &temp_percent, &temp_tension,
                                        &temp_bias, &temp_cont, &temp_linear);

    /* increment the tension by the number of units (1 unit = 0.05) */

    temp_tension += (double)(delta_units) * 0.05;


    if (temp_tension > 1.0)
    {
        temp_tension = 1.0;
    }
    else if (temp_tension < -1.0)
    {
        temp_tension = -1.0;
    }


    change_index_spline( rf_win->RPort, graph, temp_tension, temp_bias,
                            temp_cont, temp_linear);

    update_dblsgad( &_tension_str_gad, temp_tension, "%2.2lf",
                        TENSION_STR_WIDTH);

    curr_index_tension = temp_tension;

}

/* change the bias value */
void adjust_bias( struct graph_struct *graph, int delta_units )
{

    unsigned long int       temp_frame;
    double                  temp_percent;
    double                  temp_tension;
    double                  temp_bias;
    double                  temp_cont;
    int                     temp_linear;


    if (delta_units == 0)
    {
        return;
    }


    get_graph_index_values( graph, &temp_frame,  &temp_percent, &temp_tension,
                                        &temp_bias, &temp_cont, &temp_linear);

    /* increment the tension by the number of units (1 unit = 0.05) */

    temp_bias += (double)(delta_units) * 0.05;


    if (temp_bias > 1.0)
    {
        temp_bias = 1.0;
    }
    else if (temp_bias < -1.0)
    {
        temp_bias = -1.0;
    }


    change_index_spline( rf_win->RPort, graph, temp_tension, temp_bias,
                            temp_cont, temp_linear);

    update_dblsgad( &_bias_str_gad, temp_bias, "%2.2lf",
                        BIAS_STR_WIDTH);

    curr_index_bias = temp_bias;

}

/* change the continuity  value */
void adjust_cont( struct graph_struct *graph, int delta_units )
{

    unsigned long int       temp_frame;
    double                  temp_percent;
    double                  temp_tension;
    double                  temp_bias;
    double                  temp_cont;
    int                     temp_linear;


    if (delta_units == 0)
    {
        return;
    }


    get_graph_index_values( graph, &temp_frame,  &temp_percent, &temp_tension,
                                        &temp_bias, &temp_cont, &temp_linear);

    /* increment the tension by the number of units (1 unit = 0.05) */

    temp_cont += (double)(delta_units) * 0.05;


    if (temp_cont > 1.0)
    {
        temp_cont = 1.0;
    }
    else if (temp_cont < -1.0)
    {
        temp_cont = -1.0;
    }


    change_index_spline( rf_win->RPort, graph, temp_tension, temp_bias,
                            temp_cont, temp_linear);

    update_dblsgad( &_cont_str_gad, temp_cont, "%2.2lf",
                        CONT_STR_WIDTH);

    curr_index_cont = temp_cont;

}




/*-- change the aclip start point and update display --*/
void adjust_aclip_start( int delta_cframes )
{

    unsigned long int   temp_frame;
    long int            movement;

    /* make sure new value is legal */
    /* we're working with unsigned integers, have to do some backflips */
    /* to check for underflow */

    movement = 2 * delta_cframes;
    temp_frame = aclipstart + movement;
//    printf("tf = %ld\n", temp_frame);

    if (movement < 0L)
    {
        if (temp_frame > aclipstart)
        {
            temp_frame = 0;
        }
    }


    if ( ((temp_frame + trans_len) >= acliplen) )
    {
        return;
    }

    /* is's cool, update the display */

    aclipstart = temp_frame;

    update_in_out();

    draw_clipstart(rf_win->RPort, A_CLIP);
    if (preview_enabled == TRUE)
    {
        show_field(0,afilename, (2 * aclipstart));
    }

    return;
}


/*-- change the bclip start point and update display --*/
void adjust_bclip_start( int delta_cframes )
{

    unsigned long int   temp_frame;
    long int            movement;

    /* make sure new value is legal */
    /* we're working with unsigned integers, have to do some backflips */
    /* to check for underflow */

    movement = 2 * delta_cframes;
    temp_frame = bclipstart + movement;
//    printf("tf = %ld\n", temp_frame);

    if (movement < 0L)
    {
        if (temp_frame > bclipstart)
        {
            temp_frame = 0;
        }
    }


    if ( ((temp_frame + trans_len) >= bcliplen) )
    {
        return;
    }


    /* is's cool, update the display */

    bclipstart = temp_frame;

    update_in_out();

    draw_clipstart(rf_win->RPort, B_CLIP);
    if (preview_enabled == TRUE)
    {
        show_field(0,bfilename, (2 * bclipstart));
    }
    return;
}



/*-- change the clip lenght and update display --*/

void adjust_trans_len( int delta_cframes )
{

    long int value;

    /* read the value */

    value = ((struct StringInfo *)(_len_str_gad.SpecialInfo))->LongInt;

    /* increment or decrement value */

    value += ( delta_cframes * 2 );


    /* make sure it is a legal value */

    if (value < 2)
    {
        return;
    }

    if( (acliptype == VIDEO_TYPE) && (value > (acliplen - aclipstart)) )
    {
        return;
    }

    if( (bcliptype == VIDEO_TYPE) && (value > (bcliplen - bclipstart)) )
    {
        return;
    }

    /* update the gadget */


    trans_len = value;
    update_intsgad( &_len_str_gad, value, LEN_STR_WIDTH);

    update_in_out();

    draw_clipstart(rf_win->RPort, A_CLIP);
    draw_clipstart(rf_win->RPort, B_CLIP);



    return;
}

/*-- set a new transition lenght and update display --*/

int  set_trans_len( unsigned long int new_len )
{
    int    errorflag = FALSE;
    int    status = OK;

    /* make sure it is a legal value */

    if (new_len < 2)
    {
        errorflag = TRUE;
    }
    else if( (acliptype == VIDEO_TYPE) && (new_len > (acliplen - aclipstart)) )
    {
        errorflag = TRUE;
    }
    else if( (bcliptype == VIDEO_TYPE) && (new_len > (bcliplen - bclipstart)) )
    {
        errorflag = TRUE;
    }

    /* update the gadget */

    if (errorflag == FALSE)
    {
        trans_len = new_len;
    }
    else
    {
        status = ERROR;
    }

    update_intsgad( &_len_str_gad, trans_len, LEN_STR_WIDTH);

    update_in_out();

    draw_clipstart(rf_win->RPort, A_CLIP);
    draw_clipstart(rf_win->RPort, B_CLIP);



    return (status);
}


/*-- display graph data for the indicated index in the string gadgets */

int display_graph_index( struct graph_struct *graph)
{

    unsigned long int temp_index_frame;
    double  temp_index_percent;
    double  temp_index_bias;
    double  temp_index_tension;
    double  temp_index_cont;
    int  temp_index_linear;

    int     status;

    /* get the values */

    status = get_graph_index_values( graph, &temp_index_frame,
                                        &temp_index_percent,
                                        &temp_index_tension,
                                        &temp_index_bias,
                                        &temp_index_cont,
                                        &temp_index_linear);

//printf("frame  %ld  %ld\n", curr_index_frame, temp_index_frame);
//printf("percent  %lf  %lf\n", curr_index_percent, temp_index_percent);
//printf("tens  %lf  %lf\n", curr_index_tension, temp_index_tension);
//printf("bias  %lf  %lf\n", curr_index_bias, temp_index_bias);
//printf("cont  %lf  %lf\n", curr_index_cont, temp_index_cont);



    /* if successful, update the gadgets */
    if (status == OK)
    {
        if ( temp_index_frame != curr_index_frame)
        {
            curr_index_frame = temp_index_frame;
            update_intsgad(&_frame_str_gad, curr_index_frame, FRAME_STR_WIDTH);
        }

        if ( temp_index_percent != curr_index_percent)
        {
            curr_index_percent = temp_index_percent;
            update_dblsgad( &_percent_str_gad, curr_index_percent, "%5.2lf",
                            PERCENT_STR_WIDTH);
        }

        if (temp_index_linear != curr_index_linear)
        {
//printf("set linear\n");
           curr_index_linear = temp_index_linear;
            set_linear( curr_index_linear );
        }

        if (temp_index_bias != curr_index_bias)
        {
//printf("set bias\n");
            curr_index_bias = temp_index_bias;
            update_dblsgad( &_bias_str_gad, curr_index_bias, "%2.2lf",
                            BIAS_STR_WIDTH);
        }

        if (temp_index_tension != curr_index_tension)
        {
//printf("set tension\n");
            curr_index_tension = temp_index_tension;
            update_dblsgad( &_tension_str_gad, curr_index_tension, "%2.2lf",
                            TENSION_STR_WIDTH);
        }

        if (temp_index_cont != curr_index_cont)
        {
//printf("set cont\n");
            curr_index_cont = temp_index_cont;
            update_dblsgad( &_cont_str_gad, curr_index_cont, "%2.2lf",
                            CONT_STR_WIDTH);
        }


        return(OK);
    }

    return(ERROR);

}

/* set the state of the linear button, and change the color of the */
/* tension, bias, and continuity gadgets to reflect the change */

int set_linear( int linear )
{

    int             position;

    if ( (position = RemoveGadget(rf_win, &_linear_gad)) == -1)
    {
        log_message(ERROR_MSG,"unable to find linear gadget in set_lienar\n");
        return (ERROR);
    }




    if (linear == TRUE )
    {
        set_gadget( &_bias_str_gad, FALSE);
        set_gadget( &_tension_str_gad, FALSE);
        set_gadget( &_cont_str_gad, FALSE);
        set_gadget( &_bias_left_gad, FALSE);
        set_gadget( &_bias_right_gad, FALSE);
        set_gadget( &_tension_left_gad, FALSE);
        set_gadget( &_tension_right_gad, FALSE);
        set_gadget( &_cont_left_gad, FALSE);
        set_gadget( &_cont_right_gad, FALSE);


        _linear_gad.Flags |= GFLG_SELECTED;

    }
    else if (linear == FALSE )
    {
        set_gadget( &_bias_str_gad, TRUE);
        set_gadget( &_tension_str_gad, TRUE);
        set_gadget( &_cont_str_gad, TRUE);
        set_gadget( &_bias_left_gad, TRUE);
        set_gadget( &_bias_right_gad, TRUE);
        set_gadget( &_tension_left_gad, TRUE);
        set_gadget( &_tension_right_gad, TRUE);
        set_gadget( &_cont_left_gad, TRUE);
        set_gadget( &_cont_right_gad, TRUE);

        _linear_gad.Flags &= ~GFLG_SELECTED;

    }


    AddGadget(rf_win, &_linear_gad, position);
    RefreshGList(&_linear_gad, rf_win, NULL, 1);

    return (OK);
}



/* activate or deactiviate the specified gadget */
/* note, this does pretty much the same thing as OnGadget and OffGadget */
/* but does not have the overhead of refreshing ALL gadgets */

int set_gadget( struct Gadget *gad, int flag)
{


//    struct StringInfo  *stringinfo;
//    struct StringExtend *stringextend;
    int         position;

    /* remove the gadget */
    if ( (position = RemoveGadget(rf_win, gad)) == -1)
    {
        log_message(ERROR_MSG,"unable to remove gadget in set_gadget\n");
        return(ERROR);
    }

    /* either activate or deactivate the gadget */
    if (flag == FALSE)
    {
        gad->Flags |= GFLG_DISABLED;
    }
    else if (flag == TRUE)
    {
        gad->Flags &= ~GFLG_DISABLED;
    }


    AddGadget(rf_win, gad, position);
    RefreshGList(gad, rf_win, NULL, 1);

    return(OK);

}




int adjust_graph_index( struct graph_struct *graph, int movement)
{

    int new_index;

    get_graph_current_index( graph, &new_index);

    new_index += movement;


    unhilight_current_index(rf_win->RPort, graph );

    if ( (set_graph_current_index(graph, new_index)) == ERROR )
    {
        hilight_current_index(rf_win->RPort, graph );
        return(ERROR);
    }


    hilight_current_index(rf_win->RPort, graph );



    return(OK);

}


/* read the input file and initialize variables */


int read_control_file( char *filename, struct graph_struct **graph)
{


    FILE        *input_file;
    char        buff5[256];
    char        *tokenpt;
    int         reverseflag;
    double      start_percent, end_percent;

    input_file = fopen(filename,"r");
    if (input_file == NULL)
    {
        sprintf(msg_buf,"%s :No such file found\n",filename);
        log_message(ERROR_MSG, msg_buf);
        return(ERROR);
    }



    /* 1st line, output file name, quality level, */
    /* matte (reversible) flag 0=no, 1=yes */
    /* reverse effect flag  0=no (0 to 100% effect) 1=yes (100 to 0%)*/

    fgets(buff5, 255, input_file);
    tokenpt = strtok(buff5,",");
    strncpy(outfilename, tokenpt, 49);
    tokenpt = strtok(NULL,",");
    outquality = atoi(tokenpt);
    tokenpt = strtok(NULL,",");
    matte = atoi(tokenpt);
    tokenpt = strtok(NULL,",");
    reverseflag = atoi(tokenpt);



    /* 2nd line, A name, A start point, total lenght, file type */
    /* (sizes are in fields */


    fgets(buff5,255,input_file);
    tokenpt = strtok(buff5,",");
    strncpy(afilename,tokenpt,49);
    tokenpt = strtok(NULL,",");
    aclipstart = ( atol(tokenpt) / 2);  /* convert to frames */
    tokenpt = strtok(NULL,",");
    acliplen = atol(tokenpt) / 2;
    tokenpt = strtok(NULL,",");
    acliptype = atoi(tokenpt);

    /* 3nd line, B name, B start point, total lenght, file type */
    /* (sizes are in fields) */

    fgets(buff5,255,input_file);
    tokenpt = strtok(buff5,",");
    strncpy(bfilename,tokenpt,49);
    tokenpt = strtok(NULL,",");
    bclipstart = ( atol(tokenpt) / 2);  /* convert to frames */
    tokenpt = strtok(NULL,",");
    bcliplen = atol(tokenpt) / 2;
    tokenpt = strtok(NULL,",");
    bcliptype = atoi(tokenpt);

    /* 4th line, transition length, in fields */


    fgets(buff5,255,input_file);
    tokenpt = strtok(buff5,",");
    trans_len = (atol(tokenpt) / 2);    /* convert to frames */


    /* set the aclipin an bclipout values */

    update_in_out();


    /* now that we know transition length, init the the graph struct */

    *graph = init_graph( GRAPH_LEFT, GRAPH_TOP, GRAPH_WIDTH, GRAPH_HEIGHT,
                        GRAPH_XTICHEIGHT, GRAPH_YTICWIDTH,
                        (trans_len - 1), 100.0, max_points, 100, NUM_T_STEPS);

    if ( *graph == NULL)
    {
        log_message(ERROR_MSG, "Error creating graph\n");
        return(ERROR);
    }

    /* the default ramp is always a linear fade */


    if (reverseflag == 1)
    {
        start_percent = 100.0;
        end_percent = 0.0;
    }
    else
    {
        start_percent = 0.0;
        end_percent = 100.0;
    }


    if ( (append_graph_point(*graph, 0, start_percent, 0.0, 0.0, 0.0, TRUE)) ==
        ERROR)
    {
        log_message(ERROR_MSG, "error appending point 1\n");
        return(ERROR);
    }

    if ( (append_graph_point(*graph, (trans_len - 1), end_percent, 0.0, 0.0,
        0.0, TRUE)) == ERROR)
    {
        log_message(ERROR_MSG, "error appending point 2\n");
        return(ERROR);
    }




    return(OK);
}


/* write the control file for renderfx or previewfx */

int write_control_file( char *filename, struct graph_struct *graph)
{

    int                 i, j, tbar_level;
    unsigned long int   last_field;
    FILE                *output_file;
    double              percent[NUM_T_STEPS];
    unsigned long int   fields[NUM_T_STEPS];
    int                 num_points;
    int                 status;
    int                 points_written;


    /* attempt to open output file */
    output_file = fopen(filename,"w");
    if (output_file == NULL)
    {
        sprintf(msg_buf, "%s :Unable to create file\n",filename);
        log_message(ERROR_MSG, msg_buf);
        return(ERROR);
    }

    /* Line 1, output file name, output quality and matte flag (comma delimeter) */
    fprintf(output_file,"%s,%d,%d\n", outfilename, outquality, matte);

    /* Line 2, A clip name, start pt, total length, and type code */
    /* (sizes stored as frames)*/
    fprintf(output_file,"%s,%ld,%ld,%d\n",afilename, acliptype, (aclipstart*2),
                        (trans_len * 2));

    /* Line 3, B clip name, start pt, total length, and type code */
    /* (sizes stored as frames)*/
    fprintf(output_file,"%s,%ld,%ld,%d\n",bfilename, bcliptype, (bclipstart*2),
                        (trans_len * 2));

    /* Line 4, fake comment */
    fprintf(output_file,"RenderFX file, format #1\n");

    /* Line 5, Transition Length */
    fprintf(output_file,"%ld\n",(trans_len * 2));

    /* adjust frame for the last point to be equal to transition lenght */
    /* (this is required by the renderfx algorithm) */

    graph->frame[(graph->num_points-1)]++;

    /* run a dummy writer to get a count of points to be written to the file */

    points_written = 0;


    for(i=0; i<graph->num_points; i++)
    {
        status = generate_segment_points( graph, i, NUM_T_STEPS, fields,
                                            percent, &num_points);

        if (status == OK)
        {

            for(j=0; j<num_points; j++)
            {


                if( (i == 0) && (j == 0) )
                {
                    /* special case 1st point of 1st segment is used to set */
                    /* last_field */

                    last_field = fields[j];
                    points_written++;

               }
                else
                {
                    /* note, generate_segment_points does not return a value */
                    /* of < 0 for a field, so we do not have to worry about */
                    /* "unsigned wraparound" */

                    if (fields[j] > last_field)
                    {
                        last_field = fields[j];
                        points_written++;
                    }
                }
            }
        }
    }





    /* Line 6, Number of Transition Points, file ID (1) */
    fprintf(output_file,"%d,1\n", points_written);

    /* output the field number and T-Bar level for each control point */



    /* actually write the point */

    for(i=0; i<graph->num_points; i++)
    {

        status = generate_segment_points( graph, i, NUM_T_STEPS, fields,
                                            percent, &num_points);

        if (status == OK)
        {

            for(j=0; j<num_points; j++)
            {

                tbar_level = (int) (percent[j] * (double)(511) / 100.0);

                if( (i == 0) && (j == 0) )
                {
                    /* special case 1st point of 1st segment is used to set */
                    /* last_field */

                    last_field = fields[j];

                    fprintf(output_file,"%ld, %d\n", fields[j], tbar_level);

                }
                else
                {
                    /* note, generate_segment_points does not return a value */
                    /* of < 0 for a field, so we do not have to worry about */
                    /* "unsigned wraparound" */

                    if (fields[j] > last_field)
                    {
                        fprintf(output_file,"%ld, %d\n", fields[j],tbar_level);
                        last_field = fields[j];
                    }
                }
            }
        }
    }



    fclose(output_file);

    return(OK);
}


/* update the input and output clip texts */
void draw_cliptxt(void)
{

    char                *charpt, *temp_clip;
    int                 i, stringsize;
    struct TextExtent   t_ext;
    int                 out_strlen, text_width_out;
    char                temp_buff[256];
    WORD                xoffset, yoffset;


    SetAPen(rf_win->RPort, BLACKPEN);
    SetBPen(rf_win->RPort, MDGREY1PEN);
    SetDrMd(rf_win->RPort, JAM2);

    /* find the clip name, it will either be in a subdirectory '/'  */
    /*  or the root directory ':' of a drive */

    stringsize = strlen(afilename);
//    charpt = afilename;
    charpt = afilename + stringsize;

    /* stop when find either 1st '/' or ':' */

    for (i=0; i<stringsize; i++)
    {
        if( (*charpt == '/') || (*charpt == ':') )
        {
            charpt++;
            break;
        }
        charpt--;
    }


    sprintf( temp_buff, "A Clip: %s", charpt);
    TextExtent( rf_win->RPort, (STRPTR)temp_buff, strlen(temp_buff), &t_ext);

    temp_clip = clip_text(rf_win->RPort, 310, CLIP_FORWARD, temp_buff,
                            &out_strlen, &text_width_out);


    center_text(rf_win->RPort, 310, t_ext.te_Height, temp_clip,
                &xoffset, &yoffset);

    Move(rf_win->RPort, xoffset, 20);

    Text(rf_win->RPort, temp_clip, out_strlen);


    /* Display the B clip name */

    /* find the clip name, it will either be in a subdirectory '/'  */
    /*  or the root directory ':' of a drive */

    stringsize = strlen(bfilename);
//    charpt = bfilename;
    charpt = bfilename + stringsize;

    /* stop when find either 1st '/' or ':' */

    for (i=0; i<stringsize; i++)
    {
        if( (*charpt == '/') || (*charpt == ':') )
        {
            charpt++;
            break;
        }
        charpt--;
    }


    sprintf( temp_buff, "B Clip: %s", charpt);
    TextExtent( rf_win->RPort, (STRPTR)temp_buff, strlen(temp_buff), &t_ext);

    temp_clip = clip_text(rf_win->RPort, 310, CLIP_FORWARD, temp_buff,
                            &out_strlen, &text_width_out);


    center_text(rf_win->RPort, 310, t_ext.te_Height, temp_clip,
                &xoffset, &yoffset);

    Move(rf_win->RPort, (xoffset + 350), 20);

    Text(rf_win->RPort, temp_clip, out_strlen);


    /* Display the Output Clip name */

    /* find the clip name, it will either be in a subdirectory '/'  */
    /*  or the root directory ':' of a drive */

    stringsize = strlen(outfilename);
//    charpt = bfilename;
    charpt = outfilename + stringsize;

    /* stop when find either 1st '/' or ':' */

    for (i=0; i<stringsize; i++)
    {
        if( (*charpt == '/') || (*charpt == ':') )
        {
            charpt++;
            break;
        }
        charpt--;
    }


    sprintf( temp_buff, "Output Clip: %s", charpt);
    TextExtent( rf_win->RPort, (STRPTR)temp_buff, strlen(temp_buff), &t_ext);

    temp_clip = clip_text(rf_win->RPort, 630, CLIP_FORWARD, temp_buff,
                            &out_strlen, &text_width_out);


    center_text(rf_win->RPort, 630, t_ext.te_Height, temp_clip,
                &xoffset, &yoffset);

    Move(rf_win->RPort, xoffset, 40);

    Text(rf_win->RPort, temp_clip, out_strlen);


//    Text(rf_win->RPort, charpt, sizeof(charpt) );



    return;
}


/* request and load a new */

int load_graph(struct graph_struct *graph)
{
    FILE                    *infile;
    char                    srcfile[512];
    unsigned long int       frame, last_frame=0, new_trans_len, old_trans_len;
    double                  percent, tension, bias, cont;
    int                     linear;
    int                     i, num_points;
    char                    buff[512];
    int                     status;
    int                     mode;


    status = bb_filereq(rf_win, FILE_OPEN, "Select graph data file to load",
                               "prowave:graphs", NULL, srcfile, 512);

    if (status == FALSE)
    {
        return(OK);
    }

//    printf("Filename is %s\n", srcfile);

    if( (infile = fopen(srcfile,"r")) == NULL)
    {
        sprintf(msg_buf,"Unable to open %s as an input graphfile\n", srcfile);
        log_message(ERROR_MSG,msg_buf);
        post_easyrequest(rf_win, "LOAD ERROR", "Unable to open file\n", "OK");
        return(ERROR);
    }


    /* try to read the header (RFE1) */
    fscanf(infile, "%s", buff);
    if ( (strstr("RFE1",buff)) == NULL)
    {
        sprintf(msg_buf, "File not a RenderFX RFE1 file\n");
        log_message(ERROR_MSG,msg_buf);
        post_easyrequest(rf_win, "LOAD ERROR", "Not a RenderFX graph file\n",
                         "OK");
        fclose (infile);
        return(ERROR);
    }


    /* get the number of points */
    fscanf(infile, "%d", &num_points);


    /* rather than screw around with storage buffers, just read through */
    /* the file to find the last frame and then rewind the file */

    for( i=0; i<num_points; i++)
    {

        /* read the point (frame, percent, tension, bias, cont, linear) */
        if ( (fscanf(infile,"%ld %lf %lf %lf %lf %d", &frame, &percent,
                            &tension, &bias, &cont, &linear)) != 6 )
        {
            sprintf(msg_buf,"Error reading data point %d from input file\n",i);
            log_message(ERROR_MSG,msg_buf);

            sprintf(msg_buf,"Error reading point %d", i);
            post_easyrequest(rf_win, "LOAD ERROR", msg_buf, "OK");
            fclose(infile);
            return(ERROR);
        }
    }

    new_trans_len = frame + 1;
    old_trans_len = trans_len;

    /* if the frame is not the same lenght as the current lenght, give */
    /* the user the option of changing the length, scaling, or aborting */

    if ( new_trans_len != trans_len)
    {
        mode = post_easyrequest(rf_win, "GRAPH LOAD",
        "New Transition Length is not the\nsame as current length",
        "Scale to Current|Use New|Cancel");

        /* Scale to Current = 1, Use New = 2, Cancel = 0 */
    }

    /* if cancel, give up */

    if ( mode == 0)
    {
        return(OK);
    }

    /* if we will be using the new transiton length, see if it fits */

    if ( mode == 2)
    {
        status = set_trans_len( trans_len);
        if (status == ERROR)
        {
            log_message(ERROR_MSG,"Unable to set new trans len in load_graph\n");
            post_easyrequest(rf_win, "LOAD_ERROR",
     "Unable to set the new transition length\nToo long or short for clips\n",
                                "OK");
            fclose(infile);
            return(ERROR);
        }
    }

    /* load the new data */

    fseek(infile, 0, SEEK_SET);

    /* read past the header and number of entries lines */
    fscanf(infile, "%s", buff);
    fscanf(infile, "%s", buff);


    /* the current graph */
    clear_graph_points(graph);

    /* read the data and put into the graph */

    for( i=0; i<num_points; i++)
    {

        /* read the point (frame, percent, tension, bias, cont, linear) */
        if ( (fscanf(infile,"%ld %lf %lf %lf %lf %d", &frame, &percent,
                            &tension, &bias, &cont, &linear)) != 6 )
        {
            sprintf(msg_buf,"Err reading point %d in load_graph()\n",i);
            log_message(ERROR_MSG,msg_buf);
        }
        else
        {
            if( (i == 0) || (frame > last_frame) )
            {
                if ( (append_graph_point(graph, frame, percent, tension,
                        bias, cont, linear)) == ERROR)
                {
                    sprintf(msg_buf,"err append point %d in load_graph()\n",i);
                    log_message(ERROR_MSG,msg_buf);

                    sprintf(msg_buf,
                     "Error appending point %d\nResetting Graph\n", i);
                    post_easyrequest(rf_win, "LOAD ERROR", msg_buf, "OK");
                    set_trans_len(old_trans_len);
                    clear_graph_points(graph);
                    append_graph_point(graph, 0, 0.0, 0.0, 0.0, 0.0, 1);
                    append_graph_point(graph, (old_trans_len - 1), 0.0,
                                                0.0, 0.0, 0.0, 1);
                    return(ERROR);

                }
                last_frame = frame;
            }
        }
    }

    /* now scale the graph, if required */
    if (mode == 1)
    change_graph_length ( graph, old_trans_len);


    fclose(infile);
    return(OK);

}



/* request and save a graph file */

int save_graph(struct graph_struct *graph)
{
    FILE                    *outfile;
    char                    savefile[512];
    unsigned long int       frame;
    double                  percent, tension, bias, cont;
    int                     linear;
    int                     status;
    int                     i;


    status = bb_filereq(rf_win, FILE_SAVE, "Select graph data file to save",
                               "prowave:graphs", NULL, savefile, 512);

    if (status == FALSE)
    {
        return(OK);
    }

//    printf("Filename is %s\n", savefile);

    /* 1st see if file already exists, if it does, prompt for confirmation */
    /* before trashing it! */

    if( (outfile = fopen(savefile,"r")) != NULL )
    {
        fclose(outfile);

        sprintf(msg_buf,
                "\n%s\n\n   Already Exists.\n\nDo you want to overwrite?\n",
                        savefile);

        status = post_easyrequest(rf_win, "FILE OVERWITE WARNING", msg_buf,
                                        "OVERWRITE|CANCEL");

        if (status == 0)
        {
            return(OK);
        }
        else
        {
            status = remove(savefile);
        }
    }

    if( (outfile = fopen(savefile,"w")) == NULL )
    {
        sprintf(msg_buf,"Error opening %s for output\n", savefile);
        log_message(ERROR_MSG,msg_buf);

        status = post_easyrequest(rf_win, "SAVE ERROR", msg_buf,
                                        "OK");

        return(ERROR);
    }

    fprintf(outfile,"RFE1\n");
    fprintf(outfile,"%d\n", graph->num_points);

    for(i=0; i<graph->num_points; i++)
    {

        set_graph_current_index( graph, i);

        get_graph_index_values( graph, &frame, &percent, &tension,
                                 &bias, &cont, &linear);

        fprintf(outfile,"%ld %lf %lf %lf %lf %d\n", frame, percent,
                        tension, bias, cont, linear);
    }

    fclose(outfile);


    return(OK);
}



/*-- update the aclipout and bclipin values using the value of --*/
/*-- trans_len,  if there is an uneven number of colorframes in the --*/
/*-- transiiton, put the extra one on the aclipout --*/

void update_in_out(void)
{


    unsigned long int   half_trans, qtr_trans;

    /* calculate the aout and bin points using 1/2 the transition lenght */

    half_trans = (unsigned long int)(trans_len / 2);
    qtr_trans = (unsigned long int)(trans_len / 4);

    aclipout = aclipstart + half_trans;
    bclipin = bclipstart + half_trans;

    if( (2*qtr_trans) < half_trans)
    {
        aclipout += 1;
        bclipin -= 1;
    }

    return;
}







/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/

