/*********************************************************************\
*
* FSinfo - Show Flyer drive's FileSystem info
*
* $Id: FSinfo.c,v 1.0 1995/05/05 15:50:34 Flick Exp $
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* All information contained herein is property of
* NewTek, Inc. and is subject to change without
* notice.
*
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <proto/exec.h>

#include "flyerlib.h"
#include "flyer.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);

#define	ID_FLYROOT	0x524f4f54

struct Library			*FlyerBase;
struct FlyerVolume	FlyVol;
struct FlyerVolInfo	Buffer;

main(int argc,char **argv)
{
	int	unit,drive;
	ULONG	err,meg10;

	if ((argc!=3)||(argv[1][0]=='?'))
	{
		printf("Usage: %ls <board> <drive>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	FlyVol.Board = unit;
	GetArg(argv[2],&drive,0,23,"Drive");

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0)
	{
		printf("Open flyer.library failed\n");
		return;
	}

	FlyVol.SCSIdrive = drive;

	Buffer.len = sizeof(struct FlyerVolInfo);		// IMPORTANT!
	err = FlyerDriveInfo(&FlyVol,&Buffer);
	if (err)
		printf("Error %ld - %ls\n",err,Error2String(err));
	else
	{
		if (Buffer.Ident == ID_FLYROOT)
		{
			printf("Volume: %ls\n",Buffer.Title);

			printf("Total blocks: %lx\n",Buffer.Blocks);

			printf("Largest free area:  %lx (blocks)\n",Buffer.Largest);

			printf("Optimized free area:  %lx (blocks)\n",Buffer.Optimized);

			printf("Total Blocks free:  %lx\n",Buffer.BlksFree);

			meg10 = Buffer.Blocks / (100000 / Buffer.BlkSize);
			printf("Size: %ld.%ld MB\n",meg10/10,meg10%10);

			printf("%% used: %02ld%\n",
				((Buffer.Blocks-Buffer.BlksFree)*100)/Buffer.Blocks);

			meg10 = Buffer.FragBlks / (100000 / Buffer.BlkSize);
			printf("Fragmentation: %ld.%ld MB\n",meg10/10,meg10%10);

			printf("Drive can handle:\n");
			if (Buffer.Flags & FVIF_VIDEOREADY)
				printf("   Video\n");
			if (Buffer.Flags & FVIF_AUDIOREADY)
				printf("   Audio\n");

			if (Buffer.Flags & FVIF_WRITEPROT)
				printf("Write protected\n");
			else
				printf("Write enabled\n");

			printf("Status: %ls\n",Buffer.DiskOkay?"Okay":"Bad");
		}
		else
			printf("Unformatted\n");
	}

	CloseLibrary(FlyerBase);

	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi))
	{
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
