/*********************************************************************\
*
* RecClip - Record video clip
*
* $Id: RecClip.c,v 1.0 1995/05/05 15:52:26 Flick Exp $
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* All information contained herein is property of
* NewTek, Inc. and is subject to change without
* notice.
*
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include "flyerlib.h"
#include "flyer.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);
void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label);

struct Library *FlyerBase;
struct FlyerVolume FlyVol;
struct ClipAction	 Action;

BPTR	handle;
UBYTE	PauseFlag;


main(int argc,char **argv)
{
	int	board=0,vidchan,drive,vidflag,audchans;
	ULONG	err,cfrmcount;

	if ((argc!=7)||(argv[1][0]=='?'))
	{
		printf("Usage: %ls <vchan> <drive> <name> <frames> <vidflag> <audchans>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&vidchan,0,3,"Video channel");
	GetArg(argv[2],&drive,0,23,"SCSI drive");
	GetLongArg(argv[4],&cfrmcount,0,0x7FFFFFFF,"Frame Count");
	GetArg(argv[5],&vidflag,0,1,"Video Flag");
	GetArg(argv[6],&audchans,0,2,"Audio Channels");

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0)
	{
		printf("Open flyer.library failed\n");
		return;
	}

	printf("Press RETURN to abort\n");

	handle = Input();

	FlyVol.Path = argv[3];			// Name
	FlyVol.Board = board;
	FlyVol.SCSIdrive = drive;		// This overrides any specified volume name...
	FlyVol.Flags = FVF_USENUMS;	// ...because of this flag

	Action.Volume = &FlyVol;
	Action.ReturnTime = RT_IMMED;	// Come back while recording

	Action.Channel = vidchan;
	Action.Flags = 0;
	if (vidflag)
		Action.Flags |= CAF_VIDEO;
	if (audchans==2)
		Action.Flags |= (CAF_AUDIOL | CAF_AUDIOR);
	else if (audchans==1)
		Action.Flags |= CAF_AUDIOL;
	Action.VidFieldCount = cfrmcount * 4;
	Action.AudFieldCount = cfrmcount * 4;
	Action.GoClock = 0;				// Soon as possible
//	Action.VolSust1 = 0xFFFF;		// These do NOT set record gain (see CODEC.c)
//	Action.VolSust2 = 0xFFFF;
	Action.VidStartField = 0;

	err = FlyerRecord(&Action);		// Start recording and return
	if (err==FERR_OKAY)
	{
		for (;;)
		{
			if (WaitForChar(handle,30))
			{
				if (getchar()=='p')		// Pause/resume?
				{
					PauseFlag = 1-PauseFlag;
					printf("(%ls)\n",PauseFlag?"paused":"resumed");
					err = PauseAction(&Action,PauseFlag);
					getchar();
				}
				else
				{
					err = AbortAction(&Action);		// RETURN only: stop recording
					printf("Aborted\n");
					break;
				}
			}
			err = CheckAction(&Action);		// Check if stopped recording on its own
			if (err != FERR_BUSY)
			{
				printf("Complete\n");
				break;
			}
		}
	}

	printf("Cframes recorded = %ld\n",(Action.LastFieldDone+1)>>2);

	if (err)
		printf("Error %ld - %ls\n",err,Error2String(err));
	else
		printf("Okay\n");

	CloseLibrary(FlyerBase);

	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi))
	{
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}


void GetLongArg(char *arg,ULONG *var,ULONG lo,ULONG hi,char *label)
{
	stch_l(arg,(long *)var);
	if ((*var < lo)||(*var > hi))
	{
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
