/*********************************************************************\
*
* SCSI - Find and display drives on SCSI bus
*
* $Id: SCSI.c,v 1.0 1995/05/05 15:52:32 Flick Exp $
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* All information contained herein is property of
* NewTek, Inc. and is subject to change without
* notice.
*
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <proto/exec.h>

#include "flyerlib.h"
#include "flyer.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);

struct DriveParam {
	UBYTE	scsiver,inqlen;
	ULONG	size;
};

struct Library *FlyerBase;
struct FlyerVolume FlyVol;

main(int argc,char **argv)
{
	int	unit,chan,i,j,len,devtype;
	ULONG	err,blkcnt,blklen;
	UBYTE	MiscBuff[18],drives,InqBuff[255];
	struct	DriveParam	Stats[8];

	if ((argc!=3)||(argv[1][0]=='?'))
	{
		printf("Usage: %ls <board> <channel>\n",argv[0]);
		return(20);
	}

	GetArg(argv[1],&unit,0,3,"Board number");
	FlyVol.Board = unit;
	GetArg(argv[2],&chan,0,2,"Channel");
	FlyVol.SCSIdrive = (chan<<3)+0;

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0)
	{
		printf("Open flyer.library failed\n");
		return;
	}

	printf("Initializing SCSI channel\n");
	err = SCSIinit(&FlyVol);
	if (err)
		printf("Error %ld - %ls\n",err,Error2String(err));

	printf("Finding drives on bus\n");
	err = FindDrives(&FlyVol,MiscBuff);
	if (err)
		printf("Error %ld - %ls\n",err,Error2String(err));

	drives = MiscBuff[0];		/* Bit flags for each ID on bus (1=drive present) */

	for (i=0;i<8;i++)
	{
		Stats[i].scsiver = MiscBuff[2+i];			/* SCSI version */
		Stats[i].inqlen  = MiscBuff[10+i]+5;		/* Length of Inquiry data */
	}

	for (i=0;i<7;i++)
	{
		printf("Drive %d\n",i);
		if ((1<<i)&drives)			/* Drive was present? */
		{
			FlyVol.SCSIdrive = (chan<<3)+i;

			len = Stats[i].inqlen;
			/* Get inquiry data from SCSI drive */
			err = Inquiry(&FlyVol,Stats[i].inqlen,InqBuff);
			if (err)
				printf("Inquiry Error %ld - %ls\n",err,Error2String(err));

			printf("   ");
			for (j=8;j<=31;j++)					/* Print Manufacturer */
				printf("%c",InqBuff[j]);
			printf("\n   Rev: ");
			for (j=32;j<=35;j++)					/* Print model # */
				printf("%c",InqBuff[j]);
			if (len>=43)
			{
				printf("   Date: ");
				for (j=36;j<=43;j++)
					printf("%c",InqBuff[j]);	/* Print date */
			}
			if (len>=55)
			{
				printf("   SerNum: ");
				for (j=44;j<=55;j++)
					printf("%c",InqBuff[j]);	/* Print Serial # */
			}
			printf("\n");

			devtype = InqBuff[0] & 0x1F;

			printf("   Device Type = %ld\n",devtype);
			if (devtype == 0)						// Only for random-access R/W devices (hard drives)
			{
				/* Find drive's logical block size */
				err = ReadSize(&FlyVol,&blkcnt,&blklen);
				if (err)
					printf("ReadSize Error %ld - %ls\n",err,Error2String(err));

				Stats[i].size = blkcnt;
				printf("   %lx Blocks     %ld bytes/block\n",blkcnt,blklen);
			}
		}
		else
			printf("   Not Present\n");
	}

	CloseLibrary(FlyerBase);

	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi))
	{
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
