/*********************************************************************\
*
* WriteField - Creates video fields in a clip
*
* $Id: WriteField.c,v 1.0 1995/05/05 15:54:03 Flick Exp $
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* All information contained herein is property of
* NewTek, Inc. and is subject to change without
* notice.
*
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <dos/dos.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include "flyerlib.h"
#include "flyer.h"
#include "FlyerPrivate.h"

void GetArg(char *arg,int *var,int lo,int hi,char *label);


struct Library *FlyerBase;

struct FlyerVolume	FlyVol;
struct ClipAction		Action;
struct ClipInfo		Cinfo;

UBYTE Buffer[SAMPLESPERLINE],AltBuf[SAMPLESPERLINE];

main(int argc,char **argv)
{
	int	board=0,i,pattern,line,modes,fieldnum;
	ULONG	err;
	UBYTE	*buff,qualmode;

	if ((argc==4) && (!stricmp(argv[3],"NEW")))
	{
		modes = FWF_NEW;
		argc--;
	}
	else
		modes = 0;

	if ((argc!=3)||(argv[1][0]=='?'))
	{
		printf("Usage: %ls <volume:name> <pattern> [NEW]\n",argv[0]);
		return(20);
	}

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0)
	{
		printf("Open flyer.library failed\n");
		return;
	}

	GetArg(argv[2],&pattern,0,0xFF,"Pattern");

	FlyVol.Board = board;
	FlyVol.Path = argv[1];
	FlyVol.Flags = 0;			// We'll specify volume in name, not drive number

	if (modes & FWF_NEW)
		fieldnum = 0;
	else
	{
		Cinfo.len = sizeof(struct ClipInfo);
		err = GetClipInfo(&FlyVol,&Cinfo);
		if (err)
			printf("ClipInfo Error %ld - %ls\n",err,Error2String(err));
		else
		{
			fieldnum = Cinfo.Fields;
			printf("Field = %ld\n",fieldnum);
			fieldnum &= 3;		// Modulo 4
		}
	}

	Action.Volume = &FlyVol;

	Action.ReturnTime = RT_STOPPED;
	qualmode = 0;		// Best mode

	err = EasyOpenWriteField(&Action,0,modes | FWF_APPEND,qualmode);
	if (err)
		printf("\nOpen Error %ld - %ls\n",err,Error2String(err));
	else
	{
		Action.ReturnTime = RT_STOPPED;

		for (i=0;i<SAMPLESPERLINE;i++)
		{
			switch (pattern)
			{
			case 0:
				Buffer[i] = 71;					// Black
				break;
			case 1:
				Buffer[i] = 71+i/6;				// Luma ramp
				break;
			case 2:
				Buffer[i] = 71 + (i % 130);	// Multiple luma bands
				break;
			case 3:
				Buffer[i] = 200-i/6;				// Reverse luma ramp
				break;
			case 4:
				Buffer[i] = 200-(i % 130);		// Mult rvs luma bands
				break;
			case 5:
				if ((i & 3)==0)
				{
					Buffer[i+0] = 0x33;
					Buffer[i+1] = 0x4e;
					Buffer[i+2] = 0x93;
					Buffer[i+3] = 0x78;
					AltBuf[i+0] = 0x93;
					AltBuf[i+1] = 0x78;
					AltBuf[i+2] = 0x33;
					AltBuf[i+3] = 0x4e;
				}
				break;
			default:
				Buffer[i] = pattern;				// Other D2 values
			}
		}

		for (line=21;line<=260;line++)
		{
			Action.ReturnTime = RT_STOPPED;

			buff = Buffer;

			if (pattern == 5)
			{
				if ((fieldnum == 0) || (fieldnum == 3))
				{
					if (line & 1)
						buff = AltBuf;
					else
						buff = Buffer;
				}
				else
				{
					if (line & 1)
						buff = Buffer;
					else
						buff = AltBuf;
				}
			}

			err = FlyerWriteLine(&Action,buff);
			if (err)
			{
				printf("\nError %ld on line %ld - %ls\n",err,line,Error2String(err));
				break;
			}
		}

		Action.ReturnTime = RT_STOPPED;
		err = CloseField(&Action);
		if (err)
			printf("\nClose Error %ld - %ls\n",err,Error2String(err));
	}

	CloseLibrary(FlyerBase);

	return(0);
}


void GetArg(char *arg,int *var,int lo,int hi,char *label)
{
	stch_i(arg,var);
	if ((*var < lo)||(*var > hi))
	{
		printf("Error: %ls is out of range\n",label);
		exit(20);
	}
}
