/*********************************************************************\
*
* MakeAudClip - Write a Flyer audio clip
*
* $Id: MakeAudClip.c,v 1.3 1995/10/16 17:16:15 Flick Exp $
*
* $Log: MakeAudClip.c,v $
*Revision 1.3  1995/10/16  17:16:15  Flick
*Made more suitable for public consumption
*
*Revision 1.2  1995/07/11  10:53:27  Flick
*Moved Flyer FS structures into FlyerFSfmt.h for all to share
*
*Revision 1.1  1995/05/08  10:27:53  Flick
*Uses private flyerlib.h (internal only)
*
*Revision 1.0  1995/05/05  15:51:53  Flick
*FirstCheckIn
*
*
* Copyright (c) 1994 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* All information contained herein is property of
* NewTek, Inc. and is subject to change without
* notice.
*
*	03/10/95		Marty	created
\*********************************************************************/

#include <exec/types.h>
#include <string.h>
#include <stdio.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include "flyerlib.h"
#include "flyer.h"
#include "flyerClipFmt.h"

#define	MONOAUDFLDSIZE		3
#define	NUMAUDCHANS			2
#define	SAMPLESPERFIELD	735	// Audio samples per video field

struct Library *FlyerBase;

UBYTE BlkBuff[512];

UWORD	AudioBuff[256*MONOAUDFLDSIZE];


/* Prototypes */
ULONG WriteTestClip(char *pathname, ULONG frames);
ULONG Reprocess(char *pathname, ULONG frames);


main(int argc,char **argv)
{
	ULONG	err,frames;

	if ((argc!=2)||(argv[1][0]=='?')) {
		printf("Usage: %ls <pathname>\n",argv[0]);
		return(20);
	}

	frames = 16;	// This test makes a clip 16 * 4 fields long

	if (FlyerBase = OpenLibrary(FLYERLIBNAME,0)) {
		err = WriteTestClip(argv[1],frames);	// Make skeletal version
		if (err == 0) {
			err = Reprocess(argv[1],frames);		// Let Flyer process into full clip
		}

		CloseLibrary(FlyerBase);
	}
	else {
		printf("Open flyer.library failed\n");
	}

	return(0);
}


ULONG WriteTestClip(char *pathname, ULONG frames)
{
	int	i,j;
	ULONG	frm;
	BPTR	handle;
	struct ClipHeader		*ch;
	struct FrameHeader	*fh;
	struct Field			*fld;

	/* Create an ugly sawtooth sample */
	j = 0;
	for (i=0 ; i<SAMPLESPERFIELD ; i++)
		AudioBuff[i] = (j+=100);

	handle = Open(pathname,MODE_NEWFILE);
	if (!handle) {
		printf("Did not write clip successfully\n");
		return(1);
	}

	/* Build and write Clip header */
	ch = (struct ClipHeader *)&BlkBuff;
	memset(ch,0,sizeof(struct ClipHeader));

	ch->ID = ID_CLIP;
	ch->Length = 1+frames*(1+MONOAUDFLDSIZE*NUMAUDCHANS*4);
	ch->Version = 4;
	ch->VidCmpVer = 2;
	ch->AudVer = 2;
	ch->Fields = frames * 4;
	ch->DataStart = 1;
	ch->DataEnd = frames*(1+MONOAUDFLDSIZE*NUMAUDCHANS*4);
	ch->Tail = 0;					// No index table!
	ch->AudioLength = MONOAUDFLDSIZE;
	ch->VidFlag = FALSE;
	ch->AudioChans = NUMAUDCHANS;
	ch->Pedestal = 60;
	memset(ch->CustCoefs,0,sizeof(WORD)*9);
	memset(ch->CustInvCoefs,0,sizeof(WORD)*9);
	ch->LogBlkSize = 512;
	ch->FIRROM = 0;
	ch->FIRRAM = 0;

	if (Write(handle,ch,sizeof(struct ClipHeader)) != sizeof(struct ClipHeader))
		printf("Clip header write error\n");

	for (frm=0 ; frm<frames ; frm++) {
		/* Build and write each frame header */
		fh = (struct FrameHeader *)&BlkBuff;
		memset(fh,0,sizeof(struct FrameHeader));

		fh->ID = ID_CFRM;
		fh->Length = 1+MONOAUDFLDSIZE*NUMAUDCHANS*4;
		fh->Version = 4;
		fh->VidCmpVer = 2;
		fh->AudVer = 2;
		fh->PrevFrame = (frm == 0)?0:(- fh->Length);
		fh->NextFrame = ((frm+1) >= frames)?0:(fh->Length);
		fh->FrameNumber = frm;
		fh->AudioLength = MONOAUDFLDSIZE;
		fh->VidFlag = FALSE;
		fh->AudioChans = NUMAUDCHANS;

		for (i=0;i<4;i++) {
			fld = &fh->fields[i];
			/* Leave all these blank, unless we add video to clip */
		}

		if (Write(handle,fh,sizeof(struct FrameHeader)) != sizeof(struct FrameHeader))
			printf("Frame header write error\n");

		/* Write all audio data for this color frame */
		for (i=0 ; i<NUMAUDCHANS; i++) {
			for (j=0 ; j<4 ; j++) {
				if (Write(handle,AudioBuff,512*MONOAUDFLDSIZE) != 512*MONOAUDFLDSIZE)
					printf("Audio data write error\n");
			}
		}
	}

	Close(handle);


	return(0);		// Okay
}


// Reprocess -- Let Flyer re-process our clip to build all the reserved
//					structures needed.  These are quite technical (as well as
//					being private), so we'll let it do the work for us.  We
//					just provide a clip that has the minimal stuff needed.
//					This, obviously, requires that the clip was written to
//					a Flyer drive.  If not, you should copy the "skeletal"
//					clip file to a Flyer drive first, then do this function.
ULONG Reprocess(char *pathname, ULONG frames)
{
	struct FlyerVolume FlyVol,*volptr;
	struct ClipAction	 Action,*actptr;
	ULONG	res;

	actptr = &Action;
	volptr = &FlyVol;

	/* Let Flyer find original clip and prepare to process it */

	volptr->Board = 0;
	volptr->Path = pathname;		// Full path and filename (on Flyer drive)
	volptr->Flags = 0;							// None
	actptr->Volume = volptr;					// Link together
	actptr->ReturnTime = RT_STOPPED;			// Don't return until done

	res = StartClipCutList(actptr,1);		// Ready for destructive reprocessing
	if (res) {
		printf("Error %ld on StartClipCutList - %ls\n",res,Error2String(res));
		return(res);
	}


	/* Now define one clip to make from original */

	volptr->Board = 0;
	volptr->Path = pathname;					// Make new clip the same name
	volptr->Flags = 0;							// None
	actptr->Volume = volptr;					// Link together
	actptr->ReturnTime = RT_STOPPED;			// Don't return until done
	actptr->AudStartField = 0;					// Reprocess all of clip
	actptr->AudFieldCount = frames * 4;

	actptr->Flags = CAF_REPROCESS;		// Force Flyer to re-process this!
	if (NUMAUDCHANS == 2)
		actptr->Flags |= CAF_AUDIOL | CAF_AUDIOR;
	else
		actptr->Flags |= CAF_AUDIOL;

	res = AddClipCut(actptr);					// Define the new clip
	if (res) {
		printf("Error %ld on AddClipCut - %ls\n",res,Error2String(res));
		res = EndClipCutList(0);				// Let Flyer clean up
		return(res);
	}

	res = EndClipCutList(1);				// Do it now!!!
	if (res) {
		printf("Error %ld on EndClipCutList - %ls\n",res,Error2String(res));
	}

	return(FERR_OKAY);
}
