/*********************************************************************\
*
* NewSequence - Flyer test sequencer (v4.05+)
*
* $Id: newseq.c,v 1.3 1995/06/20 09:12:06 Flick Exp $
*
* Copyright (c) 1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* All information contained herein is property of
* NewTek, Inc. and is subject to change without
* notice.
*
\*********************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include "flyerlib.h"
#include "flyer.h"


/*** Prototypes ***/
ULONG ReadScriptFile(char *name);
void FreeFile(void);
ULONG AddStep(struct Entry *ent);


struct Entry {
	ULONG	clock;
	ULONG	drive;
	char	name[80];
	ULONG	vidflag;
	ULONG	chan;
	ULONG	vstartfld;
	ULONG	vfldlen;
	ULONG	audchans;
	ULONG	astartfld;
	ULONG	afldlen;
	ULONG vol1,vol2;
	ULONG pan1,pan2;
	ULONG	attack;
	ULONG	decay;
};

#define	MAXSTEPS	100					/* For this demo */
struct Entry Sequence[MAXSTEPS];

struct Library *FlyerBase;
struct ClipInfo	clipinfo;

UBYTE	*FileBuffer;
ULONG	FileLength;


main(int argc,char **argv)
{
	int	i,ndx;
	ULONG	baseclock,err;
	char *scriptptr,*endptr;
	struct Entry *entptr;
	BOOL	doitagain;

	if ((argc!=2)||(argv[1][0]=='?'))
	{
		printf("Usage: %ls <script | AGAIN>\n",argv[0]);
		return(20);
	}

	if ((FlyerBase = OpenLibrary(FLYERLIBNAME,0))==0)
	{
		printf("Open flyer.library failed\n");
		return;
	}

	doitagain = !(stricmp(argv[1],"AGAIN"));

	if ((doitagain) || (ReadScriptFile(argv[1])))
	{
		if (!doitagain)		// Need to parse input script and send to Flyer?
		{
			endptr = FileBuffer+FileLength;
			for (ndx=0,scriptptr=FileBuffer;(ndx<MAXSTEPS)&&(scriptptr<endptr);)
			{
				if (*scriptptr == '>')				// An active line?
				{
					entptr = &Sequence[ndx];
					scriptptr++;
					sscanf(scriptptr,
						"%ld %ld %ls %ld %ld %ld %ld %ld %ld %ld %ld %ld %lx %lx %lx %lx\n",
						&entptr->clock,
						&entptr->drive,
						entptr->name,
						&entptr->vidflag,
						&entptr->chan,
						&entptr->vstartfld,
						&entptr->vfldlen,
						&entptr->audchans,
						&entptr->astartfld,
						&entptr->afldlen,
						&entptr->attack,
						&entptr->decay,
						&entptr->vol1,
						&entptr->vol2,
						&entptr->pan1,
						&entptr->pan2
					);

					if (!stricmp(entptr->name,"(none)"))		// Turn "(none)" into NULL string
						entptr->name[0] = 0;

					ndx++;
				}

				// Skip to next line
		      do
	   	   {
					if (scriptptr >= endptr)
						break;
				} while (*scriptptr++ != 10);			// Hunt for LF
			}

			// Dump sequence as read from input script
			for (i=0;i<ndx;i++)
			{
				entptr = &Sequence[i];
				printf("Step %ld at %ld: %ls from drive %ld\n",i+1,entptr->clock,entptr->name,entptr->drive);
				printf("          vflag=%ld, vchan=%ld, vflds=%ld/%ld\n",
					entptr->vidflag,entptr->chan,entptr->vstartfld,entptr->vfldlen);
			}

			printf("Opening sequence\n");

			err = NewSequence(0);					// Prepare Flyer for download
			if (err)
				printf("Open Error %ld - %ls\n",err,Error2String(err));


			for (i=0;i<ndx;i++)
			{
				printf("Adding step %ld\n",i+1);

				entptr = &Sequence[i];

				err = AddStep(entptr);				// Add next step to sequencer
				if (err)
				{
					printf("AddStep Error %ld at step %ld - %ls\n",err,i+1,
						Error2String(err));
					err = EndSequence(0,0);			// MUST close sequence!
					goto abort;
				}
			}

			printf("Ready to close sequence\n");

			err = EndSequence(0,1);					// Close sequence, let Flyer prepare it
			if (err)
			{
				printf("Close Error %ld - %ls\n",err,Error2String(err));
				goto abort;
			}

			printf("Ready to play sequence\n");
		}


		/*** Play (previously) downloaded sequence ***/

		/* Get Flyer's current field clock value */
		err = GetFieldClock(&baseclock);
		if (err)
			printf("Error %ld - %ls\n",err,Error2String(err));

		err = PlaySequence(0,baseclock + 30);			// Begin 30 fields from "now"
		if (err)
			printf("Play Error %ld - %ls\n",err,Error2String(err));
	}

abort:
	CloseLibrary(FlyerBase);
	FreeFile();

	return(0);
}


ULONG ReadScriptFile(char *name)
{
	FILE	*fp;
	int	read;

	read = 0;
	fp = fopen(name,"r");
	if (fp)
	{
		fseek(fp,0,SEEK_END);
		FileLength = ftell(fp);
		fseek(fp,0,SEEK_SET);

		FileBuffer = AllocMem(FileLength,MEMF_PUBLIC);
		if (FileBuffer)
		{
			read = fread(FileBuffer,FileLength,1,fp);
			if (read != 1)
				printf("Warning: entire file not read!\n");
		}
		else
			printf("Could not allocate memory\n");
		fclose(fp);
	}
	else
		printf("Could not find file %s\n",name);

	if (read==1)
		return(FileLength);
	else
		return(0);
}


void FreeFile(void)
{
	if (FileLength)
		FreeMem(FileBuffer,FileLength);
}


ULONG AddStep(struct Entry *ent)
{
	struct FlyerVolume volume;
	struct ClipAction	 action;

	volume.Board = 0;
	volume.Flags = FVF_USENUMS;
	volume.Path = ent->name;
	volume.SCSIdrive = ent->drive;

	action.Channel = ent->chan;
	action.Flags = 0;
	if (ent->vidflag)
		action.Flags |= CAF_VIDEO;
	if (ent->audchans == 1)
		action.Flags |= CAF_AUDIOL;
	if (ent->audchans == 2)
		action.Flags |= CAF_AUDIOL | CAF_AUDIOR;
	action.PermissFlags = 0;
	action.GoClock = ent->clock;
	action.VidStartField = ent->vstartfld;
	action.VidFieldCount = ent->vfldlen;
	action.AudStartField = ent->astartfld;
	action.AudFieldCount = ent->afldlen;
	action.TotalAudStart = 0;
	action.TotalAudLength = 0;
	action.VolSust1 = ent->vol1;
	action.VolSust2 = ent->vol2;
	action.AudioPan1 = ent->pan1;
	action.AudioPan2 = ent->pan2;
	action.VolAttack = ent->attack;
	action.VolDecay = ent->decay;

	action.Volume = &volume;			// Link together
	action.ReturnTime = RT_STOPPED;

	return(AddSeqClip(&action));
}
