#ifndef NEWTEK_COMP_CONVERT_H
#define NEWTEK_COMP_CONVERT_H
/*
 * $Id: comp_convert.h 5.7 1995/09/08 18:00:14 pfrench Exp $
 *
 * $Source: n:/rcs/include/newtek/comp_convert.h $
 *
 * Copyright (c) 1995 NewTek, Inc.
 * Confidental and Proprietary. All rights reserved. 
 *
 */

typedef signed short	iq_val;
typedef unsigned short	y_val;

/* Values the Toaster Digitizer gives for different scope levels */
#define D2_LEVEL_BLACK		71
#define D2_LEVEL_WHITE		200
#define D2_LEVEL_BLANKING	60
#define D2_LEVEL_SYNC		4

#define D2_LUMA_RANGE	(D2_LEVEL_WHITE - D2_LEVEL_BLACK)

struct comp_convert
{
	long			 width;

	/* Composite Buffer, one byte per pixel */
	unsigned char	*comp_buf;

	/* WARNING WARNING WARNING WARNING */

	/* "comp_buf" MUST BE PADDED BY EIGHT BYTES OF USABLE MEMORY BEFORE
	 * AND EIGHT BYTES TRAILING THE END IF YOU INTEND TO USE VTASC ON THE DATA.
	 *
	 * Note that those eight bytes will always be preserved, they just
	 * must not generate an MMU hit when accessed by VTASC functions.
	 *
	 * If the object was allocated by "ob_AllocObj" then you are OK for
	 * the leading eight bytes.  Just allocate an extra eight bytes for
	 * "comp_buf" only.
	 *
	 * If your ByteMap or Frame was allocated with bym_Create() or
	 * fr_Create(), this has already been done for you.  I predict most
	 * problems will arise with people using the composite_convert functions
	 * and then running the FIR on the D2 buffer they've allocated.
	 */

	/* YIQ Buffers, one word per pixel */
	y_val			*y_buf;		/* Unsigned */

	iq_val			*iq_buf;	/* Alternating IQ buffer */

	iq_val			*i_buf;
	iq_val			*q_buf;

	/* RGB Buffers, one byte per pixel */
	unsigned char	*r_buf;
	unsigned char	*g_buf;
	unsigned char	*b_buf;
};

/* D2 quad position macros */

/*
 *	ODD_FIELD(field_num)
 *
 *		Returns whether this is an oddly-interpreted field beginning with field zero.
 *		Essentially, fields zero and three are odd, one and two are even.
 *
 */
#define ODD_FIELD(f)		!(((f) & 0x01) ^ (((f)>>1) & 0x01))

/*
 *	ODD_SCANLINE(field_num,y_position)
 *
 *		Determines whether the scanline pixel quads are the eqns.
 *			(y+i,y+q,y-i,y-q) or (y-i,y-q,y+i,y+q)
 *
 */
#define ODD_SCANLINE(f,y)	( ((y) & 0x01) ^ ODD_FIELD(f) )

/*
 *	QUAD_POS(field_num,x_pos,y_pos)
 *
 *		Determines the eqn position withing a given quad
 *		0 = y+i,	1 = y+q,	2 = y-i,	3 = y-q
 *
 */
#define QUAD_POS(f,x,y)		((ODD_SCANLINE((f),(y)) ? ((x)+2):(x)) & 0x03)


#endif /* NEWTEK_COMP_CONVERT_H */

