#ifndef NEWTEK_FLYERFSFORMAT_H
#define NEWTEK_FLYERFSFORMAT_H TRUE
/*
 * $Id: flyerfsfmt.h 5.1 1995/10/25 22:41:02 pfrench Exp $
 *
 * $Source: n:/rcs/include/newtek/flyerfsfmt.h $
 *
 * $Log: flyerfsfmt.h $
 * Revision 5.1  1995/10/25 22:41:02  pfrench
 * Had to add structure packing pragma for flyer clip stuff
 * Revision 5.0  1995/09/21 19:32:58  pfrench
 * Initial revision
 *
 * Copyright (c) 1995 NewTek, Inc.
 * Confidental and Proprietary. All rights reserved. 
 *
 */

#ifndef PROOF_OS_H
#include <proof/proof_os.h>
#endif

#ifdef _WIN32
	#pragma pack(1)
#endif

struct FIRSET {				// Flyer FIR coefficient/scale set
	UWORD	coefs[8];
	UWORD	scale;
};

/* Video revisions */
#define	VTASC_OLDD2		1
#define	VTASC_NEWD2		2

#define	NTSC_BLANK		60			// NTSC level for blanking
#define	NTSC_BLACK		71			// NTSC level for black

#define	SERIALMAXLEN	36

struct FLDINFO {					// Structure for individual field data
	UWORD	fi_vloc;				// Block offset of video field start
	UWORD	fi_vlen;				// Block length of video field
	UBYTE	fi_tol;					// Bit tolerance
	UBYTE	fi_rndsz;				// Random number size
	UBYTE	fi_rndfrq;				// Random number frequency
	UBYTE	fi_seed; 				// Random number seed
	UWORD	fi_aloc;				// Block offset of audio field start
	UWORD	fi_alen;				// Block length of audio field
	BYTEBITS	fi_flags;			// FIF_xxx below
	UBYTE	fi_pad1;
	UBYTE	reserved1[18];			// Future
	UBYTE	fi_FIRset;				// FIR set 1-n or 0 for custom
	UBYTE	fi_sertype;				// Type of serial device for data
	UBYTE	fi_sermake;				// Make of serial device for data
	UBYTE	fi_serlen;				// Length of serial data
	UBYTE	fi_serial[SERIALMAXLEN];// SMPTE recording
};

#define	FIF_SUBNYQ	(1<<0)			// Used sub-nyquist algorithm

struct FRAMEHDR {				// Header structure for each color frame
	ULONG	fh_id;				// Four ASCII digits "CFRM"
	ULONG	fh_chunklen;		// Blocks in this chunk of data (incl hdr)
	UBYTE	fh_version;			// Version of Flyer that wrote data
	UBYTE	fh_vidcmpver;		// Version of video compression used
	UBYTE	fh_audcmpver;		// Version of audio compression used
	UBYTE	fh_pad1;	
	ULONG	fh_prevfrm;			// SCSI offset to prev color frame
	ULONG	fh_nextfrm;			// SCSI offset to next color frame
	ULONG	fh_frmnum;			// Frame offset from start
	UBYTE	fh_pad2;	
	UBYTE	fh_audlen;			// Blocks of audio per channel per field
	UBYTE	fh_vidflag;			// Video data recorded?
	UBYTE	fh_audchans;		// Audio channels recorded (0,1,2,...)
	UWORD	fh_phase;			// Playback phase adjustment
	UBYTE	fh__FIRtype;		// FIR coefficient type - from presets
	UBYTE	fh_pad3;
	UWORD	fh__FIRcoefs[8];	// FIR coefficients used
	ULONG	fh_backptrs[15];	// Offsets to prev 15 frames

	UBYTE	fh_reserved[110];	// Future stuff

	UWORD	fh__Pragma;			// Use pragmas below - must be $beef
	UWORD	fh__Offset;			// Pragma - horiz offset for play
	UBYTE	fh__CourseA;		// Pragma - course phase adjust, chan A
	UBYTE	fh__CourseB;		// Pragma - course phase adjust, chan B

	struct FLDINFO	fh_fld[4];	// Individual field data
#define	baseof_fhfld		1
};

struct CLIPHDR {				// Header structure at start of clips
	ULONG	ch_id;				// Four ASCII digits "CLIP"
	ULONG	ch_chunklen;		// Blocks in this chunk of data (incl hdr)
	UBYTE	ch_version;			// Version of Flyer that wrote data
	UBYTE	ch_vidcmpver;		// Version of video compression used
	UBYTE	ch_audcmpver;		// Version of audio compression used
	UBYTE	ch_pad1;	
	ULONG	ch_fields;			// Number of fields contained in clip
	ULONG	ch_datastart;		// SCSI offset to first frame of data
	ULONG	ch_dataend;			// SCSI offset to sector after data
	ULONG	ch_tail;			// SCSI offset to clip's tail info
	UBYTE	ch_pad2;	
	UBYTE	ch_audlen;			// Blocks of audio per channel per field
	UBYTE	ch_vidflag;			// Video data recorded?
	UBYTE	ch_audchans;		// Audio channels recorded (0,1,2,...)
	UBYTE	ch_pedestal;		// Pedestal value to use on playback
	UBYTE	ch_FIRset;			// Constant FIR coef set for clip (GET RID)
	struct FIRSET ch_custFIRcoefs[2];	/* Custom FIR coefs for clip */
	UWORD	ch_blksize;			// Size of logical blocks
	UBYTE	ch_FIRrom;			// FIR map ROM shape
	UBYTE	ch_FIRram;			// FIR map RAM shape
	UBYTE	ch_reserved[438];	// Future stuff
};

struct TAILHDR {				// Header structure for tail info
	ULONG	th_ID;				// Four ASCII digits "TAIL"
	ULONG	th_ChunkLen;		// Blocks in this chunk of data (incl hdr)

	/* Fast indexing data */
	ULONG	th_Table;			// SCSI offset to index table start
	UWORD	th_NdxTabSize;		// Number of blocks in index table
	UWORD	th_NdxCount;		// Number of index entries
	ULONG	th_MaxFrame;		// Max legal frame for searching - UNUSED
	UBYTE	th_reserved1[16];	// Future stuff

	/* Head management */
	ULONG	th_HeadList;		// SCSI offset to headlist blocks
	UWORD	th_HeadBlks;		// Number of blocks in heads table
	UBYTE	th_reserved2[32];	// Future stuff
		
	/* User data */
	ULONG	th_UserArea;		// SCSI offset to user blocks
	UWORD	th_UserBlks;		// Number of user blocks allocated

	UBYTE	th_reserved3[432];	// Future stuff
};

struct HEADHDR {				// Header structure for clip heads
	ULONG	hh_id;				// Four ASCII digits "HEAD"
	ULONG	hh_num1;
	ULONG	hh_minus1;

	UBYTE	hh_pad[512 - (3 * sizeof(ULONG))];
};

union ANYHDR {
	struct FRAMEHDR		fh;
	struct TAILHDR		th;
	struct CLIPHDR		ch;
	struct HEADHDR		hh;
};

#define	STONESPERBLK 128		// Milestones that fit in one table block

#define	HEADERLEN	1

#define	HEADTABLESIZE 2		// Number of head tables in clip tail
#define	USERAREASIZE 40		// Number of blocks for user data in tail

#define	ID_CFRM	0x4346524D	// "CFRM"
#define	ID_TAIL	0x5441494C	// "TAIL"
#define	ID_HEAD	0x48454144	// "HEAD"
#define	ID_CLIP	0x434C4950	// "CLIP"
#define	ID_TERM	0x5445524D	// "TERM"

#ifdef _WIN32
	#pragma pack()
#endif

#endif /* NEWTEK_FLYERFSFORMAT_H */

