#ifndef PROOF_CLASSUSR_H
#define PROOF_CLASSUSR_H TRUE
/*
 * $Id: classusr.h 5.4 1995/07/10 23:55:24 pfrench Exp $
 *
 * $Source: n:/rcs/include/proof/classusr.h $
 *
 * $Log: classusr.h $
 * Revision 5.4  1995/07/10 23:55:24  pfrench
 * Cleaned up some buggy list macros.
 * Revision 5.3  1995/07/10 22:37:02  pfrench
 * Moved proof includes to new directory.
 * Revision 5.2  1995/03/23 04:50:22  pfrench
 * Moved Method/Attribute bases to class_bases.h
 * Revision 5.1  1995/03/23 03:29:58  pfrench
 * Cleaned up Object Bases
 * Revision 5.0  1995/03/22 21:55:14  pfrench
 * Initial revision
 */

#ifndef PROOF_LIB_H
#include <proof/proof_lib.h>
#endif

#ifndef PROOF_CLASS_BASES_H
#include <proof/class_bases.h>
#endif

#ifndef INTUITION_CLASSUSR_H

	typedef ULONG	Object;

	typedef UBYTE	*ClassID;

	typedef struct
	{	ULONG			MethodID;
	} *Msg;

	/* Class IDs
	 */

	#define ROOTCLASS			"rootclass"
	#define ROOTCLASS_BASE		0x100L

#endif /* INTUITION_CLASSUSR_H */

#ifndef INTUITION_CLASSUSR_H

	/* Methods for the basic objects
	 */

	enum
	{	OM_Dummy = ROOTCLASS_BASE,
		OM_NEW,
		OM_DISPOSE,
		OM_SET,
		OM_GET,
	};

	/* Method structures for the basic objects
	 */

	/* OM_NEW, OM_SET, and OM_GET */
	struct opSet
	{	ULONG		  	MethodID;
		struct TagItem	*ops_Tags;
		Object			*ops_allocator_ob;
	};

#endif /* INTUITION_CLASSUSR_H */

#define ob_Dispose(ob)	ob_DoMethod((ob),OM_DISPOSE)

/**********************************************************/
/*****************************************************************************
 *	Class:			ALLOCATORCLASS
 *	SuperClass:		ROOTCLASS
 *	Description:
 *
 *	New Methods:
 *
 *		MEMM_ALLOCMEM,	
 *		MEMM_FREEMEM,		Work like Amiga system AllocMem/FreeMem
 *
 *		MEMM_ALLOCVEC,
 *		MEMM_FREEVEC,		Work like Amiga system AllocVec/FreeVec
 *
 *		MEMM_ALLOCMEMOBJ,	Allocate a ob_Dispose()able chunk of memory
 *
 *	Changed Methods:
 *
 *	Attributes:
 *
 *****************************************************************************
 *
 */

/* Methods for memory allocator class objects
 */
enum
{	MEMM_Dummy = ALLOCATORCLASS_BASE,

	MEMM_ALLOCMEM,
	MEMM_FREEMEM,
	/*
	 *		Work like Amiga system AllocMem/FreeMem
	 */

	MEMM_ALLOCVEC,
	MEMM_FREEVEC,
	/*
	 *		Work like Amiga system AllocVec/FreeVec
	 */

	MEMM_ALLOCMEMOBJ,
	/*
	 *		Creates a Memory object, that you delete
	 *		with ob_DoMethod(memptr,OM_DISPOSE);
	 */
};

/* Method structs for  memory allocator class objects
 */

/*
 *		MEMM_ALLOCMEM
 *		MEMM_ALLOCVEC
 *		MEMM_ALLOCMEMOBJ
 */
struct opAlloc
{	ULONG				 MethodID;
	ULONG				 am_Size;
	ULONG				 am_Flags;
};

/*
 *		MEMM_FREEMEM
 *		MEMM_FREEVEC
 *
 */
struct opFree
{	ULONG				 MethodID;
	APTR				 af_Ptr;
	ULONG				 af_Size;	/* ignored by MEMM_FREEVEC */
};

/* Quick function defines for allocator class */

#define ob_AllocMem(ob,siz,flg)	(APTR) ob_DoMethod((ob),MEMM_ALLOCMEM,(siz),(flg))
#define ob_AllocVec(ob,siz,flg)	(APTR) ob_DoMethod((ob),MEMM_ALLOCVEC,(siz),(flg))
#define ob_AllocMemObj(ob,siz)	(APTR) ob_DoMethod((ob),MEMM_ALLOCMEMOBJ,(siz),MEMF_PUBLIC|MEMF_CLEAR)
#define ob_AllocObj(ob,siz,flg)	(APTR) ob_DoMethod((ob),MEMM_ALLOCMEMOBJ,(siz),(flg))

#define ob_FreeMem(ob,ptr,siz)	ob_DoMethod((ob),MEMM_FREEMEM,(ptr),(siz))
#define ob_FreeVec(ob,ptr)		ob_DoMethod((ob),MEMM_FREEVEC,(ptr))

/* stdlib functions */

#define ob_malloc(ob,siz)		(void *) ob_DoMethod((ob),MEMM_ALLOCVEC,(siz),MEMF_ANY)
#define ob_calloc(ob,n,esiz)	(void *) ob_DoMethod((ob),MEMM_ALLOCVEC,(n)*(esiz),MEMF_CLEAR)
#define ob_free(ob,ptr)			ob_DoMethod((ob),MEMM_FREEVEC,(ptr))

/**********************************************************/

/*****************************************************************************
 *	Class:			MEMPOOLCLASS
 *	SuperClass:		ALLOCATORCLASS
 *	Description:
 *
 *		Memory Pool Class.  Optimizes speed of allocations.  Prevents memory
 *		fragmentation by placing allocations smaller than MPA_Threshhold
 *		within larger memory pools (puddles).  When a memory pool object is
 *		disposed, it automatically deletes all of its puddles, no need to
 *		free all those smaller allocations.
 *
 *	New Methods:
 *
 *	Changed Methods:
 *
 *		MEMM_ALLOCMEM
 *		MEMM_FREEMEM
 *
 *	Attributes:
 *
 *		MPA_PuddleSize,		(I) Size of individual puddles
 *
 *		MPA_Threshhold,		(I) Largest allocation to fit in a pool
 *
 *		MPA_MemFlags,		(I) Type of memory to allocate for puddles
 *
 *****************************************************************************
 *
 */

/* Attributes for mempool class objects
 */
enum
{	MPA_Dummy = ((LONG)TAG_USER | MEMPOOLCLASS_BASE),

	MPA_PuddleSize,		/* (I) Size of individual puddles */

	MPA_Threshhold,		/* (I) Largest allocation to fit in a pool */

	MPA_MemFlags,		/* (I) Type of memory to allocate for puddles */

};

/**********************************************************/

/*****************************************************************************
 *	Class:			CLASSCLASS
 *	SuperClass:		ROOTCLASS
 *	Description:
 *
 *	New Methods:
 *
 *	Changed Methods:
 *
 *	Attributes:
 *
 *		CLASSA_Name,			(I) This Class's name
 *
 *		CLASSA_SuperClass,		(I) ptr to this class's superclass
 *
 *		CLASSA_SuperClassID,	(I) name of the superclass
 *
 *		CLASSA_InstSize,		(I) size of instance data for this class
 *
 *		CLASSA_Private,			(I) Is class private??
 *
 *		CLASSA_NegInstSize,		(I) neg size of instance data for this class
 *
 *		CLASSA_UserData,		(I) Class implementor's data
 *
 *		CLASSA_Dispatch,		(I) Function dispatch
 *
 *		CLASSA_Private,			(I) Is class private??
 *
 *****************************************************************************
 *
 */

/* Attributes for class class objects
 */                          
enum
{	CLASSA_Dummy = ((LONG)TAG_USER | CLASSCLASS_BASE),

	CLASSA_Name,			/* (I) This Class's name */

	CLASSA_SuperClass,		/* (I) ptr to this class's superclass */

	CLASSA_SuperClassID,	/* (I) name of the superclass */

	CLASSA_InstSize,		/* (I) size of instance data for this class */

	CLASSA_NegInstSize,		/* (I) neg size of instance data for this class */

	CLASSA_UserData,		/* (I) Class implementor's data */

	CLASSA_Dispatch,		/* (I) Function dispatch */

	CLASSA_Private,			/* (I) Is class private?? */

};

/*****************************************************************************
 *	Class:			LISTCLASS
 *	SuperClass:		ROOTCLASS
 *	Description:
 *
 *	New Methods:
 *
 *		LISTM_AddHead,		Insert struct Node at beginning of list
 *		LISTM_AddTail,		Insert struct Node at end of list
 *		LISTM_Enqueue,		Insert struct Node in list by priority (FIFO)
 *		LISTM_Insert,		Insert struct Node after node in list
 *
 *		LISTM_RemHead,		Remove first node from list
 *		LISTM_RemTail,		Remove last node from list
 *
 *		LISTM_FindName,		Find the node with name
 *
 *		LISTM_FindNum,		Find Node number N
 *		LISTM_NodePos,		Get Node's position
 *		LISTM_GetLen,		Count the number of items in the list
 *
 *		LISTM_BroadCast,	Broadcast message to all nodes
 *		LISTM_BroadCastA,	Broadcast message to all nodes
 *
 *	Changed Methods:
 *
 *	Attributes:
 *
 *****************************************************************************
 *
 */

/**********************************************************/

/* Methods for my class objects
 */
enum
{	LISTM_Dummy = LISTCLASS_BASE,

	LISTM_AddHead,
	LISTM_AddTail,
	LISTM_Enqueue,
	LISTM_Insert,

	LISTM_RemHead,
	LISTM_RemTail,

	LISTM_FindName,

	LISTM_FindNum,
	LISTM_NodePos,
	LISTM_GetLen,

	LISTM_BroadCast,
	LISTM_BroadCastA,
};

/* Method structs for my class objects
 */

/*
 * LISTM_AddHead
 * LISTM_AddTail
 * LISTM_Enqueue
 */
struct opLAdd
{	ULONG			 MethodID;
	struct Node		*la_node;
};
#define ob_AddHead(l,n)		ob_DoMethod((l),LISTM_AddHead,(n))
#define ob_AddTail(l,n)		ob_DoMethod((l),LISTM_AddTail,(n))
#define ob_Enqueue(l,n)		ob_DoMethod((l),LISTM_Enqueue,(n))

#define ob_RemHead(l)		((struct Node *)ob_DoMethod((l),LISTM_RemHead))
#define ob_RemTail(l)		((struct Node *)ob_DoMethod((l),LISTM_RemTail))

/*
 * LISTM_Insert
 */
struct opLIns
{	ULONG			 MethodID;
	struct Node		*li_node;
	struct Node		*li_pred;
};
#define ob_Insert(l,n,p)	ob_DoMethod((l),LISTM_Insert,(n),(p))

/*
 * LISTM_FindName
 */
struct opLFind
{	ULONG			 MethodID;
	STRPTR			 lf_nodename;
};
#define ob_FindName(l,s)	((struct Node *)ob_DoMethod((l),LISTM_FindName,(s)))

/*
 * LISTM_FindNum
 */
struct opLFindNum
{	ULONG			 MethodID;
	ULONG			 lf_nodenum;
};
#define ob_NodeNum(l,n)	((struct Node *)ob_DoMethod((l),LISTM_FindNum,(n)))

/*
 * LISTM_NodePos
 */
struct opLNodePos
{	ULONG			 MethodID;
	struct Node		*lf_node;
};

#define ob_NodePos(l,n)	(LONG) ob_DoMethod((l),LISTM_NodePos,(n))

/*
 * LISTM_GetLen
 */
#define ob_ListLen(l)	ob_DoMethod((l),LISTM_GetLen)

/*
 * LISTM_BroadCast
 */
struct opLBroadCast
{	ULONG			 MethodID;
	ULONG			 m_MethodID;
};

/*
 * LISTM_BroadCastA
 */
struct opLBroadCastA
{	ULONG			 MethodID;
	Msg				 m_Msg;
};

/**********************************************************/
/*****************************************************************************
 *	Class:			HASHTABLECLASS
 *	SuperClass:		ROOTCLASS
 *	Description:
 *
 *	New Methods:
 *
 *		HTM_ADDNODE,	Add a hashnode to the hash table
 *
 *		HTM_FINDNODE,	Find a hashnode with the same Key
 *
 *		HTM_REMNODE,	Remove a hashnode from the table
 *
 *		HTM_RESET,		Clears the hashtable to have no included nodes
 *
 *		HTM_BROADCAST,	Broadcast message to all nodes in table
 *		HTM_BROADCASTA,	Broadcast message to all nodes in table
 *
 *	Changed Methods:
 *
 *	Attributes:
 *
 *		HTA_NumChains,	(I) The number of chains in the hashtable
 *
 *****************************************************************************
 *
 */

/* Methods for hashtable class objects
 */

struct HashNode
{
	struct HashNode	*hn_Succ;
	ULONG			 hn_Key;
};

enum
{	HTM_Dummy = HASHTABLECLASS_BASE,

	HTM_ADDNODE,
	/*
	 *		Add a hashnode to the hash table
	 */

	HTM_FINDNODE,
	/*
	 *		Find a hashnode with the same Key
	 */

	HTM_REMNODE,
	/*
	 *		Remove a hashnode from the table
	 */

	HTM_RESET,
	/*
	 *		Clears the hashtable to have no included nodes
	 */

	HTM_BROADCAST,
	HTM_BROADCASTA,
	/*
	 *		sends a method to all nodes in the hashtable
	 *		it will be safe to dispose each object during
	 *		the traversal.
	 */
};

/*
 *	HTM_ADDNODE
 *	HTM_REMNODE
 */
struct opHTAdd
{	ULONG					 MethodID;
	struct HashNode	*hta_node;
};

/*
 *	HTM_FINDNODE
 */
struct opHTFind
{	ULONG					 MethodID;
	ULONG					 htf_key;
};

/*
 *	HTM_BROADCAST
 */
struct opHTBroadcast
{	ULONG					 MethodID;
	ULONG					 htb_command;

	/* other items after command occur here */
};

/*
 *	HTM_BROADCASTA
 */
struct opHTBroadcastA
{	ULONG					 MethodID;
	APTR					 htb_msg;
};

/* Attributes for hashtable class objects
 */
enum
{	HTA_Dummy = ((LONG)TAG_USER | HASHTABLECLASS_BASE),

	HTA_NumChains,		/* (I) The number of chains in the hashtable */
};

/**********************************************************/

/*****************************************************************************
 *	Class:			KEYTABLECLASS
 *	SuperClass:		HASHTABLECLASS
 *	Description:
 *
 *		The keytable class is a subclass of the hashtable class. Its main
 *		uses are for quickly accessing objects of unique ID's that for one
 *		reason or another cannot have a HashNode embedded at the beginning
 *		of their structures.
 *
 *		One example use can be for performing quick finds on large, sparse
 *		tables with unique IDs.
 *
 *		By supplying a hashing/compare function, the table can be used for
 *		indexing data by their identifier strings.
 *
 *	New Methods:
 *
 *		KTM_ADDKEY			- Adds a Key/ULONG pair to the table
 *								Returns the ULONG if successful
 *
 *		KTM_REMKEY			- Removes a Key/ULONG pair from the table
 *
 *		KTM_FINDKEY			- Finds ULONG data for key in the table
 *								Returns the ULONG paired with key
 *
 *	Changed Methods:
 *
 *	Attributes:
 *
 *		KTA_HashFunc,		(I) Hashing function performed on key add/find
 *
 *			ULONG ASM (*hashfunc)	( REG(d0) ULONG key );
 *
 *			EXAMPLE of s string hash function:
 *
 *			static ULONG ASM SAVEDS my_hashfunc ( REG(d0) ULONG key )
 *			{
 *				char *ch = (char *)key;
 *				key = 0;
 *
 *				while (*ch)
 *				{
 *					key += (*ch << 4) + *ch;
 *					ch++;
 *				}
 *				return(key);
 *			}
 *
 *		KTA_CompareFunc,	(I) Conflict resolution function
 *
 *			LONG  ASM (*cmpfunc)	( REG(d0) ULONG key, REG(d1) ULONG data );
 *
 *			EXAMPLE of a string resolution function:
 *
 *			static LONG ASM SAVEDS my_cmpfunc ( REG(d0) ULONG key, REG(d1) ULONG data )
 *			{
 *				struct MyStruct	*ms = (struct MyStruct *)data;
 *				return(strcmp(ms->ms_name,(char *)key));
 *			}
 *
 *****************************************************************************
 *
 */

/* Methods for keytable class objects
 */
struct KeyTableNode
{
	struct HashNode	kn_hn;
	ULONG			kn_data;
};

enum
{	KTM_Dummy = KEYTABLECLASS_BASE,

	KTM_ADDKEY,
	KTM_REMKEY,
	KTM_FINDKEY,
};

/*
 *	KTM_ADDKEY
 */
struct opKTAdd
{	ULONG					 MethodID;
	ULONG					 kta_key;
	ULONG					 kta_data;
};

/*
 *	KTM_FINDKEY
 *	KTM_REMKEY
 */
struct opKTFind
{	ULONG					 MethodID;
	ULONG					 ktf_key;
};

#define ob_AddKey(ob,key,data)	ob_DoMethod((ob),KTM_ADDKEY,(key),(data))
#define ob_FindKey(ob,key)		ob_DoMethod((ob),KTM_FINDKEY,(key))
#define ob_RemKey(ob,key)		ob_DoMethod((ob),KTM_REMKEY,(key))
#define ob_RemoveKey(ob,key)	ob_DoMethod((ob),KTM_REMKEY,(key))

/* Attributes for keytable class objects
 */
enum
{	KTA_Dummy = ((LONG)TAG_USER | KEYTABLECLASS_BASE),

	KTA_HashFunc,			/* Hash function for addkey/findkey */

	KTA_CompareFunc,		/* Conflict resolution function */
};

/**********************************************************/
/*****************************************************************************
 *	Class:			PROXYCLASS
 *	SuperClass:		ROOTCLASS
 *	Description:
 *
 *	New Methods:
 *
 *	Changed Methods:
 *
 *	Attributes:
 *
 *****************************************************************************
 *
 */

/* Methods for proxy class objects
 */
enum
{	PROXYM_Dummy = PROXYCLASS_BASE,

	PROXYM_DISPOSE_ONLYTARGET,	/* Only dispose Target object */

	PROXYM_DISPOSE_ONLYSELF,	/* Only dispose yourself */
};

/* Attributes for proxy class objects
 */
enum
{	PROXYA_Dummy = ((LONG)TAG_USER | PROXYCLASS_BASE),

	PROXYA_TargetObject,		/* (I) The Object we forward messages to */
};

/**********************************************************/
/*****************************************************************************
 *	Class:			GRIDCLASS
 *	SuperClass:		ROOTCLASS
 *	Description:
 *
 *	New Methods:
 *
 *	Changed Methods:
 *
 *	Attributes:
 *
 *****************************************************************************
 *
 */

/* Methods for grid class objects
 */
enum
{	GRIDM_Dummy = GRIDCLASS_BASE,

	GRIDM_ADDROW,				/* Add a row to a grid */

	GRIDM_GETROWPTR,			/* Get row pointer */

	GRIDM_RESET,				/* Clears all items in grid (re-initializes) */
};

/*
 *	GRIDM_GETROWPTR
 */
struct opGridRow
{	ULONG					 MethodID;
	ULONG					 gr_row;
};

/* Attributes for grid class objects
 */
enum
{	GRIDA_Dummy = ((LONG)TAG_USER | GRIDCLASS_BASE),

	GRIDA_StructSize,			/* size of structure item */
};

#endif /* PROOF_CLASSUSR_H */
