#ifndef PROOF_OS_H
#define PROOF_OS_H
/*
 * $Id: proof_os.h 5.4 1995/10/09 21:57:38 pfrench Exp $
 *
 * $Source: n:/rcs/include/proof/proof_os.h $
 *
 * $Log: proof_os.h $
 * Revision 5.4  1995/10/09 21:57:38  pfrench
 * Finally bit the bullet and long aligned Node data structure element
 * ln_Name because there is too much Win OS code that depends on longwords
 * being automatically packed to a four byte boundary by the compiler
 * Revision 5.3  1995/07/10 22:37:06  pfrench
 * Moved proof includes to new directory.
 * Revision 5.2  1995/04/02 02:20:56  pfrench
 * Fixed problem with beWORD
 * Revision 5.1  1995/04/02 02:08:30  pfrench
 * Added bigendian-littleendian data types
 * Revision 5.0  1995/03/22 21:55:52  pfrench
 * Initial revision
 *
 */

#ifdef AMIGA

	#ifndef ASM
		#define REG(x)	register __##x
		#define ASM __asm
		#define STDARGS __stdargs
		#define SAVEDS __saveds
	#endif

	#ifndef EXEC_TYPES_H
	#include <exec/types.h>
	#endif

	#ifndef EXEC_MEMOFY_H
	#include <exec/memofy.h>
	#endif

	#ifndef EXEC_NODES_H
	#include <exec/nodes.h>
	#endif

	#ifndef EXEC_LISTS_H
	#include <exec/lists.h>
	#endif

	#ifndef DOS_DOS_H
	#include <dos/dos.h>
	#endif

	#ifndef UTILITY_TAGITEM_H
	#include <utility/tagitem.h>
	#endif

	#ifndef	EXEC_LIBRARIES_H
	#include <exec/libraries.h>
	#endif

	typedef struct { unsigned char ch[4]; } leLONG;
	typedef struct { unsigned char ch[4]; } leULONG;
	typedef struct { unsigned char ch[2]; } leWORD;
	typedef struct { unsigned char ch[2]; } leUWORD;

	typedef signed long		beLONG;
	typedef unsigned long	beULONG;
	typedef signed short  	beWORD;
	typedef unsigned short	beUWORD;

#else	/* NOT_AMIGA */

/* ***********************************************************************
 *
 *		PROOF KERNEL DEFINITIONS GO HERE!!
 *
 * ***********************************************************************
 *
 */

#ifndef ASM
	#define REG(x)
	#define ASM
	#define STDARGS
	#define SAVEDS
#endif

#ifndef EXEC_TYPES_H
	#define	EXEC_TYPES_H

	#ifndef APTR_TYPEDEF
		#define APTR_TYPEDEF
		typedef void	       *APTR;	    /* 32-bit untyped pointer */
	#endif

	typedef signed long		LONG;	    /* signed 32-bit quantity */
	typedef unsigned long	ULONG;	    /* unsigned 32-bit quantity */
	typedef unsigned long	LONGBITS;   /* 32 bits manipulated individually */
	typedef unsigned short	UWORD;	    /* unsigned 16-bit quantity */
	typedef unsigned short	WORDBITS;   /* 16 bits manipulated individually */

	typedef unsigned char	UBYTE;	    /* unsigned 8-bit quantity */
	typedef unsigned char	BYTEBITS;   /* 8 bits manipulated individually */
	typedef unsigned short	RPTR;	    /* signed relative pointer */
	typedef unsigned char  *STRPTR;     /* string pointer (NULL terminated) */

	#ifndef TRUE
	#define TRUE		1
	#endif
	#ifndef FALSE
	#define FALSE		0
	#endif
	#ifndef NULL
	#define NULL		0L
	#endif

	typedef struct { unsigned char ch[4]; } beLONG;
	typedef struct { unsigned char ch[4]; } beULONG;
	typedef struct { unsigned char ch[2]; } beWORD;
	typedef struct { unsigned char ch[2]; } beUWORD;

	typedef signed long		leLONG;
	typedef unsigned long	leULONG;
	typedef signed short  	leWORD;
	typedef unsigned short	leUWORD;

#endif

#ifndef	EXEC_NODES_H
	#define	EXEC_NODES_H
	/*
	 *  List Node Structure.  Each member in a list starts with a Node
	 */
	struct Node {
	    struct  Node *ln_Succ;	/* Pointer to next (successor) */
	    struct  Node *ln_Pred;	/* Pointer to previous (predecessor) */
	    unsigned char ln_Type;
	    signed char	ln_Pri;		/* Priority, for sorting */

#ifndef AMIGA
		/* Here because of VC's Default structure packing super-nastiness.
		 * The entire Win OS has includes all over that depend on packing values of 4!
		 * I submit this hack woefully!! (piss, moan)
		 */
		unsigned short	ln_win_hack_flags;
#endif
	    char    *ln_Name;		/* ID string, null terminated */
	};	/* Note: word aligned */

	/* minimal node -- no type checking possible */
	struct MinNode {
	    struct MinNode *mln_Succ;
	    struct MinNode *mln_Pred;
	};

	/*
	** Note: Newly initialized IORequests, and software interrupt structures
	** used with Cause(), should have type NT_UNKNOWN.  The OS will assign a type
	** when they are first used.
	*/
	/*----- Node Types for LN_TYPE -----*/
	#define NT_UNKNOWN	0
	#define NT_TASK		1	/* Exec task */
	#define NT_INTERRUPT	2
	#define NT_DEVICE	3
	#define NT_MSGPORT	4
	#define NT_MESSAGE	5	/* Indicates message currently pending */
	#define NT_FREEMSG	6
	#define NT_REPLYMSG	7	/* Message has been replied */
	#define NT_RESOURCE	8
	#define NT_LIBRARY	9
	#define NT_MEMORY	10
	#define NT_SOFTINT	11	/* Internal flag used by SoftInits */
	#define NT_FONT		12
	#define NT_PROCESS	13	/* AmigaDOS Process */
	#define NT_SEMAPHORE	14
	#define NT_SIGNALSEM	15	/* signal semaphores */
	#define NT_BOOTNODE	16
	#define NT_KICKMEM	17
	#define NT_GRAPHICS	18
	#define NT_DEATHMESSAGE	19

	#define NT_USER		254	/* User node types work down from here */
	#define NT_EXTENDED	255

#endif /* EXEC_NODES_H */

#ifndef EXEC_LISTS_H
	#define EXEC_LISTS_H
	/*
	 *  Full featured list header.
	 */
	struct List {
	   struct  Node *lh_Head;
	   struct  Node *lh_Tail;
	   struct  Node *lh_TailPred;
	   UBYTE   lh_Type;
	   UBYTE   l_pad;
#ifndef AMIGA
		/* See note in Node above! */
		unsigned short l_win_hack_flags;
#endif
	};	/* word aligned */

	/*
	 * Minimal List Header - no type checking
	 */
	struct MinList {
	   struct  MinNode *mlh_Head;
	   struct  MinNode *mlh_Tail;
	   struct  MinNode *mlh_TailPred;
	};	/* longword aligned */

	/*
	 *	Check for the presence of any nodes on the given list.	These
	 *	macros are even safe to use on lists that are modified by other
	 *	tasks.	However; if something is simultaneously changing the
	 *	list, the result of the test is unpredictable.
	 *
	 *	Unless you first arbitrated for ownership of the list, you can't
	 *	_depend_ on the contents of the list.  Nodes might have been added
	 *	or removed during or after the macro executes.
	 *
	 *		if( IsListEmpty(list) )		printf("List is empty\n");
	 */
	#define IsListEmpty(x) \
		( ((x)->lh_TailPred) == (struct Node *)(x) )

	#define IsMsgPortEmpty(x) \
		( ((x)->mp_MsgList.lh_TailPred) == (struct Node *)(&(x)->mp_MsgList) )

#endif	/* EXEC_LISTS_H */

#ifndef EXEC_MEMOFY_H
	#define EXEC_MEMOFY_H

	/****** MemChunk ****************************************************/

	struct	MemChunk {
	    struct  MemChunk *mc_Next;	/* pointer to next chunk */
	    ULONG   mc_Bytes;		/* chunk byte size	*/
	};


	/****** MemHeader ***************************************************/

	struct	MemHeader {
	    struct  Node mh_Node;
	    UWORD   mh_Attributes;	/* characteristics of this region */
	    struct  MemChunk *mh_First; /* first free region		*/
	    APTR    mh_Lower;		/* lower memory bound		*/
	    APTR    mh_Upper;		/* upper memory bound+1	*/
	    ULONG   mh_Free;		/* total number of free bytes	*/
	};

	/*----- Memory Requirement Types ---------------------------*/
	/*----- See the AllocMem() documentation for details--------*/

	#define MEMF_ANY    (0L)	/* Any type of memory will do */
	#define MEMF_PUBLIC (1L<<0)
	#define MEMF_CHIP   (1L<<1)
	#define MEMF_FAST   (1L<<2)
	#define MEMF_LOCAL  (1L<<8)	/* Memory that does not go away at RESET */
	#define MEMF_24BITDMA (1L<<9)	/* DMAable memory within 24 bits of address */
	#define	MEMF_KICK   (1L<<10)	/* Memory that can be used for KickTags */

	#define MEMF_CLEAR   (1L<<16)	/* AllocMem: NULL out area before return */
	#define MEMF_LARGEST (1L<<17)	/* AvailMem: return the largest chunk size */
	#define MEMF_REVERSE (1L<<18)	/* AllocMem: allocate from the top down */
	#define MEMF_TOTAL   (1L<<19)	/* AvailMem: return total size of memory */

	#define	MEMF_NO_EXPUNGE	(1L<<31) /*AllocMem: Do not cause expunge on failure */

	/*----- Current alignment rules for memory blocks (may increase) -----*/
	#define MEM_BLOCKSIZE	8L
	#define MEM_BLOCKMASK	(MEM_BLOCKSIZE-1)

#endif

#ifndef DOS_DOS_H
	#define DOS_DOS_H

	/* Mode parameter to Open() */
	#define MODE_OLDFILE	     1005   /* Open existing file read/write
									     * positioned at beginning of file. */
	#define MODE_NEWFILE	     1006   /* Open freshly created file (delete
									     * old file) read/write, exclusive lock. */
	#define MODE_READWRITE	     1004   /* Open old file w/shared lock,
									     * creates file if doesn't exist. */

	/* Relative position to Seek() */
	#define OFFSET_BEGINNING	-1	    /* relative to Begining Of File */
	#define OFFSET_CURRENT		 0	    /* relative to Current file position */
	#define OFFSET_END			 1	    /* relative to End Of File	  */

	/* All BCPL data must be long word aligned.  BCPL pointers are the long word
	 *  address (i.e byte address divided by 4 (>>2)) */
	typedef long  BPTR;		    /* Long word pointer */
#endif	/* DOS_DOS_H */

#ifndef UTILITY_TAGITEM_H
	#define UTILITY_TAGITEM_H

	/*****************************************************************************/

	/* Tags are a general mechanism of extensible data arrays for parameter
	 * specification and property inquiry. In practice, tags are used in arrays,
	 * or chain of arrays.
	 *
	 */

	typedef ULONG Tag;

	struct TagItem
	{
	    Tag	  ti_Tag;	/* identifies the type of data */
	    ULONG ti_Data;	/* type-specific data	       */
	};

	/* constants for Tag.ti_Tag, system tag values */
	#define TAG_DONE   (0L)	  /* terminates array of TagItems. ti_Data unused */
	#define TAG_END	   (0L)   /* synonym for TAG_DONE			  */
	#define	TAG_IGNORE (1L)	  /* ignore this item, not end of array		  */
	#define	TAG_MORE   (2L)	  /* ti_Data is pointer to another array of TagItems
							   * note that this tag terminates the current array
							   */
	#define	TAG_SKIP   (3L)	  /* skip this and the next ti_Data items	  */

	/* Indication of user tag, OR this in with user tag values */
	#define TAG_USER   (1L<<31)  /* differentiates user tags from system tags */

	/* NOTE: Until further notice, tag bits 16-30 are RESERVED and should be zero.
	 *	 Also, the value (TAG_USER | 0) should never be used as a tag value.
	 */

	/*****************************************************************************/

	/* Tag filter logic specifiers for use with FilterTagItems() */
	#define TAGFILTER_AND 0		/* exclude everything but filter hits	*/
	#define TAGFILTER_NOT 1		/* exclude only filter hits		*/


	/*****************************************************************************/

	/* Mapping types for use with MapTags() */
	#define MAP_REMOVE_NOT_FOUND 0	/* remove tags that aren't in mapList */
	#define MAP_KEEP_NOT_FOUND   1	/* keep tags that aren't in mapList   */

	/*****************************************************************************/

	/* Merging types for use with MergeTagItems() */
	#define MERGE_OR_LIST_1		0	/* list 1's item is preferred	     */
	#define MERGE_OR_LIST_2		1	/* list 2's item is preferred	     */
	#define MERGE_AND_LIST_1	2	/* item must appear in both lists    */
	#define MERGE_AND_LIST_2	3	/* item must appear in both lists    */
	#define MERGE_NOT_LIST_1	4	/* item must not appear in list 1    */
	#define MERGE_NOT_LIST_2	5	/* item must not appear in list 2    */
	#define MERGE_XOR			6	/* item must appear in only one list */

#endif /* UTILITY_TAGITEM_H */

#endif	/* AMIGA */
#endif	/* PROOF_OS_H */
