/*
 * $Id: main.c 5.3 1995/10/02 17:25:52 pfrench Exp $
 *
 * $Source: n:/rcs/newtek/iff2clip/src/main.c $
 *
 * $Log: main.c $
 * Revision 5.3  1995/10/02 17:25:52  pfrench
 * Removed un-used local variable from LoadNextBM
 * Revision 5.2  1995/09/21 21:34:04  pfrench
 * Added banner
 * Revision 5.1  1995/09/21 18:58:06  pfrench
 * Fixed BitMap memory leak.  Cleaned up source a little
 * Revision 5.0  1995/09/21 18:48:02  pfrench
 * Initial revision
 */

#ifndef PROOF_LIB_H
#include <proof/proof_lib.h>
#endif

#ifndef PROOF_CLASSBASE_H
#include <proof/classbase.h>
#endif

#ifndef PROOF_STREAM_H
#include <proof/stream.h>
#endif

#ifndef NEWTEK_BITMAP_H
#include <newtek/bitmap.h>
#endif

#ifndef NEWTEK_FRAME_H
#include <newtek/frame.h>
#endif

#ifndef NEWTEK_COMP_CONVERT_H
#include <newtek/comp_convert.h>
#endif

#include <newtek/nt_tools_protos.h>
#include <newtek/flyclip_protos.h>

#include <stdio.h>

struct ClipConvert
{
	/* Flyer Clip we're outputing to */
	struct FlyClipInfo		*cc_fc;				/* FlyerClip handle */

	struct Frame			*cc_fr;				/* Single D2 field */

	struct BitMap			*cc_bm;				/* 24-bit frame bitmap */

	struct comp_convert		*cc_cc;				/* For RGB to D2 encoding */

	long					 cc_cur_field;		/* Field we're processing */
	long					 cc_frames;			/* Frames remaining in clip */

	long					 cc_src_start;		/* Start Frame to process */

	long					 cc_imagebaselen;	/* For building frame path */
	char					 cc_src_image[512];
};

struct ClassBase	*ClassBase;

static void conv_Delete(
	struct ClipConvert	*cc )
{
	if ( cc->cc_bm )
		bm_Delete(cc->cc_bm);

	if ( cc->cc_fr )
		fr_Delete(cc->cc_fr);

	if ( cc->cc_cc )
		cc_Delete(cc->cc_cc);

	ob_Dispose(cc);
}

static struct ClipConvert *conv_Create(
	APTR		mempool )
{
	struct ClipConvert *cc;

	if ( cc = (struct ClipConvert *)ob_AllocObj(mempool,sizeof(*cc),MEMF_CLEAR) )
	{
		char		ok = 0;

		/* Get a RGB-D2 converter */
		if ( cc->cc_cc = cc_Create(mempool,TOASTER_D2_WIDTH) )
		{
			if ( cc->cc_fr = fr_Create(mempool,
					TOASTER_D2_WIDTH,TOASTER_D2_HEIGHT,1) )
			{
				ok = 1;
			}
		}

		if ( !ok )
		{
			conv_Delete(cc);
			cc = NULL;
		}
	}

	return(cc);
}

static long cc_LoadNextBM(
	struct ClipConvert	*cc )
{
	long		err = 1;
	APTR		fh;

	if ( cc->cc_bm )
	{
		bm_Delete(cc->cc_bm);
		cc->cc_bm = NULL;
	}

	sprintf( &cc->cc_src_image[cc->cc_imagebaselen],"%03ld",
		cc->cc_src_start + (cc->cc_cur_field >> 1) );

	if ( fh = ob_BufferedOpen(cc->cc_src_image,MODE_OLDFILE,1024*2) )
	{
		struct LoadILBMParams	lip;

		/* Clear It Out */
		memset(&lip,0,sizeof(lip));

		if ( lip_load(&lip,fh) )
		{
			if ( lip.lip_colors )
				ob_Dispose(lip.lip_colors);

			if ( lip.lip_bm )
			{
				/* Only convert 24 bit bitmaps */
				if ( (lip.lip_bmhd.nPlanes == 24) &&
					 (lip.lip_bmhd.w == TOASTER_D2_WIDTH) &&
					 (lip.lip_bmhd.h == (TOASTER_D2_HEIGHT << 1)) )
				{
					cc->cc_bm = lip.lip_bm;
					err = 0;
				}
				else
				{
					bm_Delete(lip.lip_bm);
				}
			}
		}

		ob_Dispose(fh);
	}

	return(err);
}

static long cc_ConvertField(
	struct ClipConvert	*cc,
	long				 odd )
{
	long			err = 0;
	unsigned short	row;
	unsigned long	plane_offset = 0;
	struct BitMap	*bm = cc->cc_bm;

	cc->cc_cc->comp_buf = cc->cc_fr->fr_d2_fld_1;

	if ( odd )
		plane_offset += bm->BytesPerRow;

	for ( row = 0; row < TOASTER_D2_HEIGHT; row++ )
	{
		/* Red Pixels */
		memset(cc->cc_cc->r_buf,0,TOASTER_D2_WIDTH);

		PlanarToChunky(
			&bm->Planes[0],
			plane_offset,8,
			cc->cc_cc->r_buf,
			TOASTER_D2_WIDTH);


		/* Green Pixels */
		memset(cc->cc_cc->g_buf,0,TOASTER_D2_WIDTH);

		PlanarToChunky(
			&bm->Planes[8],
			plane_offset,8,
			cc->cc_cc->g_buf,
			TOASTER_D2_WIDTH);

		/* Blue Pixels */
		memset(cc->cc_cc->b_buf,0,TOASTER_D2_WIDTH);

		PlanarToChunky(
			&bm->Planes[16],
			plane_offset,8,
			cc->cc_cc->b_buf,
			TOASTER_D2_WIDTH);

		/* Encode RGB into D2 */
		cc_full_encode(cc->cc_cc,NULL,ODD_SCANLINE(cc->cc_cur_field,row));

		cc->cc_cc->comp_buf += cc->cc_fr->fr_field_width;

		/* Move down two scanlines since we're interlaced */
		plane_offset += (bm->BytesPerRow << 1);
	}

	return(err);
}

static long cc_ConvertFrames(
	struct ClipConvert	*cc )
{
	long			err = 0;

	while ( cc->cc_frames-- )
	{
		if ( err = cc_LoadNextBM(cc) )
		{
			break;
		}

		printf("Frame: %03ld  Src: \"%s\"\n",
			(cc->cc_cur_field >> 1),cc->cc_src_image );

		/* Convert the first interlaced field to the D2 field */
		cc_ConvertField(cc,0);

		if ( err = fc_WriteVideoField(cc->cc_fc,
				cc->cc_cur_field,cc->cc_fr->fr_d2_fld_1) )
		{
			break;
		}

		cc->cc_cur_field++;

		/* Convert the second interlaced field to the D2 field */
		cc_ConvertField(cc,1);

		if ( err = fc_WriteVideoField(cc->cc_fc,
				cc->cc_cur_field,cc->cc_fr->fr_d2_fld_1) )
		{
			break;
		}

		cc->cc_cur_field++;
	}

	return(err);
}

int main(
	int		argc,
	char	**argv )
{
	int			err = 0;

	ClassBase = pb_GetClassBase();

	printf("IFF2CLIP by Pat Brouillette  Ver 0.1 \n");
	printf("Copyright (c) 1995 NewTek, Inc.\n");
	printf("Confidential and Proprietary.  Do not distribute.\n");

	if ( argc == 5 )
	{
		struct ClipConvert	*cc;

		if ( cc = conv_Create(ClassBase->cb_allocator_ob) )
		{
			cc->cc_src_start = atol(argv[3]);
			cc->cc_frames = atol(argv[4]);
			strcpy(cc->cc_src_image,argv[1]);
			cc->cc_imagebaselen = strlen(cc->cc_src_image);

			/* Open the flyer clip */
			if ( cc->cc_fc = fc_Open(argv[2],MODE_NEWFILE) )
			{
				err = cc_ConvertFrames(cc);

				fc_Close(cc->cc_fc);
			}

			conv_Delete(cc);
		}
	}
	else
	{
		printf("Usage: %s <source_frames_basename> <flyer_clip_name> <startframe> <frames> \n",argv[0] );
	}

	return(err);
}
