/*
 * $Id: lw_tiler.c 5.1 1995/09/18 17:35:06 pfrench Exp pfrench $
 *
 * $Source: n:/rcs/newtek/fx_compile/lw_tiler.c $
 *
 *	Effects Tiler LightWave Plug-in
 *	Pat Brouillette
 *	NewTek, Inc.
 *	Copyright (c) 1995, NewTek, Inc. All Rights Reserved
 *
 *	Tiles all the available buffers in LightWave in the pattern
 *
 *
 *		RGB			alpha		RAW RGB		Special
 *								
 *
 *		Luminous	Diffuse		Specular	Mirror
 *
 *
 *		Trans		Shading		Shadow		Geometry
 *
 * $Log: lw_tiler.c $
 * Revision 5.1  1995/09/18 17:35:06  pfrench
 * Added provision to tile four most used buffers for effects
 * Revision 5.0  1995/07/26 19:55:44  pfrench
 * Initial revision
 *
 */
#include <lw/splug.h>
#include <lw/moni.h>
#include <lw/lwran.h>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define JUST_MY_FOUR_BUFFERS

#ifdef JUST_MY_FOUR_BUFFERS
	#define HORIZ_TILES	2
	#define VERT_TILES	2
#else
	#define HORIZ_TILES	4
	#define VERT_TILES	3
#endif


/*
 * HANDLER -- Instance Methods
 *
 * An instance is a particular set of user-set options for a plug-in
 * operation.  Normally the host would call these callback functions
 * to manage instances, but since the Negative image filter has no
 * options, only a dummy instance needs to be used.  In this case the
 * pointer for all instances will just be a pointer to a string and 
 * the instance handler functions will do nothing.
 */
static char		 dummyInst[] = "NIL";


	XCALL_(LWInstance)
TilerCreate (
	LWError			*err)
{
	return ((LWInstance) dummyInst);
}

	XCALL_(void)
TilerDestroy (
	LWInstance		 inst)
{
}

	XCALL_(LWError)
TilerCopy (
	LWInstance		 from,
	LWInstance		 to)
{
	return ((LWInstance) dummyInst);
}

	XCALL_(LWError)
TilerLoad (
	LWInstance		 inst,
	const LWLoadState	*lState)
{
	return (NULL);
}

	XCALL_(LWError)
TilerSave (
	LWInstance		 inst,
	const LWSaveState	*sState)
{
	return (NULL);
}

	XCALL_(static void)
DoTile(
	const FilterAccess	*fa,
	int					 bufnum,
	int					 dest_x,
	int					 dest_y,
	int					 mode_3 )
{
	int	 tile_w = fa->width / HORIZ_TILES;
	int	 tile_h = fa->height / VERT_TILES;
	int	 x,y;

	for ( y = 0; y < tile_h; y++ )
	{
		BufferValue		rgbout[4];

		if ( mode_3 )
		{
			BufferValue	*buf_1,*buf_2,*buf_3;

			buf_1 = (*fa->bufLine)(bufnum+0,(y * VERT_TILES));
			buf_2 = (*fa->bufLine)(bufnum+1,(y * VERT_TILES));
			buf_3 = (*fa->bufLine)(bufnum+2,(y * VERT_TILES));

			for ( x = 0; x < tile_w; x++ )
			{
				rgbout[0] = *buf_1;
				rgbout[1] = *buf_2;
				rgbout[2] = *buf_3;

				(*fa->setRGB)( dest_x+x, dest_y+y, rgbout );
//				(*fa->setAlpha)( dest_x+x, dest_y+y, 255 );

				buf_1 += HORIZ_TILES;
				buf_2 += HORIZ_TILES;
				buf_3 += HORIZ_TILES;
			}
		}
		else
		{
			BufferValue	*buf;

			buf = (*fa->bufLine)(bufnum,(y * VERT_TILES));

			for ( x = 0; x < tile_w; x++ )
			{
				rgbout[0] = *buf;
				rgbout[1] = *buf;
				rgbout[2] = *buf;

				(*fa->setRGB)( dest_x+x, dest_y+y, rgbout );
//				(*fa->setAlpha)( dest_x+x, dest_y+y, 255 );

				buf += HORIZ_TILES;
			}
		}
	}
}



	XCALL_(static void)
TileProcess (
	LWInstance		 inst,
	const FilterAccess	*fa)
{
	Monitor	*mon;
	int		 tile_w = fa->width / HORIZ_TILES;
	int		 tile_h = fa->height / VERT_TILES;
	unsigned int my_buffers;

	mon = fa->monitor;

	my_buffers =
#ifdef JUST_MY_FOUR_BUFFERS
		(1 << LWBUF_SPECIAL) |
		(1 << LWBUF_SHADING) |
		(1 << LWBUF_SHADOW) |
		(1 << LWBUF_RED) |
		(1 << LWBUF_GREEN) |
		(1 << LWBUF_BLUE) |
		(1 << LWBUF_ALPHA);
#else
		(1 << LWBUF_SPECIAL) |
		(1 << LWBUF_LUMINOUS) |
		(1 << LWBUF_DIFFUSE) |
		(1 << LWBUF_SPECULAR) |
		(1 << LWBUF_MIRROR) |
		(1 << LWBUF_TRANS) |
		(1 << LWBUF_RAW_RED) |
		(1 << LWBUF_RAW_GREEN) |
		(1 << LWBUF_RAW_BLUE) |
		(1 << LWBUF_SHADING) |
		(1 << LWBUF_SHADOW) |
		(1 << LWBUF_GEOMETRY) |
		(1 << LWBUF_RED) |
		(1 << LWBUF_GREEN) |
		(1 << LWBUF_BLUE) |
		(1 << LWBUF_ALPHA);
#endif
	/*
	 * Scan through the lines in the image and get image buffer
	 * line pointers for the RGB and alpha buffers of the original
	 * image.
	 */
#ifdef JUST_MY_FOUR_BUFFERS
	MON_INIT (mon, 4);

	DoTile(fa, LWBUF_RED,			(0 * tile_w),(0 * tile_h),1);
	MON_STEP(mon);

	DoTile(fa, LWBUF_ALPHA,			(1 * tile_w),(0 * tile_h),0);
	MON_STEP(mon);

	if ( my_buffers & (1 << LWBUF_SPECIAL ) )
		DoTile(fa, LWBUF_SPECIAL,	(0 * tile_w),(1 * tile_h),0);
	MON_STEP(mon);

	if ( my_buffers & (1 << LWBUF_SHADOW ) )
		DoTile(fa, LWBUF_SHADOW,	(1 * tile_w),(1 * tile_h),0);
	MON_STEP(mon);
#else
	MON_INIT (mon, 12);
	// --------------

	DoTile(fa, LWBUF_RED,		(0 * tile_w),(0 * tile_h),1);
	MON_STEP(mon);

	DoTile(fa, LWBUF_ALPHA,		(1 * tile_w),(0 * tile_h),0);
	MON_STEP(mon);

	if ( my_buffers & (1 << LWBUF_RAW_RED ) )
		DoTile(fa, LWBUF_RAW_RED,	(2 * tile_w),(0 * tile_h),1);
	MON_STEP(mon);

	if ( my_buffers & (1 << LWBUF_SPECIAL ) )
		DoTile(fa, LWBUF_SPECIAL,	(3 * tile_w),(0 * tile_h),0);
	MON_STEP(mon);

	// --------------

	if ( my_buffers & (1 << LWBUF_LUMINOUS ) )
		DoTile(fa, LWBUF_LUMINOUS,	(0 * tile_w),(1 * tile_h),0);
	MON_STEP(mon);

	if ( my_buffers & (1 << LWBUF_DIFFUSE ) )
		DoTile(fa, LWBUF_DIFFUSE,	(1 * tile_w),(1 * tile_h),0);
	MON_STEP(mon);

	if ( my_buffers & (1 << LWBUF_SPECULAR ) )
		DoTile(fa, LWBUF_SPECULAR,	(2 * tile_w),(1 * tile_h),0);
	MON_STEP(mon);

	if ( my_buffers & (1 << LWBUF_MIRROR ) )
		DoTile(fa, LWBUF_MIRROR,	(3 * tile_w),(1 * tile_h),0);
	MON_STEP(mon);

	// --------------

	if ( my_buffers & (1 << LWBUF_TRANS ) )
		DoTile(fa, LWBUF_TRANS,		(0 * tile_w),(2 * tile_h),0);
	MON_STEP(mon);

	if ( my_buffers & (1 << LWBUF_SHADING ) )
		DoTile(fa, LWBUF_SHADING,	(1 * tile_w),(2 * tile_h),0);
	MON_STEP(mon);

	if ( my_buffers & (1 << LWBUF_SHADOW ) )
		DoTile(fa, LWBUF_SHADOW,	(2 * tile_w),(2 * tile_h),0);
	MON_STEP(mon);

	if ( my_buffers & (1 << LWBUF_GEOMETRY ) )
		DoTile(fa, LWBUF_GEOMETRY,	(3 * tile_w),(2 * tile_h),0);

#endif /* !JUST_MY_FOUR_BUFFERS */

	MON_STEP(mon);

	MON_DONE (mon);
}

	XCALL_(static unsigned int)
TileFlags (
	LWInstance		 inst)
{
	unsigned int my_buffers;

	my_buffers =
#ifdef JUST_MY_FOUR_BUFFERS
		(1 << LWBUF_RED) |
		(1 << LWBUF_GREEN) |
		(1 << LWBUF_BLUE) |
		(1 << LWBUF_SPECIAL) |
		(1 << LWBUF_SHADING) |
		(1 << LWBUF_SHADOW) |
		(1 << LWBUF_ALPHA);
#else
		(1 << LWBUF_SPECIAL) |
		(1 << LWBUF_LUMINOUS) |
		(1 << LWBUF_DIFFUSE) |
		(1 << LWBUF_SPECULAR) |
		(1 << LWBUF_MIRROR) |
		(1 << LWBUF_TRANS) |
		(1 << LWBUF_RAW_RED) |
		(1 << LWBUF_RAW_GREEN) |
		(1 << LWBUF_RAW_BLUE) |
		(1 << LWBUF_SHADING) |
		(1 << LWBUF_SHADOW) |
		(1 << LWBUF_GEOMETRY) |
		(1 << LWBUF_RED) |
		(1 << LWBUF_GREEN) |
		(1 << LWBUF_BLUE) |
		(1 << LWBUF_ALPHA);
#endif

	return ( my_buffers );
}

/*
 * ACTIVATE -- Main entry point
 *
 * The host calls this function with an uninitialized image filter handler
 * and the server has to fill in the callback fields for the handler
 * functions for this handler class.
 */
	XCALL_(int)
TileActivate (
	long			 version,
	GlobalFunc		*global,
	ImageFilterHandler	*local,
	void			*serverData)
{
	XCALL_INIT;
	if (version != 1)
		return (AFUNC_BADVERSION);

	local->inst.create  = TilerCreate;
	local->inst.destroy = TilerDestroy;
	local->inst.load    = TilerLoad;
	local->inst.save    = TilerSave;
	local->inst.copy    = TilerCopy;

	local->process = TileProcess;
	local->flags   = TileFlags;

	return (AFUNC_OK);
}

/*
 * This plug-in module contains two servers -- one for the shader handler
 * and one for the interface.  The blotch instance will be passed to
 * this interface server since it has the same name as the handler.  These
 * servers do not have to be in the same module if a render-only shader
 * plug-in is required.
 */
ServerRecord ServerDesc[] = {
	{ "ImageFilterHandler",		"NewTek-EffectsTile",	TileActivate },
	{ NULL }
};
