/*
 * $Id: newtek_avi.c 5.1 1995/08/16 19:53:34 pfrench Exp $
 *
 * $Source: n:/rcs/newtek/avi_p/newtek_avi.c $
 *
 * $Log: newtek_avi.c $
 * Revision 5.1  1995/08/16 19:53:34  pfrench
 * Many changes.  Mainly some minor bug fixes.  Also now writing
 * LightWave's suggested playback rate into the file
 * Revision 5.0  1995/08/16 08:35:26  pfrench
 * Initial revision
 */

#include <lw/splug.h>
#include <lw/moni.h>
#include <lw/lwmod.h>
#include <lw/lwran.h>
#include <lw/image.h>

#define  INC_OLE2
#include <windows.h>
#include <windowsx.h>
#include <memory.h>
#include <mmsystem.h>
#include <vfw.h>

#define AVI_USES_FRAMEBUFFER	FALSE

struct AVIHandle {
	BITMAPINFOHEADER	 avi_bi;
	unsigned char		*avi_pixbuf;
	long				 avi_pixbufsiz;
	long				 avi_pixlinesiz;

	long				 avi_frame;
	long				 avi_scanline;
	long				 avi_w;
	long				 avi_h;

	PAVIFILE			 avi_pfile;
	PAVISTREAM			 avi_ps;
	PAVISTREAM			 avi_psCompressed;
	AVISTREAMINFO		 avi_strhdr;
	AVICOMPRESSOPTIONS FAR *avi_aopts[1];
	AVICOMPRESSOPTIONS	 avi_opts;
};

/* ------------------------------------------------------------ */
static const char *app_version = 
	"NewTek AVI Saver Ver 1.0 "  __DATE__
	" by Pat Brouillette";

static GlobalFunc	*gGlobalFunc;

static double	gSceneFPS;
static int		gSceneFrameStart;
static int		gSceneFrameEnd;
static int		gSceneFrameStep;

/* ------------------------------------------------------------ */

static void NT_AVIGetSceneInfo(
	GlobalFunc	*global )
{
	LWSceneInfo		*scene;

	if ( scene = (*global) ("LW Scene Info", GFUSE_TRANSIENT) )
	{
		gSceneFPS			= scene->framesPerSecond;
		gSceneFrameStart	= scene->frameStart;
		gSceneFrameEnd		= scene->frameEnd;
		gSceneFrameStep		= scene->frameStep;
	}
}

XCALL_(static void) NT_AVIDestroy(
	LWInstance inst )
{
	struct AVIHandle	*avi = (struct AVIHandle *)inst;

	XCALL_INIT;

	free(avi);
}

XCALL_(LWInstance) NT_AVICreate(
	LWError *err)
{
	struct AVIHandle	*avi;

	XCALL_INIT;

	if ( avi = malloc(sizeof(*avi)) )
	{
		char		ok = 0;

		/* Clear it out first */
		memset(avi,0,sizeof(*avi));

		avi->avi_aopts[0] = &avi->avi_opts;

		ok = TRUE;

		if ( !ok )
		{
			NT_AVIDestroy((LWInstance)avi);
			avi = NULL;
		}
	}

	return ((LWInstance)avi);
}

XCALL_(static LWError) NT_AVILoad(
	LWInstance inst,
	const LWLoadState *Lstate )
{
	LWError				 err = NULL;
	struct AVIHandle	*avi = (struct AVIHandle *)inst;

	XCALL_INIT;

    return (err);
}

XCALL_(static LWError) NT_AVISave(
	LWInstance inst,
	const LWSaveState *Sstate)
{
	LWError				 err = NULL;
	struct AVIHandle	*avi = (struct AVIHandle *)inst;

	XCALL_INIT;

    return(err);
}

XCALL_(static LWError) NT_AVICopy(
	LWInstance from,
	LWInstance to)
{
	LWError				 err = NULL;
	XCALL_INIT;

	return(err);
}

XCALL_(static void) NT_AVIClose(
	LWInstance inst )
{
	struct AVIHandle	*avi = (struct AVIHandle *)inst;
	XCALL_INIT;

	if ( avi->avi_pixbuf )
	{
		free(avi->avi_pixbuf);
		avi->avi_pixbuf = NULL;
	}

	if ( avi->avi_ps )
	{
		AVIStreamClose(avi->avi_ps);
		avi->avi_ps = NULL;
	}

	if ( avi->avi_psCompressed )
	{
		AVIStreamClose(avi->avi_psCompressed);
		avi->avi_psCompressed = NULL;
	}

	if ( avi->avi_pfile )
	{
		AVIFileRelease(avi->avi_pfile);
		avi->avi_pfile = NULL;
	}

	if ( avi->avi_w )
	{
		AVIFileExit();
		avi->avi_w = 0;
	}
}

XCALL_(static LWError) NT_AVIOpen(
	LWInstance inst,
	int w,
	int h,
	const char *file )
{
	HRESULT				 hr;
	LWError				 err = NULL;
	struct AVIHandle	*avi = (struct AVIHandle *)inst;
	XCALL_INIT;

	if ( gGlobalFunc )
	{
		NT_AVIGetSceneInfo(gGlobalFunc);
	}

	avi->avi_w = w;
	avi->avi_h = h;
	avi->avi_pixlinesiz = (((avi->avi_w * 24) + 31 ) / 32) * 4;
	avi->avi_pixbufsiz = avi->avi_h * avi->avi_pixlinesiz;

	avi->avi_bi.biSize = sizeof(BITMAPINFOHEADER);
	avi->avi_bi.biWidth = avi->avi_w;
	avi->avi_bi.biHeight = avi->avi_h;
	avi->avi_bi.biPlanes = 1;
	avi->avi_bi.biBitCount = 24;
	avi->avi_bi.biCompression = BI_RGB;
	avi->avi_bi.biSizeImage = avi->avi_pixbufsiz;
	avi->avi_bi.biXPelsPerMeter = 0;
	avi->avi_bi.biYPelsPerMeter = 0;
	avi->avi_bi.biClrUsed = 0;
	avi->avi_bi.biClrImportant = 0;

	AVIFileInit();

	hr = AVIFileOpen(
		&avi->avi_pfile,		// returned file pointer
		file,					// file name
		OF_WRITE | OF_CREATE,	// mode to open file with
		NULL);					// use handler determined
								// from file extension....

	if ( hr == AVIERR_OK )
	{
		_fmemset(&avi->avi_strhdr, 0, sizeof(avi->avi_strhdr));
		avi->avi_strhdr.fccType                = streamtypeVIDEO;// stream type
		avi->avi_strhdr.fccHandler             = 0;
		avi->avi_strhdr.dwScale                = 1;

//		avi->avi_strhdr.dwRate                 = 15;		    // 15 fps
		avi->avi_strhdr.dwRate                 = (DWORD)gSceneFPS;

		avi->avi_strhdr.dwSuggestedBufferSize  = avi->avi_bi.biSizeImage;
		avi->avi_strhdr.rcFrame.left = 0;
		avi->avi_strhdr.rcFrame.top = 0;
		avi->avi_strhdr.rcFrame.right = avi->avi_w;
		avi->avi_strhdr.rcFrame.bottom = avi->avi_h;

		hr = AVIFileCreateStream(
			avi->avi_pfile,		    // file pointer
			&avi->avi_ps,		    // returned stream pointer
			&avi->avi_strhdr);	    // stream header

		if ( hr == AVIERR_OK )
		{
			/* Ask user for compression options */
			if ( AVISaveOptions(NULL, 0, 1,
					&avi->avi_ps,
					avi->avi_aopts ) )
			{
				hr = AVIMakeCompressedStream(
					&avi->avi_psCompressed,
					avi->avi_ps,
					&avi->avi_opts, NULL);

				if ( hr == AVIERR_OK )
				{
					hr = AVIStreamSetFormat(
							avi->avi_psCompressed,
							0,
							&avi->avi_bi,		// stream format
							sizeof(avi->avi_bi) );

					if ( hr == AVIERR_OK )
					{
						/* Allocate DIB Pixel Buffer */
						if ( avi->avi_pixbuf = malloc(avi->avi_pixbufsiz) )
						{
						}
						else err = "Not Enough Memory For Pixel Buffer";
					}
					else err = "Couldn't handle compression options";
				}
				else err = "Couldn't handle compression options";
			}
			else err = "User Cancelled AVI Save";
		}
		else err = "Couldn't Create AVI Stream";
	}
	else err = "Couldn't Open AVI File";

	if ( err )
	{
		NT_AVIClose((LWInstance)avi);
	}

	return(err);
}

#if AVI_USES_FRAMEBUFFER
XCALL_(static LWError) NT_AVIOpenFB(
	LWInstance inst,
	int w,
	int h )
{
	return(NT_AVIOpen(inst,w,h,"LWOutput.avi"));
}

XCALL_(static void) NT_AVIPauseFB(
	LWInstance inst )
{
	XCALL_INIT;
}
#endif /* AVI_USES_FRAMEBUFFER */

XCALL_(static LWError) NT_AVIBegin(
	LWInstance inst )
{
	struct AVIHandle	*avi = (struct AVIHandle *)inst;
	XCALL_INIT;

	return(NULL);
}

static void scatter3(
	unsigned char	*dest,
	const BufferValue *R,
	const BufferValue *G,
	const BufferValue *B,
	long				w )
{
	while ( w-- )
	{
		*dest++ = *B++;
		*dest++ = *G++;
		*dest++ = *R++;
	}
}

XCALL_(static LWError) NT_AVIWrite(
	LWInstance inst,
	const BufferValue *R,
	const BufferValue *G,
	const BufferValue *B,
	const BufferValue *A)
{
	LWError				 err = NULL;
	struct AVIHandle	*avi = (struct AVIHandle *)inst;
	unsigned char		*dest = avi->avi_pixbuf;

	/* Windows DIBs must be bottom-up */
	dest += (avi->avi_h - avi->avi_scanline) * avi->avi_pixlinesiz;

	scatter3(dest,R,G,B,avi->avi_w);

	if ( ++avi->avi_scanline == avi->avi_h )
	{
		HRESULT				 hr;

		/* Write the frame out */
		hr = AVIStreamWrite(
			avi->avi_psCompressed,	// stream pointer
			avi->avi_frame,			// time of this frame (at 30 fps)
			1,						// number to write
			avi->avi_pixbuf,		// Pixel Data to write
			avi->avi_pixbufsiz,		// size of this frame
			AVIIF_KEYFRAME,			// flags....
			NULL,
			NULL);

		if ( hr == AVIERR_OK )
		{
		}
		else err = "Failed To Write AVI Frame";

		/* Move back to top of next frame */
		avi->avi_scanline = 0;
		avi->avi_frame++;
	}

	return(err);
}

#if AVI_USES_FRAMEBUFFER

XCALL_(static int) NT_AVIActivateFB(
	long			 version,
	GlobalFunc		 *global,
	FrameBufferHandler *local,
	void			*serverData )
{
	XCALL_INIT;

	if (version != 1)
		return AFUNC_BADVERSION;

	/* Get some globals for the AVI saver */
	gGlobalFunc = global;
	NT_AVIGetSceneInfo(global);

	local->inst.create = NT_AVICreate;
	local->inst.destroy = NT_AVIDestroy;
	local->inst.load = NT_AVILoad;
	local->inst.save = NT_AVISave;
	local->inst.copy = NT_AVICopy;
	local->open = NT_AVIOpenFB;
	local->close = NT_AVIClose;
	local->begin = NT_AVIBegin;
	local->write = NT_AVIWrite;
	local->pause = NT_AVIPauseFB;

	return AFUNC_OK;
}

ServerRecord ServerDesc[] = {
	{ "FrameBufferHandler",	"NewTek-AVI",	NT_AVIActivateFB },
	{ NULL }
};

#else /* !AVI_USES_FRAMEBUFFER */

XCALL_(static int) NT_AVIActivate(
	long			 version,
	GlobalFunc		 *global,
	AnimSaverHandler *local,
	void			 *serverData )
{
	XCALL_INIT;

	if (version != 1)
		return AFUNC_BADVERSION;

	/* Get some globals for the AVI saver */
	gGlobalFunc = global;
	NT_AVIGetSceneInfo(global);

	local->inst.create = NT_AVICreate;
	local->inst.destroy = NT_AVIDestroy;
	local->inst.load = NT_AVILoad;
	local->inst.save = NT_AVISave;
	local->inst.copy = NT_AVICopy;
	local->open = NT_AVIOpen;
	local->close = NT_AVIClose;
	local->begin = NT_AVIBegin;
	local->write = NT_AVIWrite;

	return AFUNC_OK;
}

ServerRecord ServerDesc[] = {
	{ "AnimSaverHandler",	"NewTek-AVI",	NT_AVIActivate },
	{ NULL }
};

#endif /* !AVI_USES_FRAMEBUFFER */
