/*
 * $Id: newclass.c 5.3 1995/07/11 13:35:40 pfrench Exp $
 */

#ifndef PROOF_LIB_H
#include <proof/proof_lib.h>
#endif

#ifndef PROOF_CLASSES_H
#include <proof/classes.h>
#endif

#ifndef PROOF_CLASSBASE_H
#include <proof/classbase.h>
#endif

/* *********************** CUT FROM HERE *********************************** */
/* *********************** CUT FROM HERE *********************************** */

/* When Class is compiling, move these Method/Attrribute IDs out
 * to an include file, so other programmers can use the class.
 */


/* ---------------
	Step 1.  Give your new class a unique name and method/attribute base
*/

#define MYCLASS			"myclass"
#define MYCLASS_BASE	0x0L
/* base of methods and attributes, MUST BE CHANGED BEFORE ADDING  */

#define MY_SUPERCLASS	"rootclass"


/* ---------------
	Step 2.  Document the methods/attributes while designing.
*/
/*****************************************************************************
 *	Class:			MYCLASS
 *	SuperClass:		MY_SUPERCLASS
 *	Description:
 *
 *	New Methods:
 *
 *	Changed Methods:
 *
 *	Attributes:
 *
 *****************************************************************************
 *
 */

/**********************************************************/

/* ---------------
	Step 3.		Enumerate your NEW methods (if any)
*/

/* Methods for my class objects
 */
enum
{	MYM_Dummy = MYCLASS_BASE,

	MYM_METHOD1,
	MYM_METHOD2,
	MYM_METHOD3,
};

/* Method structs for my class objects
 */

/* ---------------
	Step 4.		Design the parameter packets for your NEW methods.
*/
/*
 *	MYM_METHOD1
 */
struct opMethod1
{	ULONG				 MethodID;
	ULONG				 op1_Size;
	ULONG				 op1_Flags;
};

/*
 *	MYM_METHOD2
 *	MYM_METHOD3
 */
struct opMethod2
{	ULONG				 MethodID;
	APTR				 op2_Ptr;
	ULONG				 op2_Size;
};

/* ---------------
	Step 5.		Enumerate your classes NEW attributes (if any)
				(I) implies an attribute specified only on CREATE
				(S) Specifies a "settable" attribute - write
				(G) Specifies a "gettable" attribute - read
*/

/* Attributes for my class objects
 */
enum
{	MYA_Dummy = ((LONG)TAG_USER | MYCLASS_BASE),

	MYA_Attr1,			/* (I) Attribute only set in initialization */

	MYA_Attr2,			/* (ISG) Attribute that can be read/set any time */

	MYA_Attr3,			/* (IS) Write only attribute */

	MYA_Attr4,			/* (G) Read only attribute */
};

/* *********************** CUT TO HERE ************************************* */
/* *********************** CUT TO HERE ************************************* */
/* **** AND MOVE OUT TO AN INCLUDE FILE OTHER PEOPLE CAN USE *************** */

/*****************************************************************************/
/*				PRIVATE CODE BEGINS HERE!!!									 */
/*****************************************************************************/

/* This data structure defines the data you will need for each instance
	(object) of your new class.  It should be PRIVATE if it is not a base class.
 */

/* ---------------
	Step 5.		Define your objects PRIVATE instance data
*/
struct MyInstData
{
	struct Something	*my_ss;		/* (IS) attr3 */

	char				*my_buf;	/* (I) attr1 */

	struct Node			 my_node;	/* (G) read-only attr4 */

	LONG				 my_val;	/* (ISG) read-write attr2 */


	LONG				 my_other_info;
	struct MinNode		 my_mn;

	char				 my_chars[23];
};

extern struct ClassBase	*ClassBase;

struct Class *my_AddClass( void );

static ULONG SAVEDS ASM my_Dispatch(
	REG(a0) struct Class *cl,
	REG(a2) APTR ob,
	REG(a1) Msg msg );

static ULONG my_Init( struct Class *cl, APTR ob, struct opSet *ops );
static ULONG my_SetAttrs( struct Class *cl, APTR ob, struct opSet *ops );
static ULONG my_GetAttrs( struct Class *cl, APTR ob, struct opSet *ops );

/* ---------------
	Step 6.		Rename "my_AddClass" to something like "hashtable_AddClass"
				So it doesn't conflict at link time.

				IMPORTANT: 
					This function should be the only symbol exported from
					this source file.  Everything else should be private.
*/
struct Class *my_AddClass( void )
{
	struct Class *cl;

	if ( cl = cl_MakeClass( MYCLASS,
							MY_SUPERCLASS, NULL,
							sizeof(struct MyInstData), 0) )
	{
		cl->cl_Dispatch.h_Entry = (HOOKFUNC) my_Dispatch;
		
		if ( !pb_AddClass(cl) )
		{
			cl_FreeClass(cl);
			cl = NULL;
		}
	}

	return(cl);
}

static ULONG SAVEDS ASM my_Dispatch(
	REG(a0) struct Class *cl,
	REG(a2) APTR ob,
	REG(a1) Msg msg )
{
	ULONG					 retval = 0;

	switch ( msg->MethodID )
	{
		case OM_NEW:
		{
			if ( ob = (APTR)ob_DoSuperMethodA(cl,ob,msg) )
			{
				if ( my_Init( cl,ob, (struct opSet *)msg ) )
				{
					retval = (ULONG) ob;
				}
				else
				{
					/* retval remains NULL, object failed */

					ob_CoerceMethod(cl,ob,OM_DISPOSE);
				}
			}
		}
		break;

		case OM_DISPOSE:
		{
			struct MyInstData	*my = (struct MyInstData *)INST_DATA(cl,ob);

			if ( my->my_ss )
				ob_Dispose(my->my_ss);

			retval = ob_DoSuperMethodA(cl,ob,msg);
		}
		break;

		case OM_SET:
		{
			/* Let superclass set it's attributes first */
			retval = ob_DoSuperMethodA(cl,ob,msg);

			retval += my_SetAttrs(cl,ob,(struct opSet *)msg);
		}
		break;

		case OM_GET:
		{
			/* Let superclass fill out its tags first, in case I need
			 * to override the values.
			 */
			retval = ob_DoSuperMethodA(cl,ob,msg);

			retval += my_GetAttrs(cl,ob,(struct opSet *)msg);
		}
		break;

		/* ------------ Methods Specific to my Class ----------- */

		case MYM_METHOD1:
		{
			struct MyInstData	*my = (struct MyInstData *)INST_DATA(cl,ob);
			struct opMethod1	*op1 = (struct opMethod1 *)msg;

			my->my_val += op1->op1_Size;
			retval = my->my_val;
		}
		break;

		case MYM_METHOD2:
		{
			/* My Superclass succeeded in doing what it needed */
			if ( retval = ob_DoSuperMethodA(cl,ob,msg) )
			{
				struct MyInstData	*my = (struct MyInstData *)INST_DATA(cl,ob);
				struct opMethod2	*op2 = (struct opMethod2 *)msg;

				memcpy(&my->my_buf[my->my_len],op2->op2_Ptr,op2->op2_Size);
				my->my_len += op2->op2_Size;
			}
		}
		break;

		case MYM_METHOD3:
		{
			/* For this method, my Superclass returns an error */

			if ( !(retval = ob_DoSuperMethodA(cl,ob,msg) )
			{
				struct MyInstData	*my = (struct MyInstData *)INST_DATA(cl,ob);

				my->my_other_info += 17;

				/* Execute another command */
				retval = ob_DoMethod(ob,SUPERM_Method1,24,42,33);
			}
		}
		break;

		default:
			retval = ob_DoSuperMethodA(cl,ob,msg);
		break;
	}

	return(retval);
}

static ULONG my_Init( struct Class *cl, APTR ob, struct opSet *ops )
{
	struct MyInstData	*my = (struct MyInstData *)INST_DATA(cl,ob);
	ULONG				 retval = 0;

	my->my_buf = (char *)
		ti_GetTagData(MYA_Attr1,0,ops->ops_Tags);

	if ( my->my_buf )
	{
		if ( my->my_ss = ob_AllocMemObj( ClassBase->cb_allocator_ob,
					sizeof(*my->my_ss)) )
		{
			/* We can successfully proceed to use this object */
			retval = TRUE;

			my_SetAttrs( cl,ob,ops );
		}
	}

	return(retval);
}

static ULONG my_SetAttrs( struct Class *cl, APTR ob, struct opSet *ops )
{
	struct MyInstData	*my = (struct MyInstData *)INST_DATA(cl,ob);
	struct TagItem		*ti;
	ULONG				 retval = 0;

	struct TagItem		*s_ti = ops->ops_Tags;
	
	while ( ti = ti_NextTagItem(&s_ti) )
	{
		/* increment the number of attributes set */
		retval++;

		switch (ti->ti_Tag)
		{
			case MYA_Attr2:
				my->my_val = (LONG)ti->ti_Data;
			break;

			case MYA_Attr3:
				my->my_ss = (struct Something	*)ti->ti_Data;
			break;

			default:
				/* saves execution time to put this in default case */
				retval--;
			break;
		}
	}

	/* Maybe execute some code to adjust for the new settings */

	return(retval);
}

static ULONG my_GetAttrs( struct Class *cl, APTR ob, struct opSet *ops )
{
	struct MyInstData	*my = (struct MyInstData *)INST_DATA(cl,ob);
	struct TagItem		*ti;
	ULONG				 retval = 0;

	struct TagItem		*s_ti = ops->ops_Tags;
	
	while ( ti = ti_NextTagItem(&s_ti) )
	{
		ULONG			*stor = (ULONG *)ti->ti_Data;

		switch (ti->ti_Tag)
		{
			case MYA_Attr2:
				*stor = (ULONG) my->my_val;
				retval++;
			break;

			case MYA_Attr4:		/* (R) Read only attribute */
				*stor = (ULONG) &my->my_node;
				retval++;
			break;
		}
	}

	return(retval);
}

