/*********************************************************************\
*
* FlyerClipFmt.h - Flyer clip format
*
* $Id: FlyerClipFmt.h,v 1.0 1995/07/11 11:03:25 Flick Exp $
*
* $Log: FlyerClipFmt.h,v $
*Revision 1.0  1995/07/11  11:03:25  Flick
*FirstCheckIn
*
*
* Copyright (c) 1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
*	07/11/95		Marty		Created
\*********************************************************************/

struct ClipHeader {
	ULONG	ID;				// "CLIP" Identifies this block as the Clip header
	ULONG	Length;			// Length of clip in logical blocks (all components)
	UBYTE	Version;			// Flyer version that created file (4)
	UBYTE	VidCmpVer;		// Version of Video Compression chips used (2)
	UBYTE	AudVer;			// Audio data format/version (2)
	UBYTE	VidGrade;		// Video grade (std/HQ5)
	ULONG	Fields;			// Number of fields contained in clip (1/60 sec each)
	ULONG	DataStart;		// Block offset: clip hdr to first frame header
	ULONG	DataEnd;			// Block offset: clip hdr to last data blk of last frm+1
	ULONG	Tail;				// Block offset: clip hdr to Tail Header
	UBYTE	reserved2;
	UBYTE	AudioLength;	// No. logical blocks used per mono channel per field (3)
	UBYTE	VidFlag;			// Flag: includes video data (0 = no, 1 = yes)
	UBYTE	AudioChans;		// Number of mono audio channels included (0 or more)
	UBYTE	Pedestal;		// Digital pedestal value for playback (60)
	UBYTE	reserved3;
	WORD	CustCoefs[9];		// Custom FIR coefficients/scale used
	WORD	CustInvCoefs[9];	// Custom FIR coefficients/scale to use for playback
	UWORD	LogBlkSize;		// Logical block size (512)
	UBYTE	FIRROM;			// FIR ROM lookup table used
	UBYTE	FIRRAM;			// FIR RAM lookup table used
	UBYTE	reserved4[438];
};

struct Field {
	UWORD	VidStart;		// Block offset: frm hdr to start of field's video data
	UWORD	VidLength;		// Logical block length of this field's video data
	UBYTE	Tolerance;		// Compression error tolerance
	UBYTE	RndSize;			// Amplitude of noise generation
	UBYTE	RndFreq;			// Frequency of noise generation
	UBYTE	RndSeed;
	UWORD	reserved1;
	UWORD	reserved2;
	UBYTE	Flags;			// Private flags
	UBYTE	reserved3;
	UBYTE	reserved4[18];
	UBYTE	FIRpreset;		// FIR filter preset for recorded video data this field
	UBYTE	SerType;			// Type of serial device for recorded serial data
	UBYTE	SerMake;			// Make of serial device for recorded serial data
	UBYTE	SerialDataLen;	// Byte length of serial data captured for this field
	UBYTE	SerData[36];	// Serial data buffer (data recorded this field)
};

struct FrameHeader {
	ULONG	ID;				// "CFRM" Identifies this block as a frame header
	ULONG	Length;			// Length of this frame (including header)
	UBYTE	Version;			// Flyer version that created file (4)
	UBYTE	VidCmpVer;		// Version of Video Compression chips used (2)
	UBYTE	AudVer;			// Audio data format/version (2)
	UBYTE	reserved1;
	LONG	PrevFrame;		// Block offset from this to previous frame header
	LONG	NextFrame;		// Block offset from this to next frame header
	ULONG	FrameNumber;	// Ordinal color frame number (0 is first)
	UBYTE	reserved2;
	UBYTE	AudioLength;	// No. logical blocks used per mono channel per field (3)
	UBYTE	VidFlag;			// Flag: includes video data (0 = no, 1 = yes)
	UBYTE	AudioChans;		// Number of mono audio channels included (0 or more)
	UBYTE	reserved3[196];
	struct Field	fields[4];	// Info for each field in this frame
};

#define	GRADE_STD	0
#define	GRADE_HQ5	1

#define	ID_CLIP	0x434C4950
#define	ID_CFRM	0x4346524D
