/*********************************************************************\
*
* $IntFSprivate.h$
*
* $Id: intfsprivate.h,v 1.1 1997/01/09 14:50:32 Hayes Exp $
*
* $Log: intfsprivate.h,v $
*Revision 1.1  1997/01/09  14:50:32  Hayes
*Added three members to GRIP, g_curpos_ext, g_filelen_ext, g_physlen_ext
*to allow GRIP to deal with files >32 bits in size
*
*Revision 1.0  1995/11/06  16:52:50  Flick
*FirstCheckIn
*
*
* Copyright (c) 1995 NewTek, Inc.
* Confidental and Proprietary. All rights reserved. 
*
* (Borrowed out of Flyer/inc/FileSys.h)
*
\*********************************************************************/

#ifndef	INC_FILESYS_H
#define	INC_FILESYS_H

typedef	unsigned char		BOOL8;

#define	FS_VERSION		1				// Version of FileSystem

#define	DFLTBLKSIZE			512
#define	DFLTBLKSZSHIFT		9

#define	MEGABYTE				0x100000
#define	MEGABYTESHIFT		20

#define	AMIGATICKS	50					// Ticks per second

#define	CHIPPATCHAREA	256			// Size in blocks (128K)

#define	MINFILEHDRS			5000		// Allow at least this many files/dirs on a drive


#define	VOLNAMELENGTH		32			// Limit for length of volume names
#define	FILENAMELENGTH		32			// Limit for file names
#define	FILECOMMENTLENGTH	32			// Limit for file comments
#define	DIRNAMELENGTH		32			// Limit for directory names
#define	DIRCOMMENTLENGTH	32			// Limit for directory comments


struct FlyerDate {
	ULONG	days;				// Number of days since Jan. 1, 1978
	ULONG	minutes;			// Number of minutes past midnight
	ULONG	ticks;			// Number of ticks past minute
};


/*** Structure returned from FlyerDriveInfo call ***/
#define	FVINAMELENGTH		80			// Limit for FVI title


#define	CINAMELENGTH		80			// Limit for filenames
#define	CICOMMENTLENGTH	80			// Limit for comments


/*** Flyer object types ***/
#define	FLYER_TYPE_FILE	1
#define	FLYER_TYPE_DIR		2
#define	FLYER_TYPE_ROOT	3


#define	ROOTENTRIES		100

struct ROOTHDR {
	ULONG		id;			// Identifies this header
	BYTEBITS	flags;			// Drive characteristics
	BOOL8		diskokay;		// Disk structures all valid?
	UWORD		blksize;		// Logical block size
	ULONG		parent;			// Always 0
	char		volname[VOLNAMELENGTH];	// Name of volume
		
	ULONG		freelist;		// Location of FreeList table
	ULONG		headlist;		// Location of HeadList table
	ULONG		hdrsarea;		// File/Dir headers
	ULONG		unused;			// Pointer past used hdrs
	ULONG		dataarea;		// Start of data area (fixed)
	ULONG		headspace;		// Space used for heads
	ULONG		videoend;		// End of fast area of drive
	ULONG		blksfree;		// Data blocks free -- OBSOLETE

	UBYTE		version;		// FileSystem version
	UBYTE		type;			// Type (root)
	struct FlyerDate date;			// Date formatted

	ULONG		userblks;		// Drive size (user data blks)
	ULONG		newheadlist;		// Location of NewHeadList table

	UBYTE		pad1[10];		// Future

	ULONG		entries[ROOTENTRIES];	// Hashed children
	ULONG		checksum;		// Balance to 0
};

#define	DIRENTRIES	100

struct DIRHDR {
	ULONG		id;							// Identifies this header
	ULONG		nexthash;					// Next child w/ same hash
	ULONG		parent;						// Loc of parent's dir hdr
	char		name[DIRNAMELENGTH];			// Name of directory
	char		comment[DIRCOMMENTLENGTH];	// Comment string
	BYTEBITS	flags;						// See DF_xxx constants
	UBYTE		type;							// Type (file/dir)
	struct FlyerDate date;				// Create date/time
	ULONG		bits;							// User-defined bits
	char		pad2[14];					// Fill out to 512
	ULONG		entries[DIRENTRIES];		// Hashed children
	ULONG		checksum;					// Balance to 0
};

struct FILEHDR {
	ULONG		id;							// Identifies this header
	ULONG		nexthash;					// Next child w/ same hash
	ULONG		parent;						// Loc of parent's dir hdr
	char		name[FILENAMELENGTH];			// Name of file
	char		comment[FILECOMMENTLENGTH];	// File comment string
	UBYTE		pad1;	
	UBYTE		type;							// Type (file/dir)
	struct FlyerDate date;				// File create date/time
	ULONG		bits;							// User-defined bits
	char		pad2[8];
	ULONG		start;						// File location on drive
	ULONG		bytes;						// byte length
	ULONG		bytes_hi;					// High 32 bits of byte length
	char		pad3[394];					// Fill out to 512
	ULONG		checksum;					// Balance to 0
};

struct MAPENTRY {					// Entry in FreeMap block
	ULONG		me_Block;			// Start location on drive
	ULONG		me_Length;			// Number of blocks in this chunk
};

#define	MAPENTRIES		63
#define	MAP_ID			0x464C5354
#define	MINFREEMAPBLKS	5

struct MAPBLOCK {					// List of free drive space
	ULONG		mb_ID;				// "FLST"
	ULONG		mb_Next;				// Offset to next block (or 0)
	struct MAPENTRY mb_Entries[MAPENTRIES];
};

struct FSHEADENTRY {
	ULONG		fshe_start;			// Start block
	ULONG		fshe_length;		// Block length
	ULONG		fshe_ID;				// Sub-group ID
};

#define	FSHEADENTRIES	42
#define	HEADLIST_ID		0x484C5354	// "HLST"
#define	HEADLISTSIZE	50				// Blocks in list (for each of 2 lists)

struct FSHEADSBLOCK {		// List of area allocated to heads
	ULONG		fsh_ID;			// "HLST"
	ULONG		fsh_Next;		// Offset to next group (or 0)
	struct FSHEADENTRY fsh_Heads[FSHEADENTRIES];
};

union FSBUFF {
	struct ROOTHDR	root;
	struct DIRHDR	dir;
	struct FILEHDR	file;
	struct MAPBLOCK	map;
	UBYTE		raw[512];
};

struct BufCtl {
	struct	MinNode	node;		// For linking into lists
	UBYTE		scsidrv;				// SCSI chan/unit associated with
	UBYTE		users;				// Use count
	ULONG		block;				// Block number where data located
	UBYTE		ndx;					// Index number of this buffer (1..n)
	UBYTE		pad;					// This makes the structure a nice size
};

typedef struct GRIP {
	struct GRIP *g_next;		// Forward link
	ULONG		g_header;		// FS Header of file/dir
	ULONG		g_parent;		// FS Header of parent
	ULONG		g_DL_dir;		// Location of directory currently scanning
	ULONG		g_DL_blk;		// DirList header block found
	WORD		g_DL_hash;		// Hash ptr for dir listing
	ULONG		g_DL_link;		// Hash chain link from previous
	BOOL8		g_DL_done;		// Flag used for dir listing
	UBYTE		g_drv;			// SCSI channel/unit
	UBYTE		g_type;			// Object type
	/* File operation stuff */
	BOOL		g_opened;			// File open?
	ULONG		g_startblk;			// Start blk of file's data
	ULONG		g_curpos;			// Position within the file : lo 32 bits
	ULONG		g_curpos_ext;		// Position within the file : hi 32 bits
	ULONG		g_filelen;			// Precise length of file : lo 32 bits
	ULONG		g_filelen_ext;		// Precise length of file : hi 32 bits
	ULONG		g_physlen;			// Physical length of file (blks * 512) : lo 32 bits
	ULONG		g_physlen_ext;		// Physical length of file (blks * 512) : hi 32 bits
	ULONG		g_protbits;			// Protect bits when opened
	ULONG		g_magic;				// Is this an authentic structure?
} GRIP;

#define	GRIPMAGIC	0x47524950		// "GRIP"

/* Used with "DirTree" */
typedef UBYTE (*TREEROUT)(UBYTE,struct ClipInfo *,GRIP *,APTR);


#define	ID_ROOT		0x524F4F54		// Identifies root block
#define	ID_FILE		0x46494C45		// Identifies file header block
#define	ID_DIR		0x46444952		// Identifies directory header block

#define	TYPE_NONE	0
#define	TYPE_FILE	1
#define	TYPE_DIR		2
#define	TYPE_ROOT	3

#define	MODE_INPUT	1
#define	MODE_OUTPUT	2
#define	MODE_UPDATE	3

#define	ACCESS_SHARED	0
#define	ACCESS_EXCL		1

#define	AMIPROT_SCRIPT		(1<<6)
#define	AMIPROT_ARCHIVE	(1<<4)
#define	AMIPROT_READ		(1<<3)
#define	AMIPROT_WRITE		(1<<2)
#define	AMIPROT_EXEC		(1<<1)
#define	AMIPROT_DELETE		(1<<0)

/********************* Private stuff ***********************/

struct DEFRAGDATA {
	ULONG		dd_spot;
	ULONG		dd_bestloc;
	ULONG		dd_besthdr;
	BOOL8		dd_restart;
};


#endif	/* INC_FILESYS_H */
