
#include <exec/types.h>
#include <string.h>
#include <stdio.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include "flyerlib.h"
#include "flyer.h"
#include "flyerClipFmt.h"

#define	MONOAUDFLDSIZE		3
#define	NUMAUDCHANS			2
#define	SAMPLESPERFIELD	735	// Audio samples per video field

struct Library *FlyerBase;

UBYTE BlkBuff[512];

UWORD	AudioBuff[8][256*MONOAUDFLDSIZE];

UWORD BillStereo;
UWORD Endian = 0;

/* Prototypes */
ULONG WriteTestClip(char *pathname, int MyType, BPTR IN, BPTR IN2);
void AIFFHeader(ULONG frames, BPTR IN,BPTR IN2);
void AD516Header(ULONG frames, BPTR IN,BPTR IN2);
void WAVHeader(ULONG frames, BPTR IN,BPTR IN2);
void IFFHeader(ULONG frames, BPTR IN, BPTR IN2);
void VOCHeader(ULONG frames, BPTR IN, BPTR IN2);
void AUHeader(ULONG frames, BPTR IN, BPTR IN2);

int main(int argc, char *argv[])
{
BPTR IN, IN2;
ULONG err;
int MyType;

printf("Blackbox Read Version 1.02\n");
printf("Copyright Prowave 1996\n");

if (argc < 4)
   {
   printf("Not enough parameters \n");
   printf("%s TYPE InputFile OutputFile1 <Output File2>\n",argv[0]); 
   }

IN = Open(argv[3],MODE_NEWFILE);

MyType = atoi(argv[1]);

IN2 = NULL;

if (argc == 5)
   {
   IN2 = Open(argv[4],MODE_NEWFILE);
   }

if (FlyerBase = OpenLibrary(FLYERLIBNAME,0)) 
   {
   err = WriteTestClip(argv[2],MyType,IN,IN2);	// Make skeletal version
   CloseLibrary(FlyerBase);
   }
else 
   {
   printf("Open flyer.library failed\n");
   }

Close(IN);
Close(IN2);
return(0);
}

ULONG WriteTestClip(char *pathname, int MyType, BPTR IN, BPTR IN2)
{
int	j,k;
ULONG	i,frm[3];
BPTR	handle;

UWORD  temp, temp2;

handle = Open(pathname,MODE_OLDFILE);
if (!handle) 
   {
   printf("Did not write clip successfully\n");
   return(1);
   }

/* Read the Ignore Stuff */
Read(handle,AudioBuff,1024);

frm[1] = (ULONG)AudioBuff[0][6];
frm[2] = (ULONG)AudioBuff[0][7];

frm[0] = (frm[1] << 16) | frm[2];   

frm[0] = frm[0] >> 2;

printf("This is frame %x \n",frm[0]);

/* This is AIFF */
if (MyType == 1)
   AIFFHeader(frm[0],IN,IN2);

/* This is AD516 */
if (MyType == 2)
   AD516Header(frm[0],IN,IN2);

/* This is Wave */
if (MyType == 3)
   {
   WAVHeader(frm[0],IN,IN2);
   for (i=0;i<frm[0];i++)
      {
      Read(handle,AudioBuff,12288);
      for (k=0;k<4;k++)
         { 
         for (j=0;j<735;j++)
            {
            temp = AudioBuff[k][j];
            temp = (temp >> 8) | (temp << 8);
            Write(IN,(char *)&temp,2);
            temp2 = AudioBuff[k+4][j];
            temp2 = (temp2 >> 8) | (temp2 << 8);
            if (BillStereo == 1)
               Write(IN,(char *)&temp2,2);
            else
               Write(IN2,(char *)&temp2,2);
            }
         }
      Read(handle,AudioBuff,512);
      }
   Close(handle);
   return(0);
   }

/* This is RAW Format */
if (MyType == 4)
   {
   BillStereo = 0;
   }

/* This is Type IFF */
if (MyType == 5)
   {
   IFFHeader(frm[0],IN,IN2);
   for (i=0;i<frm[0];i++)
      {
      Read(handle,AudioBuff,12288);
      for (j=0;j<2940;j+=4)
         {
         temp = AudioBuff[0][j];
         Write(IN,(char *)&temp,1);
         if (BillStereo == 1)
            Write(IN,(char *)&AudioBuff[4][j],1);
         else
            Write(IN2,(char *)&AudioBuff[4][j],1);
         }
      Read(handle,AudioBuff,512);
      }

   Close(handle);

   return(0);
   }

if (MyType == 6)
   {
   BillStereo = 0;
   VOCHeader(frm[0],IN,IN2);
   for (i=0;i<frm[0];i++)
      {
    
      Read(handle,AudioBuff,12288);
      for (k=0;k<4;k++)
         { 
         for (j=0;j<735;j++)
            {
            temp = AudioBuff[k][j];
            if (temp < 0x8000)
               temp += 0x8000;
            else
               temp -= 0x8000;
            Write(IN,(char *)&temp,1);
            temp = AudioBuff[k+4][j];
            if (temp < 0x8000)
               temp += 0x8000;
            else
               temp -= 0x8000;
            if (BillStereo == 1)
               Write(IN,(char *)&temp,1);
            else
               Write(IN2,(char *)&temp,1);
            }
         }
      Read(handle,AudioBuff,512);
      }

   Close(handle);

   return(0);
   }

if (MyType == 7)
   {
   BillStereo = 0;
   AUHeader(frm[0],IN,IN2);
   }

printf("This is BillStereo %d \n",BillStereo);

for (i=0;i<frm[0];i++)
   {
   Read(handle,AudioBuff,12288);
   if (BillStereo == 2)
      {
      for (k=0;k<4;k++)
         {
         for (j=0;j<735;j++)
            {
            Write(IN,(char *)&AudioBuff[k][j],2);
            Write(IN,(char *)&AudioBuff[k+4][j],2);
            }
         }
      } 
   else 
      {
      for (j=0;j<4;j++)
         {
         Write(IN,AudioBuff[j],1470);
         Write(IN2,AudioBuff[j+4],1470);
         }
      } 
   Read(handle,AudioBuff,512);
   }

Close(handle);

return(0);		// Okay
}

void AIFFHeader(ULONG frames,BPTR IN,BPTR IN2)
{
UWORD Twist[27] = {0x464f,0x524D,0x0002,0xFFFF,0x4149,0x4646,
                   0x434F,0x4D4D,0x0000,0x0012,0x0001,0x0008,
                   0x624A,0x0010,0x400E,0xAC44,0x0000,0x0000,
                   0x0000,0x5353,0x4E44,0xFFFF,0xFFFF,0x0000,
                   0x0000,0x0000,0x0000}; 

ULONG length;
ULONG count;

length = frames * 5880;
count = length >> 1;

if (IN2 == NULL)
   length = length * 2;

length = length + 46; 

Write(IN,(char *)&Twist,4);

Write(IN,(char *)&length,4);

length -= 38;

Write(IN,(char *)&Twist[4],12);

if (IN2 == NULL)
   Write(IN,(char *)&Twist[2],2);
else   
   Write(IN,(char *)&Twist[10],2);

Write(IN,(char *)&count,4);

Write(IN,(char *)&Twist[13],16);

Write(IN,(char *)&length,4);

Write(IN,(char *)&Twist[23],8);

BillStereo = 2;

if (IN2 != NULL)
   {
   Write(IN2,Twist,4);
   length += 38;
   Write(IN2,(char *)&length,4);

   Write(IN2,(char *)&Twist[4],14);

   Write(IN2,(char *)&count,4);

   Write(IN2,(char *)&Twist[13],16);

   length -= 38;

   Write(IN2,(char *)&length,4);

   Write(IN2,(char *)&Twist[23],8);

   BillStereo = 1;
   }
}

void AD516Header(ULONG frames,BPTR IN,BPTR IN2)
{
UWORD Twist[17] = {0x4B57,0x4B33,0x0000,0xAC44,0x0000,0xAC44,
                   0x0C80,0x0000,0x0000,0x0000,0x0000,0x0000,
                   0x0000,0x0000,0x0000,0x0000,0x0000};
ULONG length; 
 
ULONG i = 0;

BillStereo = 1;
 
Write(IN,Twist,34);
Write(IN2,Twist,34);
 
length = frames * 2940;

Write(IN,(char *)&length,4);
Write(IN2,(char *)&length,4);

Write(IN,(char *)&length,4);
Write(IN2,(char *)&length,4);

Write(IN,(char *)&i,4);
Write(IN2,(char *)&i,4);

length -= 1;

Write(IN,(char *)&length,4);
Write(IN2,(char *)&length,4);

for (i=0;i<0x717;i++)
   {
   Write(IN,0,2);
   Write(IN2,0,2);
   }
}

void WAVHeader(ULONG frames,BPTR IN,BPTR IN2)
{

UWORD Twist[23] = {0x5249,0x4646,0xFFFF,0xFFFF,0x5741,0x5645,
                   0x666D,0x7420,0x1000,0x0000,0x0100,0x0100,
                   0x44AC,0x0000,0x8858,0x0100,0x0200,0x1000,
                   0x6461,0x7461,0x10B1,0x0200,0x0400}; 

ULONG temp, temp2;

BillStereo = 1;

temp = frames*5880;

if (IN2 == NULL)
   temp = temp<<1;

temp2 = temp + 36;
Write(IN,(char *)&Twist[0],4);
temp2 = (temp2 >> 24) | (temp2<<24) | ((temp2>>8) & 0x0000FF00) | ((temp2<<8) & 0x00FF0000); 
Write(IN,(char *)&temp2,4);

if (IN2 == NULL)
   {
   Write(IN,(char *)&Twist[4],14);
   Write(IN,(char *)&Twist[21],2);
   Write(IN,(char *)&Twist[12],4);
   Write(IN,(char *)&Twist[20],4);
   Write(IN,(char *)&Twist[22],2);
   Write(IN,(char *)&Twist[17],6);
   }
else
   {
   Write(IN,(char *)&Twist[4],32);
   }

temp = (temp>>24) | (temp<<24) | ((temp>>8) & 0x0000FF00) | ((temp<<8) & 0x00FF0000); 
Write(IN,(char *)&temp,4);

if (IN2 != NULL)
   {
   Write(IN2,(char *)&Twist[0],4);
   Write(IN2,(char *)&temp2,4);
   Write(IN2,(char *)&Twist[4],32);
   Write(IN2,(char *)&temp,4);
   BillStereo = 2;
   } 

}

void IFFHeader(ULONG frames,BPTR IN,BPTR IN2)
{
UWORD Twist[18] = {0x464f,0x524D,0x3853,0x5658,0x5648,0x4452,
                   0x2B11,0x0100,0x0001,0x0000,0x4348,0x414E,
                   0x0000,0x0004,0x0000,0x0002,0x424F,0x4459};

UWORD Channels;

ULONG Length;

ULONG temp;

BillStereo = 1;
Channels = 2;

if (IN2 != NULL)
   {
   BillStereo = 2;
   Channels = 1;
   }

Length = frames*735*BillStereo;

Write(IN,(char *)&Twist,4);

Length = Length+48;

Write(IN,(char *)&Length,4);

Write(IN,(char *)&Twist[2],8);

temp = 20;

Write(IN,(char *)&temp,4);

Length -= 12;

Write(IN,(char *)&Length,4);

temp = 0;

Write(IN,(char *)&temp,4);

Write(IN,(char *)&temp,4);

Write(IN,(char *)&Twist[6],18);

Write(IN,(char *)&Channels,2);

Write(IN,(char *)&Twist[16],4);

Length -= 36;

Write(IN,(char *)&Length,4);

if (IN2 != NULL)
   {
   BillStereo = 2;
   Write(IN2,(char *)&Twist,4);
   Length = Length+48;
   Write(IN2,(char *)&Length,4);
   Write(IN2,(char *)&Twist[2],8);
   temp = 20;
   Write(IN2,(char *)&temp,4);
   Length -= 8;
   Write(IN2,(char *)&Length,4);
   temp = 0;
   Write(IN2,(char *)&temp,4);
   Write(IN2,(char *)&temp,4);
//   Write(IN2,(char *)&Twist[6],24);
   Write(IN2,(char *)&Twist[6],18);
   Write(IN2,(char *)&Channels,2);
   Write(IN2,(char *)&Twist[16],4);
   Length -= 40;
   Write(IN2,(char *)&Length,4);
   }
}

void VOCHeader(ULONG frames,BPTR IN,BPTR IN2)
{
ULONG count;

UWORD Temp[13] = {0x4372,0x6561,0x7469,0x7665,0x2056,0x6F69,
                  0x6365,0x2046,0x696C,0x651A,0x1A00,0x0A01,
                  0x2911};

BillStereo = 1;

Write(IN,(char *)&Temp,26);

count = frames * 5880;

if (IN2 != NULL)
   {
   Write(IN2,(char *)&Temp,26);
   count /= 2;
   count = ((count >> 8) & 0x0000FF00) | 
           ((count << 8) & 0x00FF0000) |
           ((count <<16) & 0xFF000000) | 0x000000E9;
   Write(IN2,(char *)&count,4);
   BillStereo = 2;
   } 

if (IN2 == NULL)
   count = ((count >> 8) & 0x0000FF00) | 
           ((count << 8) & 0x00FF0000) |
           ((count <<16) & 0xFF000000) | 0x000000E9;

Write(IN,(char *)&count,4);
}

void AUHeader(ULONG frames,BPTR IN,BPTR IN2)
{
UWORD Twist[8] = {0x2E73,0x6E64,0x0000,0x0018,0x0000,0x0003,0x0000,0xAC44};
ULONG count;
ULONG channel;

Write(IN,(char *)&Twist,8);

BillStereo = 2;

if (IN2 == NULL)
   {
   count = 11760 * frames;
   channel = 2;
   }
else
   {
   count = 5880 * frames;
   channel = 1;
   }

Write(IN,(char *)&count,4);

Write(IN,(char *)&Twist[4],8);

Write(IN,(char *)&channel,4);

if (IN2 != NULL)
   {
   BillStereo = 1;
   Write(IN2,(char *)&Twist,8);
   Write(IN2,(char *)&count,4);
   Write(IN2,(char *)&Twist[4],8);
   Write(IN2,(char *)&channel,4);
   }

}


